#ifndef HiDrv_h
#define HiDrv_h

#ifdef __cplusplus
  extern "C"
  {
#endif  //__cplusplus

#define HIDRV_MAJOR (4)
#define HIDRV_MINOR (1)

#define HIDCMDBLK(HidCmd, HidMsg, HidFlag)  MAKELPARAM(MAKEWORD(HidCmd, HidMsg), HidFlag)
#define HIDCMD(HidCmdBlk)                   LOBYTE(HidCmdBlk)
#define HIDMSG(HidCmdBlk)                   HIBYTE(HidCmdBlk)
#define HIDFLAG(HidCmdBlk)                  LOBYTE(HIWORD(HidCmdBlk))

typedef enum
{
  HIDCMD_UNKNOWN,
  HIDCMD_OPEN,
  HIDCMD_CLOSE,
  HIDCMD_LOADFILE,
  HIDCMD_CLOSEFILE,
  HIDCMD_PLAY,
  HIDCMD_PAUSE,
  HIDCMD_STOP,                  //  HIDMSG_ ... FLUSH
  HIDCMD_STEP,
  HIDCMD_SEEK,
  HIDCMD_CUE,
  HIDCMD_CAPTURE,
  HIDCMD_SETAUDIO,              //  HIDMSG_ ... MUTE, STREAM, VOLUME, KARAOKE, MODE, SPEAKER
  HIDCMD_GETAUDIO,              //  HIDMSG_ ... STATUS, STREAM, TOTALSTREAMS, STREAMID, TYPE, BITRATE, SAMPLERATE, CHANNEL, MODE
  HIDCMD_SETVIDEO,              //  HIDMSG_ ... FREEZE, STREAM, MODE, KEY, SOURCE, BLANK, OUTPUT, BRIGHT, HUE, SATURATION
  HIDCMD_GETVIDEO,              //  HIDMSG_ ... KEY, STATUS, STREAM, TOTALSTREAMS, BITRATE, MAXBITRATE, PICTRATE
  HIDCMD_SETSUBPICT,            //  HIDMSG_ ... ENABLE, VISIBLE, STREAM, PALETTE
  HIDCMD_GETSUBPICT,            //  HIDMSG_ ... STREAM, TOTALSTREAMS, STREAMID, HANDLE
  HIDCMD_SETHIGHLIGHT,          //  HIDMSG_ ... ENABLE, POSITION, COLOR
  HIDCMD_SETDISPLAY,            //  HIDMSG_ ... MODE
  HIDCMD_GETDISPLAY,
  HIDCMD_SETMACROVISION,
  HIDCMD_GETMACROVISION,
  HIDCMD_SETOSD,
  HIDCMD_GETOSD,
  HIDCMD_SETWINDOW,             //  HIDMSG_ ... PARENT, HANDLE, STYLE, STATE, TITLE, UPDATE, POSITION
  HIDCMD_GETWINDOW,             //  HIDMSG_ ... HANDLE, VISIBLE, MINIMIZED, MAXIMIZED, TITLE, POSITION
  HIDCMD_SETSYSTEM,             //  HIDMSG_ ... FORMAT, SPEED, REGION, DATA, TIME, MODE
  HIDCMD_GETSYSTEM,             //  HIDMSG_ ... POSITION, LENGTH, STATE, FORMAT, SPEED, FILENAME, DSI, REGION, DATA, CAPS, VOLUMEID, MODE
  HIDCMD_COMPARE,               //  HIDMSG_ ... REGION
  HIDCMD_RESUME,
  HIDCMD_CSS,
  HIDCMD_SETCALLBACK,
  HIDCMD_SETCALLBACKTYPE,       // HIDMSG_ ... ENABLE, DISABLE, FLUSH
  HIDCMD_SEND,
  HIDCMD_DRIVE,                 //  HIDMSG_ ... LOCK, EJECT
  HIDCMD_FILESECTOR,            //  HIDMSG_ ... OPEN, CLOSE, READ, SEEK, LENGTH, READTOC, READCDTEXT, READFULLTOC, READSESSIONINFO
  HIDCMD_GHAL,                  // for all the ghal commands from the UI to nav to hidrv and then to ghal
  HIDCMD_VER,
  HIDCMD_TRANSPORT              //  HIDMSG_ ... OPEN, READTOC, START
} HIDCMD;

/*****************************************
 **         HiMsg for Set/Get           **
 *****************************************/
typedef enum
{
  HIDMSG_UNKNOWN,
  HIDMSG_ENABLE,                //  HIDCMD_ ... SETSUBPICT, SETHIGHLIGHT, SETCALLBACKTYPE
  HIDMSG_DISABLE,               //  HIDCMD_ ... SETCALLBACKTYPE
  HIDMSG_STREAM,                //  HIDCMD_ ... SETAUDIO, GETAUDIO, SETVIDEO, GETVIDEO, SETSUBPICT, GETSUBPICT
  HIDMSG_STREAMID,              //  HIDCMD_ ... GETAUDIO, GETSUBPICT
  HIDMSG_TOTALSTREAMS,          //  HIDCMD_ ... GETAUDIO, GETVIDEO, GETSUBPICT
  HIDMSG_PALETTE,               //  HIDCMD_ ... SETSUBPICT
  HIDMSG_COLOR,                 //  HIDCMD_ ... SETHIGHLIGHT
  HIDMSG_MODE,                  //  HIDCMD_ ... SETVIDEO, SETDISPLAY, SETAUDIO, GETAUDIO, SETSYSTEM, GETSYSTEM
  HIDMSG_SOURCE,                //  HIDCMD_ ... SETVIDEO
  HIDMSG_OUTPUT,                //  HIDCMD_ ... SETVIDEO
  HIDMSG_FREEZE,                //  HIDCMD_ ... SETVIDEO
  HIDMSG_KEY,                   //  HIDCMD_ ... SETVIDEO, GETVIDEO
  HIDMSG_MUTE,                  //  HIDCMD_ ... SETAUDIO
  HIDMSG_VOLUME,                //  HIDCMD_ ... SETAUDIO
  HIDMSG_KARAOKE,               //  HIDCMD_ ... SETAUDIO
  HIDMSG_POSITION,              //  HIDCMD_ ... SETWINDOW, GETWINDOW, GETSYSTEM, SETHIGHLIGHT
  HIDMSG_LENGTH,                //  HIDCMD_ ... GETSYSTEM, FILESECTOR
  HIDMSG_SIZE,
  HIDMSG_DATA,                  //..HIDCMD_ ... SETSYSTEM, GETSYSTEM
  HIDMSG_BITRATE,               //  HIDCMD_ ... GETAUDIO, GETVIDEO
  HIDMSG_MAXBITRATE,            //  HIDCMD_ ... GETVIDEO
  HIDMSG_SAMPLERATE,            //  HIDCMD_ ... GETAUDIO
  HIDMSG_PICTRATE,              //  HIDCMD_ ... GETVIDEO
  HIDMSG_STATUS,                //  HIDCMD_ ... GETAUDIO, GETVIDEO
  HIDMSG_STATE,                 //  HIDCMD_ ... SETWINDOW, GETSYSTEM
  HIDMSG_HANDLE,                //  HIDCMD_ ... SETWINDOW, GETWINDOW, GETSUBPICT
  HIDMSG_VISIBLE,               //  HIDCMD_ ... GETWINDOW, SETSUBPICT
  HIDMSG_MINIMIZED,             //  HIDCMD_ ... GETWINDOW
  HIDMSG_MAXIMIZED,             //  HIDCMD_ ... GETWINDOW
  HIDMSG_UPDATE,                //  HIDCMD_ ... SETWINDOW
  HIDMSG_FORMAT,                //  HIDCMD_ ... SETSYSTEM, GETSYSTEM
  HIDMSG_FILENAME,              //  HIDCMD_ ... GETSYSTEM
  HIDMSG_TITLE,                 //  HIDCMD_ ... SETWINDOW, GETWINDOW
  HIDMSG_TYPE,                  //  HIDCMD_ ... GETAUDIO
  HIDMSG_SPEED,                 //  HIDCMD_ ... SETSYSTEM, GETSYSTEM
  HIDMSG_DSI,                   //  HIDCMD_ ... GETSYSTEM
  HIDMSG_REGION,                //  HIDCMD_ ... SETSYSTEM, GETSYSTEM, COMPARE
  HIDMSG_STYLE,                 //  HIDCMD_ ... SETWINDOW
  HIDMSG_PARENT,                //  HIDCMD_ ... SETWINDOW
  HIDMSG_LOCK,                  //  HIDCMD_ ... DRIVE
  HIDMSG_EJECT,                 //  HIDCMD_ ... DRIVE
  HIDMSG_OPEN,                  //  HIDCMD_ ... FILESECTOR, TRANSPORT
  HIDMSG_CLOSE,                 //  HIDCMD_ ... FILESECTOR
  HIDMSG_READ,                  //  HIDCMD_ ... FILESECTOR
  HIDMSG_READTOC,               //  HIDCMD_ ... FILESECTOR, TRANSPORT
  HIDMSG_READCDTEXT,            //  HIDCMD_ ... FILESECTOR
  HIDMSG_READFULLTOC,           //  HIDCMD_ ... FILESECTOR
  HIDMSG_READSESSIONINFO,       //  HIDCMD_ ... FILESECTOR
  HIDMSG_START,                 //  HIDCMD_ ... TRANSPORT
  HIDMSG_CHANNEL,               //  HIDCMD_ ... GETAUDIO
  HIDMSG_SEEK,                  //  HIDCMD_ ... FILESECTOR
  HIDMSG_TIME,                  //  HIDCMD_ ... SETSYSTEM
  HIDMSG_BLANK,                 //  HIDCMD_ ... SETVIDEO
  HIDMSG_BRIGHT,                // 
  HIDMSG_HUE,
  HIDMSG_SATURATION,
  HIDMSG_CONTRAST,
  HIDMSG_GHAL_CONSTRUCT,
  HIDMSG_LASTERROR,
  HIDMSG_API,                   //  HIDCMD_ ... VER
  HIDMSG_PRODUCT,               //  HIDCMD_ ... VER
  HIDMSG_FILE,                  //  HIDCMD_ ... VER
  HIDMSG_CAPS,                  //  HIDCMD_ ... GETSYSTEM
  HIDMSG_VOLUMEID,              //  HIDCMD_ ... GETSYSTEM
  HIDMSG_SPEAKER,               //  HIDCMD_ ... SETAUDIO
  HIDMSG_FLUSH                  //  HIDCMD_ ... SETCALLBACKTYPE, STOP
} HIDMSG;

/*****************************************
 **         SubMessages                 **
 *****************************************/
enum
{
  HIDSMSG_AUD_SPDIFF = 0,
  HIDSMSG_AUD_MIX,
  HIDSMSG_AUD_SPENBL
};

/*****************************************
 **         HiFlag                      **
 *****************************************/
enum
{
  HIDFLAG_UNKNOWN   = 0x00,
  HIDFLAG_NOTIFY    = 0x01,
  HIDFLAG_TEST      = 0x02
};

/*****************************************
 **         HiCallbackTypes             **
 *****************************************/
enum
{
  HID_CALLBACK_SIGNAL     = 0x0001,
  HID_CALLBACK_NOTIFY     = 0x0002,
  HID_CALLBACK_TRANSPORT  = 0x0004,
  HID_CALLBACK_ERRORS     = 0x0008,
  HID_CALLBACK_STREAM     = 0x0010
};

/*****************************************
 **         HiNotify                    **
 *****************************************/
enum
{
  HID_NOTIFY_SUCCESSFUL = 0x01,
  HID_NOTIFY_SUPERSEDED = 0x02,
  HID_NOTIFY_ABORTED    = 0x04,
  HID_NOTIFY_FAILURE    = 0x08
};

/*****************************************
 **         HiTransportCallback         **
 *****************************************/
enum
{
  HID_CB_TS_STOPPED = 0,
  HID_CB_TS_STARTED,
  HID_CB_TS_SIGNAL_LOSS,
  HID_CB_TS_TABLE_CHANGED,
  HID_CB_TS_CHANNEL_STOPPED,
  HID_CB_TS_CHANNEL_STARTED
};

enum
{
  HID_TRANSFER_PARALLEL   = 0x0001,           // in case of Transport Stream and Memory playback
  HID_TRANSFER_SERIAL_LSB = 0x0002,           // in case of Transport Stream and Memory Playback
  HID_TRANSFER_SERIAL_MSB = 0x0003            // in case of Transport Stream and Memory Playback
};

enum
{
  HID_STREAM_VIDEO    = 0,
  HID_STREAM_AUDIO    = 1,
  HID_STREAM_SUBPICT  = 2
};

// Values for Video
enum
{
  HID_BOB = 0,
  HID_WEAVE
};

enum
{
  HID_COLOR = 0,
  HID_INDEX
};

enum
{
  HID_VIDSRC_DISABLE = 0,
  HID_VIDSRC_SVIDEO,
  HID_VIDSRC_COMPOSITE,
  HID_VIDSRC_SVID_COMP
};

enum
{
  HID_VIDOUT_DIGITAL_DISABLE = 0,
  HID_VIDOUT_DIGITAL_YUV,
  HID_VIDOUT_DIGITAL_RGB
};

enum
{
  HID_VIDMODE_DEFAULT = 0,
  HID_VIDMODE_FORCE_NTSC,
  HID_VIDMODE_FORCE_PAL
};

enum
{
  HID_SPEAKER_LEFT            = 0x0001,
  HID_SPEAKER_RIGHT           = 0x0002,
  HID_SPEAKER_LEFT_SURROUND   = 0x0004,
  HID_SPEAKER_RIGHT_SURROUND  = 0x0008,
  HID_SPEAKER_CENTER          = 0x0010,
  HID_SPEAKER_SUBWOOFER       = 0x0020
};

enum
{
  HID_VCAPS_MACROVISIONLEVEL  = 0x0001,
  HID_VCAPS_BOTH_COMP_SVIDEO  = 0x0002,
  HID_VCAPS_BRIGHTNESS        = 0x0004,
  HID_VCAPS_HUE               = 0x0008,
  HID_VCAPS_CONTRAST          = 0x0010,
  HID_VCAPS_SATURATION        = 0x0020,
  HID_VCAPS_DIGITALOUTPUT     = 0x0040,
  HID_VCAPS_FORCE_NTSC_PAL    = 0x0080,
  HID_VCAPS_POSITION          = 0x0100,
  HID_VCAPS_SCALING           = 0x0200
};

/*****************************************
 ** HiMsg for Video Set/Get             **
 *****************************************/
enum
{
  HID_DISPLAY_NORMAL = 0,
  HID_DISPLAY_WIDE,
  HID_DISPLAY_LETTERBOX,
  HID_DISPLAY_PANSCAN
};

enum
{
  HID_MACROVISION_NONE = 0,
  HID_MACROVISION_AGC,
  HID_MACROVISION_AGC2,
  HID_MACROVISION_AGC4
};


enum
{
  HID_FMT_UNKNOWN = 0,
  HID_FMT_BYTE,
  HID_FMT_TIME,
  HID_FMT_FRAME,
  HID_FMT_SECTOR
};

typedef enum
{
  HID_STATE_NOT_READY = 0,
  HID_STATE_STOPPED,
  HID_STATE_PLAYING,
  HID_STATE_PAUSED
} eHID_STATE;

enum
{
  HID_SYSTEM_NORMAL = 0,
  HID_SYSTEM_TEST
};

enum
{
  HID_AUD_UNKNOWN = 0x0000,
  HID_AUD_MPEG    = 0x0001,
  HID_AUD_AC3     = 0x0002,
  HID_AUD_LPCM    = 0x0004,
  HID_AUD_DTS     = 0x0008,
  HID_AUD_SDDS    = 0x0010,
  HID_AUD_CDDA    = 0x0020,
  HID_AUD_MP3     = 0x0040
};

enum
{
  HID_AUD_STEREO = 0,
  HID_AUD_RIGHT,
  HID_AUD_LEFT,
  HID_AUD_MIX_MONO
};

enum
{
  HID_SPDIF_LINEAR          = 0x0001,     // PCM Only
  HID_SPDIF_MPEG            = 0x0002,
  HID_SPDIF_AC3             = 0x0004,
  HID_SPDIF_DTS             = 0x0008,
  HID_SPDIF_AC3_MPEG        = HID_SPDIF_AC3|HID_SPDIF_MPEG,
  HID_SPDIF_AC3_DTS         = HID_SPDIF_AC3|HID_SPDIF_DTS,
  HID_SPDIF_MPEG_DTS        = HID_SPDIF_MPEG|HID_SPDIF_DTS,
  HID_SPDIF_AC3_MPEG_DTS    = HID_SPDIF_AC3|HID_SPDIF_MPEG|HID_SPDIF_DTS
};

enum
{
  HID_MIXMODE_STEREO      = 0x0001,
  HID_MIXMODE_PROLOGIC    = 0x0002,
  HID_MIXMODE_3D          = 0x0004,
  HID_MIXMODE_6CHANNEL    = 0x0008
};

enum
{
  HID_KARAOKE_OFF = 0,
  HID_KARAOKE_NOVOCAL,
  HID_KARAOKE_VOCAL1,
  HID_KARAOKE_VOCAL2,
  HID_KARAOKE_VOCAL12
};

enum
{
  HID_WIN_DESTINATION = 0,
  HID_WIN_SOURCE,
  HID_WIN_WINDOW,
  HID_WIN_CLIENT,
  HID_WIN_DESTINATION_MAX,
  HID_WIN_WINDOW_MAX,
  HID_WIN_SOURCE_MAX
};

/*****************************************
 ** Param for TrickPlay                 **
 *****************************************/
enum
{
  HID_FORWARD = 0,
  HID_REVERSE
};

enum
{
  HID_SPEEDMODE_ONEFRAME = 0x0001
};

/*****************************************
 ** Msg for Getversion                  **
 *****************************************/
enum
{
  HID_HIDRV = 0,
  HID_LODRV,
  HID_ATAPI
};

typedef enum
{
  HID_MEDIA_UNKNOWN        = 0,
  HID_MEDIA_DVD            = 1,
  HID_MEDIA_AUDIO_DVD      = 2,
  HID_MEDIA_VIDEOCD        = 3,
  HID_MEDIA_AUDIOCD        = 4,
  HID_MEDIA_FILE           = 5,
  HID_MEDIA_MP3            = 6,
  HID_MEDIA_MPEG_TS        = 7,
  HID_MEDIA_SVIDEOCD       = 8,
  HID_MEDIA_SAUDIOCD       = 9
} HID_MEDIA;

typedef struct
{
  DWORD   dwStrSize;          // Should be set to the sizeof(HIDRV_LOADFILE)

  LPTSTR lpFileName;          // File Name is used when dwFrom is not given 
                              // or System does not support Sector reading

  DWORD   dwFrom;             // Start Sector number of the file (Set fFrom)
                              // or Locate start sector from the sector given in dwFrom (Set fLocateFrom)

  DWORD   dwTo;               // End Sector number of the file (Set fTo)
                              // or Locate end sector from the sector given in dwTo (Set fLocateTo)

  LPVOID  lpFileRef;          // Reference Handle (Set fRef)

  BYTE    fFrom;              // Set when dwFrom is valid
  BYTE    fTo;                // Set when dwTo is valid
  BYTE    fTest;
  BYTE    fConcat;            // Set when the files are to be logically concat.

  BYTE    fRef;               // Set when lpFileRef is valid
  BYTE    fLocateFrom;        // Set when dwFrom is Locate From sector
  BYTE    fLocateTo;          // Set when dwTo is Locate To sector
  BYTE    fDontUseCss;        // Set when not to use Css

  BYTE    bMediaType;          // Refer to HID_MEDIA
  BYTE    bReserved;

  WORD    wLocateFromOffset;  // Byte offset in the sector located by dwFrom
                              // when fLocateFrom is set

  WORD    wLocateToOffset;    // Byte offset in the sector located by dwTo

  WORD    wSectorSize;        // Should be set for 2048 for sector load or 1 for byte

} HIDRV_LOADFILE;
typedef HIDRV_LOADFILE FAR* LPHIDRV_LOADFILE;


typedef struct
{
  DWORD   dwStrSize;
  DWORD   dwFrom;
  DWORD   dwTo;
  DWORD   dwSkip;
  DWORD   dwRead;
  LPDWORD lpdwSubList;
  DWORD   dwSubListEntries;

  BYTE    fFrom;
  BYTE    fTo;
  BYTE    fTest;
  BYTE    fConcat;

  BYTE    fIFrame;
  BYTE    fSkipFirst;
  BYTE    fSkipAndRead;
  BYTE    fReadAll;

  BYTE    fReverse;
  BYTE    fPreNotify;
  BYTE    bReserved[6];

} HIDRV_PLAY;
typedef HIDRV_PLAY FAR* LPHIDRV_PLAY;


typedef struct
{
  DWORD   dwStrSize;
  DWORD   dwTo;
  DWORD   dwRef;
  DWORD   dwReturnPos;
} HIDRV_SEEK;
typedef HIDRV_SEEK FAR* LPHIDRV_SEEK;

typedef struct
{
  DWORD dwStrSize;
  DWORD dwCmd;
  DWORD dwParam1;
  DWORD dwParam2;
  DWORD dwReturn;
  DWORD dwReserved;  // For pack alignment of 8 bytes
} HIDRV_SEND;
typedef HIDRV_SEND FAR* LPHIDRV_SEND;

typedef struct
{
  DWORD   dwStrSize;
  LPBYTE  lpBuffer;
  DWORD   dwSize;
  DWORD   dwReserved;  // For pack alignment of 8 bytes
} HIDRV_OSD;
typedef HIDRV_OSD FAR* LPHIDRV_OSD;

typedef struct
{
  DWORD   dwStrSize;
  LPBYTE  lpBuffer;
  DWORD   dwSize;
  DWORD   dwBytesInBuffer;
  DWORD   dwUserData;
} HIDRV_BUFF;
typedef HIDRV_BUFF FAR* LPHIDRV_BUFF;

typedef struct
{
  DWORD   dwStrSize;
  WORD    wSpeed;
  BYTE    bDirection;
  BYTE    fIsApplyTimeValid;
  DWORD   dwApplyTime;
  DWORD   dwMode;
} HIDRV_SPEED;
typedef HIDRV_SPEED FAR* LPHIDRV_SPEED;

typedef struct
{
  LPCSTR version_msg;             /* NULL or msg pointer */

  DWORD  dwMajor;
  DWORD  dwMinor;
  DWORD  dwBuild;

  DWORD  dwYear;

  BYTE   bDay;
  BYTE   bDate;
  BYTE   bMonth;

  BYTE   bHour;
  BYTE   bMinutes;
  BYTE   bSeconds;

  BYTE   bReserved1;
  BYTE   bReserved2;
}HIDRV_VERSIONINFO;
typedef HIDRV_VERSIONINFO FAR* LPHIDRV_VERSIONINFO;

typedef struct
{
  BYTE   bRegionNumber;         // Drive region number
  BYTE   bAttempt;              // Attempts left
  BYTE   bMaxAttempts;          // Max Attempts
  BYTE   bRegionDrive;          // Set when Region drive
  BYTE   bCheckDiscRegion;      // This is for GetDiscRegion.
  BYTE   bIsDiscRegionValid;    // Set when Disc region flags is valid
  BYTE   bDiscRegionFlags;      // Disc region flags
  BYTE   bDrive;                // Drive Number got from drive letter
}HIDRV_REGION;
typedef HIDRV_REGION FAR* LPHIDRV_REGION;

typedef struct
{
  BYTE  bTransferMode;
  BYTE  bAlignmentType;
  BYTE  bPlaybackMode;
}HIDRV_TSPORTCONF;
typedef HIDRV_TSPORTCONF FAR* LPHIDRV_TSPORTCONF;

typedef struct
{
  WORD    wCurrentChannel;
  WORD    wNumberOfChannels;
  LPWORD  lpwChannelList;
} HIDRV_TSCHANNELS;
typedef HIDRV_TSCHANNELS FAR* LPHIDRV_TSCHANNELS;


typedef struct
{
  WORD  wAudioSource;     // Supported Audio Source formats (Refer HID_AUD_...)
  WORD  wKaraokeSource;   // Supported Karaoke Source formats (Refer HID_AUD_...)

  BYTE  bNumbOfChannels;  // Supported Number of output Channels
  BYTE  bMixMode;         // Supported Audio Mix Mode (Refer HID_MIXMODE_...)
  BYTE  bSpDiffMode;      // Supported Audio SpDiff Mode (Refer HID_SPDIF_...)
  BYTE  bReserved;
} HIDRV_AUD_CAPS;
typedef HIDRV_AUD_CAPS FAR* LPHIDRV_AUD_CAPS;

typedef struct
{
  WORD  wVideoControl;
  WORD  wReserved;
} HIDRV_VID_CAPS;
typedef HIDRV_VID_CAPS FAR* LPHIDRV_VID_CAPS;

typedef struct
{
  HIDRV_AUD_CAPS  AudCaps;
  HIDRV_VID_CAPS  VidCaps;
} HIDRV_CAPS;
typedef HIDRV_CAPS FAR* LPHIDRV_CAPS;

typedef struct
{
  BYTE bMixValue;   // 0x00 to 0xFF where 0xFF is max mix weight video and will blank the display
  BYTE bY;
  BYTE bU;
  BYTE bV;

  BYTE fApplyNow;
  BYTE bReserved[3];
} HIDRV_BLANK_VIDEO;
typedef HIDRV_BLANK_VIDEO FAR* LPHIDRV_BLANK_VIDEO;

typedef struct
{
  long lStartVertical;
  long lStartHorizontal;
  long lEndVertical;
  long lEndHorizontal;

  long lStartDispHorizontal;
  long lStartDispVertical;

  WORD wScaleVertical;
  WORD wScaleHorizontal;

  BYTE fApplyNow;
  BYTE fToPosition;
  BYTE fToScale;
  BYTE fToCrop;
  BYTE fReserved;

} HIDRV_VIDPOS_SCALE;
typedef HIDRV_VIDPOS_SCALE FAR* LPHIDRV_VIDPOS_SCALE;

typedef struct
{
  BYTE      bTestVideo;         // Set to TRUE to test video
  BYTE      bTestAudio;         // Set to TRUE to test audio
  BYTE      bVideoMode;         // Refer to HID_VIDMODE_...
  BYTE      bAudioSpeakers;     // Refer to HID_AUDSPEAKER_...
} HIDRV_SYSTEM_TEST;
typedef HIDRV_SYSTEM_TEST FAR* LPHIDRV_SYSTEM_TEST;


typedef struct
{
  BYTE      bOffSpeakers;         // Refer to HID_AUDSPEAKER_...
  BYTE      bSmallSpeakers;       // Refer to HID_AUDSPEAKER_...
  BYTE      bMediumSpeakers;      // Refer to HID_AUDSPEAKER_...
  BYTE      bLargeSpeakers;       // Refer to HID_AUDSPEAKER_...
} HIDRV_AUDSPEAKER_SIZE;
typedef HIDRV_AUDSPEAKER_SIZE FAR* LPHIDRV_AUDSPEAKER_SIZE;

typedef DWORD HHIDRV;
typedef LONG (*LPFNHIDRVMSG)(HHIDRV, DWORD, DWORD, DWORD);


#ifdef _WINDOWS
#ifdef _WIN32
long __declspec(dllexport) HiDrvSendMsg(HHIDRV hHiDrv, DWORD dwCmd, DWORD dwParam1, DWORD dwParam2);
#else
long _far _pascal _loadds HiDrvSendMsg(HHIDRV hHiDrv, DWORD dwCmd, DWORD dwParam1, DWORD dwParam2);
#endif
#else
  DLLEXPORT long HiDrvSendMsg(HHIDRV hHiDrv, DWORD dwCmd, DWORD dwParam1, DWORD dwParam2);
#endif

#define HIDRV_MSG_EXPORT_NAME "HiDrvSendMsg"

typedef long (*PHIDRV_CALLBACK)(DWORD dwHandle, DWORD dwCmd, DWORD dwParam1, DWORD dwParam2);
#ifdef __cplusplus
  }
#endif //__cplusplus

#endif

/* eof */
