/*****************************************************************************
*
* Copyright (c) 2000, Margi Systems, Inc. Fremont, CA, U.S.A.
* All Rights Reserved.
*
******************************************************************************/
#ifndef __MARGI_ERRORS_H__
#define __MARGI_ERRORS_H__  0x0100


typedef struct sMARGI_ERR
{
  long                lErr;
  char*               pcModule;
  unsigned long       dwLineNumb;
  union
  {
    struct sMARGI_ERR*  pErrorChained;
    long                lErrChained;
  } u;
} MARGI_ERR;

#define INITIALIZE_MARGI_ERRORS() static char gcModule[] = __FILE__;

#define IS_MARGI_ERR_CODE(Error) ((Error==0) || ((Error&0xFFF00000)==0xFFF00000))

#define SET_CHAIN_TO_MARGI_ERR(Error, ErrorToChain) \
                                          {\
                                            if(IS_MARGI_ERR_CODE(Error)==0)\
                                            {\
                                              MARGI_ERR* pMargiErr = (MARGI_ERR*)Error;\
                                              pMargiErr->u.pErrorChained = (MARGI_ERR*)ErrorToChain;\
                                            }\
                                          }

#define SET_CHAINED_MARGI_ERR(Error, MargiErrCode, Reserved, ErrorToChain) \
                                          {\
                                            static MARGI_ERR MargiErr = {0};\
                                            MargiErr.lErr       = MargiErrCode;\
                                            MargiErr.pcModule   = gcModule;\
                                            MargiErr.dwLineNumb = __LINE__;\
                                            MargiErr.u.pErrorChained = (struct sMARGI_ERR*)ErrorToChain;\
                                            Error = (long)&MargiErr; \
                                          }

#define SET_MARGI_ERR(Error, MargiErrCode, Reserved) SET_CHAINED_MARGI_ERR(Error, MargiErrCode, Reserved, 0)

#define GET_MARGI_ERR_CODE(Error) (IS_MARGI_ERR_CODE(Error)?Error:((MARGI_ERR*)Error)->lErr)

#define GET_MARGI_ERR_MODULE(Error) (IS_MARGI_ERR_CODE(Error)?0:((MARGI_ERR*)Error)->pcModule)

#define GET_MARGI_ERR_LINE(Error) (IS_MARGI_ERR_CODE(Error)?0:((MARGI_ERR*)Error)->dwLineNumb)

#define GET_CHAINED_MARGI_ERR(Error) (IS_MARGI_ERR_CODE(Error)?0:((MARGI_ERR*)Error)->u.lErrChained)

/*************************************************************************/
/** This file contains general-purpose error codes.                     **/
/** It is shared across many projects, so keep that in mind when adding **/
/** new constants.                                                      **/
/**                                                                     **/
/** Also, please update IdentErr.c when adding new error codes to avoid **/
/** getting a bunch of UNK messages when using Ident().                 **/
/**                                                         Chip        **/
/*************************************************************************/
#ifdef MARGIERR_STRING_CODE
  #ifndef TCHAR
    #define TCHAR char
  #endif
  #ifndef TEXT
    #define TEXT(xx) xx
  #endif
  #define MARGIERR_STRING(Err, ErrString) TCHAR string##Err[] = TEXT(ErrString);
  #define MARGIERR_GET_STRING(Err) string##Err
#else
  #define MARGIERR_STRING(Err, String)
  #define MARGIERR_GET_STRING(Err)
#endif

#define MERRSTR MARGIERR_STRING

#define MARGI_ERR_NONE                              (0)
MERRSTR(MARGI_ERR_NONE,                             "Ok")

#define MARGI_ERR_GENERIC                           (-1)
MERRSTR(MARGI_ERR_GENERIC,                          "Generic error")

#define MARGI_ERR_BADPTR                            (-2)
MERRSTR(MARGI_ERR_BADPTR,                           "Bad pointer")

#define MARGI_ERR_BADSIZE                           (-3)
MERRSTR(MARGI_ERR_BADSIZE,                          "Bad parameter size")

#define MARGI_ERR_BADPARM                           (-4)
MERRSTR(MARGI_ERR_BADPARM,                          "Bad parameter")

#define MARGI_ERR_TOOLONG                           (-5)
MERRSTR(MARGI_ERR_TOOLONG,                          "Parameter (string?) is too long")

#define MARGI_ERR_NOMEM                             (-6)
MERRSTR(MARGI_ERR_NOMEM,                            "Unable to allocate memory")

#define MARGI_ERR_NOTFOUND                          (-7)
MERRSTR(MARGI_ERR_NOTFOUND,                         "Item not found")

#define MARGI_ERR_UNHOOK                            (-8)
MERRSTR(MARGI_ERR_UNHOOK,                           "Unable to unhook")

#define MARGI_ERR_HOOK                              (-9)
MERRSTR(MARGI_ERR_HOOK,                             "Unable to hook")

#define MARGI_ERR_DUPLICATE                         (-10)
MERRSTR(MARGI_ERR_DUPLICATE,                        "Duplicate entry")

#define MARGI_ERR_TOOMANY                           (-11)
MERRSTR(MARGI_ERR_TOOMANY,                          "Too many items")

/** Unused (-12)  **/

#define MARGI_ERR_BADHANDLE                         (-13)
MERRSTR(MARGI_ERR_BADHANDLE,                        "Bad handle")

#define MARGI_ERR_BUSY                              (-14)
MERRSTR(MARGI_ERR_BUSY,                             "Operation is busy")

#define MARGI_ERR_DMA_REGION                        (-15)
MERRSTR(MARGI_ERR_DMA_REGION,                       "Unable to use region for DMA")

#define MARGI_ERR_DMA_NOINTR                        (-16)
MERRSTR(MARGI_ERR_DMA_NOINTR,                       "Unable to obtain an interrupt for DMA")

#define MARGI_ERR_ENCRYPTION                        (-17)
MERRSTR(MARGI_ERR_ENCRYPTION,                       "Decryption authentication failed")

#define MARGI_ERR_NOCSS                             (-18)
MERRSTR(MARGI_ERR_NOCSS,                            "CSS decryption not supported")

#define MARGI_ERR_BADITEM                           (-19)
MERRSTR(MARGI_ERR_BADITEM,                          "Invalid item")

#define MARGI_ERR_READONLY                          (-20)
MERRSTR(MARGI_ERR_READONLY,                         "ReadOnly (write denied)")

#define MARGI_ERR_NOT_IMPLEMENTED                   (-21)
MERRSTR(MARGI_ERR_NOT_IMPLEMENTED,                  "Not implemented")

#define MARGI_ERR_AUTHENTICATION                    (-23)
MERRSTR(MARGI_ERR_AUTHENTICATION,                   "Authentication error")

#define MARGI_ERR_BADCMD                            (-24)
MERRSTR(MARGI_ERR_BADCMD,                           "Bad command")

#define MARGI_ERR_PREREQUISITE                      (-25)
MERRSTR(MARGI_ERR_PREREQUISITE,                     "Prerequisite not met")

#define MARGI_ERR_BADTAG                            (-26)
MERRSTR(MARGI_ERR_BADTAG,                           "Bad Tag ID")

#define MARGI_ERR_ARG1                              (-27)
MERRSTR(MARGI_ERR_ARG1,                             "Bad Arg1")

#define MARGI_ERR_ARG2                              (-28)
MERRSTR(MARGI_ERR_ARG2,                             "Bad Arg2")

#define MARGI_ERR_NOTREADY                          (-29)
MERRSTR(MARGI_ERR_NOTREADY,                         "NotReady")

#define MARGI_ERR_BADLIST                           (-30)
MERRSTR(MARGI_ERR_BADLIST,                          "List has been corrupted")

#define MARGI_ERR_BADTYPE                           (-31)
MERRSTR(MARGI_ERR_BADTYPE,                          "Bad type")

#define MARGI_ERR_BADSIG                            (-32)
MERRSTR(MARGI_ERR_BADSIG,                           "Bad Signature")

#define MARGI_ERR_CANT_START                        (-33)
MERRSTR(MARGI_ERR_CANT_START,                       "Unable to Start")

#define MARGI_ERR_WRT_FAILED                        (-34)
MERRSTR(MARGI_ERR_WRT_FAILED,                       "Failed to Write")

#define MARGI_ERR_WRT_STALLED                       (-35)
MERRSTR(MARGI_ERR_WRT_STALLED,                      "Write Stalled")

#define MARGI_ERR_PHYSMAP                           (-36)
MERRSTR(MARGI_ERR_PHYSMAP,                          "MapErr Phys2Linear")

#define MARGI_ERR_IGNORED                           (-37)
MERRSTR(MARGI_ERR_IGNORED,                          "Command or message ignored")

#define MARGI_ERR_BADCALLBACK                       (-38)
MERRSTR(MARGI_ERR_BADCALLBACK,                      "Bad callback or callback not set")

#define MARGI_ERR_UNABLE_STOP_CHNL                  (-39)
MERRSTR(MARGI_ERR_UNABLE_STOP_CHNL,                 "Unable to Stop Channel")

#define MARGI_ERR_UNABLE_START_CHNL                 (-40)
MERRSTR(MARGI_ERR_UNABLE_START_CHNL,                "Unable to Start Channel")

#define MARGI_ERR_INVALID_STATE_TRANSITION          (-41)
MERRSTR(MARGI_ERR_INVALID_STATE_TRANSITION,         "Invalid State Transition")

#define MARGI_ERR_NO_ACTION_REQD                    (-42)
MERRSTR(MARGI_ERR_NO_ACTION_REQD,                   "Already in Same State")

#define MARGI_ERR_INVALID_HW_STATE                  (-43)
MERRSTR(MARGI_ERR_INVALID_HW_STATE,                 "InValid HW State")

#define MARGI_ERR_TIMEOUT                           (-44)
MERRSTR(MARGI_ERR_TIMEOUT,                          "Timeout")

#define MARGI_ERR_UNK_MSG                           (-45)
MERRSTR(MARGI_ERR_UNK_MSG,                          "Unkown Message")

#define MARGI_ERR_NOTOPEN                           (-46)
MERRSTR(MARGI_ERR_NOTOPEN,                          "Not open")

#define MARGI_ERR_ABORT                             (-47)
MERRSTR(MARGI_ERR_ABORT,                            "Aborted")

#define MARGI_ERR_PARSE                             (-48)
MERRSTR(MARGI_ERR_PARSE,                            "Parsing error")

#define MARGI_ERR_ACCESS                            (-49)
MERRSTR(MARGI_ERR_ACCESS,                           "Access denied")

#define MARGI_ERR_INSANE                            (-50)   //Failed sanity check.
MERRSTR(MARGI_ERR_INSANE,                           "Failed sanity check")

#define MARGI_ERR_REGION_MISMATCH                   (-51)
MERRSTR(MARGI_ERR_REGION_MISMATCH,                  "Media region code is mismatched to logical unit region")

#define MARGI_ERR_NOFIELD_CDB                       (-52)
MERRSTR(MARGI_ERR_NOFIELD_CDB,                      "Invalid field in CDB")

#define MARGI_ERR_OUTOFRANGE                        (-53)
MERRSTR(MARGI_ERR_OUTOFRANGE,                       "Value out of range")

#define MARGI_ERR_TIMER                             (-54)
MERRSTR(MARGI_ERR_TIMER,                            "Unable to load Timer")

#define MARGI_ERR_TIMERSTART                        (-55)
MERRSTR(MARGI_ERR_TIMERSTART,                       "Unable to start Timer")

#define MARGI_ERR_TIMERSTOP                         (-56)
MERRSTR(MARGI_ERR_TIMERSTOP,                        "Unable to stop Timer")

#define MARGI_ERR_NOT_SUPPORTED                     (-57)
MERRSTR(MARGI_ERR_NOT_SUPPORTED,                    "This feature not supported")

#define MARGI_ERR_THREAD                            (-58)
MERRSTR(MARGI_ERR_THREAD,                           "Generic Thread error")

#define MARGI_ERR_THREAD_CREATE                     (-59)
MERRSTR(MARGI_ERR_THREAD_CREATE,                    "Unable to create Thread")

#define MARGI_ERR_MUTEX                             (-60)
MERRSTR(MARGI_ERR_MUTEX,                            "Generic Mutex error")

#define MARGI_ERR_MUTEX_CREATE                      (-61)
MERRSTR(MARGI_ERR_MUTEX_CREATE,                     "Unable to create a Mutex")

#define MARGI_ERR_EVENT                             (-62)
MERRSTR(MARGI_ERR_EVENT,                            "Generic Event error")

#define MARGI_ERR_EVENT_CREATE                      (-63)
MERRSTR(MARGI_ERR_EVENT_CREATE,                     "Unable to create a Event")

#define MARGI_ERR_EVENT_SET                         (-64)
MERRSTR(MARGI_ERR_EVENT_SET,                        "Unable to activate a Event")

#define MARGI_ERR_HW_NOTFOUND                       (-65)
MERRSTR(MARGI_ERR_HW_NOTFOUND,                      "Unable to load hardware driver")

#define MARGI_ERR_NAVIGATE                          (-66)
MERRSTR(MARGI_ERR_NAVIGATE,                         "Navigation error occurred, possibility of incorrectly coded disc.")

#define MARGI_ERR_PARENTAL                          (-67)
MERRSTR(MARGI_ERR_PARENTAL,                         "Wrong Parental level set.")

#define MARGI_ERR_REGION_INT                        (-68)
MERRSTR(MARGI_ERR_REGION_INT,                       "Region code is not initialized.")

#define MARGI_ERR_REGION_LOCK                       (-69)
MERRSTR(MARGI_ERR_REGION_LOCK,                      "Region lock. ")

#define MARGI_ERR_REGION_SUP                        (-70)
MERRSTR(MARGI_ERR_REGION_SUP,                       "Region support not available in the driver. Please re-install the software.")

#define MARGI_ERR_QUE_EMPTY                         (-71)
MERRSTR(MARGI_ERR_QUE_EMPTY,                        "Empty Que.")

#define MARGI_ERR_ATAPI                             (-72)
MERRSTR(MARGI_ERR_ATAPI,                            "ATAPI error.")

/** Unused (-73)  **/

#define MARGI_ERR_INVALID_ADDR                      (-74)
MERRSTR(MARGI_ERR_INVALID_ADDR,                     "Error invalid address.")

/** Unused (-75)  **/

#define MARGI_ERR_HEAT                              (-76)
MERRSTR(MARGI_ERR_HEAT,                             "Over Heat Discontinue until the temperature drops")

#define MARGI_ERR_NO_KEY                            (-77)
MERRSTR(MARGI_ERR_NO_KEY,                           "No key found.")

#define MARGI_ERR_INVALID_KEY                       (-78)
MERRSTR(MARGI_ERR_INVALID_KEY,                      "Invalid key.")

#define MARGI_ERR_NOTDONE                           (-79)
MERRSTR(MARGI_ERR_NOTDONE,                          "Operation not done.")

#define MARGI_ERR_UNDERFLOW                         (-80)
MERRSTR(MARGI_ERR_UNDERFLOW,                        "Buffer Underflow.")

#define MARGI_ERR_OVERFLOW                          (-81)
MERRSTR(MARGI_ERR_OVERFLOW,                         "Buffer Overflow.")

#define MARGI_ERR_PARITY                            (-82)
MERRSTR(MARGI_ERR_PARITY,                           "Parity Error.")

#define MARGI_ERR_FRAMING                           (-83)
MERRSTR(MARGI_ERR_FRAMING,                          "Framing Error.")

#define MARGI_ERR_INTERRUPT                         (-84)
MERRSTR(MARGI_ERR_INTERRUPT,                        "Interrupt Error.")

#define MARGI_ERR_INT_INSTALL                       (-85)
MERRSTR(MARGI_ERR_INT_INSTALL,                      "Interrupt Install Error.")

#define MARGI_ERR_ILLEGAL_FLAGS                     (-86)
MERRSTR(MARGI_ERR_ILLEGAL_FLAGS,                    "Illegal Flags.")

#define MARGI_ERR_NO_ROOM                           (-87)
MERRSTR(MARGI_ERR_NO_ROOM,                          "Buffer Full.")

#define MARGI_ERR_BADSTATE                          (-88)
MERRSTR(MARGI_ERR_BADSTATE,                         "Command called in wrong state.")

#define MARGI_ERR_ILLEGAL_MODE                      (-89)
MERRSTR(MARGI_ERR_ILLEGAL_MODE,                     "Illegal Mode.")

/** Unused (-90) to (-101) **/

#define MARGI_ERR_LOAD                              (-102)  //Unable to load a file (vxd)
MERRSTR(MARGI_ERR_LOAD,                             "Unable to load VxD")

#define MARGI_ERR_BADVERSION                        (-103)  //Version mismatch between app and vxd.
MERRSTR(MARGI_ERR_BADVERSION,                       "App/VxD version mismatch")

#define MARGI_ERR_UNLOAD                            (-104)  //Unable to unload vxd.
MERRSTR(MARGI_ERR_UNLOAD,                           "Unable to unload VxD")

#define MARGI_ERR_FILTERS                           (-105)
MERRSTR(MARGI_ERR_FILTERS,                          "Unable to set or get message filters")

#define MARGI_ERR_OBSOLETE                          (-107)  //No longer supported.
MERRSTR(MARGI_ERR_OBSOLETE,                         "Obsolete")

#define MARGI_ERR_NOTAVAILABLE                      (-108)  //Function is not available (IE: debug function in release build)
MERRSTR(MARGI_ERR_NOTAVAILABLE,                     "Not available")

#define MARGI_ERR_NOMEDIA                           (-109)
MERRSTR(MARGI_ERR_NOMEDIA,                          "No media")

#define MARGI_ERR_NOMEDIA_TRAY_CLOSED               (-110)
MERRSTR(MARGI_ERR_NOMEDIA_TRAY_CLOSED,              "No media and tray closed")

#define MARGI_ERR_NOMEDIA_TRAY_OPEN                 (-111)
MERRSTR(MARGI_ERR_NOMEDIA_TRAY_OPEN,                "No media and tray open")

#define MARGI_ERR_MEDIA_MAY_HAVE_CHANGED            (-112)
MERRSTR(MARGI_ERR_MEDIA_MAY_HAVE_CHANGED,           "Media may have changed")

#define MARGI_ERR_MEDIA_BAD_TOC                     (-113)
MERRSTR(MARGI_ERR_MEDIA_BAD_TOC,                    "Media has bad table of content")

#define MARGI_ERR_MEDIA_UNKNOWN_FORMAT              (-114)
MERRSTR(MARGI_ERR_MEDIA_UNKNOWN_FORMAT,             "Cannont read media - unknown format")

#define MARGI_ERR_MEDIA_INCOMPATIBLE_FORMAT         (-115)
MERRSTR(MARGI_ERR_MEDIA_INCOMPATIBLE_FORMAT,        "Incompatible media installed")

#define MARGI_ERR_MEDIA_LOAD_EJECT_FAILURE          (-116)
MERRSTR(MARGI_ERR_MEDIA_LOAD_EJECT_FAILURE,        "Medium Load or Eject operation failed")

/** Unused (-117) to (-119) **/

#define MARGI_ERR_BECOMING_READY                    (-120)
MERRSTR(MARGI_ERR_BECOMING_READY,                   "In process of becoming ready")

#define MARGI_ERR_NOTREADY_INITIALIZING_REQUIRED    (-121)
MERRSTR(MARGI_ERR_NOTREADY_INITIALIZING_REQUIRED,   "Not ready initializing required")

#define MARGI_ERR_RESET_OCCURED                     (-122)
MERRSTR(MARGI_ERR_RESET_OCCURED,                    "Power on reset or other reset occured")

#define MARGI_ERR_WAITING_SELF_CONFIG               (-123)
MERRSTR(MARGI_ERR_WAITING_SELF_CONFIG,              "Logical unit has not self configured yet")

/** Unused (-124) to (-139) **/

#define MARGI_ERR_NODEVICE                          (-140)
MERRSTR(MARGI_ERR_NODEVICE,                         "Device not found")

#define MARGI_ERR_DEVICE                            (-141)
MERRSTR(MARGI_ERR_DEVICE,                           "Device error")

#define MARGI_ERR_DEVICE_NOTREADY                   (-142)
MERRSTR(MARGI_ERR_DEVICE_NOTREADY,                  "Device not ready.")

#define MARGI_ERR_DEVICE_BUSY                       (-143)
MERRSTR(MARGI_ERR_DEVICE_BUSY,                      "Device is busy.")

#define MARGI_ERR_DEVICE_DISABLED                   (-144)
MERRSTR(MARGI_ERR_DEVICE_DISABLED,                  "Device is disabled.")

/** Unused (-144) to (-150) **/

#define MARGI_ERR_PORT                              (-150)
MERRSTR(MARGI_ERR_PORT,                             "Generic port error")

#define MARGI_ERR_PORT_OPEN                         (-151)
MERRSTR(MARGI_ERR_PORT_OPEN,                        "Unable to open port")

#define MARGI_ERR_PORT_WRITE                        (-152)
MERRSTR(MARGI_ERR_PORT_WRITE,                       "Port write failed")

#define MARGI_ERR_PORT_READ                         (-153)
MERRSTR(MARGI_ERR_PORT_READ,                        "Port read failed")

#define MARGI_ERR_PORT_SET_STATE                    (-154)
MERRSTR(MARGI_ERR_PORT_SET_STATE,                   "Port set state failed")

#define MARGI_ERR_PORT_GET_STATE                    (-155)
MERRSTR(MARGI_ERR_PORT_GET_STATE,                   "Port get state failed")

#define MARGI_ERR_PORT_SETUP                        (-156)
MERRSTR(MARGI_ERR_PORT_SETUP,                       "Port setup failed")

#define MARGI_ERR_PORT_FAILED                       (-157)
MERRSTR(MARGI_ERR_PORT_FAILED,                      "Port failed")

#define MARGI_ERR_PORT_INVALID                      (-158)
MERRSTR(MARGI_ERR_PORT_INVALID,                     "Invalid port")

#define MARGI_ERR_PORT_TIMEOUT                      (-159)
MERRSTR(MARGI_ERR_PORT_TIMEOUT,                     "Port timeout")

#define MARGI_ERR_PORT_EVENT                        (-160)
MERRSTR(MARGI_ERR_PORT_EVENT,                       "Port event error")

#define MARGI_ERR_NO_DETECTOR                       (-161)
MERRSTR(MARGI_ERR_NO_DETECTOR,                      "No detector found")

#define MARGI_ERR_UNKNOWN                           (-162)
MERRSTR(MARGI_ERR_UNKNOWN,                          "Unknown error")

/** Unused (-163) to (-199) **/

#define MARGI_ERR_VMIERR                            (-200)  //Unable to locate mem mapped VMI space
MERRSTR(MARGI_ERR_VMIERR,                           "Unable to locate memory-mapped VMI IO space")

/** Unused (-201) to (-209) **/

#define MARGI_ERR_HW                                (-210)
MERRSTR(MARGI_ERR_HW,                               "Hardware error")

/** Unused (-211) to (-219) **/

#define MARGI_ERR_DMA                               (-220)
MERRSTR(MARGI_ERR_DMA,                              "Dma error")

/** Unused (-221) to (-229) **/

#define MARGI_ERR_ISR                               (-230)
MERRSTR(MARGI_ERR_ISR,                              "Interrupt error")

/** Unused (-231) to (-239) **/

#define MARGI_ERR_WND                               (-240)
MERRSTR(MARGI_ERR_WND,                              "Window error")

/** Unused (-241) to (-249) **/

#define MARGI_ERR_WRK                               (-250)
MERRSTR(MARGI_ERR_WRK,                              "Work layer error")

/** Unused (-251) to (-259) **/

#define MARGI_ERR_NAV                               (-260)
MERRSTR(MARGI_ERR_NAV,                              "Navigation error")

/** Unused (-261) to (-269) **/

#define MARGI_ERR_HW_RSC                            (-270)
MERRSTR(MARGI_ERR_HW_RSC,                           "Hardware resource error")

/** Unused (-271) to (-299) **/

#define MARGI_ERR_CSS_HW_NOTFOUND                   (-300)
MERRSTR(MARGI_ERR_CSS_HW_NOTFOUND,                  "CSS error, hardware not found")

#define MARGI_ERR_CSS_TITLEKEY                      (-301)
MERRSTR(MARGI_ERR_CSS_TITLEKEY,                     "CSS error determining title key")

#define MARGI_ERR_CSS_DISCKEY                       (-302)
MERRSTR(MARGI_ERR_CSS_DISCKEY,                      "CSS error determining disc key")

#define MARGI_ERR_CSS_DECAUTH                       (-303)
MERRSTR(MARGI_ERR_CSS_DECAUTH,                      "CSS error during decoder authentication")

#define MARGI_ERR_CSS_DRVAUTH                       (-304)
MERRSTR(MARGI_ERR_CSS_DRVAUTH,                      "CSS error during drive authentication")

#define MARGI_ERR_CSS_GENCHALLENGE                  (-305)
MERRSTR(MARGI_ERR_CSS_GENCHALLENGE,                 "CSS error generating challenge")

#define MARGI_ERR_CSS_WRKEY40                       (-306)
MERRSTR(MARGI_ERR_CSS_WRKEY40,                      "CSS error writing Key40")

#define MARGI_ERR_CSS_WRDRVREFKEY                   (-307)
MERRSTR(MARGI_ERR_CSS_WRDRVREFKEY,                  "CSS error writing drive reference key")

#define MARGI_ERR_CSS_RDKEY40                       (-308)
MERRSTR(MARGI_ERR_CSS_RDKEY40,                      "CSS error reading Key40")

#define MARGI_ERR_CSS_NOBUSAUTH                     (-309)
MERRSTR(MARGI_ERR_CSS_NOBUSAUTH,                    "CSS no bus authentication")

#define MARGI_ERR_CSS_TIMEOUT                       (-310)
MERRSTR(MARGI_ERR_CSS_TIMEOUT,                      "CSS error bad VidState")

/** Unused (-311) to (-400) **/

#define MARGI_ERR_FILEOPEN                          (-401)
MERRSTR(MARGI_ERR_FILEOPEN,                         "Unable to open file")

#define MARGI_ERR_FILEREAD                          (-402)
MERRSTR(MARGI_ERR_FILEREAD,                         "Unable to read file")

#define MARGI_ERR_FILEWRITE                         (-403)
MERRSTR(MARGI_ERR_FILEWRITE,                        "Unable to write to file")

#define MARGI_ERR_FILESEEK                          (-404)
MERRSTR(MARGI_ERR_FILESEEK,                         "Unable to seek into file")

#define MARGI_ERR_FILEOPENREAD                      (-405)
MERRSTR(MARGI_ERR_FILEOPENREAD,                     "Unable to open file for reading")

#define MARGI_ERR_FILEOPENWRITE                     (-406)
MERRSTR(MARGI_ERR_FILEOPENWRITE,                    "Unable to open file for writing")

#define MARGI_ERR_BADPATH                           (-407)
MERRSTR(MARGI_ERR_BADPATH,                          "Bad pathname")

#define MARGI_ERR_BOGUSCOPY                         (-408)
MERRSTR(MARGI_ERR_BOGUSCOPY,                        "Invalid copy")

#define MARGI_ERR_EXPIRED                           (-409)
MERRSTR(MARGI_ERR_EXPIRED,                          "Time expired")

#define MARGI_ERR_FILE                              (-410)
MERRSTR(MARGI_ERR_FILE,                             "General file error")

/** Unused (-411) to (-419) **/

#define MARGI_ERR_DRIVE_OPEN                        (-420)
MERRSTR(MARGI_ERR_DRIVE_OPEN,                       "Drive open error")

/** Unused (-421) to (-450) **/

#define MARGI_ERR_GDIRESOURCE                       (-451)
MERRSTR(MARGI_ERR_GDIRESOURCE,                      "Unable to load GDI resource")

#define MARGI_ERR_GDICLOSE                          (-452)
MERRSTR(MARGI_ERR_GDICLOSE,                         "Unable to close GDI object")

#define MARGI_ERR_GDIOPEN                           (-453)
MERRSTR(MARGI_ERR_GDIOPEN,                          "Unable to open GDI object")

#define MARGI_ERR_GDICREATE                         (-454)
MERRSTR(MARGI_ERR_GDICREATE,                        "Unable to create GDI object")

#define MARGI_ERR_GDIDESTROY                        (-455)
MERRSTR(MARGI_ERR_GDIDESTROY,                       "Unable to destroy GDI object")

/** Unused (-456) to (-499) **/

#define MARGI_ERR_UDF_NODRV                         (-500)
MERRSTR(MARGI_ERR_UDF_NODRV,                        "No UDF drive letter")

#define MARGI_ERR_UDF_NOANCHOR                      (-501)
MERRSTR(MARGI_ERR_UDF_NOANCHOR,                     "No UDF anchor block")

#define MARGI_ERR_UDF_NOPVD                         (-502)
MERRSTR(MARGI_ERR_UDF_NOPVD,                        "No UDF primary volume descriptor")

#define MARGI_ERR_UDF_NOPARTITION                   (-503)
MERRSTR(MARGI_ERR_UDF_NOPARTITION,                  "No UDF partition table")

#define MARGI_ERR_UDF_NOROOTDIR                     (-504)
MERRSTR(MARGI_ERR_UDF_NOROOTDIR,                    "No UDF root directory")

#define MARGI_ERR_UDF_NOENTRY                       (-505)
MERRSTR(MARGI_ERR_UDF_NOENTRY,                      "No entry found")

/** Unused (-506) to (-599) **/

#define MARGI_ERR_OSD_GENERAL                       (-600)
MERRSTR(MARGI_ERR_OSD_GENERAL,                      "OSD General Error")

#define MARGI_ERR_OSD_PALETTE                       (-601)
MERRSTR(MARGI_ERR_OSD_PALETTE,                      "OSD Palette Init Error")

#define MARGI_ERR_OSD_PAGE_NOT_FOUND                (-602)
MERRSTR(MARGI_ERR_OSD_PAGE_NOT_FOUND,               "OSD Page Not Found")

#define MARGI_ERR_OSD_INIT_FAILED                   (-603)
MERRSTR(MARGI_ERR_OSD_INIT_FAILED,                  "OSD Initialization Failure")

#define MARGI_ERR_OSD_ILLEGAL_PARAM                 (-610)
MERRSTR(MARGI_ERR_OSD_ILLEGAL_PARAM,                "OSD Illegal Parameters")

#define MARGI_ERR_OSD_REGION_CREATE_FAILED          (-611)
MERRSTR(MARGI_ERR_OSD_REGION_CREATE_FAILED,         "OSD Region Creation Failure")

#define MARGI_ERR_OSD_INVALID_PALETTE               (-612)
MERRSTR(MARGI_ERR_OSD_INVALID_PALETTE,              "OSD Invalid Palette Format")

#define MARGI_ERR_OSD_INVALID_BITMAP                (-613)
MERRSTR(MARGI_ERR_OSD_INVALID_BITMAP,               "OSD Invalid Bitmap Format")

#define MARGI_ERR_OSD_SLOW_BITBLT_UNSUPPORTED       (-614)
MERRSTR(MARGI_ERR_OSD_SLOW_BITBLT_UNSUPPORTED,      "OSD Slow BitBlt Unsupported")

#define MARGI_ERR_OSD_BITBLT_UNALIGNED              (-615)
MERRSTR(MARGI_ERR_OSD_BITBLT_UNALIGNED,             "OSD BitBlt Src or Dst Unaligned")

#define MARGI_ERR_OSD_DATA_UNALIGNED                (-616)
MERRSTR(MARGI_ERR_OSD_DATA_UNALIGNED,               "OSD Data buffer is not dword aligned")

#define MARGI_ERR_OSD_BAD_POINTER                   (-617)
MERRSTR(MARGI_ERR_OSD_BAD_POINTER,                  "OSD Bad Pointer received")

#define MARGI_ERR_OSD_BITBLT_FAILED                 (-618)
MERRSTR(MARGI_ERR_OSD_BITBLT_FAILED,                "OSD Bitblt call failed")

#define MARGI_ERR_OSD_INCOSISTENT_REQ               (-619)
MERRSTR(MARGI_ERR_OSD_INCOSISTENT_REQ,              "OSD Inconsistent param in Request")


#if 0 //More OSD errors available.

#define MARGI_ERR_OSD_GENERAL                       (-600)
MERRSTR(MARGI_ERR_OSD_GENERAL,                      "OSD General Error")

#define MARGI_ERR_OSD_GENERAL                       (-600)
MERRSTR(MARGI_ERR_OSD_GENERAL,                      "OSD General Error")
#endif

#define MARGI_ERR_CPXM_KEYLOAD                      (-700)
MERRSTR(MARGI_ERR_CPXM_KEYLOAD,                     "CPXM keys load error or invalid keys")


/** Unused (-600) to (-899) **/

#define MARGI_ERR_NOINTERFACE                       (-900)
MERRSTR(MARGI_ERR_NOINTERFACE,                      "No interface found")

/** Unused (-901) to (-999) **/

#define MARGI_ERR_KERNEL_INIT                       (-1000)
MERRSTR(MARGI_ERR_KERNEL_INIT,                      "Kernel Init failed")

#define MARGI_ERR_KERNEL_START                      (-1001)
MERRSTR(MARGI_ERR_KERNEL_START,                     "Cannot start kernel")

/** Unused (-1002) to (-6999) **/

#define MARGI_ERR_DD_ALREADYINITIALIZED             (-7000)
MERRSTR(MARGI_ERR_DD_ALREADYINITIALIZED,            "The object has already been initialized.")

#define MARGI_ERR_DD_BLTFASTCANTCLIP                (-7001)
MERRSTR(MARGI_ERR_DD_BLTFASTCANTCLIP,               "A DirectDrawClipper object is attached to a source surface that has passed into a call to the IDirectDrawSurface4::BltFast method.")

#define MARGI_ERR_DD_CANNOTATTACHSURFACE            (-7002)
MERRSTR(MARGI_ERR_DD_CANNOTATTACHSURFACE,           "A surface cannot be attached to another requested surface.")

#define MARGI_ERR_DD_CANNOTDETACHSURFACE            (-7003)
MERRSTR(MARGI_ERR_DD_CANNOTDETACHSURFACE,           "A surface cannot be detached from another requested surface.")

#define MARGI_ERR_DD_CANTCREATEDC                   (-7004)
MERRSTR(MARGI_ERR_DD_CANTCREATEDC,                  "Windows can not create any more device contexts (DCs), or a DC was requested for a palette-indexed surface when the surface had no palette and the display mode was not palette-indexed (in this case DirectDraw cannot select a proper palette into the DC).")

#define MARGI_ERR_DD_CANTDUPLICATE                  (-7005)
MERRSTR(MARGI_ERR_DD_CANTDUPLICATE,                 "Primary and 3-D surfaces, or surfaces that are implicitly created, cannot be duplicated.")

#define MARGI_ERR_DD_CANTLOCKSURFACE                (-7006)
MERRSTR(MARGI_ERR_DD_CANTLOCKSURFACE,               "Access to this surface is refused because an attempt was made to lock the primary surface without DCI support.")

#define MARGI_ERR_DD_CANTPAGELOCK                   (-7007)
MERRSTR(MARGI_ERR_DD_CANTPAGELOCK,                  "An attempt to page lock a surface failed. Page lock will not work on a display-memory surface or an emulated primary surface.")

#define MARGI_ERR_DD_CANTPAGEUNLOCK                 (-7008)
MERRSTR(MARGI_ERR_DD_CANTPAGEUNLOCK,                "An attempt to page unlock a surface failed. Page unlock will not work on a display-memory surface or an emulated primary surface.")

#define MARGI_ERR_DD_CLIPPERISUSINGHWND             (-7009)
MERRSTR(MARGI_ERR_DD_CLIPPERISUSINGHWND,            "An attempt was made to set a clip list for a DirectDrawClipper object that is already monitoring a window handle.")

#define MARGI_ERR_DD_COLORKEYNOTSET                 (-7010)
MERRSTR(MARGI_ERR_DD_COLORKEYNOTSET,                "No source color key is specified for this operation.")

#define MARGI_ERR_DD_CURRENTLYNOTAVAIL              (-7011)
MERRSTR(MARGI_ERR_DD_CURRENTLYNOTAVAIL,             "No support is currently available.")

#define MARGI_ERR_DD_DCALREADYCREATED               (-7012)
MERRSTR(MARGI_ERR_DD_DCALREADYCREATED,              "A device context (DC) has already been returned for this surface. Only one DC can be retrieved for each surface.")

#define MARGI_ERR_DD_DEVICEDOESNTOWNSURFACE         (-7013)
MERRSTR(MARGI_ERR_DD_DEVICEDOESNTOWNSURFACE,        "Surfaces created by one DirectDraw device cannot be used directly by another DirectDraw device.")

#define MARGI_ERR_DD_DIRECTDRAWALREADYCREATED       (-7014)
MERRSTR(MARGI_ERR_DD_DIRECTDRAWALREADYCREATED,      "A DirectDraw object representing this driver has already been created for this process.")

#define MARGI_ERR_DD_EXCEPTION                      (-7015)
MERRSTR(MARGI_ERR_DD_EXCEPTION,                     "An exception was encountered while performing the requested operation.")

#define MARGI_ERR_DD_EXCLUSIVEMODEALREADYSET        (-7016)
MERRSTR(MARGI_ERR_DD_EXCLUSIVEMODEALREADYSET,       "An attempt was made to set the cooperative level when it was already set to exclusive.")

#define MARGI_ERR_DD_EXPIRED                        (-7017)
MERRSTR(MARGI_ERR_DD_EXPIRED,                       "The data has expired and is therefore no longer valid.")

#define MARGI_ERR_DD_GENERIC                        (-7018)
MERRSTR(MARGI_ERR_DD_GENERIC,                       "There is an undefined error condition.")

#define MARGI_ERR_DD_HEIGHTALIGN                    (-7019)
MERRSTR(MARGI_ERR_DD_HEIGHTALIGN,                   "The height of the provided rectangle is not a multiple of the required alignment.")

#define MARGI_ERR_DD_HWNDALREADYSET                 (-7020)
MERRSTR(MARGI_ERR_DD_HWNDALREADYSET,                "The DirectDraw cooperative level window handle has already been set. It cannot be reset while the process has surfaces or palettes created.")

#define MARGI_ERR_DD_HWNDSUBCLASSED                 (-7021)
MERRSTR(MARGI_ERR_DD_HWNDSUBCLASSED,                "DirectDraw is prevented from restoring state because the DirectDraw cooperative level window handle has been subclassed.")

#define MARGI_ERR_DD_IMPLICITLYCREATED              (-7022)
MERRSTR(MARGI_ERR_DD_IMPLICITLYCREATED,             "The surface cannot be restored because it is an implicitly created surface.")

#define MARGI_ERR_DD_INCOMPATIBLEPRIMARY            (-7023)
MERRSTR(MARGI_ERR_DD_INCOMPATIBLEPRIMARY,           "The primary surface creation request does not match with the existing primary surface.")

#define MARGI_ERR_DD_INVALIDCAPS                    (-7024)
MERRSTR(MARGI_ERR_DD_INVALIDCAPS,                   "One or more of the capability bits passed to the callback function are incorrect.")

#define MARGI_ERR_DD_INVALIDCLIPLIST                (-7025)
MERRSTR(MARGI_ERR_DD_INVALIDCLIPLIST,               "DirectDraw does not support the provided clip list.")

#define MARGI_ERR_DD_INVALIDDIRECTDRAWGUID          (-7026)
MERRSTR(MARGI_ERR_DD_INVALIDDIRECTDRAWGUID,         "The globally unique identifier (GUID) passed to the DirectDrawCreate function is not a valid DirectDraw driver identifier.")

#define MARGI_ERR_DD_INVALIDMODE                    (-7027)
MERRSTR(MARGI_ERR_DD_INVALIDMODE,                   "DirectDraw does not support the requested mode.")

#define MARGI_ERR_DD_INVALIDOBJECT                  (-7028)
MERRSTR(MARGI_ERR_DD_INVALIDOBJECT,                 "DirectDraw received a pointer that was an invalid DirectDraw object.")

#define MARGI_ERR_DD_INVALIDPARAMS                  (-7029)
MERRSTR(MARGI_ERR_DD_INVALIDPARAMS,                 "One or more of the parameters passed to the method are incorrect.")

#define MARGI_ERR_DD_INVALIDPIXELFORMAT             (-7030)
MERRSTR(MARGI_ERR_DD_INVALIDPIXELFORMAT,            "The pixel format was invalid as specified.")

#define MARGI_ERR_DD_INVALIDPOSITION                (-7031)
MERRSTR(MARGI_ERR_DD_INVALIDPOSITION,               "The position of the overlay on the destination is no longer legal.")

#define MARGI_ERR_DD_INVALIDRECT                    (-7032)
MERRSTR(MARGI_ERR_DD_INVALIDRECT,                   "The provided rectangle was invalid.")

#define MARGI_ERR_DD_INVALIDSTREAM                  (-7033)
MERRSTR(MARGI_ERR_DD_INVALIDSTREAM,                 "The specified stream contains invalid data.")

#define MARGI_ERR_DD_INVALIDSURFACETYPE             (-7034)
MERRSTR(MARGI_ERR_DD_INVALIDSURFACETYPE,            "The requested operation could not be performed because the surface was of the wrong type.")

#define MARGI_ERR_DD_LOCKEDSURFACES                 (-7035)
MERRSTR(MARGI_ERR_DD_LOCKEDSURFACES,                "One or more surfaces are locked, causing the failure of the requested operation.")

#define MARGI_ERR_DD_MOREDATA                       (-7036)
MERRSTR(MARGI_ERR_DD_MOREDATA,                      "There is more data available than the specified buffer size can hold.")

#define MARGI_ERR_DD_NO3D                           (-7037)
MERRSTR(MARGI_ERR_DD_NO3D,                          "No 3-D hardware or emulation is present.")

#define MARGI_ERR_DD_NOALPHAHW                      (-7038)
MERRSTR(MARGI_ERR_DD_NOALPHAHW,                     "No alpha acceleration hardware is present or available, causing the failure of the requested operation.")

#define MARGI_ERR_DD_NOBLTHW                        (-7039)
MERRSTR(MARGI_ERR_DD_NOBLTHW,                       "No blitter hardware is present.")

#define MARGI_ERR_DD_NOCLIPLIST                     (-7040)
MERRSTR(MARGI_ERR_DD_NOCLIPLIST,                    "No clip list is available.")

#define MARGI_ERR_DD_NOCLIPPERATTACHED              (-7041)
MERRSTR(MARGI_ERR_DD_NOCLIPPERATTACHED,             "No DirectDrawClipper object is attached to the surface object.")

#define MARGI_ERR_DD_NOCOLORCONVHW                  (-7042)
MERRSTR(MARGI_ERR_DD_NOCOLORCONVHW,                 "The operation cannot be carried out because no color-conversion hardware is present or available.")

#define MARGI_ERR_DD_NOCOLORKEY                     (-7043)
MERRSTR(MARGI_ERR_DD_NOCOLORKEY,                    "The surface does not currently have a color key.")

#define MARGI_ERR_DD_NOCOLORKEYHW                   (-7044)
MERRSTR(MARGI_ERR_DD_NOCOLORKEYHW,                  "The operation cannot be carried out because there is no hardware support for the destination color key.")

#define MARGI_ERR_DD_NOCOOPERATIVELEVELSET          (-7045)
MERRSTR(MARGI_ERR_DD_NOCOOPERATIVELEVELSET,         "A create function is called without the IDirectDraw4::SetCooperativeLevel method being called.")

#define MARGI_ERR_DD_NODC                           (-7046)
MERRSTR(MARGI_ERR_DD_NODC,                          "No DC has ever been created for this surface.")

#define MARGI_ERR_DD_NODDROPSHW                     (-7047)
MERRSTR(MARGI_ERR_DD_NODDROPSHW,                    "No DirectDraw raster operation (ROP) hardware is available.")

#define MARGI_ERR_DD_NODIRECTDRAWHW                 (-7048)
MERRSTR(MARGI_ERR_DD_NODIRECTDRAWHW,                "Hardware-only DirectDraw object creation is not possible; the driver does not support any hardware.")

#define MARGI_ERR_DD_NODIRECTDRAWSUPPORT            (-7049)
MERRSTR(MARGI_ERR_DD_NODIRECTDRAWSUPPORT,           "DirectDraw support is not possible with the current display driver.")

#define MARGI_ERR_DD_NOEMULATION                    (-7050)
MERRSTR(MARGI_ERR_DD_NOEMULATION,                   "Software emulation is not available.")

#define MARGI_ERR_DD_NOEXCLUSIVEMODE                (-7051)
MERRSTR(MARGI_ERR_DD_NOEXCLUSIVEMODE,               "The operation requires the application to have exclusive mode, but the application does not have exclusive mode.")

#define MARGI_ERR_DD_NOFLIPHW                       (-7052)
MERRSTR(MARGI_ERR_DD_NOFLIPHW,                      "Flipping visible surfaces is not supported.")

#define MARGI_ERR_DD_NOFOCUSWINDOW                  (-7053)
MERRSTR(MARGI_ERR_DD_NOFOCUSWINDOW,                 "An attempt was made to create or set a device window without first setting the focus window.")

#define MARGI_ERR_DD_NOGDI                          (-7054)
MERRSTR(MARGI_ERR_DD_NOGDI,                         "No GDI is present.")

#define MARGI_ERR_DD_NOHWND                         (-7055)
MERRSTR(MARGI_ERR_DD_NOHWND,                        "Clipper notification requires a window handle, or no window handle has been previously set as the cooperative level window handle.")

#define MARGI_ERR_DD_NOMIPMAPHW                     (-7056)
MERRSTR(MARGI_ERR_DD_NOMIPMAPHW,                    "The operation cannot be carried out because no mipmap capable texture mapping hardware is present or available.")

#define MARGI_ERR_DD_NOMIRRORHW                     (-7057)
MERRSTR(MARGI_ERR_DD_NOMIRRORHW,                    "The operation cannot be carried out because no mirroring hardware is present or available.")

#define MARGI_ERR_DD_NONONLOCALVIDMEM               (-7058)
MERRSTR(MARGI_ERR_DD_NONONLOCALVIDMEM,              "An attempt was made to allocate non-local video memory from a device that does not support non-local video memory.")

#define MARGI_ERR_DD_NOOPTIMIZEHW                   (-7059)
MERRSTR(MARGI_ERR_DD_NOOPTIMIZEHW,                  "The device does not support optimized surfaces.")

#define MARGI_ERR_DD_NOOVERLAYDEST                  (-7060)
MERRSTR(MARGI_ERR_DD_NOOVERLAYDEST,                 "The IDirectDrawSurface4::GetOverlayPosition method is called on an overlay that the IDirectDrawSurface4::UpdateOverlay method has not been called on to establish a destination.")

#define MARGI_ERR_DD_NOOVERLAYHW                    (-7061)
MERRSTR(MARGI_ERR_DD_NOOVERLAYHW,                   "The operation cannot be carried out because no overlay hardware is present or available.")

#define MARGI_ERR_DD_NOPALETTEATTACHED              (-7062)
MERRSTR(MARGI_ERR_DD_NOPALETTEATTACHED,             "No palette object is attached to this surface.")

#define MARGI_ERR_DD_NOPALETTEHW                    (-7063)
MERRSTR(MARGI_ERR_DD_NOPALETTEHW,                   "There is no hardware support for 16- or 256-color palettes.")

#define MARGI_ERR_DD_NORASTEROPHW                   (-7064)
MERRSTR(MARGI_ERR_DD_NORASTEROPHW,                  "The operation cannot be carried out because no appropriate raster operation hardware is present or available.")

#define MARGI_ERR_DD_NOROTATIONHW                   (-7065)
MERRSTR(MARGI_ERR_DD_NOROTATIONHW,                  "The operation cannot be carried out because no rotation hardware is present or available.")

#define MARGI_ERR_DD_NOSTRETCHHW                    (-7066)
MERRSTR(MARGI_ERR_DD_NOSTRETCHHW,                   "The operation cannot be carried out because there is no hardware support for stretching.")

#define MARGI_ERR_DD_NOT4BITCOLOR                   (-7067)
MERRSTR(MARGI_ERR_DD_NOT4BITCOLOR,                  "The DirectDrawSurface object is not using a 4-bit color palette and the requested operation requires a 4-bit color palette.")

#define MARGI_ERR_DD_NOT4BITCOLORINDEX              (-7068)
MERRSTR(MARGI_ERR_DD_NOT4BITCOLORINDEX,             "The DirectDrawSurface object is not using a 4-bit color index palette and the requested operation requires a 4-bit color index palette.")

#define MARGI_ERR_DD_NOT8BITCOLOR                   (-7069)
MERRSTR(MARGI_ERR_DD_NOT8BITCOLOR,                  "The DirectDrawSurface object is not using an 8-bit color palette and the requested operation requires an 8-bit color palette.")

#define MARGI_ERR_DD_NOTAOVERLAYSURFACE             (-7070)
MERRSTR(MARGI_ERR_DD_NOTAOVERLAYSURFACE,            "An overlay component is called for a non-overlay surface.")

#define MARGI_ERR_DD_NOTEXTUREHW                    (-7071)
MERRSTR(MARGI_ERR_DD_NOTEXTUREHW,                   "The operation cannot be carried out because no texture-mapping hardware is present or available.")

#define MARGI_ERR_DD_NOTFLIPPABLE                   (-7072)
MERRSTR(MARGI_ERR_DD_NOTFLIPPABLE,                  "An attempt has been made to flip a surface that cannot be flipped.")

#define MARGI_ERR_DD_NOTFOUND                       (-7073)
MERRSTR(MARGI_ERR_DD_NOTFOUND,                      "The requested item was not found.")

#define MARGI_ERR_DD_NOTINITIALIZED                 (-7074)
MERRSTR(MARGI_ERR_DD_NOTINITIALIZED,                "An attempt was made to call an interface method of a DirectDraw object created by CoCreateInstance before the object was initialized.")

#define MARGI_ERR_DD_NOTLOADED                      (-7075)
MERRSTR(MARGI_ERR_DD_NOTLOADED,                     "The surface is an optimized surface, but it has not yet been allocated any memory.")

#define MARGI_ERR_DD_NOTLOCKED                      (-7076)
MERRSTR(MARGI_ERR_DD_NOTLOCKED,                     "An attempt is made to unlock a surface that was not locked.")

#define MARGI_ERR_DD_NOTPAGELOCKED                  (-7077)
MERRSTR(MARGI_ERR_DD_NOTPAGELOCKED,                 "An attempt is made to page unlock a surface with no outstanding page locks.")

#define MARGI_ERR_DD_NOTPALETTIZED                  (-7078)
MERRSTR(MARGI_ERR_DD_NOTPALETTIZED,                 "The surface being used is not a palette-based surface.")

#define MARGI_ERR_DD_NOVSYNCHW                      (-7079)
MERRSTR(MARGI_ERR_DD_NOVSYNCHW,                     "The operation cannot be carried out because there is no hardware support for vertical blank synchronized operations.")

#define MARGI_ERR_DD_NOZBUFFERHW                    (-7080)
MERRSTR(MARGI_ERR_DD_NOZBUFFERHW,                   "The operation to create a z-buffer in display memory or to perform a blit using a z-buffer cannot be carried out because there is no hardware support for z-buffers.")

#define MARGI_ERR_DD_NOZOVERLAYHW                   (-7081)
MERRSTR(MARGI_ERR_DD_NOZOVERLAYHW,                  "The overlay surfaces cannot be z-layered based on the z-order because the hardware does not support z-ordering of overlays.")

#define MARGI_ERR_DD_OUTOFCAPS                      (-7082)
MERRSTR(MARGI_ERR_DD_OUTOFCAPS,                     "The hardware needed for the requested operation has already been allocated.")

#define MARGI_ERR_DD_OUTOFMEMORY                    (-7083)
MERRSTR(MARGI_ERR_DD_OUTOFMEMORY,                   "DirectDraw does not have enough memory to perform the operation.")

#define MARGI_ERR_DD_OUTOFVIDEOMEMORY               (-7084)
MERRSTR(MARGI_ERR_DD_OUTOFVIDEOMEMORY,              "DirectDraw does not have enough display memory to perform the operation.")

#define MARGI_ERR_DD_OVERLAPPINGRECTS               (-7085)
MERRSTR(MARGI_ERR_DD_OVERLAPPINGRECTS,              "Operation could not be carried out because the source and destination rectangles are on the same surface and overlap each other.")

#define MARGI_ERR_DD_OVERLAYCANTCLIP                (-7086)
MERRSTR(MARGI_ERR_DD_OVERLAYCANTCLIP,               "The hardware does not support clipped overlays.")

#define MARGI_ERR_DD_OVERLAYCOLORKEYONLYONEACTIVE   (-7087)
MERRSTR(MARGI_ERR_DD_OVERLAYCOLORKEYONLYONEACTIVE,  "An attempt was made to have more than one color key active on an overlay.")

#define MARGI_ERR_DD_OVERLAYNOTVISIBLE              (-7088)
MERRSTR(MARGI_ERR_DD_OVERLAYNOTVISIBLE,             "The IDirectDrawSurface4::GetOverlayPosition method is called on a hidden overlay.")

#define MARGI_ERR_DD_PALETTEBUSY                    (-7089)
MERRSTR(MARGI_ERR_DD_PALETTEBUSY,                   "Access to this palette is refused because the palette is locked by another thread.")

#define MARGI_ERR_DD_PRIMARYSURFACEALREADYEXISTS    (-7090)
MERRSTR(MARGI_ERR_DD_PRIMARYSURFACEALREADYEXISTS,   "This process has already created a primary surface.")

#define MARGI_ERR_DD_REGIONTOOSMALL                 (-7091)
MERRSTR(MARGI_ERR_DD_REGIONTOOSMALL,                "The region passed to the IDirectDrawClipper::GetClipList method is too small.")

#define MARGI_ERR_DD_SURFACEALREADYATTACHED         (-7092)
MERRSTR(MARGI_ERR_DD_SURFACEALREADYATTACHED,        "An attempt was made to attach a surface to another surface to which it is already attached.")

#define MARGI_ERR_DD_SURFACEALREADYDEPENDENT        (-7093)
MERRSTR(MARGI_ERR_DD_SURFACEALREADYDEPENDENT,       "An attempt was made to make a surface a dependency of another surface to which it is already dependent.")

#define MARGI_ERR_DD_SURFACEBUSY                    (-7094)
MERRSTR(MARGI_ERR_DD_SURFACEBUSY,                   "Access to the surface is refused because the surface is locked by another thread.")

#define MARGI_ERR_DD_SURFACEISOBSCURED              (-7095)
MERRSTR(MARGI_ERR_DD_SURFACEISOBSCURED,             "Access to the surface is refused because the surface is obscured.")

#define MARGI_ERR_DD_SURFACELOST                    (-7096)
MERRSTR(MARGI_ERR_DD_SURFACELOST,                   "Access to the surface is refused because the surface memory is gone. Call the IDirectDrawSurface4::Restore method on this surface to restore the memory associated with it.")

#define MARGI_ERR_DD_SURFACENOTATTACHED             (-7097)
MERRSTR(MARGI_ERR_DD_SURFACENOTATTACHED,            "The requested surface is not attached.")

#define MARGI_ERR_DD_TOOBIGHEIGHT                   (-7098)
MERRSTR(MARGI_ERR_DD_TOOBIGHEIGHT,                  "The height requested by DirectDraw is too large.")

#define MARGI_ERR_DD_TOOBIGSIZE                     (-7099)
MERRSTR(MARGI_ERR_DD_TOOBIGSIZE,                    "The size requested by DirectDraw is too large. However, the individual height and width are valid sizes.")

#define MARGI_ERR_DD_TOOBIGWIDTH                    (-7100)
MERRSTR(MARGI_ERR_DD_TOOBIGWIDTH,                   "The width requested by DirectDraw is too large.")

#define MARGI_ERR_DD_UNSUPPORTED                    (-7101)
MERRSTR(MARGI_ERR_DD_UNSUPPORTED,                   "The operation is not supported.")

#define MARGI_ERR_DD_UNSUPPORTEDFORMAT              (-7102)
MERRSTR(MARGI_ERR_DD_UNSUPPORTEDFORMAT,             "The FourCC format requested is not supported by DirectDraw.")

#define MARGI_ERR_DD_UNSUPPORTEDMASK                (-7103)
MERRSTR(MARGI_ERR_DD_UNSUPPORTEDMASK,               "The bitmask in the pixel format requested is not supported by DirectDraw.")

#define MARGI_ERR_DD_UNSUPPORTEDMODE                (-7104)
MERRSTR(MARGI_ERR_DD_UNSUPPORTEDMODE,               "The display is currently in an unsupported mode.")

#define MARGI_ERR_DD_VERTICALBLANKINPROGRESS        (-7105)
MERRSTR(MARGI_ERR_DD_VERTICALBLANKINPROGRESS,       "A vertical blank is in progress.")

#define MARGI_ERR_DD_VIDEONOTACTIVE                 (-7106)
MERRSTR(MARGI_ERR_DD_VIDEONOTACTIVE,                "The video port is not active.")

#define MARGI_ERR_DD_WASSTILLDRAWING                (-7107)
MERRSTR(MARGI_ERR_DD_WASSTILLDRAWING,               "The previous blit operation that is transferring information to or from this surface is incomplete.")

#define MARGI_ERR_DD_WRONGMODE                      (-7108)
MERRSTR(MARGI_ERR_DD_WRONGMODE,                     "This surface cannot be restored because it was created in a different mode.")

#define MARGI_ERR_DD_XALIGN                         (-7109)
MERRSTR(MARGI_ERR_DD_XALIGN,                        "The provided rectangle was not horizontally aligned on a required boundary.")

/** Unused (-7109) to (.....) **/

#endif

/**EOF: ERRORS.H**/
