#ifndef __DBGMSG_H__
#define __DBGMSG_H__ 0x0510

//16Apr98 Chip Doran

#ifdef __QNXNTO__
#include <syslog.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __QNXNTO__
#define DBGMSG                (1)
#endif

#ifndef DBGMSG
#define DBGMSG 0
#endif

//FAR needs to be defined to nothing for VXD builds.
#ifndef FAR
#define FAR
#endif

//DBGMSG_VERSION controls the API link between clients and server.
#define DBGMSG_VERSION  0x00080009	//Added data logging.
//#define DBGMSG_VERSION  0x00080008	//Making DBGCMD_INIT_REUSE the default behavior.
//#define DBGMSG_VERSION  0x00080007    //Fixed another STDCALL bug in DbgMsg.c callback.
//#define DBGMSG_VERSION  0x00080006	//Added Pentium performance counter stuff.
//#define DBGMSG_VERSION  0x00080005	//Added sanity check
//#define DBGMSG_VERSION  0x00080004	//Indexed filters
//#define DBGMSG_VERSION  0x00080003
#define DBGMSG_HNDL16   0xCD000016      //Use this for handle-less commands from 16bit

//Bits 0-7 are reserved for default classes
//Bits 8-23 are reserved for individual projects' use.
//Bits 24-31 are reserved for "must be on" filters.
//  Bits 24-27 are projects' "must be on" filters.
#define DBG_ERR     0x00000002    //Errors
#define DBG_WRN     0x00000004    //Warnings
#define DBG_MSG     0x00000008    //Identify message handler messages.
#define DBG_INFO    0x00000010    //General information.
#define DBG_MEM     0x00000020    //Memory allocations

#define DBG_FILTERS 0x3F000000    //Special classes
#define DBG_INTR    0x04000000    //Do not display message until next non-interrupt call.
#define DBG_STATWND 0x08000000    //Also put string in status window.
#define DBG_HIFREQ  0x10000000    //Messages which will be output frequently.
#define DBG_PAUSE   0x20000000    //Put message in a message box & pause.
#define DBG_INDEXED 0x40000000    //Uses new indexed scheme, not a simple bitfield.
#define DBG_CRIT    0x80000000	  //Wait without message box

//Macros for dealing with DBG_INDEXED filters.
#define DBGMSG51_MAXFLTRINDEX 32
#define DBGMSG51_INDEXFILTER(Index,Mask) (DBG_INDEXED|(((DWORD)(Index) & 0x1F)<<16)|((WORD)(Mask) & 0xFFFF))
#define DBGMSG51_FILTERINDEX(Filter) (((DWORD)(Filter)>>16) & 0x1F)
#define DBGMSG51_FILTERMASK(Filter) ((WORD)(Filter))

//DBGCMD_INIT: Arg1=DBGMSG_VERSION, Arg2= char *Name
#define DBGCMD_INIT             1
  //Flags are OR'ed with DBGCMD_INIT command.
  #define DBGCMD_INIT_16BIT       (0x00010000|DBGCMD_INIT)    //Client is a 16bit application
  #define DBGCMD_INIT_NOSWAP      (0x00020000|DBGCMD_INIT)    //Use locked memory for context info.
  #define DBGCMD_INIT_SMALL_MODEL (0x00040000|DBGCMD_INIT)    //Client is a 16bit app, compile small model
  #define DBGCMD_INIT_VIEWER      (0x00080000|DBGCMD_INIT)    //Client is the message viewer app.
  #define DBGCMD_INIT_RING0       (0x00100000|DBGCMD_INIT)    //Client is running in ring0 (is a VxD)
  #define DBGCMD_INIT_REUSE	  (0x00200000|DBGCMD_INIT)    //Do not create a 2nd client with same name as an existing one. Now DEFAULT!
  #define DBGCMD_INIT_DUP_OK	  (0x00400000|DBGCMD_INIT)    //Always create new client; duplicate names OK.
  #define DBGCMD_INIT_DUP_FAIL	  (0x00800000|DBGCMD_INIT)    //Fail if name already in use.

#define DBGCMD_FINI             2
#define DBGCMD_FILE             3
#define DBGCMD_ID               4
  #define DBGCMD_ID_WINDOW        (0x00010000|DBGCMD_ID)
  #define DBGCMD_ID_FILE          (0x00020000|DBGCMD_ID)
  #define DBGCMD_ID_TIMESTAMP     (0x01000000|DBGCMD_ID)
  #define DBGCMD_ID_ELAPSED       (0x02000000|DBGCMD_ID)
#define DBGCMD_FILTER           5                             //Client setting its own filter: Arg1=DWORD Filter
#define DBGCMD_WINDOW           6
  #define DBGCMD_WND_MINIMIZE     (0x00010000|DBGCMD_WINDOW)
  #define DBGCMD_WND_MAXIMIZE     (0x00020000|DBGCMD_WINDOW)
  #define DBGCMD_WND_RESTORE      (0x00030000|DBGCMD_WINDOW)
#define DBGCMD_ECHO             7                             //Control where msgs are sent: Arg1=DBGCMD_ECHO_
  #define DBGCMD_ECHO_WND         0x00000001
  #define DBGCMD_ECHO_FILE        0x00000002
  #define DBGCMD_ECHO_DEBUGGER    0x00000004
  #define DBGCMD_GETECHO            (0x00010000|DBGCMD_ECHO)  //Get current echo flags: Arg1= DWORD *pFlags
#define DBGCMD_CLOSE            8                             //Arg1= DBGCMD_CLOSE_
  #define DBGCMD_CLOSE_WND        0x00000001
  #define DBGCMD_CLOSE_FILE       0x00000002
#define DBGCMD_PROFILE          9
  #define DBGCMD_PROF_PORT        (0x00010000|DBGCMD_PROFILE)	//Arg1= BOOL Enable; Arg2= DWORD PortIoAddr (0 for default)
  #define DBGCMD_PROF_CLEAR       (0x00020000|DBGCMD_PROFILE)	//Clear log entries, reset timezero.
  #define DBGCMD_PROF_ENTER       (0x00030000|DBGCMD_PROFILE)	//Record entry time: Arg1= Channel
  #define DBGCMD_PROF_EXIT        (0x00040000|DBGCMD_PROFILE)	//Record exit time: Arg1= Channel
  #define DBGCMD_PROF_ENABLE      (0x00050000|DBGCMD_PROFILE)	//Freeze/Enable recording: Arg1= BOOL Enable
  #define DBGCMD_PROF_STAMP       (0x00060000|DBGCMD_PROFILE)	//Record Data value: Arg1= Channel, Arg2= DWORD Data
  #define DBGCMD_PROF_LOG         (0x00070000|DBGCMD_PROFILE)	//Not used.
  #define DBGCMD_PROF_GETTIME     (0x00080000|DBGCMD_PROFILE)   //Arg1= 0:Use CyclesTime (uSec), 1:Use system time (mSec); Arg2= (DWORD*)pHours (may be 0)
  #define DBGCMD_PROF_LOAD        (0x00090000|DBGCMD_PROFILE)	//Not used
  #define DBGCMD_PROF_SAVE        (0x000A0000|DBGCMD_PROFILE)	//Not used
  #define DBGCMD_PROF_INIT        (0x000B0000|DBGCMD_PROFILE)	//Init: All clients must call INIT before using profiler services.
  #define DBGCMD_PROF_CALIB       (0x000C0000|DBGCMD_PROFILE)   //Arg1= Length of test in seconds, (0 for default); Arg2= (DWORD*)&ClockMHz (may be zero)
  #define DBGCMD_PROF_PAUSE       (0x000D0000|DBGCMD_PROFILE)	//Freeze recording: Arg1= BOOL Freeze
  #define DBGCMD_PROF_WRITE       (0x000E0000|DBGCMD_PROFILE)	//Not used
  #define DBGCMD_PROF_FINI        (0x000F0000|DBGCMD_PROFILE)	//Close profiler: All clients should call FINI when doen using profiler services.
  #define DBGCMD_PROF_NAMES       (0x00100000|DBGCMD_PROFILE)	//Name channels: Arg1=(struct DbgMsgFilterName_s*)pNames[Max], Arg2= Max!!!
  #define DBGCMD_PROF_EVENT_SET   (0x00110000|DBGCMD_PROFILE)	//Set event to be counted and reset count: Arg1= one of DBGCTR_ from DbgCnt.h
  #define DBGCMD_PROF_EVENT_GET   (0x00120000|DBGCMD_PROFILE)	//Reads the current event count: Arg1= DBGCTR_ (Only DBGCTR_COUNTERx is used), Arg2= DWORD *pCount
#define DBGCMD_DELAY            10
  #define DBGCMD_DELAY_ENABLE     (0x01000000|DBGCMD_DELAY)
  #define DBGCMD_DELAY_SECONDS    (0x02000000|DBGCMD_DELAY)
#define DBGCMD_DEFER            11
  #define DBGCMD_DEFER_START      (0x00010000|DBGCMD_DEFER)
  #define DBGCMD_DEFER_STOP       (0x00020000|DBGCMD_DEFER)
  #define DBGCMD_DEFER_FLUSH      (0x00030000|DBGCMD_DEFER)
  #define DBGCMD_DEFER_CLEAR      (0x00040000|DBGCMD_DEFER)
#define DBGCMD_LOGFNAME         12
#define DBGCMD_GETTIME          13
#define DBGCMD_CLNTWND_CREATE   14
#define DBGCMD_CLNTWND_DESTROY  15

#define DBGCMD_CALLBACK         16    //Arg1=User callback routine, Arg2= Reserved.
  //Callback messages sent from server to client
  #define DBGCB_INIT              1   //When callback is set. (During DBGCMD_CALLBACK)
  #define DBGCB_FINI              2   //Client is being closed. (Invalidated)
  #define DBGCB_SETFILTER         3   //Your filter has been set externally. Arg1=DWORD NewFilter.
  #define DBGCB_INTERNAL          4   //Internal messages

#define DBGCMD_GETFILTER        17    //Get current filter: Arg1= (DWORD*) &dwFilter, Arg2= DWORD dwFilterIndex.
#define DBGCMD_CLIENTNAME       18    //Client setting its own name: Arg1= char *Name;
#define DBGCMD_FILTERNAMES      19    //Client setting filter names. Arg1= struct DbgMsgFilterName_s *pNames.
  struct DbgMsgFilterName_s {         //Used for DBGCMD_FILTERNAMES and DBGCMD_PROF_NAMES
    DWORD Mask;                       //Filters only: Last one should be 0
    char Name[32];
  };
#define DBGCMD_GETVERSION       20      //Arg1=Type, Arg2= DWORD *pVersion
  #define DBGCMD_VER_API          1   //Get the interface version (DBGMSG_VERSION)
  #define DBGCMD_VER_SERVER_PROD  2   //Get server build: 0xMMmmBBBB (MM=Major, mm=minor, BBBB=build)
  #define DBGCMD_VER_SERVER_FILE  3
  #define DBGCMD_VER_VIEWER_PROD  4   //Get viewer build
  #define DBGCMD_VER_VIEWER_FILE  5
#define DBGCMD_INTERNAL         21      //Internal messages
#define DBGCMD_DATASEG          24      //Arg1=DS; for 16bit small-model clients only.
#define DBGCMD_SETUSER          25      //Arg1=User DWORD
#define DBGCMD_GETUSER          26      //Returns value set by DBGCMD_SETUSER.

//PerfStat commands
#define DBGCMD_PERF             27          //0x001B
#define DBGCMD_PERFADD            (0x00000000|DBGCMD_PERF)  //Add new stat, Arg1=(struct DbgMsgPerf_s*)pStat, Arg2= (DWORD*)&hStat (returned)
#define DBGCMD_PERFDEL            (0x00010000|DBGCMD_PERF)  //Del stat: Arg1= DWORD hStat
#define DBGCMD_PERFSET            (0x00020000|DBGCMD_PERF)  //Set value: Arg1= DWORD hStat, Arg2= DWORD Value
#define DBGCMD_PERFADDRATE	  (0x00030000|DBGCMD_PERF)  //Add new rate stat, Arg1=(struct DbgMsgPerf_s*)pStat, Arg2= (DWORD*)&hStat (returned)
struct DbgMsgPerf_s {
  char FAR *Title;
  char FAR *Caption;
  char FAR *Description;
  DWORD InitialValue;
};

#define DBGCMD_LOG              28  //Obsolete
#define DBGCMD_DUMPOPEN         29  //Obsolete
#define DBGCMD_DUMPCLOSE        30  //Obsolete
#define DBGCMD_DUMPDATA         31  //Obsolete
#define DBGCMD_PRINT            32  //Internal use only.
#define DBGCMD_SANITY_CHECK     33  //Sets sanity check ptr: Arg1= DWORD* &SanityCheck [SanityCheck= DBGMSG_SANITY_VALUE]
  #define DBGMSG_SANITY_VALUE   0x50494843

//Data logging.
//For all DATA commands other than OPEN, the hLog is passed in the HIWORD of the command.
#define DBGCMD_DATAOPEN           34  //Open a new data log: Arg1= char *LogFName, Arg2= DWORD &hLog
  #define DBGCMD_DATAOPEN_BIG     (0x00010000|DBGCMD_DATAOPEN)	  //Use big buffers. (32K by default)
  #define DBGCMD_DATAOPEN_HUGE    (0x00020000|DBGCMD_DATAOPEN)	  //Use huge buffers. (256K by default)
  #define DBGCMD_DATAOPEN_REOPEN  (0x00040000|DBGCMD_DATAOPEN)	  //Open and close file on each write.
#define DBGCMD_DATACLOSE        35  //Close a log
#define DBGCMD_DATAWRITE        36  //Log data: Arg1= DWORD nBytes, Arg2= BYTE *pData
#define DBGCMD_DATACOMMENT      37  //Insert a comment: Arg1= char *pText
#define DBGCMD_DATARESET        38  //Clear server's download buffers.

#define DBGERR_UNK (-1)             //Map these to Errors.h, but don't rely on the file.
#define DBGERR_BADCMD (-19)         //ERR_BADITEM
#define DBGERR_BADARG (-4)          //ERR_BADPARM
#define DBGERR_STR_TOO_LONG (-5)    //ERR_TOOLONG
#define DBGERR_BADPTR (-2)          //ERR_BADPTR
#define DBGERR_IGNORING (-37)       //ERR_IGNORED
#define DBGERR_NOSHARE (-12)        //ERR_NODEVICE
#define DBGERR_NOTFOUND (-7)        //ERR_NOTFOUND
#define DBGERR_TOO_MANY (-11)       //ERR_TOOMANY
#define DBGERR_BADHNDL (-13)        //ERR_BADHANDLE
#define DBGERR_BADCALLBACK (-38)    //ERR_BADCALLBACK
#define DBGERR_BADVER (-103)        //ERR_BADVERSION
#define DBGERR_LOAD (-102)          //ERR_LOAD: Unable to load driver
#define DBGERR_OBSOLETE (-107)      //ERR_OBSOLETE: No longer supported.
#define DBGERR_NOMEM (-6)           //ERR_NOMEM
#define DBGERR_NOTAVAILABLE (-108)  //ERR_NOTAVAILABLE: When DBGMSG is not defined, DbgMsgCmd() defines to this.
#define DBGERR_DUPLICATE (-10)      //ERR_DUPLICATE: Thing already exists.
#define DBGERR_DEVICE (-22)         //ERR_DEVICE: Device not present or doesn't support requested capability.
#define DBGERR_BUSY (-14)           //ERR_BUSY
#define DBGERR_FILEOPEN (-401)      //ERR_FILEOPEN

#if(DBGMSG)

  #define DBGMSG51_DEVICE_ID 0xCD32   //Also defined in Vxd\VxdEntry.asm!
  #define DBGMSG51_DYNKEY "DbgMsg51\\Server"
  #define DBGMSG51_DYNKEY_SERVER "Ring0Entry"

  #define SQUAWKNUMZ(num) #num
  #define SQUAWKNUM(num) SQUAWKNUMZ(num)
  #define DBGID __FILE__"("SQUAWKNUM(__LINE__)")::"

  //Callbacks have no return values to force me to remember that they happen asynchronously.
  typedef void (FAR __cdecl *DBGMSGCB)(DWORD Handle, DWORD Msg, DWORD Arg1, DWORD Arg2);
  typedef void (FAR __cdecl *DBGMSGPRINT)(DWORD Handle, char FAR *String);

  extern long FAR __cdecl DbgMsgCmd(DWORD Item, DWORD Arg1, DWORD Arg2);
  extern int FAR __cdecl DbgMsg(DWORD MsgClass, char FAR *Format, ...);
  extern void FAR __cdecl DbgMsgSetPrint(DWORD Handle, DBGMSGPRINT pFnPrint);

  #ifndef DBGMSG_INCLUDE_CODE
    #define DBGMSG_INCLUDE_CODE 0
  #endif

  #if(DBGMSG_INCLUDE_CODE)
    DWORD FnDbgMsgPrintHandle = 0;
    DBGMSGPRINT pFnDbgMsgPrint = 0;

    void FAR __cdecl DbgMsgSetPrint(DWORD Handle, DBGMSGPRINT pFnPrint) {
      pFnDbgMsgPrint = pFnPrint;
      FnDbgMsgPrintHandle = Handle;
    }

    #ifdef _ST_
      #include <stdarg.h>

      DWORD DbgMsgFilter;
      WORD DbgMsgFilterIndexed[DBGMSG51_MAXFLTRINDEX];

      long FAR __cdecl DbgMsgCmd(DWORD Item, DWORD Arg1, DWORD Arg2) {
        return 0;
      }

      int FAR __cdecl DbgMsg(DWORD MsgClass, char FAR *Format, ...) {
        int nChr=0;
        int fOutput= 0;

        if(MsgClass & (DBG_CRIT|DBG_INDEXED)) {
          fOutput= 1;
        } else {
          DWORD dwAndFilters = MsgClass & DBG_FILTERS;
          if(!dwAndFilters || (dwAndFilters & (DbgMsgFilter & DBG_FILTERS))) {
            if(MsgClass & DBG_INDEXED) {
              if(DBGMSG51_FILTERMASK(MsgClass) & DbgMsgFilterIndexed[DBGMSG51_FILTERINDEX(MsgClass)])
                fOutput= 1;
            } else {
              if((MsgClass & ~DBG_FILTERS) & DbgMsgFilter)
                fOutput= 1;
            }
          }
        }
        if(fOutput)
        {
          char cTempString[256];
          va_list args;

          va_start(args, Format);
          vsprintf(cTempString, Format, args);
          va_end(args);

          if(pFnDbgMsgPrint)
            pFnDbgMsgPrint(FnDbgMsgPrintHandle, cTempString);
          else
            debugmessage(cTempString);
        }
        return nChr;
      }
    #elif defined(__QNXNTO__)
      #include <stdarg.h>

      DWORD DbgMsgFilter;
      WORD DbgMsgFilterIndexed[DBGMSG51_MAXFLTRINDEX];

      long FAR __cdecl DbgMsgCmd(DWORD Item, DWORD Arg1, DWORD Arg2) {
        return 0;
      }

      int FAR __cdecl DbgMsg(DWORD MsgClass, char FAR *Format, ...) {
        int nChr=0;
        int fOutput= 0;

        if(MsgClass & (DBG_CRIT|DBG_INDEXED)) {
          fOutput= 1;
        } else {
          DWORD dwAndFilters = MsgClass & DBG_FILTERS;
          if(!dwAndFilters || (dwAndFilters & (DbgMsgFilter & DBG_FILTERS))) {
            if(MsgClass & DBG_INDEXED) {
              if(DBGMSG51_FILTERMASK(MsgClass) & DbgMsgFilterIndexed[DBGMSG51_FILTERINDEX(MsgClass)])
                fOutput= 1;
            } else {
              if((MsgClass & ~DBG_FILTERS) & DbgMsgFilter)
                fOutput= 1;
            }
          }
        }
        if(fOutput)
        {
          char cTempString[256];
          va_list args;

          va_start(args, Format);
          vsprintf(cTempString, Format, args);
          va_end(args);

          if(pFnDbgMsgPrint)
            pFnDbgMsgPrint(FnDbgMsgPrintHandle, cTempString);
//          else
//            debugmessage(cTempString);
        }
        return nChr;
      }
    #else //_QNXNTO_

      static long DbgMsgInit(DWORD Msg, DWORD dwApi, BYTE FAR *ClientName);
      static long DbgMsgFini(void);

      static DWORD DbgMsgFilter;
      static WORD DbgMsgFilterIndexed[DBGMSG51_MAXFLTRINDEX];
      static HMODULE DbgMsgDLL;
      static BYTE DbgMsgClntName[32];
      static BYTE DbgMsgLoadAttempted;
      static BYTE DbgMsgErrStr[256];
      static BYTE DbgMsgExtErrStr[256];
      static BYTE *DbgMsgRegKey= (BYTE*)"Software\\Margi_Systems\\ChipUtil\\DbgMsg51";
      static BYTE DbgMsgDLLFName[260]= {"Dbg5132.dll"};
      static DWORD DbgMsgHandle;
      static DWORD DbgMsgSanityCheck= DBGMSG_SANITY_VALUE;

      typedef int (FAR __cdecl *PFDbgMsgArgPtr) (DWORD Handle, DWORD MsgClass, DWORD FAR *pArgs);      //First arg is Format
      typedef int (FAR __cdecl *PFDbgMsgArgList)(DWORD Handle, DWORD MsgClass, BYTE FAR *Format,...);
      typedef long (FAR __cdecl *PFDbgMsgCmdFunc)(DWORD Handle, DWORD Cmd, DWORD Arg1, DWORD Arg2);

      static PFDbgMsgArgPtr  DbgMsgArgPtr;
      static PFDbgMsgArgList DbgMsgArgList;
      static PFDbgMsgCmdFunc DbgMsgCmdFunc;
      static DBGMSGCB DbgMsgClntCbPtr;

      #ifdef WIN32

        #define DBGMSGSTRCPY(Dst,Src) strcpy(Dst,Src)
        #define DBGMSG32 1
        #define DBGMSGCB_TYPE

        static void DbgMsgGetDllFName(void) {
          HKEY hKey;
          DWORD Type,BufSize= sizeof(DbgMsgDLLFName);
          if(!RegOpenKeyEx(HKEY_LOCAL_MACHINE,(char*)DbgMsgRegKey,0,KEY_READ,&hKey)) {
            RegQueryValueEx(hKey,"DbgMsgDLL32",0,&Type,DbgMsgDLLFName,&BufSize);
            RegCloseKey(hKey);
          }
        }

        static void FAR *DbgMsgGetProc(BYTE FAR *sFunc) {
          DWORD ExtErr;
          void FAR *pFunc;
          if(!(pFunc= GetProcAddress(DbgMsgDLL,(char*)sFunc))) {
            ExtErr= GetLastError();
            FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,0,ExtErr,0,(char*)DbgMsgExtErrStr,sizeof(DbgMsgExtErrStr),0);
            wsprintf((char*)DbgMsgErrStr,"Unable to import function '%s'.\r\n[%d] %s.\r\n",sFunc,ExtErr,DbgMsgExtErrStr);
            OutputDebugString((char*)DbgMsgErrStr);
          }
          return(pFunc);
        }

        static HMODULE DbgMsgLoadLib(BYTE FAR *sLibName) {
          HMODULE hDLL;
          DWORD ExtErr;
          hDLL= LoadLibrary((char*)sLibName);
          if((DWORD)hDLL<32) {
            ExtErr= GetLastError();
            FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,0,ExtErr,0,(char*)DbgMsgExtErrStr,sizeof(DbgMsgExtErrStr),0);
            wsprintf((char*)DbgMsgErrStr,"Unable to load DbgMsg32 DLL.\r\n'%s' [%d]\r\n[%d] %s\r\n",sLibName,hDLL,ExtErr,DbgMsgExtErrStr);
            OutputDebugString((char*)DbgMsgErrStr);
            hDLL= 0;
          }
          return(hDLL);
        }
      #else // _WIN32
        #define DBGMSGSTRCPY(Dst,Src) _fstrcpy(Dst,Src)
        #define DBGMSG32 0
        #define DBGMSGCB_TYPE FAR _loadds

        static void DbgMsgGetDllFName(void) {
          HKEY hKey;
          DWORD BufSize= sizeof(DbgMsgDLLFName);
          if(!RegOpenKey(0x80000002,DbgMsgRegKey,&hKey)) {
            RegQueryValue(hKey,"DbgMsgDLL16",DbgMsgDLLFName,&BufSize);
            RegCloseKey(hKey);
          }
        }

        static void FAR *DbgMsgGetProc(BYTE FAR *sFunc) {
          void FAR *pProc= GetProcAddress((HMODULE)DbgMsgDLL,(char FAR*)sFunc);
          if(!pProc) {
            wsprintf((char FAR*)DbgMsgErrStr,"Unable to GetProcAddress(%ls).\r\n",sFunc);
            OutputDebugString((char FAR*)DbgMsgErrStr);
          }
          return(pProc);
        }

        static HMODULE DbgMsgLoadLib(BYTE FAR *sLibName) {
          HMODULE hDLL;
          hDLL= LoadLibrary((char FAR*)sLibName);
          if(hDLL<32) {
            wsprintf((char FAR*)DbgMsgErrStr,"Unable to load DbgMsg.dll '%ls': %d.\r\n",sLibName,hDLL);
            OutputDebugString((char FAR*)DbgMsgErrStr);
            hDLL= 0;
          }
          return(hDLL);
        }
      #endif //_WIN32

      static void DbgMsgClntCallback(DWORD Handle, DWORD Msg, DWORD Arg1, DWORD Arg2) {
        if(DbgMsgClntCbPtr && !IsBadCodePtr((FARPROC)DbgMsgClntCbPtr))
          (*DbgMsgClntCbPtr)(Handle,Msg,Arg1,Arg2);
      }

      static void DbgMsgCbSetFilter(DWORD Hndl, DWORD NewFilter) {
        if(NewFilter & DBG_INDEXED)
          DbgMsgFilterIndexed[DBGMSG51_FILTERINDEX(NewFilter)]= DBGMSG51_FILTERMASK(NewFilter);
        else
          DbgMsgFilter= NewFilter;
        DbgMsgClntCallback(Hndl,DBGCB_SETFILTER,NewFilter,0);
      }

      static void DBGMSGCB_TYPE __cdecl DbgMsgDfltCallback(DWORD Handle, DWORD Msg, DWORD Arg1, DWORD Arg2) {
        switch(Msg) {
          case DBGCB_SETFILTER:   DbgMsgCbSetFilter(Handle,Arg1); break;
          default:                DbgMsgClntCallback(Handle,Msg,Arg1,Arg2);
        }
      }

      static long DbgMsgCmdSend(DWORD Msg, DWORD Arg1, DWORD Arg2) {
        long Err;
        if(DbgMsgCmdFunc && !IsBadCodePtr((FARPROC)DbgMsgCmdFunc))
          Err= (*DbgMsgCmdFunc)(DbgMsgHandle,Msg,Arg1,Arg2);
        else
          Err= DBGERR_LOAD;
        return(Err);
      }

      static long DbgMsgInit(DWORD Msg, DWORD dwApi, BYTE FAR *ClientName) {
        long Err= 0;
        DWORD n1;
        //First check for versionitis between client app and DbgMsg.h
        if((dwApi>>16)!=(DBGMSG_VERSION>>16) || (WORD)dwApi>(DBGMSG_VERSION & 0xFFFF)) {
          Err= DBGERR_BADVER;
        } else if(!DbgMsgDLL && !DbgMsgLoadAttempted) {
          if(!ClientName)
            ClientName= (BYTE FAR*)"Unnamed";
          for(n1=0;n1<sizeof(DbgMsgClntName)-1;n1++)
            DbgMsgClntName[n1]= ClientName[n1];
          DbgMsgLoadAttempted= 1;
          DbgMsgGetDllFName();
          DbgMsgDLL= DbgMsgLoadLib(DbgMsgDLLFName);
          if((DWORD)DbgMsgDLL>=32) {
            DbgMsgArgPtr =(PFDbgMsgArgPtr)DbgMsgGetProc((BYTE FAR*)"__DbgMsgPtr");
            DbgMsgArgList=(PFDbgMsgArgList)DbgMsgGetProc((BYTE FAR*)"__DbgMsgList");
            DbgMsgCmdFunc=(PFDbgMsgCmdFunc)DbgMsgGetProc((BYTE FAR*)"__DbgMsgCmd");
            if(DbgMsgCmdFunc && !IsBadCodePtr((FARPROC)DbgMsgCmdFunc)) {
              if(!DBGMSG32) {
                Msg|= DBGCMD_INIT_16BIT;
                if(sizeof(void*)==2)
                  Msg|= DBGCMD_INIT_SMALL_MODEL;
              }
              Err= (DWORD)(*DbgMsgCmdFunc)((DWORD)(void FAR*)&DbgMsgHandle,Msg,dwApi,(DWORD)ClientName);
              if(Err) {
                wsprintf((char FAR*)DbgMsgErrStr,"DBGMSG_INIT failed: %ld.\r\n",Err);
                OutputDebugString((char FAR*)DbgMsgErrStr);
              } else {
                if(HIWORD(Msg & DBGCMD_INIT_SMALL_MODEL))
                  (*DbgMsgCmdFunc)(DbgMsgHandle,DBGCMD_DATASEG,HIWORD((DWORD)(void FAR*)&DbgMsgHandle),0);
                (*DbgMsgCmdFunc)(DbgMsgHandle,DBGCMD_SANITY_CHECK,(DWORD)(void FAR*)&DbgMsgSanityCheck,0);
                if(DbgMsgArgList && !IsBadCodePtr((FARPROC)DbgMsgArgList))
                  (*DbgMsgArgList)(DbgMsgHandle,DBG_CRIT,(BYTE FAR*)"DbgMsg.dll loaded.\r\n");
                (*DbgMsgCmdFunc)(DbgMsgHandle,DBGCMD_CLIENTNAME,(DWORD)ClientName,0);
                (*DbgMsgCmdFunc)(DbgMsgHandle,DBGCMD_CALLBACK,(DWORD)DbgMsgDfltCallback,0);
              }
            }
          }
        }
        return(Err);
      }

      static long DbgMsgFini(void) {
        long (FAR *CmdFunc)(DWORD Hndl, DWORD Item, DWORD Arg1, DWORD Arg2)= DbgMsgCmdFunc;
        HMODULE hDLL= DbgMsgDLL;
        if(hDLL) {
          DbgMsgDLL= 0;
          DbgMsgCmdFunc= 0;
          DbgMsgArgPtr= 0;
          DbgMsgArgList= 0;
          if(CmdFunc && !IsBadCodePtr((FARPROC)CmdFunc))
            (*CmdFunc)(DbgMsgHandle,DBGCMD_FINI,0,0);
          FreeLibrary(hDLL);
          DbgMsgHandle= 0;
          DbgMsgLoadAttempted= 0;
        }
        return(0);
      }

      static long DbgMsgSetFilter(DWORD NewFilter) {
        DWORD OldFilter,n1;
        if(NewFilter==(DWORD)-1) {
          OldFilter= DbgMsgFilter;
          DbgMsgFilter= (DWORD)-1 & ~DBG_INDEXED;
          if(DbgMsgCmdFunc && !IsBadCodePtr((FARPROC)DbgMsgCmdFunc))
            (*DbgMsgCmdFunc)(DbgMsgHandle,DBGCMD_FILTER,DbgMsgFilter,(DWORD)(void FAR*)&DbgMsgFilter);
          for(n1=0;n1<DBGMSG51_MAXFLTRINDEX;n1++) {
            DbgMsgFilterIndexed[n1]= (WORD)-1;
            if(DbgMsgCmdFunc && !IsBadCodePtr((FARPROC)DbgMsgCmdFunc))
              (*DbgMsgCmdFunc)(DbgMsgHandle,DBGCMD_FILTER,DBGMSG51_INDEXFILTER(n1,0xFFFF),0);
          }
        } else {
          OldFilter= DbgMsgFilter;
          DbgMsgFilter= NewFilter;
          if(DbgMsgCmdFunc && !IsBadCodePtr((FARPROC)DbgMsgCmdFunc))
            (*DbgMsgCmdFunc)(DbgMsgHandle,DBGCMD_FILTER,NewFilter,(DWORD)(void FAR*)&DbgMsgFilter);
        }
        return((long)OldFilter);
      }

      static long DbgMsgSetCb(DWORD Msg, DBGMSGCB pClntCb) {
        if(pClntCb && IsBadCodePtr((FARPROC)pClntCb))
          return(DBGERR_BADPTR);
        DbgMsgClntCbPtr= pClntCb;
        return(0);
      }

      long FAR __cdecl DbgMsgCmd(DWORD Msg, DWORD Arg1, DWORD Arg2) {
        long Err= -1;
        switch(LOWORD(Msg)) {
          case DBGCMD_FILTER:         Err= DbgMsgSetFilter(Arg1); break;
          case DBGCMD_INIT:           Err= DbgMsgInit(Msg,Arg1,(BYTE*)Arg2); break;
          case DBGCMD_FINI:           Err= DbgMsgFini(); break;
          case DBGCMD_CALLBACK:       Err= DbgMsgSetCb(Msg,(DBGMSGCB)Arg1); break;
          default:                    Err= DbgMsgCmdSend(Msg,Arg1,Arg2);
        }
        return(Err);
      }

      int FAR __cdecl DbgMsg(DWORD MsgClass, char FAR *Format, ...) {
        int fOutput= 0;
        int nChr= 0;
        if(DbgMsgDLL) {
          if(MsgClass & (DBG_CRIT|DBG_INDEXED)) {
            fOutput= 1;
          } else {
            DWORD AndFilters= MsgClass & DBG_FILTERS;
            if(!AndFilters || (AndFilters & (DbgMsgFilter & DBG_FILTERS))) {
              if(MsgClass & DBG_INDEXED) {
                if(DBGMSG51_FILTERMASK(MsgClass) & DbgMsgFilterIndexed[DBGMSG51_FILTERINDEX(MsgClass)])
                  fOutput= 1;
              } else {
                if((MsgClass & ~DBG_FILTERS) & DbgMsgFilter)
                  fOutput= 1;
              }
            }
          }
          if(fOutput && DbgMsgArgPtr && !IsBadCodePtr((FARPROC)DbgMsgArgPtr))
            nChr= (*DbgMsgArgPtr)(DbgMsgHandle,MsgClass,(DWORD FAR*)&Format);
        }
        return(nChr);
      }
    #endif //_ST_

  #else //DBGMSG_INCLUDE_CODE
  #endif //DBGMSG_INCLUDE_CODE

#else //DBGMSG

  #define DbgMsg 1 ? (void)0:(void)
  #define DbgMsgCmd(Cmd,Arg1,Arg)       DBGERR_NOTAVAILABLE
  #define DbgMsgSetPrint 1 ? (void)0:(void)
  #define DBGID

#endif //DBGMSG

#ifdef __cplusplus
  }
#endif

#endif
/**EOF: DBGMSG.H**/
