//#include <StrmIni.h>
//#include <mmsystem.h>
//#include <KsMedia.h>
#include "system.h"
#include "DbgMsg.h"
#include "margierr.h"
#include "Idents.h"
#include "Settings.h"

#include "DbgFltr.h"
#include "IdentCls.h"
#include "MidasApi.h"
#include "lsi64020.h"
#include "dvHwMain.h"
#include "Global.h"
#include "Sync.h"
#include "IniNames.h"
#include "AuxData.h"
#include "dvHwVid.h"
#include "dvHwAud.h"
#include "dvHwSub.h"


#define MAXENTRIES 256

#define AVSYNC_OFF  0
#if AVSYNC_OFF
#define AVSYNC_TEST_VID 0
#define AVSYNC_TEST_AUD 0
#pragma message("Warning:  **** A/V Sync turned off ****")
#endif

//A single audio or video sync table entry.
typedef struct SyncTblEntry_s {
  DWORD Pts;			//PTS extracted from packet data.
  DWORD PktStart;		//Offset in decoder buffer of start of packet.
  DWORD PktEnd;			//Offset in decoder buffer of end of packet.
//  DWORD PcTime;			//PC time at which entry was made.
  BYTE SyncEvents;		//Number of sync events found in this packet so far.
  BYTE Rsvd[3];
//  BYTE Timespace;		//Timespace in which entry was recorded.
} ENTRY;

//An audio or video sync table.
typedef struct SyncTbl_s {
  DWORD Name;		   	//For printfs
  DWORD StrmType;		//STRM_AUD or STRM_VID
  struct AVSync_s *pSync;	//Parent
  DWORD MaxEntry;
  DWORD Head,Tail,EntryCnt;
  DWORD HighWater;
  union {
    struct {
      DWORD Enable:1;           //Enable sync
      DWORD EnableChk:1;        //Enable drift calculations
      DWORD DisableAdj:1;       //Overriding disable of correction
      DWORD EnableAdj:1;        //Enable corrections.
      DWORD EntryStarted:1;     //PktBegin() was called.
      DWORD CaptureSCR:1;       //Enable SCR capturing.
      DWORD ErrorValid:1;       //Is Error valid?
      DWORD ErrorAhead:1;       //1=Playback has drifted ahead of PTS, 0=Playback has fallen behind.
      DWORD MaxToleranceMsg:1;  //Out of tolerance message displayed.
      DWORD AwaitingPicType:1;  //Video: PTS,SCR,RdPtr found, waiting for picture type.
      DWORD PktUncertain:1;     //Error valid, but may be off by one packet due to read FIFO.
      DWORD StartCompleted:1;   //Debug: This gets set on autostart completion.
      DWORD OutOfSync:1;        //Channel is out of sync.
      DWORD Active:1;           //Sync is current active.
      DWORD RdyToStart:1;       //Stream is awaiting start.
      DWORD IsMaster:1;         //Set if Captured Time for this stream should update SCR
    }s;
    DWORD dwFlags;
  }u;
  DWORD InSyncCount;      //Number of times-1 stream must be within tolerence to be declared !OutOfSync
  DWORD Tolerance;        //Error tolerance, this may be either Gross or Fine.
  DWORD ToleranceFreeze;	//Freeze tolerance, if drift greater than less then freeze video for multiple frames.
  DWORD ToleranceGross;		//Gross tolerance, this is used to start drift correction.
  DWORD ToleranceFine;		//Fine-tuning tolerance, this is used once drift correction has begun.
  DWORD Error;            //Current drift in milliseconds. ErrorAhead tells which direction.
  DWORD ErrorTicks;       //Current drift in SCR ticks.
  DWORD LastAdjustment;		//PcTime of previous adjustment.
//  DWORD CurRdPtr;
//  DWORD CurWrPtr;
  DWORD DropCnt;
  DWORD NewScrLo;     // Used to update SCR
  DWORD NewScrHi;

  DWORD CurPts;       //Most recently decoded PTS
  DWORD CurPtsOverFlow; //Most recently decoded PTS overflow count
  DWORD CurPtsTime;   //Time in mS when CurPts recorded.
  DWORD PtsLast;		  //Last PTS extracted from bitstream
  DWORD PtsLastScr;		//SCR extracted from bitstream corresponding to Last PTS.
  DWORD PtsMin;       //The most recent PTS consumed by decode. Used to time transition to play.
  DWORD PtsMatches;   //Number of successful matches, used to switch between audio and video capture. (Not a total count.)
  DWORD DtsLastIP;    //Used to reorder video I,P,B frames.
  DWORD DtsLastUncertainty;     //Carry PTS uncertainty through IP reorder
  DWORD ErrorStartTime;		      //Out of tolerance drift detected at this PcTime.
  DWORD InSyncErrorPersistence;	//Minimum elapsed time with out of tolerance drift before allowing adjustments when InSync.
  DWORD OutSyncErrorPersistence;//Minimum elapsed time with out of tolerance drift before allowing adjustments when Out of Sync.
  DWORD hPerfStat;    //DbgMsg performance statistic handle.
  DWORD LastIntr;     //PcTime at which last interrupt (picture start code or audio sync) detected.
  DWORD CurAvgIntr;   //Rolling median interrupt period.
  long PtsDelay;      //Fudge factor to bump PTS value.
  DWORD PendingPts;
  DWORD PendingSCR;
  DWORD PendingPtsUncertainty; //PTS uncertainty
  DWORD ReadFifoSize;         //Size of data read FIFO. PTS's found within this many bytes of a packet end have a time uncertainty of one frame.
  DWORD PktUncertaintyTime;   //If PktUncertain due to read FIFO, this is the time uncertainty. Error corrections should not be applied below this.
//  DWORD FreezeTarget;       //Used for large corrections.
  DWORD MaxFreeze;		        //Maximum freeze time.
  DWORD FrameCnt;             //Total frames (audio sync or picture start codes) seen.
  int   nResetCount;          //# consecutive table resets because error is > 2 seconds.
  struct SyncTblEntry_s *pEntries;
} TABLE;

//Global sync flags.
struct SyncFlag_s {
  union {
    struct {
      DWORD Started:1;		//AV sync is running
      DWORD PrePlay:1;		//In preplay mode?
      DWORD PlotChannels:1;	//Plot channel pointers to DbgMsg profiler?
      DWORD ScrCapPending:1;	//SCR capture has been set up, pending capture
      DWORD DtsReorder:1;	//Enable PTS->DTS frame reordering on video.
      DWORD SubAdjEnabled:1;	//Enable subpicture skips.
      DWORD Active:1;		//Set only if both audio and video are active.
    }s;
    DWORD Flags;
  }u;
};

//Global sync dump flags. Setting a flag causes that dump to occur once at the
//next opportunity.
struct Dump_s {
  union {
    struct {			//Dump at next opportunity.
      DWORD Sync:1;		//SYNC
      DWORD Table:1;		//TABLE (Video and audio)
      DWORD Entries:1;		//ENTRY
      DWORD Stats:1;		//Statistics
    }s;
    DWORD Flags;
  }u;
  DWORD dwTime;
};

//The sync object. Contains all sync information, audio and video.
#define SIGNATURE 0xCD010001
typedef struct AVSync_s {
  DWORD Signature;
  PHW pHw;
  BYTE StrmCapturing;		//Capture SCR set on this stream.
  struct SyncFlag_s F;
  struct Dump_s DumpEnable;
  struct Dump_s DumpPending;
  TABLE *pVid;			//Pointer to video sync table
  TABLE *pAud;			//Pointer to audio sync table
  DWORD ZeroTime;		//Beginning of PC time
  DWORD LastTime;		//PC time of the previous call to SyncProc().
  DWORD CurTime;		//PC time at which this call to SyncProc() began.
  DWORD LastTick;		//PC time of last TIMER command.
  DWORD CurAvgTick;		//Crude rolling median tick interval.
  DWORD dwLastVidPktTime; //Time last vid pes hdr captured.
  DWORD CurSCR;			  //Decoder SCR when this call to SyncProc() began.
  DWORD ScrOverFlow;  //Counts numebr of time SCR has wrapped
  DWORD MaxTolerance;		//If drift is beyond this, don't try to pull it back.
  DWORD MinAdjustPeriod;	//Minimum elapsed time between adjustments. (milliseconds)
  DWORD AdjHoldoff;		//Hold off performing adjustments for this period (msec)
  DWORD AdjEnableTime;		//Only enable adjustments after this PcTime.
//  DWORD Autostarting;		//Autostart in progress. Combination of STRM_AUD and/or STRM_VID.
  DWORD PrePlayDelay;		//Fudge-factor added to SCR reported to Microsoft to cover up autostart delay.
  DWORD PlayStartScr;		//SCR reported to Microsoft during autostart.
  DWORD dwVidRdPtr;     //Most recent unused VidRdPtr captured  0xffffffff if used
  int   nPicCount;
  int   nPscCount;
  int   nPrefillWatchDog;   //Count ticks to make sure prefill doesn't get stuck
  int   fParseForPTS;       // Set True if we have to extract PTS from packet otherwise use time in MDP
  int   nAudPesTick;        // Counters used to time setting of PES header capture reg.
  int   nVidPesTick;
  int   fAudPesCaptAll;     // flags set when capturing all PES headers (usually in prefill mode)
  int   fVidPesCaptAll;
} SYNC;

//struct DbgMsgPerf_s PerfStatVidError= { "Video Error","Video Error","Video Error",0 };
//struct DbgMsgPerf_s PerfStatAudError= { "Audio Error","Audio Error","Audio Error",0 };

LOCAL long SyncInit(PHW pHw);
LOCAL SYNC *SyncInitAlloc(void);
LOCAL long SyncReset(SYNC *pSync,DWORD FullReset,DWORD dwStrm);
LOCAL long SyncFini(SYNC *pSync);
//LOCAL long SyncPreplay(SYNC *pSync, DWORD Enter);
//LOCAL long SyncAutostart(SYNC *pSync, DWORD Streams, DWORD InitialScr);
//LOCAL long SyncTimeDiscontinuity(SYNC *pSync);
LOCAL long SyncGet(SYNC *pSync, DWORD Item, DWORD *pValue);
LOCAL long SyncSet(SYNC *pSync, DWORD Item, DWORD Value);
LOCAL long SyncEnable(SYNC *pSync, DWORD StrmType, DWORD Enable);
LOCAL long SyncPktHdrGetPts(SYNC *pSync,BYTE *pPktData, DWORD* pPts);
LOCAL long SyncStreamStart(SYNC* pSync,DWORD dwStrm,long lPtsErr);

LOCAL long SyncShowError(SYNC *pSync);
LOCAL long SyncApplyAdjustments(SYNC *pSync);
LOCAL long SyncInvalidMsg(DWORD wMsg);
LOCAL long SyncTimer(SYNC *pSync, DWORD IsTick, DWORD CaptureType);
LOCAL void SyncDumpChkTime(SYNC *pSync);
LOCAL void SyncDump(SYNC *pSync);
LOCAL long SyncGetScr(SYNC *pSync, DWORD* pdwTime90Hi, DWORD* pdwTime90Lo);
LOCAL long SyncSetScr(SYNC *pSync, SYNC_TIME90* pNewTime90, SYNC_TIME90* pOldTime90);
LOCAL long SyncSetPtsMode(SYNC* pSync,DWORD dwUseMdp);
LOCAL DWORD SyncGetStreamPts(TABLE *pTbl, DWORD* pdwOverFlow, DWORD* pdwTime90);
LOCAL DWORD PtsFindVid(BYTE *pData, DWORD nBytes, DWORD *pSCR);
LOCAL DWORD PtsFindAud(BYTE *pData, DWORD nBytes, DWORD *pPtsLoc);
LOCAL DWORD GetMinPts(TABLE *pTbl);


static SYNC StaticAVSync;
static TABLE StaticVidTbl;
static TABLE StaticAudTbl;
static ENTRY StaticVidEntries[MAXENTRIES];
static ENTRY StaticAudEntries[MAXENTRIES];

//Converts a handle to a SYNC pointer.
LOCAL SYNC *SyncPtr(DWORD hSync) {
  SYNC *pSync= (SYNC*)hSync;
  if(pSync && pSync->Signature!=SIGNATURE)
    pSync= 0;
  return(pSync);
}

void DbgDispPTS(DWORD dwDbgFilt,DWORD llPTS,char* szPre,char* szPost) {
  DWORD llSecs;
  if(szPre)
    DbgMsg(dwDbgFilt,"%s",szPre);
  llSecs=llPTS/90000;
  if(llSecs)
    DbgMsg(dwDbgFilt,"%d,",(int)llSecs);
  DbgMsg(dwDbgFilt,"%03d.%02dmS",(int)((llPTS%90000)/90),(int)(llPTS%90));
  if(szPost)
    DbgMsg(dwDbgFilt,"%s",szPost);
}

/*************************************************************************/
/** This is the only exported entry point to the sync functions.	**/
/*************************************************************************/
long SyncCmd(PHW pHw, DWORD Msg, DWORD Arg1, DWORD Arg2) {
  long Err= 0;
  SYNC *pSync;
  static BYTE ErrDisplayed;
//  DbgMsgCmd(DBGCMD_PROF_ENTER,DBGPRF_SYNCPROC,0);
  if(LOWORD(Msg)==SYNC_INIT) {
    ErrDisplayed= 0;
    Err= SyncInit(pHw);
  } else if(!(pSync= SyncPtr(pHw->hAvSync))) {
    if(!ErrDisplayed) {
      DbgMsg(DBG_SYNCERR,"dvHwSyncProc: No hAvSync for command %u.\r\n",LOWORD(Msg));
      ErrDisplayed= 1;
    }
    Err= MARGI_ERR_BADHANDLE;
  } else if(!pSync->F.u.s.Started) {
    if(!ErrDisplayed) {
      DbgMsg(DBG_SYNCERR,"dvHwSyncProc: AVSync has not been started for command %u.\r\n",LOWORD(Msg));
      ErrDisplayed= 1;
    }
    Err= MARGI_ERR_NOTREADY;
  } else {
    pSync->CurSCR=dvHwMainGetDecoderTime(pSync->pHw,&pSync->ScrOverFlow);
    pSync->CurTime=MargiGetTime();

    ErrDisplayed= 0;
    DbgMsg(DBG_SYNCMSG|DBG_HIFREQ,"SyncProc(%u) %ums elapsed, SCR=%u.\r\n",LOWORD(Msg),pSync->CurTime-pSync->LastTime,pSync->CurSCR);
    switch(LOWORD(Msg)) {
      case SYNC_FINI:           Err= SyncFini(pSync); break;
      case SYNC_RESET:          Err= SyncReset(pSync,Arg1,Arg2); break;
      case SYNC_PKTHDR_GET_PTS: Err= SyncPktHdrGetPts(pSync,(BYTE*)Arg1,(DWORD*)Arg2); break;
      case SYNC_SHOW_ERROR:	    Err= SyncShowError(pSync); break;
      case SYNC_TIMER:          Err= SyncTimer(pSync,1,Arg1); break;
      case SYNC_GET:            Err= SyncGet(pSync,Arg1,(DWORD*)Arg2); break;
      case SYNC_SET:            Err= SyncSet(pSync,Arg1,Arg2); break;
      case SYNC_ENABLE:	        Err= SyncEnable(pSync,Arg1,Arg2); break;
      case SYNC_GET_SCR:        Err= SyncGetScr(pSync,(DWORD*)Arg1,(DWORD*)Arg2); break;
      case SYNC_SET_SCR:        Err= SyncSetScr(pSync,(SYNC_TIME90*)Arg1,(SYNC_TIME90*)Arg2); break;
      case SYNC_SET_PTS_MODE:   Err= SyncSetPtsMode(pSync,Arg1); break;
      case SYNC_GET_VIDPTS:     Err= SyncGetStreamPts(pSync->pVid,(DWORD*)Arg1,(DWORD*)Arg2); break;
      case SYNC_GET_AUDPTS:     Err= SyncGetStreamPts(pSync->pAud,(DWORD*)Arg1,(DWORD*)Arg2); break;
      case SYNC_STRM_START:     Err= SyncStreamStart(pSync,Arg1,(long)Arg2); break;
      default:                  Err= SyncInvalidMsg(Msg); break;
    }
    SyncDump(pSync);
    pSync->LastTime= pSync->CurTime;
  }
//  DbgMsgCmd(DBGCMD_PROF_EXIT,DBGPRF_SYNCPROC,0);
  return(Err);
}

LOCAL long SyncInvalidMsg(DWORD wMsg) {
  long Err= MARGI_ERR_UNK_MSG;
  DbgMsg(DBG_SYNCERR,"Sync: Unknown message received %d \r\n",wMsg);
  return(Err);
}

/*************************************************************************/
/** Initialization code.						**/
/*************************************************************************/
LOCAL long SyncResetTbl(TABLE *pTbl, DWORD FullReset);

LOCAL long SyncInit(PHW pHw) {
  long Err= 0;
  SYNC *pSync;
  SettingFName(0,IniSectionSync);
  if(!(pSync= SyncInitAlloc())) {
    Err= MARGI_ERR_NOMEM;
  } else {
    DbgMsg(DBG_SYNCINIT,"Sync: Init.\r\n");
    pSync->Signature= SIGNATURE;
    pSync->pHw= pHw;
    pHw->hAvSync= (DWORD)pSync;
    //Global Sync settings.
    pSync->DumpEnable.u.s.Sync= SettingLoad(SET_DWORD,IniSyncDumps,0,0);
    pSync->DumpEnable.u.s.Table= SettingLoad(SET_DWORD,IniSyncDumpTable,0,0);    //Enable table dumps
    pSync->DumpEnable.u.s.Entries= SettingLoad(SET_DWORD,IniSyncDumpEntries,0,0);
    pSync->DumpEnable.u.s.Stats= SettingLoad(SET_DWORD,IniSyncDumpStats,0,0);
    pSync->DumpEnable.dwTime= SettingLoad(SET_DWORD,IniSyncDumpPeriod,0,2000);
    pSync->DumpPending.u.Flags= pSync->DumpEnable.u.Flags;
    pSync->DumpPending.dwTime= pSync->CurTime;
    pSync->F.u.s.PlotChannels= SettingLoad(SET_DWORD,IniSyncPlotChannels,0,0);
    pSync->F.u.s.DtsReorder= SettingLoad(SET_DWORD,IniSyncDtsReorder,0,1);
    pSync->MaxTolerance= SettingLoad(SET_DWORD,IniSyncMaxTolerance,0,0);
    pSync->MinAdjustPeriod= SettingLoad(SET_DWORD,IniSyncMinAdjustPeriod,0,500);
    pSync->AdjHoldoff= SettingLoad(SET_DWORD,IniSyncAdjHoldoff,0,250);
    pSync->fParseForPTS=0;
    //Video settings.
    pSync->pVid->u.s.Enable= SettingLoad(SET_DWORD,IniSyncVideo,0,1);
    pSync->pVid->u.s.EnableChk= SettingLoad(SET_DWORD,IniSyncVideoChk,0,1);
    pSync->pVid->u.s.CaptureSCR= SettingLoad(SET_DWORD,IniSyncCaptureVid,0,1);
    pSync->pVid->u.s.EnableAdj= SettingLoad(SET_DWORD,IniSyncVidAdjustEnable,0,1);
    pSync->pVid->u.s.IsMaster=0;
    pSync->pVid->ToleranceFreeze= SettingLoad(SET_DWORD,IniSyncVidToleranceFreeze,0,90);
    pSync->pVid->MaxFreeze= SettingLoad(SET_DWORD,IniSyncVidMaxFreeze,0,3000);
    pSync->pVid->ToleranceGross= SettingLoad(SET_DWORD,IniSyncVidToleranceGross,0,20);
    pSync->pVid->ToleranceFine= SettingLoad(SET_DWORD,IniSyncVidToleranceFine,0,20);
    pSync->pVid->PtsDelay= SettingLoad(SET_DWORD,IniSyncVidDelay,0,-30)*90;	    //Delay Video in mS (Convert to 90KHz ticks)
    pSync->pVid->ReadFifoSize= SettingLoad(SET_DWORD,IniSyncVidReadFifo,0,48);	//Size of video read FIFO.
    pSync->pVid->PktUncertaintyTime= SettingLoad(SET_DWORD,IniSyncVidPktUncertainty,0,250); //5 frames * 50mS (for PAL)
    pSync->pVid->InSyncErrorPersistence=  SettingLoad(SET_DWORD,IniSyncVidInSyncErrorPersistence,0,1000);
    pSync->pVid->OutSyncErrorPersistence= SettingLoad(SET_DWORD,IniSyncVidOutSyncErrorPersistence,0,0);
    //Audio settings.
    pSync->pAud->u.s.Enable= SettingLoad(SET_DWORD,IniSyncAudio,0,1);
    pSync->pAud->u.s.EnableChk= SettingLoad(SET_DWORD,IniSyncAudioChk,0,1);
    pSync->pAud->u.s.CaptureSCR= SettingLoad(SET_DWORD,IniSyncCaptureAud,0,1);
    pSync->pAud->u.s.EnableAdj= SettingLoad(SET_DWORD,IniSyncAudAdjustEnable,0,1);		//NEVER SOMETIMES change SCR, other than autostart. (Microsoftism)
    pSync->pAud->u.s.IsMaster=1;
    pSync->pAud->ToleranceGross= SettingLoad(SET_DWORD,IniSyncAudToleranceGross,0,10);
    pSync->pAud->ToleranceFine= SettingLoad(SET_DWORD,IniSyncAudToleranceFine,0,1);
    pSync->pAud->PtsDelay= SettingLoad(SET_DWORD,IniSyncAudDelay,0,0)*90;		    //Delay Audio in mS (Convert to 90KHz ticks)
    pSync->pAud->ReadFifoSize= SettingLoad(SET_DWORD,IniSyncAudReadFifo,0,8);		//Size of audio read FIFO.
    pSync->pAud->PktUncertaintyTime= SettingLoad(SET_DWORD,IniSyncAudPktUncertainty,0,40);  // greater than 1 audio frame
    pSync->pAud->InSyncErrorPersistence=  SettingLoad(SET_DWORD,IniSyncAudInSyncErrorPersistence,0,200);
    pSync->pAud->OutSyncErrorPersistence= SettingLoad(SET_DWORD,IniSyncAudOutSyncErrorPersistence,0,200);
    //Subpicture settings.
    pSync->F.u.s.SubAdjEnabled= SettingLoad(SET_DWORD,IniSyncSubAdjustEnable,0,0);
    pSync->F.u.s.Started= 1;
//    DbgMsgCmd(DBGCMD_PERFADD,(DWORD)&PerfStatVidError,(DWORD)&pSync->pVid->hPerfStat);
//    DbgMsgCmd(DBGCMD_PERFADD,(DWORD)&PerfStatAudError,(DWORD)&pSync->pAud->hPerfStat);
    SyncReset(pSync,1,STRM_VID|STRM_AUD);

#if(AVSYNC_OFF)
    pSync->pVid->u.s.DisableAdj=1;   // Temp disable AVSync Adjustments
    pSync->pAud->u.s.DisableAdj=1;   // for debug.
#endif
  }
  SettingFName(0,IniMdsSection);
  return(Err);
}

LOCAL SYNC *SyncInitAlloc(void) {
  SYNC *pSync= &StaticAVSync;
  //Eventually replace these lines with dynamic memory allocation.
  memset(&StaticAVSync,0,sizeof(StaticAVSync));
  memset(&StaticVidTbl,0,sizeof(StaticVidTbl));
  memset(&StaticVidEntries,0,sizeof(StaticVidEntries));
  memset(&StaticAudTbl,0,sizeof(StaticAudTbl));
  memset(&StaticAudEntries,0,sizeof(StaticAudEntries));
  //Set up values which are treated differently for audio or video.
  //Set up video table.
  pSync->pVid= &StaticVidTbl;
  pSync->pVid->pSync= pSync;
  pSync->pVid->StrmType= STRM_VID;
  //pSync->pVid->Name="Vid/0";//0x00646956; //"Vid/0"
  pSync->pVid->pEntries= StaticVidEntries;
  pSync->pVid->MaxEntry= sizeof(StaticVidEntries)/sizeof(StaticVidEntries[0]);
  pSync->pVid->u.s.Enable= SettingLoad(SET_DWORD,IniSyncVideo,0,1);
  //Set up audio table.
  pSync->pAud= &StaticAudTbl;
  pSync->pAud->pSync= pSync;
  pSync->pAud->StrmType= STRM_AUD;
//  strncpy(pSync->pAud->Name,"Audio",sizeof(pSync->pAud->Name));
//  pSync->pAud->Name="Aud/0"; //0x00647541; //"Aud/0"
  pSync->pAud->pEntries= StaticAudEntries;
  pSync->pAud->MaxEntry= sizeof(StaticAudEntries)/sizeof(StaticAudEntries[0]);
  pSync->pAud->u.s.Enable= SettingLoad(SET_DWORD,IniSyncAudio,0,1);
  return(pSync);
}

LOCAL long SyncReset(SYNC *pSync,DWORD FullReset,DWORD dwStrm) {
  long Err= 0;
  DbgMsg(DBG_SYNCADJ,"Sync: RESET.\r\n");
  if(FullReset) {
    DbgMsg(DBG_SYNCADJ,"Sync: FULL RESET.\r\n");
    pSync->F.u.s.PrePlay= 1;
    pSync->ZeroTime= MargiGetTime();
    pSync->CurTime= pSync->ZeroTime;
    pSync->LastTime= pSync->ZeroTime;
    pSync->LastTick= pSync->ZeroTime;
    pSync->CurAvgTick= 0;
    pSync->AdjEnableTime= 0;
    pSync->F.u.s.Active= 0;
    pSync->ScrOverFlow=0;
    pSync->CurSCR=0;
//    pSync->CurSCR=0xffe00000;  // 22 secs before wrap
    dvHwMainResetScr(pSync->pHw,pSync->CurSCR,pSync->ScrOverFlow);
  }
  pSync->nPrefillWatchDog=0;
  pSync->StrmCapturing= 0;
  pSync->F.u.s.ScrCapPending= 0;

  dvHwMainClrPesHeader(pSync->pHw);
  pSync->nVidPesTick=0;
  pSync->nAudPesTick=0;
  pSync->fVidPesCaptAll=0;
  pSync->fAudPesCaptAll=0;

  if(dwStrm&STRM_VID) {
    SyncResetTbl(pSync->pVid,FullReset);
    pSync->dwVidRdPtr=0xffffffff;
    pSync->nPicCount=0;
    pSync->nPscCount=0;
  }
  if(dwStrm&STRM_AUD)
    SyncResetTbl(pSync->pAud,FullReset);

  return(Err);
}

LOCAL long SyncResetTbl(TABLE *pTbl, DWORD FullReset) {
  long Err= 0;
  if(FullReset) {
    pTbl->HighWater= 0;
    pTbl->u.s.MaxToleranceMsg= 0;
    pTbl->u.s.StartCompleted= 0;
    pTbl->LastIntr= pTbl->pSync->CurTime;
    pTbl->FrameCnt= 0;
    pTbl->u.s.Active= 0;
    pTbl->CurPts=0;
    pTbl->CurPtsOverFlow=0;
    pTbl->CurPtsTime=0;
    pTbl->nResetCount=0;
  }
  memset(pTbl->pEntries,0,sizeof(*pTbl->pEntries)*pTbl->MaxEntry);
  pTbl->Head= pTbl->Tail= pTbl->EntryCnt= 0;
  pTbl->u.s.EntryStarted= 0;
  pTbl->u.s.ErrorValid= 0;
  pTbl->u.s.ErrorAhead= 0;
  pTbl->u.s.AwaitingPicType= 0;
  pTbl->u.s.PktUncertain= 0;
  pTbl->u.s.OutOfSync= 1;
  pTbl->Tolerance= pTbl->ToleranceGross;
  pTbl->Error= 0;
  pTbl->ErrorTicks= 0;
  pTbl->LastAdjustment= 0;
  pTbl->PtsLast= 0;
  pTbl->PtsLastScr= 0;
  pTbl->PtsMin= 0;
  pTbl->PtsMatches= 0;
  pTbl->DtsLastIP= 0;
  pTbl->DtsLastUncertainty=0;
  pTbl->ErrorStartTime= 0;
  pTbl->LastAdjustment= 0;
  pTbl->InSyncCount=0;
  return(Err);
}

LOCAL long SyncFini(SYNC *pSync) {
  long Err= 0;
  PHW pHw= pSync->pHw;
  SettingFName(0,IniSectionSync);
  Err= SyncReset(pSync,1,STRM_AUD|STRM_VID);
  StaticAVSync.Signature= 0;
  pHw->hAvSync= 0;
  DbgMsg(DBG_SYNCINIT,"Sync: FINI: %s.\r\n",Ident(ID_ERR,Err));
  SettingFName(0,IniMdsSection);
  return(Err);
}

/*************************************************************************/
/** Fetch current or minimum PTS found.					**/
/*************************************************************************/

LOCAL long SyncGet(SYNC *pSync, DWORD Item, DWORD *pValue) {
  long Err= 0;
  DWORD Value;
  switch(Item) {
    case SYNC_ITEM_AUDFRAMES:	Value= pSync->pAud->FrameCnt;  break;
    case SYNC_ITEM_VIDFRAMES:	Value= pSync->pVid->FrameCnt;  break;
    case SYNC_ITEM_AUDPTSMIN:	Value= GetMinPts(pSync->pAud); break;
    case SYNC_ITEM_AUDPTSCUR:	Value= pSync->pAud->PtsLast;   break;
    case SYNC_ITEM_VIDPTSMIN:	Value= GetMinPts(pSync->pVid); break;
    case SYNC_ITEM_VIDPTSCUR:	Value= pSync->pVid->PtsLast;   break;
    default:                    Err= MARGI_ERR_BADITEM;
  }
  if(!Err) {
    if(pValue)
      *pValue= Value;
    else
      Err= (long)Value;
  }
  return(Err);
}

LOCAL long SyncSet(SYNC *pSync, DWORD Item, DWORD Value) {
  long Err= 0;
  switch(Item) {
    default:			Err= MARGI_ERR_BADITEM;
  }
  return(Err);
}

LOCAL long SyncSetPtsMode(SYNC* pSync,DWORD dwUseMdp) {
  long Err= 0;
  if(dwUseMdp)
    pSync->fParseForPTS=0;
  else
    pSync->fParseForPTS=1;
  return Err;
}

//Return the current SCR, in 90KHz ticks.
LOCAL long SyncGetScr(SYNC *pSync, DWORD* pdwOverFlow, DWORD* pdwTime90) {
  long Err= 0;
  if(pdwTime90)
    *pdwTime90=   pSync->CurSCR;
  if(pdwOverFlow)
    *pdwOverFlow= pSync->ScrOverFlow;
  return(Err);
}

LOCAL long SyncSetScr(SYNC *pSync, SYNC_TIME90* pNewTime90, SYNC_TIME90* pOldTime90) {
#if(0)
  pNewTime90->dwTimeLo+=25000;
#endif
  if(pNewTime90->dwTimeLo==0 && pNewTime90->dwTimeHi==0)
    pNewTime90->dwTimeLo = 1;
  dvHwMainSetDecoderTime(pSync->pHw,pNewTime90->dwTimeLo,pNewTime90->dwTimeHi,pOldTime90->dwTimeLo,pOldTime90->dwTimeHi);
  return 0;
}


//Return current stream time in 90KHz ticks
LOCAL DWORD SyncGetStreamPts(TABLE *pTbl, DWORD* pdwOverFlow, DWORD* pdwTime90) {
  long Err= 0;
  if(pdwTime90)
    *pdwTime90=   pTbl->CurPts;
  if(pdwOverFlow)
    *pdwOverFlow= pTbl->CurPtsOverFlow;
  return(Err);
}

LOCAL long SyncEnable(SYNC *pSync, DWORD StrmType, DWORD Enable) {
  if(StrmType & STRM_VID) {
    pSync->pVid->u.s.Active   =  Enable&SYNC_ENABLE_ENABLE;
    if(pSync->pVid->u.s.DisableAdj)
      pSync->pVid->u.s.EnableAdj=0;
    else {
      pSync->pVid->u.s.IsMaster =((Enable&SYNC_ENABLE_VID_MASTER)!=0);
      pSync->pVid->u.s.EnableAdj=((Enable&SYNC_ENABLE_NO_ADJUST)==0);
    }
  }
  if(StrmType & STRM_AUD) {
    pSync->pAud->u.s.Active   =  Enable&SYNC_ENABLE_ENABLE;
    if(pSync->pAud->u.s.DisableAdj)
      pSync->pAud->u.s.EnableAdj=0;
    else {
      pSync->pAud->u.s.IsMaster =((Enable&SYNC_ENABLE_VID_MASTER)==0);
      pSync->pAud->u.s.EnableAdj=((Enable&SYNC_ENABLE_NO_ADJUST)==0);
    }
  }
  pSync->F.u.s.Active= (pSync->pVid->u.s.Active && pSync->pAud->u.s.Active);
  if(pSync->F.u.s.Active)
    DbgMsg(DBG_SYNCADJ,"Sync: Monitoring active.\r\n");
  else
    DbgMsg(DBG_SYNCADJ,"Sync: Monitoring idle.\r\n");
  return(0);
}

/*****************************************************************
If the video or audio started within 12 mS of required time
then clear out of sync flag.  This will cause A/V Sync filter
to operate and cause fewer disruptions at start.
******************************************************************/
LOCAL long SyncStreamStart(SYNC* pSync,DWORD dwStrm,long lPtsErr) {
  TABLE *pTbl=0;
  if(dwStrm&STRM_VID)
    pTbl=pSync->pVid;
  else if(dwStrm&STRM_AUD)
    pTbl=pSync->pAud;
  if(pTbl) {
    if(lPtsErr<0)
      lPtsErr= -lPtsErr;
    if(lPtsErr< 90*12)
      pTbl->u.s.OutOfSync=1;  
  }
  return 0;  
}

/*************************************************************************/
/** SYNC_VID_PKT_BEGIN and SYNC_AUD_PKT_BEGIN.				**/
/** -	This message is sent before video data is written to the	**/
/**	decoder.							**/
/** -	I allocate a new entry in the appropriate sync table and record	**/
/**	the PTS, PcTime, and channel address of the first byte of the	**/
/**	packet for use during sync table lookups.			**/
/** -	Note that not all packets will have a PTS, and not all packets	**/
/**	will contain a picture start code. Because the occasions when a	**/
/**	packet contains both a PTS and PSC turns out to be rare, I need	**/
/**	to generate a PTS for packet where it is missing. I do this by	**/
/**	adding the packet SCR (which is always present) difference	**/
/**	since the last known PTS to the last known PTS to generate a	**/
/**	predicted PTS. [I am not sure this is actually correct.]	**/
/*************************************************************************/
LOCAL long SyncPktBegin(TABLE *pTbl,DWORD dwChWr,DWORD Pts,DWORD Scr,BYTE *pPktData,DWORD nBytes);
LOCAL long SyncPktEnd(TABLE *pTbl,DWORD dwChWr);

LOCAL long SyncPktHdrGetPts(SYNC *pSync,BYTE *pPktData, DWORD* pPts) {
  long Err= 0;
  DWORD Pts,Scr;
  Pts= PtsFindVid(pPktData,256,&Scr);
  if(pPts)
    *pPts=Pts;
  else
    Err=MARGI_ERR_BADPTR;

  return(Err);
}

void* SyncVidPktBegin(PHW pHw,BYTE *pPktData,DWORD dwBytes,SYNC_TIME90* pTime90) {
  DWORD Pts,Scr,dwChWr;
  SYNC* pSync= SyncPtr(pHw->hAvSync);
  if(pSync) {
    if(pSync->fParseForPTS) {
      Pts=PtsFindVid(pPktData,dwBytes,&Scr);
      if(Pts!=pTime90->dwTimeLo) {
        volatile int bp=0;
      }
    } else {
      Pts=pTime90->dwTimeLo;
      Scr=0;
    }
//    dwChWr=dvHwGetChWrPtr(pSync->pHw,VIDCH);
    dwChWr=dvHwMainGetChWrCount(pSync->pHw,VIDCH);
    if(Pts) {
		SyncPktBegin(pSync->pVid,dwChWr,Pts,Scr,pPktData,dwBytes);
		SyncPktEnd(pSync->pVid,dwChWr+0x100);
    }
  }
  return (void*)pSync;
}

void* SyncAudPktBegin(PHW pHw,BYTE *pPktData,DWORD dwBytes,SYNC_TIME90* pTime90) {
  DWORD Pts,Scr,dwChWr;
  SYNC* pSync= SyncPtr(pHw->hAvSync);
  if(pSync) {
    if(pSync->fParseForPTS) {
      Pts=PtsFindVid(pPktData,dwBytes,&Scr);
      if(Pts!=pTime90->dwTimeLo) {
        volatile int bp=0;
      }
    } else {
      Pts=pTime90->dwTimeLo;
      Scr=0;
    }
//    dwChWr=dvHwGetChWrPtr(pSync->pHw,AUDCH);
    dwChWr=dvHwMainGetChWrCount(pSync->pHw,AUDCH);
    SyncPktBegin(pSync->pAud,dwChWr,Pts,Scr,pPktData,dwBytes);
  }
  return (void*)pSync;
}

LOCAL long SyncPktBegin(TABLE *pTbl,DWORD dwChWr,DWORD Pts,DWORD Scr,BYTE *pPktData,DWORD nBytes) {
  long Err= 0;
  ENTRY *pEntry= &pTbl->pEntries[pTbl->Head];
  if(pTbl->u.s.EntryStarted) {
    DbgMsg(DBG_SYNCDROP,"SyncPktBegin: %s entry already started.\r\n",pTbl->Name);
    pTbl->DropCnt++;
  } else {
    if(Pts) {
//      if(pTbl->StrmType==STRM_VID)
//        DbgMsg(DBG_SYNCPTS,"%s PTS: %u %u\r\n",pTbl->Name,pTbl->pSync->CurSCR,Pts);
      Pts+= pTbl->PtsDelay;				//Adjust for Audio - Video delay differential
      if(!Pts) Pts++;                   //If Pts is 0 the set to 1 since 0 means invalid
      pTbl->PtsLast= Pts;
      pTbl->PtsLastScr= Scr;
      if(pTbl->StrmType==STRM_VID) {
        if(!pTbl->PtsMin || Pts<pTbl->PtsMin) {
          pTbl->PtsMin= Pts;
          DbgMsg(DBG_SYNCADJ,"Sync: New %s minimum PTS %u.\r\n",pTbl->Name,Pts);
        }
      }
    }

    // Test if Table is about to fill i.e. Head will collide with Tail.
    // If it is then lose the entry at Tail
    if((pTbl->Head+1)>=pTbl->MaxEntry) {
      if(pTbl->Tail==0)
        pTbl->Tail++;
    } else {
      if((pTbl->Head+1)==pTbl->Tail)
        if(++pTbl->Tail>=pTbl->MaxEntry)
          pTbl->Tail=0;
    }

    pTbl->u.s.EntryStarted= 1;
    if(Pts) {
      pEntry->Pts= Pts;
      if(pTbl->u.s.StartCompleted) {			//Autostart has just completed?
        DbgMsg(DBG_SYNCSTART,"SyncPktBegin: %s First post-autostart PTS %u.\r\n",pTbl->Name,Pts);
        pTbl->u.s.StartCompleted= 0;
      }
    } else {
      if(0 && pTbl->PtsLast && pTbl->PtsLastScr && Scr)
        pEntry->Pts= pTbl->PtsLast+(Scr-pTbl->PtsLastScr);
      else
        pEntry->Pts= 0;
    }
    pEntry->PktStart=dwChWr;	                		//Where next video byte will be written.
//    pEntry->PcTime= pTbl->pSync->CurTime;
    pEntry->SyncEvents= 0;
//    if(pTbl->EntryCnt>pTbl->HighWater)
//      pTbl->HighWater= pTbl->EntryCnt;
  }
  return(Err);
}

//Since autostart and SCR capture use the same registers, I need to not program
//SCR capture if an autostart is in progress. The autostart code (in dvHwMain.c)
//sends a SYNC_AUTOSTART command to announce the initiation of autostart.
//The SYNC_TIMER command then checks for the autostart completion and clears
//the Autostarting flag.
LOCAL long SyncPktEnd(TABLE *pTbl,DWORD dwChWr) {
  long Err= 0;
  ENTRY *pEntry;
  if(0) { //!pTbl->Enable) { // || !pTbl->pSync->F.Active) {
  } else if(!pTbl->u.s.EntryStarted) {
    DbgMsg(DBG_SYNCDROP,"SyncPktEnd: No %s entry started.\r\n",pTbl->Name);
  } else {
    pEntry= &pTbl->pEntries[pTbl->Head];
    pEntry->PktEnd= dwChWr;
//    pEntry->PcTime= pTbl->pSync->CurTime;	//Record PC time for timeout
    if(++pTbl->Head>=pTbl->MaxEntry)
      pTbl->Head= 0;
    pTbl->u.s.EntryStarted= 0;
  }
  return(Err);
}

long SyncVidPktEnd(void* p) {
  long Err=MARGI_ERR_BADHANDLE;
  DWORD dwChWr;
  if(p) {
    SYNC* pSync=(SYNC*)p;
    dwChWr=dvHwMainGetChWrCount(pSync->pHw,VIDCH);
    Err= SyncPktEnd(pSync->pVid,dwChWr);
  }
  return(Err);
}

long SyncAudPktEnd(void* p) {
  long Err=MARGI_ERR_BADHANDLE;
  DWORD dwChWr;
  if(p) {
    SYNC* pSync=(SYNC*)p;
    dwChWr=dvHwMainGetChWrCount(pSync->pHw,AUDCH);
    Err= SyncPktEnd(pSync->pAud,dwChWr);
  }
  return(Err);
}

#define AUDPESTICKS 25
#define VIDPESTICKS 25

void SyncBuildTable(SYNC* p)
{
  BYTE  bId;
  DWORD dwPktBegin,dwPktEnd,dwPts;
#ifdef DEBUG
  DWORD dwChWr;
  // if we haven't seen a video pkt hdr for 500ms 
  // then check Channel Wr Ptr so we don't miss
  // a buffer wrap.
  if(p->CurTime>p->dwLastVidPktTime+200) {
    dvHwMainGetCheckChWrCount(p->pHw,VIDCH);
    p->dwLastVidPktTime=p->CurTime;
  }
#endif

  while(1)
  {
    bId=dvHwMainGetPesHeader(p->pHw,&dwPktBegin,&dwPktEnd,&dwPts);
    if(!bId)
      break;
    if(dwPktEnd>dwPktBegin+2047)
      DbgMsg(DBG_SYNCERR,"GetPesHeader dwPktBegin=%08lx  dwPktEnd=%08lx\r\n",dwPktBegin,dwPktEnd);
    if(bId==0xE0) {
      p->nVidPesTick=0;
      SyncPktBegin(p->pVid,dwPktBegin,dwPts,0,0,0);
      SyncPktEnd(p->pVid,dwPktEnd);
#ifdef DEBUG
      dwChWr=dvHwMainGetCheckChWrCount(p->pHw,VIDCH);
      if(p->pHw->ChInf[VIDCH].dwChkOvflow!=p->pHw->ChInf[VIDCH].dwWrOvflow)
        DbgMsg(DBG_SYNCVIDPTS,"VidPes WrPtr=%x:%08lx,ChkPtr=%x:%08lx\r\n",p->pHw->ChInf[VIDCH].dwWrOvflow,dwPktBegin,p->pHw->ChInf[VIDCH].dwChkOvflow,p->pHw->ChInf[VIDCH].dwPrvChkPtr);
#endif
      p->dwLastVidPktTime=p->CurTime;
    } else {
      if(p->nAudPesTick>AUDPESTICKS)
        p->nAudPesTick=0;
      SyncPktBegin(p->pAud,dwPktBegin,dwPts,0,0,0);
      SyncPktEnd(p->pAud,dwPktEnd);
    }
  }

  // Capture all PES Headers only if VOB data or prefilling MPEG2/MPEG1
  // Otherwise capture one video pes header every approx. 175 (25*7) mS
  if(p->pHw->nMpegType&2 || p->pHw->u.s.fVidChannelPrefill) {
    dvHwMainWriteLsiRegBits(p->pHw,VIDEO_PES_ENABLE_REG,WRITE_ALL_VID_PTS);
    p->fVidPesCaptAll=1;
  } else {
    if(p->fVidPesCaptAll || p->nVidPesTick>VIDPESTICKS) {
      dvHwMainWriteLsiRegBits(p->pHw,VIDEO_PES_ENABLE_REG,WRITE_ONE_VID_PTS);
      p->fVidPesCaptAll=0;
    }
    p->nVidPesTick++;
  }
    
  // Capture two video pes header every 175mS
  if(p->pHw->u.s.fAudChannelPrefill) {
    dvHwMainWriteLsiReg(p->pHw,PES_HEADER_ENABLE_REG,WRITE_ALL_AUD_PTS);
    p->fAudPesCaptAll=1;
  } else  {
    if(p->fAudPesCaptAll || (p->nAudPesTick==0) || (p->nAudPesTick>AUDPESTICKS)) {
      dvHwMainWriteLsiReg(p->pHw,PES_HEADER_ENABLE_REG,WRITE_ONE_AUD_PTS);
      p->fAudPesCaptAll=0;
    }
    p->nAudPesTick++;
  }
}

/*************************************************************************/
/** Periodic call to check current drifts.				**/
/** -	If autostart is in progress, check for autostart completion.	**/
/**	This is done by checking the autostart bits in register 19.	**/
/** -	IntrFlags are the state of Register1, which are cleared on read	**/
/**	and so must be read only once per timer tick.			**/
/** -	I need to see each and every audio sync code and video picture	**/
/**	start event in order to maintain all the tables.		**/
/** -	If a sync event is detected, then check to see whether SCR	**/
/**	capture was enabled for that channel. If so, read the captured	**/
/**	SCR. Otherwise pass use 0 to indicate no capture.		**/
/*************************************************************************/
LOCAL void  SyncChkTicks(SYNC *pSync);
//LOCAL void SyncChkAutostart(SYNC *pSync);
LOCAL DWORD SyncChkAutostartTbl(TABLE *pTbl, DWORD ChnlPaused);
LOCAL void  SyncChkFreeze(TABLE *pVid);
LOCAL long  SyncChkTbl(TABLE *pTbl,DWORD RdPtr,DWORD dwPicType);
LOCAL ENTRY *SyncFindEntry(TABLE *pTbl, DWORD RdPtr, BYTE *pPktUncertain);
LOCAL DWORD SyncPtsToDts(TABLE *pTbl,DWORD Pts,DWORD dwPicType);
LOCAL long  SyncSetScrCapture(SYNC *pSync);
LOCAL void  SyncUpdateScr(SYNC* pSync,DWORD dwScr);

LOCAL long SyncTimer(SYNC *pSync, DWORD IsTick, DWORD IntrFlags) {
  long Err= 0;
  DWORD RdPtr;
  DWORD dwScrLo=0;
  DWORD dwScrHi=0;
  DWORD dwPicType=0;
  PHW   pHw=pSync->pHw;
  if(IsTick)							  //Set only if a real timer tick.
    SyncChkTicks(pSync);					  //Check for missed timer ticks.

#if(0)
  dwScrLo=dvHwMainGetDecoderTime(pHw,&dwScrHi);
  pSync->CurSCR=dwScrLo;
  pSync->ScrOverFlow=dwScrHi;
  pSync->CurTime=MargiGetTime();
#endif

  if(pHw->fUsePesHdr)
    SyncBuildTable(pSync);

  if(IntrFlags & 1) {						  //Audio sync code
    RdPtr=dvHwMainGetChRdCount(pHw,AUDCH);
    DbgMsg(DBG_SYNCCHK,"Sync:Frm %08X\r\n",RdPtr);
    SyncChkTbl(pSync->pAud,RdPtr,0);
  }
  if(IntrFlags & 2) {						  //Video picture start code
    pSync->dwVidRdPtr=dvHwMainGetChRdCount(pHw,VIDCH);
    pSync->nPscCount++;
  }

  if(pSync->dwVidRdPtr!=0xffffffff) {
    DWORD dwVidEvent;
    static DWORD lastPic=0;
    if(AuxDataCmd(pHw,AUXCMD_GET,AUXGET_EVENTVID,(DWORD)&dwVidEvent)==0) {
      if(dwVidEvent & EVENTVID_SLICESTART) {		//If slice layer not decoded, picture type is not yet available.
        AuxDataCmd(pHw,AUXCMD_GET,AUXGET_PICTYPE,(DWORD)&dwPicType);
//        DbgMsg(DBG_SYNCCHK,"Sync:Pic %d %08X\r\n",dwPicType,pSync->dwVidRdPtr);
#if(0)
        if(!dwPicType)
          DbgMsg(DBG_SYNCCHK,"Sync: PicType = 0.\r\n");
        else {
          pSync->nPicCount++;
          if(pSync->nPicCount != pSync->nPscCount)
            DbgMsg(DBG_SYNCCHK,"Sync: #PicCount=%d, #PscCount=%d.\r\n",pSync->nPicCount,pSync->nPscCount);
        }

        if((lastPic < 3 && dwPicType!=3) || (lastPic==4 && (dwPicType!=1 && dwPicType!=2))) {
            DbgMsg(DBG_SYNCCHK,"Sync %s: Pic=%d, lastPic=%d.\r\n",pTbl->Name,dwPicType,lastPic);
            lastPic=dwPicType;
          } else if(dwPicType<3)
              lastPic=dwPicType;
          else
            lastPic++;
#endif
      }
    } else {
//      DbgMsg(DBG_SYNCCHK,"Sync: Wait for PicType.\r\n");
    }

  }

  if(pSync->dwVidRdPtr!=0xffffffff && dwPicType) {
    SyncChkTbl(pSync->pVid,pSync->dwVidRdPtr,dwPicType);
    pSync->dwVidRdPtr=0xffffffff;
  }
  SyncApplyAdjustments(pSync);
  SyncSetScrCapture(pSync);					  //Set new SCR capture.
  return(Err);
}

//Check for ticks, avoid lyme disease!
LOCAL void SyncChkTicks(SYNC *pSync) {
  DWORD ElapsedTime;
  if(pSync->LastTick) {						  //Check for missed timer ticks.
    ElapsedTime= pSync->CurTime-pSync->LastTick;
    if(ElapsedTime>30)						  //This is VERY BAD!
      DbgMsg(DBG_SYNCDROP,"Timer: %ums.\r\n",ElapsedTime);
    if(ElapsedTime>pSync->CurAvgTick)				  //This calculates a crude rolling median
      pSync->CurAvgTick++;					  //value for the period between timer ticks.
    else if(ElapsedTime<pSync->CurAvgTick)
      pSync->CurAvgTick--;
  }
  pSync->LastTick= pSync->CurTime;
}

//Search the sync table entries for a match. A match is found when the
//current channel read pointer is between the recorded channel write
//pointers for the packet begin and end, taking into account buffer
//wraparound. When a match is found, increment the count of events
//found within that packet. This is since there may be more than one
//sync code in a packet, but the PTS is only valid for the first one.
LOCAL long SyncChkTbl(TABLE *pTbl,DWORD RdPtr,DWORD dwPicType) {
  long Err= 0;
  ENTRY *pEntry;
  DWORD ElapsedTime,DecodeTime,CapturedSCR= 0;
  char *Direction;
  PHW pHw= pTbl->pSync->pHw;
  BYTE PktUncertain;
  DWORD DbgFltr;
  pTbl->FrameCnt++;						//Count frames (audio sync code or picture start code)
  if(pTbl->pSync->StrmCapturing==pTbl->StrmType) {		//Has SCR capture been set up for this stream?
    CapturedSCR= dvHwMainReadLsiReg32(pHw,13);			//Read captured SCR
    pTbl->pSync->F.u.s.ScrCapPending= 0;				//Clear capturing flag.
  }
  if(pTbl->LastIntr) {						//Bookkeeping only: keep track of median interval between sync events.
    ElapsedTime= pTbl->pSync->CurTime - pTbl->LastIntr;
    if(ElapsedTime>pTbl->CurAvgIntr)
      pTbl->CurAvgIntr++;
    else if(ElapsedTime<pTbl->CurAvgIntr)
      pTbl->CurAvgIntr--;
    if(ElapsedTime>60)						//Check for missed events.
      DbgMsg(DBG_SYNCDROP,"Sync: %s sync event %ums.\r\n",&pTbl->Name,ElapsedTime);
  }
  pTbl->LastIntr= pTbl->pSync->CurTime;				//Record last sync event time
  pEntry=SyncFindEntry(pTbl,RdPtr,&PktUncertain);		//Look for matching table entry.
  DecodeTime=SyncPtsToDts(pTbl,pEntry?pEntry->Pts:0,dwPicType); // Convert Video PTS to DTS This must happen every frame
  if(DecodeTime) {
    // PtsMin for video is collected here. The video decoder consumes data up to the
    // slice layer and issues pic start interrupts while in prefill mode.
    // PtsMin for audio is collected by GetPtsMin() funtion.
// Don't collect here any more video PtsMin is set in BeginPkt
//    if(pTbl->StrmType==STRM_VID && DecodeTime)
//      pTbl->PtsMin=DecodeTime;
    
//    if(pEntry->SyncEvents) {                  //Only use entry if this is its first sync event.
//	    DbgMsg(DBG_SYNCDROP,"Sync: %s, %u prior events start=%08x, end=%08x, rdptr=%08x.\r\n",pTbl->Name,pEntry->SyncEvents+1,pEntry->PktStart,pEntry->PktEnd,RdPtr);
 //   } else 
    {
      if(CapturedSCR && DecodeTime) {	            //Do I have a captured SCR and DTS for drift calculation?
        DbgFltr= (pTbl->StrmType==STRM_AUD) ? DBG_SYNCDRIFTA:DBG_SYNCDRIFTV;
        //If the strm type is audio then copy audio in the name field else copy video
        strcpy((char*)(void*)&pTbl->Name,(pTbl->StrmType==STRM_AUD)?"Aud":"Vid");

        if(CapturedSCR>DecodeTime) {				//If time at which event actually happened is later than when it was supposed to happen...
	        Direction= "behind";              //...then playback is behind schedule.
          pTbl->u.s.ErrorAhead= 0;              //Playback not ahead.
	        pTbl->ErrorTicks= CapturedSCR-DecodeTime;		//Error in SCR ticks
	        pTbl->Error= pTbl->ErrorTicks/90; //Convert to milliseconds.
        } else {                            //Actual time was earlier than scheduled time.
	        Direction= "ahead";               //so playback is ahead of schedule.
	        pTbl->u.s.ErrorAhead= 1;              //Playback is ahead.
	        pTbl->ErrorTicks= DecodeTime-CapturedSCR;		//Error in SCR ticks.
	        pTbl->Error= pTbl->ErrorTicks/90;			      //Convert to milliseconds.
        }
        pTbl->u.s.PktUncertain= PktUncertain;
        pTbl->u.s.ErrorValid= 1;                //Error calculation is now valid
        pTbl->PtsMatches++;                     //Increment the number of successful matches.
//        if(pTbl->Error && !PktUncertain)
        DbgMsg(DbgFltr,"Sync:%s%c(%u) ",&pTbl->Name,(PktUncertain ? '?':' '),dwPicType);
        DbgDispPTS(DbgFltr,DecodeTime, " DTS=","");
        DbgDispPTS(DbgFltr,CapturedSCR," SCR=","");
        DbgMsg(DbgFltr," Error=%ums %s.\r\n",pTbl->Error,Direction);
        if(pEntry)
          DbgMsg(DBG_SYNCCHK,"Sync %s: Start=%08X, End=%08X, RdPtr=%08X.\r\n",pTbl->Name,pEntry->PktStart,pEntry->PktEnd,RdPtr);
       if(pTbl->Error > 100)
          DbgMsgCmd(DBGCMD_PERFSET,pTbl->hPerfStat,pTbl->Error);
      }
      if(DecodeTime) {
        if(DecodeTime<pTbl->CurPts)
          pTbl->CurPtsOverFlow++;
        pTbl->CurPts=DecodeTime;
        pTbl->CurPtsTime=pTbl->pSync->CurTime;
      }
    }
//    if(pEntry)
//      DbgMsg(DbgFltr,"Sync: %s Start=%08X, End=%08X, RdPtr=%08X.\r\n",pTbl->Name,pEntry->PktStart,pEntry->PktEnd,RdPtr);
//    pEntry->SyncEvents++;					//Increment number of sync events found in this packet.
  }
  return(Err);
}

LOCAL DWORD GetMinPts(TABLE *pTbl) {
  DWORD ChkEntry;
  ENTRY *pEntry;
  DWORD dwPts=0;
  if(pTbl->PtsMin)
    dwPts=pTbl->PtsMin;
  else {
    for(ChkEntry=pTbl->Tail;!dwPts && ChkEntry!=pTbl->Head;) {
      pEntry= &pTbl->pEntries[ChkEntry];
      dwPts=pEntry->Pts;
      if(++ChkEntry>=pTbl->MaxEntry)
        ChkEntry= 0;
    }
  }
  return dwPts;
}

//Find an entry which includes the RdPtr.
//pPktUncertain is used to indicate whether the read pointer was found to
//be within the region of a packet where, due to the data read FIFO, the
//precise position of the last byte read is uncertain and may be either in
//the current packet or the previous packet. If PktUncertain is set, the
//drift calculation may be off by one packet (typically one frame, 42ms).
LOCAL ENTRY *SyncFindEntry(TABLE *pTbl, DWORD RdPtr, BYTE *pPktUncertain) {
  long Err= 0;
  DWORD EntryFound=0,ChkEntry;
  ENTRY *pEntry;
  if(pPktUncertain)
    *pPktUncertain= 0;
  if(pTbl->u.s.Enable && pTbl->Tail!=pTbl->Head) {			//If sync checks are enabled and there are entries written...
//    if(pTbl->StrmType==STRM_AUD)
//      DbgMsg(DBG_SYNCCHK,"Sync %s: Tail=%u, Head=%u, RdPtr=%08X.\r\n",pTbl->Name,pTbl->Tail,pTbl->Head,RdPtr);
    for(ChkEntry=pTbl->Tail;!EntryFound && ChkEntry!=pTbl->Head;) {
      pEntry= &pTbl->pEntries[ChkEntry];
      //DbgMsg(DBG_SYNCCHK,"Sync %s %u: %08X-%08X.\r\n",pTbl->Name,ChkEntry,pEntry->PktStart,pEntry->PktEnd);
      if(pEntry->PktStart<pEntry->PktEnd) {			//Normal case. Watch out for PktStart==PktEnd!
        if(RdPtr>=pEntry->PktStart && RdPtr<=pEntry->PktEnd)	//Target must be in the middle
          EntryFound= 1;
      } else if(pEntry->PktStart>pEntry->PktEnd) {		//Buffer wrapped.
        if(RdPtr>=pEntry->PktStart || RdPtr<=pEntry->PktEnd)	//Target must be at either end.
          EntryFound= 1;
      }
      if(!EntryFound && ++ChkEntry>=pTbl->MaxEntry)
        ChkEntry= 0;
    }
  }
  if(EntryFound) {
    // Release all table entries upto and including EntryFound
    pTbl->Tail=ChkEntry;
    if(++pTbl->Tail>=pTbl->MaxEntry)
      pTbl->Tail= 0;
#if(0)
    while(pTbl->Tail!=ChkEntry) {				//Free all entries since last match.
//      pTbl->EntryCnt--;
      if(++pTbl->Tail>=pTbl->MaxEntry)
        pTbl->Tail= 0;
    }
#endif
    if(pPktUncertain && (pEntry->PktEnd<pTbl->ReadFifoSize || RdPtr<pEntry->PktStart+pTbl->ReadFifoSize)) {
//      DbgMsg(DBG_SYNCCHK,"Sync %s: Uncertainty detected: Start=%08X, End=%08X, RdPtr=%08X.\r\n",pTbl->Name,pEntry->PktStart,pEntry->PktEnd,RdPtr);
      *pPktUncertain= 1;
    }
  } else {
    DbgMsg(DBG_SYNCDROP,"Sync: %s %08X not found.\r\n",pTbl->Name,RdPtr);
    pEntry= 0;
  }
  return(pEntry);
}

//Convert a PTS to DTS. Only used for video streams.
//This needs to be called on every frame.
LOCAL DWORD SyncPtsToDts(TABLE *pTbl,DWORD Pts,DWORD dwPicType) {
  DWORD Dts;
  Dts= Pts;						    //Assume DTS=PTS
  if(pTbl->pSync->F.u.s.DtsReorder) {			//If DTS reordering not enabled, DTS=PTS
    if(dwPicType==1 || dwPicType==2) {    //I and P frames are decoded before later B frame are decoded & displayed.
      Dts= pTbl->DtsLastIP;			//DTS= previously decoded I or P frame's PTS.
      pTbl->DtsLastIP= Pts;			//Store this frame's PTS for later use.
    } else {					//B frames are decoded and immediately displayed.
      Dts= Pts;					//B Frames: Dts=Pts
    }
  }
  return(Dts);
}

LOCAL long SyncSetScrCapture(SYNC *pSync) {
  long Err= 0;
  if(!pSync->F.u.s.ScrCapPending) {	//Check to make sure SCR capture is not already in progress.
#if AVSYNC_TEST_VID
    pSync->StrmCapturing=STRM_VID;
#elif AVSYNC_TEST_AUD
    pSync->StrmCapturing=STRM_AUD;
#else
    if(pSync->pAud->u.s.Active) {
      if(!pSync->StrmCapturing) {
        if(pSync->pVid->u.s.IsMaster) {
          pSync->StrmCapturing= STRM_VID;
          pSync->pVid->PtsMatches= 0;
        } else {
          pSync->StrmCapturing= STRM_AUD;
          pSync->pAud->PtsMatches= 0;
        }
      } else if(pSync->StrmCapturing==STRM_VID) {		//Currently capturing video...
        if(pSync->pVid->u.s.IsMaster && pSync->pVid->PtsMatches>=1) {
          pSync->StrmCapturing= STRM_AUD;
          pSync->pVid->PtsMatches= 0;
        } else if(pSync->pAud->u.s.OutOfSync || pSync->pVid->PtsMatches>=1) { //Capture 1 picture start code (with matching PTS) before switching to audio.
          pSync->StrmCapturing= STRM_AUD;
          pSync->pVid->PtsMatches= 0;
        }
      } else if(pSync->StrmCapturing==STRM_AUD) {		//Currently capturing audio?
        if(pSync->pVid->u.s.IsMaster && pSync->pAud->PtsMatches>=1) {
          pSync->StrmCapturing= STRM_VID;
          pSync->pAud->PtsMatches= 0;
        } else if(!pSync->pAud->u.s.OutOfSync && pSync->pAud->PtsMatches>=2) { //Capture 2 audio syncs in a row before switching to video.
          pSync->StrmCapturing= STRM_VID;
          pSync->pAud->PtsMatches= 0;
        }
      }
    } else
      pSync->StrmCapturing= STRM_VID;  // if audio not active then always capture video
#endif
    if(pSync->StrmCapturing==STRM_AUD)
      dvHwMainWriteLsiReg(pSync->pHw,CAPCOMP_MODE_REG,CAPTURE_AUD);
    else
      dvHwMainWriteLsiReg(pSync->pHw,CAPCOMP_MODE_REG,CAPTURE_VID);
    pSync->F.u.s.ScrCapPending= 1;
  }
  return(Err);
}

#if(0)
/*************************************************************************/
/** Get the current read or write video or audio buffer pointer.	**/
/** -	Having a small function allows me to do funky wrap stuff.	**/
/** -	I also record the read and write pointers to the DbgMsg		**/
/**	profiler log for analysis.					**/
/** -   Pointers are normalized to start at 0.				**/
/*************************************************************************/
LOCAL void SyncUpdateBufPtr(SYNC *pSync,int nPtr) {
  BUFINFO pBuf;
  DWORD BufPtr;

  pBuf=&pSync->pHw->BufInfo[nPtr];  
  BufPtr= dvHwMainReadLsiReg24(pSync->pHw,pBuf->wReg) & 0x7FFFF;
  if(BufPtr>=pBuf->dwChanStartAddr)
    pBuf->CurRdPtr= BufPtr-pBuf->dwChanStartAddr;

  BufPtr= dvHwMainReadLsiReg24(pSync->pHw,VID_CHAN_WRITE_ADDR_REG) & 0x7FFFF;
  if(BufPtr>=pSync->pHw->dwVidChanStartAddr)
    pSync->pVid->CurWrPtr= BufPtr-pSync->pHw->dwVidChanStartAddr;
  
  BufPtr= dvHwMainReadLsiReg24(pSync->pHw,AUD_SYNC_CODE_READ_ADDR_REG) & 0x7FFFF;
  if(BufPtr>=pSync->pHw->dwAudChanStartAddr)
    pSync->pAud->CurRdPtr= BufPtr-pSync->pHw->dwAudChanStartAddr;
  
  BufPtr= dvHwMainReadLsiReg24(pSync->pHw,AUD_CHAN_WRITE_ADDR_REG) & 0x7FFFF;
  if(BufPtr>=pSync->pHw->dwAudChanStartAddr)
    pSync->pAud->CurWrPtr= BufPtr-pSync->pHw->dwAudChanStartAddr;
  
  if(pSync->F.PlotChannels) {
    DbgMsgCmd(DBGCMD_PROF_STAMP,DBGPRF_VIDRD,pSync->pVid->CurRdPtr);
    DbgMsgCmd(DBGCMD_PROF_STAMP,DBGPRF_VIDWR,pSync->pVid->CurWrPtr);
    DbgMsgCmd(DBGCMD_PROF_STAMP,DBGPRF_AUDRD,pSync->pAud->CurRdPtr);
    DbgMsgCmd(DBGCMD_PROF_STAMP,DBGPRF_AUDWR,pSync->pAud->CurWrPtr);
  }
}
#endif

LOCAL void SyncUpdateScr(SYNC* pSync,DWORD dwScr) {
  PHW pHw=pSync->pHw;
  RegScrPause(pHw,FLAG_SCR_PAUSE);    //Stop the SCR from incrementing
  dvHwMainWriteScrReg(pHw,dwScr);			//Write new SCR
  RegScrPause(pHw,FLAG_SCR_UNPAUSE);  //Restart SCR.
}



/*************************************************************************/
/** Error evaluation.							**/
/*************************************************************************/
LOCAL DWORD SyncApplyTbl(TABLE *pTbl);
LOCAL DWORD SyncApplyVid(TABLE *pVid);
LOCAL DWORD SyncApplyAud(TABLE *pAud);
LOCAL DWORD SyncApplySubpicture(SYNC *pSync);

LOCAL long SyncShowError(SYNC *pSync) {
  long Err= 0;
  if(!pSync->F.u.s.Started) {
    DbgMsg(DBG_SYNCERR,"AV Sync NOT Started \r\n");
  }
  return(Err);
}

LOCAL long SyncApplyAdjustments(SYNC *pSync) {
  long Err= 0;
  DWORD AdjApplied= 0;
  DWORD OktoApply= 1;
  DWORD VidState= Z_STATE_STOP;
  DWORD AudState= Z_STATE_STOP;
  VideoCmd(pSync->pHw,VIDCMD_GET,VIDITEM_STATE,(DWORD)&VidState);
  AudioCmd(pSync->pHw,AUDCMD_GET,AUDITEM_STATE,(DWORD)&AudState);
	if(pSync->CurTime>pSync->AdjEnableTime) {                     //Wait a minimum period between adjustments.
	  if((AudState==Z_STATE_PLAY) && pSync->pAud->u.s.EnableAdj)	//Do audio first since it modifies the SCR.
	    AdjApplied= SyncApplyTbl(pSync->pAud);
	  if((VidState==Z_STATE_PLAY) && pSync->pVid->u.s.EnableAdj)	//If audio adjusted, wait until next period to adjust video.
	    AdjApplied= SyncApplyTbl(pSync->pVid);
	}
  return(Err);
}

//Apply drift corrections to audio or video.
LOCAL DWORD SyncApplyTbl(TABLE *pTbl) {
  DWORD AdjApplied= 0;
  SYNC *pSync= pTbl->pSync;
  DWORD MinErrorPersistence=pTbl->u.s.OutOfSync?pTbl->OutSyncErrorPersistence:pTbl->InSyncErrorPersistence;
  if(pTbl->u.s.ErrorValid) {
    if(pTbl->Error<=pTbl->Tolerance) {			//Is stream within tolerance?
      if(pTbl->ErrorStartTime)
      	DbgMsg(DBG_SYNCWATCH,"Sync: Spurious %s drift ignored.\r\n",pTbl->Name);
      pTbl->ErrorStartTime= 0;				//Reset drift persistence timer.
      pTbl->Tolerance= pTbl->ToleranceGross;		//Reset tolerance to the initial gross limits.
      if(pTbl->u.s.OutOfSync) {
        if(pTbl->InSyncCount)
          pTbl->InSyncCount--;
        else
	        pTbl->u.s.OutOfSync= 0;				//We are now officially in sync!
	      switch(pTbl->StrmType) {
	        case STRM_VID:  VideoCmd(pTbl->pSync->pHw,VIDCMD_CORRECT,0,0); break;
	        case STRM_AUD:  break;
	      }
	      DbgMsg(DBG_SYNCADJ,"Sync: %s now in sync.\r\n",pTbl->Name);
      }
    } else {						//Stream is out of sync...
      if(!pTbl->ErrorStartTime) {			//Wait until drift has persisted before correcting.
	      pTbl->ErrorStartTime= pSync->CurTime;
	      DbgMsg(DBG_SYNCWATCH,"Sync: Initiating %s drift persistence check. (%ums)\r\n",pTbl->Name,pTbl->Error);
      }
      if(pSync->CurTime > pTbl->ErrorStartTime+MinErrorPersistence) {
	      if(!pTbl->LastAdjustment || pTbl->LastAdjustment+pSync->MinAdjustPeriod<pSync->CurTime) {
	        pTbl->LastAdjustment= pSync->CurTime;
	        switch(pTbl->StrmType) {
	          case STRM_VID:  AdjApplied= SyncApplyVid(pTbl); break;
	          case STRM_AUD:  AdjApplied= SyncApplyAud(pTbl); break;
	        }
	        if(AdjApplied) {
	          pTbl->LastAdjustment= pSync->CurTime;
	          pTbl->u.s.ErrorValid= 0;
	          pTbl->ErrorStartTime= 0;
	          pTbl->Tolerance= pTbl->ToleranceFine;	//Once correction has begun, use fine tolerance to get to 0.
	        }
	      }
      }
    }
  }
  return(AdjApplied);
}

LOCAL void SyncApplyToScr(TABLE *pTbl) {
  long lScrAdj;
  char *Direction;
  if(pTbl->u.s.ErrorAhead) {
    lScrAdj=pTbl->ErrorTicks;			//Bump SCR up to current audio PTS.
    Direction= "ahead";
  } else {
    lScrAdj= 0-pTbl->ErrorTicks;			//Back SCR off to current audio PTS.
    Direction= "behind";
  }
  dvHwMainAdjustDecoderTime(pTbl->pSync->pHw,lScrAdj);		//Write new SCR
  DbgMsg(DBG_SYNCADJ,"Sync: %s %ums %s, adjusting SCR by %d counts.\r\n",pTbl->Name,pTbl->Error,Direction,lScrAdj);
}


LOCAL DWORD SyncApplyVid(TABLE *pVid) {
  PHW pHw= pVid->pSync->pHw;
  if(pVid->u.s.PktUncertain && pVid->Error<pVid->PktUncertaintyTime) {
    DbgMsg(DBG_SYNCWATCH,"Sync: Video error of %ums ignored due to FIFO uncertainty.\r\n",pVid->Error);
  } else {
    pVid->u.s.OutOfSync= 1;
    if(pVid->pSync->pAud->u.s.OutOfSync) {
      DbgMsg(DBG_SYNCADJ,"Sync: Audio must be in sync before adjusting video.\r\n");
    } else if(pVid->pSync->MaxTolerance && pVid->Error>=pVid->pSync->MaxTolerance) {
      if(!pVid->u.s.MaxToleranceMsg) {
        DbgMsg(DBG_SYNCERR,"Sync: %s is out of tolerance bounds, %ums.\r\n",pVid->Name,pVid->Error);
        pVid->u.s.MaxToleranceMsg= 1;
      }
    } else {
      pVid->u.s.MaxToleranceMsg= 0;
      if((pVid->Error<2000) || (pVid->nResetCount>2)) {
        pVid->nResetCount=0;
        if(pVid->u.s.IsMaster)
          SyncApplyToScr(pVid);
        else
          VideoCmd(pHw,VIDCMD_CORRECT,pVid->u.s.ErrorAhead,pVid->Error);
      } else {
        // if error is greater than 2 sec then reset
        SyncReset(pVid->pSync,0,STRM_VID|STRM_AUD);
        pVid->nResetCount++;
      }
    }
  }
  return(1);
}

LOCAL DWORD SyncApplyAud(TABLE *pAud) {
  PHW pHw= pAud->pSync->pHw;
  if(pAud->u.s.PktUncertain && pAud->Error<pAud->PktUncertaintyTime) {
    DbgMsg(DBG_SYNCWATCH,"Sync: Audio drift of %ums ignored due to FIFO uncertainty.\r\n",pAud->Error);
  } else {
    pAud->u.s.OutOfSync= 1;
    if((pAud->Error<2000) || (pAud->nResetCount>2)) {
      pAud->nResetCount=0;
      if(pAud->u.s.IsMaster)
        SyncApplyToScr(pAud);
        else
          AudioCmd(pHw,AUDCMD_CORRECT,pAud->u.s.ErrorAhead,pAud->Error);
      } else {
        // if error is greater than 2 sec then reset
        SyncReset(pAud->pSync,0,STRM_VID|STRM_AUD);
        pAud->nResetCount++;
      }
  }
  return(1);
}

/*************************************************************************/
/** Debug dumps								**/
/*************************************************************************/
LOCAL void SyncDumpTable(TABLE *pTbl);
LOCAL void SyncDumpTableEntries(TABLE *pTbl);

//Check to see if it is time for a periodic dump.
//Note that dumps can also be triggered by setting flags in pSync->DumpPending.Flags.
LOCAL void SyncDumpChkTime(SYNC *pSync) {
  if(pSync->DumpPending.dwTime>pSync->CurTime) {
    pSync->DumpPending.dwTime= pSync->CurTime+pSync->DumpEnable.dwTime;
    pSync->DumpPending.u.Flags= pSync->DumpEnable.u.Flags;
  }
}

LOCAL void SyncDump(SYNC *pSync) {
  if(pSync->DumpPending.u.s.Sync) {
    DbgMsg(DBG_SYNCTBL,"====>Sync=======\r\n");
    DbgMsg(DBG_SYNCTBL,"StrmCapturing= %s.\r\n",Ident(ID_HW_STRMTYPE,pSync->StrmCapturing));
    DbgMsg(DBG_SYNCTBL,"CurTime= %12u, CurSCR= %12u.\r\n",pSync->CurTime,pSync->CurSCR);
    DbgMsg(DBG_SYNCTBL,"  Average tick interval: %ums.\r\n",pSync->CurAvgTick);
    if(pSync->DumpPending.u.s.Stats) {
      pSync->DumpPending.u.s.Stats= 0;
    }
    if(pSync->DumpPending.u.s.Table) {
      if(pSync->pVid->u.s.Enable) {
	SyncDumpTable(pSync->pVid);
	if(pSync->DumpPending.u.s.Entries)
	  SyncDumpTableEntries(pSync->pVid);
      }
      if(pSync->pAud->u.s.Enable) {
	SyncDumpTable(pSync->pAud);
	if(pSync->DumpPending.u.s.Entries)
	  SyncDumpTableEntries(pSync->pAud);
      }
      pSync->DumpPending.u.s.Table= 0;
    }
    DbgMsg(DBG_SYNCTBL,"====<Sync=======\r\n");
    pSync->DumpPending.u.s.Sync= 0;
  }
}

LOCAL void SyncDumpTable(TABLE *pTbl) {
  DbgMsg(DBG_SYNCTBL,"%s table: %u entries.\r\n",pTbl->Name,pTbl->MaxEntry);
  DbgMsg(DBG_SYNCTBL,"Error= %d, DropCnt= %u.\r\n",pTbl->Error,pTbl->DropCnt);
//  DbgMsg(DBG_SYNCTBL,"H=%u T=%u EntryCnt=%u HighWater=%u.\r\n",pTbl->Head,pTbl->Tail,pTbl->EntryCnt,pTbl->HighWater);
//  DbgMsg(DBG_SYNCTBL,"Rd=%08X Wr=%08X.\r\n",pTbl->CurRdPtr,pTbl->CurWrPtr);
}

LOCAL void SyncDumpTableEntries(TABLE *pTbl) {
  DWORD n1;
  ENTRY *pEntry;
  for(n1=pTbl->Tail;n1!=pTbl->Head;) {
    pEntry= &pTbl->pEntries[n1];
    DbgMsg(DBG_SYNCTBL,"%-4u. %08X-%08X %12u %12u\r\n",n1,pEntry->PktStart,pEntry->PktEnd,pEntry->Pts);
    if(++n1>=pTbl->MaxEntry)
      n1= 0;
  }
}

/*************************************************************************/
/** Scan packet data for PTS tags.					**/
/** --	Returns the PTS value found, 0 if none.				**/
/**	*pPtsLoc will be the offset withhin pData where PTS was found.	**/
/*************************************************************************/
LOCAL DWORD PtsFindVid(BYTE *pData, DWORD nBytes, DWORD *pSCR) {
  DWORD dwPTS=0,dwScr=0;
  BYTE bGetPts= FALSE;
  BYTE bGetScr= FALSE;
  WORD wHeaderLen;
  if(nBytes>=4 && pData[3]==0xBA) {	        		//If pack start code
    pData+= 4; 						//Skip packet start code prefix
    nBytes-= 4;
    if((*pData&0xC0)==0x40 && nBytes>=5) {			//If Mpeg2
      dwScr = ((DWORD)(pData[0]&0x38))<<27;			//Bits 31 & 30
      dwScr|= ((DWORD)(pData[0]&0x03))<<28;			//Bits 29 & 28
      dwScr|= ((DWORD) pData[1])<<20;			        //Bits 27 to 20
      dwScr|= ((DWORD)(pData[2]&0xF8))<<12;			//Bits 19 to 15
      dwScr|= ((DWORD)(pData[2]&0x03))<<13;			//Bits 14 & 13
      dwScr|= ((DWORD) pData[3])<<5;			        //Bits 12 to 5
      dwScr|= ((DWORD)(pData[4]&0xF8))>>3;			//Bits 4 to 2
      pData+= 5;
      nBytes-= 5;
      if(nBytes>=4) {
        pData+= 4;
        nBytes-= 4;
      }
      wHeaderLen= (*pData&0x07);
      if(nBytes>= (DWORD)(1+wHeaderLen)) {
        pData+= 1 + wHeaderLen;
        nBytes-= 1 + wHeaderLen;
      }
    } else if(nBytes>=5) {
      dwScr = ((DWORD)(pData[0]&0x0E))<<29;
      dwScr|= ((DWORD) pData[1])<<22;
      dwScr|= ((DWORD)(pData[2]&0xFE))<<14;
      dwScr|= ((DWORD) pData[3])<<7;
      dwScr|= (DWORD)( pData[4]>>1);
      pData+= 5;
      nBytes-= 5;
      if(nBytes>=3) {
        pData+= 3;
        nBytes-= 3;
      }
    }
    if(dwScr)
      bGetScr = TRUE;
  }
  if(nBytes>=4 && pData[3]==0xB3) {
    pData+= 4; 						//Skip packet start code prefix
    nBytes-= 4;
    if(nBytes>=2) {
      wHeaderLen= pData[0] + pData[1];
      pData+= 2;
      nBytes-= 2;
      if(nBytes>=wHeaderLen) {
        pData+= wHeaderLen;
        nBytes-= wHeaderLen;
      }
    }
  }
  if(nBytes>=3) {
    pData += 3;			 			//Skip packet start code prefix
    nBytes-= 3;
  }
  if(nBytes>=3) {
    pData+= 3;					        //Skip Packet Length
    nBytes-= 3;
    if(nBytes && (*pData&0xC0)==0x80) {			//If MPEG2
      pData++;
      nBytes--;
      if(*pData&0xC0)					//If PTS_DTS flag set
        bGetPts = TRUE;
      if(nBytes>=2) {
        pData+= 2;
        nBytes-= 2;
      }
    } else {
      while(nBytes && *pData==0xFF) {			//Skip stuffing bytes
        pData++;
        nBytes--;
      }
      if(nBytes>=2 && (*pData & 0xC0)==0x40) {		//Skip STD bytes
        pData += 2;
        nBytes-= 2;
      }
      if(nBytes && (*pData & 0x30))
        bGetPts = TRUE;
    }
    if(nBytes>=5 && bGetPts) {
      dwPTS= ((DWORD)(pData[0]&0x0E))<<29;
      dwPTS|= ((DWORD) pData[1])<<22;
      dwPTS|= ((DWORD)(pData[2]&0xFE))<<14;
      dwPTS|= ((DWORD) pData[3])<<7;
      dwPTS|= (DWORD)( pData[4]>>1);
      if(dwPTS==0)
        dwPTS=1;                          // if PTS in bitstream is 0 then return 1 since 0 means not present
      pData+= 5;
      nBytes-= 5;
    }
  }
  if(pSCR)
    *pSCR= dwScr;
  return(dwPTS);
}

LOCAL DWORD PtsFindAud(BYTE *pData, DWORD nBytes, DWORD *pPtsLoc) {
  return(0);
}

/**EOF: SYNC.C**/
