//#include <StrmIni.h>
//#include <mmsystem.h>
//#include <ksmedia.h>
#include "system.h"
#include "DbgMsg.h"
#include "DbgFltr.h"
#include "Settings.h"

#include "Lsi64020.h"
#include "MidasApi.h"
#include "dvHwMain.h"
#include "Global.h"
#include "Sync.h"
#include "margierr.h"
#include "Idents.h"
#include "dvHwVid.h"
#include "dvHwAud.h"
#include "identcls.h"
#include "IniNames.h"

//#pragma message("dvhwaud.c: Optimizer disabled.")
//#pragma optimize("",off)

// define Actual Hardware States
enum {
  C_STATE_NOT_READY =0,
  C_STATE_READY,
  C_STATE_RDY_WAITDATA,
  C_STATE_GET_STARTTIME,
  C_STATE_PREFILL,
  C_STATE_STALL_TIME,
  C_STATE_PLAY_SYNC,
  C_STATE_PLAY,
  C_STATE_PAUSE,
  C_STATE_DISCARD,
  C_STATE_FLUSH,
  C_STATE_FLUSH_COMPLETE,
  C_STATE_UNK
};

struct IdentEntry_s IdentsAud[]= {
  { ID_AUD_STATE,0 },
    { C_STATE_NOT_READY,      "NotReady" },
    { C_STATE_READY,          "Ready" },
    { C_STATE_RDY_WAITDATA,   "RdyWaitData"},
    { C_STATE_GET_STARTTIME,  "GetStartTime"},
    { C_STATE_PREFILL,        "Prefill"},
    { C_STATE_STALL_TIME,     "StallTime"},
    { C_STATE_PLAY_SYNC,      "PlaySync"},
    { C_STATE_PLAY,           "Play"},
    { C_STATE_PAUSE,          "Pause"},
    { C_STATE_DISCARD,        "Discard"},
    { C_STATE_FLUSH,          "Flush"},
    { C_STATE_FLUSH_COMPLETE, "FlushComplete"},
    { C_STATE_UNK,            "Unknown" },
  { 0,0 }
};

#define SIGNATURE 0xCD010004
typedef struct Audio_s {
  DWORD Signature;
  PHW pHw;
  DWORD StrmState;
  DWORD CtrlState;
  DWORD NextState;
  DWORD StateTimeoutTime;
  DWORD CurTime;

  QUEUE DataQ;
  PMDS_DATA_BLOCK pFlushMdb;
  DWORD EnableDataTransfer;
  DWORD fDiscardData;
  DWORD fDataXfrPending;
  DWORD dwTotalBytes;
  DWORD BeginFlush;
  DWORD EndFlush;
  DWORD fInPumpData;
  DWORD ResetStream;
  DWORD ChannelFull;
  DWORD ChannelTimeout;
  DWORD CurScr;
  DWORD StartTime;

  DWORD dwStep;

//  DWORD CurState;
  DWORD Stream;
  DWORD SrcType;
  DWORD DecodeMode;
  DWORD SpdifType;
  DWORD SpdifMode;

  DWORD SampleRate;
  DWORD OutputMode;
  DWORD AC3MixMode;         //AC3 Mix Mode
  DWORD KaraokeMode;        //Current Karaoke Mode
  DWORD hPerfChnl;          //Audio channel depth
  DWORD dwPcmScale;
  DWORD dwNumChannels;
  
  int   nPauseCount;
  DWORD SyncAhead;
  DWORD SyncError;
  
  DWORD MuteTime;			      //If NZ, mute until this PC time.
  int   fNewFormat;
  int   fLocalMute;
  int   fUpdateOutput;      //set true if volume or mute should be updated
  int   fCurMute;
  int   fCurSpdifEnable;

  DWORD dwCurVol;

  DWORD TimeZero;
//  DWORD ChnlSpacePad;	      //Extra space required in channel before transferring data.
  DWORD DiscardUntilPts;    //Discard all Packets earlier than this 
  union {
    struct {
      DWORD fSpdifEnable:1;
      DWORD fFmtInUse:1;    //Set when Dolby or Mpeg Formatter is in use
    }s;
    DWORD Flags;
  }u;
} AUD;

// This is a hack used to allow SCR to be reset
// in the NEC build.  For WDM/AM we have to let the
// SCR continue to run even if the stream is reset.
// Set TRUE for NEC and FALSE for WDM/AM
int   gfResetTime=0;

static AUD *AudPtr(DWORD hAud);
static long AudCmdInvalid(AUD *pAud, DWORD Cmd);
static long AudInit(PHW pHw);
static long AudFini(AUD *pAud);
static long AudResetStateMachine(AUD*);
static long AudStreamReset(AUD*);
static long AudTimer(AUD *pAud);
static long AudSelectStream(AUD *pAud, DWORD Stream);
static long AudStart(AUD *pAud, DWORD Start);
static long AudSetClock(AUD *pAud);
static long AudRate(AUD *pAud, MDS_PLAYRATE* pPlayRate);
static long AudUpdateVolume(AUD *pAud);
static long AudUpdateMute(AUD *pAud);
static void AudTimedUnmute(AUD *pAud,DWORD dwMs);
static void AudLocalMute(AUD *pAud);

static long AudGet(AUD *pAud, DWORD Item, DWORD *pValue);
static long AudSet(AUD *pAud, DWORD Item, DWORD Value);
static DWORD AudSendData(AUD *pAud,PMDS_DATA_PACKET pMdp);

LOCAL  long AudStreamState(AUD*, int StrmState);
LOCAL  long AudStreamStep(AUD* pAud,int nStep);
LOCAL  long AudCorrectSync(AUD*,DWORD dwAhead,DWORD dwError);


static long AudProcessNotReady(AUD*);
static long AudProcessReady(AUD*);
static long AudProcessRdyWaitData(AUD*);
static long AudProcessGetStartTime(AUD*);
static long AudProcessPrefill(AUD*);
static long AudProcessStallTime(AUD*);
static long AudProcessPlaySync(AUD*);
static long AudProcessPlay(AUD*);
static long AudProcessPause(AUD*);
static long AudProcessDiscard(AUD*);
static long AudProcessFlush(AUD*);
static long AudProcessFlushComplete(AUD*);

static long AudStateEnterFlush(AUD*);
static long AudStateEnterReady(AUD*);

static long AudSetStateTimeout(AUD*, DWORD dwMilliSecs);
static long IsStateTimeout(AUD*);
static long AudStreamFlush(AUD*,int End,PMDS_DATA_BLOCK pMdp);

static void AudUpdateStream(AUD*);
static long AudUpdateSampleRate(AUD*);
static void AudUpdateMode(AUD*);
static void AudUpdateFormat(AUD*);

static long AudStateInvalid(AUD*);

static AUD StaticAud;
//static struct DbgMsgPerf_s PerfStatAudChnl= { "Audio Channel","Audio Channel","Audio Channel",0 };

/*************************************************************************/
/** Command handler.							**/
/*************************************************************************/
//Convert a handle to an audio object to a pointer to an AUD.
static AUD *AudPtr(DWORD hAud) {
  if(hAud && *(DWORD*)hAud==SIGNATURE)
    return((AUD*)hAud);
  return(0);
}

long AudioCmd(PHW pHw, DWORD Cmd, DWORD Arg1, DWORD Arg2) {
  AUD *pAud;
  long Err;
  if(LOWORD(Cmd)==AUDCMD_INIT) {
    Err= AudInit(pHw);
  } else if(!(pAud= AudPtr(pHw->hAudio))) {
    Err= MARGI_ERR_BADHANDLE;
  } else {
    if(LOWORD(Cmd)==AUDCMD_TIMER) {
      Err= AudTimer(pAud);
//    } else if(LOWORD(Cmd)==AUDCMD_DATA) {
//      Err= AudData(pAud,(void*)Arg1);
    } else {
      switch(LOWORD(Cmd)) {
        case AUDCMD_FINI:       Err= AudFini(pAud); break;
        case AUDCMD_RESET:      Err= AudStreamReset(pAud); break;
        case AUDCMD_STOP:       Err= AudStreamState(pAud,S_STATE_STOP); break;
        case AUDCMD_PAUSE:	    Err= AudStreamState(pAud,S_STATE_PAUSE); break;
        case AUDCMD_RUN:	      Err= AudStreamState(pAud,S_STATE_RUN); break;
        case AUDCMD_STEP:	      Err= AudStreamStep(pAud,(int)Arg1); break;
        case AUDCMD_BEGINFLUSH:	Err= AudStreamFlush(pAud,0,(PMDS_DATA_BLOCK)Arg1); break;
        case AUDCMD_ENDFLUSH:	  Err= AudStreamFlush(pAud,1,0); break;
        case AUDCMD_CORRECT:    Err= AudCorrectSync(pAud,Arg1,Arg2); break;

        case AUDCMD_SELECT:	    Err= AudSelectStream(pAud,Arg1);      break;
        case AUDCMD_RATE:	      Err= AudRate(pAud,(MDS_PLAYRATE*)Arg1); break;
        case AUDCMD_GET:	      Err= AudGet(pAud,Arg1,(DWORD*)Arg2);  break;
        case AUDCMD_SET:	      Err= AudSet(pAud,Arg1,Arg2);          break;
        default:		Err= AudCmdInvalid(pAud,Cmd);
      }
    }
  }
  return(Err);
}

long AudioQueueData(PHW pHw,PMDS_DATA_BLOCK pMdb) {
  long Err=0;
  AUD *pAud;
  if(pAud=AudPtr(pHw->hAudio)) {
    pMdb->dwCurPacket=0;
    qEnqueue(&pAud->DataQ,pMdb);
  } else
    Err= MARGI_ERR_BADHANDLE;
  return 0;
}

long AudioCancelData(PHW pHw,PMDS_DATA_BLOCK pMdb) {
  long Err=0;
  AUD *pAud;
  if(pAud=AudPtr(pHw->hAudio)) {
    if(pMdb) {
      pMdb->dwCurPacket=0;
      pMdb->dwStatus=MDS_Cancel;
      if(qFindAndDequeue(&pAud->DataQ,pMdb)) {
        PMDB_CALLBACK pfnCallback = (PMDB_CALLBACK)pMdb->pfnCallback;
         pfnCallback(pMdb);
      }
    } else {
      qPurgeQueue(&pAud->DataQ,MDS_Cancel);
    }
  } else
    Err= MARGI_ERR_BADHANDLE;
  return 0;
}

DWORD AudioPumpData(PHW pHw) {
  DWORD dwBytesTransfered=0;
  AUD* pAud;
  if(pAud=AudPtr(pHw->hAudio)) {
    PQUEUE pQ;
    PMDS_DATA_BLOCK  pMdb;
    PMDS_DATA_PACKET pMdp;
    pQ=&pAud->DataQ;
    
    pAud->fInPumpData=1;
    if(pAud->pFlushMdb) {
      PMDB_CALLBACK pfnCallback=(PMDB_CALLBACK)pAud->pFlushMdb->pfnCallback;
      qPurgeQueue(&pAud->DataQ,MDS_Cancel);
      pAud->pFlushMdb->dwStatus=MDS_Success;
      pfnCallback(pAud->pFlushMdb);
      pAud->pFlushMdb=0;
    }
    pMdb=qPeekqueue(pQ);
    if(pMdb && !pAud->BeginFlush) {
      while (dwBytesTransfered < 8192) {
        DWORD dwBytes;
        pMdp=&pMdb->pPackets[pMdb->dwCurPacket];
        dwBytes=AudSendData(pAud,pMdp);
        dwBytesTransfered+=dwBytes;
        if(dwBytes!=pMdp->DataUsed)
          break;
        if(++pMdb->dwCurPacket>=pMdb->dwTotalPackets) {
          PMDB_CALLBACK pfnCallback = (PMDB_CALLBACK)pMdb->pfnCallback;
          pMdb->dwStatus=MDS_Success;
          qDequeue(pQ);
          pfnCallback(pMdb);
          break;
        }
      }
    }
    pAud->fInPumpData=0;
  }
  return dwBytesTransfered;
}

static long AudCmdInvalid(AUD *pAud, DWORD Cmd) {
  DbgMsg(DBG_ERR,"Audio: Bad command %u (%X).\r\n",LOWORD(Cmd),Cmd);
  return(MARGI_ERR_BADCMD);
}

/*************************************************************************/
/** Init/Reset/Fini							**/
/*************************************************************************/
static long AudInit(PHW pHw) {
  long Err= 0;
  AUD *pAud;
  pAud= &StaticAud;					//Eventually replace with dynamic memory allocation.
  if(!pAud) {
    DbgMsg(DBG_ERR,"Audio: Unable to allocate new AUD object!\r\n");
    Err= MARGI_ERR_NOMEM;
  } else {
    memset(pAud,0,sizeof(AUD));
    pAud->Signature= SIGNATURE;
    pAud->pHw= pHw;
    pHw->hAudio= (DWORD)pAud;
    pHw->ChInf[AUDCH].dwSafety=SettingLoad(SET_DWORD,IniAudChnlSpacePad,0,16384)/8;
    pHw->ChInf[AUDCH].dwSafety=4096/8;
//    DbgMsgCmd(DBGCMD_PERFADD,(DWORD)&PerfStatAudChnl,(DWORD)&pAud->hPerfChnl);
//    DbgOutCmd(DBGCMD_TIME_NAME,0,(DWORD)"Audio Channel");
    Err= AudResetStateMachine(pAud);
    qInitQueue(&pAud->DataQ);
  }
  return(Err);
}

static long AudFini(AUD *pAud) {
  pAud->Signature= 0;
  pAud->pHw->hAudio= 0;
//  DbgMsgCmd(DBGCMD_PERFDEL,pAud->hPerfChnl,0);
//  pAud->hPerfChnl= 0;
  //Free dynamically allocated memory.
  return(0);
}

static long AudResetStateMachine(AUD *pAud) {
  pAud->CtrlState=C_STATE_NOT_READY;
  pAud->StrmState=S_STATE_STOP;
  pAud->ResetStream=0;
  pAud->DecodeMode=0;
  pAud->u.s.fFmtInUse=0;
  pAud->u.s.fSpdifEnable= 0;
  pAud->TimeZero=MargiGetTime();
  pAud->CurTime=0;
  pAud->DiscardUntilPts=0;
  return(0);
}

// Force State machine to pass thru READY to re-initialize
// this will not change the stream state.
static long AudStreamReset(AUD *pAud) {
  pAud->ResetStream=1;
  pAud->EnableDataTransfer=0;
  DbgMsg(DBG_AUDSTATE,"Audio: Reset State.\r\n");
  return 0;
}

LOCAL long AudStreamState(AUD* pAud, int StrmState) {
  long Err= 0;
  DbgMsg(DBG_AUDSTATE,"Audio: Stream State %s -> %s.\r\n",Ident(ID_STRM_STATE,pAud->StrmState),Ident(ID_STRM_STATE,StrmState));
  pAud->StrmState = StrmState;
  return Err;
}

static long AudStreamFlush(AUD* pAud,int End,PMDS_DATA_BLOCK pFlushMdb) {
  long Err= 0;
  if(End)
    pAud->EndFlush=1;
  else {
    pAud->BeginFlush=1;
    pAud->pFlushMdb=pFlushMdb;
    if(!pFlushMdb) {
      if(pAud->fInPumpData)
        MargiSleep(20);
      qPurgeQueue(&pAud->DataQ,MDS_Cancel);
    }
  }
  return Err;
}

LOCAL long AudStreamStep(AUD* pAud,int nStep) {
  long Err=0;
  pAud->dwStep=1;
  return Err;
}

// This function is responsible for determining the next state
// given the current state and the input variables.
// It does not change the state or perform any other function.  
// It should be called each time the state actually changes or 
// any of the inputs change.
static long AudDetermineNextState(AUD* pAud) {

  int fDone=0;
  int fC1,fC2,fC3;

  pAud->NextState=pAud->CtrlState;

  // If Stream State is stop or stream is reset then all 
  // states except NOT_READY will transition to C_STATE_READY
  if(pAud->StrmState==S_STATE_STOP || pAud->ResetStream) {
    pAud->ResetStream=0;
    switch(pAud->CtrlState) {
      case C_STATE_NOT_READY:
        break;
      default:
        pAud->NextState=C_STATE_READY;
        break;
    }
    fDone=1;
  }
  // All states except NOT_READY,READY and FLUSHCOMPLETE will
  // transition to FLUSH if BeginFlush is set
  if(!fDone && pAud->BeginFlush) {
    switch(pAud->CtrlState) {
      case C_STATE_NOT_READY:
      case C_STATE_READY:
      case C_STATE_FLUSH_COMPLETE:
        break;
      default:
        pAud->NextState=C_STATE_FLUSH;
        fDone=1;
        break;
    }
    pAud->BeginFlush=0;
  }
  if(!fDone) {
    DWORD dwVidState;
    switch(pAud->CtrlState) {
      default:
      case C_STATE_NOT_READY:
        if(pAud->StrmState==S_STATE_RUN)
          pAud->NextState=C_STATE_READY;
        break;
   
      case C_STATE_READY:
        if(pAud->StrmState==S_STATE_RUN && pAud->pHw->u.s.fChannelConfig && !pAud->pHw->fCssWaitForTitleKey)
          pAud->NextState=C_STATE_RDY_WAITDATA;
        break;

      case C_STATE_RDY_WAITDATA:
        if(pAud->fDataXfrPending)
          pAud->NextState=C_STATE_GET_STARTTIME;
        break;

      case C_STATE_GET_STARTTIME:
        if(pAud->pHw->dwAudRate!=RATE_NORMAL)
          pAud->NextState=C_STATE_DISCARD;
        if(pAud->StartTime)
          pAud->NextState=C_STATE_PREFILL;
        break;
    
      case C_STATE_PREFILL:
        if(pAud->pHw->dwAudRate!=RATE_NORMAL)
          pAud->NextState=C_STATE_DISCARD;
        VideoCmd(pAud->pHw,VIDCMD_GET,VIDITEM_STATE,(DWORD)&dwVidState);
        if(pAud->ChannelFull && (dwVidState!=Z_STATE_PREROLL)) {
          pAud->NextState=C_STATE_PLAY_SYNC;
        } else if(pAud->StartTime && tmCompareTimes(pAud->CurScr+40,pAud->StartTime)) {
          pAud->NextState=C_STATE_STALL_TIME;
        }
        break;

      case C_STATE_STALL_TIME:
        VideoCmd(pAud->pHw,VIDCMD_GET,VIDITEM_STATE,(DWORD)&dwVidState);
        fC1=pAud->ChannelFull;
        fC2=(dwVidState!=Z_STATE_PREROLL);
        fC3=IsStateTimeout(pAud);
        if((fC1 && fC2) || fC3) {
          DbgMsg(DBG_AUDSTATE,"Audio: state %s  transition on%s%s%s\r\n",Ident(ID_AUD_STATE,pAud->CtrlState),
                                                  fC1?" AudChannelFull":"",fC2?" VidState":"",fC3?" Timeout":"");
          pAud->NextState=C_STATE_PLAY_SYNC;
        }
        break;
    
      case C_STATE_PLAY_SYNC:
          if(tmCompareTimes(pAud->CurScr,pAud->StartTime)) {
            DbgMsg(DBG_AUDSTATE,"Audio: state %s transition ",Ident(ID_AUD_STATE,pAud->CtrlState));
            DbgDispPTS(DBG_AUDSTATE,pAud->CurScr,"Time="," ");
            DbgDispPTS(DBG_AUDSTATE,pAud->StartTime,"Target=","\r\n");
            pAud->NextState=C_STATE_PLAY;
          } 
//          else DbgDispPTS(DBG_AUDSTATE,pAud->CurScr,"Time=","\r\n");
        break;
    
      case C_STATE_PLAY:
        if(pAud->pHw->dwAudRate!=RATE_NORMAL)
          pAud->NextState=C_STATE_DISCARD;
        if(pAud->StrmState==S_STATE_PAUSE)
          pAud->NextState=C_STATE_PAUSE;
        if(!pAud->EnableDataTransfer && (pAud->pHw->ChInf[AUDCH].dwItems==0))
          pAud->NextState=C_STATE_READY;
        break;
    
      case C_STATE_PAUSE:
        if(pAud->StrmState==S_STATE_RUN)
          pAud->NextState=C_STATE_PLAY;
        if(pAud->dwStep)
          pAud->NextState=C_STATE_DISCARD;
        break;

      case C_STATE_DISCARD:
//        if(pAud->StrmState==S_STATE_PAUSE)
//          pAud->NextState=C_STATE_FLUSH;
        if(pAud->pHw->dwAudRate==RATE_NORMAL && pAud->StrmState!=S_STATE_PAUSE)
          pAud->NextState=C_STATE_FLUSH;
        break;
    
      case C_STATE_FLUSH:
        pAud->NextState=C_STATE_FLUSH_COMPLETE;
        break;

      case C_STATE_FLUSH_COMPLETE:
        if(pAud->EndFlush && !pAud->pHw->u.s.fInPumpData)
          pAud->NextState=C_STATE_READY;
        break;
    }
  }
  return 0;
}


// Do the actual work in transitioning from one state to
// the next.
static long AudProcessState(AUD* pAud) {
  long Err=0;
  if(pAud->CtrlState != pAud->NextState) {
    DbgMsg(DBG_AUDSTATE,"Audio: state %s -> %s.",Ident(ID_AUD_STATE,pAud->CtrlState),Ident(ID_AUD_STATE,pAud->NextState));
    DbgDispPTS(DBG_AUDSTATE,pAud->CurScr," (Time=",")\r\n");

    AudSetStateTimeout(pAud,0);   // If state transition about to occur then clear state timeout
  }
  switch (pAud->CtrlState) {
    default:
    case C_STATE_NOT_READY:
      Err=AudProcessNotReady(pAud);
      break;

    case C_STATE_READY:
      Err=AudProcessReady(pAud);
      break;

    case C_STATE_RDY_WAITDATA:
      Err=AudProcessRdyWaitData(pAud);
      break;

    case C_STATE_GET_STARTTIME:
      Err=AudProcessGetStartTime(pAud);
      break;

    case C_STATE_PREFILL:
      Err=AudProcessPrefill(pAud);
      break;

    case C_STATE_STALL_TIME:
      Err=AudProcessStallTime(pAud);
      break;

    case C_STATE_PLAY_SYNC:
      Err=AudProcessPlaySync(pAud);
      break;

    case C_STATE_PLAY:
      Err=AudProcessPlay(pAud);
      break;

    case C_STATE_PAUSE:
      Err=AudProcessPause(pAud);
      break;

    case C_STATE_DISCARD:
      Err=AudProcessDiscard(pAud);
      break;

    case C_STATE_FLUSH:
      Err=AudProcessFlush(pAud);
      break;

    case C_STATE_FLUSH_COMPLETE:
      Err=AudProcessFlushComplete(pAud);
      break;
  }
  return Err;
}


static long AudProcessNotReady(AUD* pAud) {
  long Err=0;
  switch(pAud->NextState) {
    case C_STATE_NOT_READY:
      break;
    case C_STATE_READY:
      Err=AudStateEnterReady(pAud);
      break;
    default:
      Err=AudStateInvalid(pAud);
  }
  return Err;
}

static long AudProcessReady(AUD* pAud) {
  long Err=0;
  switch(pAud->NextState) {
    case C_STATE_READY:
      if(pAud->ResetStream)
        Err=AudStateEnterReady(pAud);
      break;
    case C_STATE_RDY_WAITDATA:
      RegScrPause(pAud->pHw,FLAG_SCR_UNPAUSE);
      pAud->fDataXfrPending=0;
      pAud->CtrlState=pAud->NextState;
      break;
    default:
      Err=AudStateInvalid(pAud);
  }
  return Err;
}

static long AudProcessRdyWaitData(AUD* pAud) {
  long Err=0;
  switch(pAud->NextState) {
    case C_STATE_RDY_WAITDATA:
      if(pAud->ResetStream)
        Err=AudStateEnterReady(pAud);
      break;
    case C_STATE_GET_STARTTIME:
      AudUpdateFormat(pAud);
      pAud->ChannelFull=0;
      pAud->StartTime=0;
      pAud->EnableDataTransfer=1;
      pAud->pHw->u.s.fAudChannelPrefill=1;
      pAud->CtrlState=pAud->NextState;
      break;
    case C_STATE_DISCARD:
      if(pAud->pHw->dwAudRate>RATE_NORMAL)
        dvHwMainSetScrSlowRate(pAud->pHw,pAud->pHw->dwAudRate/RATE_NORMAL);
      pAud->fDiscardData=1;
      pAud->CtrlState=pAud->NextState;
      break;
    case C_STATE_FLUSH:
      Err=AudStateEnterFlush(pAud);
      break;
    default:
      Err=AudStateInvalid(pAud);  // fall thru to Ready
    case C_STATE_READY:
      Err=AudStateEnterReady(pAud);
      break;
  }
  return Err;
}

static long AudProcessGetStartTime(AUD* pAud) {
  long Err=0;
  switch(pAud->NextState) {
    case C_STATE_GET_STARTTIME:
      SyncCmd(pAud->pHw,SYNC_GET,SYNC_ITEM_AUDPTSMIN,(DWORD)&pAud->StartTime);
      break;
    case C_STATE_PREFILL:
      AudSetStateTimeout(pAud,250);
      pAud->CtrlState=pAud->NextState;
      break;
    case C_STATE_DISCARD:
      if(pAud->pHw->dwAudRate>RATE_NORMAL)
        dvHwMainSetScrSlowRate(pAud->pHw,pAud->pHw->dwAudRate/RATE_NORMAL);
      pAud->fDiscardData=1;
      pAud->CtrlState=pAud->NextState;
      break;
    case C_STATE_FLUSH:
      Err=AudStateEnterFlush(pAud);
      break;
    default:
      Err=AudStateInvalid(pAud);  // fall thru to Ready
    case C_STATE_READY:
      Err=AudStateEnterReady(pAud);
      break;
  }
  return Err;
}

static long AudProcessPrefill(AUD* pAud) {
  long Err=0;
  switch(pAud->NextState) {
    case C_STATE_PREFILL:
      break;
    case C_STATE_STALL_TIME:
      RegScrPause(pAud->pHw,FLAG_SCR_PAUSE);
      AudSetStateTimeout(pAud,250);
      pAud->CtrlState=pAud->NextState;
      break;
    case C_STATE_PLAY_SYNC:
      SyncCmd(pAud->pHw,SYNC_ENABLE,STRM_AUD,SYNC_ENABLE_ENABLE);
      pAud->pHw->u.s.fAudChannelPrefill=0;
      pAud->CtrlState=pAud->NextState;
      break;
    case C_STATE_DISCARD:
      if(pAud->pHw->dwAudRate>RATE_NORMAL)
        dvHwMainSetScrSlowRate(pAud->pHw,pAud->pHw->dwAudRate/RATE_NORMAL);
      pAud->fDiscardData=1;
      pAud->CtrlState=pAud->NextState;
      break;
    case C_STATE_FLUSH:
      Err=AudStateEnterFlush(pAud);
      break;
    default:
      Err=AudStateInvalid(pAud);  // fall thru to Ready
    case C_STATE_READY:
      Err=AudStateEnterReady(pAud);
      break;
  }
  return Err;
}

static long AudProcessStallTime(AUD* pAud) {
  long Err=0;
  switch(pAud->NextState) {
    case C_STATE_STALL_TIME:
      break;
    case C_STATE_PLAY_SYNC:
      RegScrPause(pAud->pHw,FLAG_SCR_UNPAUSE);
      SyncCmd(pAud->pHw,SYNC_ENABLE,STRM_AUD,SYNC_ENABLE_ENABLE);
      pAud->pHw->u.s.fAudChannelPrefill=0;
      pAud->CtrlState=pAud->NextState;
      break;
    case C_STATE_FLUSH:
      RegScrPause(pAud->pHw,FLAG_SCR_UNPAUSE);
      Err=AudStateEnterFlush(pAud);
      break;
    default:
      Err=AudStateInvalid(pAud);  // fall thru to Ready
    case C_STATE_READY:
      RegScrPause(pAud->pHw,FLAG_SCR_UNPAUSE);
      Err=AudStateEnterReady(pAud);
      break;
  }
  return Err;
}

static long AudProcessPlaySync(AUD* pAud) {
  long Err=0;
//  DWORD dwVidStartTime;
  switch(pAud->NextState) {
    case C_STATE_PLAY_SYNC:
      // fix for DirectX 5 get vidstart time if both aud and vid start times
      // more than 500mS in future then bump ScrReg to within 100mS
//      SyncCmd(pAud->pHw,SYNC_GET,SYNC_ITEM_VIDPTSMIN,(DWORD)&dwVidStartTime);
//      if((pAud->CurScr+(500*90) < pAud->StartTime) && (pAud->CurScr+(500*90) < dwVidStartTime)){
//        dvHwMainWriteScrReg(pAud->pHw,pAud->StartTime-(100*90));
//      }
      break;
    case C_STATE_PLAY:
//      if(pAud->StartTime)
//        dvHwMainWriteScrReg(pAud->pHw,pAud->StartTime);
      SyncCmd(pAud->pHw,SYNC_STRM_START,STRM_AUD,pAud->CurScr-pAud->StartTime);
      AudTimedUnmute(pAud,250);     // release local mute in 250 mS
      AudStart(pAud,1);
      pAud->CtrlState=pAud->NextState;
      break;
    case C_STATE_FLUSH:
      Err=AudStateEnterFlush(pAud);
      break;
    default:
      Err=AudStateInvalid(pAud);  // fall thru to Ready
    case C_STATE_READY:
      Err=AudStateEnterReady(pAud);
      break;
  }
  return Err;
}

static long AudProcessPlay(AUD* pAud) {
  long Err=0;
  switch (pAud->NextState) {
    case C_STATE_PLAY:
      break;
    case C_STATE_PAUSE:
      RegScrPause(pAud->pHw,FLAG_SCR_PAUSE);
      AudStart(pAud,0);
      pAud->CtrlState=pAud->NextState;
      break;
    case C_STATE_DISCARD:
      AudStart(pAud,0);
      if(pAud->pHw->dwAudRate>RATE_NORMAL)
        dvHwMainSetScrSlowRate(pAud->pHw,pAud->pHw->dwAudRate/RATE_NORMAL);
      pAud->fDiscardData=1;
      pAud->CtrlState=pAud->NextState;
      break;
    case C_STATE_FLUSH:
      Err=AudStateEnterFlush(pAud);
      break;
    default:
      Err=AudStateInvalid(pAud);  // fall thru to Ready
    case C_STATE_READY:
      Err=AudStateEnterReady(pAud);
      break;
  }
  return Err;
}

static long AudProcessDiscard(AUD* pAud) {
  long Err=0;
  switch (pAud->NextState) {
    case C_STATE_DISCARD:
      break;
    case C_STATE_FLUSH:
      if(pAud->StrmState==S_STATE_PAUSE)
        RegScrPause(pAud->pHw,FLAG_SCR_PAUSE);
      Err=AudStateEnterFlush(pAud);
      pAud->EndFlush=1;
      break;
    default:
      Err=AudStateInvalid(pAud);  // fall thru to Ready
    case C_STATE_READY:
      Err=AudStateEnterReady(pAud);
      break;
  }
  return Err;
}

static long AudProcessPause(AUD* pAud) {
  long Err=0;
  switch (pAud->NextState) {
    case C_STATE_PAUSE:
      break;
    case C_STATE_PLAY:
      RegScrPause(pAud->pHw,FLAG_SCR_UNPAUSE);
      AudStart(pAud,1);
      pAud->CtrlState=pAud->NextState;
      break;
    case C_STATE_DISCARD:
      pAud->fDiscardData=1;
      pAud->CtrlState=pAud->NextState;
      break;
    case C_STATE_FLUSH:
      Err=AudStateEnterFlush(pAud);
      break;
    default:
      Err=AudStateInvalid(pAud);  // fall thru to Ready
    case C_STATE_READY:
      Err=AudStateEnterReady(pAud);
      break;
  }
  return Err;
}

static long AudProcessFlush(AUD* pAud) {
  long Err=0;
  switch (pAud->NextState) {
    case C_STATE_FLUSH:
    break;
    case C_STATE_FLUSH_COMPLETE:
      pAud->EnableDataTransfer=0;
      pAud->CtrlState=C_STATE_FLUSH_COMPLETE;
      break;
    default:
      Err=AudStateInvalid(pAud);  // fall thru to Ready
    case C_STATE_READY:
      Err=AudStateEnterReady(pAud);
      break;
  }
  return Err;
}

static long AudProcessFlushComplete(AUD* pAud) {
  long Err=0;
  switch (pAud->NextState) {
    case C_STATE_FLUSH_COMPLETE:
    break;
    default:
      Err=AudStateInvalid(pAud);  // fall thru to Ready
    case C_STATE_READY:
      Err=AudStateEnterReady(pAud);
      break;
  }
  return Err;
}

static long AudStateEnterFlush(AUD* pAud) {
  long Err=0;
  pAud->ResetStream=0;
  pAud->BeginFlush=0;
  pAud->dwStep=0;
  AudSetStateTimeout(pAud,250);
  pAud->CtrlState=C_STATE_FLUSH;
  return 0;
}
 
static long AudStateEnterReady(AUD* pAud) {
  long Err=0;
  int fFullReset;
  PHW pHw= pAud->pHw;

  pAud->ResetStream=0;
  pAud->EndFlush=0;
  pAud->EnableDataTransfer=0;
  pAud->fNewFormat=0;
  pAud->fDiscardData=0;
  pAud->dwStep=0;

  // Immediate mute
  pAud->fCurSpdifEnable=-1;
  pAud->fCurMute=-1;
  AudLocalMute(pAud);
  AudUpdateMute(pAud);
  
  AudStart(pAud,0);
  dvHwMainSetScrSlowRate(pAud->pHw,0);
  SyncCmd(pAud->pHw,SYNC_ENABLE,STRM_AUD,SYNC_ENABLE_DISABLE);
  AudCorrectSync(pAud,0,0);
  fFullReset=(pAud->StrmState==S_STATE_STOP) && gfResetTime;
  SyncCmd(pAud->pHw,SYNC_RESET,fFullReset,STRM_AUD);
  dvHwMainWriteLsiRegBits(pAud->pHw,RESET_FIFO_REG,RESET_FIFO_AUD);
  dvHwMainClearChCounters(pHw,AUDCH);
  pAud->CtrlState=C_STATE_READY;
  return 0;
}

static long AudSetStateTimeout(AUD* pAud, DWORD dwMilliSecs) {
  if(dwMilliSecs) {
    pAud->StateTimeoutTime=pAud->CurTime+dwMilliSecs;
    if(pAud->StateTimeoutTime==0)
      pAud->StateTimeoutTime++;
  } else
    pAud->StateTimeoutTime=0;
  return 0;
}

static long IsStateTimeout(AUD* pAud) {
  return (pAud->StateTimeoutTime && (pAud->CurTime>=pAud->StateTimeoutTime));
}

static long AudStateInvalid(AUD *pAud) {
  DbgMsg(DBG_HWERR|DBG_AUDSTATE,"Audio: Invalid state transition %s -> %s.\r\n",Ident(ID_AUD_STATE,pAud->CtrlState),Ident(ID_AUD_STATE,pAud->NextState));
  return(MARGI_ERR_INVALID_STATE_TRANSITION);
}

LOCAL long AudCorrectSync(AUD* pAud,DWORD dwAhead,DWORD dwError) {
  pAud->SyncAhead=dwAhead;
  pAud->SyncError=dwError;
  DbgMsg(DBG_SYNCADJ,"Audio: CorrectSync Error=%dms %s \r\n",dwError,dwAhead?"ahead":"behind");
  return 0;
}

/*************************************************************************/
/** Get/Set parameters.							**/
/*************************************************************************/
static long AudSetDecodeMode(AUD *pAud,DWORD SrcType);
static long AudSetVolume(AUD *pAud, DWORD Level);
static long AudSetMute(AUD *pAud, DWORD Mute);
static long AudSetMix(AUD *pAud, DWORD Mix);
static long AudSetKaraokeMode(AUD *pAud,DWORD dwVal);
static long AudSpdifEnable(AUD *pAud,DWORD dwVal);
static long AudSetSpDiffMode(AUD *pAud, DWORD dwMode);
static DWORD AudGetState(AUD *pAud);

static long AudGet(AUD *pAud, DWORD Item, DWORD *pValue) {
  long Err= 0;
  DWORD Value;
  switch(Item) {
    case AUDITEM_STATE:		Value= AudGetState(pAud);   break;
    case AUDITEM_VOLUME:	Value= pAud->pHw->dwVolume; break;
    case AUDITEM_MUTE:		Value= pAud->pHw->dwMute;   break;
    case AUDITEM_MIX:		  Value= pAud->AC3MixMode;    break;
    case AUDITEM_SRCTYPE:	Value= pAud->SrcType;       break;
    case AUDITEM_SAMPLES:	Value= pAud->SampleRate;    break;
    case AUDITEM_OUTMODE:	Value= pAud->OutputMode;    break;
    case AUDITEM_KARAOKE: Value= (DWORD)pAud->KaraokeMode; break;
    case AUDITEM_SPDIF:   Value= (DWORD)pAud->u.s.fSpdifEnable; break;
    case AUDITEM_SPDMODE: Value= pAud->SpdifMode;    break;
    default:			Err= MARGI_ERR_BADITEM;
  }
  if(!Err) {
    if(pValue)
      *pValue= Value;
    else
      Err= (long)Value;
  }
  return(Err);
}

static long AudSet(AUD *pAud, DWORD Item, DWORD Value) {
  long Err= 0;
  switch(Item) {
    case AUDITEM_VOLUME:	Err= AudSetVolume(pAud,Value); break;
    case AUDITEM_MUTE:		Err= AudSetMute(pAud,Value); break;
    case AUDITEM_MIX:		  Err= AudSetMix(pAud,Value); break;
    case AUDITEM_SRCTYPE:	Err= AudSetDecodeMode(pAud,Value); break;
    case AUDITEM_SAMPLES:	pAud->SampleRate=Value; 
                          pAud->fNewFormat=1;
                          break;
    case AUDITEM_OUTMODE:	pAud->OutputMode=Value;
                          pAud->fNewFormat=1;
                          break;
    case AUDITEM_KARAOKE: Err= AudSetKaraokeMode(pAud,Value); break;
    case AUDITEM_SPDIF:   Err= AudSpdifEnable(pAud,Value); break;
    case AUDITEM_SPDMODE: Err= AudSetSpDiffMode(pAud,Value); break;
    case AUDITEM_NUMCHANNELS:   pAud->dwNumChannels= Value; break;
      
    default:			Err= MARGI_ERR_BADITEM;
  }
  return(Err);
}

static DWORD AudGetState(AUD *pAud) {
  DWORD Value;
  switch(pAud->CtrlState) {
    case C_STATE_NOT_READY:
    case C_STATE_READY:
      Value= Z_STATE_STOP;
      break;
    case C_STATE_PAUSE:
      Value= Z_STATE_PAUSE;
      break;
    case C_STATE_PLAY:
      Value= Z_STATE_PLAY;
      break;
    case C_STATE_GET_STARTTIME:
    case C_STATE_PREFILL:
    case C_STATE_STALL_TIME:
    case C_STATE_FLUSH:
    case C_STATE_FLUSH_COMPLETE:
      Value= Z_STATE_PREROLL;
      break;
    case C_STATE_PLAY_SYNC:
      Value= Z_STATE_SYNC;
      break;
    default:
    case C_STATE_UNK:
      Value= Z_STATE_UNK;
      break;
  }
  return(Value);
}



static long AudSpdifEnable(AUD *pAud,DWORD dwVal) {
  long Err= 0;
  pAud->u.s.fSpdifEnable=(dwVal!=0);
  pAud->fUpdateOutput=1;
  return Err;
}

/************************************************************************
 **  Set SpDiff Output Mode                                            **
 ************************************************************************/
static long AudSetSpDiffMode(AUD *pAud, DWORD dwMode) {
  if(pAud->SpdifMode!=dwMode) {
    AudSetDecodeMode(pAud,pAud->SrcType);
    pAud->SpdifMode = dwMode;
    pAud->ResetStream=1;
  }
  return 0;
}

static long AudSetKaraokeMode(AUD *pAud,DWORD dwVal) {
  long Err= 0;
  BYTE bRegVal= 0;
  BYTE bSetMelody= 0;
  if((dwVal&0xFF00)==MDS_KM_MELODY_DEFAULT) {
    switch(((BYTE)dwVal)) { //This is for old format support new code should use MDS_KM_ ...
      case 0x01:    bRegVal= 0x60; break; //Vocal1
      case 0x02:    bRegVal= 0xA0; break; //Vocal2
      case 0x03:    bRegVal= 0x40; break; //NoVacal
      case 0x00:
      default:                bRegVal= 0xE0; break;
    }
  } else {
    switch(((BYTE)dwVal)) {
      case MDS_KM_NO_VOCAL:  bRegVal = KARAOKE_NO_VOCAL; break;
      case MDS_KM_VOCAL1:    bRegVal = KARAOKE_VOCAL1; break;
      case MDS_KM_VOCAL2:    bRegVal = KARAOKE_VOCAL2; break;
      default:
      case MDS_KM_VOCAL1_2:  bRegVal = KARAOKE_VOCAL12; break;
    }
    bSetMelody = (((dwVal&0xFF00)==MDS_KM_MELODY_ON)?1:0);
    bRegVal = ((bRegVal<<6)|(bSetMelody<<5));
  }
  dvHwMainWriteLsiRegBits(pAud->pHw,KARAOKE_SEL_REG,bRegVal);
  pAud->KaraokeMode= dwVal;
  DbgMsg(DBG_CP_SET,"Karaoke Mode Set to %s \r\n",Ident(ID_KARAOKE_MODE,(BYTE)dwVal));
  return Err;
}

static long AudSetVolume(AUD *pAud, DWORD Level) {
  DWORD ActualVolume= 0;
  pAud->pHw->dwVolume= Level;
  pAud->fUpdateOutput=1;
  DbgMsg(DBG_CP_SET,"Audio: Volume set to %u.\r\n",ActualVolume);
  return(0);
}

static long AudSetMute(AUD *pAud, DWORD Mute) {
  if(Mute)
    pAud->pHw->dwMute= 1;
  else
    pAud->pHw->dwMute= 0;
  pAud->fUpdateOutput=1;
  DbgMsg(DBG_CP_SET,"Audio: Mute set to %u.\r\n",pAud->pHw->dwMute);
  return(0);
}

static long AudSetMix(AUD *pAud, DWORD Mix) {
  long Err= 0;
  Mix&= 1;
  dvHwMainWriteLsiRegBits(pAud->pHw,AC3_DOWNMIX_MODE_SELECT,(BYTE)((BYTE)Mix<<4));
  pAud->AC3MixMode= Mix;
  DbgMsg(DBG_CP_SET,"AC3 Audio Mix Mode Set to %s\r\n",Ident(ID_AC3_DOWNMIX,pAud->AC3MixMode));
  return Err;
}

static void AudUpdateStream(AUD *pAud) {
  BYTE AudSrc;
  switch(pAud->SrcType) {
    case AUD_AC3:		AudSrc= 0x60; break;
    case AUD_LPCM:  AudSrc= 0x40; break;
    default:			  AudSrc= 0x80; //MPEG
  }
#if(0)
  if(pAud->pHw->fUsePesHdr) {
    dvHwMainWriteLsiReg(pAud->pHw,PES_HEADER_ENABLE_REG,WRITE_ONE_AUD_PTS);
    DbgMsg(DBG_AUDHW,"Audio: PES headers enabled.\r\n");
  } else {
    dvHwMainWriteLsiReg(pAud->pHw,PES_HEADER_ENABLE_REG,WRITE_ONE_AUD_PTS);
    DbgMsg(DBG_AUDHW,"Audio: PES headers disabled.\r\n");
  }
#endif
  dvHwMainWriteLsiReg(pAud->pHw,AUDIO_STREAM_REG,(BYTE)(AudSrc|pAud->Stream));
  DbgMsg(DBG_AUDHW,"Audio: Selected stream %02x.\r\n",AudSrc|pAud->Stream);
}

static long AudUpdateSampleRate(AUD *pAud) {
  long Err=0;
  WORD wMkRate=0;
  PHW  pHw= pAud->pHw;

  dvHwMainWriteLsiRegBits(pHw,DIVIDER_SEL_REG,0x04);  // Set Audio divier for 256 Fs  
  switch(pAud->SampleRate) {
    case 96000: wMkRate = 0; break;
    case 48000: wMkRate = 0; break;
    case 44100: wMkRate = 1; break;
    case 32000: wMkRate = 2; break;
    default:    wMkRate = 0;
                DbgMsg(DBG_WRN,"dvHwSetAudioSampleRate(%u): Illegal value.\r\n",pAud->SampleRate);
                break;
  }
  dvHwMainWriteLsiReg(pHw,363,(BYTE)(wMkRate|0x01));   // Select non-i2s, 32 bit
  DbgMsg(DBG_AUDHW,"Audio Sampling Frequency = %d  (MkRate=%d)\r\n",pAud->SampleRate,wMkRate);

  wMkRate|=pHw->fSClkOff?0x30:0x10;
  WrByte(pHw,REG_INDEX(pHw),(BYTE)RIX_OSC_CNTRL);
  WrByte(pHw,REG_DATA(pHw),(BYTE)wMkRate);        // set rate, ZV formatter
  if(RdByte(pHw,REG_DATA(pHw))!=0x10) {
    volatile int i=0;
  }
  //AudSetKaraokeMode(pAud,KARAOKE_VOCAL12);      //Vocal 1 and 2 by default
  return Err;
}

static void AudUpdateMode(AUD *pAud) {
  PHW pHw= pAud->pHw;

  // Compute Decode Mode for LSI Reg 357
  pAud->DecodeMode=0;
  pAud->u.s.fFmtInUse=0;
  if(pAud->SrcType==AUD_LPCM) {
    if(pAud->SampleRate==96000)
      pAud->DecodeMode=0xa0;
    else
      pAud->DecodeMode=0x80;
  } else {
    if(pAud->SrcType==AUD_AC3)
      pAud->DecodeMode=0x20;
    if(pAud->SpdifType!=AUD_LPCM) {
      pAud->DecodeMode|=0x40;
      pAud->u.s.fFmtInUse=1;
    }
  }
  DbgMsg(DBG_AUDHW,"Audio: Set Decode Mode = %d.\r\n",pAud->DecodeMode);
  dvHwMainWriteLsiRegBits(pHw,AUD_FMT_COMMAND_REG,AUD_DECODER_STOP);
  dvHwMainWriteLsiRegBits(pHw,BS_SELECT_REG, (BYTE)pAud->DecodeMode);  // Set bs_sel after formatter has been stopped

  if(pAud->dwNumChannels==1)
    dvHwMainWriteLsiRegBits(pAud->pHw,AUDIO_DUAL_MONO_MODE_REG, AUD_LEFT_ONLY);


  if(pAud->u.s.fFmtInUse) {
    DbgMsg(DBG_AUDHW,"Audio: Start and Pause Formatter.\r\n");
    dvHwMainWriteLsiRegBits(pHw, AUD_FMT_COMMAND_REG,AUD_DECODER_START);
    dvHwMainWriteLsiRegBits(pHw, AUD_FMT_PLAY_MODE_REG,AUD_PAUSE_MODE);
  }
  dvHwMainWriteLsiRegBits(pHw,OVERWRITE_QUANT,0x08);                    // Set 16-bit PCM output irrespective of the source quantization level
}

static void AudUpdateFormat(AUD* pAud) {
// update Decode/Format Mode, Stream, SampleRate
  AudUpdateMode(pAud);
  AudUpdateStream(pAud);
  AudUpdateSampleRate(pAud);
}

/*************************************************************************/
/** Miscellaneous audio commands.					**/
/*************************************************************************/
static long AudSelectStream(AUD *pAud, DWORD Stream) {

  if(Stream>0x0F)
    Stream= 0;
  pAud->Stream=Stream;
  return(0);
}

static long AudSetDecodeMode(AUD *pAud,DWORD SrcType) {
  long Err= 0;
  PHW pHw= pAud->pHw;

  DbgMsg(DBG_AUDHW,"dvHwSetAudioType(%d)\r\n",SrcType);
  
  pAud->SrcType= SrcType;
  // Determine type of SPDIF output
  pAud->SpdifType=pAud->SrcType;
  if(pAud->SpdifMode!=MDS_SPDIF_ALL) {
    switch(pAud->SrcType) {
      case AUD_MPEG: if(pAud->SpdifMode==MDS_SPDIF_AC3) pAud->SpdifType=AUD_LPCM; break;
      case AUD_AC3 : if(pAud->SpdifMode==MDS_SPDIF_MPEG)  pAud->SpdifType=AUD_LPCM; break;
      default:
      case AUD_LPCM: pAud->SpdifType=AUD_LPCM; break;
    }
  }
  pAud->fNewFormat=1;
  return 0;
}

#if(0)
  if(pAud->fSpdif && pAud->dwFrmtMode!=SPDIFF_LINEAR) {
    switch(dwSrcType) {
      case AUD_MPEG: if((pAud->dwFrmtMode==SPDIFF_MPEG)||(pAud->dwFrmtMode==SPDIFF_ALL)) {
                       dwSrcType = AUD_MPEG_FMT;
                       DbgMsg(DBG_CP_SET,"Select Musicam formatter\r\n");
                     }
                     break;
      case AUD_AC3:  if((pAud->dwFrmtMode==SPDIFF_DOLBY)||(pAud->dwFrmtMode==SPDIFF_ALL)) {
                       dwSrcType = AUD_AC3_FMT;
                       DbgMsg(DBG_CP_SET,"Select Dolby Digital formatter\r\n");
                     }
                     break;
    }
  }

  pAud->fFrmtSel = (dwSrcType==AUD_AC3_FMT||dwSrcType==AUD_MPEG_FMT);

  if(pAud->fFrmtSel) {
    dvHwMainWriteLsiRegBits(pHw,BS_SELECT_REG,(BYTE)dwSrcType);  // Set bs_sel
    if(!bPrevFrmt) {
      dvHwMainWriteLsiRegBits(pHw, AUD_FMT_COMMAND_REG,AUD_DECODER_START);
      dvHwMainWriteLsiRegBits(pHw, AUD_FMT_PLAY_MODE_REG,AUD_PLAY_MODE);
    }
  } else {
    if(bPrevFrmt) {
      dvHwMainWriteLsiRegBits(pHw, AUD_FMT_COMMAND_REG,AUD_DECODER_STOP);
      dvHwMainWriteLsiRegBits(pHw, AUD_FMT_PLAY_MODE_REG,AUD_PAUSE_MODE);
    }
    dvHwMainWriteLsiRegBits(pHw, BS_SELECT_REG, (BYTE)dwSrcType);  // Set bs_sel after formater has been stopped
  }
  dvHwMainWriteLsiRegBits(pHw,OVERWRITE_QUANT,0x08); // Set 16-bit PCM output irrespective of the source quantization level

  return Err;
}
#endif

static long AudStart(AUD *pAud, DWORD Start) {
  if(Start) {
    dvHwMainWriteLsiRegBits(pAud->pHw,AUD_PLAY_MODE_REG,AUD_PLAY_MODE);
    DbgMsg(DBG_AUDHW,"Audio: Set Decoder = Play.\r\n");
    if(pAud->u.s.fFmtInUse) {
      dvHwMainWriteLsiRegBits(pAud->pHw,AUD_FMT_PLAY_MODE_REG,AUD_PLAY_MODE);
      DbgMsg(DBG_AUDHW,"Audio: Set Formatter = Play.\r\n");
    }
  } else {						//Stop decoder
    dvHwMainWriteLsiRegBits(pAud->pHw,AUD_PLAY_MODE_REG,AUD_PAUSE_MODE);
    DbgMsg(DBG_AUDHW,"Audio: Set Decoder = Pause.\r\n");
    if(pAud->u.s.fFmtInUse) {
      dvHwMainWriteLsiRegBits(pAud->pHw,AUD_FMT_PLAY_MODE_REG,AUD_PAUSE_MODE);
      DbgMsg(DBG_AUDHW,"Audio: Set Formatter = Pause.\r\n");
    }
  }
  return(0);
}

static long AudRate(AUD *pAud, MDS_PLAYRATE*pPlayRate) {
  long Err= 0;
  PHW pHw= pAud->pHw;

  DbgMsg(DBG_AUDHW,"Audio: Set Rate = %d.\r\n",pPlayRate->Rate);
  pHw->dwAudRate= pPlayRate->Rate;
  tmCueNewClkRate(pHw,pPlayRate->Rate,pPlayRate->StartTime);

  if(pPlayRate->Rate==RATE_NORMAL) {
    SyncCmd(pAud->pHw,SYNC_ENABLE,STRM_AUD,SYNC_ENABLE_ENABLE);	  //Enable a/v sync on audio if playing at normal speed.
    AudCorrectSync(pAud,0,0);
    AudTimedUnmute(pAud,500);
  } else {
    SyncCmd(pAud->pHw,SYNC_ENABLE,STRM_AUD,SYNC_ENABLE_DISABLE);	//Disable sync if not playing at normal speed.
    SyncCmd(pAud->pHw,SYNC_RESET,0,STRM_AUD);
    AudCorrectSync(pAud,0,0);
    AudUpdateMute(pAud);
  }
  return Err;
}

/*************************************************************************/
/*************************************************************************/
static long AudUpdateVolume(AUD *pAud) {
  DWORD dwVolume;
  DWORD dwCurVol= (BYTE)dvHwMainReadLsiReg(pAud->pHw,PCM_SCALE_REG);
  dwVolume= (pAud->pHw->dwVolume*255)/100; //scale 0-100 to 0-255
  if(pAud->dwPcmScale)
    dwVolume= (dwVolume*1000)/pAud->dwPcmScale;
  if(dwVolume>255)
    dwVolume=255;
  //if(dwVolume!=pAud->dwCurVol) {
  if(dwVolume!=dwCurVol) {
    DbgMsg(DBG_AUDHW,"Audio: Set Volume = %d.\r\n",dwVolume);
    dvHwMainWriteLsiReg(pAud->pHw,PCM_SCALE_REG,(BYTE)dwVolume);
    pAud->dwCurVol=dwVolume;
  }
  return(0);
}

static long AudUpdateMute(AUD *pAud) {
  PHW pHw= pAud->pHw;
  int fMute=0;
  int fSpdifEnable;

  if(pHw->dwMute) {
    fMute=1;
  } else if(pAud->MuteTime && pAud->CurTime<pAud->MuteTime) {
    fMute=1;
  } else if(pAud->fLocalMute) {
    fMute=1;
    if(pAud->MuteTime && pAud->CurTime>=pAud->MuteTime) {
      pAud->MuteTime= 0;
      pAud->fLocalMute=0;
      fMute=0;
    }
  }
  fSpdifEnable=pAud->u.s.fSpdifEnable && !fMute;
  if(fSpdifEnable!=pAud->fCurSpdifEnable) {
    DbgMsg(DBG_AUDHW,"Audio: Mute Spdif= %d.\r\n",!fSpdifEnable);
    dvHwMainSetGPIO(pAud->pHw,GPIO_PIN_SPDIF,(BYTE)fSpdifEnable,0);
    pAud->fCurSpdifEnable=fSpdifEnable;
  }

  if(fMute!=pAud->fCurMute) {
    DbgMsg(DBG_AUDHW,"Audio: Mute Analog= %d.\r\n",fMute);
    dvHwMainWriteLsiRegBits(pHw,USER_MUTE_REG,(BYTE)(fMute?0x40:0x00));
    pAud->fCurMute=fMute;
  }
  return(0);
}

static void AudTimedUnmute(AUD *pAud,DWORD dwMs) {
  pAud->fLocalMute=1;
  pAud->MuteTime=pAud->CurTime+dwMs;
}

static void AudLocalMute(AUD *pAud) {
  pAud->fLocalMute=1;
  pAud->MuteTime=0;
}


/*************************************************************************/
/** Timer tick.								**/
/*************************************************************************/
static long AudTimer(AUD *pAud) {

  if(pAud->pHw->ChInf[AUDCH].dwItems>=pAud->pHw->ChInf[AUDCH].dwPrefillLevel)
    pAud->ChannelFull=1;

  pAud->CurScr=dvHwMainGetDecoderTime(pAud->pHw,0);
  pAud->CurTime= MargiGetTime()-pAud->TimeZero;

  if(pAud->hPerfChnl) {
    DWORD BytesInChnl;
    BytesInChnl= dvHwMainGetAddr(pAud->pHw,AUD_CHAN_NUMITEM_REG)*8;
//    DbgMsgCmd(DBGCMD_PERFSET,pAud->hPerfChnl,BytesInChnl);
//    DbgOutCmd(DBGCMD_TIME_IN,0,0);
  }

  if(pAud->MuteTime || pAud->fUpdateOutput) {
    AudUpdateVolume(pAud);
    AudUpdateMute(pAud);
    pAud->fUpdateOutput=0;
  }

  AudDetermineNextState(pAud);
  AudProcessState(pAud);

  return(0);
}

/*************************************************************************/
/** Send data to decoder.						**/
/*************************************************************************/
static DWORD AudSendData(AUD *pAud,PMDS_DATA_PACKET pMdp) {
  long Err= 0;
  DWORD dwBytesWritten=0;
  PHW pHw= pAud->pHw;
  if(pMdp) {
    // if discontinuity then disable transfer until state machine has dealt with it
    if(pMdp->OptionsFlags&MDB_FLAG_DATADISCONTINUITY) {
      if(pAud->fNewFormat)
        AudStreamReset(pAud);

      if(pAud->CtrlState==C_STATE_PLAY || pAud->fNewFormat) {
        // If we respond to flag then clear so we don't see it again
        pMdp->OptionsFlags&=~MDB_FLAG_DATADISCONTINUITY;
        pAud->EnableDataTransfer=0;
      }
    }
    if(pMdp->DataUsed) {
      if(pAud->EnableDataTransfer && !pHw->fCssWaitForTitleKey) {
        if(pAud->fDiscardData) {
          dwBytesWritten=pMdp->DataUsed;
        } else {
          DWORD dwSpace;
          // Limit audio transfered to DMA buffer so the audio
          // doesn't hold off video
          if((dwSpace=pHw->ChInf[AUDCH].dwSize-pHw->ChInf[AUDCH].dwItems)>pHw->ChInf[AUDCH].dwSafety)
            dwSpace-=pHw->ChInf[AUDCH].dwSafety;
          else
            dwSpace=0;
          
          if(dwSpace*8>=pMdp->DataUsed+16384) {
            void* pS;
            SYNC_TIME90 Time90;
  #ifdef DEBUG
          if(pMdp->DataUsed!=2048) {
            static int n=0;
            if(n++<5)
              DbgMsg(DBG_CRIT,"Audio data length=%d\r\n",pMdp->DataUsed);
          }
  #endif

            if(pHw->fTimeFormat10M)
              tmConvertTime10Mto90K(&pMdp->PresentationTime.Time,&Time90);
            else {
              Time90.dwTimeLo= (DWORD)GetLow_MdsLongLong (&pMdp->PresentationTime.Time);
              Time90.dwTimeHi= (DWORD)GetHigh_MdsLongLong(&pMdp->PresentationTime.Time);
            }
            if(!pHw->fUsePesHdr)
              pS=SyncAudPktBegin(pHw,pMdp->Data,pMdp->DataUsed,&Time90);

            if(Err= TransferData(pHw,pMdp->Data,pMdp->DataUsed,&dwBytesWritten,STRM_AUD)) {
              if(Err==MARGI_ERR_WRT_FAILED) {   //RdyErr
                dvHwMainClrRdyError(pAud->pHw);
                dvHwMainReportRdyError(pAud->pHw,AUDCH);
              }
            }
            if(!pHw->fUsePesHdr)
              SyncAudPktEnd(pS);
            pHw->ChInf[AUDCH].dwItems+=dwBytesWritten/8;
          }
        }
      } else
        pAud->fDataXfrPending++;
    }
    if(pMdp->OptionsFlags&MDB_FLAG_TIMEDISCONTINUITY) {
      TransferFlushPkt(pHw,FLUSH_PIPE);
    }
  }
  return(dwBytesWritten);
}

/**EOF: DVHWAUD.C**/
