//#include <StrmIni.h>
//#include <mmsystem.h>
//#include <KsMedia.h>
#include "system.h"
#include "DbgMsg.h"
#include "DbgFltr.h"
#include "Idents.h"
#include "margierr.h"
#include "IdentCls.h"

#include "MidasApi.h"
#include "lsi64020.h"
#include "dvHwMain.h"
#include "dvHwVid.h"
#include "dvHwAud.h"
#include "dvHwSub.h"
#include "dvHwOsd.h"
#include "eeprom.h"
#include "Global.h"
#include "Sync.h"
#include "AuxData.h"
#include "userdata.h"

ULONG
_cdecl
DbgPrint(
    char* Format,
    ...
    );


//#pragma message("dwHwISr.c: Optimizer disabled.")
//#pragma optimize("",off)

volatile DWORD DebugReg;    //Use this to read registers from the debugger.
volatile DWORD DebugRegVal; //Use this to write registers from the debugger.

static long TimerGetMidasStatus(PHW pHw);
static long TimerGetLsiStatus(PHW pHw);
static long dvHwTimerCheckEOS(PMIDASCMD pCmd);
static int  dwHwTimerCheckTransferFlags(PMIDASCMD pCmd);
#ifdef _DEBUG
static void TimerDebug(PHW pHw);
static void TimerCheck();
#endif

long dvHwTimer(PMIDASCMD pCmd) {
  long Err= 0;
  PHW pHw= pCmd->hMidas;
  if(pHw) {
#ifdef _DEBUG
//    TimerCheck();
#endif
    pHw->wSaveIxAddr=RdByte(pHw,REG_INDEX(pHw));
    pHw->dwIsrCntr++;
    Err=TimerGetMidasStatus(pHw);
    if(Err==0) {
      pHw->wSaveLsiAddr=dvHwMainGetLsiAddr(pHw);
      Err=TimerGetLsiStatus(pHw);
      if(Err==0) {
        AuxDataCmd(pHw,AUXCMD_TIMER,0,0);			//Check auxillary data fifo.
        VideoCmd(pHw,VIDCMD_TIMER,0,0);
        AudioCmd(pHw,AUDCMD_TIMER,0,0);
        SubpicCmd(pHw,SUBCMD_TIMER,0,0);
        SyncCmd(pHw,SYNC_TIMER,pHw->bStatusFlag[1],0);
        UserDataCmd(pHw,USERDATACMD_TIMER,0,0);
        OsdCmd(pHw,OSDCMD_TIMER,0,0);
        EEPROM_Proc(pHw,EEPROM_TIMER,0,0);
      }
      dvHwMainSetLsiAddr(pHw,pHw->wSaveLsiAddr);
    }
    WrByte(pHw,REG_INDEX(pHw),(BYTE)pHw->wSaveIxAddr);
    if(Err)
       DbgMsg(DBG_HWERR,"Timer: Hardware Err=%d\r\n",Err);
  }
  return Err;
}

long dvHwGetChnLevels(PMIDASCMD pCmd) {
  PHW pHw= pCmd->hMidas;
  DWORD dwSpace;
  if(pHw) {
    pHw->wSaveLsiAddr=dvHwMainGetLsiAddr(pHw);
    dvHwMainGetChnLevels(pHw);
    dvHwMainSetLsiAddr(pHw,pHw->wSaveLsiAddr);

    if((dwSpace=pHw->ChInf[VIDCH].dwSize-pHw->ChInf[VIDCH].dwItems) >pHw->ChInf[VIDCH].dwSafety)
      dwSpace-=pHw->ChInf[VIDCH].dwSafety;
    else
      dwSpace=0;
    pCmd->u.ChnLevels.dwVidBytes=dwSpace*8;
    
    if((dwSpace=pHw->ChInf[AUDCH].dwSize-pHw->ChInf[AUDCH].dwItems)>pHw->ChInf[AUDCH].dwSafety)
      dwSpace-=pHw->ChInf[AUDCH].dwSafety;
    else
      dwSpace=0;
    pCmd->u.ChnLevels.dwAudBytes=dwSpace*8;

    if((dwSpace=pHw->ChInf[SUBCH].dwSize-pHw->ChInf[SUBCH].dwItems)>pHw->ChInf[SUBCH].dwSafety)
      dwSpace-=pHw->ChInf[SUBCH].dwSafety;
    else
      dwSpace=0;
    pCmd->u.ChnLevels.dwSubBytes=dwSpace*8;
  }
  return 0;
}


#ifdef _DEBUG
static void TimerCheck() {
  static DWORD dwPrevTime=0;
  static DWORD dwTicks=0;
  DWORD dwTime;
  dwTime=MargiGetTime();
  if(tmCompareTimes(dwTime,dwPrevTime+11)) {
    DbgMsg(DBG_HWERR,"Timer slow %dmS, %d good\r\n",dwTime-dwPrevTime,dwTicks);
    dwTicks=0;
  } else
    dwTicks++;
  dwPrevTime=dwTime;
}

static void TimerDebug(PHW pHw) {
  DWORD RegVal;
  if(DebugReg) {
    if(DebugReg & 0x1000)
      RegVal= dvHwMainReadLsiReg32(pHw,(WORD)(DebugReg & 0x0FFF));
    else
      RegVal= dvHwMainReadLsiReg(pHw,(WORD)DebugReg);
    DebugReg&= 0x0FFF;
    DbgMsg(DBG_HWREG,"LSI Register %04X= %02X.\r\n",DebugReg,RegVal);
    if(DebugRegVal)
      dvHwMainWriteLsiReg(pHw,(WORD)DebugReg,(BYTE)DebugRegVal);
    DebugReg= 0;
    DebugRegVal= 0;
  }
}
#endif

static long TimerGetMidasStatus(PHW pHw)
{
  long lErr = 0;
  BYTE bStat;
  if(pHw->wHwError!=HW_REMOVED)
  {
    bStat=dvHwMainGetChannelStatus(pHw);
    if(bStat==0xff)
    {
      pHw->wHwError = HW_REMOVED;
      lErr = MARGI_ERR_HW_NOTFOUND;
      DbgMsg(DBG_HWERR,"Timer: Hardware Removed.\r\n");
    }
    else if(bStat&8)
    {
      pHw->wHwError = HW_RDYERR;
      lErr = MARGI_ERR_HW;
      DbgMsg(DBG_HWERR,"Timer: RdyErr asserted.\r\n");
    }
  }
  return lErr;
}

static long TimerGetLsiStatus(PHW pHw) {
  long Err= 0;
  BYTE DebugFlags= DebugWatchFlags;
  DWORD PrevDecodeRun;
  DWORD PrevChannelRun;
  pHw->u.s.PreEos= 0;
  pHw->u.s.PostEos= 0;
  pHw->u.s.SpuPes=0;
  if(!pHw->wHwError) {
    pHw->dwClkRate=pHw->dwNewClkRate;
    pHw->dlClkStartTime=pHw->dlNewStartTime;

    PrevChannelRun=pHw->u.s.ChannelRun;
    pHw->u.s.ChannelRun=dvHwMainReadLsiReg(pHw,7)&1;
    if(PrevChannelRun!=pHw->u.s.ChannelRun) {
      DbgMsg(DBG_HWSTAT,"Timer: Channel Run State Changed to %s.\r\n",pHw->u.s.ChannelRun?"Run":"Stop");
    }
    if(pHw->u.s.ChannelRun) {
      DebugWatchFlags&= ~DBGWATCH_ENABLE;
      dvHwMainReadStatusFlags(pHw);				//Read status flags from LSI.
      DebugWatchFlags= DebugFlags;
      if(pHw->bStatusFlag[2] & 0x10) {
        DbgMsg(DBG_HWSTAT,"Timer: PreEos detected.\r\n");
        pHw->u.s.PreEos= 1;
      }
      if(pHw->bStatusFlag[0] & 0x08) {
        DbgMsg(DBG_HWSTAT,"Timer: PostEos detected.\r\n");
        pHw->u.s.PostEos= 1;
      }
       
#ifdef _NEC_DBG_
      if(pHw->bStatusFlag[0] & 0x04) {
        static BYTE b=0;
        SetDbgBit(8,b^=1);
      }
#endif
      dvHwMaintainScr(pHw);

      PrevDecodeRun=pHw->u.s.VideoDecodeRun;
      pHw->u.s.VideoDecodeRun=pHw->bStatusFlag[0]&0x01;
      if(pHw->u.s.VideoDecodeRun!=PrevDecodeRun) {
        DbgMsg(DBG_HWSTAT,"Timer: Video Decode Run State Changed to %s.\r\n",pHw->u.s.VideoDecodeRun?"Run":"Stop");
      }
      pHw->ChInf[VIDCH].dwPTUs =dvHwMainReadLsiReg(pHw,PICT_IN_VID_CHAN_REG);
      dvHwMainGetChnLevels(pHw);

      if(pHw->bStatusFlag[0]&0x80) {  // if NewField Detected
        pHw->bFieldReg = (dvHwMainReadLsiReg(pHw,275)&0xf0)|(dvHwMainReadLsiReg(pHw,276)&0x01);
      }

#ifdef _DEBUG
      if(pHw->bStatusFlag[3] & 0x40) {
        volatile int i=0;
        DbgMsg(DBG_HWSTAT,"Timer: SPU channel underflow.\r\n");
      }
#ifdef TESTSPLITERROR
      {
        BYTE bErrReg4 = dvHwMainReadLsiReg(pHw,4);
        if(bErrReg4&0x6F)
        {
          volatile int i=0;
          DbgMsg(DBG_HWSTAT,"Timer: Video channel underflow.\r\n");
        }
      }
#endif
      if((pHw->bStatusFlag[3] & 0x20)) {// || (dvHwMainReadLsiReg(pHw,4)&0x43)) {
        volatile int i=0;
        DbgMsg(DBG_HWSTAT,"Timer: Video channel underflow.\r\n");
      }
      if(pHw->bStatusFlag[3] & 0x10) {
        volatile int i=0;
        DbgMsg(DBG_HWSTAT,"Timer: Audio channel underflow.\r\n");
      }
#endif
    } else
       dvHwMainResetStatus(pHw);
    dvHwMainClrStatusLatches(pHw);
  }
  return(Err);
}

void qInitQueue(PQUEUE pQ) {
  pQ->dwEntries = 0;
  pQ->fInEnQue  = FALSE;
  pQ->pHead     = NULL;
  pQ->pTail     = NULL;
}

void qEnqueue(PQUEUE pQ,PMDS_DATA_BLOCK pMdb) {
  pMdb->pNextMdb = 0;
  pMdb->dwStatus = MDS_NoStatus;

  pQ->fInEnQue = TRUE;
  pQ->dwEntries++;         // Increment first so that we can catch in qDequene
#ifdef TESTSPLITERROR
  if (pQ && (pQ->dwEntries > 7))
  {
    _asm int 3;
    DbgMsg(DBG_HWSTAT,"Queue: Queue Overflow.\r\n");
  }
#endif
  if(pQ->pHead)
    pQ->pTail->pNextMdb=pMdb; // We make sure that pQ->pHead does not pass over pQ->pTail by Increment nEntries first 
  else
    pQ->pHead=pMdb;
  pQ->pTail=pMdb;

  pQ->pTail->dwStatus = MDS_Success;
  pQ->fInEnQue = FALSE;
}

PMDS_DATA_BLOCK qDequeue(PQUEUE pQ) {
  PMDS_DATA_BLOCK pMdb=0;
  if(pQ->pHead) {
    if(pQ->pHead==pQ->pTail && pQ->fInEnQue) {  // If we are in the middle of adding
      pMdb = 0;
    } else if(pQ->pHead->dwStatus!=MDS_NoStatus) {
      pMdb=pQ->pHead;
      pQ->pHead=pQ->pHead->pNextMdb;
      pQ->dwEntries--;
    }
  }
  return(pMdb);
}

PMDS_DATA_BLOCK qPeekqueue(PQUEUE pQ) {
  PMDS_DATA_BLOCK pMdb=0;
  if(pQ->pHead) {
    if(pQ->pHead==pQ->pTail && pQ->fInEnQue) {  // If we are in the middle of adding
      pMdb = 0;
    } else if(pQ->pHead->dwStatus!=MDS_NoStatus) {
      pMdb=pQ->pHead;
    }
  }
  return(pMdb);
}

PMDS_DATA_BLOCK qFindAndDequeue(PQUEUE pQ,PMDS_DATA_BLOCK pMdb) {
  PMDS_DATA_BLOCK p;
  PMDS_DATA_BLOCK pPrev;

  p=pQ->pHead;
  pPrev= 0;
  while(p && p!=pMdb) {
    pPrev=p;
    p=p->pNextMdb;
  }
  if(p) {
    if(pPrev) {
      pPrev->pNextMdb=p->pNextMdb;
    } else {
      pQ->pHead=p->pNextMdb;
    }
    if(p==pQ->pTail)
      pQ->pTail=pPrev;
    pQ->dwEntries--;
  }
  return p;
}

void qPurgeQueue(PQUEUE pQ,DWORD dwStatus) {
  PMDS_DATA_BLOCK pMdb=0;
  if(pQ) {
    while(pMdb=qDequeue(pQ)) {
      pMdb->dwStatus=dwStatus;
      if(pMdb->pfnCallback) {
        PMDB_CALLBACK pfnCallback = (PMDB_CALLBACK)pMdb->pfnCallback;
         pfnCallback(pMdb);
       }
    }
  }
}

long dvHwTimerPumpData(PMIDASCMD pCmd) {
  long Err= 0;
  PHW pHw= pCmd->hMidas;
  int fAudEnable=1;
  int fVidEnable=1;
  int fSubEnable=1;

  pHw->u.s.fInPumpData=1;
  while (fAudEnable || fVidEnable || fSubEnable) {
    DWORD dwBytes;
    if(fAudEnable) {
      dwBytes=AudioPumpData(pHw);
      if(!dwBytes)
        fAudEnable=0;
    }
    if(fVidEnable) {
      dwBytes=VideoPumpData(pHw);
      if(!dwBytes)
        fVidEnable=0;
    }
    if(fSubEnable) {
      dwBytes=SubpicPumpData(pHw);
      if(!dwBytes)
        fSubEnable=0;
    }
  }
  pHw->u.s.fInPumpData=0;
  return Err;
}

/**EOF: DVHWISR.C**/
