//#include <StrmIni.h>
//#include <mmsystem.h>
//#include <ksmedia.h>
#include "system.h"
#include "DbgMsg.h"
#include "DbgFltr.h"
#include "MidasApi.h"
#include "margierr.h"
#include "settings.h"
#include "deviceid.h"
#include "Idents.h"

#include "Lsi64020.h"

#include "ininames.h"

#include "dvHwMain.h"
#include "Global.h"
#include "dvHwVid.h"
#include "dvHwAud.h"
#include "dvHwSub.h"
#include "Sync.h"
#include "AuxData.h"
#include "TvOut.h"
#include "eeprom.h"

#define ASIC_MIDAS_REV0    0
#define ASIC_MIDAS_REV1    1
#define ASIC_JUNO_REV2     2

#define DRAM_BUFFER_SIZE (2L*1024*1024)  // LSI 64020 Dram is 2MBytes

#define BOARD_CAP_FLAGS_VALID_MASK  0xfff00000
#define BOARD_CAP_FLAGS_VALID       0xcaf00000
#define BOARD_CAPS_USE_MEM_XFR      0x000000001

#define OSD_STOP_MAX_BYTES_PER_TICK  3000
#define OSD_RUN_MAX_BYTES_PER_TICK   1000 /*80*/

#ifdef _NEC_DBG_
int   TimeTableIx=0;
DWORD TimeTable[1024]={0};
#endif

static WORD gw5446RegisterTable[]= {
  REG_5446_INDEX,
  REG_5446_DATA,
  REG_5446_AUDIO_CHANNEL,
  REG_5446_VIDEO_CHANNEL,
  REG_5446_LSI_INDEXL,
  REG_5446_LSI_INDEXH,
  REG_5446_LSI_DATA
};

static WORD gw5480RegisterTable[]= {
  REG_ISA_INDEX,          // 5480 and ISA use same offsets
  REG_ISA_DATA,
  REG_ISA_AUDIO_CHANNEL,
  REG_ISA_VIDEO_CHANNEL,
  REG_ISA_LSI_INDEXL,
  REG_ISA_LSI_INDEXH,
  REG_ISA_LSI_DATA
};

static WORD gw5465RegisterTable[]= {
  REG_ISA_INDEX,          // 5465 and ISA use same offsets
  REG_ISA_DATA,
  REG_ISA_AUDIO_CHANNEL,
  REG_ISA_VIDEO_CHANNEL,
  REG_ISA_LSI_INDEXL,
  REG_ISA_LSI_INDEXH,
  REG_ISA_LSI_DATA
};                             

static WORD gwIsaRegisterTable[]= {
  REG_ISA_INDEX,
  REG_ISA_DATA,
  REG_ISA_AUDIO_CHANNEL,
  REG_ISA_VIDEO_CHANNEL,
  REG_ISA_LSI_INDEXL,
  REG_ISA_LSI_INDEXH,
  REG_ISA_LSI_DATA
};

LOCAL long dvHwMainTestHardware(PHW pHw);
LOCAL void dvHwMainResetLsi(PHW pHw);

LOCAL int   dvHwMainTestLsiIxReg(PHW pHw);
LOCAL int   dvHwMainTestLsiRegs(PHW pHw);
LOCAL void  dvHwMainReportScrFreq(PHW pHw);
LOCAL void  dvHwMainCheckVCO(PHW pHw);
LOCAL DWORD dvHwMainAllocateFrameBuffer(PHW pHw,WORD wReg,DWORD dwAddr,DWORD dwSize);
LOCAL DWORD dvHwMainAllocateChannelBuffer(PHW pHw,WORD wReg,DWORD dwAddr,DWORD dwSize);
LOCAL long  dvHwMainSetCssCmd(PHW pHw,WORD wCmd);
LOCAL long  dvHwMainSendCssCmd(PHW pHw,WORD wCmd);
LOCAL long  dvHwMainSetAVOffset(PHW pHw,DWORD dwVidPTS,DWORD dwAudPTS);
LOCAL long  dvHwMainResetChannel(PHW pHw);
LOCAL long  dvHwMainReadDramWord(PHW pHw,BYTE *pData);
LOCAL int   dvHwMainPauseScrReg(PHW pHw,int fPause);

DWORD DebugWatch[DBGWATCH_MAXENTRIES]= { 0xDEADBEEF };  //Set this from debugger to trap writes to a particular register.
BYTE DebugWatchFlags;    //Bit0: Enable
        //Bit1: Reads
        //Bit2: Writes
        //Bit3: INT3
        //Bit4: PassThroughOnce
#if(0)
void WatchBreak(PHW pHw, DWORD Reg, DWORD IsWrite, DWORD Val) {
  DWORD n1,IsBreak=0;
  char *Action;
  if(DebugWatchFlags & DBGWATCH_ENABLE) {
    if(DebugWatchFlags & DBGWATCH_PASSTHRU) {
      DebugWatchFlags&= ~DBGWATCH_PASSTHRU;
    } else {
      if(IsWrite) {
  if(DebugWatchFlags & DBGWATCH_WRITE) {
    for(n1=0;!IsBreak && n1<DBGWATCH_MAXENTRIES;n1++) {
      if(DebugWatch[n1]==Reg) {
        Action= "WatchBreak: Writing %02X to register %u.\r\n";
        IsBreak= 1;
      }
    }
  }
      } else {
  if(DebugWatchFlags & DBGWATCH_READ) {
    for(n1=0;!IsBreak && n1<DBGWATCH_MAXENTRIES;n1++) {
      if(DebugWatch[n1]==Reg) {
        Action= "WatchBreak: Reading %02X from register %u.\r\n";
        IsBreak= 1;
      }
    }
  }
      }
    }
    if(IsBreak) {
      DbgMsg(DBG_HWREG,Action,Val,Reg);
      if(DBGWATCH_INT3) {
        _asm {
          mov  eax,[IsWrite]
          shl  eax,16
          mov  ax,WORD PTR[Reg]
          mov  ebx,[Val]
          INT  3
        }
      }
    }
  }
}
#else
#define WatchBreak(a,b,c,d)
#endif


#ifdef _X86_
// Default I/O and Memory access routines for X86 compile
LOCAL void __stdcall IoRegWrByte(DWORD dwIoAddr,BYTE bDat) {
  _outp((WORD)dwIoAddr,bDat);
}

LOCAL void __stdcall IoRegWrWord(DWORD dwIoAddr,WORD wDat) {
  _outpw((WORD)dwIoAddr,wDat);
}

LOCAL BYTE __stdcall IoRegRdByte(DWORD dwIoAddr) {
  return (BYTE)_inp((WORD)dwIoAddr);
}

LOCAL WORD __stdcall IoRegRdWord(DWORD dwIoAddr) {
  return _inpw((WORD)dwIoAddr);
}

LOCAL DWORD __stdcall IoChnWrWords(DWORD dwIoAddr,BYTE* pDat,DWORD dwWords,DWORD dwFlags) {
  if((dwFlags&~MDS_XFR_FLAGS)==0)
    return 0;
  _asm {
    push esi
    mov edx,[dwIoAddr]
    mov esi,[pDat]
    mov ecx,[dwWords]
    shr ecx,1
    jz skipsd
    rep outsd
skipsd:
    mov eax,[dwWords]
    and eax,1
    jz  skipw
    outsw
skipw:
    mov [pDat],esi
    pop esi
  }
  return dwWords;
}

LOCAL DWORD __stdcall MemChnWrWords(DWORD dwMemAddr,BYTE* pDat,DWORD dwWords,DWORD dwFlags) {
  if((dwFlags&~MDS_XFR_FLAGS)==0)
    return 0;
  _asm {
    push esi
    push edi
    mov esi,[pDat]
    mov edi,[dwMemAddr]
    mov ecx,[dwWords]
    shr ecx,1
    jz skipsd
xfrloop:
    movsd
    sub edi,4
    dec ecx
    jnz xfrloop
skipsd:
    mov eax,[dwWords]
    and eax,1
    jz  skipw
    movsw
skipw:
    mov [pDat],esi
    pop edi
    pop esi
  }
  return dwWords;
}
#elif _NEC_
// Stub I/O routines in Non-X86 compile
LOCAL void __stdcall IoRegWrByte(DWORD dwIoAddr,BYTE bDat) {*((BYTE*)dwIoAddr)=(BYTE)bDat; }
LOCAL void __stdcall IoRegWrWord(DWORD dwIoAddr,WORD wDat) {*((WORD*)dwIoAddr)=(WORD)wDat; }
LOCAL BYTE __stdcall IoRegRdByte(DWORD dwIoAddr)   {return (BYTE)(*((BYTE*)dwIoAddr)); }
LOCAL WORD __stdcall IoRegRdWord(DWORD dwIoAddr)   {return (WORD)(*((WORD*)dwIoAddr));}
LOCAL DWORD __stdcall IoChnWrWords(DWORD dwIoAddr,BYTE* pDat,DWORD dwWords)
#if(1)
{
  asm( "  cmp r0, r8");
  asm( "  bz _ZeroWords");
  asm( "  addi -8,r3,r3");    // reserve 2 words on stack
  asm( "  st.w r8,0[r3]");    // save dwWords
  asm( "  st.w r26,4[r3]");   // save r26
  asm( "_Loop1:");
  asm( "  ld.h 0[r7],r26");   // r26= *pDat
  asm( "  st.h r26, 0[r6]");  // *io= r26
  asm( "  addi 2,r7,r7");     // pDat+=2;
  asm( "  addi -1,r8,r8");    // dwWords--;
  asm( "  cmp r0,r8");
  asm( "  bnz _Loop1");       // loop if not done
  asm( "  ld.w 4[r3],r26");   
  asm( "  ld.w 0[r3],r8");
  asm( "  addi 8,r3,r3");
  asm( "_ZeroWords:");
  asm( "  mov r8,r10");
}
#else
// do transfer with DWORDS
{
  asm( "  cmp r0, r8");
  asm( "  bz _ZeroWords");
  asm( "  addi -c,r3,r3");    // reserve 2 words on stack
  asm( "  st.w r8,0[r3]");    // save dwWords
  asm( "  st.w r26,4[r3]");   // save r26
  asm( "  st.w r27,8[r3]");   // save r27
  asm( "  mov 1,r27");        // r27 = 1
  asm( "_Loop1:");
  asm( "  ld.w 0[r7],r26");   // r26= *pDat
  asm( "  st.w r26, 0[r6]");  // *io= r26
  asm( "  addi 4,r7,r7");     // pDat+=2;
  asm( "  addi -2,r8,r8");    // dwWords-=2;
  asm( "  cmp r27,r8");       // check to see if count >1
  asm( "  bh _Loop1");        // loop if count greater than 1
  asm( "  cmp r0,r8");        // check if equal to 0
  asm( "  bz _Loop2");        // Jump to Loop2 else send the last 16bits
  asm( "  ld.h 0[r7],r26");   // r26= *pDat
  asm( "  st.h r26, 0[r6]");  // *io= r26
  asm( "  addi 2,r7,r7");     // pDat+=2;
  asm( "  addi -1,r8,r8");    // dwWords--;
  asm( "_Loop2:");
  asm( "  ld.w 8[r3],r27");   
  asm( "  ld.w 4[r3],r26");   
  asm( "  ld.w 0[r3],r8");
  asm( "  addi c,r3,r3");
  asm( "_ZeroWords:");
  asm( "  mov r8,r10");
}
#endif
LOCAL DWORD __stdcall MemChnWrWords(DWORD dwMemAddr,BYTE* pDat,DWORD dwWords){return 0;}

#elif _WIN32_WCE
// Stub I/O routines in WINCE compile
LOCAL void __stdcall IoRegWrByte(DWORD dwIoAddr,BYTE bDat) {*((BYTE*)dwIoAddr)=(BYTE)bDat;}
LOCAL void __stdcall IoRegWrWord(DWORD dwIoAddr,WORD wDat) {*((WORD*)dwIoAddr)=(WORD)wDat; }
LOCAL BYTE __stdcall IoRegRdByte(DWORD dwIoAddr)   {return (BYTE)(*((BYTE*)dwIoAddr)); }
LOCAL WORD __stdcall IoRegRdWord(DWORD dwIoAddr)   {return (WORD)(*((WORD*)dwIoAddr));}
// needs to be fixed
LOCAL DWORD __stdcall MemChnWrWords(DWORD dwMemAddr,BYTE* pDat,DWORD dwWords,DWORD dwFlags) {return 0;}
LOCAL DWORD __stdcall IoChnWrWords(DWORD dwIoAddr,BYTE* pDat,DWORD dwWords, DWORD dwFlags)
{
  int i;
  PWORD pwData = (WORD*)pDat;
  i=dwWords;
  if(i<16)
    goto Loop1;
  while(i>16)
  {
    *((WORD*)dwIoAddr) = *pwData++;
    *((WORD*)dwIoAddr) = *pwData++;
    *((WORD*)dwIoAddr) = *pwData++;
    *((WORD*)dwIoAddr) = *pwData++;
    *((WORD*)dwIoAddr) = *pwData++;
    *((WORD*)dwIoAddr) = *pwData++;
    *((WORD*)dwIoAddr) = *pwData++;
    *((WORD*)dwIoAddr) = *pwData++;
    *((WORD*)dwIoAddr) = *pwData++;
    *((WORD*)dwIoAddr) = *pwData++;
    *((WORD*)dwIoAddr) = *pwData++;
    *((WORD*)dwIoAddr) = *pwData++;
    *((WORD*)dwIoAddr) = *pwData++;
    *((WORD*)dwIoAddr) = *pwData++;
    *((WORD*)dwIoAddr) = *pwData++;
    *((WORD*)dwIoAddr) = *pwData++;
    *((WORD*)dwIoAddr) = *pwData++;
    i-=16;
  }
Loop1:
  *((WORD*)dwIoAddr) = *pwData++;
  i--;
  if(i)
    goto Loop1;

  return dwWords;
}

#else // for non X86 and non NEC
LOCAL void __stdcall IoRegWrByte(DWORD dwIoAddr,BYTE bDat) {}
LOCAL void __stdcall IoRegWrWord(DWORD dwIoAddr,WORD wDat) {}
LOCAL BYTE __stdcall IoRegRdByte(DWORD dwIoAddr) {return 0;}
LOCAL WORD __stdcall IoRegRdWord(DWORD dwIoAddr) {return 0;}
LOCAL DWORD __stdcall IoChnWrWords(DWORD dwIoAddr,BYTE* pDat,DWORD dwWords) {return 0;}
LOCAL DWORD __stdcall MemChnWrWords(DWORD dwMemAddr,BYTE* pDat,DWORD dwWords) {return 0;}


#endif

/************************************************************************
 **  Read Write Midas Registers
 ************************************************************************/
BYTE RdByte(PHW pHw,WORD wReg) {
  if(!pHw->wHwError)
    return (BYTE)pHw->pfnRegRdWord(pHw->dwIoAddr+wReg);
  else
    return 0;
}

void WrByte(PHW pHw,WORD wReg,BYTE Data) {
  if(!pHw->wHwError)
    pHw->pfnRegWrWord(pHw->dwIoAddr+wReg,(WORD)Data);
}

LOCAL WORD RdWord(PHW pHw,WORD wReg) {
  if(!pHw->wHwError)
    return pHw->pfnRegRdWord(pHw->dwIoAddr+wReg);
  else
    return 0;
}

void WrWord(PHW pHw,WORD wReg,WORD Data) {
  if(!pHw->wHwError)
    pHw->pfnRegWrWord(pHw->dwIoAddr+wReg,(WORD)Data);
}

/************************************************************************
 **  dvHwMainSetGPIO                                                   **
 ************************************************************************/
void dvHwMainSetGPIO(PVOID pRef,BYTE bPinNumber,BYTE bEnable,BYTE bValue)
{
  PHW pHw=(PHW)pRef;
  BYTE bPin = (0x01<<bPinNumber);

  pHw->bGpioCtrl = (bEnable ? pHw->bGpioCtrl|bPin : pHw->bGpioCtrl&(~bPin));
  WrByte(pHw, REG_INDEX(pHw), RIX_GPIO_CTRL);
  WrByte(pHw, REG_DATA(pHw), pHw->bGpioCtrl);

  pHw->bGpioPin = (bValue ? pHw->bGpioPin|bPin : pHw->bGpioPin&(~bPin));
  WrByte(pHw, REG_INDEX(pHw), RIX_GPIO_PINS);
  WrByte(pHw, REG_DATA(pHw), pHw->bGpioPin);
}

/************************************************************************
 **  dvHwMainGetGPIO                                                   **
 ************************************************************************/
BYTE dvHwMainGetGPIO(PVOID pRef, BYTE bPinNumber, LPBYTE lpEnable, LPBYTE lpValue)
{
  PHW pHw = (PHW) pRef;
  BYTE bPin = (0x01<<bPinNumber);
  BYTE bEnable;
  BYTE bValue;

  bEnable = ((pHw->bGpioCtrl&bPin)!=0);
  WrByte(pHw, REG_INDEX(pHw), RIX_GPIO_PINS);
  bValue = (RdByte(pHw, REG_DATA(pHw))&bPin)?1:0;
  pHw->bGpioPin = (bValue ? pHw->bGpioPin|bPin : pHw->bGpioPin&(~bPin));

  if(lpEnable)
    *lpEnable = bEnable;
  if(lpValue)
    *lpValue = bValue;

  return bValue;
}

/************************************************************************
 **  HwMidasResetGPIO                                                  **
 ************************************************************************/
void dvHwMainResetGPIO(PHW pHw)
{
  pHw->bGpioCtrl = 0;
  WrByte(pHw, REG_INDEX(pHw), RIX_GPIO_CTRL);
  WrByte(pHw, REG_DATA(pHw), pHw->bGpioCtrl);

  pHw->bGpioPin = 0;
  WrByte(pHw, REG_INDEX(pHw), RIX_GPIO_PINS);
  WrByte(pHw, REG_DATA(pHw), pHw->bGpioPin);
}

long dvHwMainConstruct(PHW pHw,PMDS_HW_RESOURCE pRsrc) {
  DWORD dwIoAddr=0;
  DWORD dwMemAddr=0;
  DWORD dwDMAAddr=0;
  DWORD dwChnAddr=0;
  long Err= 0;
  if(ISBADPTR(pHw,sizeof(DVHW))) {
    DbgMsg(DBG_HWERR,"Bad ptr for dvHwMainConstruct().\r\n");
    Err=MARGI_ERR_ARG1;
  } else if((pRsrc->dwFlags&3)==0) {
    DbgMsg(DBG_HWERR,"Harware location not specified.\r\n");
    Err=MARGI_ERR_HW_NOTFOUND;
  } else {
    memset(pHw,0,sizeof(DVHW));
    pHw->dwSignature= DVHW_SIGNATURE;
    if(pRsrc->dwFlags&1) {
      // Setup I/O base address - this must be present and is used for reg i/o
      dwIoAddr = (DWORD)GetLow_MdsLongLong(&pRsrc->dlIOBaseAddr);
      dwChnAddr=dwIoAddr+4; // channel xfr uses i/o this may be changed to mem in dvHwMainDetectPcmcia
      pHw->fUseDMA=0;
    }     
    if(pRsrc->dwFlags&4) {
      // Setup DMA resource for channel data - dlMemBaseAddr is used as reference for DMA Handler
      dwDMAAddr=(DWORD)GetLow_MdsLongLong(&pRsrc->dlDMABaseAddr);
      dwChnAddr=dwDMAAddr; // channel xfr uses i/o this may be changed to mem in dvHwMainDetectPcmcia
      pHw->fUseDMA=1;
    }
    else if(pRsrc->dwFlags&2) {
      // Setup Mem Address for channel data
      dwMemAddr=(DWORD)GetLow_MdsLongLong(&pRsrc->dlMemBaseAddr);
      dwChnAddr=dwMemAddr+8; // channel xfr uses i/o this may be changed to mem in dvHwMainDetectPcmcia
      pHw->fUseDMA=0;
    }

    if(pHw->fUseDMA)
      pHw->fUsePesHdr=1;
    else
      pHw->fUsePesHdr=0;

    pHw->fUsePesHdr=1;

    pHw->dwIoAddr=  dwIoAddr;
    pHw->dwMemAddr= dwMemAddr;
    pHw->dwDMAAddr= dwDMAAddr;
    pHw->dwChnAddr= dwChnAddr; 
    pHw->fDMAAvailable = pRsrc->fDMAAvailable;
    pHw->ChInf[VIDCH].wWrReg=VID_CHAN_WRITE_ADDR_REG;
    pHw->ChInf[VIDCH].wRdReg=PICT_START_CODE_READ_ADDR_REG;
    pHw->ChInf[AUDCH].wWrReg=AUD_CHAN_WRITE_ADDR_REG;
    pHw->ChInf[AUDCH].wRdReg=AUD_SYNC_CODE_READ_ADDR_REG;
    pHw->ChInf[SUBCH].wWrReg=SUB_CHAN_WRITE_ADDR_REG;
    
    if(pRsrc->pfnRegWrByte)
      pHw->pfnRegWrByte=pRsrc->pfnRegWrByte;
    else
      pHw->pfnRegWrByte= IoRegWrByte;

    if(pRsrc->pfnRegWrWord)
      pHw->pfnRegWrWord=pRsrc->pfnRegWrWord;
    else
      pHw->pfnRegWrWord= IoRegWrWord;
    
    if(pRsrc->pfnRegRdByte)
      pHw->pfnRegRdByte=pRsrc->pfnRegRdByte;
    else
      pHw->pfnRegRdByte= IoRegRdByte;
    
    if(pRsrc->pfnRegRdWord)
      pHw->pfnRegRdWord=pRsrc->pfnRegRdWord;
    else
      pHw->pfnRegRdWord= IoRegRdWord;

    if(pRsrc->pfnChnWrWords)
      pHw->pfnChnWrWords=pRsrc->pfnChnWrWords;
    else
      pHw->pfnChnWrWords=0;  // setup the default ch wr when we know capabilities
  }
  return(Err);
}

long dvHwMainDestruct(PHW pHw) {
  long Err= 0;
  if(pHw) { 
    VideoCmd(pHw,VIDCMD_ENABLE,0,0);      //Disable video
    memset(pHw,0,sizeof(DVHW));
  }
  return(Err);
}

/*************************************************************************/
/** long dvHwMainDetect(pHw)                                                **/
/** --  Detects any Midas boards.                                       **/
/** --  Returns an error code.                                          **/
/*************************************************************************/
LOCAL DWORD gDevIoPhysical,gDevIoLinear;
LOCAL long dvHwMainDetectIsa(PHW pHw);
LOCAL long dvHwMainDetectPci(PHW pHw);
LOCAL long dvHwMainDetectPcmcia(PHW pHw);

long dvHwMainDetect(PHW pHw) {
  WORD wRegId=0;         
  WORD wIsaIoAddr=0x02d0;
  long Err;
  pHw->BoardID =0;
  pHw->wHwError=0;
  if(fSWOnly) {
    Err= 0;
    DbgMsg(DBG_HWINIT,"Detect Hardware ... Detected SWOnly \r\n");
  } else {
    if(!(Err= dvHwMainDetectPcmcia(pHw))) {
      DbgMsg(DBG_HWINIT,"PCMCIA Card is Detected\r\n");
    } else {
      DbgMsg(DBG_HWERR,"No Hardware Found\r\n");
    }
    if(!Err) {
      WrByte(pHw,REG_INDEX(pHw),RIX_BOARD_ID);
      pHw->BoardID= RdByte(pHw,REG_DATA(pHw));
    }
    //If ChnWr not setup then use I/O
    if(!pHw->pfnChnWrWords)
      pHw->pfnChnWrWords=IoChnWrWords;

  }
  return(Err);
}

LOCAL long dvHwMainDetectPcmcia(PHW pHw) {
  long Err= MARGI_ERR_NODEVICE;
  DWORD dwIoAddr;
  WORD wMemOffset;
  BYTE bId=0;
  BYTE bMode=0;
  DWORD dwCaps;
  if(!pHw->dwIoAddr)
    dwIoAddr= SettingLoad(SET_DWORD,IniPcmciaIoAddr,0,0);
  else 
    dwIoAddr= pHw->dwIoAddr;
  pHw->pfnRegWrWord(pHw->dwIoAddr,RIX_BOARD_ID);
  bId=(BYTE)pHw->pfnRegRdWord(dwIoAddr+2);
  pHw->wAsicType=0;
  if(bId==0xDD) {
    pHw->wIoType= IOTYPE_IO;
    pHw->dwIoAddr= dwIoAddr;
    pHw->BoardID= 0xDD;
    pHw->pRegTbl= gwIsaRegisterTable;
    pHw->u.s.f8BitZV= 0;
    pHw->dwBoardCaps=0;
    Err=EEPROM_Proc(pHw,EEPROM_READ_CAPS,(DWORD)&dwCaps,0);
    if(Err==0) {
      if((dwCaps&BOARD_CAP_FLAGS_VALID_MASK)==BOARD_CAP_FLAGS_VALID)
        pHw->dwBoardCaps=dwCaps&~BOARD_CAP_FLAGS_VALID_MASK;
    }
    pHw->pfnRegWrWord(pHw->dwIoAddr,RIX_MODE);
    bMode=(BYTE)pHw->pfnRegRdWord(pHw->dwIoAddr+2);
    if((bMode&0x38)==0x00) {
      pHw->wAsicType=ASIC_MIDAS_REV0;
      pHw->dwBoardCaps&=0x00000000;
      DbgMsg(DBG_INFO,"Asic Midas Rev 0 detected. Cap Flags=%08x\r\n",pHw->dwBoardCaps);
    } else if((bMode&0x38)==0x10) {
      pHw->wAsicType=ASIC_JUNO_REV2;
      pHw->dwBoardCaps|=BOARD_CAPS_USE_MEM_XFR;
      wMemOffset=0x0008;
      DbgMsg(DBG_INFO,"Asic Juno Rev 0 detected. Cap Flags=%08x\r\n",pHw->dwBoardCaps);
    } else {
      pHw->wAsicType=ASIC_MIDAS_REV1;
      wMemOffset=0x000C;
      DbgMsg(DBG_INFO,"Asic Midas Rev 1 detected. Cap Flags=%08x\r\n",pHw->dwBoardCaps);
    }
    if(pHw->dwBoardCaps&BOARD_CAPS_USE_MEM_XFR && pHw->dwMemAddr) {
      pHw->dwChnAddr=pHw->dwMemAddr+wMemOffset;
      // If ChnWr not setup then use mem xfr
      if(!pHw->pfnChnWrWords)
        pHw->pfnChnWrWords=MemChnWrWords;
    }
    Err= 0;
  }
  return(Err);
}

#if(0)
LOCAL long dvHwMainDetectIsa(PHW pHw) {
  long Err= MARGI_ERR_NODEVICE;
  WORD wIsaIoAddr;
  BYTE BoardID;
  wIsaIoAddr= (WORD)SettingLoad(SET_DWORD,IniIsaIoAddr,0,0x02D0);
  _outpw(wIsaIoAddr,RIX_BOARD_ID);
  BoardID= (BYTE)_inp((WORD)(wIsaIoAddr+2));
  if(BoardID==0xDD) {
    pHw->wIoType= IOTYPE_IO;
    pHw->dwIoAddr= wIsaIoAddr & 0xFFFE;
    pHw->BoardID= BoardID;
    pHw->pRegTbl= gwIsaRegisterTable;
    pHw->f8BitZV= 0;
  }
  return(Err);
}

LOCAL long dvHwMainDetectPci(PHW pHw) {
  long Err= MARGI_ERR_NODEVICE;

#ifdef _WIN98_ONLY_

  DWORD Device,PciDeviceID;
  PCI_COMMON_CONFIG PciConfig;
  PHYSICAL_ADDRESS Addr;
  if(!gDevIoPhysical) {
    gDevIoLinear= 0;
    if(gDevIoPhysical= SettingLoad(SET_DWORD,IniPciVgaPhysAddr,0,0)) {
      DbgMsg(DBG_HWINIT,"dvHwMainDetectPci: Using VGA physical address override at %08X.\r\n",gDevIoPhysical);
    } else {
      PciDeviceID= SettingLoad(SET_DWORD,IniPciVgaID,0,DEVICE_5465);
      DbgMsg(DBG_HWINIT,"dvHwMainDetectPci: Searching for VGA device %s.\r\n",Ident(ID_PCIDEV,PciDeviceID));
      if(!(Device= aPciDeviceFind(PciDeviceID))) {
        Err= MARGI_ERR_NODEVICE;
      } else {
        DbgMsg(DBG_HWINIT,"dvHwMainDetectPci: PCI device %s found: %08X\r\n",Ident(ID_PCIDEV,PciDeviceID),Device);
        aPciReadData(Device,&PciConfig,sizeof(PciConfig));
        gDevIoPhysical= PciConfig.u.type0.BaseAddresses[1];
      }
    }
  }
  if(gDevIoPhysical) {
    Addr.HighPart= 0;
    Addr.LowPart= gDevIoPhysical;
    if(!(gDevIoLinear= (DWORD)MmMapIoSpace(Addr,0x10000,0)+0x180)) {
      DbgMsg(DBG_HWERR,"dvHwMainDetectPci: Unable to map physical(%08X) to linear address.\r\n",gDevIoPhysical);
      gDevIoPhysical= 0;
      Err= MARGI_ERR_PHYSMAP;
    }
  }
  if(!Err) {
    pHw->wIoType= IOTYPE_MEMORYMAPPED;
    pHw->dwIoAddr= gDevIoLinear;
    pHw->BoardID= 0xDD;             //Really should read the device ID byte.
    pHw->pRegTbl= gw5465RegisterTable;
    pHw->f8BitZV= 1;
    DbgMsg(DBG_HWINIT,"dvHwMainDetectPci: Memory-mapped Midas hardware found.\r\n");
    DbgMsg(DBG_HWINIT,"               Physical address= %08X.\r\n",gDevIoPhysical);
    DbgMsg(DBG_HWINIT,"               Linear   address= %08X.\r\n",gDevIoLinear);
  }

#endif // _WIN98_ONLY_

  return(Err);
}
#endif

long dvHwMainDeviceInit(PHW pHw) {
  long Err= 0;
  WORD nCount;
  TVOUTINIT_S TvOutInit;
  if(fSWOnly) {
  } else {
    if(!pHw->u.s.fInitDone) {
      DbgMsg(DBG_HWINIT,"Initializing Midas hardware...\r\n");
      dvHwMainResetLsi(pHw);
      VideoCmd(pHw,VIDCMD_INIT,0,0);
      AudioCmd(pHw,AUDCMD_INIT,0,0);
      SubpicCmd(pHw,SUBCMD_INIT,0,0);
      SyncCmd(pHw,SYNC_INIT,0,0);
      AuxDataCmd(pHw,AUXCMD_INIT,0,0);

      TvOutInit.pSetGPIO=(PSETGPIO)dvHwMainSetGPIO;
      TvOutInit.pGetGPIO=(PGETGPIO)dvHwMainGetGPIO;
      TvOutInit.hGPIORef=(LPVOID)pHw;
      TvOutInit.hTvOut  =0;
      if(TVOutProc(0,TVOUT_INIT,(DWORD)&TvOutInit,0)==0) {
        pHw->hTvOut=TvOutInit.hTvOut;
        TVOutProc(pHw->hTvOut,TVOUT_READ_CONFIG,0,0);
      }

      if(Err= dvHwMainTestHardware(pHw)) {
        DbgMsg(DBG_HWERR,"DVD decoder test failed: %s.\r\n",Ident(ID_ERRORS,Err));
      } else {
        pHw->wHwError=0;
        dvHwMainClrRdyError(pHw);
        for(nCount=1;nCount<=MAX_LSIREG;nCount++)
          dvHwMainReadLsiReg(pHw,nCount);
        dvHwMainWriteLsiRegBits(pHw,STREAM_SELECT_REG,STREAM_SELECT_PRG|PARALLEL_CHAN);
        dvHwMainWriteLsiReg(pHw,PRIVATE_STREAM_REG,0x00);
        dvHwMainWriteLsiReg(pHw,AUDIO_STREAM_REG,0x00);  //For Audio
        dvHwMainWriteLsiRegBits(pHw,TOP_OF_SECTOR_REG,FALSE);     // Disable Top of Sector
        dvHwMainWriteLsiReg(pHw,68,0x00);   // Clear reset buff on error
        dvHwMainStopDecoderChannel(pHw);
        dvHwMainResetChannel(pHw);
        dvHwMainStartDecoderChannel(pHw);
        RegScrPause(pHw,FLAG_SCR_PAUSE|FLAG_SCR_RESET);
        pHw->dwScrLo=0;
        pHw->dwScrHi=0;
        dvHwMainWriteLsiRegBits(pHw,239,0x08,0x08); //Reset User Data
//        VideoCmd(pHw,VIDCMD_DISPFORMAT_,NORMAL_RECT,0);
        pHw->fCssTitleKeySet=0;
        pHw->fCssApplyingTitleKey=0;
        pHw->fCssWaitForTitleKey=0;
        pHw->u.s.fInitDone= 1;
//        pHw->dwOsdRequestedSize=80000;
      }
    }
  }
  return(Err);
}

long dvHwMainDeviceShutdown(PHW pHw) {
  long Err= 0;
  if(pHw->u.s.fInitDone) {
    DbgMsg(DBG_HWINIT,"Shutting down Midas hardware...\r\n");
    VideoCmd(pHw,VIDCMD_ENABLE,0,0);      //Disable video
 
    SyncCmd(pHw,SYNC_FINI,0,0);
    TVOutProc(pHw->hTvOut,TVOUT_WRITE_CONFIG,0,0);
    TVOutProc(pHw->hTvOut,TVOUT_FINI,0,0);
    AuxDataCmd(pHw,AUXCMD_FINI,0,0);
    SubpicCmd(pHw,SUBCMD_FINI,0,0);
    AudioCmd(pHw,AUDCMD_FINI,0,0);
    VideoCmd(pHw,VIDCMD_FINI,0,0);
    pHw->u.s.fInitDone= 0;
  }
  return(Err);
}

LOCAL long dvHwMainTestHardware(PHW pHw) {
  long Err=0;
  if(fSWOnly) {
    Err= 0;
  } else {
    if(Err= dvHwMainTestLsiIxReg(pHw))
      Err= dvHwMainTestLsiRegs(pHw);
//    dvHwMainReportScrFreq(pHw);
    dvHwMainCheckVCO(pHw);
  }
  return(Err);
}

LOCAL void dvHwMainResetLsi(PHW pHw) { 
  DWORD dwTime;
  dwTime=MargiGetTime();
  //dvHw.wLsiAddr=0xffff;
  WrByte(pHw,REG_INDEX(pHw),(BYTE)RIX_LSI_CNTRL);
  // Reset 020, Reset Ready
  WrByte(pHw,REG_DATA(pHw),0x20);
  //if(gdwResetTime)
//  while(Margi_timeGetTime()<dwTime+RESET_HOLD_TIME);    // wait for clock to stablize if PnP power manage is on
  // Release 020 reset
  WrByte(pHw,REG_DATA(pHw),0x17);
  // Set AClk to 12.288MHz,ZV formatter on, Turn SCLK off
  WrByte(pHw,REG_INDEX(pHw),(BYTE)RIX_OSC_CNTRL);
  WrByte(pHw,REG_DATA(pHw),(BYTE)(pHw->fSClkOff?0x30:0x10));
  dvHwMainWriteLsiRegBits(pHw,DIVIDER_SEL_REG,0x04);  // Set Audio divier for 256 Fs
  dvHwMainWriteLsiReg(pHw,363,(BYTE)0x01);   // Select non-i2s, 32 bit
  // Enable HSync, VSync Generator
  WrByte(pHw,REG_INDEX(pHw),(BYTE)RIX_ZV_CNTRL);
  WrByte(pHw,REG_DATA(pHw),(BYTE)0);

  dvHwMainResetGPIO(pHw);
}

BYTE dvHwMainReadLsiReg(PHW pHw,WORD wReg) {
  BYTE bReg;
  dvHwMainSetLsiAddr(pHw,wReg);
  bReg=(BYTE)RdByte(pHw,REG_LSI_DATA(pHw));
  if(wReg>=114 && wReg<=116)
    bReg=(BYTE)RdByte(pHw,REG_LSI_DATA(pHw)); // read twice (problem in '020)
  pHw->bReg[wReg]=bReg;
  WatchBreak(pHw,wReg,0,bReg);
  return bReg;
}

void dvHwMainWriteLsiReg(PHW pHw,WORD wReg, BYTE bDat) {
  WatchBreak(pHw,wReg,1,bDat);
  if(wReg!=17)
    DbgMsg(DBG_HWREGWR,"Reg(%u) <- %02X\r\n",wReg,bDat);
  dvHwMainSetLsiAddr(pHw,wReg);
  WrByte(pHw,REG_LSI_DATA(pHw),bDat);
  pHw->bReg[wReg] = bDat;
}

void dvHwMainWriteLsiReg32(PHW pHw, WORD Reg, DWORD Data) {
  dvHwMainWriteLsiReg(pHw,Reg++,(BYTE)(Data>>0));
  dvHwMainWriteLsiReg(pHw,Reg++,(BYTE)(Data>>8));
  dvHwMainWriteLsiReg(pHw,Reg++,(BYTE)(Data>>16));
  dvHwMainWriteLsiReg(pHw,Reg++,(BYTE)(Data>>24));
}

BYTE dvHwMainReadLsiRegBits(PHW pHw,WORD wReg,BYTE bBits) {
  return dvHwMainReadLsiReg(pHw,wReg) & bBits;
}

void dvHwMainWriteLsiRegBits(PHW pHw,WORD wReg, BYTE bBits, BYTE bDat) {
  BYTE bReg=(pHw->bReg[wReg]&(~bBits))|bDat;
  dvHwMainWriteLsiReg(pHw,wReg,bReg);
}

void dvHwMainSetAddr(PHW pHw,WORD wReg, DWORD dwAddr) {
  wReg+=2;
  dvHwMainWriteLsiReg(pHw,wReg--,(BYTE)(dwAddr>>16));
  dvHwMainWriteLsiReg(pHw,wReg--,(BYTE)(dwAddr>>8));
  dvHwMainWriteLsiReg(pHw,wReg,(BYTE)dwAddr);
}

LOCAL int dvHwMainTestLsiIxReg(PHW pHw) {
  int nErr=0;
  WORD tst,res;
  for(tst=0; tst<0x200; tst++) {
    dvHwMainSetLsiAddr(pHw,tst);
    res=dvHwMainGetLsiAddr(pHw);
    if(tst!=res) {
      nErr++;
      DbgMsg(DBG_HWERR,"Error: LSI Index wrote:%02x,  read:%02X\r\n",tst,res);
      if(nErr>8)
        break;
    }
  }
  return nErr;
}

LOCAL int dvHwMainTestLsiRegs(PHW pHw) {
  int n;
  int nErr=0;
  DWORD dwTst;
  DWORD dwRes;
  DWORD dwPat[]={0x00000000,0xffffffff,0x55555555,0xaaaaaaaa};
  dvHwMainPauseScrReg(pHw, 1);
  for(n=0;n<4;n++) {
    dwTst=dwPat[n];
    dvHwMainWriteScrReg(pHw, dwTst);
    dwRes=dvHwMainReadScrReg(pHw);
    if(dwTst!=dwRes) {
      nErr++;
      DbgMsg(DBG_HWERR,"Error: SCR Reg wrote:%08lx,  read:%08X\r\n",dwTst,dwRes);
      if(nErr>8)
        break;
    }
  }
  for(dwTst=1; dwTst; dwTst<<=1) {
    dvHwMainWriteScrReg(pHw, dwTst);
    dwRes=dvHwMainReadScrReg(pHw);
    if(dwTst!=dwRes) {
      nErr++;
      DbgMsg(DBG_HWERR,"Error: SCR Reg wrote:%08lx,  read:%08X\r\n",dwTst,dwRes);
      if(nErr>8)
        break;
    }
  }
  dvHwMainWriteScrReg(pHw,0);
  dvHwMainPauseScrReg(pHw,0);
  return nErr;
}

LOCAL void dvHwMainReportScrFreq(PHW pHw) {
  int nErr=0;
  DWORD dwScrIn;
  DWORD dwScrOut;
  DWORD dwTime;
  int nScrFreq;
  dwScrIn=dvHwMainReadScrReg(pHw);
  dwTime=MargiGetTime();
  while(MargiGetTime() < dwTime+500);
  dwScrOut=dvHwMainReadScrReg(pHw);
  DbgMsg(DBG_HWSTAT,"Scr Reg In = %08X\r\n",dwScrIn);
  DbgMsg(DBG_HWSTAT,"Scr Reg Out= %08X\r\n",dwScrOut);
  nScrFreq=(int)(dwScrOut-dwScrIn)*2;
  DbgMsg(DBG_HWSTAT,"Scr freq = %d.%03d KHz\r\n",nScrFreq/1000,nScrFreq%1000);
}

LOCAL void dvHwMainCheckVCO(PHW pHw) {
  BYTE bVCOStat, bDramStat;
  bVCOStat=dvHwMainReadLsiReg(pHw,204);
  if((bVCOStat&0x08)==0x08)
    DbgMsg(DBG_HWERR,"VCO out of sync.  Reg 204=%02X\r\n",bVCOStat);
  else {
    bDramStat=dvHwMainReadLsiReg(pHw,205);
    if((bDramStat&0x07)!=0x07)
      DbgMsg(DBG_HWERR,"VCO not phase locked.  Reg 205=%02X\r\n",bDramStat);
    else {
      if((bDramStat&0x38)!=0x28)
        DbgMsg(DBG_HWERR,"SDRAM not ready.  Reg 205=%02X\r\n",bDramStat);
      else
        DbgMsg(DBG_HWINIT,"VCO ok.  Reg 204=%02X\r\n",bVCOStat);
    }
  }
}

void dvHwMainSetLsiAddr(PHW pHw,WORD wLsiAddr) {
  if(pHw->wAsicType==ASIC_JUNO_REV2) {
    WrWord(pHw,REG_LSI_INDEXL(pHw),wLsiAddr);
  } else {
    WrByte(pHw,REG_LSI_INDEXL(pHw),(BYTE)(wLsiAddr&0x00ff));
    WrByte(pHw,REG_LSI_INDEXH(pHw),(BYTE)((wLsiAddr>>8)&0x00ff));
  }
}

WORD dvHwMainGetLsiAddr(PHW pHw) {
  WORD w;
  if(pHw->wAsicType==ASIC_JUNO_REV2) {
    w=RdWord(pHw,REG_LSI_INDEXL(pHw));
  } else {
	  w=RdByte(pHw,REG_LSI_INDEXL(pHw));
	  w|=RdByte(pHw,REG_LSI_INDEXH(pHw))<<8;
  }
  return w;
}

int dvHwMainPauseScrReg(PHW pHw,int fPause) {
  int PrevState;
  BYTE bReg7=dvHwMainReadLsiReg(pHw,CHANNEL_STATUS_REG);
  PrevState= (bReg7 & 0x10) ? 1:0;
  if(fPause) {
    DbgMsg(DBG_SYNCHW,"dvHwMain: SCR paused.\r\n");
    bReg7|= 0x10;
  } else {
    DbgMsg(DBG_SYNCHW,"dvHwMain: SCR running.\r\n");
    bReg7&= ~0x10;
  }
  dvHwMainWriteLsiReg(pHw,CHANNEL_STATUS_REG,bReg7);
  return(PrevState);
}

int RegScrPause(PHW pHw, DWORD dwPauseFlags) {
  int fPause;
  if(dwPauseFlags&FLAG_SCR_RESET)
    pHw->nScrPauseRef=0;
  if(dwPauseFlags&FLAG_SCR_PAUSE)
    pHw->nScrPauseRef++;
  else if(pHw->nScrPauseRef)
    pHw->nScrPauseRef--;
  fPause =(pHw->nScrPauseRef>0);
  dvHwMainPauseScrReg(pHw,fPause);
  return fPause;
}

DWORD dvHwMainReadScrReg(PHW pHw) {
  DWORD ClockCnt;
  ClockCnt= (DWORD)dvHwMainReadLsiReg(pHw,SCR_VALUE_7_DOWNTO_0);
  ClockCnt|=(DWORD)dvHwMainReadLsiReg(pHw,SCR_VALUE_15_DOWNTO_8)<< 8;
  ClockCnt|=(DWORD)dvHwMainReadLsiReg(pHw,SCR_VALUE_23_DOWNTO_16)<<16;
  ClockCnt|=(DWORD)dvHwMainReadLsiReg(pHw,SCR_VALUE_31_DOWNTO_24)<<24;
  return(ClockCnt);
}

void dvHwMainWriteScrReg(PHW pHw, DWORD NewCount) {
  int fPaused;
  fPaused= dvHwMainPauseScrReg(pHw,1);
  dvHwMainWriteLsiReg(pHw,SCR_VALUE_7_DOWNTO_0,(BYTE)(NewCount & 0x000000ff));
  dvHwMainWriteLsiReg(pHw,SCR_VALUE_15_DOWNTO_8,(BYTE)((NewCount & 0x0000ff00)>> 8));
  dvHwMainWriteLsiReg(pHw,SCR_VALUE_23_DOWNTO_16,(BYTE)((NewCount & 0x00ff0000)>>16));
  dvHwMainWriteLsiReg(pHw,SCR_VALUE_31_DOWNTO_24,(BYTE)((NewCount & 0xff000000)>>24));
  dvHwMainPauseScrReg(pHw,fPaused);
  DbgMsg(DBG_SYNCHW,"dvHwMain: SCR set to %u.\r\n",NewCount);
}

LOCAL DWORD dvHwMainAllocateFrameBuffer(PHW pHw,WORD wReg,DWORD dwAddr,DWORD dwSize) {
  WORD wBuffAddr = (WORD)((dwAddr+63)/64);   // Frame Buffer addresses are in 64 byte resolution
  WORD wBuffSize = (WORD)((dwSize+63)/64);
  WORD wNextAddr = wBuffAddr + wBuffSize;

  dvHwMainWriteLsiReg(pHw,wReg++,(BYTE)wBuffAddr);
  dvHwMainWriteLsiReg(pHw,wReg,(BYTE)(wBuffAddr>>8));
#ifdef _DEBUG
  {
    ADDRESS uStartAddr, uEndAddr;
    wReg--;
    uStartAddr.dwAddr   = 0;
    uStartAddr.bAddr[0] = dvHwMainReadLsiReg(pHw,wReg++);
    uStartAddr.bAddr[1] = dvHwMainReadLsiReg(pHw,wReg++);
    uStartAddr.dwAddr  *= 64;
    uEndAddr.dwAddr  = uStartAddr.dwAddr + (((DWORD)wBuffSize)*64);
    DbgMsg(DBG_HWBUFCFG,"Frame buffer Start Addr = %lx End Addr = %lx Length = %lx\r\n",uStartAddr.dwAddr,uEndAddr.dwAddr,uEndAddr.dwAddr-uStartAddr.dwAddr);
  }
#endif
  return ((DWORD)wNextAddr)*64;
}

/************************************************************************
 ** Allocate Memory for Channel Buffer                                 **
 **    input:  20 register identifies channel buffer                   **
 **            base address of buffer in 1 byte resolution             **
 **            size of buffer in bytes                                 **
 **    return: byte address after allocated buffer                     **
 ************************************************************************/
LOCAL DWORD dvHwMainAllocateChannelBuffer(PHW pHw,WORD wReg,DWORD dwAddr,DWORD dwSize) {
  WORD wBuffAddr = (WORD)((dwAddr+255)/256);  // Channel Buffer addresses are in 256 byte resolution
  WORD wBuffSize = (WORD)((dwSize+255)/256);
  WORD wNextAddr = wBuffAddr + wBuffSize;

  // Channel Buffer Start Address
  dvHwMainWriteLsiReg(pHw,wReg++,(BYTE)wBuffAddr);
  dvHwMainWriteLsiReg(pHw,wReg++,(BYTE)(wBuffAddr>>8));

  // Channel Buffer End Address  
  if(wNextAddr) {
    dvHwMainWriteLsiReg(pHw,wReg++,(BYTE)(wNextAddr-1));
    dvHwMainWriteLsiReg(pHw,wReg,(BYTE)((wNextAddr-1)>>8));
  } else {
    dvHwMainWriteLsiReg(pHw,wReg++,0x00);
    dvHwMainWriteLsiReg(pHw,wReg,0x00);
  }
#ifdef _DEBUG
  {
    ADDRESS uStartAddr, uEndAddr;
    wReg -= 3;
    uStartAddr.dwAddr   = 0;
    uStartAddr.bAddr[0] = dvHwMainReadLsiReg(pHw,wReg++);
    uStartAddr.bAddr[1] = dvHwMainReadLsiReg(pHw,wReg++);
    uStartAddr.dwAddr  *= 256;

    uEndAddr.dwAddr   = 0;
    uEndAddr.bAddr[0] = dvHwMainReadLsiReg(pHw,wReg++);
    uEndAddr.bAddr[1] = dvHwMainReadLsiReg(pHw,wReg);
    uEndAddr.dwAddr  *= 256;

    DbgMsg(DBG_HWBUFCFG,"Chnl Start Addr= %lu EndAddr= %lu Len= %lu\r\n",uStartAddr.dwAddr,uEndAddr.dwAddr,uEndAddr.dwAddr-uStartAddr.dwAddr);
  }
#endif
  return (DWORD)wNextAddr*256;
}

DWORD dvHwGetChWrPtr(PHW pHw,int nCh) {
  DWORD dwPtr;
  dwPtr= dvHwMainReadLsiReg24(pHw,pHw->ChInf[nCh].wWrReg) & 0x7FFFF;
  if(dwPtr>=pHw->ChInf[nCh].dwStartAddr)
    dwPtr-=pHw->ChInf[nCh].dwStartAddr;
  else
    dwPtr=0;
  return dwPtr;
}

DWORD dvHwGetSubChBytes(PHW pHw) {
  DWORD dwWrPtr;
  DWORD dwRdPtr;
  dwWrPtr=dvHwMainReadLsiReg24(pHw,pHw->ChInf[SUBCH].wWrReg) & 0x7FFFF;
  dwRdPtr=dvHwMainReadLsiReg24(pHw,428) & 0x7FFFF;
  if(dwRdPtr>dwWrPtr)
    return pHw->ChInf[SUBCH].dwSize+dwWrPtr-dwRdPtr;
  else
    return dwWrPtr-dwRdPtr;
}

#if(0)
/************************************************************************
 ************************************************************************/
DWORD dvHwCalcExtChPtr(int fVid,DWORD dwAddr,DWORD dwSize,DWORD* pdwPrevPtr,DWORD* pdwOvflCount)
{
  DWORD dwBase;

  if((dwAddr&0x80000)==((*pdwPrevPtr)&0x00080000)) {
#if(1)
// if ovfl bit unchanged the wrap by 2 if the ptr less than previous
// Stevie Ray Vaughan failiure if this is enabled.
    if(fVid && dwAddr<(*pdwPrevPtr))
      (*pdwOvflCount)+=2;
#endif
  } else
    (*pdwOvflCount)++;

  *pdwPrevPtr=dwAddr;
  dwBase=(*pdwOvflCount)*dwSize;
  return (dwAddr&0x0007ffff)+dwBase;
}
#endif

DWORD dvHwMainGetChWrCount(PHW pHw,int nCh) {
  DWORD dwWrPtr;
  DWORD dwWrCount=0;
  CHAN_INFO *pCh=&pHw->ChInf[nCh];

  dwWrPtr=dvHwMainReadLsiReg24(pHw,pCh->wWrReg);
  if(!pCh->dwPrvWrPtr)
    pCh->dwPrvWrPtr=dwWrPtr;
  // Test is ovflw bit changed
  if((dwWrPtr^pCh->dwPrvWrPtr)&0x80000)
    pCh->dwWrOvflow++;
  pCh->dwPrvWrPtr=dwWrPtr;
  return (dwWrPtr&0x7ffff)-pCh->dwStartAddr+(pCh->dwWrOvflow*pCh->dwSize);
}

DWORD dvHwMainGetCheckChWrCount(PHW pHw,int nCh) {
  DWORD dwWrPtr;
  DWORD dwWrCount=0;
  CHAN_INFO *pCh=&pHw->ChInf[nCh];

  dwWrPtr=dvHwMainReadLsiReg24(pHw,pCh->wWrReg);
  if(!pCh->dwPrvChkPtr)
    pCh->dwPrvChkPtr=dwWrPtr;
  // Test is ovflw bit changed
  if((dwWrPtr^pCh->dwPrvChkPtr)&0x80000)
    pCh->dwChkOvflow++;
  pCh->dwPrvChkPtr=dwWrPtr;
  return (dwWrPtr&0x7ffff)-pCh->dwStartAddr+(pCh->dwChkOvflow*pCh->dwSize);
}



DWORD dvHwMainGetChRdCount(PHW pHw,int nCh) {
  DWORD dwRdPtr;
  DWORD dwRdCount=0;
  CHAN_INFO *pCh=&pHw->ChInf[nCh];

  dwRdPtr=dvHwMainReadLsiReg24(pHw,pCh->wRdReg);
  if(!pCh->dwPrvRdPtr)
    pCh->dwPrvRdPtr=dwRdPtr;
  // Test is ovflw bit changed
  if((dwRdPtr^pCh->dwPrvRdPtr)&0x80000)
    pCh->dwRdOvflow++;
//  if(nCh==VIDCH)
//    DbgMsg(DBG_SYNCCHK,"GetRdCount: RdPtr=%08X Prv=%08lx Ovf=%d\r\n",dwRdPtr,pCh->dwPrvRdPtr,pCh->dwRdOvflow);
  pCh->dwPrvRdPtr=dwRdPtr;
  return (dwRdPtr&0x7ffff)-pCh->dwStartAddr+(pCh->dwRdOvflow*pCh->dwSize);
}


void dvHwMainClrRdyError(PHW pHw) {
  if(pHw->wHwError!=HW_REMOVED) {
    pHw->wHwError=0;
    WrByte(pHw,REG_INDEX(pHw),(BYTE)RIX_LSI_CNTRL);
    WrByte(pHw,REG_DATA(pHw),0x21);
    WrByte(pHw,REG_DATA(pHw),0x01);
  }
}

void dvHwMainReportRdyError(PHW pHw,int nCh) {
 DWORD dwVidItems=dvHwMainGetAddr(pHw,VID_CHAN_NUMITEM_REG);
 DWORD dwAudItems=dvHwMainGetAddr(pHw,AUD_CHAN_NUMITEM_REG);
 DWORD dwSubItems=dvHwGetSubChBytes(pHw);
 DbgMsg(DBG_CRIT,"%s RdyErr! Vid=%u, Aud=%u, Sub=%u \r\n",(nCh==VIDCH)?"VID":(nCh==AUDCH)?"AUD":(nCh==SUBCH)?"SUB":"???",
                                                          dwVidItems,dwAudItems,dwSubItems);
}

long dvHwOsdMinModeBuffConfig(PHW pHw,DWORD dwBytes) {
  long Err=0;
  if(dwBytes) {
    pHw->dwOsdEndAddr  =DRAM_BUFFER_SIZE/8;
    pHw->dwOsdStartAddr=pHw->dwOsdEndAddr-dwBytes/8;
//    pHw->fOsdMinMode=1;
    pHw->fOsdDisable=0;
  } else {
    pHw->dwOsdStartAddr=0;
    pHw->dwOsdEndAddr=0;
//    pHw->fOsdMinMode=1;
    pHw->fOsdDisable=1;
  }
  pHw->dwOsdRequestedSize=dwBytes;
  return Err;
}

long  dvHwOsdBuffReserve(PHW pHw,DWORD dwBytes) {
  pHw->dwOsdRequestedSize=dwBytes;
  pHw->dwOsdEndAddr  =DRAM_BUFFER_SIZE/8;
  pHw->dwOsdStartAddr=pHw->dwOsdEndAddr-dwBytes/8;
  return 0;
}


DWORD dvHwMainGetDramSize(PHW pHw) {
  return DRAM_BUFFER_SIZE;
}

long dvHwMainBuffConfig(PHW pHw) {
  long Err= 0;
  DWORD dwFrameSize;
  DWORD dwBFrameLuma;
  DWORD dwFrameBuffSize;
  DWORD dwAddr = 0;
  DWORD dwTotalBytes = DRAM_BUFFER_SIZE;
  long  lMemoryAvailable;
  WORD  wPicHeight=pHw->wPicHeight;
  WORD  wPicWidth= pHw->wPicWidth;
  DWORD dwVidBuffSize=0;
  DWORD dwAudBuffSize=0;
  DWORD dwSpuBuffSize=0;
  DWORD dwPesBuffSize=0;
  DWORD dwOsdBuffSize=0;

  int fChannelStopped=0;
  int fFrameSizeChange=0;
  int fVidChSizeChange=0;
  int fAudChSizeChange=0;

  LOGTIME(LOGVIDCONFIG,dvHwMainReadScrReg(pHw));

  //Force the Width to 720 even if 704, this stop green flash in Madonna Menu, which is 720 and video is 704
  wPicWidth = 720;
  if(wPicHeight<289)
    wPicHeight*=2;
  dwFrameSize = ((DWORD)wPicHeight)*wPicWidth;
  if(wPicHeight==576) {                             //If PAL
    BYTE bNumOfSegments = (BYTE)(wPicHeight/13);
    dwBFrameLuma    = (DWORD)bNumOfSegments*wPicWidth*8L;                  //LumaFrameStoreMemory   = NumSegment * PixelWidth * 8 = (576/13) * 720 * 8 = 253,440bytes
    dwFrameBuffSize = (dwFrameSize/2)*6 + dwBFrameLuma + (dwBFrameLuma/2); //ChromaFrameStoreMemory = NumSegment * PixelWidth * 4 = (576/13) * 720 * 4 = 126,720bytes
    dvHwMainWriteLsiRegBits(pHw,NUM_OF_SEGMENTS_RMM_REG,(BYTE)(bNumOfSegments<<1));
    dvHwMainWriteLsiRegBits(pHw,REDUCED_MEMORY_MODE_REG,0x01);
  } else {
    dwBFrameLuma    = dwFrameSize;
    dwFrameBuffSize = (dwFrameSize/2)*9;     // Mem for frame buffers= #pels*4.5 - *1 for Y, *0.5 for U and V, all *3 for 2 anchors and display 
    dvHwMainWriteLsiRegBits(pHw,REDUCED_MEMORY_MODE_REG,0x00);
  }

  if(pHw->nMpegType&1) {    // if MPEG 1
    dwVidBuffSize=80*1024;
    dwAudBuffSize=40*1024;
  } else {
    dwVidBuffSize= (DWORD)232*1024;
    dwAudBuffSize= (DWORD)58*1024;
    dwSpuBuffSize= (DWORD)60*1024;
  }

  if(pHw->fUsePesHdr)
    dwPesBuffSize=2048;
  else
    dwPesBuffSize=0;

  //Ensure Channel buffers are multiple of 256
  dwVidBuffSize=(dwVidBuffSize+255)&~0x00ff;
  dwAudBuffSize=(dwAudBuffSize+255)&~0x00ff;
  dwSpuBuffSize=(dwSpuBuffSize+255)&~0x00ff;
  dwPesBuffSize=(dwPesBuffSize+255)&~0x00ff;

  // Calculate amoute of memory remaining after minimum buff allocation.
  lMemoryAvailable=dwTotalBytes-(dwFrameBuffSize+dwSpuBuffSize+dwPesBuffSize+dwVidBuffSize+dwAudBuffSize);

  if(lMemoryAvailable<0) {
    DWORD dwAudRed;
    DWORD dwVidRed;
    dwAudRed = (DWORD)(-lMemoryAvailable)*(dwAudBuffSize/256)/(dwVidBuffSize/256);
    dwVidRed = (DWORD)(-lMemoryAvailable)-dwAudRed;
    dwAudBuffSize-=dwAudRed&~0x00ff;
    dwVidBuffSize-=dwVidRed&~0x00ff;
    DbgMsg(DBG_ERR,"Buffer Memory Over Allocated - reducing Channel buffers\r\n");
  }
  if(pHw->dwOsdRequestedSize) {
    dwOsdBuffSize=(lMemoryAvailable<(long)pHw->dwOsdRequestedSize)?
                            lMemoryAvailable:(long)pHw->dwOsdRequestedSize;
    lMemoryAvailable-=(long)dwOsdBuffSize;
  }

  if(lMemoryAvailable && ((pHw->nMpegType&1)==0)) {   // If MPEG2 then share remaining memory between audio and video channels
    DWORD dwAudInc;
    DWORD dwVidInc;
    if(dwVidBuffSize>dwAudBuffSize)
      dwAudInc = (DWORD)lMemoryAvailable*(dwAudBuffSize/256)/(dwVidBuffSize/256);
    else
      dwAudInc = (DWORD)lMemoryAvailable*(dwVidBuffSize/256)/(dwAudBuffSize/256);
    dwVidInc = (DWORD)lMemoryAvailable-dwAudInc;
    dwAudBuffSize+=dwAudInc&~0x00ff;
    dwVidBuffSize+=dwVidInc&~0x00ff;
  }
  pHw->ChInf[VIDCH].dwPrefillLevel=(dwVidBuffSize*6/8)/8;
  
  pHw->ChInf[AUDCH].dwPrefillLevel=(dwAudBuffSize/2)/8;
  if(pHw->ChInf[AUDCH].dwPrefillLevel>8192/8)
    pHw->ChInf[AUDCH].dwPrefillLevel=8192/8;

// Start/Stop Channel must always be done to ensure
// channels are empty to avoid the Q table problem
#if(0)
  fChannelStopped= !(dvHwMainReadLsiReg(pHw,7)&0x01);
  fFrameSizeChange=(pHw->dwFrameBuffSize!=dwFrameBuffSize);
  fVidChSizeChange=(pHw->ChInf[VIDCH].dwSize!=dwVidBuffSize/8);
  fAudChSizeChange=(pHw->ChInf[VIDCH].dwSize!=dwVidBuffSize/8);

  if(fChannelStopped||fFrameSizeChange||fVidChSizeChange||fAudChSizeChange)
#endif
  {
    // Flip anchor buffers to counteract 020 flip when channel restarted
    static int fFlip=0;
    WORD wAnchor1Luma=FORWARD_ANCHOR_LUMA_REG;
    WORD wAnchor1Chroma=FORWARD_ANCHOR_CHROMA_REG;
    WORD wAnchor2Luma=BACKWARD_ANCHOR_LUMA_REG;
    WORD wAnchor2Chroma=BACKWARD_ANCHOR_CHROMA_REG;
    if(pHw->fFlipAnchors)
      ++fFlip;
    pHw->fFlipAnchors=0;

    if(fFlip&1) {
      wAnchor2Luma=FORWARD_ANCHOR_LUMA_REG;
      wAnchor2Chroma=FORWARD_ANCHOR_CHROMA_REG;
      wAnchor1Luma=BACKWARD_ANCHOR_LUMA_REG;
      wAnchor1Chroma=BACKWARD_ANCHOR_CHROMA_REG;
    }
      
    dvHwMainStopDecoderChannel(pHw);

    pHw->dwFrameBuffSize = dwFrameBuffSize;
    DbgMsg(DBG_HWBUFCFG,"Forward Anchor Buffer Start=%08lx Len=%08lx\r\n",dwAddr,dwFrameSize+dwFrameSize/2);

    dwAddr=dvHwMainAllocateFrameBuffer(pHw,wAnchor1Luma,dwAddr,dwFrameSize);
    dwAddr=dvHwMainAllocateFrameBuffer(pHw,wAnchor1Chroma,dwAddr,dwFrameSize/2);

    DbgMsg(DBG_HWBUFCFG,"Display Anchor Buffer Start=%08lx Len=%08lx\r\n",dwAddr,dwBFrameLuma+dwBFrameLuma/2);
    dwAddr=dvHwMainAllocateFrameBuffer(pHw,DISPLAY_LUMA_REG,dwAddr,dwBFrameLuma);
    dwAddr=dvHwMainAllocateFrameBuffer(pHw,DISPLAY_CHROMA_REG,dwAddr,dwBFrameLuma/2);

    DbgMsg(DBG_HWBUFCFG,"Backward Anchor Buffer Start=%08lx Len=%08lx\r\n",dwAddr,dwFrameSize+dwFrameSize/2);
    dwAddr=dvHwMainAllocateFrameBuffer(pHw,wAnchor2Luma,dwAddr,dwFrameSize);
    dwAddr=dvHwMainAllocateFrameBuffer(pHw,wAnchor2Chroma,dwAddr,dwFrameSize/2);
  
    dvHwMainAllocateChannelBuffer(pHw,DATA_DUMP_REG,0,0);
    dvHwMainAllocateChannelBuffer(pHw,DSI_CHAN_START_REG,0,0);
    if(dwPesBuffSize) {
      pHw->dwPesHdrStartAddr=dwAddr/8;
      DbgMsg(DBG_HWBUFCFG,"Pes Buffer Start=%08lx Len=%08lx\r\n",dwAddr,dwPesBuffSize);
      dwAddr=dvHwMainAllocateChannelBuffer(pHw,AUD_PES_START_REG,dwAddr,dwPesBuffSize);
      pHw->dwPesHdrEndAddr=(dwAddr/8)-1;
    } else {
      dvHwMainAllocateChannelBuffer(pHw,AUD_PES_START_REG,0,0);
    }

    pHw->ChInf[SUBCH].dwStartAddr = dwAddr/8;
    pHw->ChInf[SUBCH].dwSize      = dwSpuBuffSize/8;
    DbgMsg(DBG_HWBUFCFG,"SPUPicture Channel Buffer Start=%08lx Len=%08lx\r\n",dwAddr,dwSpuBuffSize);
    dwAddr=dvHwMainAllocateChannelBuffer(pHw,SPU_CHAN_START_REG,dwAddr,dwSpuBuffSize);
  
    pHw->ChInf[VIDCH].dwStartAddr = dwAddr/8;
    pHw->ChInf[VIDCH].dwSize      = dwVidBuffSize/8;
    DbgMsg(DBG_HWBUFCFG,"Vid Channel Buffer Start=%08lx %u bytes.\r\n",dwAddr,dwVidBuffSize);
    dwAddr=dvHwMainAllocateChannelBuffer(pHw,VID_CHAN_START_REG,dwAddr,dwVidBuffSize);

    pHw->ChInf[AUDCH].dwStartAddr = dwAddr/8;
    pHw->ChInf[AUDCH].dwSize      = dwAudBuffSize/8;
    DbgMsg(DBG_HWBUFCFG,"Aud Channel Buffer Start=%08lx %u bytes.\r\n",dwAddr,dwAudBuffSize);
    dwAddr=dvHwMainAllocateChannelBuffer(pHw,AUD_CHAN_START_REG,dwAddr,dwAudBuffSize);

    DbgMsg(DBG_HWBUFCFG,"Decoder Buffer Allocation End=%08lx bytes.\r\n",dwAddr);

//    pHw->fOsdMinMode=1;
    if(pHw->dwOsdStartAddr && pHw->dwOsdEndAddr) {
      if(dwAddr/8 > pHw->dwOsdStartAddr) {
        pHw->fOsdDisable=1;
        pHw->dwOsdStartAddr=0;
        pHw->dwOsdEndAddr=0;
      }
    }
    pHw->dwOsdMaxBytesPerTick=OSD_RUN_MAX_BYTES_PER_TICK;

    dvHwMainStartDecoderChannel(pHw);
  }

#if(0)
  dvHwFlushPacket(pHw,0);
  dvHwWriteLsiRegBits(RESET_FIFO_REG,RESET_FIFO_ALL);
  dvHwWriteLsiReg(355,0);
  dvHwWriteLsiReg(355,0x80);
#endif
  TransferInit(pHw);

  LOGTIME(LOGVIDCONFIG,dvHwMainReadScrReg(pHw));
  pHw->fVidBuffConfigured= TRUE;
  pHw->fAudBuffConfigured= TRUE;
  pHw->fSubBuffConfigured= TRUE;
  pHw->u.s.fChannelConfig=1;
  return(Err);
}



long dvHwMainStopDecoderChannel(PHW pHw) {
  long Err= 0;
  DbgMsg(DBG_HWCMD,"dvHwStopDecoderChannel()\r\n");
  dvHwMainWriteLsiRegBits(pHw,DECODER_COMMAND_REG,DECODER_STOP);            // Ensure Video decoder stopped
  dvHwMainWriteLsiRegBits(pHw,SPU_DECODER_COMMAND_REG,DECODER_STOP);        // Ensure SPUPicture decoder stopped
  dvHwMainWriteLsiRegBits(pHw,AUD_DECODER_COMMAND_REG,AUD_DECODER_STOP);
  dvHwMainWriteLsiRegBits(pHw,AUD_PLAY_MODE_REG,AUD_PAUSE_MODE);

  dvHwMainWriteLsiRegBits(pHw,VIDEO_PES_ENABLE_REG,WRITE_NO_HEADERS);
  dvHwMainWriteLsiReg(pHw,PES_HEADER_ENABLE_REG,WRITE_NO_HEADERS);

  dvHwMainWriteLsiRegBits(pHw,RESET_FIFO_REG,RESET_FIFO_ALL);
  dvHwMainWriteLsiRegBits(pHw,CHANNEL_CTRL_REG,CHANNEL_RESET);
  if((dvHwMainReadLsiReg(pHw,7) & 0x01)) {
    DbgMsg(DBG_HWERR,"Error:dvHwStartDecoderChannel(): Unable to stop channel\r\n");
    Err= MARGI_ERR_UNABLE_STOP_CHNL;
  } else {
    DbgMsg(DBG_SYNCHW,"dvHwMain: Decoder stopped.\r\n");
  }
  // Since decoder stop allow entire buffer for OSD
//  pHw->fOsdMinMode=0;
  pHw->dwOsdMaxBytesPerTick=OSD_STOP_MAX_BYTES_PER_TICK;

  return Err;
}

void dvHwMainGetChnLevels(PHW pHw) {
  pHw->ChInf[VIDCH].dwItems=dvHwMainGetAddr(pHw,VID_CHAN_NUMITEM_REG);
  pHw->ChInf[AUDCH].dwItems=dvHwMainGetAddr(pHw,AUD_CHAN_NUMITEM_REG);
  pHw->ChInf[SUBCH].dwItems=dvHwGetSubChBytes(pHw);
}


void dvHwMainClearChCounters(PHW pHw, int nCh) {
  pHw->ChInf[nCh].dwPrvWrPtr=0;
  pHw->ChInf[nCh].dwPrvChkPtr=0;
  pHw->ChInf[nCh].dwPrvRdPtr=0;
  pHw->ChInf[nCh].dwWrOvflow=0;
  pHw->ChInf[nCh].dwChkOvflow=0;
  pHw->ChInf[nCh].dwRdOvflow=0;
  if(nCh==VIDCH) {
    pHw->dwVidTotalXfr=0;
    pHw->dwPrvVidTotalXfr=0;
  }
}

void dvHwMainClearAllChCounters(PHW pHw) {
  dvHwMainWriteLsiRegBits(pHw,RESET_FIFO_REG,RESET_FIFO_SYS);
  dvHwMainClearChCounters(pHw,AUDCH);
  dvHwMainClearChCounters(pHw,VIDCH);
}

long dvHwMainStartDecoderChannel(PHW pHw) {
  long Err= 0;
  WORD wReg;
  DbgMsg(DBG_HWCMD,"dvHwStartDecoderChannel()\r\n");
#if(1)
  WrByte(pHw,REG_INDEX(pHw),(BYTE)RIX_IRQ_CNTRL);
  wReg=RdWord(pHw,REG_DATA(pHw));
  if(pHw->fUseDMA)
    WrWord(pHw,REG_DATA(pHw),(WORD)(wReg|1));
  else
    WrWord(pHw,REG_DATA(pHw),(WORD)(wReg&~1));
#endif  
  dvHwMainWriteLsiRegBits(pHw,CHANNEL_CTRL_REG,CHANNEL_RESET);
  if((dvHwMainReadLsiReg(pHw,7) & 0x01))
    DbgMsg(DBG_HWERR,"Error:dvHwStartDecoderChannel(): Unable to stop channel\r\n");
//  dvHwMainWriteLsiRegBits(pHw,VIDEO_PES_ENABLE_REG,WRITE_NO_HEADERS);
//  dvHwMainWriteLsiReg(pHw,PES_HEADER_ENABLE_REG,WRITE_NO_HEADERS);
  dvHwMainWriteLsiRegBits(pHw,STREAM_SELECT_REG,STREAM_SELECT_PRG|PARALLEL_CHAN);
  dvHwMainWriteLsiRegBits(pHw,CHANNEL_CTRL_REG,CHANNEL_START);
  if(!(dvHwMainReadLsiReg(pHw,7) & 0x01)) {
    DbgMsg(DBG_HWERR,"Error:dvHwStartDecoderChannel(): Unable to start channel\r\n");
    Err= MARGI_ERR_UNABLE_START_CHNL;
  } else {
    //dvHwMainWriteLsiReg(pHw,355,0);
    //dvHwMainWriteLsiReg(pHw,355,0x80);
    dvHwMainWriteLsiRegBits(pHw,AUD_PLAY_MODE_REG,AUD_PAUSE_MODE);
    dvHwMainWriteLsiRegBits(pHw,AUD_DECODER_COMMAND_REG,AUD_DECODER_START);
    // Reset Audio/Video Read/Write Channel Counters
    dvHwMainClearAllChCounters(pHw);
    DbgMsg(DBG_SYNCHW,"dvHwMain: Decoder started.\r\n");
  }
  return Err;
}

void dvHwMainGetTimeDiscontinuity(PHW pHw,DWORD* pdwOldScrHi,DWORD* pdwOldScrLo,DWORD* pdwNewScrHi,DWORD* pdwNewScrLo) {
  if(pHw->bIsNewScr) {
    *pdwOldScrLo=pHw->dwOldScrLo;
    *pdwOldScrHi=pHw->dwOldScrHi;
    *pdwNewScrLo=pHw->dwNewScrLo;
    *pdwNewScrHi=pHw->dwNewScrHi;
  } else {
    *pdwOldScrLo=0;
    *pdwOldScrHi=0;
    *pdwNewScrLo=0;
    *pdwNewScrHi=0;
  }
}


DWORD dvHwMainGetDecoderTime(PHW pHw,DWORD* pdwScrHi) {
  MDSLONGLONG mdsTime;
  DWORD dwScrLo;
  DWORD dwScrHi;
  // If a new immediate SCR setting is to be applied immediately (OldScr==0)
  // then use the new setting time as the current time.
  // Otherwise use the currently recorded time.
  // NOTE: store dwNewScrLo/Hi before checking IsNewScr in case Timer ISR 
  //       occurs immediately after the check.  Timer ISR will clear dwNewScr
  dwScrLo=pHw->dwNewScrLo;
  dwScrHi=pHw->dwNewScrHi;
  if(!pHw->bIsNewScr || (pHw->dwOldScrLo!=0) || (pHw->dwOldScrHi!=0)) {
    dwScrLo=pHw->dwScrLo;
    dwScrHi=pHw->dwScrHi;
  }

  Set_MdsLongLong(&mdsTime,dwScrLo,dwScrHi);  // mdsTime  = ((ULONGLONG)pHw->dwScrHi<<32)+(ULONGLONG)pHw->dwScrLo;
  if(pdwScrHi)
    *pdwScrHi=(DWORD)GetHigh_MdsLongLong(&mdsTime);
  return (DWORD)GetLow_MdsLongLong(&mdsTime);
}

DWORD dvHwMainSetDecoderTime(PHW pHw,DWORD dwNewScrLo,DWORD dwNewScrHi,DWORD dwOldScrLo,DWORD dwOldScrHi) {
  if(dwNewScrLo==0 && dwNewScrHi==0)
    dwNewScrLo = 1;
  pHw->dwNewScrLo=dwNewScrLo;
  pHw->dwNewScrHi=dwNewScrHi;
  pHw->dwOldScrLo=dwOldScrLo;
  pHw->dwOldScrHi=dwOldScrHi;
  if(dwOldScrLo || dwOldScrHi) {
    // Subtract 20mS from OldScr Time.  We can only update SCR after BAV which occurs about every 30mS
    // Back off time at which to change SCR so error is between 10mS too late and 20mS too early
    pHw->dwOldScrLo-=90*20;
    if(pHw->dwOldScrLo>dwOldScrLo)
      pHw->dwOldScrHi--;
    // make sure adjusted OldScr is not 0
    if(pHw->dwOldScrLo==0)
      pHw->dwOldScrLo++;
  }
  pHw->bIsNewScr = TRUE;
  return 0;
}

void dvHwMainResetScr(PHW pHw,DWORD dwScrLo,DWORD dwScrHi) {
  pHw->dwScrHi=dwScrHi;
  pHw->dwScrLo=dwScrLo;
  dvHwMainWriteScrReg(pHw,dwScrLo);
}


DWORD dvHwMainAdjustDecoderTime(PHW pHw,long lAdjScr) {
  pHw->lAdjScr=lAdjScr;
  return 0;
}

void dvHwMainSetScrSlowRate(PHW pHw,DWORD dwRate) {
  pHw->dwScrReqSlowRate=dwRate;
}

DWORD dvHwMainUpdateScrVars(PHW pHw) {
  DWORD dwScr;
  pHw->dwScrReadTime=MargiGetTime();
  dwScr=dvHwMainReadScrReg(pHw);
    // Check for SCR wrap around
  if((pHw->dwScrLo&0x80000000) && !(dwScr&0x80000000))
    pHw->dwScrHi++;
  return pHw->dwScrLo=dwScr;
}

DWORD dvHwMaintainScr(PHW pHw) {
  DWORD dwScr=dvHwMainUpdateScrVars(pHw);
  // If SCR is to be changed then do it on 1st tick after BAV
  if(pHw->bStatusFlag[1] & 0x10) {
    if(pHw->lAdjScr) {
      if(pHw->lAdjScr<0) {
        if(dwScr+pHw->lAdjScr > dwScr)
          pHw->dwScrHi--;
      } else {
        if(dwScr+pHw->lAdjScr < dwScr)
          pHw->dwScrHi++;
      }
      dwScr+=pHw->lAdjScr;
      dvHwMainWriteScrReg(pHw,dwScr);
      pHw->dwScrLo=dwScr;
      pHw->lAdjScr=0;
    }
    if(pHw->bIsNewScr) {
      if((pHw->dwScrHi>pHw->dwOldScrHi)||((pHw->dwScrHi==pHw->dwOldScrHi) && pHw->dwScrLo>=pHw->dwOldScrLo)) {
        pHw->bIsNewScr = FALSE;
        dvHwMainWriteScrReg(pHw,pHw->dwNewScrLo);
        pHw->dwScrLo=pHw->dwNewScrLo;
        pHw->dwScrHi=pHw->dwNewScrHi;
        pHw->dwNewScrLo=0;
        pHw->dwNewScrHi=0;
        pHw->dwOldScrLo=0;
        pHw->dwOldScrHi=0;
        pHw->lAdjScr=0;
      }
    }
  }
  if(pHw->dwScrReqSlowRate && !pHw->dwScrSlowRate) {
    pHw->dwScrSlowRateCount=0;
    pHw->dwScrSlowRate=pHw->dwScrReqSlowRate;
  }

  //If slow SCR rate and SCR is not paused then slow SCR
  if(pHw->dwScrSlowRate && pHw->nScrPauseRef==0) {
    switch(pHw->dwScrSlowRateCount++%pHw->dwScrSlowRate) {
      case 0: dvHwMainPauseScrReg(pHw,0);
              if(!pHw->dwScrReqSlowRate)
                pHw->dwScrSlowRate=0;
              break;
      case 1: dvHwMainPauseScrReg(pHw,1);
              break;
    }
  } else if(pHw->dwScrSlowRateCount && (pHw->dwScrSlowRate==0)) {
    // When SCR Slow Rate is remove make sure SCR is in required state
    pHw->dwScrSlowRateCount=0;
    dvHwMainPauseScrReg(pHw,pHw->nScrPauseRef!=0);
  }

  return 0;
}

BYTE dvHwMainRead8(PHW pHw, WORD Reg) {
  BYTE Data1;
  dvHwMainSetLsiAddr(pHw,Reg);
  Data1= RdByte(pHw,REG_LSI_DATA(pHw));
  if(Reg<MAX_LSIREG)
    pHw->bReg[Reg]= Data1;
  return(Data1);
}

DWORD dvHwMainReadLsiReg24(PHW pHw, WORD Reg) {
  DWORD Data;
  if(Reg<=MAX_LSIREG-3) {
    Data= dvHwMainReadLsiReg(pHw,Reg);
    Data|= (DWORD)dvHwMainReadLsiReg(pHw,(WORD)(Reg+1))<<8;
    Data|= (DWORD)dvHwMainReadLsiReg(pHw,(WORD)(Reg+2))<<16;
  } else {
    Data= (DWORD)-1;
  }
  return(Data);
}

DWORD dvHwMainReadLsiReg32(PHW pHw, WORD Reg) {
  DWORD Data;
  if(Reg<=MAX_LSIREG-4) {
    Data= dvHwMainReadLsiReg(pHw,Reg);
    Data|= (DWORD)dvHwMainReadLsiReg(pHw,(WORD)(Reg+1))<<8;
    Data|= (DWORD)dvHwMainReadLsiReg(pHw,(WORD)(Reg+2))<<16;
    Data|= (DWORD)dvHwMainReadLsiReg(pHw,(WORD)(Reg+3))<<24;
  } else {
    Data= (DWORD)-1;
  }
  return(Data);
}

BYTE dvHwMainGetChannelStatus(PHW pHw) {
  BYTE bStat;
  bStat=RdByte(pHw,REG_AUDIO_CHANNEL(pHw));
  if(bStat&0x08) {
    DbgMsg(DBG_HWERR,"Ready Error occured. Stat=%02x\r\n",bStat);
    WrByte(pHw,REG_INDEX(pHw),RIX_BOARD_ID);
    bStat=RdByte(pHw,REG_DATA(pHw));
    if(bStat!=pHw->BoardID)
      bStat=0xff;
  }          
  return bStat;
}

DWORD dvHwMainGetAddr(PHW pHw, WORD wReg) {
  BYTE  bVal1;
  DWORD dwAddr = 0;
  dwAddr  = (DWORD) dvHwMainReadLsiReg(pHw, wReg++);
  bVal1=dvHwMainReadLsiReg(pHw, wReg++);
  dwAddr |= ((DWORD)bVal1)<<8; 
  bVal1=dvHwMainReadLsiReg(pHw, wReg);
  dwAddr |= ((DWORD)bVal1)<<16;
  return dwAddr;
}


long dvHwMainSetCompareScrReg(PHW pHw, BYTE bReg, DWORD TickCnt) {
  long Err= 0;
  dvHwMainWriteLsiReg(pHw,bReg++,(BYTE)TickCnt);
  dvHwMainWriteLsiReg(pHw,bReg++,(BYTE)(TickCnt>>8));
  dvHwMainWriteLsiReg(pHw,bReg++,(BYTE)(TickCnt>>16));
  dvHwMainWriteLsiReg(pHw,bReg,(BYTE)(TickCnt>>24));
  return Err;
}

LOCAL long dvHwMainResetChannel(PHW pHw) {
  long Err= 0;
  dvHwMainWriteLsiRegBits(pHw,CHANNEL_CTRL_REG,CHANNEL_RESET);
  dvHwMainWriteLsiRegBits(pHw,CH_CONFIG_REG,CH_CONFIG);
  dvHwMainWriteLsiRegBits(pHw,AUD_DECODER_COMMAND_REG,AUD_DECODER_STOP);
  dvHwMainWriteLsiRegBits(pHw,CH_PAUSE_REG,CH_UNPAUSE);
  dvHwMainWriteLsiRegBits(pHw,AUD_PLAY_MODE_REG,AUD_PAUSE_MODE);
  return Err;
}

long dvHwMainReadStatusFlags(PHW pHw) {
  long Err= 0;
  pHw->bStatusLatch[0]|=dvHwMainReadLsiReg(pHw,0);
  pHw->bStatusLatch[1]|=dvHwMainReadLsiReg(pHw,1);
  pHw->bStatusLatch[2]|=dvHwMainReadLsiReg(pHw,2);
  pHw->bStatusLatch[3]|=dvHwMainReadLsiReg(pHw,3);
  pHw->bStatusFlag[0]=pHw->bStatusLatch[0];
  pHw->bStatusFlag[1]=pHw->bStatusLatch[1];
  pHw->bStatusFlag[2]=pHw->bStatusLatch[2];
  pHw->bStatusFlag[3]=pHw->bStatusLatch[3];
  return Err;
}

long dvHwMainClrStatusLatches(PHW pHw) {
  long Err= 0;
  pHw->bStatusLatch[0]=0;
  pHw->bStatusLatch[1]=0;
  pHw->bStatusLatch[2]=0;
  pHw->bStatusLatch[3]=0;
  return Err;
}

long dvHwMainResetStatus(PHW pHw) {
  long Err= 0;
  pHw->bStatusLatch[0]=pHw->bStatusFlag[0]=0; dvHwMainReadLsiReg(pHw,0);
  pHw->bStatusLatch[1]=pHw->bStatusFlag[1]=0; dvHwMainReadLsiReg(pHw,1);
  pHw->bStatusLatch[2]=pHw->bStatusFlag[2]=0; dvHwMainReadLsiReg(pHw,2);
  pHw->bStatusLatch[3]=pHw->bStatusFlag[3]=0; dvHwMainReadLsiReg(pHw,3);
  return Err;
}

BYTE dvHwMainImmediateCheckStatus(PHW pHw,BYTE bReg,BYTE bBits) {
  pHw->bStatusLatch[bReg]|=dvHwMainReadLsiReg(pHw,bReg);
  return (pHw->bStatusLatch[bReg]|pHw->bStatusFlag[bReg])&bBits;
}

BYTE dvHwMainCheckStatus(PHW pHw,BYTE bReg,BYTE bBits) {
  return pHw->bStatusFlag[bReg]&bBits;
}

long dvHwMainGetFieldStatus(PHW pHw) {
  long FieldStatus=0;

  if(pHw->bStatusFlag[0]&0x80) {  // if NewField Detected
    FieldStatus|=NEW_FIELD;
    if(pHw->bFieldReg&0x01)
      FieldStatus|=LAST_FIELD;
    if(pHw->bFieldReg&0x80)
      FieldStatus|=TOP_FIELD;
    if(pHw->bFieldReg&0x40)
      FieldStatus|=ODD_FIELD;
    if(pHw->bFieldReg&0x20)
      FieldStatus|=FIRST_FIELD;
  }
  return FieldStatus;
}

/************************************************************************
 **  dvHwClrPesHeader
 ************************************************************************/
void dvHwMainClrPesHeader(PHW pHw)
{
  pHw->dwPesHdrRdAddr=pHw->dwPesHdrEndAddr+1;
}


/************************************************************************
 **  dvHwGetPesHeader
 ************************************************************************/
BYTE dvHwMainGetPesHeader(PHW pHw,DWORD* pdwBegin,DWORD* pdwEnd,DWORD* pdwPts)
{
  BYTE  bId=0;
  BYTE  bSkip;
  BYTE  Data[32];
  DWORD dwPts,dwChPtr,dwPktLen;
  DWORD dwPesWrAddr;
  int   nNumWords;

//  DbgMsgCmd(DBGCMD_PROF_ENTER,PROFILE_READFIFO,0);

  // If dwPesHdrRdAddr is not valid then initiate 1st read.
  if(pHw->dwPesHdrRdAddr>pHw->dwPesHdrEndAddr) {
    dvHwMainWriteLsiReg(pHw,193,0);     // set Host transfer mode, reset endian
    dwPesWrAddr=dvHwMainGetAddr(pHw,114)&0x0007ffff;
    pHw->dwPesHdrRdAddr=dwPesWrAddr;
    pHw->bPesHdrState=0;
  }

  dwPesWrAddr=dvHwMainGetAddr(pHw,114)&0x0007ffff;
  if(dwPesWrAddr>=pHw->dwPesHdrRdAddr)
    nNumWords=(int)(dwPesWrAddr-pHw->dwPesHdrRdAddr);
  else
    nNumWords=(int)((dwPesWrAddr-pHw->dwPesHdrRdAddr)+(pHw->dwPesHdrEndAddr+1-pHw->dwPesHdrStartAddr));
  if(nNumWords>8) {
    DbgMsg(DBG_SYNCERR,"GetPesHeader Numwords = %d\r\n",nNumWords);
    nNumWords=8;
  }

  while(nNumWords-- && pHw->bPesHdrState<4) {
    dvHwMainSetAddr(pHw,199,pHw->dwPesHdrRdAddr);
    dvHwMainReadDramWord(pHw,Data);
    if(++pHw->dwPesHdrRdAddr>pHw->dwPesHdrEndAddr) {
      pHw->dwPesHdrRdAddr=pHw->dwPesHdrStartAddr;
      if(0){
        BYTE  bPesBuffer[512];
        int n;
        dvHwMainSetAddr(pHw,199,0);
        for(n=0;n<64;n++)
          dvHwMainReadDramWord(pHw,&bPesBuffer[n*8]);
      }
    }
    switch(pHw->bPesHdrState++) {
      case 0: if(Data[0]==0 && Data[1]==0 && Data[2]==1) {
                pHw->bPesHdrId=Data[3];
                dwPktLen=(Data[4]<<8)+Data[5];
                pHw->bPesHdrSkip=0;
                if((Data[6]&0xc0)==0x80) {
                  pHw->fPesHdrMpeg2=1;
                  pHw->nPesPtsIx=9;
                }else {
                  pHw->fPesHdrMpeg2=0;
                  pHw->nPesPtsIx=6;
                  if(Data[6]&0x80) {
                    pHw->nPesPtsIx=7;
                    if(Data[7]&0x80)
                      pHw->nPesPtsIx=8;
                    else if((Data[7]&0xc0)==0x40)
                      pHw->nPesPtsIx=9;
                  } else if((Data[6]&0xc0)==0x40)
                      pHw->nPesPtsIx=8;
                }
                if(pHw->nPesPtsIx==6) {
                  dwPts=(DWORD)(Data[6]&0x0e)<<29;
                  dwPts|=(DWORD)(Data[7]    )<<22;
                } else if(pHw->nPesPtsIx==7)
                  dwPts=(DWORD)(Data[7]&0x0e)<<29;
                pHw->dwPesHdrPts=dwPts;
                if(dwPktLen> 2047) {
                  DbgMsg(DBG_SYNCERR,"GetPesHeader dwPktLen = %d\r\n",dwPktLen);
                }
              } else
                pHw->bPesHdrState=0;
              break;
      case 1: if(pHw->fPesHdrMpeg2)
                pHw->bPesHdrSkip=Data[0]/8;
              else if((pHw->nPesPtsIx==8) & ((Data[0]&0xf0)==0x30))
                pHw->bPesHdrSkip=1;

              pHw->nPesPtsIx-=8;
              dwPts=pHw->dwPesHdrPts;
              if(pHw->nPesPtsIx>=0)
                dwPts=  (DWORD)(Data[pHw->nPesPtsIx]&0x0e)<<29;     // Get Pts/2 from Pes Hdr
              if(++pHw->nPesPtsIx>=0)
                dwPts|= (DWORD)(Data[pHw->nPesPtsIx]     )<<22;
              if(++pHw->nPesPtsIx>=0)
                dwPts|= (DWORD)(Data[pHw->nPesPtsIx]&0xfe)<<14;
              if(++pHw->nPesPtsIx>=0)
                dwPts|= (DWORD)(Data[pHw->nPesPtsIx]     )<< 7;
              if(++pHw->nPesPtsIx>=0)
                dwPts|= (DWORD)(Data[pHw->nPesPtsIx]     )>> 1;
              if(dwPts==0) dwPts++;  // make sure the PTS is not 0 (since 0 is considered not valid)
              pHw->dwPesHdrPts=dwPts;
      case 2: bSkip=pHw->bPesHdrSkip;
              if(nNumWords<bSkip) {
                pHw->bPesHdrSkip=bSkip-nNumWords-1;
                bSkip=nNumWords;
                pHw->bPesHdrState=2;
                nNumWords=0;
              } else {
                nNumWords-=bSkip;
                pHw->bPesHdrState=3;
              }
              if((pHw->dwPesHdrRdAddr+=bSkip)>pHw->dwPesHdrEndAddr)
                pHw->dwPesHdrRdAddr-=pHw->dwPesHdrEndAddr+1-pHw->dwPesHdrStartAddr;
              break;
      case 3: dwChPtr=  (DWORD)Data[7];
              dwChPtr|= (DWORD)Data[6]<< 8;
              dwChPtr|= (DWORD)Data[5]<<16;
              if(dwChPtr >= 0x000c0000)
                DbgMsg(DBG_SYNCERR,"Bad ChPtr\r\n");
              break;
    }
  }
  if(pHw->bPesHdrState==4) {
    CHAN_INFO* pCh;
    pHw->bPesHdrState=0;
    bId      =pHw->bPesHdrId;
    *pdwPts  =pHw->dwPesHdrPts;
    pCh=(bId==0xe0)?&pHw->ChInf[VIDCH]:&pHw->ChInf[AUDCH];

    if((dwChPtr^pCh->dwPrvWrPtr)&0x80000)
      pCh->dwWrOvflow++;
    else if(pCh->dwPrvWrPtr>dwChPtr) {
      DbgMsg(DBG_SYNCERR,"Vid Single wrap PrvWrPtr=%08lx, WrPtr=%08lx\r\n",pCh->dwPrvWrPtr,dwChPtr);
      pCh->dwWrOvflow+=2;
    } else if((bId==0xe0) && (pCh->dwPrvWrPtr+0x2000>dwChPtr)) {
      DbgMsg(DBG_SYNCERR,"Vid Double wrap PrvWrPtr=%08lx, WrPtr=%08lx, Xfr=%08lx\r\n",pCh->dwPrvWrPtr,dwChPtr,(pHw->dwVidTotalXfr-pHw->dwPrvVidTotalXfr));
    }

    *pdwBegin=(dwChPtr&0x7ffff)-pCh->dwStartAddr+(pCh->dwWrOvflow*pCh->dwSize);
    *pdwEnd  =*pdwBegin+(dwPktLen/8)-2;
    pCh->dwPrvWrPtr=dwChPtr;

    if(*pdwEnd>*pdwBegin+2047)
      DbgMsg(DBG_SYNCERR,"GetPesHeader dwPktBegin=%08lx  dwPktEnd=%08lx\r\n",*pdwBegin,*pdwEnd);

    if(bId==0xe0) {
      pHw->dwPrvVidTotalXfr=pHw->dwVidTotalXfr;
      DbgMsg(DBG_SYNCVIDPTS,"VidPes ChPtr=%x:%08lx,Begin=%08lx,End=%08lx",pCh->dwWrOvflow,dwChPtr,*pdwBegin,*pdwEnd);
      DbgDispPTS(DBG_SYNCVIDPTS,pHw->dwPesHdrPts,",PTS=","\r\n");
//      DbgMsg(DBG_SYNCVIDPTS,"Count=%d\r\n",pHw->dwVidTotalXfr);
    } else {
      DbgMsg(DBG_SYNCAUDPTS,"AudPes ChPtr=%x:%08lx,Begin=%08lx,End=%08lx",pCh->dwWrOvflow,dwChPtr,*pdwBegin,*pdwEnd);
      DbgDispPTS(DBG_SYNCAUDPTS,pHw->dwPesHdrPts,",PTS=","\r\n");
    }
  }
//  DbgMsgCmd(DBGCMD_PROF_EXIT,PROFILE_READFIFO,0);
  return bId;
}


/************************************************************************
 **  dvHwMainReadDramWord
 ************************************************************************/
long dvHwMainReadDramWord(PHW pHw,BYTE *pData) {
  DWORD n;
  DWORD nTimeOut=1000;
  while((--nTimeOut) && (dvHwMainReadLsiReg(pHw,192)&0x01));  //Wait until Read FIFO is not empty 
  if(nTimeOut==0) {
    DbgMsg(DBG_HWERR,"Timeout Reading from DRAM\r\n");
    return 1;
  }
  for(n=8;n;n--)
    pData[n-1] = dvHwMainReadLsiReg(pHw,194);

  return 0;
}

/************************************************************************
 **  dvHwMainReadDram                                                      **
 ************************************************************************/
long dvHwMainReadDram(PHW pHw,DWORD dwStartAddr,BYTE *pData,DWORD nBytes) {
  DWORD n,m;
  DWORD nTimeOut;
  long fReturn = ((dwStartAddr%8)!=0);
  dwStartAddr/=8;   // convert to 64 bit DRAM Addresses
  dvHwMainWriteLsiReg(pHw,193,8);     // set Host transfer mode, set endian
  dvHwMainWriteLsiReg(pHw,193,0);     // set Host transfer mode, reset endian
  nTimeOut = 1000;
  while((--nTimeOut) && (dvHwMainReadLsiReg(pHw,192)&0x02)==0);   //Wait until read FIFO is full
  dvHwMainSetAddr(pHw,199,dwStartAddr);
  for(n=0;n<nBytes/8; n++) {
    nTimeOut = 1000;
    while((--nTimeOut) && (dvHwMainReadLsiReg(pHw,192)&0x01));  //Wait until Read FIFO is not empty 
    if(nTimeOut==0) {
      DbgMsg(DBG_HWERR|DBG_HIFREQ,"Timeout Reading from DRAM\r\n");
      break;
    }
    for(m=8;m;m--)
      pData[m-1] = dvHwMainReadLsiReg(pHw,194);
    pData += 8;
  }
  return(fReturn);
}

/************************************************************************
 **  dvHwMainWriteDram                                                 **
 ************************************************************************/
long dvHwMainWriteDram(PHW pHw,DWORD dwStartAddr,BYTE* pData,DWORD dwBytes)
{
  DWORD dwCount;
  int m;
  int nTimeOut;
  long fReturn = ((dwStartAddr%8)!=0);
  dwStartAddr/=8;   // convert to 64 bit DRAM Addresses
  dvHwMainWriteLsiReg(pHw,193,8);     // set Host transfer mode, set endian
  dvHwMainWriteLsiReg(pHw,193,0);     // set Host transfer mode, reset endian
  nTimeOut = 1000;
  while((--nTimeOut) && (dvHwMainReadLsiReg(pHw,192)&0x04)==0);  //Wait until write FIFO is empty 
  dvHwMainSetAddr(pHw,196,dwStartAddr);
  for(dwCount=0;dwCount<dwBytes/8; dwCount++)
  {
    nTimeOut = 1000;
     while((--nTimeOut) && (dvHwMainReadLsiReg(pHw,192)&0x08));       //Wait until write FIFO is not full
    if(nTimeOut==0) {
      DbgMsg(DBG_HWERR|DBG_HIFREQ,"Timeout Writing to DRAM\r\n");
      break;
    }
    dvHwMainSetLsiAddr(pHw,195);
    for(m=8;m;m--)
      WrByte(pHw,REG_LSI_DATA(pHw),pData[m-1]);
//    for(m=8;m;m--)
//      dvHwMainWriteLsiReg(pHw,195,pData[m-1]);
    pData += 8;
  }
  return fReturn;
}


/************************************************************************
 **  dvHwMainFillDram                                                 **
 ************************************************************************/
long dvHwMainFillDram(PHW pHw,DWORD dwStartAddr,BYTE* pData,DWORD dwBytes)
{
  DWORD dwCount;
  int m;
  int nTimeOut;
  long fReturn = ((dwStartAddr%8)!=0);
  dwStartAddr/=8;   // convert to 64 bit DRAM Addresses
  dvHwMainWriteLsiReg(pHw,193,8);     // set Host transfer mode, set endian
  dvHwMainWriteLsiReg(pHw,193,0);     // set Host transfer mode, reset endian
  nTimeOut = 1000;
  while((--nTimeOut) && (dvHwMainReadLsiReg(pHw,192)&0x04)==0);  //Wait until write FIFO is empty 
  dvHwMainSetAddr(pHw,196,dwStartAddr);
  for(dwCount=0;dwCount<dwBytes/8; dwCount++)
  {
    nTimeOut = 1000;
    while((--nTimeOut) && (dvHwMainReadLsiReg(pHw,192)&0x08));     //Wait until write FIFO is not full
    if(nTimeOut==0) {
      DbgMsg(DBG_HWERR|DBG_HIFREQ,"Timeout Writing to DRAM\r\n");
      break;
    }
    dvHwMainSetLsiAddr(pHw,195);
    for(m=8;m;m--)
      WrByte(pHw,REG_LSI_DATA(pHw),pData[m-1]);

//    for(m=8;m;m--)
//      dvHwMainWriteLsiReg(pHw,195,pData[m-1]);
//    pData += 8;
  }
  return fReturn;
}

long dvHwMainEEPROMDataWrite(PHW pHw, LPBYTE lpBuff, DWORD dwSize)
{
  return EEPROM_Proc(pHw,EEPROM_TIMED_WRITE,(DWORD)lpBuff,dwSize);
}

long dvHwMainEEPROMDataRead(PHW pHw, LPBYTE lpBuff, DWORD dwSize)
{
  return EEPROM_Proc(pHw,EEPROM_READ_DATA,(DWORD)lpBuff, dwSize);
}

/* EOF : dvHwMain.c */
