#ifndef __DVHWMAIN_H__
#define __DVHWMIAN_H__ 0x0200

#if(0)
enum {
  FLAG_WAIT_FOR_CSS       =0x00000001,
  FLAG_DATA_DISCONTINUITY =0x00000002
};
#endif

enum {
  FLAG_SCR_UNPAUSE=0x00000000,
  FLAG_SCR_PAUSE  =0x00000001,
  FLAG_SCR_RESET  =0x00000002
};

enum {
  Z_STATE_STOP        =0,
  Z_STATE_PREROLL,
  Z_STATE_SYNC,
  Z_STATE_PLAY,
  Z_STATE_PAUSE,
  Z_STATE_UNK
};

enum {
  S_STATE_STOP   =0,
  S_STATE_PAUSE,
  S_STATE_RUN,
  S_STATE_UNK
};

enum {
  NEW_FIELD     =0x01,
  ODD_FIELD     =0x02,
  TOP_FIELD     =0x04,
  FIRST_FIELD   =0x08,
  LAST_FIELD    =0x10
};

enum {
  ASPECT_4x3      =0,
  ASPECT_16x9
};

enum
{
  AUD_MPEG    = 0x00,
  AUD_AC3     = 0x20,
  AUD_MPEG_FMT= 0x40,
  AUD_AC3_FMT = 0x60,
  AUD_LPCM_96 = 0x80,
  AUD_LPCM    = 0xA0,   //LPCM 96kHz needs to be decimated to 48kHz, since our Audio Encoder CS4333 supports upto 48kHz Only
  AUD_UNKNOWN = 0xFF
};

enum GPIO_PINS
{
  GPIO_PIN_RESET = 0,
  GPIO_PIN_CLOCK = 1,
  GPIO_PIN_DATA  = 2,
  GPIO_PIN_SPDIF = 3
};

enum
{
  SPU_FORCE_OFF = 0,
  SPU_OFF,
  SPU_ON
};

enum
{
  SPDIFF_LINEAR = 0,
  SPDIFF_DOLBY,
  SPDIFF_MPEG,
  SPDIFF_ALL
};

#define  RESET_RDY 0x00              
#define  CFG5465_REG   0x7777
#define  CFG5465_CH    0x0030

// Hardware Error Conditions
#define  HW_REMOVED   1
#define  HW_RDYERR    2


//IoType enumeration
#define IOTYPE_MEMORYMAPPED 1
#define IOTYPE_IO           2
#define IOTYPE_TELEPATHIC   3

#define RESET_HOLD_TIME     5
#define MAX_LSIREG 512
#define DVHW_SIGNATURE 0x70595843
typedef unsigned __int64 DLONG;

//Constants that are used to eliminate multiple Flush Packet scheme.
//#define HW_CAN_BE_FLUSHED           0
//#define HW_INSERT_FLUSH_PACKET      1
//#define HW_INSERTED_FLUSH_PACKET    2

#define Z_FLUSH_STATE_IDLE                         0
#define Z_FLUSH_STATE_BEGIN                        1
#define Z_FLUSH_STATE_NEED_TO_SEND_FLUSH_PACKET    2
#define Z_FLUSH_STATE_INSERT_FLUSH_PACKET          3
#define Z_FLUSH_STATE_INSERTED_FLUSH_PACKET        4


union uPTS_s {
  BYTE  bPTS[4];
  DWORD dwPTS;
};

struct PTS_s {
  BOOL fSet:1;
  union uPTS_s Initial;
  union uPTS_s Current;
};

typedef union
{
  BYTE  bAddr[4];
  DWORD dwAddr;
} ADDRESS;

typedef struct {
  int nRunDuration;
  int nRunCount;
  int nStopDuration;
  int nStopCount;
  int fIssueRun;
  int fIssueStop;
  int fIssueFreezeFrame;
  int fRepeat;
} FREEZE_STRUCT;

typedef struct {
  int fSkipIssued;
  int nSkipCount;
} SKIP_STRUCT;

enum {
  PULLDOWN_NONE,
  PULLDOWN_MAN32,
  PULLDOWN_AUTO
};

typedef struct {
  int  fManage;
  int  fOn;
  BYTE bMode;
  BYTE bCount;
} PULLDOWN_STRUCT;

#define VIDCH  0
#define AUDCH  1
#define SUBCH  2

typedef struct {
  WORD  wWrReg;
  WORD  wRdReg;
  DWORD dwStartAddr;
  DWORD dwSize;
  DWORD dwPrvWrPtr;      // previous write pointer
  DWORD dwWrOvflow;
  DWORD dwPrvChkPtr;     // previous write pointer for check
  DWORD dwChkOvflow;
  DWORD dwPrvRdPtr;      // previous read  pointer
  DWORD dwRdOvflow;
  DWORD dwPrefillLevel;
  DWORD dwSafety;        // num items to adjust level to prevent overflow
  DWORD dwItems;         // num items in video channel, updated in TimeGetStatus
  DWORD dwPTUs;          // num presentation items in channel buffer
} CHAN_INFO;

typedef struct {
  DWORD dwBytes;
  DWORD dwAddr;
  BYTE* pData;
  BYTE  buffer[64];
} EEPROM;

typedef struct {
  DWORD dwSignature;          //Must be DVHW_SIGNATURE
  WORD  BoardID;              //DVD Decoder board id number
  WORD  wAsicType;            //Margi Asic Type;
  DWORD dwBoardCaps;          //Capabilities flags from EEPROM
  DWORD dwIoAddr;             //IO base address
  DWORD dwMemAddr;            //Memory mapped address for data transfer
  DWORD dwDMAAddr;            //DMA address;
  DWORD dwChnAddr;            //Channel address may be memory or I/O
  WORD  wIoType;              //One of IOTYPE
  WORD  wHwError;             //Indicates Hardware Error State: Ready Error or Hardware removed
  WORD  *pRegTbl;             //Points to array which contains register indices.
  WORD  DecoderState;

  BYTE  fUseDMA;
  BYTE  fDMAAvailable;
  BYTE  bDisplayCount;
  BYTE  bStatusLatch[4];
  BYTE  bStatusFlag[4];
  BYTE  bFieldReg;
  BYTE  bGpioCtrl;
  BYTE  bGpioPin;

  void  (__stdcall *pfnRegWrByte)(DWORD dwIoAddr,BYTE bDat);
  void  (__stdcall *pfnRegWrWord)(DWORD dwIoAddr,WORD wDat);
  BYTE  (__stdcall *pfnRegRdByte)(DWORD dwIoAddr);
  WORD  (__stdcall *pfnRegRdWord)(DWORD dwIoAddr);
  DWORD (__stdcall *pfnChnWrWords)(DWORD dwIoAddr,BYTE* pDat,DWORD dwWords,DWORD dwFlags);

  union {
    struct {
      DWORD fInitDone:1;
      DWORD fUsePanic:1;
      DWORD fAudioClockSet:1;
      DWORD f8BitZV:1;
      DWORD fAutoStartDecode:1;
      DWORD PreEos:1;		  //EOS detected in channel.
      DWORD PostEos:1;		//EOS detected at decoder.
      DWORD SpuPes:1;     //SPU Pes detected in channel
      DWORD RdyErr:1;		//If set, decoder bus is hung.
      DWORD VideoDecodeRun:1;   //Set when Video decode status is Run.
      DWORD StartSubPicture:1;  //Set to signal start subpicture
      DWORD EnableSubPicture:1; //Set to signal supicture channel enabled
      DWORD ChannelRun:1;       //Set when Channel status is Run
      DWORD fInPumpData:1;      //Set when in dvHwTimerPumpData() (dvHwIsr.c)
      DWORD fChannelConfig:1;   //Set when channels configured. Clear on all video states leading to channel config in RdyWaitForData
      DWORD fVidChannelPrefill:1;  //Set when video channel is prefilling. Use to control PES header capture in Sync.c
      DWORD fAudChannelPrefill:1;  //Set when audio channel is prefilling. Use to control PES header capture in Sync.c
    }s;
    DWORD dwFlags;
  }u;
  DWORD dwScrReadTime;
  DWORD dwScrLo;
  DWORD dwScrHi;
  DWORD dwNewScrLo;             // New Scr time to be applied
  DWORD dwNewScrHi;
  DWORD dwOldScrLo;             // Scr time at which new time must be applied
  DWORD dwOldScrHi;
  BYTE  bIsNewScr;              // TRUE if new Scr time must be applied
  long  lAdjScr;
  DWORD dwScrSlowRateCount;
  DWORD dwScrReqSlowRate;
  DWORD dwScrSlowRate;

  WORD  wSaveLsiAddr;           // Used by Hi-Freq timer to save and restore LS addr reg
  WORD  wSaveIxAddr;            // Used by Hi-Freq timer to save and restore Midas Ix reg

  BYTE  fOddByte;
  BYTE  bOddByte;

  BYTE  fUsePesHdr;
  DWORD dwFrameBuffSize;
  DWORD dwPesHdrStartAddr;
  DWORD dwPesHdrEndAddr;
  DWORD dwPesHdrRdAddr;
  DWORD dwPesHdrPts;
  BYTE  bPesHdrState;
  BYTE  bPesHdrId;
  BYTE  bPesHdrSkip;
  BYTE  fPesHdrMpeg2;
  int   nPesPtsIx;

  BYTE  fOsdDisable;            // Set True will disable OSD used if Osd min mode buffer too big
//  BYTE  fOsdMinMode;            // Set True indicates Buffers set for play back used mini osd buffer
  DWORD dwOsdStartAddr;         // Start address in 8 byte words of min mode OSD buffer
  DWORD dwOsdEndAddr;           // End address in 8 byte words of min mode OSD buffer
  DWORD dwOsdRequestedSize;     // Requested min mode OSD buffer in bytes
  DWORD dwOsdMaxBytesPerTick;   // Approx maximum # bytes to xfer in one tick

  CHAN_INFO ChInf[3];
  DWORD dwVidTotalXfr;
  DWORD dwPrvVidTotalXfr;

  WORD  wPicWidth;
  WORD  wPicHeight;
  DWORD dwPicRate; 
  BYTE bReg[MAX_LSIREG+1];
  DWORD MasterClock;

  BOOL fVidBuffConfigured;
  BOOL fAudBuffConfigured;
  BOOL fSubBuffConfigured;
 
  WORD  wStreamsOpened;
  DWORD dwVidRate;				//Moved to dvHwVid.c
  DWORD dwAudRate;
  DWORD dwSubRate;
  DWORD dwVidMaxRate;				//Moved to dvHwVid.c
  DWORD dwAudMaxRate;
  DWORD dwSubMaxRate;

  DWORD dwClkRate;
  MDSLONGLONG dlClkStartTime;
  DWORD dwNewClkRate;
  MDSLONGLONG dlNewStartTime;
  DWORD dwScrTrigTime;
  DWORD dwScrTrigValue;
  BOOL  fTimeFormat10M;

  PULLDOWN_STRUCT pulldown;     //3:2 pulldown control struct
  SKIP_STRUCT     skip;
  FREEZE_STRUCT   freeze;       //Freeze frame control struct
  DWORD  dwSlowStepFrame;

  DWORD hAvSync;				//Handle to Sync object.
  DWORD hAuxData;				//Handle to AuxData object.
  DWORD hVideo;					//Handle to video decoder object.
  DWORD hAudio;					//Handle to audio decoder object.
  DWORD hSubpicture;	  //Handle to subpicture object.
  DWORD hCc;
  DWORD dwAspectRatio;                          //4x3 or 16x9
  DWORD dwVidDispFormatPreference;              //16x9 format Preference: LetterBox, PanScan, Wide
  DWORD dwVidDispFormat;                        //16x9 format Actual:     LetterBox, PanScan, Wide
  DWORD dwTVOutMode;                            //Composite / S-Video
  DWORD dwSpDiffMode;
  DWORD dwSpDiffOut;
  DWORD dwVOLuma;				        // video out balance
  DWORD dwVOChroma;

  DWORD dwCorrectionEnable;			// computer screen
  DWORD dwScreenContrast;			  // Not-used
  DWORD dwScreenBrightness;			// Not-used
  DWORD dwScreenGamma;				  // Not-used
  DWORD dwMute;                 // mute 0/1
  DWORD dwVolume;               // volume 0-100
  DWORD AC3MixMode;
  DWORD dwKaraokeMode;          //Karaoke Mode

  BOOL  fSClkOff;

  int   nMpegType;              //Bit 0 0=MPEG2, 1=MPEG1, bit 1 0=non VOB, 1=VOB
  int   nScrPauseRef;
  int   fFlipAnchors;
  int   fCssTitleKeySet;
  int   fCssApplyingTitleKey;   //Set indicates Title Key in process
  int   fCssWaitForTitleKey;    //All data transfer disabled until Title Key set or not required
  int   fCssNewTitleKey;        //Set when title key applied but no data transfered
  int   fCssEnabled;            //Set/unSet by hwCssEnableChannel on Css Channel Enable/Disable


  int   nCssApplyTitle;
  int   nCSSUseNew;
  int   nCSSNoKey;


  EEPROM  eeprom;

  PVOID hTvOut;              // Handle for TV out module
  PVOID hOsd;                // Handle for Osd module
  DWORD fWrEEPROMInProgress;
  DWORD dwTmrRunIndicator;
  DWORD dwIsrCntr;
} DVHW;
typedef DVHW *PHW;


// Css Command codes: if bit 7 is '1' then requires start pulse
#define CssCmdNone      0x80
#define CssCmdNoDescram 0x01
#define CssCmdDescram   0x85
#define CssCmdGenCh     0x88
#define CssCmdRdCh      0x89
#define CssCmdWrCh      0x8a
#define CssCmdWrDrvRef  0x8b
#define CssCmdDrvAuth   0x8c
#define CssCmdDecAuth   0x8d
#define CssCmdDiscKey   0x8e
#define CssCmdTitleKey  0x8f

#ifndef midas_h
typedef void* PHWResource_s;
#endif

extern long  dvHwMainConstruct(PHW pHw,PMDS_HW_RESOURCE pRsrc);
extern long  dvHwMainDestruct(PHW pHw);
extern long  dvHwMainDetect(PHW pHw);
extern long  dvHwMainDeviceShutdown(PHW pHw);
extern void  WrByte(PHW pHw,WORD wReg,BYTE Data);
extern BYTE  RdByte(PHW pHw,WORD wReg);
extern void  WrWord(PHW pHw,WORD wReg,WORD Data);
//extern WORD  RdWord(PHW pHw,WORD wReg);
extern void  dvHwMainWriteLsiRegBits(PHW pHw,WORD wReg, BYTE bBits, BYTE bDat);
extern BYTE  dvHwMainReadLsiRegBits(PHW pHw,WORD wReg,BYTE bBits);
extern void  dvHwMainWriteLsiReg(PHW pHw,WORD wReg, BYTE bDat);
extern void  dvHwMainWriteLsiReg32(PHW pHw, WORD Reg, DWORD Data);
extern BYTE  dvHwMainReadLsiReg(PHW pHw,WORD wReg);
extern void  dvHwMainGetChnLevels(PHW pHw);
extern void  dvHwMainClearChCounters(PHW pHw, int nCh);
extern void  dvHwMainClearAllChCounters(PHW pHw);
extern DWORD dvHwMainGetChWrCount(PHW pHw,int nCh);
extern DWORD dvHwMainGetCheckChWrCount(PHW pHw,int nCh);
extern DWORD dvHwMainGetChRdCount(PHW pHw,int nCh);
extern DWORD dvHwGetSubChBytes(PHW pHw);
extern void  dvHwMainClrPesHeader(PHW pHw);
extern BYTE  dvHwMainGetPesHeader(PHW pHw,DWORD* pdwBegin,DWORD* pdwEnd,DWORD* pdwPts);

//extern DWORD dvHwMainAllocateFrameBuffer(PHW pHw,WORD wReg,DWORD dwAddr,DWORD dwSize);
//extern DWORD dvHwMainAllocateChannelBuffer(PHW pHw,WORD wReg,DWORD dwAddr,DWORD dwSize);
extern long  dvHwMainDeviceInit(PHW pHw);
extern void  dvHwMainClrRdyError(PHW pHw);
extern void  dvHwMainReportRdyError(PHW pHw,int nCh);
extern long  dvHwMainStartDecoderChannel(PHW pHw);
extern long  dvHwMainStopDecoderChannel(PHW pHw);
//extern long  dvHwOsdMinModeBuffConfig(PHW pHw,DWORD dwBytes);
extern long  dvHwOsdBuffReserve(PHW,DWORD dwBytes);
extern DWORD dvHwMainGetDramSize(PHW pHw);
extern long  dvHwMainBuffConfig(PHW pHw);
extern void  dvHwMainWriteScrReg(PHW pHw,DWORD dw);
extern int   RegScrPause(PHW pHw, DWORD dwPauseFlags);
extern void  dvHwMainSetScrSlowRate(PHW pHw,DWORD dwRate);
extern DWORD dvHwMainReadScrReg(PHW pHw);
extern void  dvHwMainResetScr(PHW pHw,DWORD dwScrLo,DWORD dwScrHi);

extern long  dvHwMainSetAVOffset(PHW pHw,DWORD dwVidPTS,DWORD dwAudPTS);
extern DWORD dvHwMainReadLsiReg24(PHW pHw, WORD Reg);
extern DWORD dvHwMainReadLsiReg32(PHW pHw, WORD Reg);
extern BYTE  dvHwMainRead8(PHW pHw, WORD Reg);
extern BYTE  dvHwMainGetChannelStatus(PHW pHw);
extern DWORD dvHwMainGetAddr(PHW pHw, WORD wReg);
extern void  dvHwMainSetAddr(PHW pHw, WORD wReg, DWORD dwAddr);
extern long  dvHwMainSetCompareScrReg(PHW pHw,BYTE bReg, DWORD TickCnt);
extern long  dvHwMainStartDecodingNow(PHW pHw);
extern long  dvHwMainResetChannel(PHW pHw);

extern long  dvHwMainReadStatusFlags  (PHW pHw);
extern BYTE  dvHwMainImmediateCheckStatus(PHW pHw,BYTE bReg,BYTE bBits);
extern BYTE  dvHwMainCheckStatus      (PHW pHw,BYTE bReg,BYTE bBits);
extern long  dvHwMainClrStatusLatches (PHW pHw);
extern long  dvHwMainResetStatus      (PHW pHw);

extern long  dvHwMainGetFieldStatus(PHW pHw);
extern long  dvHwMainReadDram(PHW pHw,DWORD dwStartAddr,BYTE *pData,DWORD nBytes);
extern long  dvHwMainWriteDram(PHW pHw,DWORD dwStartAddr,BYTE *pData,DWORD nBytes);
extern long  dvHwMainFillDram(PHW pHw,DWORD dwStartAddr,BYTE* pData,DWORD dwBytes);

//extern long  dvHwMainAutoStart(PHW pHw);
#ifndef _NEC_
  extern BYTE  dvHwMainGetGPIO(PVOID pRef, BYTE bPinNumber, BYTE* pEnable, BYTE* pValue);
  extern void  dvHwMainSetGPIO(PVOID pRef, BYTE bPinNumber, BYTE bEnable, BYTE bValue);
#endif

extern DWORD dvHwMainGetDecoderTime(PHW pHw,DWORD* pdwScrHi);
extern DWORD dvHwMainSetDecoderTime(PHW pHw,DWORD dwNewScrLo,DWORD dwNewScrHi,DWORD dwOldScrLo,DWORD dwOldScrHi);
extern DWORD dvHwMainAdjustDecoderTime(PHW pHw,long lAdjScr);
extern DWORD dvHwMaintainScr(PHW pHw);
extern void  dvHwMainGetTimeDiscontinuity(PHW pHw,DWORD* pdwOldScrHi,DWORD* pdwOldScrLo,DWORD* pdwNewScrHi,DWORD* pdwNewScrLo);



extern void dvHwMainSetLsiAddr(PHW pHw,WORD wLsiAddr);
extern WORD dvHwMainGetLsiAddr(PHW pHw);
extern long dvHwMainEEPROMDataWrite(PHW pHw, LPBYTE lpBuff, DWORD dwSize);
extern long dvHwMainEEPROMDataRead(PHW pHw, LPBYTE lpBuff, DWORD dwSize);

extern DWORD DebugWatch[];
extern BYTE DebugWatchFlags;
#define DBGWATCH_MAXENTRIES	8
#define DBGWATCH_ENABLE		0x01	//Enable DebugWatch on LSI registers
#define DBGWATCH_READ		0x02	//Watch for reads
#define DBGWATCH_WRITE		0x04	//Watch for writes
#define DBGWATCH_INT3		0x08	//Enable break to INT3
#define DBGWATCH_PASSTHRU	0x10	//Pass through once without breaking.

//From Pci.asm
//extern DWORD _cdecl aPciDeviceFind(DWORD DeviceID);
//extern long _cdecl aPciReadData(DWORD PciDevice, void *pDst, DWORD nBytes);

#endif
/**EOF: DVHWMAIN.H**/
