//#include <StrmIni.h>
//#include <mmsystem.h>
//#include <ksmedia.h>
#include "system.h"
#include "DbgMsg.h"
#include "DbgFltr.h"
#include "Idents.h"
#include "margierr.h"
#include "Settings.h"

#include "Lsi64020.h"
#include "MidasApi.h"
#include "dvHwMain.h"
#include "dvHwVid.h"
#include "Global.h"
#include "sync.h"
#include "dvHwSub.h"
#include "IniNames.h"

#define DUMP_SPU 0
#if DUMP_SPU
#pragma message("Warning:  **** Dump SPU enabled ****")
#endif

// define Actual Hardware States
enum {
  C_STATE_NOT_READY =0,
  C_STATE_READY,
  C_STATE_RDY_WAITDATA,
  C_STATE_RDY_PREFILL,
  C_STATE_PLAY_SYNC,
  C_STATE_PLAY,
  C_STATE_PAUSE,
  C_STATE_FLUSH,
  C_STATE_FLUSH_COMPLETE,
  C_STATE_DISCONTINUITY,
  C_STATE_UNK
};

struct IdentEntry_s IdentsSub[]= {
  { ID_SUB_STATE,0 },
    { C_STATE_NOT_READY,      "NotReady" },
    { C_STATE_READY,          "Ready" },
    { C_STATE_RDY_WAITDATA,   "RdyWaitData"},  
    { C_STATE_RDY_PREFILL,    "RdyPrefill"},
    { C_STATE_PLAY_SYNC,      "PlaySync"},
    { C_STATE_PLAY,           "Play"},
    { C_STATE_PAUSE,          "Pause"},
    { C_STATE_FLUSH,          "Flush"},
    { C_STATE_FLUSH_COMPLETE, "FlushComplete"},
    { C_STATE_DISCONTINUITY,  "Discontinuity"},
    { C_STATE_UNK,            "Unknown" },
  { 0,0 }
};

typedef struct SubPicSet_s {
  DWORD dwStream;
  DWORD dwRate;
  DWORD dwDisplayReg;
  DWORD dwEnableTime;
  union {
    struct {
      DWORD Enable:1;
      DWORD DisplayReg:1;
      DWORD NewStream:1;
      DWORD Stream:1;
      DWORD Rate:1;
    }f;
    DWORD dwFlags;
  }update;
} SUBPICSET;

typedef struct HiLiteSet_s {
  DWORD dwColor;
  RECT  rcPosition;
  DWORD dwStartTime;	//SCR at which time to enable highlight.
  DWORD dwEndTime;	  
  int   fEnable;
  DWORD Palette[16];
  int   nPalSize;
  union {
    struct {
      DWORD Color:1;
      DWORD Position:1;
      DWORD Enable:1;
      DWORD Palette:1;
    }f;
    DWORD dwFlags;
  }update;
} HILITESET;

#define SIGNATURE 0xCD010005
typedef struct SubPict_s {
  DWORD Signature;
  PHW   pHw;
  DWORD StrmState;
  DWORD CtrlState;
  DWORD NextState;
  DWORD BeginFlush;
  DWORD EndFlush;
  DWORD fInPumpData;
  DWORD ResetStream;

  QUEUE DataQ;
  PMDS_DATA_BLOCK pFlushMdb;
  DWORD fTransferGap;
  DWORD fDiscontinuity;
  DWORD dw1stSPULength;
  DWORD DataXfrPending;
  DWORD EnableDataTransfer;
  DWORD dwTotalBytes;

//  DWORD ChnlSpacePad;

  DWORD StateTimeoutTime;
  DWORD CurTime;
  DWORD CurSCR;
  DWORD dwStartTime;
  DWORD dwActivateTime;

  DWORD fNewPalette;
  DWORD dwPaletteTime;
  
  SUBPICSET  CurSubPic;
  HILITESET  CurHiLite;

} SUB;

static long SubCmdInvalid(SUB *pSub, DWORD Cmd);
static long SubInit(PHW pHw);
static long SubFini(SUB *pSub);
static long SubResetStateMachine(SUB *pSub);
static long SubStreamState(SUB* pSub, int StrmState);
static long SubStreamReset(SUB *pSub);
static long SubStreamFlush(SUB* pSub,int End,PMDS_DATA_BLOCK pMdb);
static long SubGet(SUB *pSub, DWORD Item, DWORD *pValue);
static long SubSet(SUB *pSub, DWORD Item, DWORD Value);
static long SubTimer(SUB *pSub);
static long SubEnable(SUB *pSub, DWORD Mode);
static long SubSelectStream(SUB *pSub, DWORD Stream);
static long SubHiLiteEnable(SUB *pSub, DWORD Enable,DWORD StartTime,DWORD EndTime);
static long SubPalette(SUB *pSub, DWORD *pPal, DWORD Colors);
static long SubRate(SUB *pSub, MDS_PLAYRATE* pPlayRate);
static DWORD SubSendData(SUB *pSub,PMDS_DATA_PACKET pMdp);
static long SubSetStateTimeout(SUB* pSub, DWORD dwMilliSecs);
static long IsStateTimeout(SUB* pSub);

static long SubProcessNotReady(SUB* pSub);
static long SubProcessReady(SUB* pSub);
static long SubProcessRdyWaitData(SUB* pSub);
static long SubProcessRdyPrefill(SUB* pSub);
static long SubProcessPlaySync(SUB* pSub);
static long SubProcessPlay(SUB* pSub);
static long SubProcessPause(SUB* pSub);
static long SubProcessFlush(SUB* pSub);
static long SubProcessFlushComplete(SUB* pSub);
static long SubProcessDiscontinuity(SUB* pSub);
static long SubStateEnterFlush(SUB* pSub);
static long SubStateEnterReady(SUB* pSub);

#if DUMP_SPU
static void DumpSPUData(PHW pHw);
#endif

static SUB StaticSub;

//Convert a subpicture object handle to a SUB pointer.
static SUB *SubPtr(DWORD hSub) {
  if(hSub && *(DWORD*)hSub==SIGNATURE)
    return((SUB*)hSub);
  return(0);
}


// Reset all timeouts if CurScr has been rewound
static void SubNewTimeDomain(SUB* pSub) {
  pSub->dwStartTime=1;
  pSub->dwActivateTime=0;
  pSub->dwPaletteTime=0;
  pSub->CurSubPic.dwEnableTime=0;
}


long SubpicCmd(PHW pHw, DWORD Cmd, DWORD Arg1, DWORD Arg2) {
  long Err;
  SUB *pSub;
  if(LOWORD(Cmd)==SUBCMD_INIT) {
    Err= SubInit(pHw);
  } else if(!(pSub= SubPtr(pHw->hSubpicture))) {
    Err= MARGI_ERR_BADHANDLE;
  } else {
    DWORD dwSCR= dvHwMainGetDecoderTime(pSub->pHw,0);
//    if(dwSCR < pSub->CurSCR)
//      SubNewTimeDomain(pSub);
    pSub->CurSCR=dwSCR;
    if(LOWORD(Cmd)==SUBCMD_TIMER) {
      Err= SubTimer(pSub);
//    } else if(LOWORD(Cmd)==SUBCMD_DATA) {
//      Err= SubData(pSub,(PMDS_DATA_BLOCK)Arg1);
    } else {
      switch(LOWORD(Cmd)) {
        case SUBCMD_FINI:	      Err= SubFini(pSub); break;
        case SUBCMD_RESET:      Err= SubStreamReset(pSub); break;
        case SUBCMD_STOP:       Err= SubStreamState(pSub,S_STATE_STOP);  break;
        case SUBCMD_RUN:        Err= SubStreamState(pSub,S_STATE_RUN);   break;
        case SUBCMD_PAUSE:      Err= SubStreamState(pSub,S_STATE_PAUSE); break;
        case SUBCMD_BEGINFLUSH: Err= SubStreamFlush(pSub,0,(PMDS_DATA_BLOCK)Arg1); break;
        case SUBCMD_ENDFLUSH:   Err= SubStreamFlush(pSub,1,0); break;
        case SUBCMD_RATE:       Err= SubRate(pSub,(MDS_PLAYRATE*)Arg1); break;
        case SUBCMD_GET:        Err= SubGet(pSub,Arg1,(DWORD*)Arg2); break;
        case SUBCMD_SET:        Err= SubSet(pSub,Arg1,Arg2); break;
        case SUBCMD_ENABLE:     Err= SubEnable(pSub,Arg1); break;
        case SUBCMD_SELECT:	    Err= SubSelectStream(pSub,Arg1); break;
        case SUBCMD_HIDISABLE:  Err= SubHiLiteEnable(pSub,0,Arg1,Arg2); break;
        case SUBCMD_HIENABLE:	  Err= SubHiLiteEnable(pSub,1,Arg1,Arg2); break;
        case SUBCMD_PALETTE:	  Err= SubPalette(pSub,(DWORD*)Arg1,Arg2); break;
        default:		            Err= SubCmdInvalid(pSub,Cmd);
      }
    }
  }
  return(Err);
}

long SubpicQueueData(PHW pHw,PMDS_DATA_BLOCK pMdb) {
  long lErr=0;
  SUB *pSub;
  if(pSub=SubPtr(pHw->hSubpicture)) {
    pMdb->dwCurPacket=0;
    qEnqueue(&pSub->DataQ,pMdb);
  } else
    lErr= MARGI_ERR_BADHANDLE;
  return lErr;
}

long SubpicCancelData(PHW pHw,PMDS_DATA_BLOCK pMdb) {
  long Err=0;
  SUB *pSub;
  if(pSub=SubPtr(pHw->hSubpicture)) {
    if(pMdb) {
      pMdb->dwCurPacket=0;
      pMdb->dwStatus=MDS_Cancel;
      if(qFindAndDequeue(&pSub->DataQ,pMdb)) {
        PMDB_CALLBACK pfnCallback = (PMDB_CALLBACK)pMdb->pfnCallback;
        pfnCallback(pMdb);
      }
    } else {
      qPurgeQueue(&pSub->DataQ,MDS_Cancel);
    }
  } else
    Err= MARGI_ERR_BADHANDLE;
  return 0;
}

DWORD SubpicPumpData(PHW pHw) {
  DWORD dwBytesTransfered=0;
  SUB* pSub;
  if(pSub=SubPtr(pHw->hSubpicture)) {
    PQUEUE pQ;
    PMDS_DATA_BLOCK  pMdb;
    PMDS_DATA_PACKET pMdp;
    pQ=(PQUEUE)&(pSub->DataQ);

    pSub->fInPumpData=1;
    if(pSub->pFlushMdb) {
      PMDB_CALLBACK pfnCallback=(PMDB_CALLBACK)pSub->pFlushMdb->pfnCallback;
      qPurgeQueue(&pSub->DataQ,MDS_Cancel);
      pSub->pFlushMdb->dwStatus=MDS_Success;
      pfnCallback(pSub->pFlushMdb);
      pSub->pFlushMdb=0;
    }
    pMdb=qPeekqueue(pQ);
    if(pMdb && !pSub->BeginFlush) {
      while (dwBytesTransfered < 16384) {
        DWORD dwBytes;
        pMdp=&pMdb->pPackets[pMdb->dwCurPacket];
        dwBytes=SubSendData(pSub,pMdp);
        dwBytesTransfered+=dwBytes;
        if(dwBytes!=pMdp->DataUsed)
          break;
        if(++pMdb->dwCurPacket>=pMdb->dwTotalPackets) {
          PMDB_CALLBACK pfnCallback = (PMDB_CALLBACK)pMdb->pfnCallback;
          pMdb->dwStatus=MDS_Success;
          qDequeue(pQ);
          pfnCallback(pMdb);
          break;
        }
      }
    }
    // If we have transfered data but not this time
    // then flag transfer gap.  This will advance ctrl state
    // from prefill state.
    if(!dwBytesTransfered && pSub->dwTotalBytes)
      pSub->fTransferGap=1;

    pSub->fInPumpData=0;
  }
  return dwBytesTransfered;
}

static long SubCmdInvalid(SUB *pSub, DWORD Cmd) {
  DbgMsg(DBG_ERR,"Sub: Bad command %u (%X)\r\n",LOWORD(Cmd),Cmd);
  return(MARGI_ERR_BADCMD);
}

/*************************************************************************/
/** Init/Fini/Reset							**/
/*************************************************************************/
static long SubInit(PHW pHw) {
  long Err= 0;
  SUB *pSub;
  pSub= &StaticSub;					//Replace with dynamic memory allocation
  if(!pSub) {
    Err= MARGI_ERR_NOMEM;
  } else {
    memset(pSub,0,sizeof(SUB));
    pSub->Signature= SIGNATURE;
    pSub->pHw= pHw;
    pHw->ChInf[SUBCH].dwSafety=SettingLoad(SET_DWORD,IniSubChnlSpacePad,0,2048)/8;
    pHw->hSubpicture= (DWORD)pSub;
    SubResetStateMachine(pSub);
    qInitQueue(&pSub->DataQ);
  }
  return(Err);
}

static long SubFini(SUB *pSub) {
  pSub->Signature= 0;
  pSub->pHw->hSubpicture= 0;
  //Free pSub
  return(0);
}

static void ResetSubPic(SUBPICSET* pCur) {
  pCur->update.dwFlags=0;
  pCur->dwDisplayReg=SPU_DISPLAY_OFF;
  pCur->dwStream=0;
  pCur->dwRate=RATE_NORMAL;
  pCur->update.f.DisplayReg=1;
  pCur->update.f.Stream=1;
  pCur->update.f.Rate=1;
}

static void ResetHiLite(HILITESET* pCur) {
  pCur->update.dwFlags=0;
  pCur->dwColor=0;
  pCur->dwStartTime=0;
  pCur->fEnable=0;
  pCur->nPalSize=0;
  pCur->update.f.Enable=1;
}

static void UpdateSubPic(SUB* pSub) {
  SUBPICSET* pCur=&pSub->CurSubPic;

#ifdef _NEC_DBG_
  {
    BYTE bStat;
    bStat=dvHwMainReadLsiReg(pSub->pHw,458);
    if(bStat) {
      volatile int i=0;
      LOGTIME(LOGSUBSTAT,bStat);
    }
  }
#endif

  if(pCur->update.f.Stream) {
    pCur->update.f.Stream=0;
    if(pCur->dwStream>31) {
      dvHwMainWriteLsiRegBits(pSub->pHw,SPU_JUMP_TO_NEXT_REG,0x20);
      dvHwMainWriteLsiRegBits(pSub->pHw,SPU_STREAM_ENABLE_REG,FALSE);
      pCur->dwStream=(DWORD)-1;
    } else {
      dvHwMainWriteLsiReg(pSub->pHw,146,(BYTE)(pCur->dwStream|0x20));
      pCur->update.f.NewStream=1;
      pCur->update.f.DisplayReg=1;
    }
  }
  if(pCur->update.f.Rate) {
    pCur->update.f.Rate=0;
    if(pCur->dwRate!=RATE_NORMAL)
      pCur->dwDisplayReg|=0x20;      // Jump to Next
    pCur->update.f.DisplayReg=1;
  }
  if(pSub->CurHiLite.update.f.Palette) {
    pCur->dwDisplayReg|=0x10;   // SPU_RESET_PALETTE
    pCur->update.f.DisplayReg=1;
  }

  if(pCur->update.f.Enable && (!pCur->dwEnableTime || tmCompareTimes(pSub->CurSCR,pCur->dwEnableTime))) {
    pCur->update.f.Enable=0;
    pCur->dwEnableTime=0;
    pCur->update.f.DisplayReg=1;
  }

  if(pCur->update.f.DisplayReg) {
    BYTE bReg=(BYTE)pCur->dwDisplayReg;
    pCur->update.f.DisplayReg=0;
    pCur->dwDisplayReg&=0xcf;        // Clear Jump to Next,Reset Palette
    // Disable SPU until time for new stream (dwEnableTime is set by SubSendData)
    if(pCur->dwEnableTime || pCur->update.f.NewStream) 
      bReg|=0x04;
    dvHwMainWriteLsiReg(pSub->pHw,416,bReg);
    LOGTIME(LOGSUBCMD,dvHwMainReadScrReg(pSub->pHw));
    LOGTIME(LOGSUBCMD,bReg);
  }
}

static void UpdateHiLite(SUB* pSub) {
  HILITESET* pCur=&pSub->CurHiLite;

  if(pCur->update.f.Color) {
    pCur->update.f.Color=0;
    dvHwMainWriteLsiReg(pSub->pHw,HIGHLIGHT_COLOR_REG,     (BYTE)(pCur->dwColor>> 8));
    dvHwMainWriteLsiReg(pSub->pHw,HIGHLIGHT_COLOR_REG+1,   (BYTE)(pCur->dwColor    ));
    dvHwMainWriteLsiReg(pSub->pHw,HIGHLIGHT_CONTRAST_REG,  (BYTE)(pCur->dwColor>>24));
    dvHwMainWriteLsiReg(pSub->pHw,HIGHLIGHT_CONTRAST_REG+1,(BYTE)(pCur->dwColor>>16));
  }
  if(pCur->update.f.Position) {
    WORD wReg= HIGHLIGHT_POSITION_REG;
    WORD wData;
    RECT* pPos=&pCur->rcPosition;
    pCur->update.f.Position=0;
    wData = (WORD)(pPos->top<<12);
    wData |= (pPos->bottom&0x03FF);
    dvHwMainWriteLsiReg(pSub->pHw,wReg++,(BYTE)wData);
    dvHwMainWriteLsiReg(pSub->pHw,wReg++,(BYTE)(wData>>8));
    wData = (WORD)(pPos->right<<8);
    wData |= ((pPos->top>>4)&0x3F);
    dvHwMainWriteLsiReg(pSub->pHw,wReg++,(BYTE)wData);
    dvHwMainWriteLsiReg(pSub->pHw,wReg++,(BYTE)(wData>>8));
    wData = (WORD)((pPos->left<<4)&0x3FFF);
    wData |= ((pPos->right>>8)&0x03);
    dvHwMainWriteLsiReg(pSub->pHw,wReg++,(BYTE)wData);
    dvHwMainWriteLsiReg(pSub->pHw,wReg,(BYTE)(wData>>8));
  }
  if(pCur->update.f.Palette && tmCompareTimes(pSub->CurSCR,pSub->dwPaletteTime)) {
    int n;
    DWORD* pPal=pCur->Palette;
    pCur->update.f.Palette=0;
    for(n=0;n<pCur->nPalSize;n++) {
      dvHwMainWriteLsiReg(pSub->pHw,SPU_PALETTE_DATA_REG,(BYTE)(pPal[n]>>8));
      dvHwMainWriteLsiReg(pSub->pHw,SPU_PALETTE_DATA_REG,(BYTE)(pPal[n]>>16));
      dvHwMainWriteLsiReg(pSub->pHw,SPU_PALETTE_DATA_REG,(BYTE)(pPal[n]>>24));
    }
    LOGTIME(LOGSUBPAL,dvHwMainReadScrReg(pSub->pHw));
  }
  if(pCur->update.f.Enable && (!pCur->dwStartTime || tmCompareTimes(pSub->CurSCR,pCur->dwStartTime))) {
    pCur->update.f.Enable=0;
    pCur->dwStartTime=pSub->CurSCR;
    DbgMsg(DBG_SUBHI,"Sub: Deferred HiLite %se occurring at ",Ident(ID_ENABL,pCur->fEnable));
    DbgDispPTS(DBG_SUBHI,pSub->CurSCR,"","");
    DbgDispPTS(DBG_SUBHI,pCur->dwStartTime," Target=","\r\n");
    dvHwMainWriteLsiRegBits(pSub->pHw,HIGHLIGHT_ENABLE_REG,(BYTE)pCur->fEnable);
  }
}

static long SubResetStateMachine(SUB *pSub) {
  pSub->CtrlState=C_STATE_NOT_READY;
  pSub->StrmState=S_STATE_STOP;
  pSub->ResetStream=0;
  pSub->BeginFlush=0;
  pSub->EndFlush=0;
  pSub->dwTotalBytes=0;
  pSub->EnableDataTransfer=0;
  ResetSubPic(&pSub->CurSubPic);
  ResetHiLite(&pSub->CurHiLite);
  return(0);
}

// Force State machine to pass thru READY to re-initialize
// this will not change the stream state.
static long SubStreamReset(SUB *pSub) {
  DbgMsg(DBG_SUBSTATE,"SubPic: Reset State.\r\n");
  pSub->ResetStream=1;
  pSub->EnableDataTransfer=0;
  return 0;
}


static long SubStreamState(SUB* pSub, int StrmState) {
  long Err= 0;
  DbgMsg(DBG_SUBSTATE,"SubPic: Stream State %s -> %s.",Ident(ID_STRM_STATE,pSub->StrmState),Ident(ID_STRM_STATE,StrmState));
  DbgDispPTS(DBG_SUBSTATE,pSub->CurSCR," (Time=",")\r\n");
  pSub->StrmState = StrmState;
  return Err;
}

static long SubStreamFlush(SUB* pSub,int End,PMDS_DATA_BLOCK pFlushMdb) {
  long Err= 0;
  if(End)
    pSub->EndFlush=1;
  else {
    pSub->BeginFlush=1;
    pSub->pFlushMdb=pFlushMdb;
    if(!pFlushMdb) {
      while(pSub->fInPumpData)
        MargiSleep(20);
      qPurgeQueue(&pSub->DataQ,MDS_Cancel);
    }
  }
  return Err;
}

static long SubDetermineNextState(SUB* pSub) {
  int fDone=0;
  PHW pHw= pSub->pHw;
  pSub->NextState=pSub->CtrlState;

  // If Stream State is stop or stream is reset then all 
  // states except NOT_READY will transition to C_STATE_READY
  if(pSub->StrmState==S_STATE_STOP || pSub->ResetStream) {
    pSub->ResetStream=0;
    switch(pSub->CtrlState) {
      case C_STATE_NOT_READY:
        break;
      default:
        pSub->NextState=C_STATE_READY;
        break;
    }
    fDone=1;
  }
  // All states except NOT_READY,READY and FLUSHCOMPLETE will
  // transition to FLUSH if BeginFlush is set
  if(!fDone && pSub->BeginFlush) {
    switch(pSub->CtrlState) {
      case C_STATE_NOT_READY:
      case C_STATE_READY:
        break;
      default:
        pSub->NextState=C_STATE_FLUSH;
        fDone=1;
        break;
    }
    pSub->BeginFlush=0;
  }
  if(!fDone) {
    switch(pSub->CtrlState) {
      default:
      case C_STATE_NOT_READY:
        if(pSub->StrmState==S_STATE_RUN)
          pSub->NextState=C_STATE_READY;
        break;
   
      case C_STATE_READY:
        if((pSub->StrmState==S_STATE_RUN) && pHw->u.s.fChannelConfig)
          pSub->NextState=C_STATE_RDY_WAITDATA;
        break;
    
      case C_STATE_RDY_WAITDATA:
        if(!pHw->u.s.fChannelConfig)
          pSub->NextState=C_STATE_READY;
        else if(pSub->DataXfrPending && !pHw->fCssWaitForTitleKey) {
          pSub->NextState=C_STATE_RDY_PREFILL;
        }
        break;

      case C_STATE_RDY_PREFILL:
        if(!pHw->u.s.fChannelConfig)
          pSub->NextState=C_STATE_READY;
        else if(pSub->EnableDataTransfer==0)
          pSub->NextState=C_STATE_DISCONTINUITY;
        else if((pHw->ChInf[SUBCH].dwItems*8)>=pSub->dw1stSPULength || pSub->fTransferGap) {
          pSub->NextState=C_STATE_PLAY_SYNC;
          LOGTIME(LOGSUBPLAYSYNC,dvHwMainReadScrReg(pSub->pHw));
        }
        break;
    
      case C_STATE_PLAY_SYNC:
        if(!pHw->u.s.fChannelConfig)
          pSub->NextState=C_STATE_READY;
        else if(pSub->EnableDataTransfer==0)
          pSub->NextState=C_STATE_DISCONTINUITY;
        else if(pSub->dwStartTime && tmCompareTimes(pSub->CurSCR,pSub->dwStartTime)) {
          DbgMsg(DBG_SUBSTATE,"Subpic: state %s transition ",Ident(ID_SUB_STATE,pSub->CtrlState));
          DbgDispPTS(DBG_SUBSTATE,pSub->CurSCR,"Time="," ");
          DbgDispPTS(DBG_SUBSTATE,pSub->dwStartTime,"Target=","\r\n");
          LOGTIME(LOGSUBPLAY,dvHwMainReadScrReg(pSub->pHw));
          pSub->NextState=C_STATE_PLAY;
        }
        break;
    
      case C_STATE_PLAY:
        if(pSub->EnableDataTransfer==0)
          pSub->NextState=C_STATE_DISCONTINUITY;
        else if(pSub->StrmState==S_STATE_PAUSE)
          pSub->NextState=C_STATE_PAUSE;
        break;
    
      case C_STATE_PAUSE:
        if(pSub->StrmState==S_STATE_RUN)
          pSub->NextState=C_STATE_PLAY;
        break;

      case C_STATE_DISCONTINUITY:
        if(!pHw->u.s.fInPumpData)
          pSub->NextState=C_STATE_READY;
        break;

      case C_STATE_FLUSH:
        if(!pSub->dwActivateTime || tmCompareTimes(pSub->CurSCR,pSub->dwActivateTime))
          pSub->NextState=C_STATE_FLUSH_COMPLETE;
        break;

      case C_STATE_FLUSH_COMPLETE:
        if(pSub->EndFlush && !pHw->u.s.fInPumpData)
          pSub->NextState=C_STATE_READY;
        break;
    }
  }
  return 0;
}

static long SubStateInvalid(SUB *pSub) {
  DbgMsg(DBG_HWERR|DBG_SUBSTATE,"SubPic: Invalid state transition %s -> %s.\r\n",Ident(ID_SUB_STATE,pSub->CtrlState),Ident(ID_SUB_STATE,pSub->NextState));
  return(MARGI_ERR_INVALID_STATE_TRANSITION);
}

static long SubSetStateTimeout(SUB* pSub, DWORD dwMilliSecs) {
  if(dwMilliSecs) {
    pSub->StateTimeoutTime=pSub->CurTime+dwMilliSecs;
    if(pSub->StateTimeoutTime==0)
      pSub->StateTimeoutTime++;
  } else
    pSub->StateTimeoutTime=0;
  return 0;
}

static long IsStateTimeout(SUB* pSub) {
  return (pSub->StateTimeoutTime && (pSub->CurTime>=pSub->StateTimeoutTime));
}

// Do the actual work in transitioning from one state to
// the next.
static long SubProcessState(SUB* pSub) {
  long Err=0;
  if(pSub->CtrlState != pSub->NextState) {
    DbgMsg(DBG_SUBSTATE,"SubPic:state %s -> %s.",Ident(ID_SUB_STATE,pSub->CtrlState),Ident(ID_SUB_STATE,pSub->NextState));
    DbgDispPTS(DBG_SUBSTATE,pSub->CurSCR," (Time=",")\r\n");
    SubSetStateTimeout(pSub,0);   // If state transition about to occur then clear state timeout
  }

  switch (pSub->CtrlState) {
    default:
    case C_STATE_NOT_READY:
      Err=SubProcessNotReady(pSub);
      break;

    case C_STATE_READY:
      Err=SubProcessReady(pSub);
      break;

    case C_STATE_RDY_WAITDATA:
      Err=SubProcessRdyWaitData(pSub);
      break;

    case C_STATE_RDY_PREFILL:
      Err=SubProcessRdyPrefill(pSub);
      break;

    case C_STATE_PLAY_SYNC:
      Err=SubProcessPlaySync(pSub);
      break;

    case C_STATE_PLAY:
      Err=SubProcessPlay(pSub);
      break;

    case C_STATE_PAUSE:
      Err=SubProcessPause(pSub);
      break;

    case C_STATE_DISCONTINUITY:
      Err=SubProcessDiscontinuity(pSub);
      break;

    case C_STATE_FLUSH:
      Err=SubProcessFlush(pSub);
      break;

    case C_STATE_FLUSH_COMPLETE:
      Err=SubProcessFlushComplete(pSub);
      break;
 }
  return Err;
}

static long SubProcessNotReady(SUB* pSub) {
  long Err=0;
  switch(pSub->NextState) {
    case C_STATE_NOT_READY:
      break;
    case C_STATE_READY:
      Err=SubStateEnterReady(pSub);
      break;
    default:
      Err=SubStateInvalid(pSub);
  }
  return Err;
}

static long SubProcessReady(SUB* pSub) {
  long Err=0;
  switch(pSub->NextState) {
    case C_STATE_READY:
      if(pSub->ResetStream)
        Err=SubStateEnterReady(pSub);
      break;
    case C_STATE_RDY_WAITDATA:
      SubSetStateTimeout(pSub,250);
      pSub->dwStartTime=0;
      pSub->dw1stSPULength=0;
      pSub->dwTotalBytes=0;
      pSub->fTransferGap=0;
      pSub->CtrlState=pSub->NextState;
      break;
    default:
      Err=SubStateInvalid(pSub);
  }
  return Err;
}

static long SubProcessRdyWaitData(SUB* pSub) {
  long Err=0;
  switch(pSub->NextState) {
    case C_STATE_RDY_WAITDATA:
      break;
    case C_STATE_RDY_PREFILL:
      pSub->CurSubPic.update.f.Stream=1;
      pSub->CurHiLite.update.f.Color=1;
      pSub->CurHiLite.update.f.Position=1;
      pSub->CurHiLite.update.f.Enable=1;
      pSub->CurHiLite.update.f.Palette=1;
      pSub->EnableDataTransfer=1;

      pSub->CtrlState=pSub->NextState;
      break;
    case C_STATE_DISCONTINUITY:
      pSub->CurSubPic.dwDisplayReg|=0x20;        //Jump to Next
      pSub->CurSubPic.update.f.DisplayReg=1;
      pSub->CtrlState=pSub->NextState;
      break;
    case C_STATE_FLUSH:
      Err=SubStateEnterFlush(pSub);
      break;
    default:
      Err=SubStateInvalid(pSub);  // fall thru to Ready
    case C_STATE_READY:
      Err=SubStateEnterReady(pSub);
      break;
  }
  return Err;
}

static long SubProcessRdyPrefill(SUB* pSub) {
  long Err=0;
  switch(pSub->NextState) {
    case C_STATE_RDY_PREFILL:
      break;
    case C_STATE_PLAY_SYNC:
      pSub->CurSubPic.dwDisplayReg|=0x01;        //Start Decoder
      pSub->CurSubPic.update.f.DisplayReg=1;
      pSub->CtrlState=pSub->NextState;
      break;
    case C_STATE_DISCONTINUITY:
      pSub->CurSubPic.dwDisplayReg|=0x20;        //Jump to Next
      pSub->CurSubPic.update.f.DisplayReg=1;
      pSub->CtrlState=pSub->NextState;
      break;
    case C_STATE_FLUSH:
      Err=SubStateEnterFlush(pSub);
      break;
    default:
      Err=SubStateInvalid(pSub);  // fall thru to Ready
    case C_STATE_READY:
      Err=SubStateEnterReady(pSub);
      break;
  }
  return Err;
}

static long SubProcessPlaySync(SUB* pSub) {
  long Err=0;
  switch(pSub->NextState) {
    case C_STATE_PLAY_SYNC:
      break;
    case C_STATE_PLAY:
      pSub->CurSubPic.dwDisplayReg&=0xfd;        //Un-Pause decoder.
      pSub->CurSubPic.update.f.DisplayReg=1;
#if DUMP_SPU
      DumpSPUData(pSub->pHw);
#endif
      pSub->CtrlState=pSub->NextState;
      break;
    case C_STATE_DISCONTINUITY:
      pSub->CurSubPic.dwDisplayReg|=0x20;        //Jump to Next
      pSub->CurSubPic.update.f.DisplayReg=1;
      pSub->CtrlState=pSub->NextState;
      break;
    case C_STATE_FLUSH:
      Err=SubStateEnterFlush(pSub);
      break;
    default:
      Err=SubStateInvalid(pSub);  // fall thru to Ready
    case C_STATE_READY:
      Err=SubStateEnterReady(pSub);
      break;
  }
  return Err;
}

static long SubProcessPlay(SUB* pSub) {
  long Err=0;
  switch (pSub->NextState) {
    case C_STATE_PLAY:
      break;
    case C_STATE_PAUSE:
      pSub->CurSubPic.dwDisplayReg|=0x02;        //Pause decoder.
      pSub->CurSubPic.update.f.DisplayReg=1;
      pSub->CtrlState=pSub->NextState;
      break;
    case C_STATE_DISCONTINUITY:
      pSub->CurSubPic.dwDisplayReg|=0x20;        //Jump to Next
      pSub->CurSubPic.update.f.DisplayReg=1;
      pSub->CtrlState=pSub->NextState;
      break;
    case C_STATE_FLUSH:
      Err=SubStateEnterFlush(pSub);
      break;
    default:
      Err=SubStateInvalid(pSub);  // fall thru to Ready
    case C_STATE_READY:
      Err=SubStateEnterReady(pSub);
      break;
  }
  return Err;
}

static long SubProcessPause(SUB* pSub) {
  long Err=0;
  switch (pSub->NextState) {
    case C_STATE_PAUSE:
      break;
    case C_STATE_PLAY:
      pSub->CurSubPic.dwDisplayReg&=0xfd;        //Un-Pause decoder.
      pSub->CurSubPic.update.f.DisplayReg=1;
      pSub->CtrlState=pSub->NextState;
      break;
    case C_STATE_FLUSH:
      Err=SubStateEnterFlush(pSub);
      break;
    default:
      Err=SubStateInvalid(pSub);  // fall thru to Ready
    case C_STATE_READY:
      Err=SubStateEnterReady(pSub);
      break;
  }
  return Err;
}

static long SubProcessDiscontinuity(SUB* pSub) {
  long Err=0;
  switch (pSub->NextState) {
    case C_STATE_DISCONTINUITY:
      break;
    case C_STATE_FLUSH:
      Err=SubStateEnterFlush(pSub);
      break;
    default:
      Err=SubStateInvalid(pSub);  // fall thru to Ready
    case C_STATE_READY:
      Err=SubStateEnterReady(pSub);
      break;
  }
  return Err;
}

static long SubProcessFlush(SUB* pSub) {
  long Err=0;
  switch (pSub->NextState) {
    case C_STATE_FLUSH:
    break;
    case C_STATE_FLUSH_COMPLETE:
      dvHwMainWriteLsiRegBits(pSub->pHw,RESET_FIFO_REG,RESET_FIFO_SUB); //Flush Decoder channel
      pSub->CurSubPic.dwDisplayReg&=0xfe;        //Stop Decoder
      pSub->CurSubPic.update.f.DisplayReg=1;
      pSub->EnableDataTransfer=0;
      pSub->CtrlState=C_STATE_FLUSH_COMPLETE;
      break;
    default:
      Err=SubStateInvalid(pSub);  // fall thru to Ready
    case C_STATE_READY:
      Err=SubStateEnterReady(pSub);
      break;
  }
  return Err;
}

static long SubProcessFlushComplete(SUB* pSub) {
  long Err=0;
  switch (pSub->NextState) {
    case C_STATE_FLUSH_COMPLETE:
    break;
    default:
      Err=SubStateInvalid(pSub);  // fall thru to Ready
    case C_STATE_READY:
      Err=SubStateEnterReady(pSub);
      break;
  }
  return Err;
}

static long SubStateEnterFlush(SUB* pSub) {
  long Err=0;
  pSub->ResetStream=0;
  pSub->BeginFlush=0;
  pSub->EnableDataTransfer=0;
  SubSetStateTimeout(pSub,250);
  pSub->CtrlState=C_STATE_FLUSH;
  return 0;
}
 
static long SubStateEnterReady(SUB* pSub) {
  long Err=0;
  pSub->ResetStream=0;
  pSub->dwActivateTime=0;
  pSub->EndFlush=0;
  pSub->dwStartTime=0;
  pSub->EnableDataTransfer=0;
  pSub->DataXfrPending=0;
  pSub->fTransferGap=0;
  pSub->fDiscontinuity=1;
  pSub->dwTotalBytes=0;
  dvHwMainWriteLsiRegBits(pSub->pHw,274,0x20,0x20);   //Chroma Filter Enable
  dvHwMainWriteLsiRegBits(pSub->pHw,309,0x02,0x02);   //Mix Enable
  dvHwMainWriteLsiRegBits(pSub->pHw,HIGHLIGHT_ENABLE_REG,0);

  pSub->CtrlState=C_STATE_READY;
  return 0;
}

/*************************************************************************/
/** Get/Set parameters.							**/
/*************************************************************************/

static long SubGet(SUB *pSub, DWORD Item, DWORD *pValue) {
  long Err=MARGI_ERR_BADPTR;
  if(pValue) {
    switch(Item) {
      case SUBITEM_STATE:   *pValue= pSub->StrmState; break;
      case SUBITEM_HICOLOR:	*pValue= pSub->CurHiLite.dwColor; break;
      case SUBITEM_HIPOS:		*((RECT*)pValue)=pSub->CurHiLite.rcPosition; break;
      default:              Err= MARGI_ERR_BADITEM;
    }
  }
  return(Err);
}

static long SubSet(SUB *pSub, DWORD Item, DWORD Value) {
  long Err=MARGI_ERR_BADPTR;;
  switch(Item) {
    case SUBITEM_HICOLOR:	pSub->CurHiLite.dwColor=Value;
                          pSub->CurHiLite.update.f.Color=1;
                          break;
    case SUBITEM_HIPOS:	  if(Value) {
                            pSub->CurHiLite.rcPosition=*((RECT*)Value);
                            pSub->CurHiLite.update.f.Position=1;
                          }
                          break;
    default:              Err= MARGI_ERR_BADITEM;
  }
  return(Err);
}

/*************************************************************************/
/** Timer tick								**/
/*************************************************************************/
static long SubTimer(SUB *pSub) {
  pSub->CurTime=MargiGetTime();
  SubDetermineNextState(pSub);
  SubProcessState(pSub);
  if(pSub->CurSubPic.update.dwFlags)
    UpdateSubPic(pSub);
  if(pSub->CurHiLite.update.dwFlags)
    UpdateHiLite(pSub);
  return 0;
}

/*************************************************************************/
/** Miscellaneous subpicture commands.					**/
/*************************************************************************/
static long SubEnable(SUB *pSub, DWORD Mode) {
  long Err= 0;
  SUBPICSET* pCur=&pSub->CurSubPic;
  pCur->dwDisplayReg&=~(SPU_DISPLAY_FORCE_OFF|SPU_DISPLAY_OFF);
  if(pCur->dwRate==RATE_NORMAL) {
    switch(Mode) {
      case SPU_OFF:       pCur->dwDisplayReg|=SPU_DISPLAY_OFF;       break;
      case SPU_FORCE_OFF: pCur->dwDisplayReg|=SPU_DISPLAY_FORCE_OFF; break;
      case SPU_ON:
      default:            pCur->dwDisplayReg|=SPU_DISPLAY_ON;        break;
    }
  } else
    pCur->dwDisplayReg|=SPU_DISPLAY_FORCE_OFF;
  pCur->update.f.DisplayReg=1;
  return Err;
}

static long SubSelectStream(SUB *pSub, DWORD Stream) {
  if(pSub->CurSubPic.dwStream!=Stream) {
    pSub->CurSubPic.dwStream=Stream;
    pSub->CurSubPic.update.f.Stream=1;
  }
  return 0;
}

static long SubHiLiteEnable(SUB *pSub, DWORD Enable, DWORD StartTime, DWORD EndTime) {
  pSub->CurHiLite.fEnable=Enable;
  if(!Enable && pSub->dwActivateTime) {
    pSub->CurHiLite.dwStartTime=tmCompareTimes(StartTime,pSub->dwActivateTime)?StartTime:pSub->dwActivateTime;
    pSub->CurHiLite.dwEndTime=  EndTime;
  } else { 
    pSub->CurHiLite.dwStartTime=StartTime;
    pSub->CurHiLite.dwEndTime=  EndTime;
  }
  // dwActivateTime is used to hold off disable Hilite and
  // Flush subpicture for a fraction of a second.  Makes up
  // for poor timing in the MS Navigator when showing 
  // action hilite.
  if(Enable)
    pSub->dwActivateTime=pSub->CurSCR+12500;

  pSub->CurHiLite.update.f.Enable=1;
  return 0;
}

static long SubPalette(SUB *pSub, DWORD *pPal, DWORD Colors) {
  long Err=0;
  if(!pPal) {
    Err= MARGI_ERR_BADPTR;
  } else {
    if(Colors>16) Colors=16;
    memcpy(pSub->CurHiLite.Palette,pPal,Colors*sizeof(*pPal));
    pSub->CurHiLite.nPalSize=Colors;
    pSub->fNewPalette=1;
  }
  return Err;
}

static long SubRate(SUB *pSub, MDS_PLAYRATE* pRate) {
  long Err=0;
  pSub->pHw->dwSubRate= pRate->Rate;
  pSub->CurSubPic.dwRate=pRate->Rate;
  pSub->CurSubPic.update.f.Rate=1;
  return Err;
}

static DWORD SubGetPacketScr(BYTE* pData,int nBytes) {
  DWORD dwScr=0;
  if(nBytes>=4 && pData[3]==0xBA) {	        		//If pack start code
    pData+= 4; 						                      //Skip packet start code prefix
    nBytes-= 4;
    if((*pData&0xC0)==0x40 && nBytes>=5) {			//If Mpeg2
      dwScr = ((DWORD)(pData[0]&0x38))<<27;			//Bits 31 & 30
      dwScr|= ((DWORD)(pData[0]&0x03))<<28;			//Bits 29 & 28
      dwScr|= ((DWORD) pData[1])<<20;			        //Bits 27 to 20
      dwScr|= ((DWORD)(pData[2]&0xF8))<<12;			//Bits 19 to 15
      dwScr|= ((DWORD)(pData[2]&0x03))<<13;			//Bits 14 & 13
      dwScr|= ((DWORD) pData[3])<<5;			        //Bits 12 to 5
      dwScr|= ((DWORD)(pData[4]&0xF8))>>3;			//Bits 4 to 2
    } else if(nBytes>=5) {
      dwScr = ((DWORD)(pData[0]&0x0E))<<29;
      dwScr|= ((DWORD) pData[1])<<22;
      dwScr|= ((DWORD)(pData[2]&0xFE))<<14;
      dwScr|= ((DWORD) pData[3])<<7;
      dwScr|= (DWORD)( pData[4]>>1);
    }
  }
  return dwScr;
}

static DWORD SubGetSPULen(BYTE* pData,int nBytes) {
  int   xLen;
  DWORD dwSPULen=0;
  if(nBytes>=128) {
    if(pData[0x11]==0xBD) {	        		//If private data start code
      xLen=pData[0x16];
      dwSPULen=(pData[0x18+xLen]*256)+pData[0x19+xLen];
    }
  }
  return dwSPULen;
}

#define MDB_FLAG_USENEWCSSKEY       0x00000001
#define MDB_FLAG_DATADISCONTINUITY  0x00000004

/*************************************************************************/
/** Send data to decoder.						**/
/*************************************************************************/
static DWORD SubSendData(SUB *pSub,PMDS_DATA_PACKET pMdp) {
  long Err= 0;
  DWORD dwBytesWritten=0;
  int   fTimeDiscon=0;
  PHW pHw= pSub->pHw;
  SYNC_TIME90 Time90;

  if(pMdp) {
    if(pMdp->OptionsFlags&MDB_FLAG_DATADISCONTINUITY) {
      if(pSub->CtrlState==C_STATE_PLAY || pSub->CtrlState==C_STATE_PLAY_SYNC) {
        // If we respond to flag then clear so we don't see it again
        pMdp->OptionsFlags&=~MDB_FLAG_DATADISCONTINUITY;
        pSub->EnableDataTransfer=0;
        pSub->fDiscontinuity=1;
      }
    }
    if(pMdp->DataUsed) {
      if(pHw->fTimeFormat10M)
        tmConvertTime10Mto90K(&pMdp->PresentationTime.Time,&Time90);
      else {
        Time90.dwTimeLo= (DWORD)GetLow_MdsLongLong (&pMdp->PresentationTime.Time);
        Time90.dwTimeHi= (DWORD)GetHigh_MdsLongLong(&pMdp->PresentationTime.Time);
      }

      if(pSub->EnableDataTransfer && !pHw->fCssWaitForTitleKey) {
        DWORD dwSpace;
        if(Time90.dwTimeLo) {
          // dwPaletteTime is used to synchronize palette with subpicture data
          if(pSub->fNewPalette) {
            pSub->dwPaletteTime=Time90.dwTimeLo;
            pSub->CurHiLite.update.f.Palette=1;
            pSub->fNewPalette=0;
          }
          // dwEnableTime is used to synchronize disable/enable for stream change
          if(pSub->CurSubPic.update.f.NewStream) {
            pSub->CurSubPic.dwEnableTime=Time90.dwTimeLo;
            pSub->CurSubPic.update.f.Enable=1;
            pSub->CurSubPic.update.f.NewStream=0;
          }
          // Don't transfer if Subpicture is more than 0.25 secs early
          // and we have just had a data ot time discontinuity
          // this fixes WDM problem of subpicture arriving for menu before
          // end of previous video has finished playing
          if(pSub->fDiscontinuity && tmCompareTimes(Time90.dwTimeLo,pSub->CurSCR+90*250))
            return 0;
        }
        // If subpicture time is current when the subpicture is being transfered then 
        // the decoder will underflow and discard the subpicture.  For the NAV Test Disk karaoke
        // Test the subpicture comes in multiple sectors and its time is about 1/2 sec and indefinate. 
        // If we back off the SCR then the -ve SCR is considered positive and valid for the subpicture
        // So the decoder underflows.
        if(pSub->pHw->dwScrHi==0xffffffff)
          return 0;

        // If a Time Discontinuity is pending then don't transfer subpicture
        if(pSub->pHw->bIsNewScr)
          return 0;

        pSub->fDiscontinuity=0;
        if((dwSpace=pHw->ChInf[SUBCH].dwSize-pHw->ChInf[SUBCH].dwItems)>pHw->ChInf[SUBCH].dwSafety)
          dwSpace-=pHw->ChInf[SUBCH].dwSafety;
        else
          dwSpace=0;

        if(dwSpace*8>=pMdp->DataUsed+8192) {

          pSub->dwTotalBytes+=pMdp->DataUsed;
          DbgMsg(DBG_SUBSTATE,"Sub: Transfer %d bytes.",pMdp->DataUsed);
          DbgDispPTS(DBG_SUBSTATE,pSub->CurSCR," (Time=",")\r\n");
          if(Time90.dwTimeLo) {
            LOGTIME(LOGSUBPTS,Time90.dwTimeLo);
          }
          LOGTIME(LOGSUBTRANSFER,dvHwMainReadScrReg(pSub->pHw));
          if(Err= TransferData(pHw,pMdp->Data,pMdp->DataUsed,&dwBytesWritten,STRM_SUB)) {
            DbgMsg(DBG_ERR,"Sub: Unable to transfer data: %s.\r\n",Ident(ID_ERR,Err));
            if(Err==MARGI_ERR_WRT_FAILED) {   //RdyErr
              dvHwMainClrRdyError(pSub->pHw);
              dvHwMainReportRdyError(pSub->pHw,SUBCH);
            }
          }
          pHw->ChInf[SUBCH].dwItems+=dwBytesWritten/8;
        }
      } else {
        pSub->DataXfrPending++;
        if(pSub->dwStartTime==0) {
          // dwStartTime is used to synchronize decoder being un-paused
          // on transition to PLAY state
          pSub->dwStartTime=Time90.dwTimeLo-90*25;
          DbgDispPTS(DBG_SUBSTATE,pSub->dwStartTime,"Sub: Start Time=","\r\n");
        }
        // Get length of 1st SPU for Prefill
        if(pSub->dw1stSPULength==0) {
          pSub->dw1stSPULength=SubGetSPULen(pMdp->Data,pMdp->DataUsed);
        }
      }
    }
    if(pMdp->OptionsFlags&MDB_FLAG_TIMEDISCONTINUITY) {
      TransferFlushPkt(pHw,FLUSH_PIPE);
      pSub->fDiscontinuity=1;
    }
  }
  return dwBytesWritten;
}

#if DUMP_SPU
int fDump=1;

BYTE DramBuffer[65536];

void DumpSPUData(PHW pHw) {
  if(fDump) {
    volatile int i;
    volatile DWORD dwAddr,dwWrAddr,dwEnd;
    dwAddr= (DWORD)dvHwMainReadLsiReg(pHw,SPU_CHAN_START_REG)<<8;
    dwAddr|=(DWORD)dvHwMainReadLsiReg(pHw,SPU_CHAN_START_REG+1)<<16;
    dwWrAddr= (DWORD)dvHwMainReadLsiReg(pHw,102)<<3;
    dwWrAddr|=(DWORD)dvHwMainReadLsiReg(pHw,103)<<11;
    dwWrAddr|=(DWORD)dvHwMainReadLsiReg(pHw,104)<<19;
    dwEnd=((DWORD)&DramBuffer)+dwWrAddr-dwAddr;

    dvHwMainReadDram(pHw,dwAddr,DramBuffer,sizeof(DramBuffer));
    i=0;
  }
}
#endif


/**EOF: DVHWSUB.C**/
