//#include <StrmIni.h>
//#include <mmsystem.h>
//#include <ksmedia.h>
#include "system.h"
#include "DbgMsg.h"
#include "DbgFltr.h"
#include "Settings.h"

#include "Lsi64020.h"
#include "MidasApi.h"
#include "dvHwMain.h"
#include "dvHwVid.h"
#include "dvHwAud.h"
#include "dvHwSub.h"
#include "UserData.h"
#include "Global.h"
#include "Sync.h"
#include "margierr.h"
#include "Idents.h"
#include "tvout.h"
#include "IniNames.h"

#define DUMP_VID  0


#define BLANK_COLOR_BLACK    0x00008080
#define BLANK_COLOR_BLUE     0x0072d423
#define STOP_BLANK_COLOR     BLANK_COLOR_BLUE
#define UNBLANK_DELAY_TIME   250

// define Actual Hardware States
enum {
  C_STATE_UNINITIALIZED=0,
  C_STATE_NOT_READY,
  C_STATE_READY,
  C_STATE_RDY_WAITDATA,
  C_STATE_RDY_TITLEKEY,
  C_STATE_RDY_PREFILL,
  C_STATE_START,
  C_STATE_PLAY,
  C_STATE_FAST_PLAY,
  C_STATE_SLOW_PLAY,
  C_STATE_PAUSE,
  C_STATE_STEP,
  C_STATE_DATA_DISCON,
  C_STATE_DISCON_FLUSH,
  C_STATE_TIME_DISCON,
  C_STATE_WAIT_EOS,
  C_STATE_EOS_TO_STILL,
  C_STATE_STILL,
  C_STATE_PLAY_TITLEKEY,
  C_STATE_PLAY_PREFILL,
  C_STATE_PLAY_SYNC,
  C_STATE_STALL_TIME,
  C_STATE_FLUSH,
  C_STATE_FLUSH_COMPLETE,
  C_STATE_NULL_FLUSH,
  C_STATE_PLAY_TO_FLUSH,
  C_STATE_UNK
};

struct IdentEntry_s IdentsVid[]= {
  { ID_VID_STATE,0 },
    { C_STATE_UNINITIALIZED,  "Uninitialized"},
    { C_STATE_NOT_READY,      "NotReady" },
    { C_STATE_READY,          "Ready" },
    { C_STATE_RDY_WAITDATA,   "RdyWaitData" },
    { C_STATE_RDY_TITLEKEY,   "RdyTitleKey" },
    { C_STATE_RDY_PREFILL,    "RdyPrefill"},
    { C_STATE_START,          "Start"},
    { C_STATE_PLAY,           "Play"},
    { C_STATE_SLOW_PLAY,      "Slow Play"},
    { C_STATE_FAST_PLAY,      "Fast Play"},
    { C_STATE_PAUSE,          "Pause"},
    { C_STATE_STEP,           "Step"},
    { C_STATE_DATA_DISCON,    "Data Discontinuity"},
    { C_STATE_DISCON_FLUSH,   "Discontinuity Flush"},
    { C_STATE_TIME_DISCON,    "Time Discontinuity"},
    { C_STATE_WAIT_EOS,       "WaitEos"},
    { C_STATE_EOS_TO_STILL,   "EosToStill"},
    { C_STATE_STILL,          "Still"},
    { C_STATE_PLAY_TITLEKEY,  "PlayTitleKey"},
    { C_STATE_PLAY_PREFILL,   "PlayPrefill"},
    { C_STATE_PLAY_SYNC,      "PlaySync"},
    { C_STATE_STALL_TIME,     "StallTime"},
    { C_STATE_FLUSH,          "Flush"},
    { C_STATE_FLUSH_COMPLETE, "FlushComplete"},
    { C_STATE_NULL_FLUSH,     "NullFlush"},
    { C_STATE_PLAY_TO_FLUSH,  "PlayToFlush"},
    { C_STATE_UNK,            "Unknown" },
  { 0,0 }
};


#define SIGNATURE 0xCD010003
typedef struct Video_s {
  DWORD Signature;
  PHW pHw;
  DWORD StrmState;              // Logical Stream state = STOP, PAUSE, RUN
  DWORD CtrlState;              // Actual Hardware state
  DWORD NextState;
  DWORD StateTimeoutTime;       // Time at which Current State will Timeout (0=never) 
  DWORD BeginFlush;             // Set if Flush Command Received
  DWORD EndFlush;               // Set if End Flush Command Received
  DWORD fInPumpData;            // Set TRUE while in DATA TIMER
  DWORD ResetStream;            // Force transition from any state to Ready
  DWORD VidDisplayInitialized;  // Set TRUE when video output has been initialized

  QUEUE DataQ;
  PMDS_DATA_BLOCK pFlushMdb;    // Used to indicate begin flush is complete
  DWORD PurgeQ;                 // Flag flush in progress must flush queue on next data timer

  int   fScanForVidSeq;        // Set when scaning bitstream for Video Sequence Header
  WORD  wSeqHdrWidth;          // Width  of Video extracted from Video Sequence Header
  WORD  wSeqHdrHeight;         // Height of Video extracted from Video Sequence Header

  DWORD EnableDataTransfer;
  DWORD DataXfrPending;
  DWORD dwMaxXfer;             // Max # bytes to transfer in each VidPunp loop
  DWORD NoDataCount;
  DWORD TransferFlushPacket;
  int   fDataInPipe;
  DWORD fSetBrokenGop;
  DWORD StartOddField;
  DWORD StartEvenField;
  DWORD StartLastField;
  int   nPlayStateOddCount;
  int   nOddFieldCount;
  DWORD DisplayType;
  DWORD hPerfChnl;		      //Handle to DbgMsg performance statistic for bytes in video channel.
  DWORD hPerfPics;		      //Handle to DbgMsg performance statistic for pictures in channel.

  DWORD StartTime;
  DWORD CurTime;		        //Current PC time in millisecs
  DWORD CurScr;             //Current Decode time in 90KHz clocks
  DWORD LastDataTime;	      //Time last data was sent.
  DWORD TimeToUnblank;		  //Time at which video will be unblanked.
  int   fBlankChange;       //Set to indicate fBlankVideo has changed
  int   fBlankVideo;        //Set if video is to be continously blanked
  DWORD dwBlankColor;       //YUV color for Blank Special values 0=Black,1=Blue
  DWORD dwChnlHiWater;		  //Level at which channel is full enough to start decoding
//  DWORD ChnlSpacePad;	      //Extra space required before transferring data.
  DWORD ChannelFull;        //Set if channel reached HiWater

  DWORD dwStep;             //Frame step count used to advance frames in pause.
  long  CurPlayRate;        //Current playback rate. 10000=normal. <0=reverse, <10000= slow, >10000= fast.
  long  ReqPlayRate;	      //New playback rate.     10000=normal. <0=reverse, <10000= slow, >10000= fast..
  int   fPlayRateMode;      //Set 1 for fast play pacing(PIONEER), 0 for as fast as possible(WDM)

  DWORD PlayRateMax;	      //Max playback speed. Take effect on next timer
  DWORD SlowNextFrame;	    //During slow play, repeat a frame on this frame count.
  DWORD SyncAhead;
  DWORD SyncError;

  DWORD SkipBPs;

  BYTE  bZVCtrl;            //Keep track of values in Midas Vid registers
  BYTE  bVidCtrl;           //so only reprogram if changing
  WORD  wHDelay;
  WORD  wHActive;

  int   fSetupZV;           //Set if ZV output must be updated
  int   fEnableZV;          //Zoom Video output from Midas enabled
  DWORD Blanked;
  DWORD PreEos;		          // Eos seen at some point in the past in the channel
  DWORD PostEos;		        //Eos decoded at some point in the past.

  DWORD dwDispMode;         // flags for Pan Scan, Letterbox, Normal
  int   fUpdateDispFormat;  // Update Width,Height, DispMode etc.
  VID_DISP_FORMAT DspFmt; 
  int   fNewDspFmt;

  PMDS_DATA_BLOCK pCssMdb;
  PMDS_DATA_BLOCK pDataMdb;
} VID;

LOCAL long VidCmdInvalid(VID *pVid, DWORD Cmd);
LOCAL long VidInit(PHW pHw);
LOCAL long VidFini(VID *pVid);
LOCAL long VidResetStateMachine(VID*);
LOCAL long VidClearCssPhase(VID*);

LOCAL long VidGet(VID *pVid, DWORD Item, DWORD *pValue);
LOCAL long VidSet(VID *pVid, DWORD Item, DWORD Value);
//LOCAL long VidState(VID *pVid, DWORD Target);
LOCAL long VidEnable(VID *pVid, DWORD Enable);
LOCAL long VidSetupTimebase(VID *pVid);
LOCAL long VidSyncTitleKey(VID* pVid,PMDS_DATA_BLOCK pMdb);
LOCAL long VidCssOk(VID* pVid);

LOCAL long VidBlank(VID *pVid, DWORD Blank,DWORD dwColor);
LOCAL long VidPlay(VID *pVid);
//LOCAL long VidStart(VID *pVid, DWORD Start);
LOCAL long VidPicParmSet(VID *pVid);
LOCAL long VidSelectStream(VID *pVid, DWORD Stream);
LOCAL long VidCalcChnlSpace(VID *pVid, DWORD *pBytes);
//LOCAL long VidFlushOver(VID *pVid);
//LOCAL long VidPanic(VID *pVid, DWORD PanicMode, DWORD Items);
LOCAL long VidRate(VID *pVid,MDS_PLAYRATE *pPlayRate);
LOCAL long VidDisplayFormat(VID *pVid,PVID_DISP_FORMAT);
LOCAL long ApplyVidDisplayFormat(VID *pVid, PVID_DISP_FORMAT pFrm);
LOCAL long VidSetMpgPicture(VID *pVid);

LOCAL long  VidTimer(VID *pVid);
LOCAL DWORD VidSendData(VID *pVid,PMDS_DATA_PACKET pMdp);
//LOCAL long VidData(VID *pVid, struct Send_s *pSend);
LOCAL long VidCorrectSync(VID*,DWORD dwAhead,DWORD dwError);
LOCAL long VidStep(VID *pVid,DWORD dwStep);

LOCAL long VidStreamState(VID*,int StreamState);
LOCAL long VidStreamFlush(VID*,int End,PMDS_DATA_BLOCK pMdp);
LOCAL long VidStreamReset(VID *pVid);

LOCAL long VidPanicOff(VID *pVid);
LOCAL long VidPanicLow(VID *pVid);
LOCAL long VidPanicHigh(VID *pVid);

LOCAL long VidProcessNotReady(VID*);
LOCAL long VidProcessReady(VID*);
LOCAL long VidProcessRdyWaitData(VID*);
LOCAL long VidProcessRdyTitleKey(VID*);
LOCAL long VidProcessRdyPrefill(VID*);
LOCAL long VidProcessStart(VID*);
LOCAL long VidProcessPlay(VID*);
LOCAL long VidProcessFastPlay(VID*);
LOCAL long VidProcessSlowPlay(VID*);
LOCAL long VidProcessPause(VID*);
LOCAL long VidProcessStep(VID*);
LOCAL long VidProcessDataDiscon(VID*);
LOCAL long VidProcessDisconFlush(VID*);
LOCAL long VidProcessWaitEos(VID*);
LOCAL long VidProcessEosToStill(VID*);
LOCAL long VidProcessStill(VID*);
LOCAL long VidProcessPlayTitleKey(VID*);
LOCAL long VidProcessPlayPrefill(VID*);
LOCAL long VidProcessStallTime(VID*);
LOCAL long VidProcessPlaySync(VID*);
LOCAL long VidProcessFlush(VID*);
LOCAL long VidProcessFlushComplete(VID*);
LOCAL long VidProcessNullFlush(VID*);
LOCAL long VidProcessPlayToFlush(VID*);
LOCAL long VidProcessCss(VID*);

LOCAL long VidStateEnterWaitEos(VID*);
LOCAL long VidStateEnterPlayPrefill(VID*);
LOCAL long VidStateEnterPlaySync(VID*);
LOCAL long VidStateEnterFlush(VID*);
LOCAL long VidStateEnterReady(VID*);
LOCAL long VidNormalPlayHouseWork(VID*);
LOCAL long VidTrickPlayHouseWork(VID*);
LOCAL long VidSetEnableDataTransfer(VID*);
LOCAL long VidBlankManager(VID*);
LOCAL long VidDispFormatManager(VID*);
LOCAL long VidUpdateDisplayFormat(VID*);

LOCAL void VidDecodeFreezeReset(VID*);
LOCAL void VidDecodeFreezeHandler(VID*);
LOCAL void VidDecodeStep(VID*);
LOCAL void VidDecodeSlow(VID*,int nRateDiv);
LOCAL void VidDecodeFreeze(VID*);
LOCAL void VidDecodeUnfreeze(VID*);
LOCAL void VidDecodeFreezeNframes(VID*,int n);
LOCAL void VidDecodeFreezeFrame(VID*);
LOCAL void VidDecodeUnfreezeFrame(VID*);
LOCAL void VidDecodeSkipReset(VID*);
LOCAL int  VidDecodeSkipHandler(VID*);
LOCAL void VidDecodeSkipNframes(VID*,int n);
LOCAL void VidDecodeSetupPulldown(VID*,BYTE bMode);
LOCAL void VidDecodeEnablePulldown(VID*,BYTE bEnable);
LOCAL int  VidDecodePulldownHandler(VID*);

LOCAL long VidSetStateTimeout(VID* pVid, DWORD dwMilliSecs);
LOCAL long IsStateTimeout(VID* pVid);
LOCAL long VidStateInvalid(VID*);

LOCAL long VidSetPlayRateImmediate(VID* pVid,long Rate);
LOCAL long VidFreeze(VID* pVid,int nFreeze);
LOCAL long VidSetNormalPlay(VID* pVid);

LOCAL long VidSetBlank(VID* pVid,DWORD dwBlankState,DWORD dwColor);

LOCAL VID StaticVideo;	      //Replace with dynamic memory allocation later.
//LOCAL struct DbgMsgPerf_s PerfStatVidChnl= { "Video Channel Bytes","Video Channel","Video Channel",0 };
//LOCAL struct DbgMsgPerf_s PerfStatVidPics= { "Video Channel Pictures","Pictures","Number of complete compressed pictures awaiting decoding.",0 };

//Convert a video handle to a pointer.
LOCAL VID *VidPtr(PHW pHw) {
  if(pHw->hVideo && *((DWORD*)pHw->hVideo)==SIGNATURE)
    return((VID*)pHw->hVideo);
  return(0);
}

long VideoCmd(PHW pHw, DWORD Cmd, DWORD Arg1, DWORD Arg2) {
  VID *pVid;
  long Err=0;
  if(LOWORD(Cmd)==VIDCMD_INIT) {
    Err= VidInit(pHw);
  } else if(!(pVid= VidPtr(pHw))) {
    Err= MARGI_ERR_BADHANDLE;
  } else {
    if(LOWORD(Cmd)==VIDCMD_TIMER) {
      Err= VidTimer(pVid);
    } else {
      int fEn;
      switch(LOWORD(Cmd)) {
        case VIDCMD_FINI:       Err= VidFini(pVid); break;
        case VIDCMD_RESET:	    Err= VidStreamReset(pVid); break;
//        case VIDCMD_CLRCSSPHASE:Err= VidClearCssPhase(pVid); break;
        case VIDCMD_STOP:       Err= VidStreamState(pVid,S_STATE_STOP); break;
        case VIDCMD_RUN:        Err= VidStreamState(pVid,S_STATE_RUN);  break;
        case VIDCMD_PAUSE:      Err= VidStreamState(pVid,S_STATE_PAUSE);break;
        case VIDCMD_BEGINFLUSH: Err= VidStreamFlush(pVid,0,(PMDS_DATA_BLOCK)Arg1); break;
        case VIDCMD_ENDFLUSH:   Err= VidStreamFlush(pVid,1,0); break;
        case VIDCMD_ENABLE:	    fEn=Arg1?1:0;
                                if(pVid->fEnableZV!=fEn) {
                                  pVid->fSetupZV=1;
#ifdef _NEC_
                                  pVid->fEnableZV=0;
#else
                                  pVid->fEnableZV=fEn;
#endif
                                }
                                break;
        case VIDCMD_PICSET:	    Err= VidPicParmSet(pVid); break;
        case VIDCMD_SELECT:	    Err= VidSelectStream(pVid,Arg1); break;
        case VIDCMD_RATE:       Err= VidRate(pVid,(MDS_PLAYRATE*)Arg1); break;
        case VIDCMD_DISPFORMAT: Err= VidDisplayFormat(pVid,(PVID_DISP_FORMAT)Arg1); break;
        case VIDCMD_BLANK:      Err= VidSetBlank(pVid,Arg1,Arg2); break;
        case VIDCMD_GET:        Err= VidGet(pVid,Arg1,(DWORD*)Arg2); break;
        case VIDCMD_SET:        Err= VidSet(pVid,Arg1,Arg2); break;
        case VIDCMD_CORRECT:    Err= VidCorrectSync(pVid,Arg1,Arg2); break;
        case VIDCMD_SYNCTITLE:  Err= VidSyncTitleKey(pVid,(PMDS_DATA_BLOCK)Arg1); break;
        case VIDCMD_CSS_OK:     Err= VidCssOk(pVid); break;
        case VIDCMD_STEP:       Err= VidStep(pVid,Arg1); break;
        default:                Err= VidCmdInvalid(pVid,Cmd);
      }
    }
  }
  return(Err);
}

long VideoQueueData(PHW pHw,PMDS_DATA_BLOCK pMdb) {
  long lErr=0;
  VID *pVid;
  if(pVid=VidPtr(pHw)) {
    pMdb->dwCurPacket=0;
    pVid->pDataMdb = pMdb;
    qEnqueue(&pVid->DataQ,pMdb);
  } else
    lErr= MARGI_ERR_BADHANDLE;
  return lErr;
}

long VideoCancelData(PHW pHw,PMDS_DATA_BLOCK pMdb) {
  long Err=0;
  VID *pVid;
  if(pVid=VidPtr(pHw)) {
    if(pMdb) {
      pMdb->dwCurPacket=0;
      pMdb->dwStatus=MDS_Cancel;
      if(qFindAndDequeue(&pVid->DataQ,pMdb)) {
         PMDB_CALLBACK pfnCallback = (PMDB_CALLBACK)pMdb->pfnCallback;
         pfnCallback(pMdb);
       }
    } else {
      qPurgeQueue(&pVid->DataQ,MDS_Cancel);
    }
  } else
    Err= MARGI_ERR_BADHANDLE;
  return 0;
}

DWORD VideoPumpData(PHW pHw) {
  DWORD dwBytesTransfered=0;
  VID* pVid;
  if(pVid=VidPtr(pHw)) {
    PQUEUE pQ;
    PMDS_DATA_BLOCK  pMdb;
    PMDS_DATA_PACKET pMdp;
    pQ=(PQUEUE)&(pVid->DataQ);
    pVid->fInPumpData=1;
    if(pVid->pFlushMdb) {
      PMDB_CALLBACK pfnCallback=(PMDB_CALLBACK)pVid->pFlushMdb->pfnCallback;
      qPurgeQueue(&pVid->DataQ,MDS_Cancel);
      pVid->pFlushMdb->dwStatus=MDS_Success;
      pfnCallback(pVid->pFlushMdb);
      pVid->pFlushMdb=0;
    }
    pMdb=qPeekqueue(pQ);
    // only pump data in RUN state.  This improves activemovie FFwd-Pause-Play problem. Fills channel with I frames in pause mode
    if(pMdb && !pVid->BeginFlush && (pVid->StrmState==S_STATE_RUN)) {
      while (dwBytesTransfered < pVid->dwMaxXfer) {
        DWORD dwBytes;
        pMdp=( PMDS_DATA_PACKET)&pMdb->pPackets[pMdb->dwCurPacket];
        dwBytes=VidSendData(pVid,pMdp);
        dwBytesTransfered+=dwBytes;
        if(dwBytes!=pMdp->DataUsed)
          break;
        if(++pMdb->dwCurPacket>=pMdb->dwTotalPackets) {
          PMDB_CALLBACK pfnCallback = (PMDB_CALLBACK)pMdb->pfnCallback;
          pMdb->dwStatus=MDS_Success;
          qDequeue(pQ);
          pfnCallback(pMdb);
          break;
        }
        if(pMdp->OptionsFlags&MDB_FLAG_DATADISCONTINUITY)
          UserDataCmd(pHw,USERDATACMD_DISCONTINUITY,MDB_FLAG_DATADISCONTINUITY,0);
      }
    } else
      VidSendData(pVid,0);  // if no data then call anyway incase need to transfer flush packet

    // 1st title key is applied before packets sent
    if(pVid->pCssMdb && !pVid->fDataInPipe && !pVid->pHw->fCssTitleKeySet) {
      DbgMsg(DBG_VIDSTATE,"VidPumpData() No Current Title Key  fCssWaitForTitleKey=1;\r\n");
//      pHw->fCssApplyingTitleKey=1;
      pHw->fCssWaitForTitleKey=1;
    }
    if(pHw->fCssApplyingTitleKey)
      VidProcessCss(pVid);

    pVid->fInPumpData=0;
  }
  return dwBytesTransfered;
}

LOCAL long VidCmdInvalid(VID *pVid, DWORD Cmd) {
  DbgMsg(DBG_ERR,"Video: Bad command %u (%08X).\r\n",Cmd);
  return(MARGI_ERR_BADCMD);
}

/*************************************************************************/
/** Init/Reset/Fini							**/
/*************************************************************************/
LOCAL long VidInit(PHW pHw) {
  long Err= 0;
  VID *pVid= &StaticVideo;				//Replace with dynamic memory allocation.
  if(!pVid) {
    DbgMsg(DBG_ERR,"Video: Unable to allocate new VID object!\r\n");
    Err= MARGI_ERR_NOMEM;
  } else {
    memset(pVid,0,sizeof(VID));
    pHw->hVideo= (DWORD)pVid;
    pVid->Signature= SIGNATURE;
    pVid->pHw= pHw;
    pHw->ChInf[VIDCH].dwSafety=SettingLoad(SET_DWORD,IniVidChnlSpacePad,0,32768)/8;
//    DbgMsgCmd(DBGCMD_PERFADD,(DWORD)&PerfStatVidChnl,(DWORD)&pVid->hPerfChnl);
//    DbgMsgCmd(DBGCMD_PERFADD,(DWORD)&PerfStatVidPics,(DWORD)&pVid->hPerfPics);
//    DbgOutCmd(DBGCMD_TIME_NAME,1,(DWORD)"Video Channel Bytes");
//    DbgOutCmd(DBGCMD_TIME_NAME,2,(DWORD)"Video Channel Picts");
    VidResetStateMachine(pVid);
    qInitQueue(&pVid->DataQ);
    pVid->dwBlankColor=STOP_BLANK_COLOR;
  }
  return(Err);
}

LOCAL long VidFini(VID *pVid) {
  long Err= 0;
  pVid->Signature= 0;
  pVid->pHw->hVideo= 0;
//  DbgMsgCmd(DBGCMD_PERFDEL,pVid->hPerfPics,0);
//  DbgMsgCmd(DBGCMD_PERFDEL,pVid->hPerfChnl,0);
  pVid->hPerfChnl= 0;
  pVid->VidDisplayInitialized=0;
  //Free dynamically allocated memory.
  return(Err);
}

LOCAL long VidClearCssPhase(VID* pVid) {
  return 0;
}

// 2 approaches to sync Title Key with incoming data.
// 1st: VidSyncTitleKey used in WDM type environment.  We hold onto CssMdb 
//      until we get data with UseNewCSS flag
LOCAL long VidSyncTitleKey(VID* pVid,PMDS_DATA_BLOCK pMdb) {
//  if(pMdb) {
  DbgMsg(DBG_VIDSTATE,"VideoSyncTitleKey\r\n");
  pVid->pCssMdb=pMdb;
    // If we already have title key then wait for MDB
    // with use new title key flag. If we haven't set title
    // then apply immediately
//    if(pVid->pHw->fCssTitleKeySet) {
//     pVid->pHw->fCssTitleKeyPending=1;
//    } else
//      VidProcessCss(pVid);
//  }
  return 0;
}

//2nd: VidCssOk used with hidriver. Sleep until Vid state machine gets 
//     to state where Vid data transfer will not happen.
LOCAL long VidCssOk(VID* pVid) {
  long Err=0;
  int  fBreak=0;
  int  nTimeOutCount=0;
#if(0)
  if(!pVid->pHw->fCssTitleKeyPending) {
    pVid->pHw->fCssTitleKeyPending=1;
    while(!fBreak && (pVid->StrmState==S_STATE_RUN)) {
      switch(pVid->CtrlState) {
  //      case C_STATE_NOT_READY:
  //      case C_STATE_READY:
        case C_STATE_RDY_WAITDATA:
        case C_STATE_STILL:
          fBreak=1;
          break;
        default:
          MargiSleep(20);
          if(nTimeOutCount++ > 15) {
            fBreak=1;
            Err=1;
          }
          break;
      }
    }
  }
#endif
  return Err;
}



LOCAL long VidResetStateMachine(VID *pVid) {
  pVid->StrmState= S_STATE_STOP;
  if(pVid->VidDisplayInitialized)
    pVid->CtrlState= C_STATE_NOT_READY;
  else
    pVid->CtrlState= C_STATE_UNINITIALIZED;
  DbgMsg(DBG_VIDSTATE,"Video: Reset Stream State:%s   Ctrl State:%s.\r\n",Ident(ID_STRM_STATE,pVid->StrmState),Ident(ID_VID_STATE,pVid->CtrlState));
  pVid->pHw->u.s.fChannelConfig=0;  // about to reconfigure channel
  pVid->LastDataTime= 0;
  pVid->PlayRateMax = RATE_NORMAL/8;
  pVid->CurPlayRate= RATE_NORMAL;
  pVid->ReqPlayRate= RATE_NORMAL;

  pVid->DataXfrPending=0;
  pVid->EnableDataTransfer=0;
  pVid->TransferFlushPacket=0;
  pVid->PreEos=0;
  pVid->PostEos=0;
  pVid->BeginFlush=0;
  pVid->EndFlush=0;
  pVid->ResetStream=0;
  pVid->SyncAhead=0;
  pVid->SyncError=0;
  pVid->fSetupZV=1;
  pVid->bZVCtrl=0;          //Keep track of values in Midas Vid registers
  pVid->bVidCtrl=0;         //so only reprogram if changing
  pVid->wHDelay=0;
  pVid->wHActive=0;
//  VidBlank(pVid,1);
//  VidDecodeFreezeReset(pVid);
  return(0);
}

// Force State machine to pass thru READY to re-initialize
// this will not change the stream state.
LOCAL long VidStreamReset(VID *pVid) {
  pVid->ResetStream=1;
  pVid->EnableDataTransfer=0;
  pVid->fSetupZV=1;
  DbgMsg(DBG_VIDSTATE,"Video: Reset State.\r\n");
  return 0;
}

LOCAL long VidStreamState(VID* pVid, int StrmState) {
  long Err= 0;
  DbgMsg(DBG_VIDSTATE,"Video: Stream State %s -> %s.\r\n",Ident(ID_STRM_STATE,pVid->StrmState),Ident(ID_STRM_STATE,StrmState));
  pVid->StrmState = StrmState;
  return Err;
}

LOCAL long VidStreamFlush(VID* pVid,int End,PMDS_DATA_BLOCK pFlushMdb) {
  long Err= 0;
  if(End)
    pVid->EndFlush=1;
  else {
    pVid->BeginFlush=1;
    pVid->EnableDataTransfer=0;
    pVid->pFlushMdb=pFlushMdb;
    if(!pFlushMdb) {
      while(pVid->fInPumpData) 
        MargiSleep(20);
      qPurgeQueue(&pVid->DataQ,MDS_Cancel);
    }
  }
  return Err;
}

// This function is responsible for determining the next state
// given the current state and the input variables.
// It does not change the state or perform any other function.  
// It should be called each time the state actually changes or 
// any of the inputs change.
long VidDetermineNextState(VID* pVid) {

  int fDone=0;

  pVid->NextState=pVid->CtrlState;

  // If Stream State is stop then all states except C_STATE_UNINITIALIZED
  // will transition to C_STATE_READY
  if(pVid->StrmState==S_STATE_STOP) {
    pVid->ReqPlayRate=RATE_NORMAL;
    switch(pVid->CtrlState) {
      case C_STATE_UNINITIALIZED:
        if(pVid->VidDisplayInitialized)
          pVid->NextState=C_STATE_NOT_READY;
        fDone=1;
        break;
      default:
      case C_STATE_READY:
        UserDataCmd(pVid->pHw,USERDATACMD_FLUSH,0,0);
        VidBlank(pVid,1,pVid->dwBlankColor);
        pVid->TimeToUnblank=0;
        pVid->NextState=C_STATE_READY;
        fDone=1;
        break;
      case C_STATE_PLAY_TO_FLUSH:
      case C_STATE_FLUSH:
      case C_STATE_FLUSH_COMPLETE:
        break;
    }
  }
  if((pVid->ResetStream || pVid->pHw->wHwError) && !fDone) {
    pVid->NextState=C_STATE_READY;
    fDone=1;
  }
  // All states except NOT_READY,READY and FLUSHCOMPLETE will
  // transition to FLUSH if BeginFlush is set
  if(!fDone && pVid->BeginFlush) {
    switch(pVid->CtrlState) {
      case C_STATE_UNINITIALIZED:
      case C_STATE_NOT_READY:
      case C_STATE_READY:
      case C_STATE_FLUSH_COMPLETE:
      case C_STATE_RDY_TITLEKEY:
      case C_STATE_RDY_WAITDATA:
      case C_STATE_NULL_FLUSH:
      case C_STATE_DISCON_FLUSH:
      case C_STATE_WAIT_EOS:
      case C_STATE_EOS_TO_STILL:
        break;
      case C_STATE_STILL:
        pVid->NextState=C_STATE_NULL_FLUSH;
        fDone=1;
        break;
      case C_STATE_PLAY:
        pVid->NextState=C_STATE_PLAY_TO_FLUSH;
        fDone=1;
        break;
      default:
        pVid->NextState=C_STATE_FLUSH;
        fDone=1;
        break;
    }
    pVid->BeginFlush=0;
  }

  // EnableTransfer is set false in VidSendData
  // when a discontinuity is received
  if(!fDone && !pVid->EnableDataTransfer) {
    switch(pVid->CtrlState) {
      case C_STATE_RDY_PREFILL:
      case C_STATE_START:
        pVid->NextState=C_STATE_READY;
        fDone=1;
        break;
      case C_STATE_PLAY_PREFILL:
      case C_STATE_STALL_TIME:
        pVid->NextState=C_STATE_PLAY_SYNC;
        break;
      case C_STATE_PLAY:
      case C_STATE_FAST_PLAY:
      case C_STATE_SLOW_PLAY:
      case C_STATE_STEP:
        pVid->NextState=C_STATE_DATA_DISCON;
        fDone=1;
        break;
    }
  }

  if(!fDone) {
    switch(pVid->CtrlState) {
      default:
      case C_STATE_UNINITIALIZED:
        if(pVid->VidDisplayInitialized)
          pVid->NextState=C_STATE_NOT_READY;
        break;
      case C_STATE_NOT_READY:
        if(pVid->StrmState==S_STATE_RUN)
          pVid->NextState=C_STATE_READY;
        break;
   
      case C_STATE_READY:
        if(pVid->StrmState==S_STATE_RUN)
          pVid->NextState=C_STATE_RDY_WAITDATA;
        break;

      case C_STATE_RDY_WAITDATA:
        if(pVid->DataXfrPending || pVid->pHw->fCssWaitForTitleKey)
          pVid->NextState=C_STATE_RDY_TITLEKEY;
        break;

      case C_STATE_RDY_TITLEKEY:
        if(pVid->DataXfrPending && !pVid->pHw->fCssWaitForTitleKey)
          pVid->NextState=C_STATE_RDY_PREFILL;
        break;

      case C_STATE_RDY_PREFILL:
        if(pVid->pDataMdb && pVid->pDataMdb->dwRsvd[1])
          pVid->StartTime = 1;
        if(pVid->StartTime && (pVid->PreEos||pVid->ChannelFull||(pVid->NoDataCount>5))) {
          DbgMsg(DBG_VIDSTATE,"Video: state %s  transition on %s\r\n",Ident(ID_VID_STATE,pVid->CtrlState),
                                                  pVid->PreEos?"PreEos":(pVid->ChannelFull?"VidChannelFull":"Timeout5"));
          pVid->NextState=C_STATE_START;
        } else if(pVid->NoDataCount>500) {
          DbgMsg(DBG_VIDSTATE,"Video: state %s  transition on Timeout500\r\n",Ident(ID_VID_STATE,pVid->CtrlState));
          pVid->NextState=C_STATE_START;
        }
        break;
    
      case C_STATE_START:
        if(pVid->pHw->u.s.VideoDecodeRun)
          pVid->NextState=C_STATE_PLAY_PREFILL;
        break;
    
      case C_STATE_PLAY_PREFILL:
        if(pVid->StartTime && (pVid->PreEos||pVid->ChannelFull||(pVid->NoDataCount>5))) {
          DbgMsg(DBG_VIDSTATE,"Video: state %s  transition on %s\r\n",Ident(ID_VID_STATE,pVid->CtrlState),
                                                  pVid->PreEos?"PreEos":(pVid->ChannelFull?"VidChannelFull":"Timeout5"));
          pVid->NextState=C_STATE_PLAY_SYNC;
        } else if(pVid->StartTime && tmCompareTimes(pVid->CurScr+40,pVid->StartTime)) {
          pVid->NextState=C_STATE_STALL_TIME;
        } else if(pVid->NoDataCount>500) {
          DbgMsg(DBG_VIDSTATE,"Video: state %s  transition on Timeout500\r\n",Ident(ID_VID_STATE,pVid->CtrlState));
          pVid->NextState=C_STATE_PLAY_SYNC;
        }
        break;

      case C_STATE_STALL_TIME:
        if(pVid->PreEos || pVid->ChannelFull || (pVid->NoDataCount>5)) {
          DbgMsg(DBG_VIDSTATE,"Video: state %s  transition on %s\r\n",Ident(ID_VID_STATE,pVid->CtrlState),
                                                  pVid->PreEos?"PreEos":(pVid->ChannelFull?"VidChannelFull":"Timeout"));
          pVid->NextState=C_STATE_PLAY_SYNC;
        }
        break;

      case C_STATE_PLAY_SYNC:
        if(pVid->StartOddField) {
          DWORD dwTime=pVid->StartTime;
          if(tmCompareTimes(pVid->CurScr,dwTime) || pVid->ReqPlayRate!=RATE_NORMAL) { 
            pVid->StartTime=0;
            DbgMsg(DBG_VIDSTATE,"Video: state %s transition ",Ident(ID_VID_STATE,pVid->CtrlState));
            DbgDispPTS(DBG_VIDSTATE,pVid->CurScr,"Time="," ");
            DbgDispPTS(DBG_VIDSTATE,dwTime,"Target=","\r\n");
            pVid->NextState=C_STATE_PLAY;
          }
        }
        break;
    
      case C_STATE_PLAY:
        if(pVid->PreEos)
          pVid->NextState=C_STATE_WAIT_EOS;
        else if(pVid->StrmState==S_STATE_PAUSE) {
          if(pVid->StartOddField)
            pVid->NextState=C_STATE_PAUSE;
        } else if(pVid->ReqPlayRate>RATE_NORMAL)
          pVid->NextState=C_STATE_SLOW_PLAY;
        else if(pVid->ReqPlayRate<RATE_NORMAL)
          pVid->NextState=C_STATE_FAST_PLAY;
        break;
    
      case C_STATE_SLOW_PLAY:
        if(pVid->PreEos || (pVid->StrmState==S_STATE_PAUSE) || (pVid->ReqPlayRate<=RATE_NORMAL))
          pVid->NextState=C_STATE_PLAY;
        break;
    
      case C_STATE_FAST_PLAY:
        if(pVid->PreEos || (pVid->StrmState==S_STATE_PAUSE) || (pVid->ReqPlayRate>=RATE_NORMAL))
          pVid->NextState=C_STATE_PLAY;
        break;
    
      case C_STATE_PAUSE:
        if((pVid->StrmState==S_STATE_RUN) && pVid->StartOddField)
          pVid->NextState=C_STATE_PLAY;
        else if(pVid->dwStep  && pVid->StartOddField)
          pVid->NextState=C_STATE_STEP;
        break;

      case C_STATE_STEP:
        pVid->NextState=C_STATE_PAUSE;
        break;

      case C_STATE_DATA_DISCON:
//        if(pVid->PreEos)
//          pVid->NextState=C_STATE_WAIT_EOS;
//        else 
        if((pVid->pHw->ChInf[VIDCH].dwPTUs<5) || IsStateTimeout(pVid))
          pVid->NextState=C_STATE_DISCON_FLUSH;
        break;

      case C_STATE_DISCON_FLUSH:
        if(!pVid->pHw->u.s.fInPumpData || IsStateTimeout(pVid))
          pVid->NextState=C_STATE_READY;
        break;
    
      case C_STATE_WAIT_EOS:
        if(pVid->PostEos || IsStateTimeout(pVid))
          pVid->NextState=C_STATE_EOS_TO_STILL;
        break;

      case C_STATE_EOS_TO_STILL:
        // Wait 3 Odd Fields before going to STILL which will freeze frame
        // nOddFieldCount is reset on entry to EOS_TO_STILL
        if(pVid->nOddFieldCount>2)
            pVid->NextState=C_STATE_STILL;
        break;

      case C_STATE_STILL:
        // Wait atleast another 7 Odd Fields before going to PreFill
        // This allows us to set Hi Panic 2 fields before Transfering new data
//        if(pVid->DataXfrPending && pVid->nOddFieldCount>10)
        if((pVid->DataXfrPending || pVid->pHw->fCssWaitForTitleKey) && pVid->nOddFieldCount>3)
          pVid->NextState=C_STATE_PLAY_TITLEKEY;
        break;

      case C_STATE_PLAY_TITLEKEY:
        if(pVid->DataXfrPending && !pVid->pHw->fCssWaitForTitleKey)
          pVid->NextState=C_STATE_PLAY_PREFILL;
        break;

      case C_STATE_PLAY_TO_FLUSH:
        if(pVid->nOddFieldCount>3)      
          pVid->NextState=C_STATE_FLUSH;
        break;
    
      case C_STATE_FLUSH:
        if(!pVid->pHw->u.s.VideoDecodeRun && !pVid->pFlushMdb)
          pVid->NextState=C_STATE_FLUSH_COMPLETE;
        if(IsStateTimeout(pVid) && !pVid->pHw->u.s.fInPumpData)
          pVid->NextState=C_STATE_READY;
        break;

      case C_STATE_FLUSH_COMPLETE:
        if(pVid->EndFlush &&  !pVid->pHw->u.s.fInPumpData)
          pVid->NextState=C_STATE_READY;
        break;
      case C_STATE_NULL_FLUSH:
        if(pVid->EndFlush)
          pVid->NextState=C_STATE_STILL;
        if(IsStateTimeout(pVid))
          pVid->NextState=C_STATE_STILL;
        break;
    }
  }
  return 0;
}


// Do the actual work in transitioning from one state to
// the next.
LOCAL long VidProcessState(VID* pVid) {
  long Err=0;
  if(pVid->CtrlState != pVid->NextState) {
    DbgMsg(DBG_VIDSTATE,"Video: state %s -> %s.",Ident(ID_VID_STATE,pVid->CtrlState),Ident(ID_VID_STATE,pVid->NextState));
    DbgDispPTS(DBG_VIDSTATE,pVid->CurScr," (Time=",")\r\n");
    VidSetStateTimeout(pVid,0);   // If state transition about to occur then clear state timeout
  }
  switch (pVid->CtrlState) {
    default:
    case C_STATE_UNINITIALIZED:
      VidDispFormatManager(pVid);
      pVid->CtrlState=pVid->NextState;
      break;
    case C_STATE_NOT_READY:
      VidDispFormatManager(pVid);
      Err=VidProcessNotReady(pVid);
      break;
    case C_STATE_READY:
      VidDispFormatManager(pVid);
      Err=VidProcessReady(pVid);
      break;

    case C_STATE_RDY_WAITDATA:
      Err=VidProcessRdyWaitData(pVid);
      break;

    case C_STATE_RDY_TITLEKEY:
      Err=VidProcessRdyTitleKey(pVid);
      break;
    
    case C_STATE_RDY_PREFILL:
      Err=VidProcessRdyPrefill(pVid);
      break;

    case C_STATE_START:
      Err=VidProcessStart(pVid);
      break;

    case C_STATE_PLAY:
      VidDispFormatManager(pVid);
      Err=VidProcessPlay(pVid);
      break;

    case C_STATE_SLOW_PLAY:
      VidDispFormatManager(pVid);
      Err=VidProcessSlowPlay(pVid);
      break;

    case C_STATE_FAST_PLAY:
      VidDispFormatManager(pVid);
      Err=VidProcessFastPlay(pVid);
      break;

    case C_STATE_PAUSE:
      VidDispFormatManager(pVid);
      Err=VidProcessPause(pVid);
      break;

    case C_STATE_STEP:
      Err=VidProcessStep(pVid);
      break;

    case C_STATE_DATA_DISCON:
      VidProcessDataDiscon(pVid);
      break;

    case C_STATE_DISCON_FLUSH:
      VidProcessDisconFlush(pVid);
      break;

    case C_STATE_WAIT_EOS:
      Err=VidProcessWaitEos(pVid);
      break;

    case C_STATE_EOS_TO_STILL:
      Err=VidProcessEosToStill(pVid);
      break;

    case C_STATE_STILL:
      Err=VidProcessStill(pVid);
      break;

    case C_STATE_PLAY_TITLEKEY:
      Err=VidProcessPlayTitleKey(pVid);
      break;

    case C_STATE_PLAY_PREFILL:
      Err=VidProcessPlayPrefill(pVid);
      break;

    case C_STATE_STALL_TIME:
      Err=VidProcessStallTime(pVid);
      break;

    case C_STATE_PLAY_SYNC:
      Err=VidProcessPlaySync(pVid);
      break;

    case C_STATE_PLAY_TO_FLUSH:
      Err=VidProcessPlayToFlush(pVid);
      break;

    case C_STATE_FLUSH:
      Err=VidProcessFlush(pVid);
      break;

    case C_STATE_FLUSH_COMPLETE:
      Err=VidProcessFlushComplete(pVid);
      break;
    
    case C_STATE_NULL_FLUSH:
      Err=VidProcessNullFlush(pVid);
      break;
  }
  return Err;
}


LOCAL long VidProcessNotReady(VID* pVid) {
  long Err=0;
  switch(pVid->NextState) {
    case C_STATE_NOT_READY:
      break;
    case C_STATE_READY:
      VidBlank(pVid,1,pVid->dwBlankColor);
      pVid->TimeToUnblank=0;
      Err=VidStateEnterReady(pVid);
      break;
    default:
      Err=VidStateInvalid(pVid);
  }
  return Err;
}

LOCAL long VidProcessReady(VID* pVid) {
  long Err=0;
  switch(pVid->NextState) {
    case C_STATE_READY:
      if(pVid->ResetStream)
        Err=VidStateEnterReady(pVid);
      break;
    case C_STATE_RDY_WAITDATA:
      dvHwMainBuffConfig(pVid->pHw);               //Config,Start all channel buffers
      VidSetNormalPlay(pVid);
      pVid->pHw->u.s.EnableSubPicture=1;
      pVid->CtrlState=pVid->NextState;
      break;
    default:
      Err=VidStateInvalid(pVid);
  }
  return Err;
}

LOCAL long VidProcessRdyWaitData(VID* pVid) {
  long Err=0;
  switch(pVid->NextState) {
    case C_STATE_RDY_WAITDATA:
      if(pVid->ResetStream)
        Err=VidStateEnterReady(pVid);
      break;
    case C_STATE_RDY_TITLEKEY:
//      if(pVid->pHw->fCssWaitForTitleKey)
//        pVid->pHw->fCssApplyingTitleKey=1;
      VidSetStateTimeout(pVid,500);
      pVid->CtrlState=pVid->NextState;
      break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}

LOCAL long VidProcessRdyTitleKey(VID* pVid) {
  long Err=0;
  switch(pVid->NextState) {
    case C_STATE_RDY_TITLEKEY:
      if(pVid->ResetStream)
        Err=VidStateEnterReady(pVid);
//      if(pVid->pHw->fCssWaitForTitleKey || (pVid->pCssMdb && pVid->pHw->fCssNewTitleKey)) {
      if(pVid->pHw->fCssWaitForTitleKey) {
        pVid->pHw->fCssWaitForTitleKey=1;
        pVid->pHw->fCssApplyingTitleKey=1;
      }
      break;
    case C_STATE_RDY_WAITDATA:
      pVid->CtrlState=pVid->NextState;
      break;
    case C_STATE_RDY_PREFILL:
      VidPanicHigh(pVid);
      VidSelectStream(pVid,0);	  //Must find out how the Stream ID is passed from the Class Driver
      pVid->PreEos=  0;
      pVid->PostEos= 0;
      pVid->ChannelFull=0;
      VidSetEnableDataTransfer(pVid);
      VidSetStateTimeout(pVid,500);
      pVid->CtrlState=pVid->NextState;
      break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}

LOCAL long VidProcessRdyPrefill(VID* pVid) {
  long Err=0;
  switch(pVid->NextState) {
    case C_STATE_RDY_PREFILL:
      // poll to see if StartTime is available
      SyncCmd(pVid->pHw,SYNC_GET,SYNC_ITEM_VIDPTSMIN,(DWORD)&pVid->StartTime);
      break;
    case C_STATE_START:
      if(pVid->PreEos)
        VidPanicOff(pVid);
      else
        VidPanicLow(pVid);
      dvHwMainWriteLsiRegBits(pVid->pHw,DECODER_COMMAND_REG,DECODER_START);   //Start (or restart) the decoder
      pVid->CtrlState=pVid->NextState;
      break;
    case C_STATE_FLUSH:
      Err=VidStateEnterFlush(pVid);
      break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}

LOCAL long VidProcessStart(VID* pVid) {
  long Err=0;
  switch(pVid->NextState) {
    case C_STATE_START:
      if(pVid->PreEos)
        VidPanicOff(pVid);
      dvHwMainWriteLsiRegBits(pVid->pHw,DECODER_COMMAND_REG,DECODER_START);   //Start (or restart) the decoder
      break;
    case C_STATE_PLAY_PREFILL:
      VidStateEnterPlayPrefill(pVid);
      break;
    case C_STATE_PLAY:
      VidFreeze(pVid,0);
      pVid->TimeToUnblank= pVid->CurTime+UNBLANK_DELAY_TIME;
      pVid->pHw->u.s.StartSubPicture=1;
      pVid->CtrlState=pVid->NextState;
      break;
    case C_STATE_FLUSH:
      Err=VidStateEnterFlush(pVid);
      break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}

LOCAL long VidProcessPlay(VID* pVid) {
  long Err=0;
  switch (pVid->NextState) {
    case C_STATE_PLAY:
      VidNormalPlayHouseWork(pVid);
      break;
    case C_STATE_PAUSE:
      VidSetPlayRateImmediate(pVid,RATE_NORMAL);
      VidFreeze(pVid,1);
      pVid->CtrlState=pVid->NextState;
      break;
    case C_STATE_DATA_DISCON:
      VidSetStateTimeout(pVid,5000);      // Timeout if buffer doesn't empty after 5 second
      pVid->pHw->u.s.fChannelConfig=0;    // About to reconfig channel in RdyWaitForData
      pVid->DataXfrPending=0;
      pVid->CtrlState=pVid->NextState;
      break;
    case C_STATE_WAIT_EOS:
      VidStateEnterWaitEos(pVid);
      break;
    case C_STATE_PLAY_TO_FLUSH:
      pVid->nOddFieldCount=0;          // Use OddFieldCount to delay state transition to CSTATE_FLUSH
      pVid->pHw->u.s.fChannelConfig=0; // About to reconfig channel in RdyWaitForData
      pVid->CtrlState=pVid->NextState;
      break;
    case C_STATE_NULL_FLUSH:
      pVid->BeginFlush=0;
      VidSetStateTimeout(pVid,250);
      pVid->CtrlState=C_STATE_NULL_FLUSH;
      break;
    case C_STATE_SLOW_PLAY:
      VidSetPlayRateImmediate(pVid,pVid->ReqPlayRate);
      pVid->CtrlState=pVid->NextState;
      break;
    case C_STATE_FAST_PLAY:
      dvHwMainWriteLsiRegBits(pVid->pHw,PULL_DOWN_REG,0);
      VidSetPlayRateImmediate(pVid,pVid->ReqPlayRate);
      pVid->CtrlState=pVid->NextState;
      break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}

LOCAL long VidProcessSlowPlay(VID* pVid) {
  long Err=0;
  switch (pVid->NextState) {
    case C_STATE_SLOW_PLAY:
      VidTrickPlayHouseWork(pVid);
      break;
    case C_STATE_PLAY:
      VidSetPlayRateImmediate(pVid,RATE_NORMAL);
      pVid->CtrlState=pVid->NextState;
      break;
    case C_STATE_FLUSH:
      Err=VidStateEnterFlush(pVid);
      break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}

LOCAL long VidProcessFastPlay(VID* pVid) {
  long Err=0;
  switch (pVid->NextState) {
    case C_STATE_FAST_PLAY:
      VidTrickPlayHouseWork(pVid);
      break;
    case C_STATE_PLAY:
      VidSetPlayRateImmediate(pVid,RATE_NORMAL);
      pVid->CtrlState=pVid->NextState;
      break;
    case C_STATE_FLUSH:
      Err=VidStateEnterFlush(pVid);
      break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}

LOCAL long VidProcessPause(VID* pVid) {
  long Err=0;
  switch (pVid->NextState) {
    case C_STATE_PAUSE:
      VidBlankManager(pVid);
      break;
    case C_STATE_STEP:
      VidFreeze(pVid,0);
      VidFreeze(pVid,1);
      dvHwMainAdjustDecoderTime(pVid->pHw,3000);

      pVid->CtrlState=pVid->NextState;
      break;
    case C_STATE_PLAY:
      VidFreeze(pVid,0);
      pVid->CtrlState=pVid->NextState;
      break;
    case C_STATE_FLUSH:
      Err=VidStateEnterFlush(pVid);
      break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}

LOCAL long VidProcessStep(VID* pVid) {
  long Err=0;
  switch (pVid->NextState) {
    case C_STATE_STEP:
      break;
    case C_STATE_PAUSE:
      if(pVid->dwStep)
        pVid->dwStep--;
      pVid->CtrlState=pVid->NextState;
      break;
    case C_STATE_FLUSH:
      Err=VidStateEnterFlush(pVid);
      break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}

LOCAL long VidProcessDataDiscon(VID* pVid) {
  long Err=0;
  switch (pVid->NextState) {
    case C_STATE_DATA_DISCON:
      break;
    case C_STATE_DISCON_FLUSH:
      pVid->EnableDataTransfer=0;
      pVid->DataXfrPending=0;
      Err=VidStateEnterFlush(pVid);
      pVid->CtrlState=pVid->NextState;
      break;
    case C_STATE_WAIT_EOS:
      pVid->pHw->u.s.fChannelConfig=1;    // Not going to config channel after all since we got an EOS
      VidStateEnterWaitEos(pVid);
      break;
    case C_STATE_FLUSH:
      Err=VidStateEnterFlush(pVid);
      break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}

LOCAL long VidProcessDisconFlush(VID* pVid) {
  long Err=0;
  switch (pVid->NextState) {
    case C_STATE_DISCON_FLUSH:
    break;
    case C_STATE_FLUSH_COMPLETE:
      pVid->EnableDataTransfer=0;
      pVid->DataXfrPending=0;
      pVid->CtrlState=C_STATE_FLUSH_COMPLETE;
      break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}

LOCAL long VidProcessWaitEos(VID* pVid) {
  long Err=0;
  switch (pVid->NextState) {
    case C_STATE_WAIT_EOS:
      VidNormalPlayHouseWork(pVid);
      break;
    case C_STATE_EOS_TO_STILL:
//     VidSetStateTimeout(pVid,1500);      // Timeout after 1.5 S incase Post-EOS gets lost
       pVid->nOddFieldCount=0;
       pVid->CtrlState=pVid->NextState;
      break;
    case C_STATE_FLUSH:
      Err=VidStateEnterFlush(pVid);
      break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}

LOCAL long VidProcessEosToStill(VID* pVid) {
  long Err=0;
  switch (pVid->NextState) {
    case C_STATE_EOS_TO_STILL:
      break;
    case C_STATE_STILL:
      VidFreeze(pVid,1);
      pVid->CtrlState=pVid->NextState;
      break;
    case C_STATE_FLUSH:
      Err=VidStateEnterFlush(pVid);
      break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}


LOCAL long VidProcessStill(VID* pVid) {
  long Err=0;
  switch (pVid->NextState) {
    case C_STATE_STILL:
      VidBlankManager(pVid);
      // Nav Test Disk Audio test High Panic doesn't always work
      // if applied in PrePlay state. (Don't know why).
      // Apply Hi Panic well after VidFreeze has taken effect.
      if(pVid->nOddFieldCount>7)      
        VidPanicHigh(pVid);
      break;
    case C_STATE_PLAY_TITLEKEY:
      if(pVid->pHw->fCssWaitForTitleKey)
        pVid->pHw->fCssApplyingTitleKey=1;
      pVid->CtrlState=C_STATE_PLAY_TITLEKEY;
      break;
    case C_STATE_NULL_FLUSH:
      pVid->BeginFlush=0;
      VidSetStateTimeout(pVid,250);
      pVid->CtrlState=C_STATE_NULL_FLUSH;
      break;
    case C_STATE_FLUSH:
      Err=VidStateEnterFlush(pVid);
      break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}

LOCAL long VidProcessPlayTitleKey(VID* pVid) {
  long Err=0;
  switch (pVid->NextState) {
    case C_STATE_PLAY_TITLEKEY:
//      if(pVid->pHw->fCssWaitForTitleKey || (pVid->pCssMdb && pVid->pHw->fCssNewTitleKey)) {
      if(pVid->pHw->fCssWaitForTitleKey) {
        pVid->pHw->fCssWaitForTitleKey=1;
        pVid->pHw->fCssApplyingTitleKey=1;
      }
      break;
    case C_STATE_PLAY_PREFILL:
      pVid->PreEos=  0;
      pVid->PostEos= 0;
      pVid->ChannelFull=0;
      SyncCmd(pVid->pHw,SYNC_RESET,0,STRM_VID);
      VidStateEnterPlayPrefill(pVid);
      break;
    case C_STATE_FLUSH:
      Err=VidStateEnterFlush(pVid);
      break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}

LOCAL long VidProcessPlayPrefill(VID* pVid) {
  long Err=0;
  switch (pVid->NextState) {
    case C_STATE_PLAY_PREFILL:
      // poll to see if StartTime is available
      SyncCmd(pVid->pHw,SYNC_GET,SYNC_ITEM_VIDPTSMIN,(DWORD)&pVid->StartTime);
      break;
    case C_STATE_STALL_TIME:
      RegScrPause(pVid->pHw,FLAG_SCR_PAUSE);
      pVid->CtrlState=C_STATE_STALL_TIME;
      break;
    case C_STATE_PLAY_SYNC:
      Err=VidStateEnterPlaySync(pVid);
      break;
    case C_STATE_FLUSH:
      Err=VidStateEnterFlush(pVid);
      break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}

LOCAL long VidProcessStallTime(VID* pVid) {
  long Err=0;
  switch (pVid->NextState) {
    case C_STATE_STALL_TIME:
      break;
    case C_STATE_PLAY_SYNC:
      Err=VidStateEnterPlaySync(pVid);
      RegScrPause(pVid->pHw,FLAG_SCR_UNPAUSE);
      break;
    case C_STATE_FLUSH:
      Err=VidStateEnterFlush(pVid);
      break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}

LOCAL long VidProcessPlaySync(VID* pVid) {
  long Err=0;
  switch (pVid->NextState) {
    case C_STATE_PLAY_SYNC:
      pVid->pHw->u.s.StartSubPicture=1;
      break;
    case C_STATE_PLAY:
      if(pVid->fNewDspFmt) {
        ApplyVidDisplayFormat(pVid,&pVid->DspFmt);
        pVid->fNewDspFmt=0;
      }
      VidFreeze(pVid,0);
      pVid->pHw->u.s.StartSubPicture=1;
      pVid->pHw->fFlipAnchors=1;
      pVid->TimeToUnblank=pVid->CurTime+UNBLANK_DELAY_TIME;
      if(pVid->PreEos)
        VidPanicOff(pVid);
      else
        VidPanicLow(pVid);
      pVid->CtrlState=C_STATE_PLAY;
      break;
    case C_STATE_FLUSH:
      Err=VidStateEnterFlush(pVid);
      break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}

LOCAL long VidProcessPlayToFlush(VID* pVid) {
  long Err=0;
  switch (pVid->NextState) {
    case C_STATE_PLAY_TO_FLUSH:
      break;
    case C_STATE_FLUSH:
      pVid->CtrlState=pVid->NextState;
      Err=VidStateEnterFlush(pVid);
      break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}

LOCAL long VidProcessFlush(VID* pVid) {
  long Err=0;
  switch (pVid->NextState) {
    case C_STATE_FLUSH:
    break;
    case C_STATE_FLUSH_COMPLETE:
      pVid->EnableDataTransfer=0;
      pVid->DataXfrPending=0;
      pVid->CtrlState=C_STATE_FLUSH_COMPLETE;
      break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}

LOCAL long VidProcessFlushComplete(VID* pVid) {
  long Err=0;
  switch (pVid->NextState) {
    case C_STATE_FLUSH_COMPLETE:
    break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}

LOCAL long VidProcessNullFlush(VID* pVid) {
  long Err=0;
  switch (pVid->NextState) {
    case C_STATE_NULL_FLUSH:
    break;
    case C_STATE_STILL:
      pVid->CtrlState=C_STATE_STILL;
    break;
    case C_STATE_FLUSH:
      Err=VidStateEnterFlush(pVid);
      break;
    default:
      Err=VidStateInvalid(pVid);  // fall thru to Ready
    case C_STATE_READY:
      Err=VidStateEnterReady(pVid);
      break;
  }
  return Err;
}

LOCAL long VidProcessCss(VID* pVid) {
  long Err=0;
  if(pVid->pCssMdb) {
    PMDB_CALLBACK pfnCallback = (PMDB_CALLBACK)pVid->pCssMdb->pfnCallback;
    DbgMsg(DBG_VIDSTATE,"VidProcessCss()\r\n");
    pVid->pCssMdb->dwStatus=MDS_Success;
    pfnCallback(pVid->pCssMdb);
    pVid->pCssMdb=0;
  }
  return Err;
}

LOCAL long VidStateEnterWaitEos(VID* pVid) {
  long Err=0;
  pVid->EnableDataTransfer=0;
  pVid->DataXfrPending=0;
  pVid->TransferFlushPacket=FLUSH_NORMAL;
  VidPanicOff(pVid);                  // Panic OFF will allow last picture to decode and display
  VidSetStateTimeout(pVid,1500);      // 250mS in case PreEOS not detected
  pVid->CtrlState=C_STATE_WAIT_EOS;
  return Err;
}

LOCAL long VidStateEnterPlayPrefill(VID* pVid) {
  long Err=0;
  VidPanicHigh(pVid);
  VidSetEnableDataTransfer(pVid);
//  SyncCmd(pVid->pHw,SYNC_RESET,0,STRM_VID);
  SyncCmd(pVid->pHw,SYNC_ENABLE,STRM_VID,SYNC_ENABLE_ENABLE);
  VidSetStateTimeout(pVid,500);
  pVid->dwMaxXfer=0x20000;
  pVid->CtrlState=C_STATE_PLAY_PREFILL;
  return Err;
}

LOCAL long VidStateEnterPlaySync(VID* pVid) {
  long Err=0;
  SyncCmd(pVid->pHw,SYNC_GET,SYNC_ITEM_VIDPTSMIN,(DWORD)&pVid->StartTime);
  if(pVid->StartTime) {
    pVid->StartTime-=115*90;
    DbgMsg(DBG_VIDSTATE,"Video: state %s Start Time=",Ident(ID_VID_STATE,pVid->CtrlState));
    DbgDispPTS(DBG_VIDSTATE,pVid->StartTime,"","\r\n");
  } else {
    DbgMsg(DBG_VIDSTATE,"Video: state %s Start Time Not Found\r\n",Ident(ID_VID_STATE,pVid->CtrlState));
  }
  pVid->dwMaxXfer=0x4000;
  pVid->pHw->u.s.fVidChannelPrefill=0;
  pVid->CtrlState=C_STATE_PLAY_SYNC;
  return Err;
}


LOCAL long VidStateEnterFlush(VID* pVid) {
  long Err=0;
  pVid->ResetStream=0;
  pVid->BeginFlush=0;
  pVid->PurgeQ=1;
  pVid->pHw->u.s.fChannelConfig=0;  // about to reconfigure channel
  dvHwMainWriteLsiRegBits(pVid->pHw,DECODER_COMMAND_REG,DECODER_STOP);
//  VidPanicOff(pVid);
  VidSetStateTimeout(pVid,250);
  pVid->CtrlState=C_STATE_FLUSH;
  return Err;
}
 
LOCAL long VidStateEnterReady(VID* pVid) {
  long Err=0;
  PHW pHw= pVid->pHw;

  pVid->ResetStream=0;
  pVid->EndFlush=0;
  pVid->EnableDataTransfer=0;
  pVid->DataXfrPending=0;
  pVid->dwMaxXfer=0x20000;

  TransferFlushPkt(pVid->pHw,FLUSH_DISCARD); // Transfer Video Packet to flush all data in channel
  pVid->pHw->u.s.fChannelConfig=0;  // about to reconfigure channel
  SyncCmd(pVid->pHw,SYNC_ENABLE,0,SYNC_ENABLE_DISABLE);
  SyncCmd(pVid->pHw,SYNC_RESET,0,STRM_VID);
  VidCorrectSync(pVid,0,0);
  dvHwMainClrRdyError(pHw);

  VidPicParmSet(pVid);
  dvHwMainStopDecoderChannel(pHw);              //Stops audio,video,sub
  if(pVid->fNewDspFmt) {
    ApplyVidDisplayFormat(pVid,&pVid->DspFmt);
    pVid->fNewDspFmt=0;
  }
  SubpicCmd(pVid->pHw,SUBCMD_RESET,0,0);        //Reset Audio and subpicture since we just stopped them
  AudioCmd(pVid->pHw, AUDCMD_RESET,0,0);        //The channel will be configured in transit to RdyWaitForData
  {
    MIDASCMD Cmd;
    Cmd.hMidas = pHw;
    Cmd.cmd= MDS_CSS_ENABLE_CHANNEL;
    Cmd.u.M.Arg1=1;
    Cmd.u.M.Arg2=pHw->fCssEnabled;
    hwCopyProtection(&Cmd);
  }

  TVOutProc(pHw->hTvOut,TVOUT_ENABLE,1,0);

  pVid->CtrlState=C_STATE_READY;
  return Err;
}

LOCAL long VidSetStateTimeout(VID* pVid, DWORD dwMilliSecs) {
  if(dwMilliSecs) {
    pVid->StateTimeoutTime=pVid->CurTime+dwMilliSecs;
    if(pVid->StateTimeoutTime==0)
      pVid->StateTimeoutTime++;
  } else
    pVid->StateTimeoutTime=0;
  return 0;
}

LOCAL long IsStateTimeout(VID* pVid) {
  return (pVid->StateTimeoutTime && tmCompareTimes(pVid->CurTime,pVid->StateTimeoutTime));
}

LOCAL long VidSetNormalPlay(VID* pVid) {
  VidDecodeUnfreeze(pVid);
  dvHwMainWriteLsiRegBits(pVid->pHw,VIDEO_SKIP_MODE_REG,SKIP_NONE);
  dvHwMainWriteLsiRegBits(pVid->pHw,VIDEO_CONT_SKIP_MODE_REG,FALSE);
  pVid->SkipBPs=0;

  VidDecodeEnablePulldown(pVid,1);
  SyncCmd(pVid->pHw,SYNC_ENABLE,STRM_VID,SYNC_ENABLE_ENABLE);//Enable a/v sync on video if playing at normal speed.
  VidCorrectSync(pVid,0,0);
  pVid->CurPlayRate=RATE_NORMAL;
  DbgMsg(DBG_VIDSTATE,"Vid: Normal play.\r\n");
  return 0;
}

LOCAL long VidSetFastPlay(VID* pVid,long Rate) {
  VidDecodeUnfreeze(pVid);
  dvHwMainWriteLsiRegBits(pVid->pHw,VIDEO_SKIP_MODE_REG,SKIP_P_FRAME);
  dvHwMainWriteLsiRegBits(pVid->pHw,VIDEO_CONT_SKIP_MODE_REG,0x04);
  pVid->SkipBPs=1;

  VidDecodeEnablePulldown(pVid,0);
  SyncCmd(pVid->pHw,SYNC_ENABLE,STRM_VID,SYNC_ENABLE_DISABLE);		//Disable sync if not playing at normal speed.
  VidCorrectSync(pVid,0,0);

  // Use the slow play algorithm to control rate
  // of fast scan playback.
  if(pVid->fPlayRateMode)
    VidDecodeSlow(pVid,6);    // 6 was chosen to match the pioneer drives slow performance
                              // a faster setting will play less evenly
  pVid->CurPlayRate=Rate;
  if(Rate<0)
    DbgMsg(DBG_VIDSTATE,"Vid: Reverse play (%d).\r\n",Rate);
  else
    DbgMsg(DBG_VIDSTATE,"Vid: Fast play (%d).\r\n",Rate);
  return 0;
}

LOCAL long VidSetSlowPlay(VID* pVid,long Rate) {
  dvHwMainWriteLsiRegBits(pVid->pHw,VIDEO_SKIP_MODE_REG,SKIP_NONE);
  dvHwMainWriteLsiRegBits(pVid->pHw,VIDEO_CONT_SKIP_MODE_REG,FALSE);
  pVid->SkipBPs=0;
  VidDecodeEnablePulldown(pVid,0) ;
  VidDecodeSlow(pVid,Rate/RATE_NORMAL);

  SyncCmd(pVid->pHw,SYNC_ENABLE,STRM_VID,SYNC_ENABLE_ENABLE|SYNC_ENABLE_VID_MASTER);
  VidCorrectSync(pVid,0,0);

  pVid->CurPlayRate=Rate;
  DbgMsg(DBG_VIDSTATE,"Vid: Slow play (%d).\r\n",Rate);
  return 0;
}

LOCAL long VidStep(VID *pVid,DWORD dwStep) {
  pVid->dwStep=dwStep;
  VidStreamState(pVid,S_STATE_PAUSE);
  return 0;
}

LOCAL long VidSetPlayRateImmediate(VID* pVid,long Rate) {
  if(pVid->CurPlayRate!=Rate || pVid->SkipBPs) {
    pVid->pHw->dwVidRate=Rate;
    if(Rate<RATE_NORMAL)
      VidSetFastPlay(pVid,Rate);
    else if(Rate>RATE_NORMAL)
      VidSetSlowPlay(pVid,Rate);
    else
      VidSetNormalPlay(pVid);
  }
  return 0;
}

LOCAL long VidSetEnableDataTransfer(VID* pVid) {
  pVid->EnableDataTransfer=1;
  pVid->fScanForVidSeq=1;
  pVid->nPlayStateOddCount=0;
  pVid->NoDataCount=0;
  pVid->pHw->u.s.fVidChannelPrefill=1;
  dvHwMainWriteLsiRegBits(pVid->pHw,HOST_FORCE_BROKEN_LINK_REG,FORCE_BROKEN_LINK);
  VidSetPlayRateImmediate(pVid,pVid->ReqPlayRate);
  return 0;
}

LOCAL long VidBlankManager(VID* pVid) {
  if(pVid->fBlankChange) {
    pVid->fBlankChange=0;
    if(pVid->TimeToUnblank==0 || pVid->fBlankVideo)
      VidBlank(pVid,pVid->fBlankVideo,pVid->dwBlankColor);
  }
  if(pVid->TimeToUnblank && tmCompareTimes(pVid->CurTime,pVid->TimeToUnblank)) {
    if(pVid->wSeqHdrWidth!=pVid->pHw->wPicWidth || pVid->wSeqHdrHeight!=pVid->pHw->wPicHeight) {
      pVid->pHw->wPicWidth =pVid->wSeqHdrWidth;
      pVid->pHw->wPicHeight=pVid->wSeqHdrHeight;
      VidSetMpgPicture(pVid);
    }
    VidBlank(pVid,pVid->fBlankVideo,pVid->dwBlankColor);
    pVid->TimeToUnblank= 0;
  }
  return 0;
}

LOCAL long VidDispFormatManager(VID* pVid) {
  if(pVid->fUpdateDispFormat || !pVid->VidDisplayInitialized) {
    pVid->fUpdateDispFormat=0;
    VidUpdateDisplayFormat(pVid);
  }
  return 0;
}

LOCAL long VidNormalPlayHouseWork(VID* pVid) 
{
  VidBlankManager(pVid);

  if(pVid->SyncError) {
    if(pVid->SyncAhead) {
      VidDecodeFreezeNframes(pVid,(pVid->SyncError+21)/42);
      pVid->SyncError=0;
    } else {
      VidDecodeSkipNframes(pVid,(pVid->SyncError+21)/42);
      pVid->SyncError=0;
    }
  }

  if(pVid->StartLastField){
    if(VidDecodeSkipHandler(pVid)==0) {
      VidDecodePulldownHandler(pVid);
    }
  }

  if(pVid->StartEvenField) {
    VidDecodeFreezeHandler(pVid);
    if(pVid->nPlayStateOddCount++ == 3)
      dvHwMainWriteLsiRegBits(pVid->pHw,HOST_FORCE_BROKEN_LINK_REG,0);
  }
  return 0;
}

LOCAL long VidTrickPlayHouseWork(VID* pVid) 
{
  VidBlankManager(pVid);
  if(pVid->StartOddField) {
    VidDecodeFreezeHandler(pVid);
    if(pVid->ReqPlayRate!=pVid->CurPlayRate)
      // if rate change from fast to fast or slow to slow then set new rate.
      // Change between fast,normal and slow handled by state change.
      if( (pVid->ReqPlayRate>RATE_NORMAL && pVid->CurPlayRate>RATE_NORMAL)
        ||(pVid->ReqPlayRate<RATE_NORMAL && pVid->CurPlayRate<RATE_NORMAL))
        VidSetPlayRateImmediate(pVid,pVid->ReqPlayRate);
  }
  return 0;
}

/*************************************************************************/
/** Get/Set parameters.							**/
/*************************************************************************/
LOCAL DWORD VidGetState(VID *pVid);

LOCAL long VidGet(VID *pVid, DWORD Item, DWORD *pValue) {
  long Err= 0;
  DWORD Value;
  switch(Item) {
    case VIDITEM_STATE:		Value= VidGetState(pVid);  break;
    case VIDITEM_DISPLAY:	Value= pVid->DisplayType;  break;
    case VIDITEM_RATECUR:	Value= pVid->CurPlayRate; break;
    case VIDITEM_RATEMAX:	Value= pVid->PlayRateMax; break;
    default:			Err= MARGI_ERR_BADITEM;
  }
  if(!Err) {
    if(pValue)
      *pValue= Value;
    else
      Err= (long)Value;
  }
  return(Err);
}


LOCAL DWORD VidGetState(VID *pVid) {
  DWORD Value;
  switch(pVid->CtrlState) {
    case C_STATE_NOT_READY:
    case C_STATE_READY:
    case C_STATE_RDY_WAITDATA:
      Value= Z_STATE_STOP;
      break;
    case C_STATE_STEP:
    case C_STATE_PAUSE:
      Value= Z_STATE_PAUSE;
      break;
    case C_STATE_FAST_PLAY:
    case C_STATE_SLOW_PLAY:
    case C_STATE_PLAY:
      Value= Z_STATE_PLAY;
      break;
    case C_STATE_RDY_PREFILL:
    case C_STATE_START:
    case C_STATE_DATA_DISCON:
    case C_STATE_TIME_DISCON:
    case C_STATE_PLAY_TO_FLUSH:
    case C_STATE_WAIT_EOS:
    case C_STATE_EOS_TO_STILL:
    case C_STATE_STILL:
    case C_STATE_PLAY_PREFILL:
    case C_STATE_STALL_TIME:
    case C_STATE_FLUSH:
    case C_STATE_FLUSH_COMPLETE:
    case C_STATE_NULL_FLUSH:
      Value= Z_STATE_PREROLL;
      break;
    case C_STATE_PLAY_SYNC:
      Value= Z_STATE_SYNC;
      break;
    default:
    case C_STATE_UNK:
      Value= Z_STATE_UNK;
      break;
  }
  return(Value);
}

LOCAL long VidSet(VID *pVid, DWORD Item, DWORD Value) {
  long Err= 0;
  switch(Item) {
    default:			Err= MARGI_ERR_BADITEM;
  }
  return(Err);
}

LOCAL long VidStateInvalid(VID *pVid) {
  DbgMsg(DBG_VIDSTATE,"Video: Invalid state transition %s -> %s.\r\n",Ident(ID_VID_STATE,pVid->CtrlState),Ident(ID_VID_STATE,pVid->NextState));
  pVid->CtrlState=C_STATE_NOT_READY;
  return(MARGI_ERR_INVALID_STATE_TRANSITION);
}

LOCAL long VidCorrectSync(VID* pVid,DWORD dwAhead,DWORD dwError) {
  pVid->SyncAhead=dwAhead;
  pVid->SyncError=dwError;
  DbgMsg(DBG_SYNCADJ,"Video: CorrectSync Error=%dms %s \r\n",dwError,dwAhead?"ahead":"behind");
  return 0;
}


/*************************************************************************/
/** Miscellaneous video control functions.				**/
/*************************************************************************/
LOCAL long VidEnable(VID *pVid, DWORD Enable) {
  long Err= 0;
  if(pVid->fEnableZV!=(int)Enable) {
    pVid->fSetupZV=1;
    pVid->bZVCtrl=0;          //Keep track of values in Midas Vid registers
    pVid->bVidCtrl=0;         //so only reprogram if changing
    pVid->wHDelay=0;
    pVid->wHActive=0;
    pVid->fEnableZV=Enable;
  }

  Err=VidSetupTimebase(pVid);
  return(Err);
}


LOCAL long VidSetupTimebase(VID *pVid) {
  long Err= 0;
  WORD wHDelay,wHActive,wHCorrect;
  BYTE bZVCtrl= 0,bVidCtrl= 0;
  PHW pHw= pVid->pHw;

  if(pHw->u.s.f8BitZV) {
  // Set ZV port to 8 bit, invert HSync,VSync,
    bZVCtrl = (BYTE)((pVid->DisplayType==NTSC)?0x21:0x22);
    bVidCtrl= (BYTE)(Opt.dwInvertClock?0x01:0x09);
    wHCorrect=2;
  } else {
   // Set ZV port to 16 bit, -ve HSync, +ve VSync,
    bZVCtrl= (BYTE)((pVid->DisplayType==NTSC)?0x01:0x02);
    //bZVCtrl = 0xE3; //Include this for 24 bit display
    bVidCtrl= (BYTE)(Opt.dwInvertClock?0x17:0x1F);
    wHCorrect=1;
  }
  if(pVid->fEnableZV) 
    bZVCtrl|=0xc0;

  if(pVid->bZVCtrl != bZVCtrl) {
    WrByte(pHw,REG_INDEX(pHw),(BYTE)(RIX_ZV_CNTRL));
    WrByte(pHw,REG_DATA(pHw), bZVCtrl);
    pVid->bZVCtrl=bZVCtrl;
  }
  if(pVid->bVidCtrl != bVidCtrl) {
    WrByte(pHw,REG_INDEX(pHw),(BYTE)RIX_VID_CNTRL);
    WrByte(pHw,REG_DATA(pHw), bVidCtrl);
    pVid->bVidCtrl=bVidCtrl;
  }
  DbgMsg(DBG_MSGVID,"ZVCtrl=0x%02x, VidCtrl=0x%02x, wHCorrect=%d \r\n",bZVCtrl,bVidCtrl,wHCorrect);

  wHDelay = 244 - wHCorrect;
  wHActive= wHDelay+1440;
  wHDelay+=(pVid->DisplayType==NTSC)?0:20;  // adjust offset for PAL

  if(wHDelay!=pVid->wHDelay || wHActive!=pVid->wHActive) {
    WrByte(pHw,REG_INDEX(pHw),(BYTE)RIX_VID_HDELAY);
    WrByte(pHw,REG_DATA(pHw), (BYTE)(wHDelay&0x0ff));
    // Set HActive=244+1440-1 = 1683 (0x693)
    WrByte(pHw,REG_INDEX(pHw),(BYTE)RIX_VID_HACTIVE);
    WrByte(pHw,REG_DATA(pHw), (BYTE)(wHActive&0x0ff));
    // Set hi nibbles of HActive and HDelay
    WrByte(pHw,REG_INDEX(pHw),(BYTE)RIX_VID_MSBS);
    WrByte(pHw,REG_DATA(pHw), (BYTE)(((wHActive&0x700)>>4)|((wHDelay &0x700)>>8)));
    pVid->wHDelay =wHDelay;
    pVid->wHActive=wHActive;
  }
  return(Err);
}



LOCAL long VidBlank(VID *pVid, DWORD Blank,DWORD dwColor) {
  PHW pHw= pVid->pHw;
  if(Blank) {
    DbgMsg(DBG_HWCMD,"dvHwBlankVideo\r\n");
    switch(dwColor) {
      case 0: dwColor=0x00808000; break;  //Black
      case 1: dwColor=0x0072d423; break;  //Blue
      default: break;                                //Custom color
    }
    dvHwMainWriteLsiReg(pHw,266,(BYTE) dwColor);
    dvHwMainWriteLsiReg(pHw,267,(BYTE)(dwColor>>8));
    dvHwMainWriteLsiReg(pHw,268,(BYTE)(dwColor>>16));

    dvHwMainWriteLsiRegBits(pHw,VID_BACKGROUND_REG,VIDEO_BLUE);
    dvHwMainWriteLsiRegBits(pHw,DISPLAY_START_REG,TRUE);
    dvHwMainWriteLsiRegBits(pHw,SPU_MIX_ENABLE_REG,FALSE);
    pVid->Blanked= 1;
  } else {						//Unblanking video
    DbgMsg(DBG_HWCMD,"dvHwUnBlankVideo\r\n");
    dvHwMainWriteLsiRegBits(pVid->pHw,DISPLAY_OVERRIDE_REG,DISP_NORMAL_MODE);
    dvHwMainWriteLsiRegBits(pHw,VID_BACKGROUND_REG,VIDEO_NORMAL);
    dvHwMainWriteLsiRegBits(pHw,DISPLAY_START_REG,TRUE);
    dvHwMainWriteLsiRegBits(pHw,SPU_MIX_ENABLE_REG,0x02);
    pVid->Blanked= 0;
  }
  return(0);
}

LOCAL long VidPicParmSet(VID *pVid) {
  PHW pHw= pVid->pHw;
  if(pHw->wPicHeight==288 || pHw->wPicHeight==576) {
    pVid->DisplayType = PAL;
  } else {
    pVid->DisplayType = NTSC;
  }

  if(pHw->nMpegType&1) {  // If MPEG 1
    if(pHw->dwPicRate < 29970)
      VidDecodeSetupPulldown(pVid,PULLDOWN_MAN32);
    else
      VidDecodeSetupPulldown(pVid,PULLDOWN_NONE);
  } else
      VidDecodeSetupPulldown(pVid,PULLDOWN_AUTO);

  return(0);
}

LOCAL long VidSelectStream(VID *pVid, DWORD Stream) {
  PHW pHw= pVid->pHw;
  BYTE RegVal;
  if(Stream==-1)
    dvHwMainWriteLsiRegBits(pHw,VIDEO_STREAM_REG,0x00); //Discard all Video
  else {
    RegVal= (BYTE)(0x10|Stream);
//    if(pHw->fUsePesHdr)
//      RegVal|= 0xc0;
    dvHwMainWriteLsiReg(pHw,145,RegVal);
  }
  return(0);
}

LOCAL long VidSetBlank(VID* pVid,DWORD dwBlankState,DWORD dwColor) {
  long Err=0;
  int  fSetBlank;
  switch(dwBlankState) {
    case 0: fSetBlank=0; break;
    case 1: fSetBlank=1; break;
    default:fSetBlank=pVid->fBlankVideo; break;
  }
  if(pVid->dwBlankColor!=dwColor || pVid->fBlankVideo!=fSetBlank) {
    pVid->dwBlankColor=dwColor;
    pVid->fBlankVideo=fSetBlank;
    pVid->fBlankChange=1;
  }
  return Err;
}


/*************************************************************************/
/** Panic modes								**/
/*************************************************************************/
//Panic OFF: This allows video channel to completely empty, possibly underrunning.
LOCAL long VidPanicOff(VID *pVid) {
  DbgMsg(DBG_VIDPANIC,"Video: Panic OFF.\r\n");
  dvHwMainSetAddr(pVid->pHw,VID_CHAN_PANIC_REG,0);
  dvHwMainWriteLsiRegBits(pVid->pHw,VID_PANIC_MODE_REG,0);
  return(0);
}

//Panic LOW: Video decoder will freeze before underrunning.
LOCAL long VidPanicLow(VID *pVid) {
  DbgMsg(DBG_VIDPANIC,"Video: Panic LOW.\r\n");
  dvHwMainSetAddr(pVid->pHw,VID_CHAN_PANIC_REG,0);
  dvHwMainWriteLsiRegBits(pVid->pHw,VID_PANIC_MODE_REG,PICS_IN_CHAN_PANIC);
  return(0);
}

//Panic HIGH: Video decoder always frozen due to panic level.
LOCAL long VidPanicHigh(VID *pVid) {
  DbgMsg(DBG_VIDPANIC,"Video: Panic HIGH.\r\n");
  dvHwMainSetAddr(pVid->pHw,VID_CHAN_PANIC_REG,(DWORD)0x003ff);
  dvHwMainWriteLsiRegBits(pVid->pHw,VID_PANIC_MODE_REG,PICS_IN_CHAN_PANIC);
//  dvHwMainWriteLsiRegBits(pVid->pHw,VID_PANIC_MODE_REG,VID_NUMITEMS_PANIC);
  return(0);
}

LOCAL long VidFreeze(VID* pVid,int nFreeze) {
  if(nFreeze)
    VidDecodeFreezeFrame(pVid);
  else
    VidDecodeUnfreezeFrame(pVid);
  return 0;
}

/*************************************************************************/
/** Reprogram video picture rate.					**/
/*************************************************************************/
LOCAL long VidRate(VID *pVid, MDS_PLAYRATE *pRate) {
  pVid->ReqPlayRate=(pRate->Rate>=0)?pRate->Rate:-pRate->Rate;
  pVid->fPlayRateMode=pRate->Mode;
  return(0);
}

/*************************************************************************/
/** Set display mode							**/
/*************************************************************************/
LOCAL long VidDisplayLetterbox(VID *pVid);
LOCAL long VidDisplayPanScan(VID *pVid);
LOCAL long VidDisplayNormal(VID *pVid);

LOCAL long VidDisplayFormat(VID *pVid, PVID_DISP_FORMAT pFrm) {
  memcpy(&pVid->DspFmt,pFrm,sizeof(pVid->DspFmt));
  pVid->fNewDspFmt=1;
  return 0;
}

LOCAL long ApplyVidDisplayFormat(VID *pVid, PVID_DISP_FORMAT pFrm) {
  long Err=0;
  PHW pHw= pVid->pHw;
  int fResetStream=0;

  if(  (pVid->dwDispMode!=pFrm->dwDispMode)
     ||(pHw->wPicWidth!=pFrm->wWidth)||(pHw->wPicHeight!=pFrm->wHeight)
     ||(pHw->nMpegType!=pFrm->nMpegType) ) {

    pVid->dwDispMode=pFrm->dwDispMode;
    pHw->wPicWidth= pFrm->wWidth;
    pHw->wPicHeight=pFrm->wHeight;
    pHw->nMpegType=pFrm->nMpegType;
    pHw->dwPicRate=pFrm->dwPicRate;
    pHw->dwAspectRatio=pFrm->dwAspectRatio;

    if(pHw->wPicHeight==288 || pHw->wPicHeight==576) {
      if(pVid->DisplayType!=PAL)
        fResetStream=1;
      pVid->DisplayType=PAL; 
    } else {
      if(pVid->DisplayType!=NTSC)
        fResetStream=1;
      pVid->DisplayType=NTSC;
    }
    
    if(!pHw->dwVidRate)
      pHw->dwVidRate= RATE_NORMAL;
    if(!pHw->dwVidMaxRate)
      pHw->dwVidMaxRate= VID_MAX_RATE;

    if(pHw->nMpegType&1) {    // if MPEG 1
      if(pHw->dwPicRate < 29970)
        VidDecodeSetupPulldown(pVid,PULLDOWN_MAN32);
      else
        VidDecodeSetupPulldown(pVid,PULLDOWN_NONE);
    } else
        VidDecodeSetupPulldown(pVid,PULLDOWN_AUTO);
    pVid->fUpdateDispFormat=1;
    if(fResetStream) {
      VidStreamReset(pVid);
      SubpicCmd(pHw,SUBCMD_RESET,0,0);
      AudioCmd(pHw,AUDCMD_RESET,0,0);
    }
  }
  return(Err);
}

LOCAL long VidUpdateDisplayFormat(VID *pVid) {
  long Err=0;
  VidDisplayNormal(pVid);	//First set everything to normal?
  switch(pVid->dwDispMode) {
    case MDS_LETTER_BOX:  Err= VidDisplayLetterbox(pVid); break;
    case MDS_PAN_SCAN:    Err= VidDisplayPanScan(pVid); break;
    case MDS_NORMAL_RECT: break;
    default:			    Err= MARGI_ERR_BADITEM;
  }
  pVid->VidDisplayInitialized=1;
  return(Err);
}

LOCAL long VidDisplayLetterbox(VID *pVid) {
  PHW pHw= pVid->pHw;
  WORD wDivider;
  WORD wStart;
  WORD wEnd;

  //wDivider = (lpDvHw->fMpeg1?1000:2000);
  wDivider = 2000;
  wStart = (WORD)(23 + ((pHw->wPicHeight*125)/wDivider));   //Calcuate the start (480*0.125)/2 field
  wEnd   = (WORD)(wStart + ((pHw->wPicHeight*750)/wDivider) - 3);

  dvHwMainWriteLsiRegBits(pHw,MAIN_START_ROW_LOREG,(BYTE)wStart);
  dvHwMainWriteLsiRegBits(pHw,MAIN_START_ROW_HIREG,(BYTE)(wStart>>8));
  dvHwMainWriteLsiRegBits(pHw,MAIN_END_ROW_LOREG,(BYTE)wEnd); 
  dvHwMainWriteLsiRegBits(pHw,MAIN_END_ROW_HIREG,(BYTE)((wEnd&0xFF00)>>4));

  //if(lpDvHw->fMpeg1)
  //  dvHwWriteLsiRegBits(DISPLAY_MODE_REG,PROGRESSIVE_LETTERBOX);
  //else
    dvHwMainWriteLsiRegBits(pHw,DISPLAY_MODE_REG,INTERLACED_LETTERBOX);

  DbgMsg(DBG_HWINITSRB,"Setting Up Display mode to Letter Box\r\n");
  return 0;
}

LOCAL long VidDisplayPanScan(VID *pVid) {
  PHW pHw= pVid->pHw;
  BYTE bHScale;
  dvHwMainWriteLsiRegBits(pHw,PANSCAN_FROM_BITSTREAM_REG,0x40);
  bHScale = (BYTE)((((DWORD)540*256)+719)/720);
  dvHwMainWriteLsiReg(pHw,HORIZONTAL_FILTER_SCALE,bHScale);
  dvHwMainWriteLsiReg(pHw,MAIN_READ_LINE_REG,(BYTE)(540/8));  // Set main reads per line = 720/8
  dvHwMainWriteLsiRegBits(pHw,HORIZONTAL_FILTER_ENABLE,2);
  DbgMsg(DBG_HWINITSRB,"Setting Up Display mode to PanScan\r\n");
  return 0;
}

LOCAL long VidSetMpgPicture(VID *pVid) {
  PHW pHw= pVid->pHw;
  BYTE bHScale;

  bHScale=(BYTE)(((pHw->wPicWidth*256)+719)/720);
  dvHwMainWriteLsiReg(pHw,HORIZONTAL_FILTER_SCALE,bHScale);
  dvHwMainWriteLsiReg(pHw,MAIN_READ_LINE_REG,(BYTE)(pHw->wPicWidth/8));  // Set main reads per line = 720/8
  dvHwMainWriteLsiRegBits(pHw,HORIZONTAL_FILTER_ENABLE,2);
  if(pHw->wPicHeight>=480) {
    if(pVid->DisplayType==PAL)
      dvHwMainWriteLsiRegBits(pHw,DISPLAY_MODE_REG,INTER_CHROMA_LINE_REPEAT_FILTER);
    else
      dvHwMainWriteLsiRegBits(pHw,DISPLAY_MODE_REG,INTER_CHROMA_FIELD_REPEAT_FILTER);
  } else {
    dvHwMainWriteLsiRegBits(pHw,DISPLAY_MODE_REG,PROGRESSIVE_LINE_REPEAT);
  }
  return 0;
}

LOCAL long VidDisplayNormal(VID *pVid) {
  PHW pHw= pVid->pHw;

  if(pVid->DisplayType==PAL) {
    dvHwMainWriteLsiRegBits(pHw,DISPLAY_MODE_REG,INTER_CHROMA_LINE_REPEAT_FILTER);
    dvHwMainWriteLsiRegBits(pHw,TV_STANDARD_REG,PAL);
  } else {
    dvHwMainWriteLsiRegBits(pHw,DISPLAY_MODE_REG,INTER_CHROMA_FIELD_REPEAT_FILTER);
    dvHwMainWriteLsiRegBits(pHw,TV_STANDARD_REG,NTSC);
  }
  VidSetMpgPicture(pVid);

  dvHwMainWriteLsiRegBits(pHw,PANSCAN_FROM_BITSTREAM_REG,0x00);
  dvHwMainWriteLsiReg(pHw,284,0x53);
  dvHwMainWriteLsiReg(pHw,279,0x80);  //field inversion
  VidSetupTimebase(pVid);

  if(pVid->DisplayType==PAL) {
    TVOutProc(pHw->hTvOut,TVOUT_SET_STANDARD,TV_TYPE_PAL,0);
  } else {
    TVOutProc(pHw->hTvOut,TVOUT_SET_STANDARD,TV_TYPE_NTSC,0);
  }

  DbgMsg(DBG_HWINITSRB,"Setting Up Normal Rect Display Mode \r\n");
  return 0;
}


/*************************************************************************/
/** Video Decoder Rate Handling
/*************************************************************************/
void VidDecodeFreezeReset(VID *pVid) 
{
  PHW pHw= pVid->pHw;
  memset(&pHw->freeze,0,sizeof(pHw->freeze));
}

void VidDecodeFreezeHandler(VID *pVid) 
{
  PHW pHw= pVid->pHw;
  //If Stop then compute when to Run
  if(pHw->freeze.nStopCount) {
    if(--pHw->freeze.nStopCount==0) {
      pHw->freeze.fIssueRun=1;
      if(!pHw->freeze.fRepeat)
        pHw->freeze.nRunDuration=0;
    }
  }

  if(pHw->freeze.fIssueRun) {
    dvHwMainWriteLsiReg(pHw,237,0);
//    dvHwMainWriteLsiRegBits(pHw,FREEZE_MODE_REG,UNFREEZE);
#ifdef _NEC_DBG_
    SetDbgBit(2,1);
#endif
    pHw->freeze.nRunCount=pHw->freeze.nRunDuration;
    pHw->freeze.fIssueRun=0;
  }

  //If Run then compute when to Stop
  //Note: to step 1 frame must Run then Stop in same tick
  if(pHw->freeze.nRunCount) {
    if(--pHw->freeze.nRunCount==0) {
      pHw->freeze.fIssueStop=1;
      if(!pHw->freeze.fRepeat)
        pHw->freeze.nStopDuration=0;
    }
  }

  if(pHw->freeze.fIssueStop) {
    dvHwMainWriteLsiReg(pHw,237,3);
//    dvHwMainWriteLsiRegBits(pHw,FREEZE_MODE_REG,FREEZE_FIRST_FIELD);
#ifdef _NEC_DBG_
    SetDbgBit(2,0);
#endif
    pHw->freeze.nStopCount=pHw->freeze.nStopDuration;
    pHw->freeze.fIssueStop=0;
  }
}

void VidDecodeStep(VID* pVid)
{
  PHW pHw= pVid->pHw;
  pHw->freeze.nRunDuration=0;
  pHw->freeze.nStopDuration=0;
  pHw->freeze.fRepeat=0;
  pHw->freeze.fIssueRun=0;
  dvHwMainWriteLsiReg(pHw,237,0);
  dvHwMainWriteLsiRegBits(pHw,FREEZE_MODE_REG,UNFREEZE);
  dvHwMainWriteLsiRegBits(pHw,FREEZE_MODE_REG,FREEZE_FRAME);
}

void VidDecodeSlow(VID *pVid, int nRateDiv)
{
  PHW pHw= pVid->pHw;
  pHw->freeze.nRunDuration=2;
  pHw->freeze.nStopDuration=nRateDiv+1;
  pHw->freeze.fRepeat=1;
  pHw->freeze.fIssueStop=1;
}

void VidDecodeFreeze(VID *pVid) 
{
  PHW pHw= pVid->pHw;
  pHw->freeze.nRunDuration=0;
  pHw->freeze.nStopDuration=0;
  pHw->freeze.fRepeat=0;
  pHw->freeze.fIssueStop=1;
}

void VidDecodeUnfreeze(VID *pVid) 
{
  PHW pHw= pVid->pHw;
  pHw->freeze.nRunDuration=0;
  pHw->freeze.nStopDuration=0;
  pHw->freeze.fRepeat=0;
  pHw->freeze.fIssueStop=0;
  pHw->freeze.fIssueRun=1;
}

void VidDecodeFreezeNframes(VID *pVid, int n)
{
  PHW pHw= pVid->pHw;
  pHw->freeze.nStopDuration=n;
  pHw->freeze.fIssueStop=1;
  pHw->freeze.fRepeat=0;
  pHw->skip.nSkipCount=0;  
  DbgMsg(DBG_SYNCSTART,"Sync : Freeze = %d\r\n",n);
}

void VidDecodeFreezeFrame(VID *pVid) 
{
  PHW pHw= pVid->pHw;
//  pHw->freeze.nRunDuration=0;
//  pHw->freeze.nStopDuration=0;
//  pHw->freeze.fRepeat=0;
//  pHw->freeze.fIssueStop=0;
  dvHwMainWriteLsiRegBits(pHw,FREEZE_MODE_REG,FREEZE_FRAME);
}

void VidDecodeUnfreezeFrame(VID *pVid) 
{
  PHW pHw= pVid->pHw;
//  pHw->freeze.nRunDuration=0;
//  pHw->freeze.nStopDuration=0;
//  pHw->freeze.fRepeat=0;
 // pHw->freeze.fIssueStop=0;
  dvHwMainWriteLsiRegBits(pHw,FREEZE_MODE_REG,UNFREEZE);
}

void VidDecodeSkipReset(VID *pVid) 
{
  PHW pHw= pVid->pHw;
  pHw->skip.fSkipIssued=0;
  pHw->skip.nSkipCount=0;
}

// return TRUE if skip in progress
int VidDecodeSkipHandler(VID *pVid) 
{
  PHW pHw= pVid->pHw;
  static int nCount=0;
  if(pHw->skip.fSkipIssued) {
    nCount++;
    if(dvHwMainReadLsiRegBits(pHw,VIDEO_SKIP_MODE_REG)==0) {
      DbgMsg(DBG_SYNCSTART,"Sync : frame skipped after %d ticks\r\n",nCount);
      if(pHw->skip.nSkipCount)
        pHw->skip.nSkipCount--;
      pHw->skip.fSkipIssued=0;
    }
  }
  if(pHw->skip.nSkipCount && !pHw->skip.fSkipIssued) {
    dvHwMainWriteLsiRegBits(pHw,VIDEO_SKIP_MODE_REG,SKIP_B_FRAME);
    pHw->skip.fSkipIssued=1;
    DbgMsg(DBG_SYNCSTART,"Sync : Skip B frame\r\n");
    nCount=0;
  }
  return pHw->skip.nSkipCount || pHw->skip.fSkipIssued;
}


void  VidDecodeSkipNframes(VID *pVid, int n)
{
  PHW pHw= pVid->pHw;
  pHw->freeze.fIssueRun=1;
  pHw->freeze.nStopDuration=0;
  pHw->freeze.fIssueStop=0;
  if(pHw->skip.nSkipCount==0) {
    pHw->skip.nSkipCount=n;  
//    pHw->skip.nSkipCount=2;  
    DbgMsg(DBG_SYNCSTART,"Sync : Skip = %d\r\n",n);
  } else {
    DbgMsg(DBG_SYNCSTART,"Sync : Skip %d ignored\r\n",n);
  }
}

void VidDecodeSetupPulldown(VID *pVid, BYTE bMode) {
  PHW pHw= pVid->pHw;
  pHw->pulldown.bMode=bMode;
  if(bMode==PULLDOWN_AUTO) {
    dvHwMainWriteLsiRegBits(pHw,PULL_DOWN_REG,AUTO_32);
  } else {
    pHw->pulldown.fManage=1;
    pHw->pulldown.fOn=(pHw->pulldown.bMode!=PULLDOWN_NONE);
  }
}

void VidDecodeEnablePulldown(VID *pVid, BYTE bEnable) 
{
  PHW pHw= pVid->pHw;
  if(bEnable) {
    pHw->pulldown.fManage=1;
    pHw->pulldown.fOn=1;
  } else {
     dvHwMainWriteLsiRegBits(pHw,PULL_DOWN_REG,0);
      pHw->pulldown.fManage=0;
      pHw->pulldown.fOn=0;
  }
}

int VidDecodePulldownHandler(VID *pVid) 
{
  PHW pHw= pVid->pHw;
  if(pHw->pulldown.fManage) {
    pHw->pulldown.fManage=0;
    if(pHw->pulldown.fOn) {
      if(pHw->pulldown.bMode==PULLDOWN_AUTO)
        dvHwMainWriteLsiRegBits(pHw,PULL_DOWN_REG,AUTO_32);
      else if(pHw->pulldown.bMode==PULLDOWN_MAN32) {
        pHw->pulldown.fManage=1;
        switch(pHw->pulldown.bCount++) {
          case 0: dvHwMainWriteLsiRegBits(pHw,PULL_DOWN_REG,TOP_FIELD_1ST|REPEAT_1ST_FIRST); break;
          case 1: dvHwMainWriteLsiRegBits(pHw,PULL_DOWN_REG,0); break;
          case 2: dvHwMainWriteLsiRegBits(pHw,PULL_DOWN_REG,REPEAT_1ST_FIRST); break;
          default:
          case 3: dvHwMainWriteLsiRegBits(pHw,PULL_DOWN_REG,TOP_FIELD_1ST);
                  pHw->pulldown.bCount=0;
                  break;
        }
      } else
        dvHwMainWriteLsiRegBits(pHw,PULL_DOWN_REG,0);
    } else {
      dvHwMainWriteLsiRegBits(pHw,PULL_DOWN_REG,0);
      pHw->pulldown.fManage=0;
      pHw->pulldown.bCount=0;
    }
  }
  return 0;
}

LOCAL void VidDbgProfile(VID* pVid) {
#ifdef _DEBUG
  DWORD BytesInChnl,PicsInChnl;
  BytesInChnl= dvHwMainGetAddr(pVid->pHw,VID_CHAN_NUMITEM_REG)*8;
  PicsInChnl= dvHwMainReadLsiReg(pVid->pHw,150)|((WORD)dvHwMainReadLsiReg(pVid->pHw,151)<<8);
  if(pVid->hPerfChnl)
    DbgMsgCmd(DBGCMD_PERFSET,pVid->hPerfChnl,BytesInChnl);
  if(pVid->hPerfPics)
    DbgMsgCmd(DBGCMD_PERFSET,pVid->hPerfPics,PicsInChnl);
#endif
}

/*************************************************************************/
/** TIMER								**/
/** -	When an EOS is detected in the channel (PreEos), send a normal	**/
/**	flush packet and set Panic OFF.					**/
/** -	Wait until EOS is detected at decoder (PostEos), then set Panic	**/
/**	LOW.								**/
/*************************************************************************/
LOCAL long VidTimer(VID *pVid) {
  long Err= 0;
  long fieldstatus;
  PHW pHw= pVid->pHw;

  VidDbgProfile(pVid);

  // Update Control State machine inputs
  if(pHw->u.s.PreEos)  //pHw flags are transient, pVid flags are more permanent.
    pVid->PreEos++;
  if(pHw->u.s.PostEos)
    pVid->PostEos++;

  // if channel contains more than 1 picture or contains 1 picture and more than prefill # of bytes
  // the set channel full flag so we transition out of prefill state
  if((pHw->ChInf[VIDCH].dwPTUs>1)||((pHw->ChInf[VIDCH].dwPTUs>0) && (pHw->ChInf[VIDCH].dwItems>=pHw->ChInf[VIDCH].dwPrefillLevel)))
      pVid->ChannelFull=1;

  fieldstatus=dvHwMainGetFieldStatus(pHw);
  pVid->StartOddField=0;
  pVid->StartEvenField=0;
  pVid->StartLastField=0;
  if(fieldstatus&NEW_FIELD) {
    if(fieldstatus&LAST_FIELD)
      pVid->StartLastField=1;
    if(fieldstatus&ODD_FIELD) {
      pVid->StartOddField=1;
      pVid->nOddFieldCount++;
    } else
      pVid->StartEvenField=1;
  }
  
  pVid->CurScr= dvHwMainGetDecoderTime(pHw,0);
  pVid->CurTime=MargiGetTime();

  VidDetermineNextState(pVid);
  VidProcessState(pVid);

  if(pVid->fSetupZV) {
    VidEnable(pVid,pVid->fEnableZV);
    pVid->fSetupZV=0;
  }

  return(Err);
}

#ifdef _DEBUG
void VidScanForEOS(BYTE* p,DWORD dw) {
  while (dw-- > 3) {
    if(*p==0) 
      if(*(p+1)==0)
        if(*(p+2)==1)
          if(*(p+3)==0xb7) {
            volatile int i=0;
            break;
          }
    p++;
  }     
}
#endif

void VidScanForVidSeq(VID* pVid,BYTE* p, DWORD dw) {
  while (dw-- > 3) {
    if(*p==0) 
      if(*(p+1)==0)
        if(*(p+2)==1)
          if(*(p+3)==0xb3) {
            p+=4;
            pVid->wSeqHdrWidth =(*p++)<<4;
            pVid->wSeqHdrWidth+=(*p&0xf0)>>4;
            pVid->wSeqHdrHeight=(*p++&0x0f)<<8;
            pVid->wSeqHdrHeight+=*p;
            pVid->fScanForVidSeq=0;
            break;
          }
    p++;
  }     
}


LOCAL DWORD InsertMP2PacketHeader(PHW pHw, BYTE* Data, DWORD dLength, DWORD* pdwBytesWritten)
{
  Data[4] = (BYTE) ((dLength + 3) >> 8);
  Data[5] = (BYTE) (dLength + 3);
  return TransferData(pHw, Data, 9, pdwBytesWritten, STRM_VID);
}

#define  PACKETDATANUMBER   0xe000
#define  GROUPDATANUMBER    0x1000

LOCAL DWORD VidSendMP2Data(PHW pHw, BYTE* Data, PMDS_DATA_PACKET pMdp, DWORD* pdwBytesWritten) 
{
  DWORD  dwBytesTransfered = 0;
  ULONG   uDataLeft, uInPackDataLeft, SendLength;
  PVOID   pDataPoint = pMdp->Data;
  DWORD  Err = 0;

  *pdwBytesWritten = 0;
  uDataLeft = pMdp->DataUsed;
  while (uDataLeft > 0)  // Packtes
  {
    if (uDataLeft > PACKETDATANUMBER)
    {
      uInPackDataLeft = PACKETDATANUMBER;
      uDataLeft -= PACKETDATANUMBER;
    }
    else
    {
      uInPackDataLeft = uDataLeft;
      uDataLeft = 0;
    }
    Err = InsertMP2PacketHeader(pHw, Data, uInPackDataLeft, &dwBytesTransfered);
    if (Err)
      return 1;
//    *pdwBytesWritten += dwBytesTransfered;

    while (uInPackDataLeft > 0)
    {
      if (uInPackDataLeft > GROUPDATANUMBER)  
      {
        SendLength = GROUPDATANUMBER;
      }
      else
        SendLength = uInPackDataLeft;

      uInPackDataLeft -= SendLength;
      Err = TransferData(pHw, pDataPoint, SendLength, &dwBytesTransfered, STRM_VID);
      if (Err)
        return 1;
      *pdwBytesWritten += dwBytesTransfered;
      pDataPoint = (BYTE*) pDataPoint + SendLength;
    }
  }
  return 0;
}  

#if DUMP_VID
BYTE DramBuffer[65536];
#endif

/*************************************************************************/
/** Send data to decoder.						**/
/*************************************************************************/
LOCAL DWORD VidSendData(VID *pVid,PMDS_DATA_PACKET pMdp) {
  long Err= 0;
  DWORD dwBytesWritten=0;
  PHW pHw= pVid->pHw;

#if DUMP_VID
static int fDump=0;
DWORD dwStartWrAddr;
    dwStartWrAddr= (DWORD)dvHwMainReadLsiReg(pHw,96)<<3;
    dwStartWrAddr|=(DWORD)dvHwMainReadLsiReg(pHw,97)<<11;
    dwStartWrAddr|=(DWORD)dvHwMainReadLsiReg(pHw,98)<<19;
#endif

  if(pMdp) {
    pVid->LastDataTime= pVid->CurTime;

    // if discontinuity then disable transfer until state machine has dealt with it
    // disabled broken gop  for now SetBrokenGop doesn't work has to wait until channel empty
    if(pMdp->OptionsFlags&MDB_FLAG_DATADISCONTINUITY) {
      if(pVid->CurPlayRate==RATE_NORMAL) {  // Check PlayRate so don't disable in FFwd
        switch(pVid->CtrlState) {
          case C_STATE_RDY_PREFILL:
          case C_STATE_PLAY_PREFILL:
          case C_STATE_STALL_TIME:
          case C_STATE_PLAY_SYNC:
          case C_STATE_PLAY:
            pVid->EnableDataTransfer=0;
          case C_STATE_RDY_WAITDATA:
            // If we respond to flag then clear so we don't see it again
            pMdp->OptionsFlags&=~MDB_FLAG_DATADISCONTINUITY;
            DbgMsg(DBG_VIDSTATE,"VidSendData() Data Discon EnableDataTransfer=%d\r\n",pVid->EnableDataTransfer);
            break;
        }
      }
    }
    // If UseNewCss then wait for new Title Key by disabling data transfer with WaitForTitleKey
    if(pMdp->TypeSpecificFlags&MDB_FLAG_USENEWCSSKEY) {
      pMdp->TypeSpecificFlags&=~MDB_FLAG_USENEWCSSKEY;
//      if(pHw->fCssEnabled && !pHw->fCssNewTitleKey) {
      if(pHw->fCssEnabled) {
        pHw->fCssWaitForTitleKey=1;
      }
      DbgMsg(DBG_VIDSTATE,"VidSendData() MDB_FLAG_USENEWCSSKEY fCssWaitForTitleKey=%d;\r\n",pHw->fCssWaitForTitleKey);
    }


  }//if(pMdp)

  if(pVid->TransferFlushPacket && !pHw->fCssWaitForTitleKey) {
    // don't transfer eos flush if PreEos detected
    if(pVid->TransferFlushPacket!=FLUSH_EOS || !pVid->PreEos) {
      DbgMsg(DBG_VIDSTATE,"Video: TransferFlushPacket\r\n");
      TransferFlushPkt(pVid->pHw,pVid->TransferFlushPacket); // Transfer Video Packet to flush all data in channel
    }
    pVid->TransferFlushPacket=0; 
    pVid->fDataInPipe=0;
  }
  else if(pMdp && pMdp->DataUsed) {
    BYTE bPreEos;
    pVid->NoDataCount=0;
    
    //Test for PreEos flag. If this is set then must interrupt
    //data transfer to let state machine handle Panic correctly.
    bPreEos=dvHwMainImmediateCheckStatus(pHw,PRE_SEQ_END_CODE_REG);
    if(pVid->EnableDataTransfer && !pHw->fCssWaitForTitleKey && !pVid->PreEos && !bPreEos)
    {
      DWORD dwSpace=0;
      if(!pHw->fUseDMA) {
        if((dwSpace=pHw->ChInf[VIDCH].dwSize-pHw->ChInf[VIDCH].dwItems)>pHw->ChInf[VIDCH].dwSafety)
          dwSpace-=pHw->ChInf[VIDCH].dwSafety;
        else
          dwSpace=0;
      }
      if(pHw->fUseDMA || (dwSpace*8>=pMdp->DataUsed)) {
        void* pS;
        SYNC_TIME90 Time90;
#ifdef _DEBUG
        if(pMdp->DataUsed!=2048) {
          static int n=0;
          if(n++<5)
            DbgMsg(DBG_CRIT,"Video data length=%d\r\n",pMdp->DataUsed);
        }
#endif
      
        if(pHw->fTimeFormat10M)
          tmConvertTime10Mto90K(&pMdp->PresentationTime.Time,&Time90);
        else {
          Time90.dwTimeLo= (DWORD)GetLow_MdsLongLong (&pMdp->PresentationTime.Time);
          Time90.dwTimeHi= (DWORD)GetHigh_MdsLongLong(&pMdp->PresentationTime.Time);
        }
#ifdef _DEBUG
        VidScanForEOS(pMdp->Data,pMdp->DataUsed);
#endif
        if(pVid->fScanForVidSeq)
          VidScanForVidSeq(pVid,pMdp->Data,pMdp->DataUsed);

        if(!pHw->fUsePesHdr)
          pS=SyncVidPktBegin(pHw,pMdp->Data,pMdp->DataUsed,&Time90);
        if(pVid->pDataMdb && pVid->pDataMdb->dwRsvd[1])
        {
          Err=VidSendMP2Data(pHw, (BYTE*)pVid->pDataMdb->dwRsvd[0],  pMdp, &dwBytesWritten);
        }
        else
        {
          Err=TransferData(pHw,pMdp->Data,pMdp->DataUsed,&dwBytesWritten,STRM_VID);
        }
        if(Err) {
          DbgMsg(DBG_ERR,"Unable to Send Video Data \r\n",Ident(ID_MARGIERRORS,Err));
          if(Err==MARGI_ERR_WRT_FAILED) {   //RdyErr
            dvHwMainClrRdyError(pHw);
            dvHwMainReportRdyError(pHw,VIDCH);
          }
        } 
        if(!pHw->fUseDMA) {
          pHw->ChInf[VIDCH].dwItems+=dwBytesWritten/8;
          if(pHw->ChInf[VIDCH].dwItems>pHw->ChInf[VIDCH].dwSize)
            pHw->ChInf[VIDCH].dwItems=pHw->ChInf[VIDCH].dwSize;
        }
        pHw->fCssNewTitleKey=0;
        pVid->fDataInPipe=1;
#if DUMP_VID
      if(fDump) {
        volatile int i;
        volatile DWORD dwAddr,dwWrAddr,dwEnd;
        dwAddr= (DWORD)dvHwMainReadLsiReg(pHw,VID_CHAN_START_REG)<<8;
        dwAddr|=(DWORD)dvHwMainReadLsiReg(pHw,VID_CHAN_START_REG+1)<<16;
        dwWrAddr= (DWORD)dvHwMainReadLsiReg(pHw,96)<<3;
        dwWrAddr|=(DWORD)dvHwMainReadLsiReg(pHw,97)<<11;
        dwWrAddr|=(DWORD)dvHwMainReadLsiReg(pHw,98)<<19;
        dwEnd=((DWORD)&DramBuffer)+dwWrAddr-dwAddr;

//        dvHwMainReadDram(pHw,dwAddr,DramBuffer,sizeof(DramBuffer));
        dvHwMainReadDram(pHw,dwStartWrAddr,DramBuffer,0x800);
        i=0;
      }
#endif

#if(0)
      if(!pHw->fUsePesHdr)
          SyncVidPktEnd(pS);
#endif
        pHw->dwVidTotalXfr+=dwBytesWritten/8;
      }
    } else {
      pVid->DataXfrPending++;
#if(0)
      //If we are waiting for title key but not even started after ten tick then
      //assume we are not going to get one: Clear WaitForTitleKey flag will enable data transfers
      if(!pHw->fCssApplyingTitleKey && pVid->DataXfrPending>10) {
        pHw->fCssWaitForTitleKey=0;
        DbgMsg(DBG_VIDSTATE,"VidSendData: Cancel WaitForTitleKey\r\n");
      }
#endif
    }
  } else {
    pVid->NoDataCount++; //NoDataCount used to count ticks when no data to transfer (used to get out of prefill)
    if((pVid->NoDataCount&0x07)==0) {
      if(pVid->fDataInPipe && !pVid->TransferFlushPacket) {
        pVid->TransferFlushPacket=FLUSH_PIPE;
      }
    }
  }
#if(1)
  if(pMdp && pMdp->OptionsFlags&MDB_FLAG_TIMEDISCONTINUITY) {
    // Do Flush Eos if all data has been transfered
    if(dwBytesWritten==pMdp->DataUsed)
      pVid->TransferFlushPacket=FLUSH_EOS;
  }
#endif
  return(dwBytesWritten);
}

/**EOF: DVHWVID.C**/
