//#include <StrmIni.h>
//#include <KS.h>
//#include <KsMedia.h>
#include "system.h"
#include "DbgMsg.h"
#include "Dbgfltr.h"
#include "Settings.h"

#include "MidasApi.h"
#include "dvHwMain.h"
#include "lsi64020.h"
#include "eeprom.h"
#include "ininames.h"
#include "margierr.h"

#ifndef MAKELPARAM
#define MAKELPARAM(l, h)      (LPARAM)MAKELONG(l, h)
#endif
#ifndef MAX_REGION_COUNT
#define MAX_REGION_COUNT 5
#endif

#define EEPROM_REGION_ADDR  0x01C0
#define EEPROM_CAPS_ADDR    0x01FC


//long glDelay= 0x40000;
long glDelay= 0x40;

static BYTE Verbose= 1;
static WORD wEEPROM_RegVal= 0x0000;
static WORD wWritePort= WRITE_PORT_DEFAULT;
static WORD wReadPort= READ_PORT_DEFAULT;
static WORD CS_Mask=  CS_MASK_DEFAULT;
static WORD CLK_Mask= CLK_MASK_DEFAULT;
static WORD DI_Mask=  DI_MASK_DEFAULT;
static WORD DO_Mask=  DO_MASK_DEFAULT;
static DWORD dwDelayTime= 20; //1000;
static DWORD dwStdDelay= 20; //1000;
static WORD wCurrTestCount= 0;
static WORD wNumOfAddrBits= NUM_OF_ADDRESS_BITS_DEFAULT;
WORD wNumOfDataBits= NUM_OF_DATA_BITS_DEFAULT;
WORD wMaxAddress= 0x0200; //MAX_ADDRESS_DEFAULT;

static void SetMaxAddres(void);
static void Delay(BYTE bOuterLoop);
static void MyOut(PHW pHw,WORD wValue);
static WORD Myinp(PHW pHw);
static void EEPROM_Start(PHW pHw);
static void EEPROM_Stop(PHW pHw);
static void EEPROM_WriteBit(PHW pHw,BYTE bBit);
static BYTE EEPROM_ReadBit(PHW pHw);
static void EEPROM_WriteBits(PHW pHw,WORD wValue,WORD wNumOfBits);
static WORD EEPROM_ReadBits(PHW pHw,WORD wNumOfBits);
static void EEPROM_ModifyClock(PHW pHw,BYTE bFlag);
static void EEPROM_ModifyData(PHW pHw,BYTE bFlag);
static void WriteOneByte(PHW pHw,WORD bAddr,BYTE bVal);
static BYTE ReadOneByte(PHW pHw,WORD bAddr);
static DWORD OprnNOTSupported(WORD Message);
static DWORD EEPROM_ReadBytes(PHW pHw,LPBYTE pBuff,DWORD dwAddrAndCount);
static DWORD EEPROM_WriteBytes(PHW pHw,LPBYTE pBuff,DWORD dwAddrAndCount);
static DWORD EEPROM_Erase(PHW pHw,WORD wAddr);
static DWORD EEPROM_EW_Enable(PHW pHw);
static DWORD EEPROM_EW_Disable(PHW pHw);
static DWORD EEPROM_WriteAll(PHW pHw,WORD wVal);
static DWORD EEPROM_EraseAll(PHW pHw);
static DWORD EEPROM_SetConfig(DWORD Option,struct EEPROM_Config_s *EEPROM_Config);
static DWORD EEPROM_DisplayConfig(void);
static void EEPROM_WaitForDone(PHW pHw);

static DWORD WriteReadTest(PHW pHw,BYTE bVal);
static DWORD EEPROM_StickyBitTest(PHW pHw);
static DWORD EEPROM_WalkingOneTest(PHW pHw);
static DWORD EEPROM_WalkingZeroTest(PHW pHw);
static DWORD EEPROM_TestAll(PHW pHw,WORD wTimes);
static DWORD EEPROM_ReadCaps(PHW pHw,LPDWORD pdwCaps);
static long  EEPROM_ReadRegion(PHW, LPDWORD pdwRegion);
static DWORD EEPROM_WriteRegion(PHW,WORD wRegion);
static long EEPROM_ReadData(PHW pHw, LPBYTE lpBuff, DWORD dwSize);
static long EEPROM_WriteData(PHW pHw, LPBYTE lpBuff, DWORD dwSize);
static long EEPROM_TimedWriteData(PHW pHw, LPBYTE lpBuff, DWORD dwSize);
static long EEPROM_Timer(PHW pHw);

DWORD EEPROM_Proc(PHW pHw,WORD Msg,DWORD Parm1,DWORD Parm2) {
  DWORD Err= 0;
  //SetMaxAddres();
  switch(Msg) {
    case EEPROM_CONFIG:         Err= EEPROM_SetConfig(Parm1,(struct EEPROM_Config_s *)Parm2); break;
    case EEPROM_WRITE_BYTES:    Err= EEPROM_WriteBytes(pHw,(LPBYTE)Parm2,Parm1); break;
    case EEPROM_READ_BYTES:     Err= EEPROM_ReadBytes(pHw,(LPBYTE)Parm2,Parm1); break;
    case EEPROM_ERASE:          Err= EEPROM_Erase(pHw,(WORD)Parm1); break;
    case EEPROM_EW_ENABLE:      Err= EEPROM_EW_Enable(pHw); break;
    case EEPROM_EW_DISABLE:     Err= EEPROM_EW_Disable(pHw); break;
    case EEPROM_WRITE_ALL:      Err= EEPROM_WriteAll(pHw,(WORD)Parm1); break;
    case EEPROM_ERASE_ALL:      Err= EEPROM_EraseAll(pHw); break;
    case EEPROM_TEST_ALL:       Err= EEPROM_TestAll(pHw,(WORD)Parm1); break;
    case EEPROM_READ_REGION:    Err= EEPROM_ReadRegion(pHw,(LPDWORD)Parm1); break;
    case EEPROM_WRITE_REGION:   Err= EEPROM_WriteRegion(pHw,(WORD)Parm1); break;
    case EEPROM_READ_CAPS:      Err= EEPROM_ReadCaps(pHw,(LPDWORD)Parm1); break;
    case EEPROM_WRITE_DATA:     Err= EEPROM_WriteData(pHw,(LPBYTE)Parm1,Parm2); break;
    case EEPROM_TIMED_WRITE:    Err= EEPROM_TimedWriteData(pHw,(LPBYTE)Parm1,Parm2); break;
    case EEPROM_READ_DATA:      Err= EEPROM_ReadData(pHw,(LPBYTE)Parm1,Parm2); break;
    case EEPROM_TIMER:          Err= EEPROM_Timer(pHw); break;
    default:                    Err= OprnNOTSupported(Msg); break;
    
  }
  return Err;
}

static DWORD OprnNOTSupported(WORD Message) {
  DbgMsg(DBG_INFO,"The Requested Operation is NOT Supported (0x%02X) \r\n",Message);
  return(1);
}

static DWORD WriteReadTest(PHW pHw,BYTE bVal) {
  DWORD Err= 0;
  WORD bAddrCnt;
  BYTE bReadBackVal;
  DbgMsg(DBG_INFO,"Testing Using Pattern 0x%04X - (Paases Left - %02d) \r",bVal,wCurrTestCount);
  for(bAddrCnt=0;bAddrCnt<wMaxAddress;bAddrCnt++) {
    WriteOneByte(pHw,bAddrCnt,bVal);
    bReadBackVal= ReadOneByte(pHw,bAddrCnt);
    if(bReadBackVal!=bVal) {
      DbgMsg(DBG_INFO,"Value Expected=0x%04X; Value Read=0x%04X; at Address 0x%02X \r\n",bVal,bReadBackVal,bAddrCnt);
      Err= 1;
      break;
    } else {   
      //DbgMsg(DBG_INFO,DBGID" Tested Addr 0x%04X with Value 0x%04X \r\n",bAddrCnt,wVal);
    }
  }
  return Err;
}

static DWORD EEPROM_StickyBitTest(PHW pHw) {
  DWORD Err= 0;
  if(!(Err=WriteReadTest(pHw,0xAA)))
    Err=WriteReadTest(pHw,0x55);
  return Err;
}

static DWORD EEPROM_WalkingOneTest(PHW pHw) {
  DWORD Err= 0;
  BYTE bCnt= 0;
  BYTE Pattern= 0x80;
  for(bCnt=0;bCnt<8;bCnt++) {
    if(Err= WriteReadTest(pHw,(BYTE)(Pattern>>bCnt)))
      break;
  }
  return Err;
}

static DWORD EEPROM_WalkingZeroTest(PHW pHw) {
  DWORD Err= 0;
  BYTE bCnt= 0;
  BYTE Pattern= 0x80;
  for(bCnt=0;bCnt<8;bCnt++) {
    if(Err= WriteReadTest(pHw,(BYTE)(~(Pattern>>bCnt))))
      break;
  }
  return Err;
}

static DWORD EEPROM_TestAll(PHW pHw,WORD wTimes) {
  DWORD Err= 0;              
  BYTE wVerboseRestore= 0; 
  if(wTimes) {
    wVerboseRestore= Verbose;
    Verbose= 0;
    EEPROM_Proc(pHw,EEPROM_EW_ENABLE,0,0);
    while(wTimes--) {                    
      wCurrTestCount= wTimes;
      if(!(Err=EEPROM_StickyBitTest(pHw)))
        if(!(Err=EEPROM_WalkingOneTest(pHw)))
          if((Err=EEPROM_WalkingZeroTest(pHw)))
            break;
    }
    Verbose= wVerboseRestore;
    EEPROM_Proc(pHw,EEPROM_EW_DISABLE,0,0);
    if(Err)
      DbgMsg(DBG_INFO,"Failed Write Read Verify Test ...                                           \r\n");
    else
      DbgMsg(DBG_INFO,"Passed Write Read Verify Test ...                                           \r\n");
  }
  return Err;
}

static DWORD EEPROM_EraseAll(PHW pHw) {
  long Err= 0;
  EEPROM_Start(pHw);
  EEPROM_WriteBits(pHw,OPCODE_ERAL,NUM_OF_OPCODE_BITS);
  EEPROM_WriteBits(pHw,(WORD)((ADDRESS_ERAL<<(wNumOfAddrBits-2))),wNumOfAddrBits);
  EEPROM_Stop(pHw);
  return Err;
}

static DWORD EEPROM_WriteAll(PHW pHw,WORD wVal) {
  long Err= 0;
  EEPROM_Start(pHw);
  EEPROM_WriteBits(pHw,OPCODE_WRAL,NUM_OF_OPCODE_BITS);
  EEPROM_WriteBits(pHw,(WORD)((ADDRESS_WRAL<<(wNumOfAddrBits-2))),wNumOfAddrBits);
  EEPROM_WriteBits(pHw,wVal,wNumOfDataBits);
  EEPROM_Stop(pHw);
  DbgMsg(DBG_INFO,"Done Write All with %04X \r\n",wVal);
  return Err;
}

static DWORD EEPROM_EW_Disable(PHW pHw) {
  long Err= 0;
  EEPROM_Start(pHw);
  EEPROM_WriteBits(pHw,OPCODE_EWDS,NUM_OF_OPCODE_BITS);
  EEPROM_WriteBits(pHw,(WORD)((ADDRESS_EWDS<<(wNumOfAddrBits-2))),wNumOfAddrBits);
  EEPROM_Stop(pHw);
  return Err;
}

static DWORD EEPROM_EW_Enable(PHW pHw) {
  DWORD Err= 0;
  EEPROM_Start(pHw);
  EEPROM_WriteBits(pHw,OPCODE_EWEN,NUM_OF_OPCODE_BITS);
  EEPROM_WriteBits(pHw,(WORD)((ADDRESS_EWEN<<(wNumOfAddrBits-2))),wNumOfAddrBits);
  EEPROM_Stop(pHw);
  return Err;
}

static DWORD EEPROM_Erase(PHW pHw,WORD wAddr) {
  DWORD Err= 0;
  EEPROM_Start(pHw);
  EEPROM_WriteBits(pHw,OPCODE_ERASE,NUM_OF_OPCODE_BITS);
  EEPROM_WriteBits(pHw,wAddr,wNumOfAddrBits);
  EEPROM_Stop(pHw);
  return Err;
}

static BYTE ReadOneByte(PHW pHw,WORD bAddr) {
  BYTE bVal=0;
  if(bAddr<wMaxAddress) {
    EEPROM_Start(pHw);
    EEPROM_WriteBits(pHw,OPCODE_READ,NUM_OF_OPCODE_BITS);
    EEPROM_WriteBits(pHw,bAddr,wNumOfAddrBits);
    bVal=(BYTE)Myinp(pHw);
    if(!bVal&DO_Mask)
      DbgMsg(DBG_INFO,"Dummy ZERO is NOT received from EEPROM. Error During Read \r\n");
    else 
      if(Verbose)
        DbgMsg(DBG_INFO," Dummy ZERO is received from EEPROM \r\n");
    bVal=(BYTE)EEPROM_ReadBits(pHw,8);
    EEPROM_Stop(pHw);
    //DbgMsg(DBG_INFO,DBGID" Read Word @ %d \r\n",bAddr);
  }
  return bVal;
}

static void WriteOneByte(PHW pHw,WORD bAddr,BYTE bVal) {
  if(bAddr<wMaxAddress) {
    EEPROM_Start(pHw);
    EEPROM_WriteBits(pHw,OPCODE_WRITE,NUM_OF_OPCODE_BITS);
    EEPROM_WriteBits(pHw,bAddr,wNumOfAddrBits);
    EEPROM_WriteBits(pHw,bVal,8);
    EEPROM_ModifyClock(pHw,0);
    EEPROM_Stop(pHw);
    EEPROM_WaitForDone(pHw);
  }
  return;
}

static DWORD EEPROM_ReadBytes(PHW pHw,LPBYTE pBuff,DWORD dwAddrAndCount) {
  DWORD Err= 0;  
  WORD wAddr;
  WORD wBytes;
  wAddr= (WORD)((dwAddrAndCount&0xFFFF0000)>>16);
  wBytes=(WORD)(dwAddrAndCount&0x0000FFFF);
  while(wBytes--) {
    *pBuff=ReadOneByte(pHw,wAddr);
    DbgMsg(DBG_INFO,"Read 0x%04X from EEPROM @ Address 0x%04X \r\n",*pBuff,wAddr);
    pBuff++;
    wAddr++;
  }          
  return Err;
}

static DWORD EEPROM_WriteBytes(PHW pHw,LPBYTE pBuff,DWORD dwAddrAndCount) {
  DWORD Err= 0;  
  WORD wAddr;
  WORD wBytes;
  wAddr= (WORD)((dwAddrAndCount&0xFFFF0000)>>16);
  wBytes=(WORD)(dwAddrAndCount&0x0000FFFF);
  while(wBytes--) {
    WriteOneByte(pHw,wAddr,*pBuff);
    DbgMsg(DBG_INFO," Written 0x%04X to EEPROM @ Address 0x%04X \r\n",*pBuff,wAddr);
    pBuff++;
    wAddr++;
  }
  return Err;
}

  
  
/****************************************************************************
  Generates START/STOP Condition
*****************************************************************************/
static void EEPROM_Start(PHW pHw) {
  wEEPROM_RegVal= 0x0000;
  MyOut(pHw,wEEPROM_RegVal);
  Delay(1);
  wEEPROM_RegVal|=(CS_Mask|DI_Mask);
  MyOut(pHw,wEEPROM_RegVal);
  
//  while(!Myinp())
//    DbgMsg(DBG_INFO,DBGID" Waiting to Complete Programming...\r\n");
  
  Delay(1);
  wEEPROM_RegVal|=CLK_Mask;
  MyOut(pHw,wEEPROM_RegVal);
  Delay(1);
  if(Verbose)
    DbgMsg(DBG_INFO,DBGID"Start EEPROM \r\n");
  return;
}
static void EEPROM_Stop(PHW pHw) {
  WORD wCnt= 0;
  wEEPROM_RegVal= 0x0000;
  MyOut(pHw,wEEPROM_RegVal);
//  MyOut(CS_Mask);
#if(0)
  while(!Myinp(pHw)) {
    MyOut(0);
    Delay(150);
    MyOut(pHw,CS_Mask);
    if(!(++wCnt%10))
      //DbgMsg(DBG_INFO,DBGID" Waiting for EEPROM to complete programming (%04d) \r\n",wCnt);
  }
#endif
  if(Verbose)
    DbgMsg(DBG_INFO,"Stop EEPROM \r\n");
  return;
}

static void EEPROM_WaitForDone(PHW pHw) {
  volatile long n=0x1000;
  WORD wCnt= 0;
  MyOut(pHw,CS_Mask);

  if(glDelay) 
    for(n=glDelay;n;n--);

  n=0x1000;
  for(;n>=0;n--) {
    if (Myinp(pHw))
      break;
  }
  //if (!Myinp())
   //DbgMsg(DBG_INFO,"EEprom Wait for done time out\n");

  return;
}

/****************************************************************************
  Multi Bit Write / Read
*****************************************************************************/
static void EEPROM_WriteBits(PHW pHw,WORD wValue,WORD wNumOfBits) {
  WORD wCnt= 0;
  for(wCnt=0;wCnt<wNumOfBits;wCnt++)
    EEPROM_WriteBit(pHw,(BYTE)((wValue>>((wNumOfBits-1)-wCnt))&1));
  return;
}
static WORD EEPROM_ReadBits(PHW pHw,WORD wNumOfBits) {
  WORD Ret= 0;
  WORD wCnt= 0;
  for(wCnt=0;wCnt<wNumOfBits;wCnt++)
    Ret= (Ret<<1)|(EEPROM_ReadBit(pHw)&1);
  return Ret;
}


/****************************************************************************
  Single Bit Write / Read
*****************************************************************************/
static void EEPROM_WriteBit(PHW pHw,BYTE bBit) {
  EEPROM_ModifyClock(pHw,0);
  Delay(1);
  EEPROM_ModifyData(pHw,(BYTE)(bBit&1));
  Delay(0);
  EEPROM_ModifyClock(pHw,1);
  Delay(1);
  EEPROM_ModifyClock(pHw,0);
  //if(Verbose)
  //  DbgMsg(DBG_INFO,DBGID"Written Bit 0x%02X \r\n",(bBit&1));
  return;
}                                     
static BYTE EEPROM_ReadBit(PHW pHw) {
  BYTE Ret= 0;
  EEPROM_ModifyClock(pHw,0);
  Delay(1);
  EEPROM_ModifyClock(pHw,1);
  Delay(1);
  Ret= (BYTE) Myinp(pHw); 
  //if(Verbose)
  //  DbgMsg(DBG_INFO,DBGID"Read Bit 0x%02X \r\n",Ret);
  return Ret;
}


/****************************************************************************
  Routines used for Set/Reset Clock/Data Bits
  Also Updates the Image
*****************************************************************************/
static void EEPROM_ModifyClock(PHW pHw,BYTE bFlag) {
  if(bFlag)
    wEEPROM_RegVal|=CLK_Mask;
  else
    wEEPROM_RegVal&=~CLK_Mask;
  MyOut(pHw,wEEPROM_RegVal);
  return;
}
static void EEPROM_ModifyData(PHW pHw,BYTE bFlag) {
  if(bFlag)
    wEEPROM_RegVal|=DI_Mask;
  else
    wEEPROM_RegVal&=~DI_Mask;
  MyOut(pHw,wEEPROM_RegVal);
  return;
}


/****************************************************************************
  The Delay Routines. (CountDown Delay)
*****************************************************************************/
static void Delay(BYTE bOuterLoop) {
  DWORD dwCount= 0x0000;

  while(bOuterLoop--) {
    dwCount= dwDelayTime;
    while(dwCount--);
  }
  return;
}


/****************************************************************************
 The Lowest Level routine that outputs the values to ASIC and Printer Port 
*****************************************************************************/
static void MyOut(PHW pHw,WORD wValue) {
  WrByte(pHw,REG_INDEX(pHw),(BYTE)wWritePort);
  WrByte(pHw,REG_DATA(pHw),(BYTE)wValue);
  //DbgMsg(DBG_INFO,DBGID"Out 0x%02X \r\n",(BYTE)wValue);
  return;
}

static WORD Myinp(PHW pHw) {
  WORD Ret= 0;
  WrByte(pHw,REG_INDEX(pHw),(BYTE)wWritePort);
  Ret= (BYTE) ( (RdByte(pHw,REG_DATA(pHw))&DO_Mask) ? 1: 0);
  //DbgMsg(DBG_INFO,DBGID"In  0x%02X \r\n",(BYTE)Ret);
  return Ret;
}

static DWORD EEPROM_SetConfig(DWORD Option,struct EEPROM_Config_s *EEPROM_Config) {
  DWORD Err= 0;
  BYTE bTemp= 0;
  if(Option==EEPROM_SHOW_CONFIG)
    Err= EEPROM_DisplayConfig();
  else {
    if(EEPROM_Config->wWritePort)
      wWritePort= (WORD)EEPROM_Config->wWritePort;
    if(EEPROM_Config->wReadPort)
      wReadPort= (WORD)EEPROM_Config->wReadPort;
    dwDelayTime= dwStdDelay*EEPROM_Config->wDelayMultiplier;
    CS_Mask=  EEPROM_Config->CS_Mask;
    CLK_Mask= EEPROM_Config->CLK_Mask;
    DI_Mask=  EEPROM_Config->DI_Mask;
    DO_Mask=  EEPROM_Config->DO_Mask;
    wNumOfAddrBits= EEPROM_Config->wNumOfAddrBits;
    wNumOfDataBits= EEPROM_Config->wNumOfDataBits;
    wMaxAddress= 1;
    for(bTemp=0;bTemp<wNumOfAddrBits;bTemp++) 
      wMaxAddress*=2; 
  }
  return Err;                      
}
  
static DWORD EEPROM_DisplayConfig(void) {
  DWORD Err= 0;
  //DbgMsg(DBG_INFO,DBGID" Port Values : Write=0x%04X, Read=%04X. \r\n",wWritePort,wReadPort);
  //DbgMsg(DBG_INFO,DBGID" Mask Bits CS=0x%04X, CLK=0x%04X, DI=0x%04X, DO=0x%04X. \r\n",CS_Mask,CLK_Mask,DI_Mask,DO_Mask);
  //DbgMsg(DBG_INFO,DBGID" # of Bits. Address=0x%04X, Data=0x%04X  (Max Addr=0x%04X)\r\n",wNumOfAddrBits,wNumOfDataBits,wMaxAddress);
  //DbgMsg(DBG_INFO,DBGID" Delay Multiplier is Set to 0x%04X \r\n",(WORD)(dwDelayTime/dwStdDelay));
  return Err;
}

static void SetMaxAddres(void) {
  BYTE bTemp= 0;
  wMaxAddress= 1;
  for(bTemp=0;bTemp<wNumOfAddrBits;bTemp++) 
    wMaxAddress*=2; 
}


/************************************************************************
 **  dvHwEEPROMServices                                                **
 ************************************************************************

  Uses Address "x"   for Region Code
               "x+1" for Count (# of times the region code been changed)
               "x+2" for Initialization is done or not. ("0x00" for Not done)
               where "x" is the Address passed by the Caller  
  Message Parameter
                Zero for a read
                  (returns the region code stored at Address "x" (default address is 0x1F0)
                NonZero for a write
                  (returns 1 for a successful write & 0 for a unsuccessful write.

 ************************************************************************/

#pragma warning(disable:4769)

static struct EEPROM_Config_s Config_93LC66 = {
  0x0c,               //WORD wWritePort;
  0x0c,               //WORD wReadPort;
  9,                  //WORD wNumOfAddrBits;
  8,                  //WORD wNumOfDataBits;
  10,                  //WORD wDelayMultiplier;
  1,                  //WORD CS_Mask;
  2,                  //WORD CLK_Mask;
  4,                  //WORD DI_Mask;
  8                   //WORD DO_Mask;
};


#define REGION_ZERO  "0412-2g45-5645-2309"
#define REGION_ONE   "0699-1346-789q-vc32"
#define REGION_TWO   "5c34-we34-34ds-234v"
#define REGION_THREE "04q3-c223-5fv6-2w4d"
#define REGION_FOUR  "04ge-2f45-h455-f35e"
#define REGION_FIVE  "04we-e237-7777-1234"
#define REGION_SIX   "0341-aaaa-w767-6983"

#define MAX_REGION_CHANGES_ALLOWED  5

#define TAG_OFFSET            0x00
#define LENGTH_OFFSET         0x01
#define REGION_CODE_OFFSET    0x02
#define REGION_COUNT_OFFSET   0x03

#define REGION_CODE_TAG_1     0x57
#define REGION_CODE_LENGTH    0x02

#define ENCRYPTION_VERSION1   REGION_CODE_TAG_1

#if(0)
DWORD dvHwWriteEEPROM(PHW pHw,WORD wAddr,WORD wValue) {
  DWORD dwOffSet= 0, dwRet= 0;
  dwOffSet= (DWORD)(((DWORD)(wAddr)<<16)|1);
  EEPROM_Proc(pHw,EEPROM_EW_ENABLE,0,0);
  dwRet= EEPROM_Proc(pHw,EEPROM_WRITE,dwOffSet,(DWORD)(LPWORD)&wValue);
  EEPROM_Proc(pHw,EEPROM_EW_DISABLE,0,0);
  return dwRet;
}

DWORD dvHwReadEEPROM(PHW pHw,WORD wAddr) {
  DWORD dwOffSet= 0;
  WORD wRet= 0;
  dwOffSet= (DWORD)(((DWORD)(wAddr)<<16)|1);
  EEPROM_Proc(pHw,EEPROM_WRITE,dwOffSet,(DWORD)(LPWORD)&wRet);
  return wRet;
}
#endif

DWORD dvHwDecryptRegionCode(LPBYTE bRgnInfo, WORD wSize) {
  DWORD dwRet= 0;
  if((BYTE)(bRgnInfo[TAG_OFFSET])==(BYTE)ENCRYPTION_VERSION1) {
    DbgMsg(DBG_INFO," Region code Encryption Version - 0x%02X \r\n",ENCRYPTION_VERSION1);
  } else {
    DbgMsg(DBG_INFO," Unrecognised Encryption Version...... \r\n");
  }
  return dwRet;
}

DWORD dvHwEncryptRegionCode(LPBYTE bRgnInfo, WORD wSize) {
  DWORD dwRet= 0;
  if((BYTE)(bRgnInfo[TAG_OFFSET])==(BYTE)ENCRYPTION_VERSION1) {
    DbgMsg(DBG_INFO," Region code Encryption Version - 0x%02X \r\n",ENCRYPTION_VERSION1);
  } else {
    DbgMsg(DBG_INFO," Unrecognised Encryption Version...... \r\n");
  }
  return dwRet;
}

WORD ReadRegionCodeFromRegistry(void) {
  WORD wRgnCode= 0xFF;
  BYTE gfOverride[8];
  if(SettingLoad(SET_STR,IniMargin,gfOverride,0)) {
    if(gfOverride[0]=='0')
      wRgnCode=0;
    else if(gfOverride[0]=='1')
      wRgnCode=1;
    else if(gfOverride[0]=='2')
      wRgnCode=2;
    else if(gfOverride[0]=='3')
      wRgnCode=3;
    else if(gfOverride[0]=='4')
      wRgnCode=4;
    else if(gfOverride[0]=='5')
      wRgnCode=5;
    else if(gfOverride[0]=='6')
      wRgnCode=6;
    else
      wRgnCode=0xFF;
  }
  return wRgnCode;
}

static long EEPROM_ReadRegion(PHW pHw,LPDWORD pdwRegion) {
  long  lErr = 0;
  DWORD dwOffSet= 0;
  WORD  wTemp= 0;
  WORD  wNumOfBytes= 1;
  WORD  wNewRegionCode= 0;
//  BYTE  gfOverride[80];
  BYTE  bRegionInfo[64];
  WORD  wRegionInfoSize= 0;
  WORD  wRetRegionCode, wRetRegionCount;

  wRetRegionCount= 5;
/*  if(SettingLoad(SET_STR,IniMarginOR,gfOverride,0)) {
    if(!(RtlCompareUnicodeString(gfOverride,REGION_ZERO)))
      *pwRegion=wRetRegionCode=0;
    else if(!(stricmp(gfOverride,REGION_ONE)))
      *pwRegion=wRetRegionCode=1;
    else if(!(stricmp(gfOverride,REGION_TWO)))
      *pwRegion=wRetRegionCode=2;
    else if(!(stricmp(gfOverride,REGION_THREE)))
      *pwRegion=wRetRegionCode=3;
    else if(!(stricmp(gfOverride,REGION_FOUR)))
      *pwRegion=wRetRegionCode=4;
    else if(!(stricmp(gfOverride,REGION_FIVE)))
      *pwRegion=wRetRegionCode=5;
    else if(!(stricmp(gfOverride,REGION_SIX)))
      *pwRegion=wRetRegionCode=6;
    else
      *pwRegion=wRetRegionCode=0xFF;
    DbgMsg(DBG_INFO,"Region Code Override set ? %d \r\n",*pwRegion);
  //} else if(dvHw.pRsc->dwPciDevId==PCI_PLUTO || dvHw.pRsc->dwPciDevId==PCI_S3T3D || dvHw.pRsc->dwPciDevId==PCI_S3GX3) {
  //  wRetRegionCount= 5;
  //  *pwRegion=wRetRegionCode=1;
  } else {
*/    
    if((0x200-EEPROM_REGION_ADDR)>64)
      wRegionInfoSize= 64;
    else
      wRegionInfoSize= 0x200-EEPROM_REGION_ADDR;
    dwOffSet= (DWORD)(((DWORD)(EEPROM_REGION_ADDR)<<16)|wRegionInfoSize);
    memset((LPVOID)bRegionInfo,0xFF,sizeof(bRegionInfo));
    EEPROM_Proc(pHw,EEPROM_CONFIG,0,(DWORD)(struct EEPROM_Config_s *)&Config_93LC66);
    lErr = EEPROM_Proc(pHw,EEPROM_READ_BYTES,dwOffSet,(DWORD)bRegionInfo);
    dvHwDecryptRegionCode(bRegionInfo,wRegionInfoSize);
    if(bRegionInfo[TAG_OFFSET]==REGION_CODE_TAG_1) {
      wRetRegionCode= bRegionInfo[REGION_CODE_OFFSET];
      wRetRegionCount= bRegionInfo[REGION_COUNT_OFFSET];
      DbgMsg(DBG_INFO,"Region Code read from EEPROM %d \r\n",wRetRegionCode);
    } else {
      DbgMsg(DBG_INFO,"Starting Driver for the First Time ... \r\n");
      wNewRegionCode= ReadRegionCodeFromRegistry(); //(BYTE)SettingLoad(SET_DWORD,IniMargin,0,0xFF);
      if(wNewRegionCode==0xFF) {
        //If it is 0xFF do not write it to the HW. Just return with 0xFF
        wRetRegionCode= 0xFF;
        wRetRegionCount= MAX_REGION_COUNT;
      } else {
        bRegionInfo[TAG_OFFSET]= REGION_CODE_TAG_1;
        bRegionInfo[LENGTH_OFFSET]= REGION_CODE_LENGTH;
        bRegionInfo[REGION_CODE_OFFSET]= (BYTE)wNewRegionCode;
        bRegionInfo[REGION_COUNT_OFFSET]= 1;
        dvHwEncryptRegionCode(bRegionInfo,wRegionInfoSize);
        EEPROM_Proc(pHw,EEPROM_EW_ENABLE,0,0);
        lErr = EEPROM_Proc(pHw,EEPROM_WRITE_BYTES,dwOffSet,(DWORD)bRegionInfo);
        EEPROM_Proc(pHw,EEPROM_EW_DISABLE,0,0);
        wRetRegionCode  = bRegionInfo[REGION_CODE_OFFSET];
        wRetRegionCount = bRegionInfo[REGION_COUNT_OFFSET];
      }
    }
  //}
  if(pdwRegion)
    *pdwRegion=(DWORD)(MAX_REGION_COUNT<<16)|(DWORD)((wRetRegionCount&0xff)<<8)|(DWORD)(wRetRegionCode&0xff);

  return lErr;
}


static DWORD EEPROM_WriteRegion(PHW pHw,WORD wRegion) {
  DWORD dwRet= 0;
  DWORD dwOffSet= 0;
  WORD  wTemp= 0;
  WORD  wNumOfBytes= 1;
  WORD  wNewRegionCode= 0, wRegionCount= 0;
  BYTE  bRegionInfo[64];
  WORD  wRegionInfoSize= 0;
  WORD  wRetRegionCode, wRetRegionCount;

  wNewRegionCode= (BYTE)wRegion;
  if((0x200-EEPROM_REGION_ADDR)>64)
    wRegionInfoSize= 64;
  else
    wRegionInfoSize= 0x200-EEPROM_REGION_ADDR;
  dwOffSet= (DWORD)(((DWORD)(EEPROM_REGION_ADDR)<<16)|wRegionInfoSize);
  memset((LPVOID)bRegionInfo,0xFF,sizeof(bRegionInfo));
  EEPROM_Proc(pHw,EEPROM_CONFIG,0,(DWORD)(struct EEPROM_Config_s *)&Config_93LC66);
  if(EEPROM_Proc(pHw,EEPROM_READ_BYTES,dwOffSet,(DWORD)bRegionInfo)) {
    dwRet = 0xFFFFFFFF; //REGION_ERR_INFO;
  }
  dvHwDecryptRegionCode(bRegionInfo,wRegionInfoSize);
  if(bRegionInfo[TAG_OFFSET]==REGION_CODE_TAG_1) {
    bRegionInfo[REGION_CODE_OFFSET]= (BYTE)wNewRegionCode;
    wRegionCount= bRegionInfo[REGION_COUNT_OFFSET];
  } else {
    DbgMsg(DBG_INFO,"Starting Driver for the First Time ... \r\n");
    wRegionCount= 0;
    bRegionInfo[TAG_OFFSET]= REGION_CODE_TAG_1;
    bRegionInfo[LENGTH_OFFSET]= REGION_CODE_LENGTH;
  }
  if(wRegionCount<MAX_REGION_COUNT) {
    bRegionInfo[REGION_CODE_OFFSET]= (BYTE)wNewRegionCode;
    bRegionInfo[REGION_COUNT_OFFSET]= (BYTE)wRegionCount+1;
    dvHwEncryptRegionCode(bRegionInfo,wRegionInfoSize);
    EEPROM_Proc(pHw,EEPROM_EW_ENABLE,0,0);
    if(EEPROM_Proc(pHw,EEPROM_WRITE_BYTES,dwOffSet,(DWORD)bRegionInfo)) {
      dwRet = 0xFFFFFFFF; //REGION_ERR_INFO;
    }
    EEPROM_Proc(pHw,EEPROM_EW_DISABLE,0,0);

    wRetRegionCode = bRegionInfo[REGION_CODE_OFFSET];
    wRetRegionCount= bRegionInfo[REGION_COUNT_OFFSET];
  } else {
    dwRet= 0xFFFFFFFF; //REGION_ERR_MAX_WRITE;
  }
  if(!dwRet)
    dwRet=(DWORD)(MAX_REGION_COUNT<<16)|(DWORD)((wRetRegionCount&0xff)<<8)|(DWORD)(wRetRegionCode&0xff);
  return dwRet;
}

static DWORD EEPROM_ReadCaps(PHW pHw,LPDWORD pdwCaps) {
  DWORD dwRet= 0;
  DWORD dwOffSet;
  WORD  wCapsInfoSize;

  wCapsInfoSize=4;
  dwOffSet= (DWORD)(((DWORD)(EEPROM_CAPS_ADDR)<<16)|wCapsInfoSize);
  *pdwCaps=0;
  EEPROM_Proc(pHw,EEPROM_CONFIG,0,(DWORD)(struct EEPROM_Config_s *)&Config_93LC66);
  if(EEPROM_Proc(pHw,EEPROM_READ_BYTES,dwOffSet,(DWORD)pdwCaps)) {
    dwRet = 0xFFFFFFFF; //CAPS_ERR_INFO
  }
  return dwRet;
}

static long EEPROM_ReadData(PHW pHw, LPBYTE lpBuff, DWORD dwSize) {
  long lErr = 0;  
  WORD wAddr=0x1C4;  // 0x1c4 is tag and 0x1c5 is the size, 0x1c6 starts the data.
  WORD wBytes = 0;
  BYTE bTag,bChk,bLen,bRsv;
  BYTE bSum=0;

  bSum+=bTag=ReadOneByte(pHw,wAddr++);  // nav structure tag should be 0x58
  bSum+=bChk=ReadOneByte(pHw,wAddr++);  // check sum
  bSum+=bLen=ReadOneByte(pHw,wAddr++);  // length of nav structure
  bSum+=bRsv=ReadOneByte(pHw,wAddr++);  // reserved field (need to read for checksum)
  if(bTag==EEPROM_TAG1 && bLen<=dwSize) {
    while(bLen--)
      bSum+=*lpBuff++=ReadOneByte(pHw,wAddr++);
  } else {
    lErr = MARGI_ERR_BADTAG; 
  }
  if(bSum!=0)
    lErr = MARGI_ERR_BADTAG; 
  return lErr;
}


static long EEPROM_WriteData(PHW pHw, LPBYTE lpBuff, DWORD dwSize) {
  long lErr = 0;  
  WORD wAddr = 0x1C4;    // 0x1c4 is tag and 0x1c5 is the size, 0x1c6 starts the data.
  WORD wBytes = 0;

  if(dwSize >= 60)
    lErr = MARGI_ERR_BADSIZE;
  else
  {
    EEPROM_Proc(pHw,EEPROM_EW_ENABLE,0,0);
    WriteOneByte(pHw,wAddr, EEPROM_TAG1);  // 0x58 is fixed for TAG1.
    wAddr +=1;

    WriteOneByte(pHw,wAddr, (BYTE)(dwSize&0xff)); //write the size of the block excluding the tag and size.
    wAddr +=1;

    wBytes=(WORD)dwSize;
    while(wBytes--) {
      WriteOneByte(pHw,wAddr,*lpBuff);
      DbgMsg(DBG_INFO," Written 0x%04X to EEPROM @ Address 0x%04X \r\n",*lpBuff,wAddr);
      lpBuff++;
      wAddr++;
    }
    EEPROM_Proc(pHw,EEPROM_EW_DISABLE,0,0);
  }
  return lErr;
}

static BYTE EEPChecksum(BYTE* p,DWORD dwBytes) {
  int n;
  BYTE b=0;
  for(n=0;n<(int)dwBytes;n++)
    b+=p[n];
  return b;
}

static long EEPROM_TimedWriteData(PHW pHw, LPBYTE lpBuff, DWORD dwSize) {
  long lErr = 0;
  if(dwSize >= 60)
    lErr = MARGI_ERR_BADSIZE;
  else
  {
    pHw->fWrEEPROMInProgress = 1;                 // set it first so that the terminate cannot complete 
    pHw->eeprom.dwBytes=0;                        // so current timed transfer will stop
    pHw->eeprom.buffer[0]=EEPROM_TAG1;            // nav structure tag
    pHw->eeprom.buffer[1]=0;                      // checksum will go here       
    pHw->eeprom.buffer[2]=(BYTE)(dwSize&0x0ff);   // size of nav structure
    pHw->eeprom.buffer[3]=0;                      // reserved
    memcpy(pHw->eeprom.buffer+4,lpBuff,dwSize);
    pHw->eeprom.buffer[1]=-EEPChecksum(pHw->eeprom.buffer,dwSize+4);
    pHw->eeprom.dwAddr=0x1c4;
    pHw->eeprom.pData=pHw->eeprom.buffer;
    pHw->eeprom.dwBytes=dwSize+4;
  }
  return lErr;
}

static long EEPROM_Timer(PHW pHw) {
  long lErr = 0;  
  if(pHw->eeprom.dwBytes) {
    if(pHw->eeprom.dwBytes<=64) {
      EEPROM_Proc(pHw,EEPROM_EW_ENABLE,0,0);
      WriteOneByte(pHw,(WORD)pHw->eeprom.dwAddr,*pHw->eeprom.pData);
      pHw->eeprom.dwAddr++;
      pHw->eeprom.pData++;
      pHw->eeprom.dwBytes--;
      if(pHw->eeprom.dwBytes == 0)
        pHw->fWrEEPROMInProgress = 0; 
      EEPROM_Proc(pHw,EEPROM_EW_DISABLE,0,0);
    }
  }
  return lErr;
}


/* EOF : EEPROM.C */