
#ifndef CIS_TEST_SERIAL_
#define CIS_TEST_SERIAL_ 0x1000

#ifndef _MARGITYPESDEFINED
typedef unsigned char       BYTE;   // From Windows.h
typedef unsigned short      WORD;
typedef unsigned long       DWORD;
typedef unsigned int        UINT;
#endif

#define DELAY_MULTIPLIER_DEFAULT        1
#define MAX_ADDRESS_DEFAULT             64

#define WRITE_PORT_DEFAULT 0x0284
#define READ_PORT_DEFAULT  0x0280
#define CS_MASK_DEFAULT    0x0002
#define CLK_MASK_DEFAULT   0x0004
#define DI_MASK_DEFAULT    0x0008
#define DO_MASK_DEFAULT    0x0010
#define NUM_OF_ADDRESS_BITS_DEFAULT 9
#define NUM_OF_DATA_BITS_DEFAULT    8

// Operation Flags
#define EEPROM_ERASE       0x01
#define EEPROM_WRITE_BYTES 0x02
#define EEPROM_READ_BYTES  0x03
#define EEPROM_WRITE_ALL   0x04
#define EEPROM_ERASE_ALL   0x05
#define EEPROM_EW_ENABLE   0x06
#define EEPROM_EW_DISABLE  0x07
#define EEPROM_TEST_ALL    0x08
#define EEPROM_CONFIG      0x09
#define EEPROM_SHOW_CONFIG 0x10
#define EEPROM_READ_REGION 0x11
#define EEPROM_WRITE_REGION 0x12
#define EEPROM_READ_CAPS   0x13
#define EEPROM_READ_DATA   0x14
#define EEPROM_WRITE_DATA  0x15
#define EEPROM_TIMED_WRITE 0x16
#define EEPROM_TIMER       0x17


// Opcodes for various Operations
#define NUM_OF_OPCODE_BITS 2
#define OPCODE_READ     0x02
#define OPCODE_WRITE    0x01
#define OPCODE_ERASE    0x03
#define OPCODE_EWEN     0x00
#define OPCODE_EWDS     0x00
#define OPCODE_ERAL     0x00
#define OPCODE_WRAL     0x00

// Tag defines.
#define EEPROM_TAG1         0x58

// Address Bits need to sent to EEPROM for various operations


#define ADDRESS_EWEN    0x03   //0x30
#define ADDRESS_EWDS    0x00   //0x00
#define ADDRESS_ERAL    0x02   //0x20
#define ADDRESS_WRAL    0x01   //0x10

//Delay (in mSec) for various Cycles
#define TWC_WRITE_CYCLETIME     2
#define TWC_ERAL_CYCLETIME      15
#define TWC_WRAL_CYCLETIME      15
#define TEC_ERASE_CYCLETIME     1

// Error Messages 
#define UNABLE_TO_OPEN_FILE     1

extern DWORD EEPROM_Proc(PHW pHw,WORD Msg,DWORD Parm1,DWORD Parm2);
//extern DWORD dvHwEEPROMServices(PHW pHw,WORD wMsg,LPWORD Data);

struct EEPROM_Config_s {
  WORD wWritePort;
  WORD wReadPort;
  WORD wNumOfAddrBits;
  WORD wNumOfDataBits;
  WORD wDelayMultiplier;
  WORD CS_Mask;
  WORD CLK_Mask;
  WORD DI_Mask;
  WORD DO_Mask;
};

#endif
