/********************************************************************************
        Copyright Margi Systems, Inc. (c) 1994

File:       i2c.c
Purpose:    Provide I2C interface functions
Developed:  jec
Date:       29 Nov 1947 ???

********************************************************************************/
//#include <StrmIni.h>
//#include <mmsystem.h>
//#include <windows.h>
//#include <memory.h>
//#include <malloc.h>
//#include <conio.h>
#include "system.h"
#define  _DECLARE_I2C
#include "i2c.h"
#include "DbgMsg.h"
#include "DbgFltr.h"

//extern LPVOID lpParent;
//extern WORD gfEncodeI2C;
//extern void (FAR *dvHwSetGPIO)(LPVOID lpParent, BYTE bPinNumber, BYTE bEnable, BYTE bValue);
//extern BYTE (FAR *dvHwGetGPIO)(LPVOID lpParent, BYTE bPinNumber, LPBYTE lpEnable, LPBYTE lpValue);

I2C gsI2C;

enum TV_GPIO
{
  GPIO_PIN_RESET = 0,
  GPIO_PIN_CLOCK = 1,
  GPIO_PIN_DATA  = 2
};

void ClrClk(PI2C p)
{
  (*p->pSet)(p->hRef, GPIO_PIN_CLOCK, 1, 0);
}

void SetClk(PI2C p)
{
  (*p->pSet)(p->hRef, GPIO_PIN_CLOCK, 0, 0);
}

void ClrData(PI2C p)
{
  (*p->pSet)(p->hRef, GPIO_PIN_DATA, 1, 0);
}

void SetData(PI2C p)
{
  (*p->pSet)(p->hRef, GPIO_PIN_DATA, 0, 0);
}

int GetData(PI2C p)
{
  return (*p->pGet)(p->hRef, GPIO_PIN_DATA, NULL, NULL);
}

int GetClk(PI2C p)
{
  return (*p->pGet)(p->hRef, GPIO_PIN_CLOCK, NULL, NULL);
}

#if(1)
void usDelay(int nTime)
{
  volatile int n;
  for (n=0;n<nTime*7;n++);
}     
#else
#define usDelay(t)
#endif


PI2C i2cNew(PSETGPIO pSetGpio,PGETGPIO pGetGpio,LPVOID hRef)
{
//  PI2C p = (PI2C)malloc(sizeof(I2C));
  PI2C p = &gsI2C;
  if(p != 0)
  {
    p->pSet=pSetGpio;
    p->pGet=pGetGpio;
    p->hRef=hRef;
  }
  return p;
}

void i2cDelete(PI2C p)
{
//  free(p);
}

int i2cReset(PI2C p)
{
  ClrClk(p);
  ClrData(p);
  return 0;
}

int i2cStart(PI2C p)
{
  SetData(p);
  SetClk(p);
  usDelay(5);
  ClrData(p);
  usDelay(1);
  ClrClk(p);
  usDelay(2);
  return 0;
}

int i2cStop(PI2C p)
{
  ClrClk(p);
  ClrData(p);
  usDelay(4);
  SetClk(p);
  usDelay(4);
  SetData(p);
//  usDelay(2);
//  ClrClk(p);
  return 0;
}

int i2cReadBit(PI2C p)
{                             
  int ack;  
  ClrData(p);
  ClrClk(p); 
  SetData(p);
  SetClk(p);
  usDelay(4);
  ack=GetData(p);

  ClrClk(p);
  ClrData(p);
  return ack;
}

void i2cWriteBit(PI2C p, BYTE b )
{
  b &= 1;
  if(b == 1)
  {
    ClrData(p);
    ClrClk(p);
    SetData(p);
 usDelay(1);
    SetClk(p);
 usDelay(1);
    SetClk(p);
    ClrClk(p);
    ClrData(p);
  }
  else
  {
    ClrData(p);
    ClrClk(p);
    SetClk(p);
 usDelay(1);
    SetClk(p);
    ClrClk(p);
  }
}

int i2cWriteByte(PI2C p, BYTE data)
{
  unsigned int i,ack;

  for( i = 0; i < 8; i++ )
  {
    i2cWriteBit(p, (unsigned char)(data >> (7 - i)) );
  }
  ack = !i2cReadBit(p);
  return ack;
}

int i2cWriteWord(PI2C p, WORD w)
{
  int ack = i2cWriteByte(p, (BYTE)(w>>8));
  ack += i2cWriteByte(p, (BYTE)w);
  return ack==2;
}

int i2cWriteDWord(PI2C p, DWORD dw)
{
  int ack=i2cWriteByte(p, (BYTE)(dw>>16));
  ack+=i2cWriteByte(p, (BYTE)(dw>>8));
  ack+=i2cWriteByte(p, (BYTE)dw);
  return ack==3;
}

int i2cWriteStream(PI2C p, BYTE* bData, int nLength)
{       
  int i,ack=0;
  for (i=0; i<nLength; i++)
    ack += i2cWriteByte(p, bData[i]);

  return ack = nLength;
}                          

BYTE i2cReadByte(PI2C p)
{
  int i;
  unsigned char data = 0;

  for( i = 0; i < 8; i++ )
  {
    data <<= 1;
    data |= i2cReadBit(p);
  }
  return( data );
}                     

int i2cSend(PI2C p, int nAddr, int nReg, int nValue)
{
  i2cStart(p);
  i2cWriteByte(p,(BYTE)(nAddr&0xfe));   // Slave Address
  i2cWriteByte(p,(BYTE)nReg);           // Sub Address  
  i2cWriteByte(p,(BYTE)nValue);         // Data
  i2cStop(p);              
  return 0;
}


int i2cReadBt819Reg(PI2C p, BYTE bAddr, int nReg)
{             
  int data;
  i2cStart(p);
  i2cWriteByte(p,bAddr);
  i2cWriteByte(p,(BYTE)nReg);
  i2cStart(p);
  i2cWriteByte(p,(BYTE)(bAddr|1));
  data=i2cReadByte(p);
  i2cStop(p);
  return data;
}

int i2cWriteBt819Reg(PI2C p, BYTE bAddr, int nReg, int nData)
{             
  i2cStart(p);
  i2cWriteByte(p,bAddr);
  i2cWriteByte(p,(BYTE)nReg);
  i2cWriteByte(p,(BYTE)nData);
  i2cStop(p);
  return nData;
}
          



int i2cWriteCommand(PI2C p, BYTE bAddr, BYTE bCommand, WORD wValue)
{
//  ENTER_CRITICAL; 
  i2cReset(p);
  i2cStart(p);
  i2cWriteByte(p,bAddr);

  i2cWriteByte(p,bCommand);
  i2cWriteByte(p,(BYTE)(wValue>>8));
  i2cWriteByte(p,(BYTE)(wValue));

  i2cWriteByte(p,bCommand);
  i2cWriteByte(p,(BYTE)(wValue>>8));
  i2cWriteByte(p,(BYTE)(wValue));

  i2cStop(p);
//  EXIT_CRITICAL; 
  return 0;
}                       


/* eof */
