/**********************************************************************
 *                                                                    *
 *                      SystemSoft Corporation                        *
 *                      313 Speen Street                              *
 *                      Natick, MA 01760                              *
 *                      (508) 651-0088                                *
 *                                                                    *
 * Copyright 1993-1996 SystemSoft Corporation.  All Rights Reserved.  *
 *                                                                    *
 * Module:  cardserv.h                                                *
 *                                                                    *
 * Author:  MRT                                                       *
 *                                                                    *
 **********************************************************************/


#ifndef _CARDSERV_H

#define _CARDSERV_H

/*************************************************************************/
/** Trying to remove funky typedefs.					**/
/** Chip Doran								**/
/*************************************************************************/
typedef unsigned short LOGICAL_SOCKET;
typedef unsigned char PHYSICAL_ADAPTER;
typedef unsigned char PHYSICAL_SOCKET;
typedef unsigned char PHYSICAL_WINDOW;
typedef unsigned short TIMER_MS;
typedef void (*FAR_FUNCTION_PTR)(void);
typedef unsigned short RET_CODE;

/**********************************************************************
 * card services Function codes.                                      *
 **********************************************************************/

#define F_CLOSE_MEMORY                  0x00
#define F_COPY_MEMORY                   0x01
#define F_DEREGISTER_CLIENT             0x02
#define F_GET_CLIENT_INFO               0x03
#define F_GET_CONFIGURATION_INFO        0x04
#define F_GET_FIRST_PARTITION           0x05
#define F_GET_FIRST_REGION              0x06
#define F_GET_FIRST_TUPLE               0x07
#define F_GET_NEXT_PARTITION            0x08
#define F_GET_NEXT_REGION               0x09
#define F_GET_NEXT_TUPLE                0x0A
#define F_GET_CARD_SERVICES_INFO        0x0B
#define F_CS_GET_STATUS                 0x0C
#define F_GET_TUPLE_DATA                0x0D
#define F_GET_FIRST_CLIENT              0x0E
#define F_REGISTER_ERASE_QUEUE          0x0F
#define F_REGISTER_CLIENT               0x10
#define F_RESET_CARD                    0x11
#define F_MAP_LOG_SOCKET                0x12
#define F_MAP_LOG_WINDOW                0x13
#define F_MAP_MEM_PAGE                  0x14
#define F_MAP_PHY_SOCKET                0x15
#define F_MAP_PHY_WINDOW                0x16
#define F_MODIFY_WINDOW                 0x17
#define F_OPEN_MEMORY                   0x18
#define F_READ_MEMORY                   0x19
#define F_REGISTER_MTD                  0x1A
#define F_RELEASE_IO                    0x1B
#define F_RELEASE_IRQ                   0x1C
#define F_RELEASE_WINDOW                0x1D
#define F_RELEASE_CONFIGURATION         0x1E
#define F_REQUEST_IO                    0x1F
#define F_REQUEST_IRQ                   0x20
#define F_REQUEST_WINDOW                0x21
#define F_REQUEST_SOCKET_MASK           0x22
#define F_RETURN_SS_ENTRY               0x23
#define F_WRITE_MEMORY                  0x24
#define F_DEREGISTER_ERASE_QUEUE        0x25
#define F_CHECK_ERASE_QUEUE             0x26
#define F_MODIFY_CONFIGURATION          0x27
#define F_REGISTER_TIMER                0x28
#define F_SET_REGION                    0x29
#define F_GET_NEXT_CLIENT               0x2A
#define F_VALIDATE_CIS                  0x2B
#define F_REQUEST_EXCLUSIVE             0x2C
#define F_RELEASE_EXCLUSIVE             0x2D
#define F_GET_EVENT_MASK                0x2E
#define F_RELEASE_SOCKET_MASK           0x2F
#define F_REQUEST_CONFIGURATION         0x30
#define F_SET_EVENT_MASK                0x31
#define F_ADD_SOCKET_SERVICES           0x32
#define F_REPLACE_SOCKET_SERVICES       0x33
#define F_CS_VENDOR_SPECIFIC            0x34
#define F_ADJUST_RESOURCE_INFO          0x35
#define F_ACCESS_CONFIGURATION_REGISTER 0x36
#define F_GET_FIRST_WINDOW		0x37
#define F_GET_NEXT_WINDOW		0x38
#define F_GET_MEM_PAGE			0x39
#define F_REQUEST_DMA			0x3A
#define F_RELEASE_DMA			0x3B
#define MAX_CS_FUNCTIONS                0x3C

/**********************************************************************
 * EVent Codes.                                                       *
 * Event codes are defined by the Card Services spec. to be a byte.   *
 * We handle event codes internally as a word (see EVENT_CODE in      *
 * basetype.h), and the upper eight bits are used as flags for special*
 * internal handling of certain event codes (see COMMON.H).           *
 **********************************************************************/

#define EV_PM_RESUME                    0x0B
#define EV_PM_SUSPEND                   0x0C
#define EV_BATTERY_DEAD                 0x01
#define EV_BATTERY_LOW                  0x02
#define EV_CARD_INSERTION               0x40
#define EV_CARD_LOCK                    0x03
#define EV_CARD_READY                   0x04
#define EV_CARD_REMOVAL                 0x05
#define EV_CARD_RESET                   0x11
#define EV_CARD_UNLOCK                  0x06
#define EV_EJECTION_COMPLETE            0x07
#define EV_EJECTION_REQUEST             0x08
#define EV_ERASE_COMPLETE               0x81
#define EV_EXCLUSIVE_COMPLETE           0x0D
#define EV_EXCLUSIVE_REQUEST            0x0E
#define EV_INSERTION_COMPLETE           0x09
#define EV_INSERTION_REQUEST            0x0A
#define EV_REGISTRATION_COMPLETE        0x82
#define EV_REQUEST_ATTENTION            0x18
#define EV_RESET_COMPLETE               0x80
#define EV_RESET_PHYSICAL               0x0F
#define EV_RESET_REQUEST                0x10
#define EV_MTD_REQUEST                  0x12
#define EV_CLIENT_INFO                  0x14
#define EV_TIMER_EXPIRED                0x15
#define EV_SS_UPDATED                   0x16
#define EV_WRITE_PROTECT                0x17

/* VENDOR SPECIFIC EVENTS */
#define EV_SOCKET_CONFIGURATION         0x83
#define EV_DOCKING                      0x84

/*
** Power Management Callback Event defines.
*/

#define PM_BEGIN_RESUME                 0x01
#define PM_END_RESUME                   0x02

    /*
    ** Resume Types
    */

#define PM_RT_NORMAL_RESUME             0x01
#define PM_RT_CRITICAL_RESUME           0x02

    /*
    ** Suspend Types
    */

#define PM_ST_SNAPSHOT_REQUEST          0x00
#define PM_ST_QUERY_REQUEST             0x01
#define PM_ST_CONSERVE_POWER            0x02    /* socket bitmap required */
#define PM_ST_NO_POWER                  0x03

/**********************************************************************
 * Card Services Interface Structures.                                *
 * (Must be packed to one byte to guarantee conformance to the        *
 *  card services specification).                                     *
 **********************************************************************/

#pragma pack(1)

    /******************************************************************
     * Get card services information structures.                      *
     * NOTE:                                                          *
     *   - The get cs info packet is appended with a variable length  *
     *     ASCIIZ vendor string buffer area.                          *
     ******************************************************************/

    typedef struct
	{
	WORD info_len;
	WORD signature;
	WORD socket_cnt;
	WORD revision;
	WORD cs_level;
	WORD vstr_offset;
	WORD vstr_length;
	WORD funcs_per_skt;
	} CS_INFO_PKT;

    /******************************************************************
     * Event Mask Structures.                                         *
     ******************************************************************/

/*
 * define the event masks in EVT_MASK_PKT and CLT_REGISTRATION_PKT
 */
    typedef WORD      EVT_MASK;
#define EVTM_WP_CHG             0x0001	//BIT0
#define EVTM_CRD_LOCK_CHG       0x0002	//BIT1
#define EVTM_EJECT_REQ          0x0004	//BIT2
#define EVTM_INSERT_REQ         0x0008	//BIT3
#define EVTM_BAT_DEAD           0x0010	//BIT4
#define EVTM_BAT_LOW            0x0020	//BIT5
#define EVTM_READY_CHG          0x0040	//BIT6
#define EVTM_CRD_DETECT_CHG     0x0080	//BIT7
#define EVTM_PWR_MGMT_CHG       0x0100	//BIT8
#define EVTM_RESET              0x0200	//BIT9
#define EVTM_SS_UPDATED         0x0400	//BIT10
#define EVTM_EXT_STAT_REG_CHG   0x0800	//BIT11
#define EVTM_RESERVED           0xF000	//(BIT12+BIT13+BIT14+BIT15)

/*
 * define the event attributes in the EVT_MASK_PKT
 */
    typedef WORD      EVT_ATTRIBUTES;
#define EVTA_SOCKET_ONLY        0x0001	//BIT0
#define EVTA_RESERVED           0xFFFE	//(~BIT0)

    typedef struct
	{
	EVT_ATTRIBUTES  attr;
	EVT_MASK        evt_mask;
	LOGICAL_SOCKET  l_socket;
	} EVT_MASK_PKT;

    /******************************************************************
     * Client packet structures.                                      *
     ******************************************************************/

    typedef struct
	{
	BYTE    flags;
	BYTE    info_subfunction;       /* GetClientInfo CALLS ONLY   */
	} CLT_ATTRIBUTES;

#define CLTA_DRIVER_BITS_MASK   0x0007	//(BIT0+BIT1+BIT2)
#define CLTA_MEM_DRIVER         0x0001	//BIT0
#define CLTA_MTD_DRIVER         0x0002	//BIT1
#define CLTA_IO_DRIVER          0x0004	//BIT2
#define CLTA_AI_SHARED          0x0008	//BIT3
#define CLTA_AI_EXCLUSIVE       0x0010	//BIT4
#define CLTA_RESERVED           0x00E0	//(BIT5+BIT6+BIT7)

#define CLT_DATA_SIZE              4

    typedef struct
	{
	CLT_ATTRIBUTES  attr;
	EVT_MASK        evt_mask;
	WORD          data[CLT_DATA_SIZE];
	WORD          version;
	} CLT_REGISTRATION_PKT;

    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	WORD          attr;
	} CLT_GET_PKT;

/*
 * bits in the "attr" field of the CLT_GET_PKT
 */
#define CLTG_SOCKET_ONLY        0x0001	//BIT0
#define CLTG_RESERVED           0xFFFE	//(~BIT0)

    typedef struct
	{
	WORD          max_length;
	WORD          info_length;
	CLT_ATTRIBUTES  attr;           /* see CLT_REGISTRATION_PKT */
	WORD          clt_revision;
	WORD          cs_level;       /* there's more - but we only */
	} CLT_INFO_PKT;                 /* care about this much of it.*/


    /******************************************************************
     * MTD packet structures                                          *
     ******************************************************************/

    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	WORD  attr;       /* see GET_REGION_PKT "attr" field */
	DWORD  crd_offset;
	WORD  media_id;
	} REGISTER_MTD_PKT;

    typedef void far *  DATA_POINTER;
    typedef void far *  CODE_POINTER;

    typedef struct
	{
	WORD          length;
	LOGICAL_SOCKET  l_socket;
	DWORD          src_offset;
	DWORD          dest_offset;
	DWORD          transfer_length;
	BYTE            function;
	BYTE            access_speed;
	WORD          MTD_id;
	WORD          MTD_status;
	WORD          timeout_count;
	DATA_POINTER    MAT_data_ptr;
	CODE_POINTER    CardSetAddress_ptr;
	CODE_POINTER    CardSetAutoInc_ptr;
	CODE_POINTER    CardReadByte_ptr;
	CODE_POINTER    CardReadWord_ptr;
	CODE_POINTER    CardReadWords_ptr;
	CODE_POINTER    CardReadByteAI_ptr;
	CODE_POINTER    CardReadWordAI_ptr;
	CODE_POINTER    CardReadWordsAI_ptr;
	CODE_POINTER    CardWriteByte_ptr;
	CODE_POINTER    CardWriteWord_ptr;
	CODE_POINTER    CardWriteWords_ptr;
	CODE_POINTER    CardWriteByteAI_ptr;
	CODE_POINTER    CardWriteWordAI_ptr;
	CODE_POINTER    CardWriteWordsAI_ptr;
	CODE_POINTER    CardCompareByte_ptr;
	CODE_POINTER    CardCompareByteAI_ptr;
	CODE_POINTER    CardCompareWords_ptr;
	CODE_POINTER    CardCompareWordsAI_ptr;
	CODE_POINTER    MTDHelperEntry_ptr;
	} MTD_REQUEST;

/*
 * definitions for the "function" field of MTD_REQUEST
 */
#define MTD_F_FUNC              (3<<0)
#define MTD_F_ERASE             (0<<0)
#define MTD_F_READ              (1<<0)
#define MTD_F_WRITE             (2<<0)
#define MTD_F_COPY              (3<<0)
#define MTD_F_DISABLE_ERASE     0x0004	//BIT2
#define MTD_F_VERIFY            0x0008	//BIT3
#define MTD_F_READY_CONT        0x0010	//BIT4
#define MTD_F_TIMEOUT_CONT      0x0020	//BIT5
#define MTD_F_FIRST             0x0040	//BIT6
#define MTD_F_LAST              0x0080	//BIT7

/*
 * definitions for the "status" field of MTD_REQUEST
 */
#define MTD_S_WAITREQ           0
#define MTD_S_WAITTIMER         1
#define MTD_S_WAITRDY           2
#define MTD_S_WAITPOWER         3

    /******************************************************************
     * Socket packet structures.                                      *
     ******************************************************************/

    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	EVT_MASK        evt_mask;
	} SKT_REQUEST_PKT;

    typedef struct
	{
	LOGICAL_SOCKET l_socket;
	} SKT_RELEASE_PKT;

    typedef struct
	{
	LOGICAL_SOCKET   l_socket;
	PHYSICAL_ADAPTER p_adapter;
	PHYSICAL_SOCKET  p_socket;
	} SKT_MAP_PKT;

    /******************************************************************
     * Window packet structures.                                      *
     ******************************************************************/

#define WDW_ATTRIBUTE_MEM_TYPE      TRUE
#define WDW_COMMON_MEM_TYPE         FALSE

#define WDW_ISA_IO_TYPE             FALSE
#define WDW_EISA_IO_TYPE            TRUE

#define WDW_8_BIT_DATA_WIDTH        FALSE
#define WDW_16_BIT_DATA_WIDTH       TRUE

    typedef WORD      WDW_ATTRIBUTES;
#define WDWA_MOD_RESERVED       0xFFF1	//(~(BIT1|BIT2|BIT3))
#define WDWA_IO_ADDR_SPACE      0x0001	//BIT0    /* Window type: 1 = IO, 0 = MEM */
#define WDWA_ATTRIB             0x0002	//BIT1    /* for memory windows only */
#define WDWA_ENABLED            0x0004	//BIT2
#define WDWA_16BIT              0x0008	//BIT3    /* for RequestWindow only */
#define WDWA_SPEED_VALID        0x0008	//BIT3    /* for ModifyWindow only */
#define WDWA_PAGED              0x0010	//BIT4
#define WDWA_SHARED             0x0020	//BIT5
#define WDWA_FIRST_SHARED       0x0040	//BIT6
#define WDWA_BINDING_SPECIFIC   0x0080	//BIT7
#define WDWA_OFFSETS_SIZED      0x0100	//BIT8
#define WDWA_32BIT              0x0200	//BIT9    /* If reset, see bit3 for 8/16 bit */
#define WDWA_RESERVED           0x0400	//BIT10
#define WDWA_PREFETCHABLE       0x0800	//BIT11   /* Prefetchable, not cacheable */
#define WDWA_CACHEABLE          0x1000	//BIT12   /* Both Prefetchable and Cacheable */
                                        /* Note: If cacheable, it must be  */
                                        /*   prefetchable. Prefetchable is */
                                        /*   not necessarily cacheable.    */
#define WDWA_PREF_CACHE_MASK    (WDWA_PREFETCHABLE | WDWA_CACHEABLE)
#define WDWA_BASE_ADDR_REG      0xE000	//(BIT13 | BIT14 | BIT15)

    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	WDW_ATTRIBUTES  attr;
	DWORD          base;
	DWORD          size;
	BYTE            access_speed;
	} WDW_REQUEST_PKT;


    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	WDW_ATTRIBUTES  attr;
	DWORD          base;
	DWORD          size;
	union
           { 
           BYTE access_speed;
           BYTE io_addr_lines;   
           } type;
	} WDW_REQUEST_PKT_50;

    typedef struct
	{
	WDW_ATTRIBUTES  attr;
	BYTE            access_speed;
	} WDW_MODIFY_PKT;

    typedef struct
	{
	PHYSICAL_ADAPTER p_adapter;
	PHYSICAL_WINDOW  p_window;
	} WDW_MAP_PKT;

    /******************************************************************
     * I/O packet structures.                                         *
     ******************************************************************/

    typedef struct
	{
	WORD          base_port;
	BYTE            num_ports;
	BYTE            attr;
	} IO_PARMS;

#define IOA_SHARED              0x0001	//BIT0
#define IOA_FIRST_SHARED        0x0002	//BIT1
#define IOA_FORCE_ALIAS         0x0004	//BIT2
#define IOA_16BIT               0x0008	//BIT3
#define IOA_RESERVED            0x00F0	//(BIT4|BIT5|BIT6|BIT7)

#define MAX_IO_WDWS_PER_SKT     2

    typedef struct
	{
	LOGICAL_SOCKET l_socket;
	IO_PARMS       io_parms[MAX_IO_WDWS_PER_SKT];
	BYTE           io_addr_lines;
	} IO_REQUEST_PKT;

    typedef IO_REQUEST_PKT IO_RELEASE_PKT;

    /******************************************************************
     * IRQ packet structures                                          *
     ******************************************************************/

    typedef WORD      IRQ_ATTRIBUTES;
#define IRQA_IRQ_TYPE_MASK      0x03
#define IRQA_EXCLUSIVE          0
#define IRQA_TIME_MUX           1
#define IRQA_DYNAMIC            2
#define IRQA_FORCE_PULSE        0x0004	//BIT2
#define IRQA_FIRST_SHARED       0x0008	//BIT3
#define IRQA_ISRADDR_VALID      0x0010	//BIT4
#define IRQA_PULSE_ALLOC        0x0100	//BIT8
#define IRQA_RESERVED           0x0EE0	//(~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT8))

    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	IRQ_ATTRIBUTES  attr;
	BYTE            assigned_irq;
	BYTE            info1;
	WORD          info2;
	} IRQ_REQUEST_PKT;

/* bits in info1 */
#define IRQI_NUMBER_MASK        0x0F
#define IRQI_NMI                0x0001	//BIT0
#define IRQI_IOCK               0x0002	//BIT1
#define IRQI_BERR               0x0004	//BIT2
#define IRQI_VEND               0x0008	//BIT3
#define IRQI_MASK_FLAG          0x0010	//BIT4
#define IRQI_LEVEL              0x0020	//BIT5
#define IRQI_PULSE              0x0040	//BIT6
#define IRQI_SHARE              0x0080	//BIT7

    typedef struct
	{
	LOGICAL_SOCKET l_socket;
	IRQ_ATTRIBUTES attr;
	BYTE           assigned_irq;
	} IRQ_RELEASE_PKT;

/*
 * DMA resource structures and definitions.
 */

    typedef BYTE                DMA_ATTRIBUTES;

#define DMAA_MODE_MASK          0x03
#define DMAA_REQSIG_MASK        0x0C
#define DMAA_DMA_WIDTH          0x0010	//BIT4
#define DMAA_RESERVED           0xFF1F	//(~(BIT5|BIT6|BIT7))

    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	WORD          desired_channel;
	DMA_ATTRIBUTES  attr;
	BYTE            assigned_channel;
	} DMA_REQUEST_PKT, DMA_RELEASE_PKT;


    /******************************************************************
     * Configuration packet structures.                               *
     * NOTE: if the present flags change, they must also be changed in*
     * the socket database definition.                                *
     ******************************************************************/

#define CNFG_ATTR_EXCLUSIVELY_USED      0x0001 /* get config info only */
#define CNFG_ATTR_ENABLE_IRQ            0x0002
#define CNFG_ATTR_ENABLE_DMA		    0x0040 
#define CNFG_ATTR_VALID_CLIENT          0x0100 /* get config info only */
#define CNFG_ATTR_VS_OVERRIDE           0x0200
#define CNFG_ATTR_RESERVED_3_15         0xFFF8
#define CNFG_ATTR_RESERVED_BITS         CNFG_ATTR_RESERVED_3_15
#define CNFG_ATTR_REQ_RESERVED_BITS     (CNFG_ATTR_EXCLUSIVELY_USED |  \
					 CNFG_ATTR_VALID_CLIENT |      \
					 CNFG_ATTR_RESERVED_BITS)

/* Following TWO are for Ver 5.00 and up only - KGS 8/28/95 */
#define CNFG_ATTR_REQ_MASK              (CNFG_ATTR_ENABLE_IRQ | \
                     CNFG_ATTR_ENABLE_DMA | CNFG_ATTR_VS_OVERRIDE)
#define CNFG_ATTR_REQ_RESERVED_BITS_50  (~CNFG_ATTR_REQ_MASK)


#define CNFG_REQ_OPTION_FLAG            0x0001
#define CNFG_REQ_STATUS_FLAG            0x02
#define CNFG_REQ_PIN_FLAG               0x04
#define CNFG_REQ_COPY_FLAG              0x08
#define CNFG_REQ_EXTSTAT_FLAG           0x10
#define CNFG_REQ_PRESENT_FLAGS          0x1F

#define CNFG_INTTYPE_MEMORY_ONLY        0x01
#define CNFG_INTTYPE_MEMORY_AND_IO      0x02
#define CNFG_INTTYPE_CUSTOM             0x08   
#define CNFG_INTTYPE_RESERVED_4_7       0xF0
#define CNFG_INTTYPE_MEM_IO_BITS        (CNFG_INTTYPE_MEMORY_ONLY |    \
					 CNFG_INTTYPE_MEMORY_AND_IO)
#define CNFG_INTTYPE_RESERVED_BITS              CNFG_INTTYPE_RESERVED_4_7

#define CNFG_DMA_ATTR_MODE_MASK         0x03
#define CNFG_DMA_ATTR_EXCLUSIVE         0x00
#define CNFG_DMA_ATTR_TIMESHARED        0x01
#define CNFG_DMA_ATTR_DYNMCSHARED       0x02
#define CNFG_DMA_ATTR_REQSIG_MASK       0x0C
#define CNFG_DMA_ATTR_DREQ_SPKR         0x04
#define CNFG_DMA_ATTR_DREQ_IOIS16       0x08
#define CNFG_DMA_ATTR_DREQ_INPACK       0x0C
#define CNFG_DMA_ATTR_DMA_WIDTH16       0x10
#define CNFG_DMA_ATTR_RSVD_BITS         0xE0

    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	WORD          attr;
	BYTE            vcc;
	BYTE            vpp1;
	BYTE            vpp2;
	BYTE            int_type;
	DWORD          cnfg_base;
	BYTE            status;
	BYTE            pin;
	BYTE            copy;
	BYTE            option;
	BYTE            present_flags;
	} CNFG_REQUEST_PKT;

    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	WORD          attr;
	BYTE            vcc;
	BYTE            vpp1;
	BYTE            vpp2;
	BYTE            int_type;
	DWORD          cnfg_base;
	BYTE            status;
	BYTE            pin;
	BYTE            copy;
	BYTE            option;
	BYTE            present_flags;
	BYTE            extended_status;
	} CNFG_REQUEST_PKT50;

    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	WORD          attr;
	BYTE            vcc;
	BYTE            vpp1;
	BYTE            vpp2;
	BYTE            int_type;
	DWORD          cnfg_base;
	BYTE            status;
	BYTE            pin;
	BYTE            copy;
	BYTE            option;
	BYTE            present_flags;
	BYTE            extended_status;
	DWORD          CustomIF_id_num;
	} CNFG_REQUEST_PKT501;

    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	} CNFG_RELEASE_PKT;

    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	WORD          attr;
	BYTE            vcc;
	BYTE            vpp1;
	BYTE            vpp2;
	BYTE            int_type;
	DWORD          cnfg_base;
	BYTE            status;
	BYTE            pin;
	BYTE            copy;
	BYTE            option;
	BYTE            present_flags;
	BYTE            first_dev_type;
	BYTE            func_code;
	BYTE            sys_init_mask;
	WORD          manuf_code;
	WORD          manuf_info;
	BYTE            valid_flags;
	BYTE            assigned_irq;
	IRQ_ATTRIBUTES  irq_attr;
	IO_PARMS        io_parms[MAX_IO_WDWS_PER_SKT];
	BYTE            io_addr_lines;
	} CNFG_INFO_PKT;

    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	WORD          attr;
	BYTE            vcc;
	BYTE            vpp1;
	BYTE            vpp2;
	BYTE            int_type;
	DWORD          cnfg_base;
	BYTE            status;
	BYTE            pin;
	BYTE            copy;
	BYTE            option;
	BYTE            present_flags;
	BYTE            first_dev_type;
	BYTE            func_code;
	BYTE            sys_init_mask;
	WORD          manuf_code;
	WORD          manuf_info;
	BYTE            valid_flags;
	BYTE            assigned_irq;
	IRQ_ATTRIBUTES  irq_attr;
	IO_PARMS        io_parms[MAX_IO_WDWS_PER_SKT];
	BYTE            io_addr_lines;
	BYTE            extended_status;
	BYTE            dma_attrib;
	BYTE            dma_channel;
	BYTE            num_io_windows;
	BYTE            num_mem_windows;
	} CNFG_INFO_PKT50;

    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	WORD          attr;
	BYTE            vcc;
	BYTE            vpp1;
	BYTE            vpp2;
	BYTE            int_type;
	DWORD          cnfg_base;
	BYTE            status;
	BYTE            pin;
	BYTE            copy;
	BYTE            option;
	BYTE            present_flags;
	BYTE            first_dev_type;
	BYTE            func_code;
	BYTE            sys_init_mask;
	WORD          manuf_code;
	WORD          manuf_info;
	BYTE            valid_flags;
	BYTE            assigned_irq;
	IRQ_ATTRIBUTES  irq_attr;
	IO_PARMS        io_parms[MAX_IO_WDWS_PER_SKT];
	BYTE            io_addr_lines;
	BYTE            extended_status;
	BYTE            dma_attrib;
	BYTE            dma_channel;
	BYTE            num_io_windows;
	BYTE            num_mem_windows;
	DWORD          CustomIF_id_num;
	} CNFG_INFO_PKT501;

#define CNFG_MOD_ATTR_RESERVED_0        0x0001
#define CNFG_MOD_ATTR_ENABLE_IRQ        0x0002
#define CNFG_MOD_ATTR_IRQ_CHG           0x0004
#define CNFG_MOD_ATTR_RESERVED_3        0x0008
#define CNFG_MOD_ATTR_VCC_CHG           0x0008
#define CNFG_MOD_ATTR_VPP1_CHG          0x0010
#define CNFG_MOD_ATTR_VPP2_CHG          0x0020
#define CNFG_MOD_ATTR_ENABLE_DMA        0x0040
#define CNFG_MOD_ATTR_DMA_CHG           0x0080
#define CNFG_MOD_ATTR_RESERVED_8        0x0100
#define CNFG_MOD_ATTR_RESERVED_8_15     0xFF00
#define CNFG_MOD_ATTR_RESERVED_10_15    0xFC00
#define CNFG_MOD_ATTR_CHANGE_BITS       (CNFG_MOD_ATTR_IRQ_CHG |       \
					 CNFG_MOD_ATTR_VCC_CHG |       \
					 CNFG_MOD_ATTR_VPP1_CHG |      \
					 CNFG_MOD_ATTR_VPP2_CHG |      \
					 CNFG_MOD_ATTR_DMA_CHG)
#define CNFG_MOD_ATTR_RESERVED_BITS     (CNFG_MOD_ATTR_RESERVED_0 |    \
					 CNFG_MOD_ATTR_RESERVED_8_15)
#define CNFG_MOD_ATTR_RESERVED_BITS_50  (CNFG_MOD_ATTR_RESERVED_0 |    \
                    CNFG_MOD_ATTR_RESERVED_3 |  \
                    CNFG_MOD_ATTR_RESERVED_8 |  \
                    CNFG_MOD_ATTR_RESERVED_10_15)

    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	WORD          attr;
	BYTE            vcc;
	BYTE            vpp1;
	BYTE            vpp2;
	} CNFG_MODIFY_PKT;

#define CNFG_READ_REGISTER      0
#define CNFG_WRITE_REGISTER     1

    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	BYTE            action;
	BYTE            offset;
	BYTE            value;
	} CNFG_ACCESS_PKT;

    /******************************************************************
     * Socket Services Entry Point structures.                        *
     ******************************************************************/

#define SS_ATTR_REAL_MODE           0
#define SS_ATTR_1616_PROTECT_MODE   1
#define SS_ATTR_1632_PROTECT_MODE   2
#define SS_ATTR_0_32_PROTECT_MODE   3

    typedef struct
	{
	WORD ss_attr;
	} SS_ENTRY_PNT_PKT;

    typedef struct
	{
	LOGICAL_SOCKET l_socket;
	WORD         num_sockets;
	WORD         ss_attr;
	BYTE far      *data_pointer;
	} SS_REPLACE_PKT;

    typedef struct
	{
	WORD         ss_attr;
	BYTE far      *data_pointer;
	} SS_ADD_PKT;

    /******************************************************************
     * Status request structures.                                     *
     ******************************************************************/

    typedef WORD      CRD_STATE;
    typedef WORD      SKT_STATE;
#define CS_SBM_WP       0x0001	//BIT0
#define CS_SBM_LOCKED   0x0002	//BIT1
#define CS_SBM_EJECT    0x0004	//BIT2
#define CS_SBM_INSERT   0x0008	//BIT3
#define CS_SBM_DEAD     0x0010	//BIT4
#define CS_SBM_WARN     0x0020	//BIT5
#define CS_SBM_READY    0x0040	//BIT6
#define CS_SBM_CD       0x0080	//BIT7
#define CS_SBM_REQ_ATN  0x0800	//BIT11
#define CS_SBM_FNC_CD   0x8000	//BIT15           /* --- INTERNAL CS USE ONLY --- */

    typedef struct
	{
	LOGICAL_SOCKET l_socket;
	CRD_STATE      crd_state;
	SKT_STATE      skt_state;
	} STATUS_PKT;

    /******************************************************************
     * Tuple request structures.                                      *
     * NOTE:                                                          *
     *   - The tuple flags structure is only used internally by card  *
     *     services and is defined as required by our implementation. *
     *   - The tuple data packet structure is appended with a variable*
     *     length tuple data area.                                    *
     ******************************************************************/

    typedef WORD      TUPLE_ATTRIBUTES;
/*
 * bits of TUPLE_ATTRIBUTES
 */
#define TA_RET_LINKS    0x0001	//BIT0
#define TA_RESERVED     0xFFFE	//(~BIT0)

    typedef WORD      TUPLE_FLAGS;
/*
 * bits of TUPLE_FLAGS
 */
#define TF_COMMON               0x0001	//BIT0
#define TF_LINK_TO_A            0x0002	//BIT1
#define TF_LINK_TO_C            0x0004	//BIT2
#define TF_NO_LINK              0x0008	//BIT3
#define TF_IMPLIED_LINK         0x0010	//BIT4
#define TF_INDIRECT_LINK        0x0020	//BIT5 /* indirect memory access */
#define TF_LINK_FLAGS           0x001D	//(BIT1|BIT2|BIT3|BIT4)
#define TF_EXPLICIT_LINK_FLAGS  0x000E	//(BIT1|BIT2|BIT3)


    typedef struct
	{
	LOGICAL_SOCKET   l_socket;
	TUPLE_ATTRIBUTES attr;
	BYTE             desired_tuple_code;
	BYTE             reserved;
	TUPLE_FLAGS      cs_tuple_flags;
	DWORD           cs_link_offset;
	DWORD           cs_cis_offset;
	BYTE             tuple_code;
	BYTE             tuple_link;
	} TUPLE_PKT;

    typedef struct
	{
	LOGICAL_SOCKET   l_socket;
	TUPLE_ATTRIBUTES attr;
	BYTE             desired_tuple_code;
	BYTE             tuple_offset;
	TUPLE_FLAGS      cs_tuple_flags;
	DWORD           cs_link_offset;
	DWORD           cs_cis_offset;
	WORD           tuple_data_max;
	WORD           tuple_data_len;
	} TUPLE_DATA_PKT;


    /******************************************************************
     * Resource packet structures.                                    *
     ******************************************************************/

    typedef struct
	{
	BYTE action;                    /* add or remove indicator    */
	BYTE rsrc_type;                 /* type of resource           */
	} RSRC_HDR_PKT;

#define RSRC_MEM_ATTR_RSVD_0_4      0x001F
#define RSRC_MEM_ATTR_SHARED        0x0020
#define RSRC_MEM_ATTR_RESERVE_RSRC  0x0040
#define RSRC_MEM_ATTR_ALLOCATED     0x0080
#define RSRC_MEM_ATTR_RSVD_8_15     0xFF00
#define RSRC_MEM_ATTR_RESERVED_BITS (RSRC_MEM_ATTR_RSVD_8_15 |         \
				     RSRC_MEM_ATTR_RSVD_0_4)

    typedef struct
	{
	WORD  attr;
	DWORD  base;
	DWORD  size;
	} RSRC_MEM_PKT;

#define RSRC_IO_ATTR_SHARED         0x01
#define RSRC_IO_ATTR_RSVD_1_5       0x3E
#define RSRC_IO_ATTR_RESERVE_RSRC   0x40
#define RSRC_IO_ATTR_ALLOCATED      0x80
#define RSRC_IO_ATTR_RESERVED_BITS  RSRC_IO_ATTR_RSVD_1_5

    typedef struct
	{
	WORD  base_port;
	BYTE    port_cnt;
	BYTE    attr;
	BYTE    io_addr_lines;
	} RSRC_IO_PKT;

    typedef struct
	{
	DWORD  base_port;
	DWORD  port_cnt;
	BYTE    attr;
	BYTE    io_addr_lines;
	} RSRC_IO_PKT50;

#define SHARE_MODE_EXCLUSIVE            0
#define SHARE_MODE_MULTIPLEXED          1
#define SHARE_MODE_DYNAMIC              2

#define RSRC_IRQ_ATTR_TYPE              0x03
#define RSRC_IRQ_ATTR_RSVD_2_5          0x3C
#define RSRC_IRQ_ATTR_RESERVE_RSRC      0x40
#define RSRC_IRQ_ATTR_ALLOCATED         0x80
#define RSRC_IRQ_ATTR_RESERVED_BITS     RSRC_IRQ_ATTR_RSVD_2_5

    typedef struct
	{
	BYTE    attr;
	BYTE    irq_number;
	} RSRC_IRQ_PKT;

#define RSRC_DMA_ATTR_MODE          0x03
#define RSRC_DMA_ATTR_WIDTH8        0x08
#define RSRC_DMA_ATTR_WIDTH16       0x10
#define RSRC_DMA_ATTR_SPECIFIC      0x40
#define RSRC_DMA_ATTR_ALLOCATED     0x80
#define RSRC_DMA_ATTR_RESERVED_BITS 0x24

    typedef struct
	{
	BYTE    attr;
	BYTE    dma_channel;
	} RSRC_DMA_PKT;


    typedef struct
	{
	RSRC_HDR_PKT     hdr;
	union
	    {
	    RSRC_MEM_PKT mem;
	    RSRC_IO_PKT  io;
	    RSRC_IO_PKT50 io50;
	    RSRC_IRQ_PKT irq;
	    RSRC_DMA_PKT dma;
	    }            type;
	} RSRC_PKT;

#define RSRC_MEM_PKT_SIZE    (sizeof(RSRC_HDR_PKT)+sizeof(RSRC_MEM_PKT))
#define RSRC_IO_PKT_SIZE     (sizeof(RSRC_HDR_PKT)+sizeof(RSRC_IO_PKT))
#define RSRC_IO_PKT50_SIZE   (sizeof(RSRC_HDR_PKT)+sizeof(RSRC_IO_PKT50))
#define RSRC_IRQ_PKT_SIZE    (sizeof(RSRC_HDR_PKT)+sizeof(RSRC_IRQ_PKT))
#define RSRC_DMA_PKT_SIZE    (sizeof(RSRC_HDR_PKT)+sizeof(RSRC_DMA_PKT))

#define RSRC_TYPE_MEM           0
#define RSRC_TYPE_IO            1
#define RSRC_TYPE_IRQ           2
#define RSRC_TYPE_DMA           3
#define RSRC_TYPE_HEAP          0xFF

#define RSRC_ACTION_REMOVE      0
#define RSRC_ACTION_ADD         1
#define RSRC_ACTION_GET_FIRST   2
#define RSRC_ACTION_GET_NEXT    3

    /******************************************************************
     * Paging packet structures.                                      *
     ******************************************************************/

    typedef struct
	{
	DWORD crd_offset;
	BYTE   page;
	} PAGE_MAPMEM_PKT;

    /******************************************************************
     * CIS packet structure.                                          *
     ******************************************************************/

    typedef struct
	{
	LOGICAL_SOCKET l_socket;
	WORD         chain_cnt;       /* OUT: validated count       */
	} CIS_VALIDATE_PKT;

    /******************************************************************
     * Exclusive usage packet structure.                              *
     ******************************************************************/

    typedef struct
	{
	LOGICAL_SOCKET l_socket;
	WORD         attr;
	} EXCLUSIVE_PKT;

    /******************************************************************
     * Reset Card packet structure.                                   *
     ******************************************************************/

    typedef struct
	{
	LOGICAL_SOCKET l_socket;
	WORD         attr;
	} RESET_CARD_PKT;

    /******************************************************************
     * Timer packet structure.                                        *
     ******************************************************************/

    typedef struct
	{
	TIMER_MS wait_ms;
	} TIMER_PKT;

    /*******************************************************
     * Region and Partition structures
     *******************************************************/

    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	WORD  attr;
	BYTE    tuple_mask;
	BYTE    access_speed;
	WORD  flags;          /* reserved for internal use */
	DWORD  link_offset;    /* reserved for internal use */
	DWORD  cis_offset;     /* reserved for internal use */
	DWORD  crd_offset;
	DWORD  region_size;
	DWORD  eff_block_size;
	WORD  part_multiple;
	WORD  jedec_id;
	} GET_REGION_PKT;

    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	WORD  attr;
	DWORD  crd_offset;
	DWORD  region_size;
	DWORD  eff_block_size;
	WORD  part_multiple;
	WORD  jedec_id;
	DWORD  bias_offset;
	BYTE    access_speed;
	} SET_REGION_PKT;

    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	WORD  attr;
	BYTE    tuple_mask;
	BYTE    access_speed;
	WORD  flags;          /* reserved for internal use */
	DWORD  link_offset;    /* reserved for internal use */
	DWORD  cis_offset;     /* reserved for internal use */
	DWORD  crd_offset;
	DWORD  region_size;
	DWORD  eff_block_size;
	WORD  part_multiple;
	WORD  jedec_id;
	WORD  partition_type;
	} GET_PARTITION_PKT;

/*
 * bits in the region and partition "attr" fields
 */
#define REG_ATTR_MEM_IS_ATTR            0x0001	//BIT0
#define REG_ATTR_DELETE_REGION          0x0002	//BIT1    /* for SetRegion only */
#define REG_ATTR_VIRTUAL_REGION         0x0100	//BIT8
#define REG_ATTR_VIRTUAL_PARTITION      0x0100	//BIT8
#define MTD_ATTR_WRITE_ERASE_FIELD      (3<<9)
#define MTD_ATTR_WRITE_NO_ERASE         (0<<9)
#define MTD_ATTR_WRITE_WITH_ERASE       (1<<9)
#define MTD_ATTR_DISABLEABLE_ERASE      (3<<9)
#define MTD_ATTR_WRITE_WITH_VERIFY      0x0800	//BIT11
#define MTD_ATTR_ERASE_SUPPORTED        0x1000	//BIT12

/*
 * bits in the region and partition "tuple_mask" fields
 */
#define TM_ACCESS_SPEED_FROM_TUPLES     0x0001	//BIT0
#define TM_CARD_OFFSET_FROM_TUPLES      0x0002	//BIT1
#define TM_REGION_SIZE_FROM_TUPLES      0x0004	//BIT2
#define TM_PARTITION_SIZE_FROM_TUPLES   0x0004	//BIT2        /* (sic) */
#define TM_EFF_BLOCK_SIZE_FROM_TUPLES   0x0008	//BIT3
#define TM_PART_MULTIPLE_FROM_TUPLES    0x0010	//BIT4
#define TM_JEDEC_ID_FROM_TUPLES         0x0020	//BIT5
#define TM_PART_TYPE_FROM_TUPLES        0x0040	//BIT6

/*******************************************************
 * Bulk memory packet structures
 *******************************************************/

    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	WORD  attr;
	DWORD  crd_offset;
	} OPEN_MEMORY_PKT;

/*
 * definitions for OpenMemory "attr" bits
 */
#define MEM_OPEN_ATTR_MEM       0x0001	//BIT0
#define MEM_OPEN_ATTR_EXCLUSIVE 0x0002	//BIT1

    typedef struct
	{
	DWORD  src_offset;
	DWORD  count;
	} READ_MEMORY_PKT;

    typedef struct
	{
	DWORD  dest_offset;
	DWORD  count;
	WORD  attr;
	} WRITE_MEMORY_PKT;

    typedef struct
	{
	DWORD  src_offset;
	DWORD  dest_offset;
	DWORD  count;
	WORD  attr;
	} COPY_MEMORY_PKT;

/*
 * definitions for WriteMemory and CopyMemory "attr" bits
 */
#define MEM_DISABLE_ERASE       0x0004	//BIT2
#define MEM_VERIFY              0x0008	//BIT3

/*******************************************************
 * Erase packet structures
 *******************************************************/

/* MTD Erase Entry States */

#define MTD_ES_IDLE                        0xFF
#define MTD_ES_QUEUED_FOR_ERASE            0x00
#define MTD_ES_ERASE_IN_PROGRESS_LOW       0x01
#define MTD_ES_ERASE_IN_PROGRESS_HIGH      0x7F
#define MTD_ES_ERASE_PASSED                0xE0
#define MTD_ES_ERASE_FAILED                0xE1
#define MTD_ES_MEDIA_WRITE_PROTECTED       0x84
#define MTD_ES_MEDIA_NOT_ERASABLE          0x86
#define MTD_ES_MEDIA_MISSING               0x80
#define MTD_ES_MEDIA_NOT_WRITABLE          0x87
#define MTD_ES_BAD_SOCKET                  0xC3
#define MTD_ES_BAD_TECHNOLOGY              0xC2
#define MTD_ES_BAD_OFFSET                  0xC1
#define MTD_ES_BAD_VCC                     0xC4
#define MTD_ES_BAD_VPP                     0xC5
#define MTD_ES_BAD_SIZE                    0xC6

    typedef struct
	{
	WORD  mem_handle;
	BYTE    state;
	BYTE    size;
	DWORD  offset;
	BYTE    optional[1];
	} ERASE_PKT;

    typedef struct
	{
	WORD      entry_len;
	WORD      entry_cnt;
	ERASE_PKT   entry_array[1];
	} ERASE_HDR_PKT;

    /******************************************************************
     * Vendor specific packet structure.                              *
     * When adding a new function:                                    *
     * 1) If your new function falls under the catagory of one of the *
     *    subcomponent IDs below, then use it and add an appropriate  *
     *    function ID which currently does not exist.                 *
     *    Else add a new subcomponent ID (the current MAX and make it *
     *    general in nature like a catagory), bump the MAX by one,    *
     *    define an appropriate function ID,  create a new function   *
     *    in vendor.c, and add that function to the end of vs_dispatch*
     *    in data.h.                                                  *
     * 2) Define the necessary subpacket and add it to the union in   *
     *    VENDOR_SPECIFIC_PKT.  Subpackets for the special CS API     *
     *    subcomponent MUST contain the length of the standard packet *
     *    for the function in the first two bytes follow by the       *
     *    standard packet itself (see VS_REGISTER_CLIENT).            *
     * 3) define any length constants as needed.                      *
     * 4) Provide documentation on the API to our tech writers.       *
     ******************************************************************/


/*
** SUBCOMPONENT ID DEFINITIONS
*/

#define VS_ID_SPECIAL_CS_API    0
#define VS_ID_PWR_MGMT          1       /* USE LENGTHS DEFINED BELOW    */
#define VS_ID_RING_IND          2       /* USE LENGTHS DEFINED BELOW    */
#define VS_ID_CSC_MASK          3       /* USE LENGTHS DEFINED BELOW    */
#define VS_ID_CARDBOOT          5       /* USE LENGTHS DEFINED BELOW    */
#define VS_ID_BEEP_CTRL         6       /* USE LENGTHS DEFINED BELOW    */
#define VS_ID_GET_INFO          7       /* USE LENGTHS DEFINED BELOW    */
#define VS_ID_WIN95             8       /* USE LENGTHS DEFINED BELOW    */
#define VS_ID_MAX               9


/*
** FUNCTION ID DEFINITIONS
*/

    /*
    ** VS_ID_SPECIAL_CS_API
    */

    /* USE STANDARD CS FUNCTION CODES */


    /*
    ** VS_ID_PWR_MGMT
    */

#define VS_F_PWR_ALL_OFF_NOW    1
#define VS_F_PWR_ALL_ON_NOW     2
#define VS_F_PWR_SKT_OFF_NOW    3
#define VS_F_PWR_SKT_ON_NOW     4
#define VS_F_PWR_OEM_RESERVED1  5
#define VS_F_PWR_OEM_RESERVED2  6
#define VS_F_PWR_SUSPEND        7       /* 5.0 Spec. Suspend Processing */
#define VS_F_PWR_RESUME         8       /* 5.0 Spec. Resume Processing  */
#define VS_F_PWR_ALL_OFF_NOWAIT 9       /* ALL_OFF_NOW with no wait     */
#define VS_F_PWR_ALL_ON_NOWAIT  10      /* ALL_ON_NOW  with no wait     */
#define VS_F_PWR_SKT_OFF_NOWAIT 11      /* SKT_OFF_NOW with no wait     */
#define VS_F_PWR_SKT_ON_NOWAIT  12      /* SKT_ON_NOW  with no wait     */
#define VS_F_PWR_FNC_OFF_NOWAIT 13
#define VS_F_PWR_FNC_ON_NOWAIT  14
#define VS_F_PWR_FNC_OFF_NOW    15
#define VS_F_PWR_FNC_ON_NOW     16

    /*
    ** VS_ID_RING_IND
    */

#define VS_F_RING_IND_OFF       1
#define VS_F_RING_IND_ON        2

    /*
    ** VS_ID_CSC_MASK
    */

#define VS_F_CSC_MASK_ALL_ON    0

    /*
    ** VS_ID_CARDBOOT
    */

#define VS_F_CARDBOOT_GET_INFO  0

    /*
    ** VS_ID_BEEP_CTRL
    */

#define VS_F_BEEP_GET           0
#define VS_F_BEEP_SET           1

    /*
    ** VS_ID_GET_INFO
    */

#define VS_F_GET_WINDOW_INFO    0
#define VS_F_GET_ERROR_INFO     1
#define VS_F_CL_CONFIG_INFO     0xFF

    /*
    ** VS_ID_WIN95
    */
#define VS_F_WIN95_CHG_STATUS           1
#define VS_F_WIN95_ALLOC_SOCKET         2
#define VS_F_WIN95_SET_CALLBACK         3

/*
** SUBPACKET DEFINITIONS
*/

    /*
    ** VS_ID_SPECIAL_CS_API
    */

    typedef struct
	{
	WORD                  arglength;   /* length of standard packet */
	CLT_REGISTRATION_PKT    pkt;
	DWORD                  vs_attr;
	FAR_FUNCTION_PTR        vs_cb_addr;
	} VS_REGISTER_CLIENT;

#define VS_CLTA_REMOVAL_EV      0x00000001   /* removal CB on powered off sockets */
#define VS_CLTA_CONFIG_EV       0x00000002   /* VS card config callback */
#define VS_CLTA_DOCKING_EV      0x00000003   /* docking events */
#define VS_CLTA_RESERVED_BITS   0xFFFFFFF8

    typedef struct
	{
	WORD          arglength;          /* length of standard packet */
	IO_REQUEST_PKT  pkt;
	BYTE            reserved;       /* for alignment */
	WORD          card_base[MAX_IO_WDWS_PER_SKT];
	} VS_REQUEST_IO;

    typedef struct
	{
	WORD          arglength;      /* length of standard packet */
	CNFG_INFO_PKT   pkt;
	BYTE            reserved;       /* for alignment */
	WORD          card_base[MAX_IO_WDWS_PER_SKT];
	} VS_GET_CNFG_INFO;

    typedef struct
	{
	WORD          arglength;      /* length of standard packet */
	CNFG_INFO_PKT50 pkt;
	BYTE            reserved;       /* for alignment */
	WORD          card_base[MAX_IO_WDWS_PER_SKT];
	} VS_GET_CNFG_INFO50;

    typedef struct
	{
	WORD          arglength;      /* length of standard packet */
	STATUS_PKT      pkt;
	WORD          cnfg_state;     /* same as socket configuration */
	} VS_GET_STATUS;                /* callback information defined */
					/* below                        */

    /*
    ** VS_ID_PWR_MGMT
    */

    typedef struct
	{
	LOGICAL_SOCKET l_socket;
	} VS_PWR_MGMT_SKT;

    typedef struct
	{
	WORD          suspend_type;
	BYTE            socket_bitmap[1];
	} VS_PWR_MGMT_SUSPEND;

    typedef struct
	{
	WORD          resume_type;
	} VS_PWR_MGMT_RESUME;

    /*
    ** VS_ID_RING_IND
    */

    typedef struct
	{
	LOGICAL_SOCKET l_socket;
	} VS_RING_IND;

    /*
    ** VS_ID_CSC_MASK
    */

    typedef struct
	{
	LOGICAL_SOCKET l_socket;
	} VS_CSC_MASK;

    /*
    ** VS_ID_CARDBOOT
    */

    typedef struct
	{
	BYTE rsrc_type;     /* Cardboot resource type (memory or I/O) */
	BYTE boot_status;
	WORD id_seg;      /* segment of address of CardBoot ID string */
	WORD id_off;      /* offset of address of CardBoot ID string */
	BYTE boot_adapter;
	BYTE boot_socket;
	BYTE boot_window;
	WORD boot_base;
	WORD boot_size;
	WORD ireq_info;
	} VS_CARDBOOT;

    /*
    ** VS_ID_BEEP_CTRL
    */

    typedef struct
	{
	WORD  request;
	WORD  setting;
	} BEEP_PKT;

    /*
    ** VS_ID_GET_INFO
    */

    typedef struct
	{
	DWORD          base;
	DWORD          size;
	DWORD          card_offset;
	WDW_ATTRIBUTES  attr;
	BYTE            access_speed;
	BYTE            page;
	} WINDOW_ENTRY;
    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	WORD          wdw_cnt;
	WINDOW_ENTRY    wdw_entries[1];
	} VS_WINDOW_INFO;


    typedef struct
	{
	WDW_ATTRIBUTES  attr;
	DWORD          base;
	DWORD          size;
	BYTE            access_speed;
	} MEM_ERROR_PKT;

    typedef struct
	{
	IO_PARMS       io_parms[MAX_IO_WDWS_PER_SKT];
	BYTE           io_addr_lines;
	} IO_ERROR_PKT;

    typedef struct
	{
	IRQ_ATTRIBUTES  attr;
	BYTE            assigned_irq;
	BYTE            info1;
	WORD          info2;
	} IRQ_ERROR_PKT;


    typedef struct
        {
	WORD          desired_channel;
	DMA_ATTRIBUTES  attr;
	BYTE            assigned_channel;
        } DMA_ERROR_PKT;


/**********************************************************************
 * Error structure                                                     *
 **********************************************************************/
#define ERROR_INFO_READ         0x0001	//BIT0
#define ERROR_LAST_SUCCESS      0x0002	//BIT1

    typedef struct
	{
	    
	BYTE           type;       /*  type of resource requested */
	BYTE           attrib;     /*  type of resource requested */    
	WORD         handle;     /* client handle     */
	RET_CODE       error;      /*  error returned */
	union
	    {
	    MEM_ERROR_PKT mem;
	    IO_ERROR_PKT  io;
	    IRQ_ERROR_PKT irq;
	    DMA_ERROR_PKT dma;
	   } error_type;
	} ERROR_INFO, *ERROR_INFO_PTR;


    typedef struct
	{
	LOGICAL_SOCKET  l_socket;           
	ERROR_INFO      error_info;
	} VS_ERROR_INFO;

    /*
    ** CARD LITE ENTRY FOR VS_ID_GET_INFO
    */

#define CL_MAX_MANUF_NAME       32
#define CL_MAX_PRODUCT_NAME     32
#define CL_MAX_ADDITIONAL_NAME  32

    typedef struct
	{
	BYTE            first_dev_type;
	BYTE            func_code;
	WORD          manuf_code;
	WORD          product_code;
	char            manuf_name[CL_MAX_MANUF_NAME];
	char            product_name[CL_MAX_PRODUCT_NAME];
	char            additional_name1[CL_MAX_ADDITIONAL_NAME];
	char            additional_name2[CL_MAX_ADDITIONAL_NAME];
	} CL_TUPLE_INFO;

    typedef struct
	{
	WORD          base_port;
	BYTE            num_ports;
	} CL_IO_PARMS;

    typedef struct
	{
	DWORD          base;
	DWORD          size;
	} CL_WINDOW_ENTRY;

#define VS_CL_CIA_SOCKET_CONFIGURED     0x0001
#define VS_CL_CIA_SOCKET_RECOGNIZED     0x0002

    typedef struct
	{
	LOGICAL_SOCKET  l_socket;
	WORD          attr;
	CL_TUPLE_INFO   tuple_info;
	CL_IO_PARMS     io_parms[2];
	CL_WINDOW_ENTRY wdw_entries[2];
	BYTE            assigned_irq;
	} VS_CL_CNFG_INFO;


    /*
    ** VS_ID_WIN95
    */
    typedef struct						
	{
	WORD          arglength;      /* length of standard packet */
	STATUS_PKT      pkt;
	WORD          wait_flag;
	} VS_CHG_STATUS;					/* for simulate status change */

    typedef struct						
	{
	WORD		logical_socket;
	WORD		flags;
	} VS_ALLOC_SOCKET;		/* for allocate socket  */

    typedef struct						
	{
	DWORD		alloc_callback_adr;
	DWORD		flags;
	} VS_SET_ALLOC_CB;		/* for set allocation callback */

/* flags defines for VS_SET_ALLOC_CALL */
#define W95_EVENTMODE_NORMAL (0x00)     /* do the usual thing */
#define W95_EVENTMODE_SLAVE  (0x01)     /* use VS_F_WIN95_CHG_STATUS call */
#define W95_EVENTMODE_POLL   (0x02)     /* don't use SC IRQ */

/*
** MAIN PACKET DEFINITION
*/

    typedef struct
	{
	WORD                  info_len;
	BYTE                    subcomponent_id; /* SUB-COMPONENT ID */
	BYTE                    function;        /* FUNCTION ID OF SUB-COMPONENT */
	union
	    {
	    VS_PWR_MGMT_SKT     pwr;
	    VS_PWR_MGMT_SUSPEND suspend;
	    VS_PWR_MGMT_RESUME  resume;
	    VS_RING_IND         ring;
	    VS_CSC_MASK         scim;
	    BEEP_PKT            beep;
	    VS_CARDBOOT         cardboot;
	    VS_REGISTER_CLIENT  regclt;
	    VS_REQUEST_IO       reqio;
	    VS_GET_CNFG_INFO    cnfginfo;
	    VS_GET_CNFG_INFO50  cnfginfo50;
	    VS_GET_STATUS       getstat;
	    VS_WINDOW_INFO      wdwinfo;
	    VS_ERROR_INFO       errorinfo;
	    VS_CL_CNFG_INFO     cl_cnfg_info;
	    VS_CHG_STATUS       chgstat;
	    VS_ALLOC_SOCKET     allocsocket;
	    VS_SET_ALLOC_CB		setalloc;
	    }                   type;
	} VENDOR_SPECIFIC_PKT;

/*
** PACKET LENGTH DEFINITIONS
*/

#define VS_BASE_LENGTH          4
#define VS_F_PWR_LENGTH         (VS_BASE_LENGTH)
#define VS_F_PWR_SKT_LENGTH     (VS_BASE_LENGTH+sizeof(VS_PWR_MGMT_SKT))
#define VS_F_PWR_SUSPEND_LENGTH (VS_BASE_LENGTH+sizeof(VS_PWR_MGMT_SUSPEND)-1)
  /* must add length of bitmap for conserve power syspend type */
#define VS_F_PWR_RESUME_LENGTH  (VS_BASE_LENGTH+sizeof(VS_PWR_MGMT_RESUME))
#define VS_F_RING_IND_LENGTH    (VS_BASE_LENGTH+sizeof(VS_RING_IND))
#define VS_F_CSC_MASK_LENGTH    (VS_BASE_LENGTH+sizeof(VS_CSC_MASK))
#define VS_F_GET_WINDOW_INFO_LEN (VS_BASE_LENGTH+sizeof(VS_WINDOW_INFO))
#define VS_F_GET_ERROR_INFO_LEN (VS_BASE_LENGTH+sizeof(VS_ERROR_INFO))  
#define VS_F_REGISTER_CLIENT_LEN (VS_BASE_LENGTH+sizeof(VS_REGISTER_CLIENT))
#define VS_F_REQUEST_IO_LEN     (VS_BASE_LENGTH+sizeof(VS_REQUEST_IO))
#define VS_F_GET_CNFG_INFO_LEN  (VS_BASE_LENGTH+sizeof(VS_GET_CNFG_INFO))
#define VS_F_GET_CNFG_INFO50_LEN (VS_BASE_LENGTH+sizeof(VS_GET_CNFG_INFO50))
#define VS_F_GET_STATUS_LEN     (VS_BASE_LENGTH+sizeof(VS_GET_STATUS))
#define VS_F_CL_CNFG_INFO_LEN   (VS_BASE_LENGTH+sizeof(VS_CL_CNFG_INFO))
#define VS_F_CHG_STATUS_LEN     (VS_BASE_LENGTH+sizeof(VS_CHG_STATUS))
#define VS_F_SET_ALLOC_CB_LEN   (VS_BASE_LENGTH+sizeof(VS_SET_ALLOC_CB))
#define VS_F_ALLOC_SOCKET_LEN   (VS_BASE_LENGTH+sizeof(VS_ALLOC_SOCKET))


    /*******************************************************************
     * Defines for vendor specific callback information.               *
     *******************************************************************

     /* CONFIGURATION STATE BITMAP FOR A SPECIFIC SOCKET */
#define CNFG_ST_MEMORY_WINDOWS          0x0001
#define CNFG_ST_CONFIGURED              0x0002
#define CNFG_ST_IN_IO_MODE              0x0004
#define CNFG_ST_MTD_REGISTERED          0x0008
#define CNFG_ST_OPEN_MEMORY             0x0010

#pragma pack()                          /* return packing to normal   */


#endif  /* _CARDSERV_H */

