#ifndef __GLOBALS_H__
#define __GLOBALS_H__ 0x0100

#include "MidasApi.h"

#define CLOCK_CYCLE  200000 /* Clock cycle in 100 ns */

#define DMA_BUFFER_SIZE 32768

#define TVOUTPIN  0
#define CCOUTPIN  1
#define INCLUDE_REGIONCODE  0
#define USE_LUX_TIMER       0

#define ONE_SECOND    (10000000*1)
#define TEN_SECONDS   (10000000*10)
#define HALF_SECOND   (10000000/2)
#define EIGHTH_SECOND (10000000/8)

// KS_SYNC_OFF will set the HwInitData.TurnOffSynchronization to TRUE
// this turns off the Kernel Streaming synchronization. The driver works
// with this off but may not be completely stable at this time
#define KS_SYNC_OFF  0

typedef struct _WORKTIME{
	KSEVENT_TIME_INTERVAL tim;
	LONGLONG LastTime;
} WORKTIME, *PWORKTIME;

typedef enum tagStreamType {
   InMpeg2Video = 0,
   InMpeg2Ac3,
#if TVOUTPIN
   OutNTSCVideo,
#endif
   InMpeg2Subpicture,
   OutVideoPort,
#if CCOUTPIN
   OutCloseCaption,
#endif
   NUMBER_OF_STREAMS
}  STREAMTYPES;

typedef struct tagActiveSrb {
  PHW_STREAM_REQUEST_BLOCK currentSRB;
  PKSSTREAM_HEADER StreamHdr;
  char *buffer;
  unsigned int bufsize;
  unsigned int bufoffset;
  unsigned int HdrsToProcess;
  unsigned int TotalHdrs;
} ACTIVESRB;

#define NUM_DMA_BLOCKS 2

typedef enum {
  BLK_STATE_EMPTY=0,
  BLK_STATE_FILL,
  BLK_STATE_READY,
  BLK_STATE_XFR
} BLK_STATE;

typedef struct _DMABLOCK {
  volatile BLK_STATE State;
  STREAM_PHYSICAL_ADDRESS Phys;
  PVOID     pBase;
  DWORD     dwSize;
  char*     pCur;
  DWORD     dwTotalBytes;
  DWORD     dwVidBytes;
  DWORD     dwAudBytes;
  DWORD     dwSubBytes;
}DMA_BLOCK, *PDMA_BLOCK;

typedef struct _DMA_CONTROL {
  DWORD   dwCtrlIO;

  STREAM_PHYSICAL_ADDRESS  PhysAddr;
  char* pBuffer;
  DWORD dwBuffSize;
  DWORD dwVidBytes;     //Keep track of space available in channel buffers
  DWORD dwAudBytes;
  DWORD dwSubBytes;

  int nBlkIn;
  int nBlkOut;
  DMA_BLOCK Block[NUM_DMA_BLOCKS];

}DMA_CONTROL, *PDMA_CONTROL;

NTSTATUS DeviceHandlePNPIrp(PDEVICE_OBJECT pDevObj,PIRP pIrp);

typedef struct _DEVICE_EXTENSION {
   PDEVICE_OBJECT  PhysicalDeviceObject;
   PDEVICE_OBJECT  ClassDeviceObject;

   PHW_STREAM_OBJECT pStream  [NUMBER_OF_STREAMS];
   int StreamsOpened;
   KTIMER Timer;
   KDPC TimerRoutine;
   int currentStream;
   int process_data;
   int process_ccdata;
   int nHardwareSleep;
   PHW_STREAM_REQUEST_BLOCK currentCcSRB;
   LONGLONG ddLastStreamTime;
   LONGLONG ddLastSystemTime;
   int nCssKeyPhase;
   int nBusAuthCount;
   int HwError;
   DWORD dwCropTop;
   PVOID hMidas;

   int          fDmaAvailable;
   DWORD        dwWin98PnpDevNode;
   DWORD        dwWin2kPnpDevNode;
   ULONGLONG    dlMemAddr;
   ULONGLONG    dlIoAddr;
   ULONGLONG    dlDmaIoAddr;
   DWORD        dwRscFlags;

   DMA_CONTROL  DmaControl;

   UNICODE_STRING SymbolicLinkName;
} DEVICE_EXTENSION, *PDEVICE_EXTENSION;

void __stdcall DmaTimer(PDEVICE_EXTENSION);

typedef struct srb_ext_struct {
  char szId[32];
  int  srbnumber;
  int  nTimeoutCount;
  MDS_DATA_BLOCK Mdb;
} SRB_EXTENSION, *PSRB_EXTENSION;

typedef HW_STREAM_REQUEST_BLOCK SRB;

typedef struct queue_struct {
   PHW_STREAM_REQUEST_BLOCK head;
   PHW_STREAM_REQUEST_BLOCK tail;
   PHW_STREAM_REQUEST_BLOCK queue;
   STREAMTYPES              StreamNumber;
   int count;
} StreamQueue;
   

#if 0
struct vp_struct {
   unsigned int ddrawHandle;
   unsigned int VidPortID;
   unsigned int SurfaceHandle;
   KS_AMVPDATAINFO VPFmt;
};
#endif

struct vp_struct {
   HANDLE ddrawHandle_Ring0;
   HANDLE VidPortHandle_Ring0;
   HANDLE SurfaceHandle_Ring0;
   DWORD  ddrawHandle_Ring3;
   DWORD  VidPortHandle_Ring3;
   DWORD  SurfaceHandle_Ring3;
   KS_AMVPDATAINFO VPFmt;
};

struct audio_struct {
   int AudioOutputMode;
   int AudioSource;
   int AudioSampleRate;
   int ClockOpens;
   ULONGLONG PauseTime;
   ULONGLONG StartTime;
   ULONGLONG CurrentTime;
   int TimeMark;
   int TimeInterval;
   int ClockTicking;
};

struct subpicture_struct {
   KSPROPERTY_SPPAL palette;
   KSPROPERTY_SPHLI highlite;
   KSPROPERTY_COMPOSIT_ON composite_on;
};

typedef struct _STREAM_EXTENSION {
   KSSTATE   stream_state;
   KSSTATE   prev_stream_state;
   DWORD     dwStateChangeTime;

   PDEVICE_EXTENSION pDevice;
   KSPIN_LOCK  streamExtensionLock;

   HANDLE MasterClock;
   ULONGLONG SystemPTS;
   DWORD  dwEventCount;
   int    internal_streamid;
   ULONGLONG RateStartTime;  //stream time at which to start this rate
   LONG      Rate;       //new rate * 10000 (decimal)
   union {
      KS_MPEGVIDEOINFO2        video;
      struct audio_struct      audio;
      struct subpicture_struct subpicture;
      struct vp_struct         vp;
   } u;
   PHW_STREAM_REQUEST_BLOCK pNewCssSRB;
   int fCssReqd;
   int nCssKeyPhase;
   int fFixupStreamTime;
   BOOL fIsMpeg2Payload;
   BYTE            Data[10];
 } STREAM_EXTENSION, *PSTREAM_EXTENSION;

#define IsEqualGUID2(guid1, guid2) \
        (!memcmp((guid1), (guid2), sizeof(GUID)))

//DrvEntry.c
extern long DrvDbgMsgInit(DWORD fInit);

//Audio.c
extern void __stdcall AudioReceive(PHW_STREAM_REQUEST_BLOCK srb);
extern void __stdcall AudioControl(PHW_STREAM_REQUEST_BLOCK srb);
extern long __stdcall AudioEvent(PHW_EVENT_DESCRIPTOR EventDescriptor);
extern long __stdcall AudioClock(PHW_TIME_CONTEXT TimeContext);

extern void UpdateClockEvents(struct _KDPC *Dpc, PDEVICE_EXTENSION pDevice, PVOID arg1, PVOID arg2);
extern void ProcessAudioFormat(PKSDATAFORMAT pFmt,SRB *pSrb);
extern int  AudClockFixupStrmTime(PDEVICE_EXTENSION pDevice, LONGLONG Time);

//From VP.C
extern void __stdcall VPReceive(PHW_STREAM_REQUEST_BLOCK srb);
extern void __stdcall VPControl(PHW_STREAM_REQUEST_BLOCK srb);
extern long __stdcall VPEvent(PHW_EVENT_DESCRIPTOR EventDescriptor);

//From CC.C
extern void __stdcall CCReceive(PHW_STREAM_REQUEST_BLOCK srb);
extern void __stdcall CCControl(PHW_STREAM_REQUEST_BLOCK srb);
extern long __stdcall CCEvent(PHW_EVENT_DESCRIPTOR EventDescriptor);
extern void CcClearClockEvents(PDEVICE_EXTENSION pDev);

//From CopyProt.h
extern void CopyProtectionGet(PHW_STREAM_REQUEST_BLOCK pSrb);
extern int  CopyProtectionSet(PHW_STREAM_REQUEST_BLOCK pSrb);

//From Device.c
extern void DeviceInitialize (PHW_STREAM_REQUEST_BLOCK pSrb);
extern void DeviceInitComplete(SRB* pSrb);
extern void DevicePagingOut(SRB* pSrb);
extern void DeviceChangePowerState(SRB* pSrb);
extern void DeviceSurpriseRemoval(SRB* pSrb);
extern void DeviceClose(PHW_STREAM_REQUEST_BLOCK pSrb);
extern int  DeviceCheckHardware(PHW_STREAM_REQUEST_BLOCK pSrb);
extern void DevicePowerDown(PHW_STREAM_REQUEST_BLOCK pSrb);
extern void DevicePrepareForPowerDown(PDEVICE_EXTENSION pdx);


//From Stream.h
extern void StreamInfo(PHW_STREAM_REQUEST_BLOCK pSrb);
extern void StreamOpen(PHW_STREAM_REQUEST_BLOCK pSrb);
extern void StreamClose(PHW_STREAM_REQUEST_BLOCK pSrb);
extern void StreamGetIntersection(PHW_STREAM_REQUEST_BLOCK pSrb);
extern void StreamSetState(PSTREAM_EXTENSION pStream,KSSTATE StreamState);
//extern void StreamSetState(PHW_STREAM_REQUEST_BLOCK pSrb);
extern void SignalDeviceRequestComplete(SRB* pSrb);
extern void SignalStreamRequestComplete(SRB* pSrb);
extern void SignalStreamRequestCompleteReadyForNextData(SRB* pSrb);
extern void SignalStreamRequestCompleteReadyForNextControl(SRB* pSrb);
extern void SignalStreamReadyForNextControl(SRB* pSrb);
extern void SignalStreamReadyForNextData(SRB* pSrb);
extern void  __stdcall StreamDataCallback(PMDS_DATA_BLOCK pMdb);
extern void  __stdcall StreamControlCallback(PMDS_DATA_BLOCK pMdb);
extern void CancelStream(PVOID hMidas,int StrmId);


//From Queue.h
extern int enqueue(StreamQueue *stream, PHW_STREAM_REQUEST_BLOCK ptr);
extern PHW_STREAM_REQUEST_BLOCK dequeue(StreamQueue *stream);
extern long TimerSetPowerState(PHW_STREAM_REQUEST_BLOCK pPowerSrb);
extern long TimerStart(DWORD Start,void *pRef);
extern long TimerStopComplete();

extern DWORD gTimerThreshold;
extern DWORD gTimerPeriod;
extern DWORD gDataSendMaxTime;
extern DWORD QPktTotal,QPktInQueue;

//From Video.c
extern void __stdcall VideoReceive(PHW_STREAM_REQUEST_BLOCK srb);
extern void __stdcall VideoControl(PHW_STREAM_REQUEST_BLOCK srb);
extern long __stdcall VideoEvent(PHW_EVENT_DESCRIPTOR EventDescriptor);
extern void ProcessVideoFormat(PKSDATAFORMAT pVidFmt,SRB *pSrb);
extern KS_AMVPDATAINFO VPFmt;

//From NTSC.C
extern void __stdcall NTSCReceive(PHW_STREAM_REQUEST_BLOCK srb);
extern void __stdcall NTSCControl(PHW_STREAM_REQUEST_BLOCK srb);
extern long __stdcall NTSCEvent(PHW_EVENT_DESCRIPTOR EventDescriptor);
extern void GetMacroVisionLevel(SRB* pSrb);
extern void SetMacroVisionLevel(SRB* pSrb);

//From SubPict.c
extern void __stdcall SubpictureReceive(PHW_STREAM_REQUEST_BLOCK srb);
extern void __stdcall SubpictureControl(PHW_STREAM_REQUEST_BLOCK srb);
extern long __stdcall SubpictureEvent(PHW_EVENT_DESCRIPTOR EventDescriptor);

//From CfgMgr.c
extern DWORD ConfigFindDevNode(void);
extern void  ConfigReleaseDevNode(DWORD);
extern int   ConfigCheckDma(DWORD DevNode);
extern int   ConfigSetupDma(DWORD DevNode,DWORD dwIoBase);


//From Speed.c
extern void RateChangeRequestGet(SRB* pSrb);
extern void RateChangeRequestSet(SRB* pSrb);

#endif
/**EOF: GLOBALS.H**/
