#include <BaseDef.h>
#include <VMM.h>
#include <VMMReg.h>
#include <String.h>
#include "Settings.h"
typedef DWORD HKEY;

//10Sep97 Chip Doran

static DWORD LoadDWord(DWORD dwType, char *Entry, DWORD *pDst, DWORD dwDefault);
static DWORD LoadStr(DWORD dwType, char *Entry, char *pDst, char *pDflt);
static void SaveDWord(DWORD dwType, char *Entry, DWORD dwValue);
static void SaveStr(DWORD dwType, char *Entry, char *sValue);

static char RegKeyRoot[80]= {"Software\\Margi_Systems"};
static char IniFName[40]= {"ChipUtil"};
static char IniSection[40]= {"FileMonitor"};
static char RegKeyPath[260]= {"Software\\Margi_Systems\\ChipUtil\\FileMonitor"};

/*************************************************/
static long MyRegCreateKey(HKEY hRoot, char *Path, HKEY *hKey) {
  long Err;
  Err= VMM_RegCreateKey(hRoot,Path,hKey);
  return(Err);
}

static long MyRegOpenKey(HKEY hRoot, char *Path, HKEY *pKey) {
  long Err;
  Err= VMM_RegOpenKey(hRoot,Path,pKey);
  return(Err);
}

static long MyRegCloseKey(HKEY hKey) {
  long Err;
  Err= VMM_RegCloseKey(hKey);
  return(Err);
}

static long MyRegQueryValue(HKEY hKey, char *Entry, DWORD *pType, void *pDst, DWORD *pBytes) {
  long Err;
  Err= VMM_RegQueryValueEx(hKey,Entry,0,pType,pDst,pBytes);
  return(Err);
}

static long MyRegSetValue(HKEY hKey, char *Entry, DWORD dwType, void *pData, DWORD dwBytes) {
  long Err;
  Err= VMM_RegSetValueEx(hKey,Entry,0,dwType,pData,dwBytes);
  return(Err);
}

/*************************************************/
void __stdcall SettingFName(char *NewFName, char *NewSection) {
  char *pSrc,*pDst,*StrEnd;
  if(NewFName) {
    for(pSrc=NewFName,pDst=IniFName;*pSrc && *pSrc!='.';*pDst++= *pSrc++);
    *pDst= 0;
  }
  if(NewSection)
    strncpy(IniSection,NewSection,sizeof(IniSection));
  pDst= RegKeyPath;
  StrEnd= RegKeyPath+sizeof(RegKeyPath);
  for(pSrc=RegKeyRoot;pDst<StrEnd && *pSrc;*pDst++= *pSrc++);
  if(pDst<StrEnd) *pDst++='\\';
  for(pSrc=IniFName;pDst<StrEnd && *pSrc;*pDst++= *pSrc++);
  if(pDst<StrEnd) *pDst++='\\';
  for(pSrc=IniSection;pDst<StrEnd && *pSrc;*pDst++= *pSrc++);
  *pDst= 0;
}

DWORD __stdcall SettingLoad(DWORD dwType, char *Entry, void *pDst, DWORD dwDefault) {
  DWORD Val= 0;
  switch(dwType & 0x00FF) {
    case SET_FLAG:  Val= LoadDWord(dwType,Entry,0,dwDefault); break;
    case SET_DWORD: Val= LoadDWord(dwType,Entry,pDst,dwDefault); break;
    case SET_STR:   Val= LoadStr(dwType,Entry,pDst,(char*)dwDefault); break;
  }
  return(Val);
}

void __stdcall SettingSave(DWORD dwType, char *Entry, DWORD dwValue) {
  switch(dwType & 0x00FF) {
    case SET_FLAG:
    case SET_DWORD: SaveDWord(dwType,Entry,dwValue); break;
    case SET_STR:   SaveStr(dwType,Entry,(char*)dwValue); break;
  }
}

static DWORD LoadDWord(DWORD dwType, char *Entry, DWORD *pDst, DWORD dwDefault) {
  HKEY hKey;
  DWORD dwBytes,Ret,dwValueType;
  Ret= pDst ? 0:dwDefault;
  dwBytes= sizeof(Ret);
  if(!MyRegOpenKey(HKEY_LOCAL_MACHINE,RegKeyPath,&hKey)) {
    if(!MyRegQueryValue(hKey,Entry,&dwValueType,(void*)&Ret,&dwBytes) || dwValueType!=REG_BINARY) {
      if(pDst) {
        *pDst= Ret;
        Ret= 1;
      }
    }
    MyRegCloseKey(hKey);
  }
  return(Ret);
}

static DWORD LoadStr(DWORD dwType, char *Entry, char *pDst, char *pDflt) {
  HKEY hKey;
  DWORD dwValueType,Ret= 80;
  if(!MyRegOpenKey(HKEY_LOCAL_MACHINE,RegKeyPath,&hKey)) {
    MyRegQueryValue(hKey,Entry,&dwValueType,pDst,&Ret);
    MyRegCloseKey(hKey);
    if(dwValueType!=REG_SZ) {
      *pDst= 0;
      Ret= 0;
    }
  }
  if(!Ret && pDflt)
    strcpy(pDst,pDflt);
  return(Ret);
}

static void SaveDWord(DWORD dwType, char *Entry, DWORD dwValue) {
  HKEY hKey;
  if(!MyRegCreateKey(HKEY_LOCAL_MACHINE,RegKeyPath,&hKey)) {
    MyRegSetValue(hKey,Entry,REG_BINARY,(void*)&dwValue,sizeof(dwValue));
    MyRegCloseKey(hKey);
  }
}

static void SaveStr(DWORD dwType, char *Entry, char *sValue) {
  HKEY hKey;
  if(!MyRegCreateKey(HKEY_LOCAL_MACHINE,RegKeyPath,&hKey)) {
    MyRegSetValue(hKey,Entry,REG_SZ,sValue,strlen(sValue)+1);
    MyRegCloseKey(hKey);
  }
}

/**EOF: SETTINGS.C**/
