#include <StrmIni.h>
#include <mmsystem.h>
#include <KsMedia.h>
#include "DbgMsg.h"
#include "DbgFltr.h"

#include "Globals.h"
#include "MidasApi.h"

void RateChangeRequestGet(SRB* pSrb);
void RateChangeRequestSet(SRB* pSrb);

static void RateChangeGetSimple(SRB* pSrb);
static void RateChangeSetSimple(SRB* pSrb);
static void RateChangeGetMaxRate(SRB* pSrb);

void RateChangeRequestGet(SRB* pSrb) {
  if(pSrb->CommandData.PropertyInfo->Property->Id==KS_AM_RATE_SimpleRateChange) {
    RateChangeGetSimple(pSrb);
  } else if(pSrb->CommandData.PropertyInfo->Property->Id==KS_AM_RATE_MaxFullDataRate) {
    RateChangeGetMaxRate(pSrb);
  } else {
    pSrb->Status = STATUS_NOT_IMPLEMENTED;
  }
}

void RateChangeRequestSet(SRB* pSrb) {
  if(pSrb->CommandData.PropertyInfo->Property->Id==KS_AM_RATE_SimpleRateChange) {
    RateChangeSetSimple(pSrb);
  } else {
    pSrb->Status = STATUS_NOT_IMPLEMENTED;
  }
}

static void RateChangeGetSimple(SRB* pSrb) {
  PDEVICE_EXTENSION pDevice;
  PSTREAM_EXTENSION pStream;
  KS_AM_SimpleRateChange* pKsRate=(KS_AM_SimpleRateChange*)pSrb->CommandData.PropertyInfo->PropertyInfo;
  MIDASCMD Cmd;
  pDevice = (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  Cmd.hMidas=pDevice->hMidas;
  Cmd.cmd = MDS_GETRATE;
  Cmd.wCurrStream= pStream->internal_streamid;
  MidasEntry(&Cmd);
  pKsRate->Rate=     Cmd.u.PlayRate.Rate;
  pKsRate->StartTime=Cmd.u.PlayRate.StartTime;
  pSrb->ActualBytesTransferred=sizeof(KS_AM_SimpleRateChange);
  pSrb->Status= STATUS_SUCCESS;
}

static void RateChangeSetSimple(SRB* pSrb) {
  PDEVICE_EXTENSION pDevice;
  PSTREAM_EXTENSION pStream;
  KS_AM_SimpleRateChange* pKsRate=(KS_AM_SimpleRateChange*)pSrb->CommandData.PropertyInfo->PropertyInfo;
  MIDASCMD Cmd;

  pDevice = (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;

  pStream->RateStartTime=pKsRate->StartTime;
  pStream->Rate         =pKsRate->Rate;

  Cmd.hMidas=pDevice->hMidas;
  Cmd.cmd = MDS_SETRATE;
  Cmd.wCurrStream= pStream->internal_streamid;
  Cmd.u.PlayRate.Rate=pKsRate->Rate;
  Cmd.u.PlayRate.StartTime=pKsRate->StartTime;
  Cmd.u.PlayRate.Mode=0;
  MidasEntry(&Cmd);
  pSrb->ActualBytesTransferred=sizeof(KS_AM_SimpleRateChange);
  pSrb->Status= STATUS_SUCCESS;
}

static void RateChangeGetMaxRate(SRB* pSrb) {
  PDEVICE_EXTENSION pDevice;
  PSTREAM_EXTENSION pStream;
  MIDASCMD Cmd;
  pDevice = (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  Cmd.hMidas=pDevice->hMidas;
  Cmd.cmd = MDS_GETMAXRATE;
  Cmd.wCurrStream= pStream->internal_streamid;
  Cmd.u.M.Arg1= (DWORD)pSrb->CommandData.PropertyInfo->PropertyInfo;
  MidasEntry(&Cmd);
  pStream->Rate= *(DWORD*)pSrb->CommandData.PropertyInfo->PropertyInfo;
  pSrb->ActualBytesTransferred = sizeof (KS_AM_RATE_MaxFullDataRate);
  pSrb->Status= STATUS_SUCCESS;
}

/* EOF : SPEED3.C  */
