#include <strmini.h>
#include <mmsystem.h>
#include <ksmedia.h>
#include "DbgMsg.h"
#include "DbgFltr.h"
#include "Idents.h"

#include "Globals.h"
#include "MidasApi.h"
#include "log.h"
#include "UpdtReg.h"
#include "settings.h"
#include "ininames.h"

typedef struct 
{
  DWORD    dwModified;
  DWORD    dwMixMode;
  DWORD    dwSpDiffMode;
  DWORD    dwSpDiffOut;
  DWORD    dwVOMode;              // video out mode
  DWORD    dwVOLuma;              // video out balance
  DWORD    dwVOChroma;
  DWORD    dwVOHue;
  DWORD    dwCorrectionEnable;   // computer screen
  DWORD    dwScreenContrast;      
  DWORD    dwScreenBrightness;
  DWORD    dwScreenGamma;
  DWORD    dwVideoMode;
  DWORD    dwVolume;
  DWORD    dwMute;
  DWORD    dwDisplayMode;
  DWORD    dwDisplayFormat;
  DWORD    dwKaraokeMode;
} REGDVCONFIGINFO;
typedef REGDVCONFIGINFO FAR*  LPREGDVCONFIGINFO;

int gfPreferredDisplayMode=MDS_NORMAL_RECT;


// call this function at hw startup to post current info to dvdshow.dll
// If the DVDSHOW.DLL settings modifed flag has not been set then
// copy the driver settings (owned by Midas layer) into the DVDSHOW settings.
// If the modified flag is set then do nothing and the timer will update the
// driver settings later on.
extern void __stdcall SetDefaultsInTempRegistry(PDEVICE_EXTENSION pDevice)
{
  DWORD dwNotModified=0,dwIsModified;
  REGDVCONFIGINFO info;
  MIDASCMD cmd;
  memset(&cmd,0,sizeof(cmd));
  memset(&info,0,sizeof(info));

  SettingFName(IniFileNameDvd,IniTempKey);
  dwIsModified= SettingLoad(SET_DWORD,IniModified,0,0);
  if(dwIsModified == TRUE)
  {
    // settings were modified using the control panel 
    // when the driver wasn't loaded -- don't overwrite,
    // let the timer pick up the changes.
    return;
  }

  /* read from settings registry */
  SettingFName(IniFName,IniSettingsKey);

  info.dwMixMode= SettingLoad(SET_DWORD,IniMixmode,0,0);
  info.dwSpDiffMode= SettingLoad(SET_DWORD,IniSpdiffmode,0,0);
  info.dwSpDiffOut= SettingLoad(SET_DWORD,IniSpdiffout,0,1);
  info.dwVOMode= SettingLoad(SET_DWORD,IniVomode,0,0);      
  info.dwVOLuma= SettingLoad(SET_DWORD,IniVoluma,0,50);
  info.dwVOChroma= SettingLoad(SET_DWORD,IniVochroma,0,50);
  info.dwVOHue= SettingLoad(SET_DWORD,IniVohue,0,0);
//  info.dwMute= SettingLoad(SET_DWORD,IniVOMute,0,0);
//  info.dwVolume= SettingLoad(SET_DWORD,IniVOVolume,0,100);
//  info.dwDisplayMode= SettingLoad(SET_DWORD,IniDisplayMode,0,0);
  info.dwKaraokeMode= SettingLoad(SET_DWORD,IniKaraokeMode,0,0);
  gfPreferredDisplayMode= SettingLoad(SET_DWORD,IniDisplayFmt,0,MDS_NORMAL_RECT);

  /* write to temp registry */
  /* Note: settings.c writes as BINARY data */
  SettingFName(IniFileNameDvd,IniTempKey);
  info.dwMute= SettingLoad(SET_DWORD,IniVOMute,0,0);
  info.dwVolume= SettingLoad(SET_DWORD,IniVOVolume,0,100);
  SettingSave(SET_DWORD,IniMixmode,info.dwMixMode);
  SettingSave(SET_DWORD,IniSpdiffmode,info.dwSpDiffMode);
  SettingSave(SET_DWORD,IniSpdiffout,info.dwSpDiffOut);
  SettingSave(SET_DWORD,IniVomode,info.dwVOMode);      
  SettingSave(SET_DWORD,IniVoluma,info.dwVOLuma);
  SettingSave(SET_DWORD,IniVochroma,info.dwVOChroma);
  SettingSave(SET_DWORD,IniVohue,info.dwVOHue);
//  SettingSave(SET_DWORD,IniVOVolume,info.dwVolume);
//  SettingSave(SET_DWORD,IniVOMute,info.dwMute);
//  SettingSave(SET_DWORD,IniDisplayMode,info.dwDisplayMode);
  SettingSave(SET_DWORD,IniKaraokeMode,info.dwKaraokeMode);
  SettingSave(SET_DWORD,IniDisplayFmt,gfPreferredDisplayMode);

  /* clear the "modified" flag -- only used by scenario.wdm */
  SettingSave(SET_DWORD,IniModified,(DWORD)dwNotModified);

  /* restore to original setting location */
  SettingFName(IniFName,IniSection);
}

/* checks the dvdshow.dll registry for changes to the driver settings */
extern int __stdcall CheckTempRegistryForChanges(PDEVICE_EXTENSION pDevice)
{
  DWORD dwModified=0;
  REGDVCONFIGINFO info;
  MIDASCMD cmd;
  memset(&info,0,sizeof(info));

  /* check for "modified" flag posted by the dvdshow.dll */
  SettingFName(IniFileNameDvd,IniTempKey);
  info.dwModified= SettingLoad(SET_DWORD,IniModified,0,0);
  DbgMsg(DBG_CP_TIMER|DBG_HIFREQ,"CP: %s changes found in registry\r\n",(info.dwModified?" ":"No"));

  if(info.dwModified == 1)
  {
        /* clear the "modified" flag */
        SettingSave(SET_DWORD,IniModified,(DWORD)dwModified);
        memset(&cmd,0,sizeof(cmd));

        /* user has changed info in the "driver" dialog */
        /* read registry & update each of the audio & video propertiees */

        info.dwMixMode= SettingLoad(SET_DWORD,IniMixmode,0,0);
        info.dwSpDiffMode= SettingLoad(SET_DWORD,IniSpdiffmode,0,0);
        info.dwSpDiffOut= SettingLoad(SET_DWORD,IniSpdiffout,0,1);
        info.dwVOMode= SettingLoad(SET_DWORD,IniVomode,0,0);      
        info.dwVOLuma= SettingLoad(SET_DWORD,IniVoluma,0,50);
        info.dwVOChroma= SettingLoad(SET_DWORD,IniVochroma,0,50);
        info.dwVOHue= SettingLoad(SET_DWORD,IniVohue,0,0);
        info.dwMute= SettingLoad(SET_DWORD,IniVOMute,0,0);
        info.dwVolume= SettingLoad(SET_DWORD,IniVOVolume,0,100);
//        info.dwDisplayMode= SettingLoad(SET_DWORD,IniDisplayMode,0,0);
        info.dwKaraokeMode= SettingLoad(SET_DWORD,IniKaraokeMode,0,0);
        gfPreferredDisplayMode= SettingLoad(SET_DWORD,IniDisplayFmt,0,MDS_NORMAL_RECT);


        cmd.cmd= MDS_SETMIXMODE;
        cmd.hMidas=pDevice->hMidas;
        cmd.u.M.Arg1= info.dwMixMode;
        cmd.u.M.Arg2= 0;
        MidasEntry(&cmd);

        cmd.cmd= MDS_SETSPDIFMODE;
        cmd.hMidas=pDevice->hMidas;
        cmd.u.M.Arg1= info.dwSpDiffMode;
        cmd.u.M.Arg2= 0;
        MidasEntry(&cmd);

        cmd.cmd= MDS_SETSPDIFENABLE;
        cmd.hMidas=pDevice->hMidas;
        cmd.u.M.Arg1= info.dwSpDiffOut;
        cmd.u.M.Arg2= 0;
        MidasEntry(&cmd);

        cmd.cmd= MDS_SETTVOUT;
        cmd.hMidas=pDevice->hMidas;
        cmd.u.M.Arg1= TVOUT_SETCOMP_SVIDEO;
        cmd.u.M.Arg2= info.dwVOMode;
        MidasEntry(&cmd);

        cmd.cmd= MDS_SETTVOUT;
        cmd.hMidas=pDevice->hMidas;
        cmd.u.M.Arg1= TVOUT_SETBRIGHTNESS;
        cmd.u.M.Arg2= info.dwVOLuma;
        MidasEntry(&cmd);

        cmd.cmd= MDS_SETTVOUT;
        cmd.hMidas=pDevice->hMidas;
        cmd.u.M.Arg1= TVOUT_SETCONTRAST;
        cmd.u.M.Arg2= info.dwVOChroma;
        MidasEntry(&cmd);

        cmd.cmd= MDS_SETTVOUT;
        cmd.hMidas=pDevice->hMidas;
        cmd.u.M.Arg1= TVOUT_SETHUE;
        cmd.u.M.Arg2= info.dwVOHue+50;  // info.hue is -50 to +50, Midas wants 0 to 100
        MidasEntry(&cmd);

        cmd.cmd= MDS_SETMUTE;
        cmd.hMidas=pDevice->hMidas;
        cmd.u.M.Arg1= info.dwMute;
        cmd.u.M.Arg2= 0;
        MidasEntry(&cmd);

        cmd.cmd= MDS_SETVOLUME;
        cmd.hMidas=pDevice->hMidas;
        cmd.u.M.Arg1= info.dwVolume;
        cmd.u.M.Arg2= 0;
        MidasEntry(&cmd);

//        cmd.cmd= MDS_VIDOUTMODE;
//        cmd.hMidas=pDevice->hMidas;
//        cmd.u.M.Arg1= info.dwDisplayMode;
//        cmd.u.M.Arg2= 0;
//        MidasEntry(&cmd);

//        cmd.cmd= MDS_VIDOUTFORMAT;
//        cmd.hMidas=pDevice->hMidas;
//        cmd.u.M.Arg1= info.dwDisplayFormat;
//        cmd.u.M.Arg2= 0;
//        MidasEntry(&cmd);

        cmd.cmd= MDS_SETKARAOKEMODE;
        cmd.hMidas=pDevice->hMidas;
        cmd.u.M.Arg1= info.dwKaraokeMode;
        cmd.u.M.Arg2= 0;
        MidasEntry(&cmd);
       
  }

  /* restore to original setting location */
  SettingFName(IniFName,IniSection);

  return 1;
}

