#include <StrMini.h>
#include <mmsystem.h>
#include <ksmedia.h>
#include "DbgMsg.h"
#include "DbgFltr.h"
#include "Idents.h"
#include "Globals.h"
#include "MidasApi.h"
#include "Log.h"
#include "dbgout.h"

#pragma message("Audio.c: Optimizer disabled.")
#pragma optimize("",off)

static void AudSetState(SRB *pSrb);
static void AudSetStateStop(SRB *pSrb);
static void AudSetStatePause(SRB *pSrb);
static void AudSetStateRun(SRB *pSrb);
static void AudGetState(SRB *pSrb);
static void AudGetProperty(SRB *pSrb);
static void AudGetPropDecoderModes(SRB *pSrb);
static void AudGetPropDecoderMode(SRB *pSrb);
static int  AudSetProperty(SRB *pSrb);
static void AudSetPropDecoderMode(SRB *pSrb);
static void AudClockClose(SRB *pSrb);
static void AudClockOpen(SRB *pSrb);
static void AudClockIndicate(SRB *pSrb);
static void AudProposeDataFmt(SRB* pSrb);
static void AudSetStreamRate(SRB* pSrb);
static void AudControlUnkMsg(SRB* pSrb);
static void AudBeginFlush(SRB* pSrb);
static void AudEndFlush(SRB* pSrb);
void ProcessAudioFormat(PKSDATAFORMAT pFmt,SRB *pSrb);


//From Stream.c
extern KSDATAFORMAT_WAVEFORMATEX hwfmtiMpeg2Aud;
extern KSDATAFORMAT_WAVEFORMATEX hwfmtiMpeg2AudPESMpeg2;
extern KSDATAFORMAT_WAVEFORMATEX hwfmtiLPCMAud;
extern KSDATAFORMAT_WAVEFORMATEX hwfmtiMpeg2AudPES;
extern KSDATAFORMAT_WAVEFORMATEX InMpeg1Aud_Fmt0X;

void __stdcall AudioReceive(SRB *pSrb) {
  PSTREAM_EXTENSION pStream= (void*)pSrb->StreamObject->HwStreamExtension;
  logSrb("AudioReceive",pSrb);
  DbgDispKsStreamHeaders(DBG_MSGAUD,pSrb);
  if(pStream->stream_state==KSSTATE_STOP) {
    pSrb->Status= STATUS_CANCELLED;
    SignalStreamRequestCompleteReadyForNextData(pSrb);
  } else if(pSrb->CommandData.DataBufferArray->OptionsFlags & KSSTREAM_HEADER_OPTIONSF_TYPECHANGED) {
    PKSSTREAM_HEADER pStrmHdr= pSrb->CommandData.DataBufferArray;
    ProcessAudioFormat((PKSDATAFORMAT)pStrmHdr->Data,pSrb);
    pSrb->Status= STATUS_SUCCESS;
    SignalStreamRequestCompleteReadyForNextData(pSrb);
  } else {
    KIRQL Irql;  
    MIDASCMD Cmd;
    PDEVICE_EXTENSION pDevice= (PDEVICE_EXTENSION)pSrb->HwDeviceExtension;
    PMDS_DATA_BLOCK  pMdb=&((PSRB_EXTENSION)pSrb->SRBExtension)->Mdb;

#if(0)
    if(pSrb->CommandData.DataBufferArray->OptionsFlags&KSSTREAM_HEADER_OPTIONSF_TIMEVALID) {
      if(AudClockFixupStrmTime(pDevice,pSrb->CommandData.DataBufferArray->PresentationTime.Time)) {
        DbgMsg(DBG_MSGAUD,"Audio Fixup:");
        DbgDispKsStreamHeaders(DBG_MSGAUD,pSrb);
      }
    }
#endif
    pMdb->nSize=sizeof(MDS_DATA_BLOCK);
    pMdb->dwStatus=0;
    pMdb->dwStrmType=pStream->internal_streamid;
    pMdb->dwTotalPackets=pSrb->NumberOfBuffers;
    pMdb->pPackets=(PMDS_DATA_PACKET)pSrb->CommandData.DataBufferArray;
    pMdb->pNextMdb=0;
    pMdb->dwDataFlags=0;
    pMdb->hRef=(void*)pSrb;
    pMdb->pfnCallback=StreamDataCallback;

    Cmd.hMidas=pDevice->hMidas;
    Cmd.cmd = MDS_QUEUEDATA;
    Cmd.wCurrStream=pStream->internal_streamid;
    Cmd.u.pMdb=pMdb;
    KeAcquireSpinLock(&pStream->streamExtensionLock, &Irql); 
    MidasEntry(&Cmd);
	  KeReleaseSpinLock(&pStream->streamExtensionLock, Irql);   
    SignalStreamReadyForNextData(pSrb);
  }
}

void __stdcall AudioControl(SRB *pSrb) {
  int fSignaled=0;
  logSrb("AudioControl", pSrb);
  pSrb->Status = STATUS_NOT_IMPLEMENTED;

  // determine the type of packet.  All calls here use the pointers
  // to the individual stream functions initialised at open time
  switch (pSrb->Command) {
    case SRB_SET_STREAM_STATE:      AudSetState(pSrb); break;
    case SRB_GET_STREAM_STATE:      AudGetState(pSrb); break;
    case SRB_GET_STREAM_PROPERTY:   AudGetProperty(pSrb); break;
    case SRB_SET_STREAM_PROPERTY:   fSignaled=AudSetProperty(pSrb); break;
    case SRB_PROPOSE_DATA_FORMAT:   AudProposeDataFmt(pSrb); break;
    case SRB_SET_STREAM_RATE:       AudSetStreamRate(pSrb); break; //Should be set on an individual stream basis
    case SRB_OPEN_MASTER_CLOCK:     AudClockOpen(pSrb); break;
    case SRB_CLOSE_MASTER_CLOCK:    AudClockClose(pSrb); break;
    case SRB_INDICATE_MASTER_CLOCK: AudClockIndicate(pSrb); break;
    case SRB_BEGIN_FLUSH:           AudBeginFlush(pSrb); break;
    case SRB_END_FLUSH:             AudEndFlush(pSrb); break;
    default:                        AudControlUnkMsg(pSrb); break;
  }

  unlogSrb("AudioControl", pSrb);
  if(!fSignaled && pSrb->Status!=STATUS_PENDING) {
    SignalStreamRequestCompleteReadyForNextControl(pSrb);
  } else {
    DbgMsg(DBG_INFO,"Returning Status Pending %s\r\n",Ident(ID_STRM_REQ,pSrb->Command));
  }
}

static void AudControlUnkMsg(SRB* pSrb) {
  DbgMsg(DBG_WRN,"AudControl: Unhandled message %s \r\n",Ident(ID_STRM_REQ,pSrb->Command));
}

// convert from ticks to 100ns clock
ULONGLONG GetSystemTime(void) {
  ULONGLONG ticks;
  ULONGLONG rate;
  ticks= (ULONGLONG)KeQueryPerformanceCounter((PLARGE_INTEGER)&rate).QuadPart; //milliseconds
  ticks= (ticks & 0xFFFFFFFF00000000) / rate * 10000000 +
			(ticks & 0xFFFFFFFF) * 10000000 / rate;
  return(ticks);
}

static void AudEndFlush(SRB* pSrb) {
  PDEVICE_EXTENSION pDevice;
  PSTREAM_EXTENSION pStream;
  MIDASCMD Cmd;
  pDevice = (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  Cmd.hMidas=pDevice->hMidas;
  Cmd.cmd = MDS_FLUSH_END;
  Cmd.wCurrStream= pStream->internal_streamid;
  MidasEntry(&Cmd);
  pSrb->Status= STATUS_SUCCESS;
}

static void AudBeginFlush(SRB* pSrb) {
  MIDASCMD Cmd;
  PDEVICE_EXTENSION pDevice=(PDEVICE_EXTENSION)pSrb->HwDeviceExtension;
  PSTREAM_EXTENSION pStream=(PSTREAM_EXTENSION)pSrb->StreamObject->HwStreamExtension; 
  PMDS_DATA_BLOCK   pMdb=  &((PSRB_EXTENSION)pSrb->SRBExtension)->Mdb;
  
  pMdb->nSize=sizeof(MDS_DATA_BLOCK);
  pMdb->dwStatus=0;
  pMdb->dwStrmType=pStream->internal_streamid;
  pMdb->dwTotalPackets=0;
  pMdb->pPackets=0;
  pMdb->pNextMdb=0;
  pMdb->dwDataFlags=0;
  pMdb->hRef=(void*)pSrb;
  pMdb->pfnCallback=StreamControlCallback;

  Cmd.hMidas=pDevice->hMidas;
  Cmd.cmd = MDS_FLUSH;
  Cmd.wCurrStream= pStream->internal_streamid;
  Cmd.u.pMdb=pMdb;
  MidasEntry(&Cmd);
  pSrb->Status= STATUS_PENDING;
}

// change the stream state (i.e.  play or pause)
static void AudSetState(SRB *pSrb) {
  pSrb->Status = STATUS_SUCCESS;
  DbgMsg(DBG_MSGAUD,"AudControl: SetState to %s \r\n",Ident(ID_KS_STATE,pSrb->CommandData.StreamState));
  switch (pSrb->CommandData.StreamState) {
    case KSSTATE_STOP:  AudSetStateStop(pSrb); break;
    case KSSTATE_PAUSE: AudSetStatePause(pSrb); break;
    case KSSTATE_RUN:   AudSetStateRun(pSrb); break;
  }
}

static void AudSetStateStop(SRB *pSrb) {
  struct _HW_STREAM_OBJECT *pStreamObj= pSrb->StreamObject;
  PDEVICE_EXTENSION pDevice= (void*)pSrb->HwDeviceExtension;
  PSTREAM_EXTENSION pStream= (void*)pStreamObj->HwStreamExtension;
  MIDASCMD Cmd;
  Cmd.hMidas=pDevice->hMidas;
  if(pStream->stream_state != KSSTATE_STOP) {
    Cmd.cmd = MDS_STOP;
    Cmd.wCurrStream= pStream->internal_streamid;
    MidasEntry(&Cmd);
  }
  StreamSetState(pStream,KSSTATE_STOP);
}

static void AudSetStatePause(SRB *pSrb) {
  struct _HW_STREAM_OBJECT *pStreamObj= pSrb->StreamObject;
  PDEVICE_EXTENSION pDevice= (void*)pSrb->HwDeviceExtension;
  PSTREAM_EXTENSION pStream= (void*)pStreamObj->HwStreamExtension;
  MIDASCMD Cmd;
  Cmd.hMidas=pDevice->hMidas;
  if(pStream->stream_state != KSSTATE_PAUSE) {
    Cmd.cmd = MDS_PAUSE;
    Cmd.wCurrStream= pStream->internal_streamid;
    MidasEntry(&Cmd);
  }
  StreamSetState(pStream,KSSTATE_PAUSE);
}

static void AudSetStateRun(SRB *pSrb) {
  struct _HW_STREAM_OBJECT *pStreamObj= pSrb->StreamObject;
  PDEVICE_EXTENSION pDevice= (void*)pSrb->HwDeviceExtension;
  PSTREAM_EXTENSION pStream= (void*)pStreamObj->HwStreamExtension;
  MIDASCMD Cmd;
  if(DeviceCheckHardware(pSrb)) {
    Cmd.hMidas=pDevice->hMidas;
    if (pStream->stream_state != KSSTATE_RUN) {
      Cmd.cmd = MDS_PLAY;
      Cmd.u.nData = MDS_PLAY_NORMAL;
      Cmd.wCurrStream= pStream->internal_streamid;
      MidasEntry(&Cmd);
    }
    StreamSetState(pStream,KSSTATE_RUN);
  }
}

// Return the stream state
static void AudGetState(SRB *pSrb) {
  struct _HW_STREAM_OBJECT *pStreamObj= pSrb->StreamObject;
  PDEVICE_EXTENSION pDevice= (void*)pSrb->HwDeviceExtension;
  PSTREAM_EXTENSION pStream= (void*)pStreamObj->HwStreamExtension;
  pSrb->CommandData.StreamState = pStream->stream_state;
  DbgMsg(DBG_MSGAUD,"AudControl: GetState returns %s \r\n",Ident(ID_KS_STATE,pSrb->CommandData.StreamState));
  pSrb->Status = STATUS_SUCCESS;
}

static void AudGetProperty(SRB *pSrb) {
  long lID= pSrb->CommandData.PropertyInfo->PropertySetID;
  logGetProperty("Audio", pSrb);
  if(lID==0) {
    switch(pSrb->CommandData.PropertyInfo->Property->Id) {
      case KSPROPERTY_AUDDECOUT_MODES:    AudGetPropDecoderModes(pSrb); break;
      case KSPROPERTY_AUDDECOUT_CUR_MODE: AudGetPropDecoderMode(pSrb); break;
    }
    DbgMsg(DBG_MSGAUD,"AudControl: GetProperty for AudioDecoderOutput (returns)\r\n");
    DbgDispKsPropertyAud(DBG_MSGAUD,pSrb->CommandData.PropertyInfo->Property);
  } else if(lID==1) {
    CopyProtectionGet(pSrb);
    DbgMsg(DBG_MSGAUD,"AudControl: GetProperty-Css (returns)\r\n");
    DbgDispKsCssPropertyInfo(DBG_MSGAUD,pSrb->CommandData.PropertyInfo->PropertyInfo,
                             pSrb->CommandData.PropertyInfo->Property->Id);
  } else if(lID==2) {
    RateChangeRequestGet(pSrb);
    DbgMsg(DBG_MSGAUD,"AudControl: GetProperty-Rate (returns)\r\n");
    DbgDispKsRateChangeInfo(DBG_MSGAUD,pSrb->CommandData.PropertyInfo->PropertyInfo,
                               pSrb->CommandData.PropertyInfo->Property->Id);
  } else {
    DbgMsg(DBG_MSGERR,"AudControl: GetProperty called with an unknown ID (%08x)\r\n",lID);
  }
}

// available output modes of decoder
static void AudGetPropDecoderModes(SRB *pSrb) {
  DWORD Modes= KSAUDDECOUTMODE_STEREO_ANALOG | KSAUDDECOUTMODE_PCM_51 | KSAUDDECOUTMODE_SPDIFF;
  *(DWORD*)(pSrb->CommandData.PropertyInfo->PropertyInfo)= Modes;
  pSrb->ActualBytesTransferred= sizeof(Modes);
  pSrb->Status= STATUS_SUCCESS;
}

// current mode of the decoder
static void AudGetPropDecoderMode(SRB *pSrb) {
  struct _HW_STREAM_OBJECT *pStreamObj= pSrb->StreamObject;
  PDEVICE_EXTENSION pDevice= (void*)pSrb->HwDeviceExtension;
  PSTREAM_EXTENSION pStream= (void*)pStreamObj->HwStreamExtension;
  DWORD *pInfo= (void*)&pSrb->CommandData.PropertyInfo->PropertyInfo;
  switch (pStream->u.audio.AudioOutputMode) {
    case MDS_DAC:        *pInfo= KSAUDDECOUTMODE_STEREO_ANALOG; break;
    case MDS_IEC958:     *pInfo= KSAUDDECOUTMODE_PCM_51; break;
    case MDS_CD_BYPASS:  *pInfo= KSAUDDECOUTMODE_SPDIFF; break;
  }
  pSrb->ActualBytesTransferred= sizeof(*pInfo);
  pSrb->Status= STATUS_SUCCESS;
}

static int AudSetProperty(SRB *pSrb) {
  int fSignaled=0;
  DWORD lID= pSrb->CommandData.PropertyInfo->PropertySetID;
  logSetProperty("Audio", pSrb);
  if(lID==0) {
    DbgMsg(DBG_MSGAUD,"AudControl: SetProperty for AudioDecoderOutput (returns)\r\n");
    DbgDispKsPropertyAud(DBG_MSGAUD,pSrb->CommandData.PropertyInfo->Property);
    switch(pSrb->CommandData.PropertyInfo->Property->Id) {
      case KSPROPERTY_AUDDECOUT_CUR_MODE: AudSetPropDecoderMode(pSrb); break;
    }
  } else if(lID==1) {
    DbgMsg(DBG_MSGAUD,"AudControl: SetProperty-Css \r\n");
    DbgDispKsCssPropertyInfo(DBG_MSGAUD,pSrb->CommandData.PropertyInfo->PropertyInfo,
                             pSrb->CommandData.PropertyInfo->Property->Id);
    fSignaled=CopyProtectionSet(pSrb);
  } else if(lID==2) {
    DbgMsg(DBG_MSGAUD,"AudControl: GetProperty-Rate (returns)\r\n");
    DbgDispKsRateChangeInfo(DBG_MSGAUD,pSrb->CommandData.PropertyInfo->PropertyInfo,
                               pSrb->CommandData.PropertyInfo->Property->Id);
    RateChangeRequestSet(pSrb);
  } else {
    DbgMsg(DBG_MSGERR,"AudControl: SetProperty called with an unknown ID \r\n");
  }
  return fSignaled;
}

// current mode of the decoder
static void AudSetPropDecoderMode(SRB *pSrb) {
  DWORD PrevMode,NewMode;
  struct _HW_STREAM_OBJECT *pStreamObj= pSrb->StreamObject;
  PDEVICE_EXTENSION pDevice= (void*)pSrb->HwDeviceExtension;
  PSTREAM_EXTENSION pStream= (void*)pStreamObj->HwStreamExtension;
  MIDASCMD Cmd;
  PrevMode= pStream->u.audio.AudioOutputMode;
  NewMode= PrevMode;
  switch(*(DWORD*)(pSrb->CommandData.PropertyInfo->PropertyInfo)) {
    case KSAUDDECOUTMODE_STEREO_ANALOG: NewMode= MDS_DAC; break;
    case KSAUDDECOUTMODE_PCM_51:        NewMode= MDS_IEC958; break;
    case KSAUDDECOUTMODE_SPDIFF:        NewMode= MDS_CD_BYPASS; break;
  }
  pStream->u.audio.AudioOutputMode= NewMode;
  if(NewMode!=PrevMode) {
    Cmd.cmd = MDS_INITIALIZEAUDIO;
    Cmd.hMidas=pDevice->hMidas;
    Cmd.u.InitAudio.AudioSource = pStream->u.audio.AudioSource;
    Cmd.u.InitAudio.AudioSampleRate = pStream->u.audio.AudioSampleRate;
    Cmd.u.InitAudio.AudioOutputMode = pStream->u.audio.AudioOutputMode;
    Cmd.wCurrStream= pStream->internal_streamid;
    MidasEntry(&Cmd);
  }
  pSrb->ActualBytesTransferred= sizeof(NewMode);
  pSrb->Status = STATUS_SUCCESS;
}

static void AudClockClose(SRB *pSrb) {         
  struct _HW_STREAM_OBJECT *pStreamObj= pSrb->StreamObject;
  PDEVICE_EXTENSION pDevice= (void*)pSrb->HwDeviceExtension;
  PSTREAM_EXTENSION pStream= (void*)pStreamObj->HwStreamExtension;
  /* Though it doesn't make any sense, MasterClockHandle is passed in as */
  /* 0, so this check is not doable */
//if (pSrb->CommandData.MasterClockHandle == pStream->MasterClock)
  {
    pStream->u.audio.ClockOpens--;
    pSrb->Status = STATUS_SUCCESS;
  }
  DbgMsg(DBG_MSGAUD,"AudControl: Closing the MasterClock (Wht is this Supposed to mean)\r\n");
}

static void AudClockOpen(SRB *pSrb) {
  struct _HW_STREAM_OBJECT *pStreamObj= pSrb->StreamObject;
  PDEVICE_EXTENSION pDevice= (void*)pSrb->HwDeviceExtension;
  PSTREAM_EXTENSION pStream= (void*)pStreamObj->HwStreamExtension;
  pStream->u.audio.ClockOpens++;
  pDevice->ddLastStreamTime=0;
  DbgMsg(DBG_MSGAUD,"AudControl: CurrAudStream has the MasterClock \r\n");
  pSrb->Status = STATUS_SUCCESS;
}

static void AudClockIndicate(SRB *pSrb) {
  struct _HW_STREAM_OBJECT *pStreamObj= pSrb->StreamObject;
  PDEVICE_EXTENSION pDevice= (void*)pSrb->HwDeviceExtension;
  PSTREAM_EXTENSION pStream= (void*)pStreamObj->HwStreamExtension;
  pStream->MasterClock = pSrb->CommandData.MasterClockHandle;
  pSrb->Status = STATUS_SUCCESS;
  DbgMsg(DBG_MSGAUD,"AudControl: MasterClockHandle is set to %08x \r\n",pSrb->CommandData.MasterClockHandle);
}

/*************************************************************************/
/** Audio event handler                                                 **/
/*************************************************************************/
static long AudEventClockPosMark(PHW_EVENT_DESCRIPTOR pEvent);
static long AudEventCloseIntMark(PHW_EVENT_DESCRIPTOR pEvent);
static long AudEventUnk(PHW_EVENT_DESCRIPTOR pEvent);

long __stdcall AudioEvent(PHW_EVENT_DESCRIPTOR pEvent) {
  long Status= STATUS_SUCCESS;
  PSTREAM_EXTENSION pStrm= pEvent->StreamObject->HwStreamExtension;
  logEvent("AudioEvent", pEvent);
  if(pEvent->Enable)
    pStrm->dwEventCount++;
  else
    pStrm->dwEventCount--;

  switch(pEvent->EventEntry->EventItem->EventId) {
    case KSEVENT_CLOCK_POSITION_MARK: Status= AudEventClockPosMark(pEvent); break;
    case KSEVENT_CLOCK_INTERVAL_MARK: Status= AudEventCloseIntMark(pEvent); break;
    default:                          Status= AudEventUnk(pEvent);
  }
  return(Status);
}

static long AudEventClockPosMark(PHW_EVENT_DESCRIPTOR pEvent) {
  long Status= STATUS_SUCCESS;
  PWORKTIME pTim= ((PWORKTIME)(pEvent->EventEntry+1));
  struct _HW_STREAM_OBJECT *pStreamObj= pEvent->StreamObject;
  PSTREAM_EXTENSION pStream = pStreamObj->HwStreamExtension;
  struct audio_struct *audio= &pStream->u.audio;
  if(pEvent->Enable) {
    if(pEvent->EventEntry->EventItem->DataInput!=sizeof(KSEVENT_TIME_MARK)) {
      unlogEvent("AudioEvent", STATUS_INVALID_BUFFER_SIZE);
      Status= STATUS_INVALID_BUFFER_SIZE;
    } else {
      HW_TIME_CONTEXT Time;
      Time.Function=TIME_GET_STREAM_TIME;
      Time.HwDeviceExtension=(void*)pStream->pDevice;
      Time.HwStreamObject=   (void*)pEvent->StreamObject;
      AudioClock(&Time);
      audio->TimeMark++;
      RtlCopyMemory(pEvent->EventEntry+1,pEvent->EventData,sizeof(KSEVENT_TIME_MARK));
      DbgMsg(DBG_MSGEVENT,"CLKEVE: Registering Event for POSITION Callback at");
      DbgDispStrmTime(DBG_MSGEVENT,pTim->tim.TimeBase," Pts=","");
      DbgDispStrmTime(DBG_MSGEVENT,Time.Time," (",")\r\n");
      pTim->tim.TimeBase=Time.Time; // TimeBase seems to have big problems so don't use it
    }
  } else {
    if(audio->TimeMark) {
      audio->TimeMark--;
    } else {
      DEBUG_BREAKPOINT();
    }
  }
  return(Status);
}

static long AudEventCloseIntMark(PHW_EVENT_DESCRIPTOR pEvent) {
  long Status= STATUS_SUCCESS;
  PWORKTIME pTim= ((PWORKTIME)(pEvent->EventEntry+1));
  struct _HW_STREAM_OBJECT *pStreamObj= pEvent->StreamObject;
  PSTREAM_EXTENSION pStream = pStreamObj->HwStreamExtension;
  struct audio_struct *audio= &pStream->u.audio;
  if(pEvent->Enable) {
    if(pEvent->EventEntry->EventItem->DataInput != sizeof(KSEVENT_TIME_INTERVAL)) {
      unlogEvent("AudioEvent", STATUS_INVALID_BUFFER_SIZE);
      Status= STATUS_INVALID_BUFFER_SIZE;
    } else {
      HW_TIME_CONTEXT Time;
      Time.Function=TIME_GET_STREAM_TIME;
      Time.HwDeviceExtension=(void*)pStream->pDevice;
      Time.HwStreamObject=   (void*)pEvent->StreamObject;
      AudioClock(&Time);
      audio->TimeInterval++;
      RtlCopyMemory(pEvent->EventEntry+1,pEvent->EventData,sizeof(KSEVENT_TIME_INTERVAL));
      DbgMsg(DBG_MSGEVENT,"CLKEVE: Registering Event for INTERVAL Callback ");
      DbgDispStrmTime(DBG_MSGEVENT,pTim->tim.TimeBase," Base:","(forced to 0)");
      DbgDispStrmTime(DBG_MSGEVENT,pTim->tim.Interval," Interval:","\r\n");
      pTim->tim.TimeBase=Time.Time; // TimeBase seems to have big problems so don't use it
      pTim->LastTime= 0;
    }
  } else {
    if(audio->TimeInterval) {
      audio->TimeInterval--;
    } else {
      DEBUG_BREAKPOINT();
    }
  }
  return(Status);
}

static long AudEventUnk(PHW_EVENT_DESCRIPTOR pEvent) {
  unlogEvent("AudioEvent", STATUS_NOT_IMPLEMENTED);
  return(STATUS_NOT_IMPLEMENTED);
}

/*************************************************************************/
/** StreamClock                                                         **/
/** routine to call for current master clock value                      **/
/*************************************************************************/
static BOOL  AudClockGetStrmTime(PHW_TIME_CONTEXT pTime);
static void  AudClockSetStrmTime(PDEVICE_EXTENSION pDevice, ULONGLONG Time);
static DWORD AudClockRead(PHW_TIME_CONTEXT pTime);
static DWORD AudClockSet(PHW_TIME_CONTEXT pTime);
static DWORD AudClockUnk(PHW_TIME_CONTEXT pTime);

long __stdcall AudioClock(PHW_TIME_CONTEXT pTime) {
  BOOL Handled= 0;
  DbgMsg(DBG_MSGTIMER, "StreamClock %s ", Ident(ID_CLOCKCMD,pTime->Function));
  switch(pTime->Function) {
    case TIME_GET_STREAM_TIME:    Handled= AudClockGetStrmTime(pTime); break;
    case TIME_READ_ONBOARD_CLOCK: Handled= AudClockRead(pTime); break;
    case TIME_SET_ONBOARD_CLOCK:  Handled= AudClockSet(pTime); break;
    default:			  Handled= AudClockUnk(pTime); break;
  }
  DbgDispStrmTime(DBG_MSGTIMER,pTime->Time," Time=","\r\n");
  return(Handled);
}

// Used to detect if KS has changed the stream time on us
// can occur at a DataDiscontinuity if the 32 bit SCR has wrapped
// This is called by Audio and Video with 1st Time stamp after a 
// Datadiscontinuity
int AudClockFixupStrmTime(PDEVICE_EXTENSION pDevice, LONGLONG ddPresentationTime) {
  MIDASCMD Cmd;
  LONGLONG ddCurStrmTime;
  LONGLONG ddTimeDiff;
  int      fFixed=0;
  Cmd.hMidas=pDevice->hMidas;
  Cmd.cmd = MDS_GET_STREAM_TIME;
  MidasEntry(&Cmd);
  ddCurStrmTime=Cmd.u.GetTime.CurTime;
  // If our current stream time is later than the new time stamp
  // by 2 seconds or it is earlier by 10 Seconds or more
  // then apply new time stamp - 1Sec to our stream clock
  ddTimeDiff=ddPresentationTime-ddCurStrmTime;
  if(ddTimeDiff>(ONE_SECOND*10) || ddTimeDiff<-(ONE_SECOND*2)) {
    AudClockSetStrmTime(pDevice,ddPresentationTime-ONE_SECOND);
    fFixed=1;
  }
  return fFixed;
}

ULONGLONG AudGetSystemTime() {
	ULONGLONG ticks=0;
	ULONGLONG rate=1;
	ticks = (ULONGLONG)KeQueryPerformanceCounter((PLARGE_INTEGER)&rate).QuadPart;
	//
	// convert from ticks to 100ns clock
	//
	ticks = (ticks & 0xFFFFFFFF00000000) / rate * 10000000 +
			(ticks & 0xFFFFFFFF) * 10000000 / rate;
	return(ticks);
}


BOOL AudClockGetStrmTime(PHW_TIME_CONTEXT pTime) {
  PDEVICE_EXTENSION pDevice= (PDEVICE_EXTENSION)pTime->HwDeviceExtension;
  PSTREAM_EXTENSION pStream= (PSTREAM_EXTENSION)pTime->HwStreamObject->HwStreamExtension;
  MIDASCMD Cmd;
  LONGLONG ddTime;

//  struct audio_struct *pAudio= &pStream->u.audio;
//  KeQuerySystemTime((LARGE_INTEGER*)&pTime->SystemTime);
//  pAudio->CurrentTime= pTime->SystemTime;
  Cmd.hMidas=pDevice->hMidas;
  Cmd.cmd = MDS_GET_STREAM_TIME;
  MidasEntry(&Cmd);

  ddTime=Cmd.u.GetTime.CurTime;

  if(ddTime < pDevice->ddLastStreamTime)
    ddTime=pDevice->ddLastStreamTime;

#if(0)
  if(!pStream->Rate)
    pStream->Rate= AUD_MAX_RATE;
  lRate=pStream->Rate;
  if(lRate<0)
    lRate=-lRate;
  ddTime=Cmd.u.Time;
  if(pStream->stream_state==KSSTATE_RUN) {
    if(ddTime<=pDevice->ddLastStreamTime) {
      ddElapsedTime=pTime->SystemTime-pDevice->ddLastSystemTime;
      if(lRate)
        ddTime+=ddElapsedTime*10000/lRate;
    }
    else
      pDevice->ddLastSystemTime=pTime->SystemTime;
  }
#endif
  pDevice->ddLastStreamTime=ddTime;
  pTime->Time=ddTime;
  pTime->SystemTime=AudGetSystemTime();
  return 1;
}

void AudClockSetStrmTime(PDEVICE_EXTENSION pDevice, ULONGLONG Time) {
  MIDASCMD Cmd;
  Cmd.hMidas=pDevice->hMidas;
  Cmd.cmd = MDS_SET_STREAM_TIME;
  Cmd.u.SetTime.NewTime= Time;
  Cmd.u.SetTime.OldTime= 0;
  MidasEntry(&Cmd);
  pDevice->ddLastStreamTime=Time;
  return;
}

static DWORD AudClockRead(PHW_TIME_CONTEXT pTime) {
  return(0);
}

static DWORD AudClockSet(PHW_TIME_CONTEXT pTime) {
  return(0);
}

static DWORD AudClockUnk(PHW_TIME_CONTEXT pTime) {
  return(0);
}

/*************************************************************************/
/** UpdateClockEvents ()                                                **/
/**     handle any time events                                          **/
/*************************************************************************/
void UpdateClockEvents(struct _KDPC *Dpc, PDEVICE_EXTENSION pDevice, PVOID arg1, PVOID arg2) {
  PKSEVENT_ENTRY pEvent;
  PHW_STREAM_OBJECT pStreamObj;
  PSTREAM_EXTENSION pStream;
  struct audio_struct *audio;
  PWORKTIME pTim;
  ULONGLONG MinIntTime;
  ULONGLONG strmTime;
  BOOL Events;
  pStreamObj = pDevice->pStream[InMpeg2Ac3];
  if(pStreamObj) {
    pStream = (PSTREAM_EXTENSION)pStreamObj->HwStreamExtension;
    audio = &pStream->u.audio;
    strmTime = GetSystemTime();
    // loop through all time_mark events
    if (audio->TimeMark) {
      pEvent= NULL;
      for(Events=1;Events;) {
        pEvent= StreamClassGetNextEvent(pDevice,pStreamObj,(GUID *)&KSEVENTSETID_Clock,KSEVENT_CLOCK_POSITION_MARK,pEvent);
        if(!pEvent) {
          Events= 0;
        } else {
          if ((ULONGLONG)((PKSEVENT_TIME_MARK)(pEvent +1))->MarkTime <= strmTime ) {
            // signal the event here
            StreamClassStreamNotification(SignalStreamEvent, pStreamObj, pEvent);
            // tell the stream class to disable this event
            StreamClassStreamNotification(DeleteStreamEvent, pStreamObj, pEvent);
          }
	}
      }
    }
    // loop through all time_interval events
    if (audio->TimeInterval) {
      pEvent = NULL;
      for(Events=1;Events;) {
        pEvent= StreamClassGetNextEvent(pDevice,pStreamObj,(GUID*)&KSEVENTSETID_Clock,KSEVENT_CLOCK_INTERVAL_MARK,pEvent);
        if(!pEvent) {
          Events= 0;
        } else {
          // check if this event has been used for this interval yet
          pTim = ((PWORKTIME)(pEvent + 1));
          if(pTim->tim.Interval) {
            if ((ULONGLONG)pTim->tim.TimeBase <= strmTime) {
	      MinIntTime = (strmTime - pTim->tim.TimeBase) / pTim->tim.Interval;
	      MinIntTime *= pTim->tim.Interval;
	      MinIntTime +=  pTim->tim.TimeBase;
              if (MinIntTime > (ULONGLONG)pTim->LastTime  ) {
    	        // signal the event here
	        pTim->LastTime = strmTime;
		StreamClassStreamNotification(SignalStreamEvent, pStreamObj, pEvent);
              }
            }
          }
        }
      }
    }
  }
  //RAVI send_data(pDevice);
}

static void AudProposeDataFmt(SRB* pSrb) {
  PKSDATAFORMAT pKsDataFmt= pSrb->CommandData.OpenFormat;
  WAVEFORMATEX *pBlock = (WAVEFORMATEX *)((ULONG)pKsDataFmt+sizeof(KSDATAFORMAT));
  DbgMsg(DBG_MSGAUD,"AudControl: Proposing New Format \r\n");
  DbgDispKsDataFormat(DBG_MSGAUD,pSrb->CommandData.OpenFormat);
  DbgMsg(DBG_TODO,"Accepting the New Audio Format ...? \r\n");
  pSrb->Status= STATUS_SUCCESS;
}

static void AudSetStreamRate(SRB* pSrb) {
  pSrb->Status= STATUS_SUCCESS;
  DbgMsg(DBG_TODO,"AudControl: Set Stream Rate \r\n");
}

GUID MPEG_AudioSubType= {STATIC_KSDATAFORMAT_SUBTYPE_MPEG2_AUDIO};

void ProcessAudioFormat(PKSDATAFORMAT pFmt,SRB *pSrb) {

  long Err= 0;
  struct _HW_STREAM_OBJECT *pStreamObj= pSrb->StreamObject;
  PDEVICE_EXTENSION pDevice= (void*)pSrb->HwDeviceExtension;
  PSTREAM_EXTENSION pStream= (void*)pStreamObj->HwStreamExtension;
  WAVEFORMATEX *pBlock=0;
  MIDASCMD Cmd;

  if(pFmt->FormatSize>=sizeof(KSDATAFORMAT)+sizeof(WAVEFORMATEX))
    pBlock = (WAVEFORMATEX *)((ULONG)pFmt+sizeof(KSDATAFORMAT));

  if(pBlock)
    pStream->u.audio.AudioSampleRate= pBlock->nSamplesPerSec;
  
  if(pBlock)
    Cmd.u.InitAudio.AudioNumChannels= (unsigned char)pBlock->nChannels;
  else
    Cmd.u.InitAudio.AudioNumChannels= 2;
  
  if(!pStream->u.audio.AudioOutputMode) {
    pStream->u.audio.AudioOutputMode= MDS_DAC;
    DbgMsg(DBG_MSGAUD,"Audio Outpur mode is NOT yet set, setting up DAC as default\r\n");
  }
  Cmd.cmd = MDS_INITIALIZEAUDIO;
  Cmd.hMidas=pDevice->hMidas;
  Cmd.u.InitAudio.AudioOutputMode = pStream->u.audio.AudioOutputMode;
  Cmd.u.InitAudio.AudioSampleRate = pStream->u.audio.AudioSampleRate;
  Cmd.u.InitAudio.AudioSource= MDS_AS_MPEG;

  if( ( (IsEqualGUID2(&(pFmt->MajorFormat), &(hwfmtiMpeg2Aud.DataFormat.MajorFormat))) ||
        (IsEqualGUID2(&(pFmt->MajorFormat), &(hwfmtiMpeg2AudPES.DataFormat.MajorFormat))) ) &&
      (IsEqualGUID2(&(pFmt->SubFormat), &(hwfmtiMpeg2Aud.DataFormat.SubFormat))) ) {
    DbgMsg(DBG_MSGAUD," Setup Audio Decoder for AC3 \r\n");
    Cmd.u.InitAudio.AudioSource= MDS_AS_AC3;
  } else if( (IsEqualGUID2(&(pFmt->MajorFormat), &(hwfmtiLPCMAud.DataFormat.MajorFormat)) &&
                IsEqualGUID2(&(pFmt->SubFormat), &(hwfmtiLPCMAud.DataFormat.SubFormat))) ) {
    DbgMsg(DBG_MSGAUD,"Setup Audio Decoder for LPCM \r\n");
    pStream->u.audio.AudioSampleRate= pBlock->nSamplesPerSec;
    Cmd.u.InitAudio.AudioSource= MDS_AS_LPCM;
  } else if( ( (IsEqualGUID2(&(pFmt->MajorFormat), &(hwfmtiLPCMAud.DataFormat.MajorFormat)) &&
                IsEqualGUID2(&(pFmt->SubFormat), &(MPEG_AudioSubType))) ) ||
             ( (IsEqualGUID2(&(pFmt->MajorFormat), &(hwfmtiMpeg2AudPESMpeg2.DataFormat.MajorFormat)) &&
                IsEqualGUID2(&(pFmt->SubFormat), &(hwfmtiMpeg2AudPESMpeg2.DataFormat.SubFormat))) ) ||
             ( (IsEqualGUID2(&(pFmt->MajorFormat), &(InMpeg1Aud_Fmt0X.DataFormat.MajorFormat)) &&
                IsEqualGUID2(&(pFmt->SubFormat), &(InMpeg1Aud_Fmt0X.DataFormat.SubFormat))) ) ) {
    DbgMsg(DBG_MSGAUD,"Setup Audio Decoder for MPEG \r\n");
    pStream->u.audio.AudioSampleRate= pBlock->nSamplesPerSec;
    Cmd.u.InitAudio.AudioSource= MDS_AS_MPEG;
  } else {
    DbgDispKsDataFormat(DBG_MSGAUD,pFmt);
    DbgMsg(DBG_MSGAUD,"Unrecognised Audio Format \r\n");
  }
  Err= MidasEntry(&Cmd);
  if(!Err)
    pStream->u.audio.AudioSource=Cmd.u.InitAudio.AudioSource;

/*
#if(0)
    if( ( (IsEqualGUID2(&(pFmt->MajorFormat), &(hwfmtiMpeg2Aud.DataFormat.MajorFormat))) ||
        (IsEqualGUID2(&(pFmt->MajorFormat), &(hwfmtiMpeg2AudPES.DataFormat.MajorFormat))) ) &&
      (IsEqualGUID2(&(pFmt->SubFormat), &(hwfmtiMpeg2Aud.DataFormat.SubFormat))) ) {
    DbgMsg(DBG_MSGAUD," Setup Audio Decoder for AC3 \r\n");
    Cmd.u.InitAudio.AudioSource= MDS_AS_AC3;
    if(pFmt->FormatSize==sizeof(KSDATAFORMAT)+sizeof(WAVEFORMATEX))
      Cmd.u.InitAudio.lpLPCMFmt= pBlock;
    else
      Cmd.u.InitAudio.lpLPCMFmt= 0;
    Err= MidasEntry(&Cmd);
    if(!Err)
      pStream->u.audio.AudioSource= MDS_AS_AC3;

  } else if( (IsEqualGUID2(&(pFmt->MajorFormat), &(hwfmtiLPCMAud.DataFormat.MajorFormat)) &&
                IsEqualGUID2(&(pFmt->SubFormat), &(hwfmtiLPCMAud.DataFormat.SubFormat))) ) {
    DbgMsg(DBG_MSGAUD,"Setup Audio Decoder for LPCM \r\n");
    pStream->u.audio.AudioSampleRate= pBlock->nSamplesPerSec;
    Cmd.u.InitAudio.AudioSource= MDS_AS_LPCM;
    Cmd.u.InitAudio.AudioOutputMode = pStream->u.audio.AudioOutputMode;
    if(pFmt->FormatSize==sizeof(KSDATAFORMAT)+sizeof(WAVEFORMATEX))
      Cmd.u.InitAudio.lpLPCMFmt= pBlock;
    else
      Cmd.u.InitAudio.lpLPCMFmt= 0;
    Err= MidasEntry(&Cmd);
    if(!Err)
      pStream->u.audio.AudioSource= MDS_AS_LPCM;
  } else if( ( (IsEqualGUID2(&(pFmt->MajorFormat), &(hwfmtiLPCMAud.DataFormat.MajorFormat)) &&
                IsEqualGUID2(&(pFmt->SubFormat), &(MPEG_AudioSubType))) ) ||
             ( (IsEqualGUID2(&(pFmt->MajorFormat), &(hwfmtiMpeg2AudPESMpeg2.DataFormat.MajorFormat)) &&
                IsEqualGUID2(&(pFmt->SubFormat), &(hwfmtiMpeg2AudPESMpeg2.DataFormat.SubFormat))) ) ) {
    DbgMsg(DBG_MSGAUD,"Setup Audio Decoder for MPEG \r\n");
    pStream->u.audio.AudioSampleRate= pBlock->nSamplesPerSec;
    Cmd.u.InitAudio.AudioSource= MDS_AS_MPEG;
    Cmd.u.InitAudio.AudioOutputMode = pStream->u.audio.AudioOutputMode;
    if(pFmt->FormatSize==sizeof(KSDATAFORMAT)+sizeof(WAVEFORMATEX))
      Cmd.u.InitAudio.lpLPCMFmt= pBlock;
    else
      Cmd.u.InitAudio.lpLPCMFmt= 0;
    Err= MidasEntry(&Cmd);
    if(!Err)
      pStream->u.audio.AudioSource= MDS_AS_MPEG;
  } else if( (IsEqualGUID2(&(pFmt->MajorFormat), &(InMpeg1Aud_Fmt0X.DataFormat.MajorFormat)) &&
                IsEqualGUID2(&(pFmt->SubFormat), &(InMpeg1Aud_Fmt0X.DataFormat.SubFormat))) ) {
    //MPEG1 Audio.
    DbgMsg(DBG_MSGAUD,"Setup Audio Decoder for MPEG1 \r\n");
    pStream->u.audio.AudioSampleRate= pBlock->nSamplesPerSec;
    Cmd.u.InitAudio.AudioSource= MDS_AS_MPEG;
    Cmd.u.InitAudio.AudioOutputMode = pStream->u.audio.AudioOutputMode;
    if(pFmt->FormatSize==sizeof(KSDATAFORMAT)+sizeof(WAVEFORMATEX))
      Cmd.u.InitAudio.lpLPCMFmt= pBlock;
    else
      Cmd.u.InitAudio.lpLPCMFmt= 0;
    Err= MidasEntry(&Cmd);
  } else {
    DbgDispKsDataFormat(DBG_MSGAUD,pFmt);
    DbgMsg(DBG_MSGAUD,"Unrecognised Audio Format \r\n");
  }
#endif
*/
  DbgMsg(DBG_MSGAUD,"Process Audio Format (Err= 0x%08x)\r\n",Err);

  return ;
}

/**EOF: AUDIO.C**/
