#define WANTVXDWRAPS 1

#include <WinDef.h>
#include <VMM.h>
#include <VxdWraps.h>
#include <ConfigMg.h>

#include "DbgMsg.h"
#include "DbgFltr.h"
#include "Settings.h"

#include "csgatew.h"
#include "cardserv.h"
#include "margipnp.h"
#include "cardsrvc.h"
#include "util.h"
#include "regkeys.h"
#include "cardid.h"
#include "retcodes.h"

#if(CS_VER_500)
#define CS_REVISION 0x0500
#else
#define CS_REVISION 0x0210
#endif

char ClientNameString[] = "MARGIPNP";
char VendorNameString[] = "MARGI Systems, Inc.";

// Global Device Structure
CLIENTREG_STRUCT gCardServiceClient={0};        // Single Card Services Registration
DEVICE_STRUCT    gCardDevice[NUM_SOCKETS]={0};  // One Device Structure for Each Socket
NOTIFY_S         gNotify[NUM_CALLBACKS]={0};    // One Notify Structure for each application
struct ClientInfo_s gClientInfo= {0};
DWORD gCSDevice = 0;  // VXD Device ID for Card Services being used
WORD gfAPM_Suspended=0;

static DWORD FindProvider(void);
static WORD CheckVersion(DWORD CardSrvcID);
static void InitClientInfo(struct ClientInfo_s *pInfo);
static int RegisterClient(CLIENTREG_STRUCT *pClnt);
static int DeRegisterClient(CLIENTREG_STRUCT *pClnt);
static int SendReqSystemSoft(int iFunc, WORD *pHdl, void *pPtr, int iArgLen, void *pArgPtr);
static int SendReqPCCard(int iFunc, WORD *pHdl, void *pPtr, int iArgLen, void *pArgPtr);
static int SendReqRealMode(int iFunc, WORD *pHdl, void *pPtr, int iArgLen, void *pArgPtr);
static int CS_EventHandler(int nFunction,WORD wLogSocket, BYTE* pBuff);
static int GetFirstTuple(WORD Handle, LOGICAL_SOCKET lSocket, BYTE bCode, TUPLE_PKT* pTuplePacket);
static int GetNextTuple(LOGICAL_SOCKET lSocket, BYTE bCode, TUPLE_PKT* pTuplePacket);
static int GetTupleData(TUPLE_DATA_PKT* pTupleData, BYTE bOffset, int nMax);
static DWORD GetCardCisId(WORD Handle, LOGICAL_SOCKET lSocket);
static int  RequestIO(PDEVICE pDev);
static int  ReleaseIO(PDEVICE pDev);
static int RequestIRQ(PDEVICE pDev);
static int  ReleaseIRQ(PDEVICE pDev);
static int RequestConfiguration(PDEVICE pDev, int fPwr);
static int ReleaseConfiguration(PDEVICE pDev);

// Flag indicated remove power when not in use
// Lowest power if this is 1 but some browsers won't see card
int gfPowerOff = 0;
static BYTE fConnected= 0;

int CardServiceConnect(void) {
  int Status,fOK= 0;
  if(fConnected) {
    fOK= 1;
  } else {
    InitDeviceList();
    if(!(gCSDevice= FindProvider())) {
      ErrorMessage(ERR_NO_CS);
    } else {
      if(fOK= CheckVersion(gCSDevice)) {
        InitClientInfo(&gClientInfo);
        Status= RegisterClient(&gCardServiceClient);
        if(Status!=R_SUCCESS) {
	  SettingSave(SET_STR,KEYVAL_ERROR,(DWORD)"Unable to register CardService client.");
          ErrorMessage(ERR_CS_VER);
        } else {
          fOK= 1;
          fConnected= 1;
        }
      }
    }
  }
  return(fOK);
}

int CardServiceDisconnect(void) {
  int Err= 0;
  if(fConnected) {
    DeRegisterClient(&gCardServiceClient);
    fConnected= 0;
  }
  return(Err);
}

// Figure out which Card Service Provider we should use
// Check for Systemsoft 1st, then PCCARD.386, then Real Mode Services
static DWORD FindProvider(void) {
  DWORD Status;
  DWORD CardSrvcID= SSCSGW_DEVICE_ID;
  VxDCall(SSCSGW_Get_Version);
  _asm mov [Status],eax
  if(!Status) {
    CardSrvcID= PCCARD_DEVICE_ID;
    VxDCall(PCCARD_Get_Version);
    _asm mov [Status],eax
    if(!Status) {
      CardSrvcID= REALMODE_CS_ID;
    }
  }
  return(CardSrvcID);
}

   /*
    *  Call GetCardServicesInfo to get the compliance version
    *  and number of logical sockets supported (a loop index).
    */
static WORD CheckVersion(DWORD CardSrvcID) {
  PCLIENT pClnt= &gCardServiceClient;
  DWORD Status;
  WORD fOK= 0;
  CSREQ Req;
  pClnt->csi.signature= 0;
  Req.iFunc= F_GET_CARD_SERVICES_INFO;
  Req.pHdl= 0;
  Req.pPtr= 0;
  Req.iArgLen= sizeof(CS_INFO_PKT);
  Req.pArgPtr= &pClnt->csi;
  Status= CardServiceSendRequest(&Req);
  if(Status!=R_SUCCESS || pClnt->csi.signature!=0x5343) {       /*CS*/
    if(CardSrvcID==REALMODE_CS_ID) {            //No CardServices available.
      SettingSave(SET_STR,KEYVAL_ERROR,(DWORD)"No CardServices available.");
      ErrorMessage(ERR_NO_CS);
    } else {                                    //CS returned an error.
      if(Status!=R_SUCCESS) {
        strcpy(TmpStr,"CardServices error ");
        AppendHexStr(TmpStr,Status);
        SettingSave(SET_STR,KEYVAL_ERROR,(DWORD)TmpStr);
      } else
        SettingSave(SET_STR,KEYVAL_ERROR,(DWORD)"CardServices bad signature.");
      ErrorMessage(ERR_CS_ERR);
    }
  } else if(pClnt->csi.cs_level<0x0500) {       //Must be version 5.00 or later.
    SettingSave(SET_STR,KEYVAL_ERROR,(DWORD)"Invalid CardServices version.");
    ErrorMessage(ERR_CS_VER);
  } else {
    fOK= 1;
  }
  return(fOK);
}

static void InitClientInfo(struct ClientInfo_s *pInfo) {
  WORD Len;
  pInfo->pkt.max_length = sizeof(gClientInfo);
  pInfo->pkt.attr.flags = CLTA_IO_DRIVER | CLTA_AI_EXCLUSIVE;
  pInfo->pkt.attr.info_subfunction = 0;
  pInfo->pkt.clt_revision = VXD_REVISION;
  pInfo->pkt.cs_level= CS_REVISION;
  pInfo->sub.RevDate = VXD_DATE;
  pInfo->sub.NstrOff = sizeof(pInfo->pkt) + sizeof(pInfo->sub);
  pInfo->sub.NstrLen = sizeof(ClientNameString);
  pInfo->sub.VstrOff = pInfo->sub.NstrOff + pInfo->sub.NstrLen;
  pInfo->sub.VstrLen = sizeof(VendorNameString);
  strcpy(pInfo->Strings,ClientNameString);
  strcpy(&pInfo->Strings[sizeof(ClientNameString)],VendorNameString);
  Len= sizeof(pInfo->pkt) + sizeof(pInfo->sub) + pInfo->sub.NstrLen + pInfo->sub.VstrLen;
  pInfo->pkt.info_length= Len;
}

static int RegisterClient(CLIENTREG_STRUCT *pClnt) {
  int Status;
  struct CSReqPkt_s ReqPkt;
  pClnt->ClientHandle= 0;
  pClnt->regpkt.attr.flags = CLTA_IO_DRIVER | CLTA_AI_SHARED;
  pClnt->regpkt.attr.info_subfunction = 0;
  // Fixing system soft in win 95 screws up phoenix in WIN 3.11, renable EVTM_PWR_MGMT_CHG 
  pClnt->regpkt.evt_mask = EVTM_CRD_DETECT_CHG|EVTM_PWR_MGMT_CHG;
  pClnt->regpkt.data[1] = 0;
  pClnt->regpkt.version = CS_REVISION;
  ReqPkt.iFunc= F_REGISTER_CLIENT;
  ReqPkt.pHdl= &pClnt->ClientHandle;
  ReqPkt.pPtr= MARGIPNP_Client_Callback;
  ReqPkt.iArgLen= sizeof(CLT_REGISTRATION_PKT);
  ReqPkt.pArgPtr= &pClnt->regpkt;
  Status= CardServiceSendRequest(&ReqPkt);
  return(Status);
}

static int DeRegisterClient(CLIENTREG_STRUCT *pClnt) {
  int Status;
  CSREQ Req;
  Req.iFunc= F_DEREGISTER_CLIENT;
  Req.pHdl= &pClnt->ClientHandle;
  Req.pPtr= 0;
  Req.iArgLen= 0;
  Req.pArgPtr= 0;
  if(pClnt->ClientHandle) {
    Status= CardServiceSendRequest(&Req);
    pClnt->ClientHandle= 0;
  }
  return(Status);
}

int _stdcall CardServiceSendRequest(PCSREQ pR) {
  int Ret;
  switch(gCSDevice) {
    case SSCSGW_DEVICE_ID: Ret= SendReqSystemSoft(pR->iFunc,pR->pHdl,pR->pPtr,pR->iArgLen,pR->pArgPtr); break;
    case PCCARD_DEVICE_ID: Ret= SendReqPCCard(pR->iFunc,pR->pHdl,pR->pPtr,pR->iArgLen,pR->pArgPtr); break;
    case REALMODE_CS_ID:   Ret= SendReqRealMode(pR->iFunc,pR->pHdl,pR->pPtr,pR->iArgLen,pR->pArgPtr); break;
    default:               Ret= -1;
  }
  return(Ret);
}

int _stdcall MARGIPNP_Client_Callback(void) {
  int nFunction;               // function
  WORD wLogSocket;             // socket on which event occured
  BYTE *pBuff;                 // buffer

  _asm {                     /* XXX: move to ASM wrapper in future */
        movzx  eax, al
        mov    nFunction, eax
        mov    wLogSocket, cx
        mov    pBuff, ebx
  }
  return(CS_EventHandler(nFunction,wLogSocket,pBuff));
}

static int SendReqSystemSoft(int iFunc, WORD *pHdl, void *pPtr, int iArgLen, void *pArgPtr) {
  WORD Status= 0;
  int iHdl;
  iHdl= pHdl ? *pHdl:0;
  if(!iArgLen)   // this fixes the hang on deregister
    pArgPtr= 0;
  _asm {
         push   ebx
         push   ecx
         push   edx
         push   esi
         mov    eax,[iFunc]
         mov    edx,[iHdl]
         mov    esi,[pPtr]
         mov    ecx,[iArgLen]
         mov    ebx,[pArgPtr]
  }
  VxDCall(SSCSGW_DoRequest);
  _asm {                        ; Card Services returns:
         mov    [Status],ax     ; ax = Status
         mov    [iHdl],edx
         pop    esi
         pop    edx
         pop    ecx
         pop    ebx
  }
  // copy the new value of the handle
  if(pHdl)
    *pHdl= iHdl;
  return(Status);
}

static int SendReqPCCard(int iFunc, WORD *pHdl, void *pPtr, int iArgLen, void *pArgPtr) {
  WORD Status= 0;
  int iHdl;
  iHdl= pHdl ? *pHdl:0;
  if(!iArgLen)  // this fixes the hang on deregister
    pArgPtr= 0;
  _asm {
        push    ebx
        push    ecx
        push    edx
        push    esi
                                ; standard Card Services bindings:
        mov     eax,[iFunc]
        mov     esi,[pPtr]
        mov     ecx,[iArgLen]
        mov     ebx,[pArgPtr]
  }
  VxDCall(PCCARD_Card_Services);
  _asm {                        ; Card Services returns:
        mov     [Status],ax     ; ax = Status
        mov     [iHdl],edx      ; dx = Handle
        pop     esi
        pop     edx
        pop     ecx
        pop     ebx
  }
   // copy the new value of the handle
   if(pHdl)
     *pHdl= iHdl;
   return(Status);
}

static int SendReqRealMode(int iFunc, WORD *pHdl, void *pPtr, int iArgLen, void *pArgPtr) {
  return RealModeDoReq(iFunc, pHdl, pPtr, iArgLen, pArgPtr);
}

/*************************************************************************
*
*   Procedure:
*       CS_EventHandler
*
*   Purpose:
*       Card Services client callback handler.
*
\************************************************************************/
static int EventRegComplete(void);
static int EventInsertion(PDEVICE pDev, WORD wLogSocket);
static int EventRemoval(PDEVICE pDev, WORD wLogSocket);
static int EventClientInfo(PDEVICE pDev, CLT_INFO_PKT *pBuff);

static int CS_EventHandler(int nFunction,WORD wLogSocket, BYTE* pBuff) {
  PDEVICE pDev;
  int Status= R_SUCCESS;

  wLogSocket&=0x00ff;
  if(wLogSocket<NUM_SOCKETS)
  {

    pDev= &gCardDevice[wLogSocket];
    pDev->l_socket= (LOGICAL_SOCKET)wLogSocket;
    pDev->ClientHandle= gCardServiceClient.ClientHandle;

    // ensure system calls not made from Util.c
    EnterUnsafe();

    // don't respond if Config Manager APM has suspended
    // this filters all the CARD_REMOVAL messages after APM_Suspend
    if(!gfAPM_Suspended) {
      switch (nFunction) {
        case EV_REGISTRATION_COMPLETE:    Status= EventRegComplete(); break;
        case EV_PM_RESUME:
        case EV_CARD_INSERTION:           Status= EventInsertion(pDev,wLogSocket); break;
        case EV_PM_SUSPEND:
        case EV_CARD_REMOVAL:             Status= EventRemoval(pDev,wLogSocket); break;
        // To react/interact with the events below, add code underneath the
        // appropriate event constant
        case EV_CLIENT_INFO:              Status= EventClientInfo(pDev,(CLT_INFO_PKT*)pBuff); break;
        case EV_EXCLUSIVE_COMPLETE:
        case EV_EXCLUSIVE_REQUEST:
        //case EV_PM_RESUME:
        //case EV_PM_SUSPEND:
        case EV_TIMER_EXPIRED:
        case EV_BATTERY_DEAD:
        case EV_BATTERY_LOW:              break;
      }
    }
  }
  ExitUnsafe();
  return(Status);
}

// CS has completed the registration process
// Be carefull here the EV_CARD_INSERTION may occur before this message
static int EventRegComplete(void) {
  DbgMsg(DBG_CSEVNT,"CardSrvc: Registration complete.\r\n");
  return(R_SUCCESS);
}

// Read card ID from newly inserted PC-Card
static int EventInsertion(PDEVICE pDev, WORD wLogSocket) {
  int CardId,Status;
  CNFG_INFO_PKT cfgInfo;
  CSREQ Req;
  Status= R_SUCCESS;
  SettingSave(SET_STR,KEYVAL_CURSTATE,(DWORD)KEYVAL_STATE_INSERTIONSTART);

  IdentifyHostMachine(pDev);

//  DbgBox("EventInsertion()");         //Causes system to hang
  CardId= GetCardCisId(pDev->ClientHandle,wLogSocket);

  // Get Card Functional Type
  // GetCardType returns non-zero if it is a recognized type
  pDev->CardType= GetCardTypeFromCis(CardId);
  // Only respond to insertion of Recognized Functional Type
  if(pDev->CardType) {
    // Get Card Config Info (and save the VCC operating voltage)
    cfgInfo.l_socket= pDev->l_socket;
    Req.iFunc= F_GET_CONFIGURATION_INFO;
    Req.pHdl= &pDev->cfgHandle;
    Req.pPtr= 0;
    Req.iArgLen= sizeof(cfgInfo);
    Req.pArgPtr= &cfgInfo;
    Status= CardServiceSendRequest(&Req);
    if(Status==R_SUCCESS)
      pDev->vccOn= cfgInfo.vcc;
    else
      pDev->vccOn= 50;  //Default to 5 volts
    // Only request configuration if we are not already inserted
    // currently only 1 Margi card at a time can be inserted
    if(pDev->State!=S_INSERTED) {
      pDev->State= S_INSERTED;
      // Configure the card with no i/o,irq or power
      // GetResources call will reconfigure.
#if PERM_RESOURCE_ALLOC
      Status= ConfigureCard(pDev,1,1,0);
#else
      Status= ConfigureCard(pDev,0,0,0);
#endif
      if(Status==R_SUCCESS) {
        pDev->Error= 0;
        SettingSave(SET_STR,KEYVAL_CURSTATE,(DWORD)KEYVAL_STATE_INSERTIONDONE);
        SettingSave(SET_DWORD,KEYVAL_STATE_INSERTIONDONE,1);
        //Broadcast message to all registered callbacks
        NotifyCallback(0,pDev->State,pDev->l_socket,pDev->CardType);
      } else {
        pDev->Error= S_ERROR+Status;
        SettingSave(SET_STR,KEYVAL_CURSTATE,(DWORD)KEYVAL_STATE_INSERTIONFAILED);
        //Broadcast message to all registered callbacks
        NotifyCallback(0,pDev->Error,pDev->l_socket,pDev->CardType);
      }
    }
  }
  return(Status);
}

static int EventRemoval(PDEVICE pDev, WORD wLogSocket) {
  //DPFC( "Card Removal.\n" );
  if(pDev->l_socket==(LOGICAL_SOCKET)wLogSocket && pDev->State==S_INSERTED) {
    pDev->State= S_EXTRACTED;
    ReleaseCard(pDev);
    //Broadcast message to all registered callbacks
    NotifyCallback(0,pDev->State,pDev->l_socket,pDev->CardType);
    pDev->CardType= 0;
    SettingSave(SET_STR,KEYVAL_CURSTATE,(DWORD)KEYVAL_STATE_REMOVED);
  }
  return(R_SUCCESS);
}

static int EventClientInfo(PDEVICE pDev, CLT_INFO_PKT *pBuff) {
  int nBytes;
  if(pBuff) {
    nBytes = min(pBuff->max_length,gClientInfo.pkt.info_length);
    pBuff->info_length = nBytes;
    memcpy(&pBuff->attr,&gClientInfo.pkt.attr,nBytes-4);
  }
  return(R_SUCCESS);
}


///////////////////////////////////////////////////////////////////////////////
//  GetFirstTuple
//
//  wSocket = Logical socket
//
int GetFirstTuple(WORD Handle, LOGICAL_SOCKET lSocket, BYTE bCode, TUPLE_PKT* pTuplePacket) {
  CSREQ Req;
  int Status;
  pTuplePacket->l_socket = lSocket;
  pTuplePacket->attr = 0;
  pTuplePacket->desired_tuple_code = bCode;
  pTuplePacket->reserved = 0;
  Req.iFunc= F_GET_FIRST_TUPLE;
  Req.pHdl= &Handle;
  Req.pPtr= 0;
  Req.iArgLen= sizeof(TUPLE_PKT);
  Req.pArgPtr= pTuplePacket;
  Status= CardServiceSendRequest(&Req);
  return(Status);
}

///////////////////////////////////////////////////////////////////////////////
//  GetNextTuple
//
//  wSocket = Logical socket
//
int GetNextTuple(LOGICAL_SOCKET lSocket, BYTE bCode, TUPLE_PKT* pTuplePacket) {
  int Status;
  CSREQ Req;
  pTuplePacket->l_socket = lSocket;
  pTuplePacket->attr = 0;
  pTuplePacket->desired_tuple_code = bCode;
  pTuplePacket->reserved = 0;
  Req.iFunc= F_GET_NEXT_TUPLE;
  Req.pHdl= 0;
  Req.pPtr= 0;
  Req.iArgLen= sizeof(TUPLE_PKT);
  Req.pArgPtr= pTuplePacket;
  Status= CardServiceSendRequest(&Req);
  return(Status);
}

///////////////////////////////////////////////////////////////////////////////
//  GetTupleData
//
//  wSocket = Logical socket
//
int GetTupleData(TUPLE_DATA_PKT* pTupleData, BYTE bOffset, int nMax) {
  int Status;
  CSREQ Req;
  pTupleData->tuple_offset = bOffset;
  pTupleData->tuple_data_max = nMax;
  Req.iFunc= F_GET_TUPLE_DATA;
  Req.pHdl= 0;
  Req.pPtr= 0;
  Req.iArgLen= sizeof(TUPLE_DATA_PKT)+nMax;
  Req.pArgPtr= pTupleData;
  Status= CardServiceSendRequest(&Req);
  return(Status);
}

///////////////////////////////////////////////////////////////////////////////
//  GetCardId:    Return Manuf. Id and Func Id of card.
//
DWORD GetCardCisId(WORD Handle, LOGICAL_SOCKET lSocket) {
  int Status;
  union {
    TUPLE_PKT         Tuple;
    struct {
      TUPLE_DATA_PKT    Hdr;
      BYTE              Buff[128];
    } Data;
  } pkt;
  Status= GetFirstTuple(Handle,lSocket,0x20,&pkt.Tuple);
  if(Status==R_SUCCESS) {
    Status= GetTupleData(&pkt.Data.Hdr,0,sizeof(pkt.Data.Buff));
    if(Status==R_SUCCESS) {
      WORD wM, wF;
      wM = pkt.Data.Buff[0]
         + pkt.Data.Buff[1]*256;
      wF = pkt.Data.Buff[2]
         + pkt.Data.Buff[3]*256;
      return(((DWORD)wM << 16) | wF);
    }
  }
  return MAKEU32(MARGI_ID,DVDTOGO_ID);
  return(0);
}

///////////////////////////////////////////////////////////////////////////////
//  RequestIO:  Get a I/O window from CS
//
//
int  RequestIO(PDEVICE pDev) {
  int Status;
  CSREQ Req;
  DbgMsg(DBG_CSREQ,"CardSrvc: RequestIO...\r\n");
  memset(&(pDev->io),0,sizeof(pDev->io));
  pDev->io.l_socket = pDev->l_socket;
  pDev->io.io_parms[0].base_port = 0;
  pDev->io.io_parms[0].num_ports = 16;
  pDev->io.io_parms[0].attr = IOA_16BIT;
  pDev->io.io_addr_lines = 4;
  pDev->ioHandle = pDev->ClientHandle;
  Req.iFunc= F_REQUEST_IO;
  Req.pHdl= &pDev->ioHandle;
  Req.pPtr= 0;
  Req.iArgLen= sizeof(pDev->io);
  Req.pArgPtr= &pDev->io;
  Status= CardServiceSendRequest(&Req);
  if(Status!=R_SUCCESS)
    pDev->ioHandle= 0;
  DbgMsg(DBG_CSREQ,"CardSrvc: RequestIO= %d.\r\n",Status);
  return(Status);
}

///////////////////////////////////////////////////////////////////////////////
//  ReleaseIO:  Release an I/O window from CS
//
//
int  ReleaseIO(PDEVICE pDev) {
  int Status= R_SUCCESS;
  CSREQ Req;
  DbgMsg(DBG_CSREQ,"CardSrvc: ReleaseIO...\r\n");
  if(pDev->ioHandle) {
    Req.iFunc= F_RELEASE_IO;
    Req.pHdl= &pDev->ioHandle;
    Req.pPtr= 0;
    Req.iArgLen= sizeof(pDev->io);
    Req.pArgPtr= &pDev->io;
    Status= CardServiceSendRequest(&Req);
    pDev->ioHandle=0;
  }
  return(Status);
}

///////////////////////////////////////////////////////////////////////////////
//  RequestIRQ:  Get a IRQ channel from CS
//
//  wSocket = Logical socket to which this window will be associated
//
int RequestIRQ(PDEVICE pDev) {
  int Status;
  CSREQ Req;
  DbgMsg(DBG_CSREQ,"CardSrvc: RequestIRQ...\r\n");
  memset(&(pDev->irq),0,sizeof(pDev->irq));
  pDev->irq.l_socket = pDev->l_socket;
  pDev->irq.attr = 0;
  pDev->irq.assigned_irq = 0;
  pDev->irq.info1 = IRQI_MASK_FLAG | IRQI_LEVEL | IRQI_VEND;
  pDev->irq.info2 = 0xfffe;
  pDev->irqHandle = pDev->ClientHandle;
  Req.iFunc= F_REQUEST_IRQ;
  Req.pHdl= &pDev->irqHandle;
  Req.pPtr= 0;
  Req.iArgLen= sizeof(pDev->irq);
  Req.pArgPtr= &pDev->irq;
  Status= CardServiceSendRequest(&Req);
  if(Status!=R_SUCCESS )
    pDev->irqHandle= 0;
  DbgMsg(DBG_CSREQ,"CardSrvc: RequestIRQ= %d.\r\n",Status);
  return(Status);
}

///////////////////////////////////////////////////////////////////////////////
//  ReleaseIRQ:  Release IRQ channel from CS
//
//
int  ReleaseIRQ(PDEVICE pDev) {
  int Status= R_SUCCESS;
  CSREQ Req;
  DbgMsg(DBG_CSREQ,"CardSrvc: ReleaseIRQ...\r\n");
  if(pDev->irqHandle) {
    Req.iFunc= F_RELEASE_IRQ;
    Req.pHdl= &pDev->irqHandle;
    Req.pPtr= 0;
    Req.iArgLen= sizeof(IRQ_RELEASE_PKT);
    Req.pArgPtr= &pDev->irq;
    Status= CardServiceSendRequest(&Req);
    pDev->irqHandle= 0;
  }
  DbgMsg(DBG_CSREQ,"CardSrvc: ReleaseIRQ= %d\n",Status);
  return(Status);
}

///////////////////////////////////////////////////////////////////////////////
//  RequestConfiguration
//
int RequestConfiguration(PDEVICE pDev, int fPwr) {
  int Status;
  CSREQ Req;
  DbgMsg(DBG_CSREQ,"CardSrvc: RequestConfig...\r\n");
  memset(&(pDev->cfg),0,sizeof(pDev->cfg));
  pDev->cfg.l_socket= pDev->l_socket;
  if(pDev->irqHandle)
    pDev->cfg.attr= CNFG_ATTR_ENABLE_IRQ;
  else
    pDev->cfg.attr= 0;

  // If fPwr then switch on power
  // else power down (only if gfPowerOff is true)
  if(fPwr)
    pDev->cfg.vcc= pDev->vccOn;
  else
    pDev->cfg.vcc= gfPowerOff ? 0:pDev->vccOn;

  pDev->cfg.present_flags= 0x01;     // Option reg is present
  pDev->cfg.cnfg_base= 0x03f0;       // Option reg base address

  if(pDev->ioHandle) {
#if(CS_VER_500)
    pDev->cfg.int_type= CNFG_INTTYPE_CUSTOM;  // Custom interface
    pDev->cfg.CustomIF_id_num= 0x0141;        // ZV Port interface
#else
    pDev->cfg.int_type= CNFG_INTTYPE_IO;
    pDev->cfg.CustomIF_id_num= 0;
#endif
    pDev->cfg.option= 0x01;         // Option reg set to 1
  } else {
    pDev->cfg.int_type= CNFG_INTTYPE_MEMORY_ONLY; // If I/O not requested
     pDev->cfg.CustomIF_id_num= 0;         // set as straight mem card
     pDev->cfg.option= 0x00;
  }
  pDev->cfgHandle= pDev->ClientHandle;
  Req.iFunc= F_REQUEST_CONFIGURATION;
  Req.pHdl= &pDev->cfgHandle;
  Req.pPtr= 0;
  Req.iArgLen= sizeof(pDev->cfg);
  Req.pArgPtr= &pDev->cfg;
  Status= CardServiceSendRequest(&Req);
  if(Status!=R_SUCCESS)
    pDev->cfgHandle= 0;
  DbgMsg(DBG_CSREQ,"CardSrvc: RequestConfiguration= %d\n",Status);
  return(Status);
}

///////////////////////////////////////////////////////////////////////////////
//  ReleaseConfiguration
//
int ReleaseConfiguration(PDEVICE pDev) {
  int Status= R_SUCCESS;
  CSREQ Req;
  DbgMsg(DBG_CSREQ,"CardSrvc: ReleaseConfig...\r\n");
  if(pDev->cfgHandle) {
    Req.iFunc= F_RELEASE_CONFIGURATION;
    Req.pHdl= &pDev->cfgHandle;
    Req.pPtr= 0;
    Req.iArgLen= sizeof(pDev->cfg.l_socket);
    Req.pArgPtr= &pDev->cfg.l_socket;
    Status= CardServiceSendRequest(&Req);
    pDev->cfgHandle= 0;
  }
  DbgMsg(DBG_CSREQ,"CardSrvc: ReleaseConfiguration= %d\n",Status);
  return(Status);
}

void ReleaseCard(PDEVICE pDev) {
   ReleaseConfiguration(pDev);
   ReleaseIRQ(pDev);
   ReleaseIO(pDev);
}

int ConfigureCard(PDEVICE p, int fPwr, int fIO, int fIRQ) {
   int Status=0;

   // If card is configured and config is to be changed
   // then release configuration
   if(p->cfgHandle && (   fPwr!=(p->cfg.vcc!=0)
                       || fIO !=(p->ioHandle!=0)
                       || fIRQ!=(p->irqHandle!=0)) )
      ReleaseCard(p);


   if(!p->cfgHandle)
   {
     if(fIO || CONFIGURE_NO_IO)
     {
       if((Status=RequestIO(p))!=R_SUCCESS)
       {
         Status+=0x100;
       }

     if((Status==R_SUCCESS) && fIRQ)
       if((Status=RequestIRQ(p))!=R_SUCCESS)
       {
         Status+=0x200;
       }

     if((Status==R_SUCCESS) && fIO)
       if((Status=RequestConfiguration(p,fPwr))!=R_SUCCESS)
       {
         Status+=0x300;
       }

     if(Status!=R_SUCCESS)
     {
        // If one of the above requests failed then
        // ensure everything is released;
        ReleaseCard(p);
     }
     }
   }
   return Status;
}


/**EOF: CARDSRVC.C**/


