#include <StrmIni.h>
#include <mmsystem.h>
#include <ksmedia.h>
#include "DbgMsg.h"
#include "DbgFltr.h"
#include "Idents.h"
#include "Globals.h"
#include "MidasApi.h"
#include "Log.h"
#include "DbgOut.h"

//#pragma message("CC.c: Optimizer disabled.")
//#pragma optimize("",off)

static void CcSetState(SRB *pSrb);
static void CcGetState(SRB *pSrb);
static void CcGetProperty(SRB *pSrb);
static void CcGetPropConnection(SRB *pSrb);
static void CcGetPropConnAlloc(SRB *pSrb);
static void CcIndicateClock(SRB *pSrb);
static void CcGetPropUnk(SRB *pSrb);
static void CcProposeFormat(SRB* pSrb);
static void CcUnkControlCmd(SRB* pSrb);

void __stdcall CCReceive(SRB *pSrb) {
  PSTREAM_EXTENSION pStream= (void*)pSrb->StreamObject->HwStreamExtension; 
  logSrb("CCReceive",pSrb);
  DbgDispKsStreamHeaders(DBG_MSGCC,pSrb);
  if(pStream->stream_state==KSSTATE_STOP) {
    pSrb->Status= STATUS_CANCELLED;
    SignalStreamRequestCompleteReadyForNextData(pSrb);
  } else if(pSrb->Command==SRB_READ_DATA) {
    KIRQL Irql;  
    MIDASCMD Cmd;
    PDEVICE_EXTENSION pDevice= (PDEVICE_EXTENSION)pSrb->HwDeviceExtension;
    PMDS_DATA_BLOCK  pMdb=&((PSRB_EXTENSION)pSrb->SRBExtension)->Mdb;
   
    pMdb->nSize=sizeof(MDS_DATA_BLOCK);
    pMdb->dwStatus=0;
    pMdb->dwStrmType=pStream->internal_streamid;
    pMdb->dwTotalPackets=pSrb->NumberOfBuffers;
    pMdb->pPackets=(PMDS_DATA_PACKET)pSrb->CommandData.DataBufferArray;
    pMdb->pNextMdb=0;
    pMdb->dwDataFlags=0;
    pMdb->hRef=(void*)pSrb;
    pMdb->pfnCallback=StreamDataCallback;

    Cmd.hMidas=pDevice->hMidas;
    Cmd.cmd = MDS_QUEUEDATA;
    Cmd.wCurrStream=pStream->internal_streamid;
    Cmd.u.pMdb=pMdb;
    KeAcquireSpinLock(&pStream->streamExtensionLock, &Irql); 
    MidasEntry(&Cmd);
	  KeReleaseSpinLock(&pStream->streamExtensionLock, Irql);   
    SignalStreamReadyForNextData(pSrb);
  } else {
    pSrb->Status= STATUS_NOT_IMPLEMENTED;
    SignalStreamRequestCompleteReadyForNextData(pSrb);
  }
  return;
}

void __stdcall CCControl(SRB *pSrb) {
  PDEVICE_EXTENSION pDevice;
  PSTREAM_EXTENSION pStream;

  logSrb("CCControl", pSrb);
  pDevice = (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  pSrb->Status = STATUS_NOT_IMPLEMENTED;
  DbgMsg(DBG_MSGCC,"CCControl: Cmd %s \r\n",Ident(ID_STRM_REQ,pSrb->Command));

  // determine the type of packet.  All calls here use the pointers
  // to the individual stream functions initialised at open time
  switch (pSrb->Command) {
    case SRB_SET_STREAM_STATE:       CcSetState(pSrb); break;
    case SRB_GET_STREAM_STATE:       CcGetState(pSrb); break;
    case SRB_GET_STREAM_PROPERTY:    CcGetProperty(pSrb); break;
    case SRB_INDICATE_MASTER_CLOCK:  CcIndicateClock(pSrb); break;
    case SRB_PROPOSE_DATA_FORMAT:    CcProposeFormat(pSrb); break;
    default:                         CcUnkControlCmd(pSrb); break;
  }
  unlogSrb("CCControl", pSrb);
  if(pSrb->Status!=STATUS_PENDING) {
    SignalStreamRequestCompleteReadyForNextControl(pSrb);
  } else {
    DbgMsg(DBG_MSGCC,"Returning Status Pending %s\r\n",Ident(ID_STRM_REQ,pSrb->Command));
  }
  return;
}

long __stdcall CCEvent(PHW_EVENT_DESCRIPTOR pEventDescriptor) {
  long status= STATUS_SUCCESS;
  PSTREAM_EXTENSION pStrm= pEventDescriptor->StreamObject->HwStreamExtension;
  if(pEventDescriptor->Enable)
    pStrm->dwEventCount++;
  else
    pStrm->dwEventCount--;
  logEvent("CCEvent", pEventDescriptor);
  DbgMsg(DBG_MSGCC,"CC: Event Count %d, \r\n",pStrm->dwEventCount);
  return status;
}

static void CcUnkControlCmd(SRB* pSrb) {
  pSrb->Status= STATUS_NOT_IMPLEMENTED;
  DbgMsg(DBG_MSGCC,"Unknown CC Control Cmd%s\r\n",Ident(ID_STRM_REQ,pSrb->Command));
}

static void CcProposeFormat(SRB* pSrb) {
  DbgMsg(DBG_MSGCC,"CCPropose Format %s \r\n",Ident(ID_STRM_REQ,pSrb->Command));
}

// change the stream state (i.e.  play or pause)
static void CcSetState(SRB *pSrb) {
  PSTREAM_EXTENSION pStream= (void*)pSrb->StreamObject->HwStreamExtension;
  PDEVICE_EXTENSION pDevice= (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  StreamSetState(pStream,pSrb->CommandData.StreamState);
  pSrb->Status = STATUS_SUCCESS;
  if(pStream->stream_state==KSSTATE_ACQUIRE) {
    MIDASCMD Cmd;
    Cmd.hMidas=pDevice->hMidas;
    Cmd.cmd= MDS_CCRESET;
    MidasEntry(&Cmd);
  }
  DbgMsg(DBG_MSGCC,"CC: SetState to %s \r\n",Ident(ID_KS_STATE,pStream->stream_state));
  return;
}

// Return the stream state
static void CcGetState(SRB *pSrb) {
  PSTREAM_EXTENSION pStream= (void*)pSrb->StreamObject->HwStreamExtension;
  pSrb->CommandData.StreamState = pStream->stream_state;
  pSrb->Status = STATUS_SUCCESS;
  DbgMsg(DBG_MSGCC,"CC: GetState to %s \r\n",Ident(ID_KS_STATE,pStream->stream_state));
  return;
}

static void CcGetProperty(SRB *pSrb) {
  DWORD SetID= pSrb->CommandData.PropertyInfo->PropertySetID;
  DWORD MsgID= pSrb->CommandData.PropertyInfo->Property->Id;
  logGetProperty("CC", pSrb);
  if(SetID!=0) {
    DbgMsg(DBG_WRN,"CcGetProperty: SetID is not 0! (%u)\r\n",SetID);
  } else {
    switch(MsgID) {
      case KSPROPERTY_CONNECTION_STATE:             CcGetPropConnection(pSrb); break;
      case KSPROPERTY_CONNECTION_ALLOCATORFRAMING:  CcGetPropConnAlloc(pSrb); break;
      default:                                      CcGetPropUnk(pSrb); break;
    }
  }
  return;
}

static void CcGetPropUnk(SRB *pSrb) {
  pSrb->Status= STATUS_NOT_IMPLEMENTED;
  DbgMsg(DBG_MSGCC,"CC: Unknown GetProperty Message \r\n");
  return;
}

static void CcGetPropConnAlloc(SRB *pSrb) {
  PKSALLOCATOR_FRAMING pfrm= (void*)pSrb->CommandData.PropertyInfo->PropertyInfo;
  pfrm->OptionsFlags = 0; //KSALLOCATOR_OPTIONF_SYSTEM_MEMORY;
  pfrm->PoolType = NonPagedPool;
  pfrm->Frames = 10;
  pfrm->FrameSize = 200;
  pfrm->FileAlignment = 0;
  pfrm->Reserved = 0;
  pSrb->ActualBytesTransferred = sizeof(KSALLOCATOR_FRAMING);
  pSrb->Status = STATUS_SUCCESS;
  DbgMsg(DBG_MSGCC,"CC: GetPropConnAlloc. Err=0x%08x \r\n",pSrb->Status);
  return;
}

static void CcIndicateClock(SRB *pSrb) {
  PSTREAM_EXTENSION pStream= (void*)pSrb->StreamObject->HwStreamExtension;
  pStream->MasterClock = pSrb->CommandData.MasterClockHandle;
  pSrb->Status = STATUS_SUCCESS;
  return;
}

static void CcGetPropConnection(SRB *pSrb) {
  PSTREAM_EXTENSION pStream= (void*)pSrb->StreamObject->HwStreamExtension;
  *(PKSSTATE)(pSrb->CommandData.PropertyInfo->PropertyInfo)= pStream->stream_state;
  pSrb->ActualBytesTransferred= sizeof (KSSTATE);                    
  if(pStream->stream_state == KSSTATE_PAUSE)
    pSrb->Status= STATUS_NO_DATA_DETECTED;
  else
    pSrb->Status= STATUS_SUCCESS;
  pSrb->Status= STATUS_SUCCESS;
  DbgMsg(DBG_MSGCC,"CC: GetPropConn. State=%s, Err=0x%08x \r\n",Ident(ID_KS_STATE,pStream->stream_state),pSrb->Status);
  return;
}

void CcClearClockEvents(PDEVICE_EXTENSION pDev) {
  LONGLONG MinIntTime;
  PWORKTIME pTim;
  PKSEVENT_ENTRY pEvent, pLast;
//  MIDASCMD Cmd;
  HW_TIME_CONTEXT Time;
  PSTREAM_EXTENSION pAudStrmExt;
  PHW_STREAM_OBJECT pAudStrmObj;

  if(pDev && pDev->pStream[InMpeg2Ac3]) {
    pAudStrmObj= pDev->pStream[InMpeg2Ac3];
    pAudStrmExt= (PSTREAM_EXTENSION)(pDev->pStream[InMpeg2Ac3]->HwStreamExtension);
    if(pAudStrmExt->dwEventCount) {
#if 1
      Time.Function=TIME_GET_STREAM_TIME;
      Time.HwDeviceExtension= (void*)pDev;
      Time.HwStreamObject= (void*)pAudStrmObj;
      AudioClock(&Time);
#else
      Cmd.context = pDev + 1;
      Cmd.cmd = MDS_GET_STREAM_TIME;
      Cmd.u.pTime= &Time;
      MidasEntry(&Cmd);
#endif

      if(Time.Time) {

        pEvent = NULL;
        pLast = NULL;
        while((pEvent=StreamClassGetNextEvent(pDev,pAudStrmObj,
                (GUID*)&KSEVENTSETID_Clock,KSEVENT_CLOCK_POSITION_MARK,pLast))!=NULL) {
          DbgMsg(DBG_MSGCC,"ClockEvents - KSEVENT_CLOCK_POSITION_MARK in event queue 0x%08x\r\n",pEvent);
          //Dont want to signal the event when the time is greater than the current time		
          //as you would be signalling it in the future (Bill's reply to logic below)
          if(((PKSEVENT_TIME_MARK)(pEvent +1))->MarkTime <= (LONGLONG)Time.Time)  {
            StreamClassStreamNotification(SignalStreamEvent,pAudStrmObj,pEvent);// signal the event here
            //DbgMsg(DBG_MSGEVENT,"ClockEvents - KSEVENT_CLOCK_POSITION_MARK presentation tiem position mark has been passed\r\n");
            DbgMsg(DBG_MSGEVENT,"CLKEVE: Servicing the POSITION Event at");
            DbgDispStrmTime(DBG_MSGEVENT,((PKSEVENT_TIME_MARK)(pEvent +1))->MarkTime," Base=",0);
            DbgDispStrmTime(DBG_MSGEVENT,Time.Time," Interval(",")\r\n");
          }
          pLast = pEvent;
        }

        pEvent = NULL;
        pLast = NULL;
        while((pEvent=StreamClassGetNextEvent(pDev,pAudStrmObj,
              (GUID*)&KSEVENTSETID_Clock,KSEVENT_CLOCK_INTERVAL_MARK,pLast))!=NULL) {
          //DbgMsg(DBG_MSGEVENT,"ClockEvents - KSEVENT_CLOCK_INTERVAL_MARK - FOUND EVENT\r\n");
          pTim= ((PWORKTIME)(pEvent + 1));
          if(pTim && pTim->tim.Interval) {
            if(pTim->tim.TimeBase <= (LONGLONG)Time.Time) {
              MinIntTime = (Time.Time - pTim->tim.TimeBase) / pTim->tim.Interval;
              MinIntTime *= pTim->tim.Interval;
              MinIntTime +=  pTim->tim.TimeBase;
              if(MinIntTime > pTim->LastTime) {
                DbgMsg(DBG_MSGEVENT,"CLKEVE: Servicing INTERVAL Event:");
                DbgDispStrmTime(DBG_MSGEVENT,pTim->tim.TimeBase," Base:",0);
                DbgDispStrmTime(DBG_MSGEVENT,pTim->tim.Interval," Interval:","\r\n");
                StreamClassStreamNotification(SignalStreamEvent,pAudStrmObj,pEvent);// signal the event here
                pTim->LastTime = Time.Time;
              }
            }
          } else {
            //TRAP
          }
          pLast = pEvent;
        }
      }
    }
  }
  return;
}


/**EOF: CC.C**/
