#ifndef __MARGIPNP_H__
#define __MARGIPNP_H__ 0x0100

#pragma CM_PAGEABLE_DATA
#pragma CM_PAGEABLE_CODE

#define PERM_RESOURCE_ALLOC   1
#define CONFIGURE_NO_IO       1

// Macro Defines
#define DATE(m,d,y)  ((((y-80)&0x7f)<<9)|(m<<5)|d)
#define MAKEU32(h,l) (((DWORD)h<<16)|l)

#define VERMAJOR  0x02
#define VERMINOR  0x00
#define VXD_REVISION (VERMAJOR*256+VERMINOR)
#define VXD_DATE     DATE(3,04,98)

// SetCallback Error Codes
#define SC_ERR_NONE        0
#define SC_ERR_BADTYPE     0xfff0
#define SC_ERR_MAX         0xfff1

enum MACHINE_MANUF
{
  MM_UNKNOWN = 0,
  MM_COMPAQ,
  MM_TI,
  MM_FUJITSU,
  MM_TOSHIBA,
  MM_SANYO,
  MM_DIGITAL,
  MM_DIGITALULTRA,
  MM_PANASONIC_CF63,
  MM_MM104_6722
};

#define MT_UNKNOWN          MM_UNKNOWN
#define MT_COMPAQ_ALTAIR    (MM_COMPAQ|0x10000)
#define MT_COMPAQ_ARMADA    (MM_COMPAQ|0x20000)
#define MT_TI_TRAVELMATE    (MM_TI|0x10000)
#define MT_FUJITSU_CINNAMON (MM_FUJITSU|0x10000)
#define MT_FUJITSU_SAFFRON  (MM_FUJITSU|0x20000)
#define MT_FUJITSU_YELLOWSTONE (MM_FUJITSU|0x30000)

///////////////////////////////////////////////////////////////////////////////
// state machine states
enum STATE
    {
    S_INSERTED = 0x10,                     // card has been inserted
    S_EXTRACTED,                           // card has been extracted
    S_PENDING,                             // registration pending
    S_COMPLETE,                            // registration complete
    S_IDLE,
    S_ABORT,
    S_DONE,
    S_START,
    S_VXD_UNLOAD,
    S_APM_SUSPEND,
    S_ERROR    =0x8000,                    // err is ored with func and func error
};

// This driver maintains a Type Table that associates a Cards
// Manuf Id and Func Id with a Card Type.
typedef struct {
  DWORD CardId;
  WORD IoId;
  WORD Type;
} TYPE_STRUCT;

// The term "application" is used in this driver to specify
// a client to this driver.  In reality the application will be
// another driver or DLL but, since this driver is a Card Services
// Client, "application" is used to avoid confusion.

// Multiple applications may register callback window handles
// using the V86SetCallback. The application will pass
// a Callback structure which contains a function ptr, Card Type
// and a set of message values that are used during the
// callback.  A callback to all registered applications
// occurs when any recognized card is inserted or
// extracted.  The application must check wParam for the socket
// and lParam for the Type before responding to the callback.

// Callback struct is used by application to indicate function ptr
// and messages for notifying a card event.
// This driver maintains an array of these structures and notifies
// all registered callback when an event occurs.
typedef struct
{
  DWORD uFunc;
  DWORD uMsg;
  DWORD uParam1;
  DWORD uParam2;
  DWORD uEvent;
} NOTIFY_MSG_S;

typedef struct
{
  DWORD   dwHandle;        // returns VxD's handle to our callback info
  DWORD   dwProc;          // Callback function
  DWORD   dwCardType;      // Card Type required by Application
} VXD_CALLBACK_INFO_S;

#define NUM_MSG        4

typedef struct
{
  VXD_CALLBACK_INFO_S CBInfo;
  DWORD              dwVM;        // VM Handle for callback proc.
  NOTIFY_MSG_S        Msg[NUM_MSG];// multiple messages may be queued for each application
} NOTIFY_S;

typedef struct
{
  DWORD         dwDeviceId;	
  DWORD         dwPCIAddr;
  int           nSocket;
  DWORD         dwTIReg80;
  DWORD         dwTIReg84;
  DWORD         dwTIReg90;
  DWORD         dwTIReg90En;
  DWORD			dwCompaqZVState;
  WORD          wESSState;
  BYTE          bCLReg16;
  BYTE          bTopReg3C;
  WORD			socket1_timing[3];
  WORD			socket2_timing[3];
} SOCKETSTATE;

typedef int (__stdcall* PFN) (PDEVICE);

// Device Structure is used to store info about the card in a particular socket.
// This driver maintains an array of structures one for each socket.
// When a known card is plugged into a socket the card is configured
// and the info store in the corresponding array.
typedef struct
{
   DEVNODE            DevNode;          //Plug and Play Device Node
   SOCKETSTATE        SocketState;      //Keep track of register changes made on socket controller
   DWORD              dwMachineType;    //Host machine type
   DWORD              dwZVEnableMask;   //bit mask indicating sockets that are ZV enabled (bit0 = socket0)
   DWORD              dwPowerManMode;   //if True switch off DVDtoGo clock when not in use
   WORD               ClientHandle;     //Card Service Client Handle (given by CS)
   LOGICAL_SOCKET     l_socket;         //socket (should match index of this structure in array)
   DWORD              CardId;           //ID of card currently in corres socket
   WORD               CardType;         //Card Function Type
   int                nIrqChannel;
   int                nIoAddress;
   enum STATE         State;            //Status of card
   enum STATE         Error;            //Error status of card
   WORD               ioHandle;         //Io Settings
   IO_REQUEST_PKT     io;
   WORD               irqHandle;        //Irq Settings
   IRQ_REQUEST_PKT    irq;
   WORD               cfgHandle;        //Config Settings
   CNFG_REQUEST_PKT501  cfg;
   int                vccOn;            //Operating voltage
   int                fEssVxdInUse;      //True if Ess Vxd being used to control I2S
   int                nEssCount;        //Ref count for Ess Audio enable,disable

   PFN                pfnConfig;
   PFN                pfnRelease;
   PFN                pfnEnable;
   PFN                pfnDisable;
   PFN                pfnConfigDma;
   PFN                pfnReleaseDma;
   PFN                pfnActivate;

   int                fDmaSupported;    // DMA is support on this DVD-to-Go card
   DEVNODE            dnDMA;            // Devnode of DMA io space

   DWORD              dwDmaPhysAddr[8];
   DWORD              dwDmaLinAddr[8];
   WORD               wDmaNumPages;
   WORD               wDmaNumBuffs;
   WORD               wDmaCtrlIoBase;
   DWORD              dwFlags;
   DEVNODE            dnAtapi;
} DEVICE_STRUCT;

typedef DEVICE_STRUCT* PDEVICE;

// Card Services Client Structure.
// Stores Info about this drivers registration with Card Services
typedef struct
{
   WORD                 ClientHandle;
   CLT_REGISTRATION_PKT regpkt;
   CS_INFO_PKT          csi;
} CLIENTREG_STRUCT;

typedef CLIENTREG_STRUCT* PCLIENT;

#define VXD_FLAG_TIDMA 0x00000001    // flag TI Dma support
#define VXD_FLAG_IOSD  0x00000002    // flag support for IOSD

typedef struct
{
  WORD IRQ;        // Return IRQ channel
  WORD IOBase;     // Return I/O Base Address
  WORD State;      // Return State of hardware
  WORD wSocket;    // Return Socket Number (Slot0==socket 1)
  WORD CardType;   // Specify Card Type
  WORD wFlagsLo;   // Extended Flags (not supported in earlier VXD versions)
  WORD wFlagsHi;
} RESOURCES_STRUCT;

enum {
   ERR_UNKNOWN = 0,
   ERR_CS_ERR,
   ERR_NO_CS,
   ERR_CS_SIG,
   ERR_CS_VER,
   ERR_CS_REG
};

typedef struct
{                   
  WORD    wBuffs;
  WORD    wBuffPages;  
  DWORD   dwPhysMem[8];
  WORD    wCtrlIoBase;
  WORD    wSocket;
} VXD_DMA_INFO;


//From MargiPNP.asm
extern DWORD __cdecl ExecWinApp(void *pHexPacket);
extern void UnloadMe(void);
extern void __cdecl Sleep(DWORD mSec);
extern int  __cdecl RealModeDoReq(int, WORD*, void *, int, void *);
extern int  __cdecl RealModeInt15(int Func, int SubFunc, int ReqType);
extern int  __cdecl RealModeInt(int IntType, struct Client_Word_Reg_Struc*);
extern void EventNotify();

//From CSClient.c
extern void ResetRegistryFlags(WORD fComplete);
extern WORD SetRegistryFirstEntry(WORD EntryID, BYTE *ValueName);
extern void InitDeviceList(void);
extern WORD GetCardIoId(int iobase);
extern WORD GetCardTypeFromIo(WORD id);
extern WORD GetCardTypeFromCis(DWORD id);
extern DWORD GetCardCisId(WORD Handle, LOGICAL_SOCKET lSocket);
extern int ConfigureCard(PDEVICE p, int fPwr, int fIO, int fIRQ);
extern int NotifyCallback(int hCb, int msg, DWORD uParam1, DWORD uParam2);
extern void ReleaseCard(PDEVICE pDev);
DWORD HardwareAcquire(DWORD *pHndl);
DWORD HardwareRelease(DWORD *pHndl);
extern void IdentifyHostMachine(PDEVICE p);
extern int CustomConfigZV(PDEVICE p);
extern int CustomReleaseZV(PDEVICE p);
extern int CustomEnableZV(PDEVICE p);
extern int CustomDisableZV(PDEVICE p);
extern char TmpStr[];

//From CfgMgr.c
extern DWORD ConfigStart(DEVNODE DevNode);

#endif
/**EOF: MARGIPNP.H**/

