#include <strmini.h>
#include <mmsystem.h>
#include <ksmedia.h>
#include "DbgMsg.h"
#include "DbgFltr.h"
#include "Idents.h"

#include "Globals.h"
#include "MidasApi.h"
#include "Log.h"
#include "dbgout.h"

void NtscSetState(SRB* pSrb);
void NtscGetState(SRB* pSrb);
void NtscGetProperty(SRB* pSrb);
void NtscSetProperty(SRB* pSrb);
void NtscMasterClock(SRB* pSrb);
void NtscCtrlNotHandled(SRB* pSrb);
void GetMacroVisionLevel(SRB* pSrb);
void SetMacroVisionLevel(SRB* pSrb);

void __stdcall NTSCReceive(PHW_STREAM_REQUEST_BLOCK pSrb) {
  logSrb("NTSCReceive", pSrb);
  unlogSrb("NTSCReceive", pSrb);
  SignalStreamRequestCompleteReadyForNextData(pSrb);
}

void __stdcall NTSCControl(PHW_STREAM_REQUEST_BLOCK pSrb) {
  // set default status
  logSrb("NTSCControl", pSrb);
  pSrb->Status = STATUS_NOT_IMPLEMENTED;
  // determine the type of packet.  All calls here use the pointers
  // to the individual stream functions initialised at open time
  switch(pSrb->Command) {
    case SRB_SET_STREAM_STATE:      NtscSetState(pSrb); break;
    case SRB_GET_STREAM_STATE:      NtscGetState(pSrb); break;
    case SRB_GET_STREAM_PROPERTY:   NtscGetProperty(pSrb); break;
    case SRB_SET_STREAM_PROPERTY:   NtscSetProperty(pSrb); break;
    case SRB_INDICATE_MASTER_CLOCK: NtscMasterClock(pSrb); break;
    default:                        NtscCtrlNotHandled(pSrb); break;
  }
  unlogSrb("NTSCControl", pSrb);
  if(pSrb->Status!=STATUS_PENDING) {
    SignalStreamRequestCompleteReadyForNextControl(pSrb);
  } else {
    DbgMsg(DBG_INFO,"Returning Status Pending %s\r\n",Ident(ID_STRM_REQ,pSrb->Command));
  }
}


long __stdcall NTSCEvent(PHW_EVENT_DESCRIPTOR pEventDescriptor) {
  NTSTATUS status= STATUS_SUCCESS;
  PSTREAM_EXTENSION pStrm= pEventDescriptor->StreamObject->HwStreamExtension;
  if(pEventDescriptor->Enable)
    pStrm->dwEventCount++;
  else
    pStrm->dwEventCount--;
  logEvent("NTSCEvent", pEventDescriptor);
  return status;
}

void NtscSetState(SRB* pSrb) {
  PSTREAM_EXTENSION pStream;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  StreamSetState(pStream,pSrb->CommandData.StreamState);
  pSrb->Status = STATUS_SUCCESS;
}

void NtscGetState(SRB* pSrb) {
  PSTREAM_EXTENSION pStream;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  pSrb->CommandData.StreamState = pStream->stream_state;
  pSrb->Status = STATUS_SUCCESS;
}

void NtscGetProperty(SRB* pSrb) {
  logGetProperty("NTSC", pSrb);
  if(!pSrb->CommandData.PropertyInfo->PropertySetID) {
    switch(pSrb->CommandData.PropertyInfo->Property->Id) {
      case KSPROPERTY_COPY_MACROVISION: GetMacroVisionLevel(pSrb); break;
    }
  }
}

void GetMacroVisionLevel(SRB* pSrb) {
  MIDASCMD cmd;
  PDEVICE_EXTENSION pDevice;
  pDevice = pSrb->HwDeviceExtension;
  cmd.hMidas=pDevice->hMidas;
  cmd.cmd= MDS_SETTVOUT;
  cmd.u.M.Arg1= TVOUT_GETMACROVISIONLEVEL;
  cmd.u.M.Arg2= (DWORD)pSrb->CommandData.PropertyInfo->PropertyInfo;
  MidasEntry(&cmd);
  pSrb->Status= STATUS_SUCCESS;
  DbgDispKsCssPropertyInfo(DBG_MSGNTSC,pSrb->CommandData.PropertyInfo->PropertyInfo,pSrb->CommandData.PropertyInfo->Property->Id);
}

void SetMacroVisionLevel(SRB* pSrb) {
  MIDASCMD cmd;
  PDEVICE_EXTENSION pDevice;
  pDevice = pSrb->HwDeviceExtension;
  cmd.hMidas=pDevice->hMidas;
  cmd.cmd= MDS_SETTVOUT;
  cmd.u.M.Arg1= TVOUT_SETMACROVISIONLEVEL;
  cmd.u.M.Arg2= (DWORD)pSrb->CommandData.PropertyInfo->PropertyInfo;
  MidasEntry(&cmd);
  pSrb->Status= STATUS_SUCCESS;
  DbgDispKsCssPropertyInfo(DBG_MSGNTSC,pSrb->CommandData.PropertyInfo->PropertyInfo,pSrb->CommandData.PropertyInfo->Property->Id);
}

void NtscSetProperty(SRB* pSrb) {
  logSetProperty("NTSC", pSrb);
  if(!pSrb->CommandData.PropertyInfo->PropertySetID) {
    switch(pSrb->CommandData.PropertyInfo->Property->Id) {
      case KSPROPERTY_COPY_MACROVISION: SetMacroVisionLevel(pSrb); break;
    }
  }
}

void NtscMasterClock(SRB* pSrb) {
  PSTREAM_EXTENSION pStream;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  pStream->MasterClock = pSrb->CommandData.MasterClockHandle;
  pSrb->Status = STATUS_SUCCESS;
  DbgMsg(DBG_MSGNTSC,"NTSCControl: MasterClockHandle is set to %08x \r\n",pSrb->CommandData.MasterClockHandle);
}

void NtscCtrlNotHandled(SRB* pSrb) {
  DbgMsg(DBG_ERR|DBG_WRN,DBGID" NTSC Control: Unhandled Msg %d \r\n",pSrb->Command);
}

/* EOF : NTSC.C */
