#include <strmini.h>
#include <mmsystem.h>
#include <ksmedia.h>
#include "DbgMsg.h"
#include "DbgFltr.h"
#include "Idents.h"

#include "Globals.h"
#include "MidasApi.h"
#include "log.h"
#include "UpdtReg.h"

#define USE_KETIMER 0

BOOL gfDataLeft= TRUE;
DWORD gTimerThreshold= 20000;
DWORD gTimerPeriod=     500;
//DWORD gTimerPeriod=     1000;
DWORD gDataSendMaxTime= 5000;
DWORD QPktTotal;		//Total number of packets to come through.
DWORD QPktInQueue;		//Total number of packets current enqueued.

static void __stdcall TheTimer(PDEVICE_EXTENSION pDev);

DWORD __stdcall MargiGetTime() {
  ULONGLONG CurTime;
  DWORD dwMS;
  KeQuerySystemTime((PLARGE_INTEGER)&CurTime);    //100ns ticks
  dwMS= (DWORD)(CurTime/10000);
  return(dwMS);
}

#ifdef _DEBUG
// Return Time in 
ULONGLONG GetSysTime() {
  LARGE_INTEGER pf;
  LARGE_INTEGER Ticks;
  Ticks=KeQueryPerformanceCounter(&pf);
  return (Ticks.QuadPart*1000000)/pf.QuadPart;
}
#endif

void display_header(PHW_STREAM_REQUEST_BLOCK ptr) {
  int i;
  PSRB_EXTENSION logptr;
  PKSSTREAM_HEADER hdr;

  logptr= (PSRB_EXTENSION) ptr->SRBExtension;
  hdr= ptr->CommandData.DataBufferArray;
  i= ptr->NumberOfBuffers;
  while(i--) {
    DbgMsg(DBG_FIRST4BYTES,"%08x ",*(DWORD*)hdr->Data);
    if(!(i%8)) DbgMsg(DBG_FIRST4BYTES," \r\n");
    DbgMsg(DBG_HIFREQ|DBG_MSGPKT, "   Addr %08x  Size %08x PTS %08x%08x N/D %d/%d\n", hdr->Data, hdr->DataUsed,  (unsigned int)(hdr->PresentationTime.Time >> 32),
        (unsigned int)(hdr->PresentationTime.Time), hdr->PresentationTime.Numerator, hdr->PresentationTime.Denominator);
    hdr++;
  }
}              

int enqueue(StreamQueue *stream, PHW_STREAM_REQUEST_BLOCK ptr) {
  PDEVICE_EXTENSION pDevice;
  QPktInQueue++;
  QPktTotal++;
  display_header(ptr);
  ptr->NextSRB= 0;
  if(stream->head) {
    stream->tail->NextSRB= ptr;
  } else {
    stream->head= ptr;
  }
  stream->tail= ptr;
  pDevice= ptr->HwDeviceExtension;
  if(!pDevice->process_data) {
    pDevice->process_data= 1;
  }
  stream->count++;
  return 0;
}

PHW_STREAM_REQUEST_BLOCK dequeue(StreamQueue *stream) {
  PHW_STREAM_REQUEST_BLOCK pSrb= 0;
  if(stream->head) {
    pSrb= stream->head;
    stream->head= stream->head->NextSRB;
    QPktInQueue--;
    stream->count--;
  }
  return(pSrb);
}

PHW_STREAM_REQUEST_BLOCK find_and_dequeue(StreamQueue *stream, PHW_STREAM_REQUEST_BLOCK pSrb) {
  PHW_STREAM_REQUEST_BLOCK ptr;
  PHW_STREAM_REQUEST_BLOCK prev_ptr;

  ptr= stream->head;
  prev_ptr= 0;
  while(ptr && ptr != pSrb) {
    prev_ptr= ptr;
    ptr= ptr->NextSRB;
  }
  if(ptr) {
    QPktInQueue--;
    stream->count--;
    if(prev_ptr) {
      prev_ptr->NextSRB= ptr->NextSRB;
    } else {
      stream->head= ptr->NextSRB;
    }
    if(ptr== stream->tail)
      stream->tail= prev_ptr;
  }
  return ptr;
}

static struct TimerInfo_s {
  union {
    struct {
      DWORD Go:1;
      DWORD LoRunning:1;
      DWORD LoScheduled:1;
      DWORD HiRunning:1;
      DWORD HiScheduled:1;
    };
    DWORD Flags;
  };
  DWORD HiOverruns;
  DWORD LoTotal;
  DWORD HiTotal;
  PDEVICE_EXTENSION pDev;
  void* pRef;
  ULONGLONG NextMsg;
  ULONGLONG  LastHiTime;
  ULONGLONG  LastLoTime;
  ULONGLONG  LastLoSchedTime;
  ULONGLONG  MaxHiElapsedTime;
  ULONGLONG  MaxLoElapsedTime;
  ULONGLONG  TotalHiElapsedTime;
  ULONGLONG  TotalLoElapsedTime;
  ULONGLONG  MaxHiPeriod;
  ULONGLONG  MinHiPeriod;
  ULONGLONG  MaxLoPeriod;
  ULONGLONG  MinLoPeriod;
  DWORD      NoLoSched;
  PHW_STREAM_REQUEST_BLOCK pPowerSrb;
  KTIMER     TimerOb;
  KDPC       Dpc;
  LARGE_INTEGER Period;
} TimerInfo;

void TimerDpc(PKDPC pDpc,PVOID pRef,PVOID Arg1,PVOID Arg2) {
  TheTimer((PDEVICE_EXTENSION) pRef);
}

long TimerStart(DWORD Start,PDEVICE_EXTENSION pRef) {
  if(Start) {
    memset(&TimerInfo,0,sizeof(TimerInfo));
    TimerInfo.pDev=(PDEVICE_EXTENSION)pRef;
    TimerInfo.pRef=0;
    TimerInfo.Go= 1;
    TimerInfo.NextMsg= 0;
    TimerInfo.LastHiTime=0;
    TimerInfo.LastLoTime=0;
    TimerInfo.Period.QuadPart=gTimerPeriod*10;
    // Set System Timer Resolution to 1mS
    ExSetTimerResolution(10000,1);
#if(USE_KETIMER)
      KeInitializeTimerEx(&TimerInfo.TimerOb,NotificationTimer);
      KeInitializeDpc(&TimerInfo.Dpc,TimerDpc,(void*)TimerInfo.pDev);
      KeSetImportanceDpc(&TimerInfo.Dpc,HighImportance);
      KeSetTimer(&TimerInfo.TimerOb,TimerInfo.Period,&TimerInfo.Dpc);
#else
      TheTimer(TimerInfo.pDev);
#endif
  } else {
    TimerInfo.Go= 0;
    TimerInfo.pRef=pRef;
  }
  return 0;
}

long TimerSetPowerState(PHW_STREAM_REQUEST_BLOCK pPowerSrb) {
  if(TimerInfo.Go) {
    TimerInfo.pPowerSrb=pPowerSrb;
    return 1;
  }
  return 0;    
}

long TimerStopComplete() {
  return (!TimerInfo.Go && !TimerInfo.LoScheduled && !TimerInfo.LoRunning && !TimerInfo.HiScheduled && !TimerInfo.HiRunning);
}

static DWORD __stdcall TheTimerLowPriority(PDEVICE_EXTENSION pDevice) {
  static WORD nElapsed=CHECKREGISTRYAFTER+1;		//timeout for dvdshow.dll registry
  DWORD ElapsedTime;
  MIDASCMD Cmd;
#ifdef _DEBUG
  ULONGLONG IntrTime=GetSysTime();
  ULONGLONG Period=IntrTime-TimerInfo.LastLoTime;
  TimerInfo.LastLoTime=IntrTime;
  if(Period > TimerInfo.MaxLoPeriod)
    TimerInfo.MaxLoPeriod=Period;
  if(Period < TimerInfo.MinLoPeriod)
    TimerInfo.MinLoPeriod=Period;
#endif
//  DbgMsgCmd(DBGCMD_PROF_ENTER,DBGPRF_DATATIMER,0);
  TimerInfo.LoRunning= 1;
  TimerInfo.LoScheduled= 0;
  if(TimerInfo.Go) {
    TimerInfo.LoTotal++;
    if(TimerInfo.LoTotal>TimerInfo.HiTotal)
      DbgPrint("TheTimer: Assume crash positions!\r\n");

    if(pDevice->StreamsOpened) {
      Cmd.cmd= MDS_DATATIMER;
      Cmd.wCurrStream= 0;
      Cmd.hMidas=pDevice->hMidas;
      MidasEntry(&Cmd);
    }
    if(nElapsed>CHECKREGISTRYAFTER) {
      nElapsed=0;
      CheckTempRegistryForChanges(pDevice);
    }
#ifdef _DEBUG
    nElapsed++;
    ElapsedTime= (DWORD)(GetSysTime()-IntrTime);
    TimerInfo.TotalLoElapsedTime+=ElapsedTime;
    if(ElapsedTime>TimerInfo.MaxLoElapsedTime)
      TimerInfo.MaxLoElapsedTime=ElapsedTime;
#endif
  }
  TimerInfo.LoRunning= 0;
//  DbgMsgCmd(DBGCMD_PROF_EXIT,DBGPRF_DATATIMER,0);
  return 0;
}

#ifdef _DEBUG
static void TimerCheck() {
  static DWORD dw10KPrevTime=0;
  static DWORD dwTicks=0;
  DWORD     dw10KTime;
  ULONGLONG llTime;
  KeQuerySystemTime((PLARGE_INTEGER)&llTime);    //100ns ticks
  dw10KTime=(DWORD)(llTime/1000);
  if(dw10KTime>dw10KPrevTime+(gTimerPeriod/100)*2) {
    DbgMsg(DBG_HWERR,"Timer slow %d, %d good\r\n",dw10KTime-dw10KPrevTime,dwTicks);
    DbgPrint("Timer slow %d, %d good\r\n",dw10KTime-dw10KPrevTime,dwTicks);
    dwTicks=0;
  } else
    dwTicks++;
  dw10KPrevTime=dw10KTime;
}
#endif


static void __stdcall TheTimer(PDEVICE_EXTENSION pDevice) {
  long      Err=0;
  MIDASCMD  Cmd;
  ULONGLONG IntrTime;					//Current system time (in 100ns ticks)
  ULONGLONG ElapsedTime;
  ULONGLONG Period;

  TimerInfo.HiRunning= 1;
  TimerInfo.HiScheduled= 0;
  TimerInfo.HiTotal++;
//  DbgMsgCmd(DBGCMD_PROF_ENTER,DBGPRF_TIMER,0);
  IntrTime= KeQueryInterruptTime();
#ifdef _DEBUG
  IntrTime= GetSysTime();
//  TimerCheck();
  Period=IntrTime-TimerInfo.LastHiTime;
  TimerInfo.LastHiTime=IntrTime;

  if(IntrTime>=TimerInfo.NextMsg) {
    static ULONG ti=0;
    TimerInfo.NextMsg= IntrTime+1000000;		//Print message once a second.
    DbgMsg(DBG_HWISR,"Timer: still ticking...\r\n");
#if 0
    DbgPrint("Timer Max Elapsed Time Hi=%duS,Lo=%duS,  Total Time Hi=%duS,Lo=%duS,  Period Hi Max=%duS, Min=%duS Lo Max=%duS, Min=%duS\r\n",
                                            (DWORD)TimerInfo.MaxHiElapsedTime,(DWORD)TimerInfo.MaxLoElapsedTime,
                                            (DWORD)TimerInfo.TotalHiElapsedTime,(DWORD)TimerInfo.TotalLoElapsedTime,
                                            (DWORD)TimerInfo.MaxHiPeriod,(DWORD)TimerInfo.MinHiPeriod,
                                            (DWORD)TimerInfo.MaxLoPeriod,(DWORD)TimerInfo.MinLoPeriod);
#endif
    if(ti!=KeQueryTimeIncrement()) {
      ti=KeQueryTimeIncrement();
//      DbgPrint("Time Increment=%d\r\n",ti);
    }

    TimerInfo.TotalHiElapsedTime=0;
    TimerInfo.TotalLoElapsedTime=0;
    TimerInfo.MaxHiElapsedTime=0;
    TimerInfo.MaxLoElapsedTime=0;
    TimerInfo.MaxLoElapsedTime=0;
    TimerInfo.MaxHiPeriod=0;
    TimerInfo.MinHiPeriod=0xffffffffff;
    TimerInfo.MaxLoPeriod=0;
    TimerInfo.MinLoPeriod=0xffffffffff;


    // DbgPrint takes about 23mS so eliminate from time measurements
    IntrTime= GetSysTime();
    TimerInfo.LastHiTime=IntrTime;
  }
#endif

  if(TimerInfo.pPowerSrb && !TimerInfo.LoScheduled && !TimerInfo.LoRunning) {
    TimerInfo.Go= 0;
    TimerInfo.pPowerSrb->Status=STATUS_SUCCESS;
    SignalDeviceRequestComplete(TimerInfo.pPowerSrb);
    TimerInfo.pPowerSrb=0;
  } else if(!TimerInfo.pPowerSrb) {
    if(TimerInfo.Go && pDevice->StreamsOpened) {
      if((IntrTime-TimerInfo.LastLoSchedTime) >= 7000)
      {
        TimerInfo.LastLoSchedTime=IntrTime;
        Cmd.cmd= MDS_TIMER;
        Cmd.wCurrStream= 0;
        Cmd.hMidas=pDevice->hMidas;
        Err=MidasEntry(&Cmd);

        CcClearClockEvents(pDevice);
        if(Err) {
          pDevice->HwError=1;
        }

        if(!TimerInfo.LoScheduled && !TimerInfo.LoRunning) {
          TimerInfo.LoScheduled= 1;
          TimerInfo.NoLoSched=0;
          StreamClassCallAtNewPriority(0,pDevice,Low,TheTimerLowPriority,pDevice);
        } else {
        #ifdef DEBUG
          if(TimerInfo.NoLoSched++ > 100) 
            _asm int 3;
        #endif
        }
      }

      if(pDevice->DmaControl.dwCtrlIO)
        DmaTimer(pDevice);

#ifdef _DEBUG
      ElapsedTime= GetSysTime()-IntrTime;
	     TimerInfo.TotalHiElapsedTime+=ElapsedTime;
      if(ElapsedTime>TimerInfo.MaxHiElapsedTime)
        TimerInfo.MaxHiElapsedTime=ElapsedTime;
      if(Period > TimerInfo.MaxHiPeriod)
        TimerInfo.MaxHiPeriod=Period;
      if(Period < TimerInfo.MinHiPeriod)
        TimerInfo.MinHiPeriod=Period;
#endif
    } else
      TimerInfo.Go= 0;
  }
  if(TimerInfo.Go) {
    LARGE_INTEGER Period;
    Period.QuadPart=TimerInfo.Period.QuadPart;
    TimerInfo.HiScheduled= 1;
#if(USE_KETIMER)
    Period.QuadPart=-Period.QuadPart;
    KeSetTimer(&TimerInfo.TimerOb,TimerInfo.Period,&TimerInfo.Dpc);
#else
    StreamClassScheduleTimer(0,pDevice,(DWORD)Period.QuadPart/10,TheTimer,pDevice);
#endif
  }
  TimerInfo.HiRunning= 0;
//  DbgMsgCmd(DBGCMD_PROF_EXIT,DBGPRF_TIMER,0);
}

//**EOF: QUEUE.C**/
 