#include <StrMini.h>
#include <mmsystem.h>
#include <KsMedia.h>
#include <uuids.h>
#include "DbgMsg.h"
#include "DbgFltr.h"
#include "Idents.h"
#include "Settings.h"

#include "IniNames.h"

#include "Globals.h"
#include "MidasApi.h"
#include "dbgout.h"
#include "mddraw.h"
#include "ZVNt.h"

#pragma message("Stream.c: Optimizer disabled.")
#pragma optimize("",off)

typedef struct {
    KSDATAFORMAT    DataFormat;
    WAVEFORMATEX    WaveFormatEx;
    BYTE            bExtra[6];
} KSDATAFORMAT_WAVEFORMATEX1, *PKSDATAFORMAT_WAVEFORMATEX1;


// The following 6 GUID's are used to name the pins of the scenario decompression module

static const GUID VideoPin = { /* c57875b0-4bf6-11d1-808b-006097298068 */
  0xc57875b0,
  0x4bf6,
  0x11d1,
  {0x80, 0x8b, 0x00, 0x60, 0x97, 0x29, 0x80, 0x68}
};

static const GUID AudioPin = { /* c57875b1-4bf6-11d1-808b-006097298068 */
  0xc57875b1,
  0x4bf6,
  0x11d1,
  {0x80, 0x8b, 0x00, 0x60, 0x97, 0x29, 0x80, 0x68}
};

static const GUID SubpicturePin = { /* c57875b2-4bf6-11d1-808b-006097298068 */
  0xc57875b2,
  0x4bf6,
  0x11d1,
  {0x80, 0x8b, 0x00, 0x60, 0x97, 0x29, 0x80, 0x68}
};

static const GUID NTSCPin = { /* c57875b3-4bf6-11d1-808b-006097298068 */
  0xc57875b3,
  0x4bf6,
  0x11d1,
  {0x80, 0x8b, 0x00, 0x60, 0x97, 0x29, 0x80, 0x68}
};

static const GUID VPEPin = { /* c57875b4-4bf6-11d1-808b-006097298068 */
  0xc57875b4,
  0x4bf6,
  0x11d1,
  {0x80, 0x8b, 0x00, 0x60, 0x97, 0x29, 0x80, 0x68}
};

static const GUID CCPin = { /* c57875b5-4bf6-11d1-808b-006097298068 */
  0xc57875b5,
  0x4bf6,
  0x11d1,
  {0x80, 0x8b, 0x00, 0x60, 0x97, 0x29, 0x80, 0x68}
};

/* The accepted formats, properties and events for each stream are
   placed in this area as constants*/
// MPEG2 video format definition
KSDATAFORMAT InMpeg2Video_Fmt0 = {
  sizeof(KSDATAFORMAT), // + sizeof(KS_MPEGVIDEOINFO2),
  /* NOTE: MPEG2 video streams must support the MPEGVIDEOINFO2 format
  * block. This defines aspect ratios, and other stream properties */
  0,
  0,
  0,
  STATIC_KSDATAFORMAT_TYPE_DVD_ENCRYPTED_PACK,
  STATIC_KSDATAFORMAT_SUBTYPE_MPEG2_VIDEO,
  STATIC_KSDATAFORMAT_SPECIFIER_MPEG2_VIDEO
};

KSDATAFORMAT    InMpeg2Video_Fmt1 = {
  sizeof(KSDATAFORMAT),// + sizeof(KS_MPEGVIDEOINFO2),
  // NOTE: MPEG2 video streams must support the MPEGVIDEOINFO2 format
  // block. This defines aspect ratios, and other stream properties
  0,
	0,
  0,
  STATIC_KSDATAFORMAT_TYPE_MPEG2_PES,
  STATIC_KSDATAFORMAT_SUBTYPE_MPEG2_VIDEO,
  STATIC_KSDATAFORMAT_SPECIFIER_MPEG2_VIDEO
};

KSDATAFORMAT    InMpeg2Video_Fmt2 = {
  sizeof(KSDATAFORMAT),// + sizeof(KS_MPEGVIDEOINFO2),
  0,0,0,
  {0x73646976, 0x0000, 0x0010, 0x80,0x00,0x00,0xaa,0x00,0x38,0x9b,0x71},//MEDIATYPE_Video,
  {0xe436eb80, 0x524f, 0x11ce, 0x9f,0x53,0x00,0x20,0xaf,0x0b,0xa7,0x70},//MEDIASUBTYPE_MPEG1Packet,
  {0x05589f82, 0xc356, 0x11ce, 0xbf,0x01,0x00,0xaa,0x00,0x55,0x59,0x5a},//FORMAT_MPEGVideo,
};

KSDATAFORMAT    InMpeg2Video_Fmt3 = {
  sizeof(KSDATAFORMAT),// + sizeof(KS_MPEGVIDEOINFO2),
  0,0,0,
  {0x73646976, 0x0000, 0x0010, 0x80,0x00,0x00,0xaa,0x00,0x38,0x9b,0x71},//MEDIATYPE_Video,
  {0xe436eb81, 0x524f, 0x11ce, 0x9f,0x53,0x00,0x20,0xaf,0x0b,0xa7,0x70},//MEDIASUBTYPE_MPEG1Payload,
  {0x05589f82, 0xc356, 0x11ce, 0xbf,0x01,0x00,0xaa,0x00,0x55,0x59,0x5a},//FORMAT_MPEGVideo,
};
  
KSDATAFORMAT    InMpeg2Video_Fmt4 = {
  sizeof(KSDATAFORMAT),// + sizeof(KS_MPEGVIDEOINFO2),
  0,0,0,
  {0x73646976, 0x0000, 0x0010, 0x80,0x00,0x00,0xaa,0x00,0x38,0x9b,0x71},//MEDIATYPE_Video,
  STATIC_KSDATAFORMAT_SUBTYPE_MPEG2_VIDEO,       //MEDIASUBTYPE_MPEG2_VIDEO 
  STATIC_KSDATAFORMAT_SPECIFIER_MPEG2_VIDEO,   //FORMAT_MPEG2Video
};

static const PKSDATAFORMAT InMpeg2Video_Fmt[] = {
  (PKSDATAFORMAT)&InMpeg2Video_Fmt4,
  (PKSDATAFORMAT)&InMpeg2Video_Fmt3,
  (PKSDATAFORMAT)&InMpeg2Video_Fmt1,
  (PKSDATAFORMAT)&InMpeg2Video_Fmt2,
  (PKSDATAFORMAT)&InMpeg2Video_Fmt0,
};

//
// Format blocks for the supported audio formats
//

// Mpeg2(AC3)
KSDATAFORMAT_WAVEFORMATEX hwfmtiMpeg2Aud = {
   {  sizeof (KSDATAFORMAT_WAVEFORMATEX),0,1,0,
      STATIC_KSDATAFORMAT_TYPE_DVD_ENCRYPTED_PACK,
      STATIC_KSDATAFORMAT_SUBTYPE_AC3_AUDIO,
      STATIC_KSDATAFORMAT_SPECIFIER_WAVEFORMATEX
   }, {
      0,   // WAVE_FORMAT_UNKNOWN
      6,   // channels
      48000,  // sampling rate
      0,  // byte rate
      768, // align
      0,   // resolution
      0    // extra
   }
};


// Mpeg2(AC3) PES
KSDATAFORMAT_WAVEFORMATEX hwfmtiMpeg2AudPES = {
   {  sizeof (KSDATAFORMAT_WAVEFORMATEX),0,1,0,
      STATIC_KSDATAFORMAT_TYPE_MPEG2_PES,
      STATIC_KSDATAFORMAT_SUBTYPE_AC3_AUDIO,	//Even though spec says DOLBY_AC3, MS confirmed that this is same
      STATIC_KSDATAFORMAT_SPECIFIER_WAVEFORMATEX
   }, {
      0,   // WAVE_FORMAT_UNKNOWN
      6,   // channels
      48000,  // sampling rate
      0,  // byte rate
      768, // align
      0,   // resolution
      0    // extra
   }
};


// Mpeg2(AC3) PES
KSDATAFORMAT_WAVEFORMATEX hwfmtiMpeg2AudPESMpeg2 = {
   {  sizeof (KSDATAFORMAT_WAVEFORMATEX),0,1,0,
      STATIC_KSDATAFORMAT_TYPE_MPEG2_PES,
      STATIC_KSDATAFORMAT_SUBTYPE_MPEG2_AUDIO,	//Even though spec says DOLBY_AC3, MS confirmed that this is same
      STATIC_KSDATAFORMAT_SPECIFIER_WAVEFORMATEX
   }, {
      0,   // WAVE_FORMAT_UNKNOWN
      6,   // channels
      48000,  // sampling rate
      0,  // byte rate
      768, // align
      0,   // resolution
      0    // extra
   }
};


// LPCM
KSDATAFORMAT_WAVEFORMATEX hwfmtiLPCMAud = {
   {  sizeof (KSDATAFORMAT_WAVEFORMATEX),0,1,0,
      STATIC_KSDATAFORMAT_TYPE_DVD_ENCRYPTED_PACK,
      STATIC_KSDATAFORMAT_SUBTYPE_LPCM_AUDIO,
      STATIC_KSDATAFORMAT_SPECIFIER_WAVEFORMATEX
   }, {
      WAVE_FORMAT_PCM,
      2,	// channels
      48000,  // sampling rate
      192000,  // byte rate
      4,	// alignment
      16,	// bit resolution
      0	// extra
   }
};
//
// End format blocks for the supported audio formats
//
#if(0)
// 73647561-0000-0010-8000-00AA00389B71  'auds' == MEDIATYPE_Audio
OUR_GUID_ENTRY(MEDIATYPE_Audio,
0x73647561, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71)
// e436eb80-524f-11ce-9f53-0020af0ba770            MEDIASUBTYPE_MPEGPacket
OUR_GUID_ENTRY(MEDIASUBTYPE_MPEG1Packet,
0xe436eb80, 0x524f, 0x11ce, 0x9f, 0x53, 0x00, 0x20, 0xaf, 0x0b, 0xa7, 0x70)


const AMOVIESETUP_MEDIATYPE psudIpPinTypes[] =
{ { &MEDIATYPE_Audio                // clsMajorType
  , &MEDIASUBTYPE_MPEG1Packet  }    // clsMinorType
, { &MEDIATYPE_Audio                // clsMajorType
  , &MEDIASUBTYPE_MPEG1AudioPayload }    // clsMinorType
, { &MEDIATYPE_Stream               // clsMajorType
  , &MEDIASUBTYPE_MPEG1Audio   } }; // clsMinorType

0xe436eb87, 0x524f, 0x11ce, 0x9f, 0x53, 0x00, 0x20, 0xaf, 0x0b, 0xa7, 0x70)
#endif

//  MPEG1
KSDATAFORMAT InMpeg1Aud_Fmt0 = {
  sizeof(KSDATAFORMAT),
  0,0,0,
  {0x73647561, 0x0000, 0x0010, 0x80,0x00,0x00,0xaa,0x00,0x38,0x9b,0x71},//MEDIATYPE_Audio,
  STATIC_KSDATAFORMAT_SUBTYPE_STANDARD_MPEG1_AUDIO,
//  {0xe436eb80, 0x524f, 0x11ce, 0x9f,0x53,0x00,0x20,0xaf,0x0b,0xa7,0x70}//MEDIASUBTYPE_MPEG1Packet,
  {0x00000000, 0x0000, 0x0000, 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},//null
};

KSDATAFORMAT InMpeg1Aud_Fmt1 = {
  sizeof(KSDATAFORMAT),
  0,0,0,
  {0x73647561, 0x0000, 0x0010, 0x80,0x00,0x00,0xaa,0x00,0x38,0x9b,0x71},//MEDIATYPE_Audio,
  {0x00000050, 0x0000, 0x0010, 0x80,0x00,0x00,0xAA,0x00,0x38,0x9B,0x71},//MEDIASUBTYPE_MPEG1AudioPayload,
  {0x00000000, 0x0000, 0x0000, 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},//null
};


KSDATAFORMAT InMpeg1Aud_Fmt2 = {
  sizeof(KSDATAFORMAT),
  0,0,0,
  {0x73647561, 0x0000, 0x0010, 0x80,0x00,0x00,0xaa,0x00,0x38,0x9b,0x71},//MEDIATYPE_Audio,
  {0xe436eb81, 0x524f, 0x11ce, 0x9f,0x53,0x00,0x20,0xaf,0x0b,0xa7,0x70},//MEDIASUBTYPE_MPEG1Payload,
  {0x00000000, 0x0000, 0x0000, 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},//null
};

KSDATAFORMAT_WAVEFORMATEX1 InMpeg1Aud_Fmt0X = {
  {
//  sizeof(KSDATAFORMAT),
  sizeof(KSDATAFORMAT_WAVEFORMATEX1),
  0,
  0,
  0,
  {0x73647561, 0x0000, 0x0010, 0x80,0x00,0x00,0xaa,0x00,0x38,0x9b,0x71},//MEDIATYPE_Audio,
//  {0xe436eb8a, 0x524f, 0x11ce, 0x9f,0x53,0x00,0x20,0xaf,0x0b,0xa7,0x70},//MEDIASUBTYPE_PCMAudio,
//  STATIC_KSDATAFORMAT_SUBTYPE_STANDARD_MPEG1_AUDIO,
//  {0x00000000, 0x0000, 0x0000, 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},//null
  {0xe436eb80, 0x524f, 0x11ce, 0x9f,0x53,0x00,0x20,0xaf,0x0b,0xa7,0x70},//MEDIASUBTYPE_MPEG1Packet,
//  {0x00000050, 0x0000, 0x0010, 0x80,0x00,0x00,0xAA,0x00,0x38,0x9B,0x71},//MEDIASUBTYPE_MPEG1AudioPayload,
//  {0xe436eb81, 0x524f, 0x11ce, 0x9f,0x53,0x00,0x20,0xaf,0x0b,0xa7,0x70},//MEDIASUBTYPE_MPEG1Payload,
//  {0xe436eb87, 0x524f, 0x11ce, 0x9f,0x53,0x00,0x20,0xaf,0x0b,0xa7,0x70},//MEDIASUBTYPE_MPEG1Audio,
  {0x00000000, 0x0000, 0x0000, 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},//null
//  STATIC_KSDATAFORMAT_SPECIFIER_WAVEFORMATEX
  }, {
//    WORD    wFormatTag;        /* format type */
//    WORD    nChannels;         /* number of channels (i.e. mono, stereo, etc.) */
//    DWORD   nSamplesPerSec;    /* sample rate */
//    DWORD   nAvgBytesPerSec;   /* for buffer estimation */
//    WORD    nBlockAlign;       /* block size of data */
    0,   // WAVE_FORMAT_UNKNOWN
    2,   // channels
    44100,  // sampling rate
    0,  // byte rate
    1, // align
    0,   // resolution
    0    // extra
   },
  {0,0,0,0,0,0}

};

KSDATAFORMAT_WAVEFORMATEX InMpeg1Aud_Fmt1X = {
  {
  sizeof(KSDATAFORMAT_WAVEFORMATEX1),
  0,
  0,
  0,
  {0x73647561, 0x0000, 0x0010, 0x80,0x00,0x00,0xaa,0x00,0x38,0x9b,0x71},//MEDIATYPE_Audio,
  STATIC_KSDATAFORMAT_SUBTYPE_STANDARD_MPEG1_AUDIO,
//  {0x00000050, 0x0000, 0x0010, 0x80,0x00,0x00,0xAA,0x00,0x38,0x9B,0x71},//MEDIASUBTYPE_MPEG1AudioPayload,
  {0x05589f81, 0xc356, 0x11ce, 0xbf,0x01,0x00,0xaa,0x00,0x55,0x59,0x5a} //FORMAT_WaveFormatEx,
  }, {
    0,   // WAVE_FORMAT_UNKNOWN
    2,   // channels
    44100,  // sampling rate
    0,  // byte rate
    1, // align
    0,   // resolution
    0    // extra
   }
};


KSDATAFORMAT_WAVEFORMATEX InMpeg1Aud_Fmt2X = {
  {
  sizeof(KSDATAFORMAT) + sizeof(WAVEFORMATEX),
  0,
  0,
  0,
  {0x73647561, 0x0000, 0x0010, 0x80,0x00,0x00,0xaa,0x00,0x38,0x9b,0x71},//MEDIATYPE_Audio,
  {0xe436eb81, 0x524f, 0x11ce, 0x9f,0x53,0x00,0x20,0xaf,0x0b,0xa7,0x70},//MEDIASUBTYPE_MPEG1Payload,
  {0x05589f81, 0xc356, 0x11ce, 0xbf,0x01,0x00,0xaa,0x00,0x55,0x59,0x5a} //FORMAT_WaveFormatEx,
  }, {
    WAVE_FORMAT_4S16,   // WAVE_FORMAT_UNKNOWN
    2,   // channels
    44100,  // sampling rate
    0,  // byte rate
    1, // align
    0,   // resolution
    0    // extra
   }
};



//  AC3 audio format definition
static const KSDATAFORMAT InMpeg2Ac3_Fmt0 = {
  sizeof(KSDATAFORMAT) + sizeof(KS_MPEGAUDIOINFO),
  0,
  0,
  0,
  STATIC_KSDATAFORMAT_TYPE_DVD_ENCRYPTED_PACK,
  STATIC_KSDATAFORMAT_SUBTYPE_AC3_AUDIO,
  STATIC_KSDATAFORMAT_SPECIFIER_WAVEFORMATEX
};

static const PKSDATAFORMAT InMpeg2Ac3_Fmt[] = {
  (PKSDATAFORMAT)&InMpeg2Ac3_Fmt0,
  (PKSDATAFORMAT) &InMpeg1Aud_Fmt0X,
  (PKSDATAFORMAT) &hwfmtiMpeg2Aud, // pointer to the Mpeg2 format
  (PKSDATAFORMAT) &hwfmtiLPCMAud,   // pointer to the LPCM format
  (PKSDATAFORMAT) &hwfmtiMpeg2AudPESMpeg2,
  (PKSDATAFORMAT) &hwfmtiMpeg2AudPES, // pointer to the Mpeg2 PES format
  (PKSDATAFORMAT) &InMpeg1Aud_Fmt1,
  (PKSDATAFORMAT) &InMpeg1Aud_Fmt2,
};

// define the NTSC output format
static const KSDATAFORMAT OutNTSCVideo_Fmt0= {
  sizeof(KSDATAFORMAT),
  0,
  0,
  0,
  STATIC_KSDATAFORMAT_TYPE_DVD_ENCRYPTED_PACK,
  STATIC_KSDATAFORMAT_SUBTYPE_MPEG2_VIDEO,
  STATIC_KSDATAFORMAT_SPECIFIER_WAVEFORMATEX
};

static const PKSDATAFORMAT OutNTSCVideo_Fmt[]= {
  (PKSDATAFORMAT)&OutNTSCVideo_Fmt0
};

// define the DVD subpicture decoder pin format
static const KSDATAFORMAT InMpeg2Subpicture_Fmt0= {
  sizeof(KSDATAFORMAT),
  0,
  0,
  0,
  STATIC_KSDATAFORMAT_TYPE_DVD_ENCRYPTED_PACK,
  STATIC_KSDATAFORMAT_SUBTYPE_SUBPICTURE,
  STATIC_GUID_NULL
};

static const PKSDATAFORMAT InMpeg2Subpicture_Fmt[]= {
  (PKSDATAFORMAT)&InMpeg2Subpicture_Fmt0
};

KSPIN_MEDIUM VPMedium = {
	STATIC_KSMEDIUMSETID_VPBus,
	0,
	0
};

// define the VIP output pin format
static const KSDATAFORMAT OutVideoPort_Fmt0 = {
  sizeof( KSDATAFORMAT), 0, 0, 0,
  STATIC_KSDATAFORMAT_TYPE_VIDEO,
  STATIC_KSDATAFORMAT_SUBTYPE_VPVideo,
  STATIC_KSDATAFORMAT_SPECIFIER_NONE
};

static const KSDATAFORMAT OutVideoPort_Fmt1 = {
  sizeof( KSDATAFORMAT), 0, 0, 0,
  STATIC_KSDATAFORMAT_TYPE_VIDEO,
  {0x50313459, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71}, //YUV12/Y41P
  STATIC_KSDATAFORMAT_SPECIFIER_NONE
};

static const KSDATAFORMAT OutVideoPort_Fmt2 = {
  sizeof( KSDATAFORMAT), 0, 0, 0,
  STATIC_KSDATAFORMAT_TYPE_VIDEO,
  {0x32595559, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71}, //YUY2
  STATIC_KSDATAFORMAT_SPECIFIER_NONE
};

static const KSDATAFORMAT OutVideoPort_Fmt3 = {
  sizeof( KSDATAFORMAT), 0, 0, 0,
  STATIC_KSDATAFORMAT_TYPE_VIDEO,
  {0x59565955, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71}, //UYVY
  STATIC_KSDATAFORMAT_SPECIFIER_NONE
};

static const KSDATAFORMAT OutVideoPort_Fmt4 = {
  sizeof( KSDATAFORMAT), 0, 0, 0,
  STATIC_KSDATAFORMAT_TYPE_VIDEO,
  {0xFCA326A0, 0xDA60, 0x11CF, 0x9B, 0x06, 0x00, 0xA0, 0xC9, 0x03, 0xA3, 0xB8}, //DDVPTYPE_CCIR656
  STATIC_KSDATAFORMAT_SPECIFIER_NONE
};

static const KSDATAFORMAT OutVideoPort_Fmt5 = {
  sizeof( KSDATAFORMAT), 0, 0, 0,
  STATIC_KSDATAFORMAT_TYPE_VIDEO,
  {0xe436eb7f, 0x524f, 0x11ce, 0x9f, 0x53, 0x00, 0x20, 0xaf, 0x0b, 0xa7, 0x70},//Overlay
  STATIC_KSDATAFORMAT_SPECIFIER_NONE
};

static const KSDATAFORMAT OutVideoPort_Fmt6 = {
  sizeof( KSDATAFORMAT), 0, 0, 0,
  STATIC_KSDATAFORMAT_TYPE_VIDEO,
  {0xe436eb7b, 0x524f, 0x11ce, 0x9f, 0x53, 0x00, 0x20, 0xaf, 0x0b, 0xa7, 0x70},//RGB565
  STATIC_KSDATAFORMAT_SPECIFIER_NONE
};

static const PKSDATAFORMAT OutVideoPort_Fmt[] = {
  (PKSDATAFORMAT)&OutVideoPort_Fmt0,
//  (PKSDATAFORMAT)&OutVideoPort_Fmt1,
//  (PKSDATAFORMAT)&OutVideoPort_Fmt2,
//  (PKSDATAFORMAT)&OutVideoPort_Fmt3,
//  (PKSDATAFORMAT)&OutVideoPort_Fmt4,
//  (PKSDATAFORMAT)&OutVideoPort_Fmt5,
//  (PKSDATAFORMAT)&OutVideoPort_Fmt6,
};

// define the Close Caption output pin format
static const KSDATAFORMAT OutCloseCaption_Fmt0= {
  sizeof(KSDATAFORMAT),
  0,
  200,
  0,
  STATIC_KSDATAFORMAT_TYPE_AUXLine21Data,
  STATIC_KSDATAFORMAT_SUBTYPE_Line21_GOPPacket,
  STATIC_KSDATAFORMAT_SPECIFIER_VIDEOINFO
};

static const KSDATAFORMAT OutCloseCaption_Fmt1= {
  sizeof(KSDATAFORMAT),
  0,
  200,
  0,
  STATIC_KSDATAFORMAT_TYPE_AUXLine21Data,
  STATIC_KSDATAFORMAT_SUBTYPE_Line21_GOPPacket,
  STATIC_KSDATAFORMAT_SPECIFIER_NONE
};

static const KSDATAFORMAT OutCloseCaption_Fmt2= {
  sizeof(KSDATAFORMAT),
  0,
  200,
  0,
  STATIC_KSDATAFORMAT_TYPE_AUXLine21Data,
  STATIC_KSDATAFORMAT_SUBTYPE_Line21_BytePair,
  STATIC_KSDATAFORMAT_SPECIFIER_VIDEOINFO
};

static const KSDATAFORMAT OutCloseCaption_Fmt3= {
  sizeof(KSDATAFORMAT),
  0,
  200,
  0,
  STATIC_KSDATAFORMAT_TYPE_AUXLine21Data,
  STATIC_KSDATAFORMAT_SUBTYPE_Line21_BytePair,
  STATIC_KSDATAFORMAT_SPECIFIER_NONE
};

static const PKSDATAFORMAT OutCloseCaption_Fmt[]= {
  (PKSDATAFORMAT)&OutCloseCaption_Fmt0,
  //(PKSDATAFORMAT)&OutCloseCaption_Fmt2
  (PKSDATAFORMAT)&OutCloseCaption_Fmt1
  //(PKSDATAFORMAT)&OutCloseCaption_Fmt3
};

static const struct stream_format_struct {
  const PKSDATAFORMAT *StreamFormatArray;
  int NumberOfEntries;
} StreamFormats[]= {
  { InMpeg2Video_Fmt,      SIZEOF_ARRAY(InMpeg2Video_Fmt)      },
  { InMpeg2Ac3_Fmt,        SIZEOF_ARRAY(InMpeg2Ac3_Fmt)        },
#if TVOUTPIN
  { OutNTSCVideo_Fmt,      SIZEOF_ARRAY(OutNTSCVideo_Fmt)      },
#endif
  { InMpeg2Subpicture_Fmt, SIZEOF_ARRAY(InMpeg2Subpicture_Fmt) },
  { OutVideoPort_Fmt,      SIZEOF_ARRAY(OutVideoPort_Fmt)      },
#if CCOUTPIN
  { OutCloseCaption_Fmt,   SIZEOF_ARRAY(OutCloseCaption_Fmt)   }
#endif
};

// define the copy protection property support  This Property is supported in
// all of our streams
static const KSPROPERTY_ITEM CopyProtProperties[] = {
  {
    KSPROPERTY_DVDCOPY_CHLG_KEY,// DVD authentication challenge key
    (PFNKSHANDLER)TRUE,      // get property on challenge key requests the
                         // decoder to provide it's challenge key
    sizeof(KSPROPERTY),
    sizeof(KS_DVDCOPY_CHLGKEY), // minimum size of data requested
    (PFNKSHANDLER) TRUE,     // set palette is supported
    0,
    0,
    0,
    0,
    0
  },
  {
    KSPROPERTY_DVDCOPY_DVD_KEY1,// DVD authentication DVD drive key property
    FALSE,                   // get Key not supported
    sizeof(KSPROPERTY),
    sizeof(KS_DVDCOPY_BUSKEY),// minimum size of data requested
    (PFNKSHANDLER) TRUE,     // set key provides the key for the decoder
    0,
    0,
    0,
    0,
    0
  },
  {
    KSPROPERTY_DVDCOPY_DEC_KEY2,// DVD authentication DVD decoder key property
    (PFNKSHANDLER)TRUE,      // get Key requests the decoder key, in
                         // response to a previous set challenge key
    sizeof(KSPROPERTY),
    sizeof(KS_DVDCOPY_BUSKEY),   // minimum size of data requested
    (PFNKSHANDLER)FALSE,     // set key is not valid
    0,
    0,
    0,
    0,
    0
  },
#if INCLUDE_REGIONCODE
  {
    KSPROPERTY_DVDCOPY_REGION,  // DVD region request
                         // the minidriver shall fit in exactly
                         // one region bit, corresponding to the region
                            // that the decoder is currently in
    (PFNKSHANDLER)TRUE,
    sizeof(KSPROPERTY),
    sizeof(KS_DVDCOPY_REGION),   // minimum size of data requested
    0,     // set key is not valid
    0,
    0,
    0,
    0,
    0
  },
#endif
  {
    KSPROPERTY_DVDCOPY_TITLE_KEY,// DVD authentication DVD title key property
    FALSE,                   // get Key not supported
    sizeof(KSPROPERTY),
    sizeof(KS_DVDCOPY_TITLEKEY),// minimum size of data requested
    (PFNKSHANDLER) TRUE,     // set key provides the key for the decoder
    0,
    0,
    0,
    0,
    0
  },
  {
    KSPROPERTY_DVDCOPY_DISC_KEY, // DVD authentication DVD disc key property
    FALSE,                   // get Key not supported
    sizeof(KSPROPERTY),
    sizeof(KS_DVDCOPY_BUSKEY),// minimum size of data requested
    (PFNKSHANDLER) TRUE,     // set key provides the key for the decoder
    0,
    0,
    0,
    0,
    0
  },
  {
    KSPROPERTY_DVDCOPY_SET_COPY_STATE,
    (PFNKSHANDLER)TRUE,                 // get Key supported
    sizeof(KSPROPERTY),
    sizeof(KS_DVDCOPY_SET_COPY_STATE),  // minimum size of data requested
    (PFNKSHANDLER)TRUE,                 // set key supported
    0,
    0,
    0,
    0,
    0
  }
};

// Rate Change
static const KSPROPERTY_ITEM RateChangePropItm[]= {
  {
    KS_AM_RATE_SimpleRateChange,
    (PFNKSHANDLER) TRUE,
    sizeof(KSPROPERTY),
    sizeof(KS_AM_SimpleRateChange),
    (PFNKSHANDLER) TRUE,
    0,
		0,
		NULL,
		NULL,
		0,
	},


	{
		KS_AM_RATE_ExactRateChange,
		(PFNKSHANDLER) FALSE,
		sizeof (KSPROPERTY),
		sizeof (KS_AM_ExactRateChange),
		(PFNKSHANDLER) FALSE,
		NULL,
		0,
		NULL,
		NULL,
		0,
	},


	{
		KS_AM_RATE_MaxFullDataRate,
		(PFNKSHANDLER) TRUE,
		sizeof (KSPROPERTY),
		sizeof (KS_AM_MaxFullDataRate),
		(PFNKSHANDLER) FALSE,
		NULL,
		0,
		NULL,
		NULL,
		0,
	},


	{
		KS_AM_RATE_Step,
		(PFNKSHANDLER) FALSE,
		sizeof (KSPROPERTY),
		sizeof (KS_AM_Step),
		(PFNKSHANDLER) TRUE,
		NULL,
		0,
		NULL,
		NULL,
		0,
	}
};



//
// MPEG2 video Properties definition
//

static const KSPROPERTY_ITEM InMpeg2Video_Properties[] =
   {

      {
         KSPROPERTY_MPEG2VID_MODES,    // available output modes of decoder
         (PFNKSHANDLER)TRUE,                          // get palette not supported
         sizeof(KSPROPERTY),
         sizeof(KSPROPERTY),
         (PFNKSHANDLER) FALSE,         // set palette is supported
         NULL,
         0,
         NULL,
         NULL,
         0
      },
      {
         KSPROPERTY_MPEG2VID_CUR_MODE, // current mode of the decoder
         (PFNKSHANDLER)TRUE,                          // get palette not supported
         sizeof(KSPROPERTY),
         sizeof(KSPROPERTY),
         (PFNKSHANDLER) FALSE,         // set palette is supported
         NULL,
         0,
         NULL,
         NULL,
         0
      },
      {
         KSPROPERTY_MPEG2VID_4_3_RECT, // output coordinates for 4:3 source
         (PFNKSHANDLER)TRUE,                          // get palette not supported
         sizeof(KSPROPERTY),
         sizeof(KSPROPERTY),
         (PFNKSHANDLER) FALSE,         // set palette is supported
         NULL,
         0,
         NULL,
         NULL,
         0
      },
      {
         KSPROPERTY_MPEG2VID_16_9_RECT,      // output coordinates for 16:9 source
         (PFNKSHANDLER)TRUE,                          // get palette not supported
         sizeof(KSPROPERTY),
         sizeof(KSPROPERTY),
         (PFNKSHANDLER) FALSE,         // set palette is supported
         NULL,
         0,
         NULL,
         NULL,
         0
      },
      {
         KSPROPERTY_MPEG2VID_16_9_PANSCAN,   // pan and scan vectors
         (PFNKSHANDLER)TRUE,                          // get palette not supported
         sizeof(KSPROPERTY),
         sizeof(KSPROPERTY),
         (PFNKSHANDLER) FALSE,         // set palette is supported
         NULL,
         0,
         NULL,
         NULL,
         0
      }
   };

static const KSPROPERTY_SET InMpeg2Video_PropertySet[] = {
   {
      &KSPROPSETID_Mpeg2Vid,
      SIZEOF_ARRAY(InMpeg2Video_Properties),
      (PKSPROPERTY_ITEM) InMpeg2Video_Properties
   },
   {
      &KSPROPSETID_CopyProt,
      SIZEOF_ARRAY(CopyProtProperties),
      (PKSPROPERTY_ITEM) CopyProtProperties
   },
   {   
      &KSPROPSETID_TSRateChange,
      SIZEOF_ARRAY(RateChangePropItm),
      (PKSPROPERTY_ITEM)RateChangePropItm
   }

};


//
//  AC3 audio Properties definition
//


static const KSPROPERTY_ITEM InMpeg2Ac3_Properties[] = {

    {KSPROPERTY_AUDDECOUT_MODES,// available audio decoder output formats
        // property
        (PFNKSHANDLER) TRUE,    // get available modes is supported
        sizeof(KSPROPERTY),
        sizeof(ULONG),          // minimum size of data requested
        (PFNKSHANDLER) FALSE,   // set available modes is not supported
        NULL,
        0,
        NULL,
        NULL,
        0
    },

    {KSPROPERTY_AUDDECOUT_CUR_MODE, // current audio decoder output format
        // property
        (PFNKSHANDLER) TRUE,    // get current mode is supported
        sizeof(KSPROPERTY),
        sizeof(ULONG),          // minimum size of data requested
        (PFNKSHANDLER) TRUE,    // set current modes is supported
        NULL,
        0,
        NULL,
        NULL,
        0
    }
};



static const KSPROPERTY_SET InMpeg2Ac3_PropertySet[] = {
   {
      &KSPROPSETID_AudioDecoderOut,
      SIZEOF_ARRAY(InMpeg2Ac3_Properties),
      (PKSPROPERTY_ITEM) InMpeg2Ac3_Properties
   },
   {
      &KSPROPSETID_CopyProt,
      SIZEOF_ARRAY(CopyProtProperties),
      (PKSPROPERTY_ITEM) CopyProtProperties
   },
   {   
      &KSPROPSETID_TSRateChange,
      SIZEOF_ARRAY(RateChangePropItm),
      (PKSPROPERTY_ITEM)RateChangePropItm
   }

};

//
// property set for Macrovision support
//

static const KSPROPERTY_ITEM MacroVisionProperties[] = {
	{
		KSPROPERTY_COPY_MACROVISION,		// support for setting macrovision level
		(PFNKSHANDLER) FALSE, 		// get not supported
		sizeof (KSPROPERTY),
		sizeof (KS_COPY_MACROVISION),
		(PFNKSHANDLER) TRUE,		// set MACROVISION level supported
		NULL,
		0,
		NULL,
		NULL,
		0
	}
};

//
// define the NTSC output Properties
//

static const KSPROPERTY_SET OutNTSCVideo_PropertySet[] = {
   {
      &KSPROPSETID_CopyProt,
      SIZEOF_ARRAY(MacroVisionProperties),
      (PKSPROPERTY_ITEM) MacroVisionProperties
   }
};


//
// define the subpicture property items supported
//

static const KSPROPERTY_ITEM InMpeg2Subpicture_Properties[] = {

    {KSPROPERTY_DVDSUBPIC_PALETTE,  // subpicture palette property
        (PFNKSHANDLER) TRUE,         
        sizeof(KSPROPERTY),
        sizeof(KSPROPERTY_SPPAL),   // minimum size of data requested
        (PFNKSHANDLER) TRUE,    // set palette is supported
        NULL,
        0,
        NULL,
        NULL,
        0
    },


    {KSPROPERTY_DVDSUBPIC_HLI,  // subpicture highlight property
        (PFNKSHANDLER) TRUE,
        sizeof(KSPROPERTY),
        sizeof(KSPROPERTY_SPHLI),   // minimum size of data requested
        (PFNKSHANDLER) TRUE,    // set highlight is supported
        NULL,
        0,
        NULL,
        NULL,
        0
    },


    {KSPROPERTY_DVDSUBPIC_COMPOSIT_ON,  // subpicture enable status property
        (PFNKSHANDLER) TRUE,  
        sizeof(KSPROPERTY),
        sizeof(KSPROPERTY_COMPOSIT_ON), // minimum size of data requested
        (PFNKSHANDLER) TRUE,    // set enable status is supported
        NULL,
        0,
        NULL,
        NULL,
        0
    }
};

static const KSPROPERTY_SET InMpeg2Subpicture_PropSet[] = {
   {
      &KSPROPSETID_DvdSubPic,
      SIZEOF_ARRAY(InMpeg2Subpicture_Properties),
      (PKSPROPERTY_ITEM) InMpeg2Subpicture_Properties
   },

   {
      &KSPROPSETID_CopyProt,
      SIZEOF_ARRAY(CopyProtProperties),
      (PKSPROPERTY_ITEM) CopyProtProperties
   },
   {   
      &KSPROPSETID_TSRateChange,
      SIZEOF_ARRAY(RateChangePropItm),
      (PKSPROPERTY_ITEM)RateChangePropItm
   }

};


//
// property set for Video Port support
//

static const KSPROPERTY_ITEM VideoPortProperties[] = {

  {KSPROPERTY_VPCONFIG_NUMCONNECTINFO,
      (PFNKSHANDLER) TRUE,
      sizeof(KSPROPERTY),
      sizeof(ULONG),
      (PFNKSHANDLER) FALSE,
      NULL,
      0,
      NULL,
      NULL,
      0
  },

  {KSPROPERTY_VPCONFIG_NUMVIDEOFORMAT,
      (PFNKSHANDLER) TRUE,
      sizeof(KSPROPERTY),
      sizeof(ULONG),
      (PFNKSHANDLER) FALSE,
      NULL,
      0,
      NULL,
      NULL,
      0
  },

	{KSPROPERTY_VPCONFIG_GETCONNECTINFO,
   	(PFNKSHANDLER) TRUE,
   	sizeof (KSMULTIPLE_DATA_PROP),  // minimum property input size
   	sizeof (ULONG),    				
   	(PFNKSHANDLER)FALSE,
   	NULL,
   	0,
   	NULL,
   	NULL,
   	0
	},

	{KSPROPERTY_VPCONFIG_SETCONNECTINFO,
   	(PFNKSHANDLER) FALSE,
   	sizeof (KSPROPERTY),  			// minimum property input size
   	sizeof (ULONG),    				// minimum buffer size
   	(PFNKSHANDLER)TRUE,
   	NULL,
   	0,
   	NULL,
   	NULL,
   	0
	},

	{KSPROPERTY_VPCONFIG_VPDATAINFO,
	   (PFNKSHANDLER) TRUE,
	   sizeof (KSPROPERTY),
	   sizeof (KS_AMVPDATAINFO),
	   (PFNKSHANDLER)FALSE,
	   NULL,
	   0,
	   NULL,
	   NULL,
	   0
   },

	{KSPROPERTY_VPCONFIG_MAXPIXELRATE,
	   (PFNKSHANDLER) TRUE,
   	sizeof (KSVPSIZE_PROP),
   	sizeof (KSVPMAXPIXELRATE),
   	(PFNKSHANDLER)FALSE,
   	NULL,
   	0,
   	NULL,
   	NULL,
   	0
  	},

	{KSPROPERTY_VPCONFIG_INFORMVPINPUT,
   	(PFNKSHANDLER) TRUE,
   	sizeof (PKSPROPERTY),
   	sizeof (DDPIXELFORMAT),    // could be 0 too
   	(PFNKSHANDLER)FALSE,
   	NULL,
   	0,
   	NULL,
   	NULL,
   	0
	},

	{KSPROPERTY_VPCONFIG_DDRAWHANDLE,
   	(PFNKSHANDLER)FALSE,
   	sizeof (PKSPROPERTY),
   	sizeof (ULONG),    // could be 0 too
	   (PFNKSHANDLER) TRUE,
   	NULL,
	   0,
	   NULL,
	   NULL,
	   0
	},

	{KSPROPERTY_VPCONFIG_VIDEOPORTID,
	   (PFNKSHANDLER)FALSE,
	   sizeof (PKSPROPERTY),
	   sizeof (ULONG),    // could be 0 too
	   (PFNKSHANDLER) TRUE,
	   NULL,
	   0,
	   NULL,
	   NULL,
	   0
	},

	{KSPROPERTY_VPCONFIG_DDRAWSURFACEHANDLE,
   	(PFNKSHANDLER)FALSE,
	   sizeof (PKSPROPERTY),
	   sizeof (ULONG),    // could be 0 too
	   (PFNKSHANDLER) TRUE,
	   NULL,
	   0,
	   NULL,
	   NULL,
	   0
	},

	{KSPROPERTY_VPCONFIG_GETVIDEOFORMAT,
	   (PFNKSHANDLER) TRUE,
	   sizeof (KSMULTIPLE_DATA_PROP), 		// for _GET; KSPROPERTY for _SET
	   sizeof (ULONG),        		// could be 4 or more
	   (PFNKSHANDLER)FALSE,
	   NULL,
	   0,
	   NULL,
	   NULL,
	   0
	},

	{KSPROPERTY_VPCONFIG_SETVIDEOFORMAT,
	   (PFNKSHANDLER) FALSE,
	   sizeof (KSPROPERTY),  			// minimum property input size
	   sizeof (ULONG),    				// minimum buffer size
   	(PFNKSHANDLER)TRUE,  
   	NULL,
   	0,
	   NULL, 
	   NULL,                                                                                                                                                                                                                
	   0
	},


	{KSPROPERTY_VPCONFIG_INVERTPOLARITY,
//	   (PFNKSHANDLER)TRUE,
	   (PFNKSHANDLER)FALSE,
	   sizeof (KSPROPERTY),
	   0,
//	   (PFNKSHANDLER)FALSE,
	   (PFNKSHANDLER)TRUE,
	   NULL,
	   0,
	   NULL,
	   NULL,
	   0
	},

	{KSPROPERTY_VPCONFIG_DECIMATIONCAPABILITY,
	   (PFNKSHANDLER)TRUE,
	   sizeof (KSPROPERTY), 
	   sizeof (BOOL), 
	   (PFNKSHANDLER)FALSE,
	   NULL,
	   0, 
	   NULL,
	   NULL,
	   0
	},

	{KSPROPERTY_VPCONFIG_SCALEFACTOR,
//	   (PFNKSHANDLER)TRUE,
	   (PFNKSHANDLER)FALSE,
	   sizeof (KSPROPERTY),
	   sizeof (KS_AMVPSIZE),
//	   (PFNKSHANDLER)FALSE,
	   (PFNKSHANDLER)TRUE,
	   NULL,
	   0,
	   NULL,
	   NULL,
   	0
	},

	{KSPROPERTY_VPCONFIG_INFORMVPINPUT,
   	(PFNKSHANDLER) TRUE,
   	sizeof (PKSPROPERTY),
   	sizeof (DDPIXELFORMAT),    // could be 0 too
   	(PFNKSHANDLER)FALSE,
   	NULL,
   	0,
   	NULL,
   	NULL,
   	0
	},
#if(0)
  {KSPROPERTY_VPCONFIG_SURFACEPARAMS,
   	(PFNKSHANDLER) TRUE,
   	sizeof (PKSPROPERTY),
   	sizeof (DDPIXELFORMAT),    // could be 0 too
   	(PFNKSHANDLER)TRUE,
   	NULL,
   	0,
   	NULL,
   	NULL,
   	0
	},
#endif

};


static const KSPROPERTY_ITEM VPEMacroVisionProperties[] = {
	{
		KSPROPERTY_COPY_MACROVISION,		// support for setting macrovision level
		(PFNKSHANDLER) FALSE, 		// get not supported
		sizeof (KSPROPERTY),
		sizeof (KS_COPY_MACROVISION),
		(PFNKSHANDLER) TRUE,		// set MACROVISION level supported
		NULL,
		0,
		NULL,
		NULL,
		0
	}
};


static const KSPROPERTY_SET OutVideoPort_PropSet[] = {
  {
    &KSPROPSETID_VPConfig,
    SIZEOF_ARRAY(VideoPortProperties),
    (PKSPROPERTY_ITEM) VideoPortProperties
  },
  {
    &KSPROPSETID_CopyProt,
    SIZEOF_ARRAY(VPEMacroVisionProperties),
    (PKSPROPERTY_ITEM) VPEMacroVisionProperties
  }
};

//
// property set for CC support
//

static const KSPROPERTY_ITEM OutCloseCaption_Properties[] = {
   {
      KSPROPERTY_CONNECTION_ALLOCATORFRAMING,   // support for setting CC buffer size
      (PFNKSHANDLER) TRUE,                // get supported
      sizeof (KSPROPERTY),
      sizeof (KSALLOCATOR_FRAMING),
      (PFNKSHANDLER) FALSE,               // we only provide the allocator requirments
      NULL,
      0,
      NULL,
      NULL,
      0
   },
   {
      KSPROPERTY_CONNECTION_STATE,        // support for setting CC buffer size
      (PFNKSHANDLER) TRUE,                // get supported
      sizeof (KSPROPERTY),
      sizeof (KSSTATE),
      (PFNKSHANDLER) FALSE,               // we only provide the allocator requirments
      NULL,
      0,
      NULL,
      NULL,
      0
   }
};

static const KSPROPERTY_SET OutCloseCaption_PropSet[] = {
   &KSPROPSETID_Connection,
   SIZEOF_ARRAY(OutCloseCaption_Properties),
   (PKSPROPERTY_ITEM) OutCloseCaption_Properties
};



//
// define the events associated with the master clock
//

static const KSEVENT_ITEM ClockEventItm[] =
{
   {
      KSEVENT_CLOCK_POSITION_MARK,     // position mark event supported
      sizeof (KSEVENT_TIME_MARK),         // requires this data as input
       sizeof (KSEVENT_TIME_MARK),         // allocate space to copy the data
      NULL,
      NULL,
      NULL
   },
   {
      KSEVENT_CLOCK_INTERVAL_MARK,     // interval mark event supported
      sizeof (KSEVENT_TIME_INTERVAL),  // requires interval data as input
      sizeof (WORKTIME),               // requires interval data as input
                                       // size longlong for processing
                                       // this event
      NULL,
      NULL,
      NULL
   }
};

static const KSEVENT_SET ClockEventSet[] =
{
   {
      &KSEVENTSETID_Clock,
      SIZEOF_ARRAY(ClockEventItm),
      (PKSEVENT_ITEM)ClockEventItm
   }
};

//
// define the event associated with the Video Port
//
static const KSEVENT_ITEM VPEventItm[] =
{
	{
		KSEVENT_VPNOTIFY_FORMATCHANGE,
		0,
		0,
		NULL,
		NULL,
		NULL
	}
};


static const KSEVENT_SET VPEventSet[] =
{
	{
		&KSEVENTSETID_VPNotify,
		SIZEOF_ARRAY(VPEventItm),
		VPEventItm,
	}
};

static const KSTOPOLOGY_CONNECTION Connections[] = {
//  FromNode            FromPin           ToNode            ToPin
//  Providing the CC info here cause the WHQL Exclusive Mode playback test to fail
//   {(ULONG)(-1),        InMpeg2Video,     (ULONG)(-1),      OutCloseCaption},
   {(ULONG)(-1),        InMpeg2Video,     (ULONG)(-1),      OutVideoPort},
   {(ULONG)(-1),        InMpeg2Subpicture,(ULONG)(-1),      OutVideoPort}
};

static const KSTOPOLOGY Topology = {
    1,
    (GUID*)&KSCATEGORY_DATADECOMPRESSOR,
    0,
    NULL,
    SIZEOF_ARRAY(Connections),
    Connections
};


void StreamOpen_Mpeg2(SRB* pSrb);
void StreamOpen_Ac3(SRB* pSrb);
void StreamOpen_Ntsc(SRB* pSrb);
void StreamOpen_Subpict(SRB* pSrb);
void StreamOpen_VP(SRB* pSrb);
void StreamOpen_CC(SRB* pSrb);
void StreamOpen_Unknown(SRB* pSrb);
void StreamClose_Mpeg2(SRB* pSrb);
void StreamClose_Ac3(SRB* pSrb);
void StreamClose_Ntsc(SRB* pSrb);
void StreamClose_Subpict(SRB* pSrb);
void StreamClose_VP(SRB* pSrb);
void StreamClose_CC(SRB* pSrb);
void StreamClose_Unknown(SRB* pSrb);

/*
** StreamOpen()
**
**   This routine is called when an OpenStream SRB request is received By the Device
**
** Arguments:
**
**   pSrb - pointer to stream request block for the Open command
**
** Returns:
**
** Side Effects:  none
*/
void StreamOpenLowPriority(PHW_STREAM_REQUEST_BLOCK pSrb);

void StreamOpen(PHW_STREAM_REQUEST_BLOCK pSrb) {
  PDEVICE_EXTENSION pDevice= (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  pSrb->Status= STATUS_PENDING;
#if KS_SYNC_OFF
  StreamOpenLowPriority(pSrb);
#else
  StreamClassCallAtNewPriority(pSrb->StreamObject,pDevice,Low,StreamOpenLowPriority,pSrb);
#endif
}

void SignalDeviceRequestComplete(SRB* pSrb) {
#if !KS_SYNC_OFF
  StreamClassDeviceNotification(ReadyForNextDeviceRequest,pSrb->HwDeviceExtension);
#endif
  StreamClassDeviceNotification(DeviceRequestComplete,pSrb->HwDeviceExtension,pSrb);
}

void SignalStreamRequestCompleteReadyForNextData(SRB* pSrb) {
  PHW_STREAM_OBJECT pStrm=pSrb->StreamObject;
  StreamClassStreamNotification(StreamRequestComplete,pStrm,pSrb);
#if !KS_SYNC_OFF
  StreamClassStreamNotification(ReadyForNextStreamDataRequest,pStrm);
#endif
}

void SignalStreamRequestCompleteReadyForNextControl(SRB* pSrb) {
  PHW_STREAM_OBJECT pStrm=pSrb->StreamObject;
  StreamClassStreamNotification(StreamRequestComplete,pStrm,pSrb);
#if !KS_SYNC_OFF
  StreamClassStreamNotification(ReadyForNextStreamControlRequest,pStrm);
#endif
}

void SignalStreamRequestComplete(SRB* pSrb) {
  StreamClassStreamNotification(StreamRequestComplete,pSrb->StreamObject,pSrb);
}

void SignalStreamReadyForNextControl(SRB* pSrb) {
#if !KS_SYNC_OFF
  StreamClassStreamNotification(ReadyForNextStreamControlRequest,pSrb->StreamObject);
#endif
}

void SignalStreamReadyForNextData(SRB* pSrb) {
#if !KS_SYNC_OFF
  StreamClassStreamNotification(ReadyForNextStreamDataRequest,pSrb->StreamObject);
#endif
}

void __stdcall StreamDataCallback(PMDS_DATA_BLOCK pMdb) {
  SRB* pSrb=(SRB*)pMdb->hRef;
  if(pSrb) {
    switch(pMdb->dwStatus) { 
      case MDS_Success: pSrb->Status= STATUS_SUCCESS;     break;
      case MDS_Cancel:  pSrb->Status= STATUS_CANCELLED;   break;
      default:          pSrb->Status= STATUS_UNSUCCESSFUL;break;
    }

#ifdef _WIN98_ONLY_
    if(pSrb->Status==STATUS_CANCELLED)
      StreamClassCompleteRequestAndMarkQueueReady(pSrb);
    else
      StreamClassStreamNotification(StreamRequestComplete,pSrb->StreamObject,pSrb);
#else
    StreamClassStreamNotification(StreamRequestComplete,pSrb->StreamObject,pSrb);
#endif
  }
}

void __stdcall StreamControlCallback(PMDS_DATA_BLOCK pMdb) {
  SRB* pSrb=(SRB*)pMdb->hRef;
  PDEVICE_EXTENSION pDevice=(PDEVICE_EXTENSION)pSrb->HwDeviceExtension;
  PSTREAM_EXTENSION pStream=(PSTREAM_EXTENSION)pSrb->StreamObject->HwStreamExtension;
  if(pSrb) {
    switch(pMdb->dwStatus) {
      case MDS_Success: pSrb->Status= STATUS_SUCCESS;     break;
      case MDS_Cancel:  pSrb->Status= STATUS_CANCELLED;   break;
      default:          pSrb->Status= STATUS_UNSUCCESSFUL;break;
    }
    SignalStreamRequestCompleteReadyForNextControl(pSrb);
  }
}

void CancelStream(PVOID hMidas,int StrmId) {
  MIDASCMD Cmd;
  Cmd.hMidas= hMidas;
  Cmd.cmd =   MDS_CANCELDATA;
  Cmd.wCurrStream=StrmId;
  Cmd.u.pMdb=0;           // Cancel all SRB's in the stream
  MidasEntry(&Cmd);
}

void StreamSetState(PSTREAM_EXTENSION pStream,KSSTATE State) {
  if(pStream->stream_state!=State) {
    pStream->prev_stream_state=pStream->stream_state;
    pStream->stream_state=State;
    pStream->dwStateChangeTime=MargiGetTime();  
  }
}

void StreamOpenLowPriority(PHW_STREAM_REQUEST_BLOCK pSrb) {
  PHW_STREAM_OBJECT pStreamObj;
  PSTREAM_EXTENSION pStream;
  PDEVICE_EXTENSION pDevice;
  MIDASCMD Cmd;

  pStreamObj= pSrb->StreamObject;
  pStream= (PSTREAM_EXTENSION)pStreamObj->HwStreamExtension;
  pDevice= (PDEVICE_EXTENSION)pSrb->HwDeviceExtension;

  // Assume successful completion   ???? WHY WHY 
  pSrb->Status= STATUS_SUCCESS;
  pStreamObj->Pio= TRUE;
  pStreamObj->Dma= FALSE;
  RtlZeroMemory(pStream, sizeof(STREAM_EXTENSION));
  pDevice->pStream[pStreamObj->StreamNumber]= pStreamObj;
  pStream->pDevice= pDevice;
//  pStream->queue.StreamNumber=pStreamObj->StreamNumber;
  pStream->nCssKeyPhase=pDevice->nCssKeyPhase;
  pStream->prev_stream_state=KSSTATE_STOP;
  pStream->stream_state=KSSTATE_STOP;

  if(!pDevice->StreamsOpened) {
    DrvDbgMsgInit(1);		    //No harm if already opened.
    gTimerThreshold= SettingLoad(SET_DWORD,IniTimerThreshold,0,20000);
    gTimerPeriod= SettingLoad(SET_DWORD,IniTimerPeriod,0,gTimerPeriod);
    gDataSendMaxTime= SettingLoad(SET_DWORD,IniDataSendMaxTime,0,gTimerPeriod/2);
    QPktTotal= 0;
    QPktInQueue= 0;
    if(gDataSendMaxTime>=gTimerPeriod)
      gDataSendMaxTime= (gTimerPeriod*8)/10;
    Cmd.cmd= MDS_INSTALLDEVICE;
    Cmd.hMidas=pDevice->hMidas;                    
    //StreamClassCallAtNewPriority(0,pDevice,Low,MidasEntry,&Cmd);
    MidasEntry(&Cmd);

    Cmd.cmd=MDS_SET_TIME_MODE;
    Cmd.u.M.Arg1=MDS_STF_10M|MDS_STF_USE_MDP;
    MidasEntry(&Cmd);
    EnableIBMAcer();

    pDevice->StreamsOpened++;
#if USE_LUX_TIMER
    LuxTheTimer(pDevice);
#else
    TimerStart(1,pDevice);         //begins the Timer. Timer will stop by itself when pDevice->StreamsOpened==0
    //TheTimer(pDevice);         //begins the Timer. Timer will stop by itself when pDevice->StreamsOpened==0
#endif
  } else {
    pDevice->StreamsOpened++;
  }

  switch(pStreamObj->StreamNumber) {
    case InMpeg2Video:        StreamOpen_Mpeg2(pSrb); break;
    case InMpeg2Ac3:          StreamOpen_Ac3(pSrb); break;
#if TVOUTPIN
    case OutNTSCVideo:        StreamOpen_Ntsc(pSrb); break;
#endif
    case InMpeg2Subpicture:   StreamOpen_Subpict(pSrb); break;
    case OutVideoPort:        StreamOpen_VP(pSrb); break;
#if CCOUTPIN
    case OutCloseCaption:     StreamOpen_CC(pSrb); break;
#endif
    default:                  StreamOpen_Unknown(pSrb); break;
  }
#if KS_SYNC_OFF
  StreamClassDeviceNotification(DeviceRequestComplete,pSrb->HwDeviceExtension,pSrb);
#else
  StreamClassCallAtNewPriority(0,pDevice,LowToHigh,SignalDeviceRequestComplete,pSrb);
#endif
}

void StreamClose(PHW_STREAM_REQUEST_BLOCK pSrb) {
  PHW_STREAM_OBJECT pStreamObj;
  PDEVICE_EXTENSION pDevice;
  MIDASCMD Cmd;

  pSrb->Status= STATUS_SUCCESS;
  pStreamObj= pSrb->StreamObject;
  pDevice= (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  pDevice->StreamsOpened--;
//  DisnableIBMAcer();
  if(pDevice->StreamsOpened <= 0) {
    if(pDevice->StreamsOpened) {
      DEBUG_BREAKPOINT();
    }
    Cmd.cmd= MDS_UNINSTALLDEVICE;
    Cmd.hMidas=pDevice->hMidas;                    
    MidasEntry(&Cmd);
  }
  switch(pStreamObj->StreamNumber) {
    case InMpeg2Video:        StreamClose_Mpeg2(pSrb); break;
    case InMpeg2Ac3:          StreamClose_Ac3(pSrb); break;
#if TVOUTPIN
    case OutNTSCVideo:        StreamClose_Ntsc(pSrb); break;
#endif
    case InMpeg2Subpicture:   StreamClose_Subpict(pSrb); break;
    case OutVideoPort:        StreamClose_VP(pSrb); break;
#if CCOUTPIN
    case OutCloseCaption:     StreamClose_CC(pSrb); break;
#endif
    default:                  StreamClose_Unknown(pSrb); break;
  }
}


/*
** StreamInfo()
**
**   Returns the information of all streams that are supported by the
**   mini-driver
**
** Arguments:
**
**   pSrb - Pointer to the STREAM_REQUEST_BLOCK
**        pSrb->HwDeviceExtension - will be the hardware device extension for
**                                  as initialised in HwInitialise
**
** Returns:
**
** Side Effects:  none
*/
void StreamInfo(PHW_STREAM_REQUEST_BLOCK pSrb) {
  PDEVICE_EXTENSION pDevice;
  PHW_STREAM_HEADER pstrhdr;
  PHW_STREAM_INFORMATION pstrinfo;

  pDevice= ((PDEVICE_EXTENSION) pSrb->HwDeviceExtension);
  pstrhdr= &pSrb->CommandData.StreamBuffer->StreamHeader;
  pstrinfo= &pSrb->CommandData.StreamBuffer->StreamInfo;
  pstrhdr->NumberOfStreams= NUMBER_OF_STREAMS;
  pstrhdr->SizeOfHwStreamInformation= sizeof(HW_STREAM_INFORMATION);

  // store a pointer to the topology for the device
  pstrhdr->Topology= (PKSTOPOLOGY)&Topology;

  // set up the stream info structures for the MPEG2 video
  pstrinfo->NumberOfPossibleInstances= 1;
  pstrinfo->DataFlow= KSPIN_DATAFLOW_IN;
  pstrinfo->DataAccessible= TRUE;
  pstrinfo->NumberOfFormatArrayEntries= SIZEOF_ARRAY(InMpeg2Video_Fmt);
  pstrinfo->StreamFormatsArray= (PKSDATAFORMAT *)InMpeg2Video_Fmt;
  pstrinfo->NumStreamPropArrayEntries= SIZEOF_ARRAY(InMpeg2Video_PropertySet);
  pstrinfo->StreamPropertiesArray= (PKSPROPERTY_SET) InMpeg2Video_PropertySet;
  pstrinfo->Name= (GUID *)&VideoPin;
  pstrinfo->Category= pstrinfo->Name;
  pstrinfo++;

  // set up the stream info structures for the MPEG2 audio
  pstrinfo->NumberOfPossibleInstances= 1;
  pstrinfo->DataFlow= KSPIN_DATAFLOW_IN;
  pstrinfo->DataAccessible= TRUE;
  pstrinfo->NumberOfFormatArrayEntries= SIZEOF_ARRAY(InMpeg2Ac3_Fmt);
  pstrinfo->StreamFormatsArray= (PKSDATAFORMAT *)InMpeg2Ac3_Fmt;
  pstrinfo->NumStreamPropArrayEntries= SIZEOF_ARRAY(InMpeg2Ac3_PropertySet);
  pstrinfo->StreamPropertiesArray= (PKSPROPERTY_SET)InMpeg2Ac3_PropertySet ;
  pstrinfo->NumStreamEventArrayEntries= SIZEOF_ARRAY(ClockEventSet);
  pstrinfo->StreamEventsArray= (PKSEVENT_SET)ClockEventSet;
  pstrinfo->Name= (GUID *)&AudioPin;
  pstrinfo->Category= pstrinfo->Name;
  pstrinfo++;

#if TVOUTPIN
  // set up the stream info structures for the MPEG2 NTSC stream
  pstrinfo->NumberOfPossibleInstances= 1;
  pstrinfo->DataFlow= KSPIN_DATAFLOW_OUT;
  pstrinfo->DataAccessible= FALSE;
  pstrinfo->NumberOfFormatArrayEntries= SIZEOF_ARRAY(OutNTSCVideo_Fmt);
  pstrinfo->StreamFormatsArray= (PKSDATAFORMAT *)OutNTSCVideo_Fmt;
  pstrinfo->NumStreamPropArrayEntries= SIZEOF_ARRAY(OutNTSCVideo_PropertySet);
  pstrinfo->StreamPropertiesArray= (PKSPROPERTY_SET) OutNTSCVideo_PropertySet;
  pstrinfo->Name= (GUID *)&NTSCPin;
  pstrinfo->Category= pstrinfo->Name;
  pstrinfo++;
#endif

  // set up the stream info structures for the MPEG2 subpicture
  pstrinfo->NumberOfPossibleInstances= 1;
  pstrinfo->DataFlow= KSPIN_DATAFLOW_IN;
  pstrinfo->DataAccessible= TRUE;
  pstrinfo->NumberOfFormatArrayEntries= SIZEOF_ARRAY(InMpeg2Subpicture_Fmt);
  pstrinfo->StreamFormatsArray= (PKSDATAFORMAT *)InMpeg2Subpicture_Fmt;
  pstrinfo->NumStreamPropArrayEntries= SIZEOF_ARRAY(InMpeg2Subpicture_PropSet);
  pstrinfo->StreamPropertiesArray= (PKSPROPERTY_SET) InMpeg2Subpicture_PropSet;
  pstrinfo->Name= (GUID *)&SubpicturePin;
  pstrinfo->Category= pstrinfo->Name;
  pstrinfo++;

  // set up the stream info structures for the VIDEO Port
  pstrinfo->NumberOfPossibleInstances= 1;
  pstrinfo->DataFlow= KSPIN_DATAFLOW_OUT;
  pstrinfo->DataAccessible= FALSE;
  pstrinfo->NumberOfFormatArrayEntries= SIZEOF_ARRAY(OutVideoPort_Fmt);
  pstrinfo->StreamFormatsArray= (PKSDATAFORMAT *)OutVideoPort_Fmt;
  pstrinfo->NumStreamPropArrayEntries= SIZEOF_ARRAY(OutVideoPort_PropSet);
  pstrinfo->StreamPropertiesArray= (PKSPROPERTY_SET) OutVideoPort_PropSet;
  pstrinfo->NumStreamEventArrayEntries= SIZEOF_ARRAY(VPEventSet);
  pstrinfo->StreamEventsArray= (PKSEVENT_SET)VPEventSet;
  pstrinfo->Name= (GUID *)&VPEPin;
  pstrinfo->Category= pstrinfo->Name;
  pstrinfo->MediumsCount= 1;
  pstrinfo->Mediums= &VPMedium;

#if CCOUTPIN
  pstrinfo++;

  // set up the stream info structures for the Close Caption stream
  pstrinfo->NumberOfPossibleInstances= 1;
  pstrinfo->DataFlow= KSPIN_DATAFLOW_OUT;
  pstrinfo->DataAccessible= TRUE;
  pstrinfo->NumberOfFormatArrayEntries= SIZEOF_ARRAY(OutCloseCaption_Fmt);
  pstrinfo->StreamFormatsArray= (PKSDATAFORMAT *)OutCloseCaption_Fmt;
  pstrinfo->NumStreamPropArrayEntries= SIZEOF_ARRAY(OutCloseCaption_PropSet);
  pstrinfo->StreamPropertiesArray= (PKSPROPERTY_SET) OutCloseCaption_PropSet;
  pstrinfo->Name= (GUID *)&CCPin;
  pstrinfo->Category= pstrinfo->Name;
#endif
  pSrb->Status= STATUS_SUCCESS;

}

void StreamGetIntersection(PHW_STREAM_REQUEST_BLOCK pSrb) {
  NTSTATUS Status;
  PSTREAM_DATA_INTERSECT_INFO IntersectInfo;
  PKSDATARANGE   DataRange;
  PKSDATAFORMAT  pFormat;
  const PKSDATAFORMAT  *pFormatArray;
  int i,found;
  DWORD dwDbgFilt= 0;

  Status= STATUS_NOT_IMPLEMENTED;
  IntersectInfo= pSrb->CommandData.IntersectInfo;
  if(IntersectInfo->StreamNumber < NUMBER_OF_STREAMS) {
    switch(IntersectInfo->StreamNumber) {
      case InMpeg2Video:       dwDbgFilt= DBG_MSGVID; break;
      case InMpeg2Ac3:         dwDbgFilt= DBG_MSGAUD; break;
#if TVOUTPIN
      case OutNTSCVideo:       dwDbgFilt= DBG_MSGNTSC; break;
#endif
      case InMpeg2Subpicture:  dwDbgFilt= DBG_MSGSUB; break;
      case OutVideoPort:       dwDbgFilt= DBG_MSGVPE; break;
#if CCOUTPIN
      case OutCloseCaption:    dwDbgFilt= DBG_MSGCC; break;
#endif
      default:                 dwDbgFilt= DBG_WRN; break;
    }
    DataRange= IntersectInfo->DataRange;
    pFormatArray= StreamFormats[IntersectInfo->StreamNumber].StreamFormatArray;
    i= StreamFormats[IntersectInfo->StreamNumber].NumberOfEntries;
    if(i) {
      found= 0;
      do {
        pFormat= *pFormatArray++;
        if(IsEqualGUID(&DataRange->MajorFormat, &pFormat->MajorFormat) &&
           IsEqualGUID(&DataRange->SubFormat, &pFormat->SubFormat) &&
           IsEqualGUID(&DataRange->Specifier, &pFormat->Specifier) &&
           DataRange->FormatSize== pFormat->FormatSize) {
          found= 1;
        } else {
          i--;
        }
      } while(i && !found);
      if(found) {
        Status= STATUS_SUCCESS;
        if(IntersectInfo->SizeOfDataFormatBuffer== sizeof(ULONG)) {
          *(PULONG) IntersectInfo->DataFormatBuffer= pFormat->FormatSize;
          pSrb->ActualBytesTransferred= sizeof(ULONG);
          DbgMsg(dwDbgFilt," Requesting Info for %s. Setting the buf size only \r\n",\
                  Ident(ID_PIN_ID,IntersectInfo->StreamNumber));
        } else {
          if(IntersectInfo->SizeOfDataFormatBuffer >= pFormat->FormatSize) {
            RtlCopyMemory(IntersectInfo->DataFormatBuffer, pFormat, pFormat->FormatSize);
            pSrb->ActualBytesTransferred= pFormat->FormatSize;
            DbgMsg(DBG_INFO|dwDbgFilt,"Intersect Info requested (target values) ...\r\n");
            DbgDispDataIntersectInfo(dwDbgFilt,IntersectInfo);
          } else {
            Status= STATUS_BUFFER_TOO_SMALL;
            DbgMsg(DBG_INFO|dwDbgFilt," Target Buf Size smaller than the SrcBuf. \r\n");
          }
        }
      } else {
        Status= STATUS_NO_MATCH;
      }
    }
  }
  pSrb->Status= Status;
}

void InitStreamControlMdb(SRB* pSrb) {
  PHW_STREAM_OBJECT pStreamObj= pSrb->StreamObject;
  PSTREAM_EXTENSION pStream= (PSTREAM_EXTENSION)pStreamObj->HwStreamExtension;
  DWORD  Header[3];

  Header[0] = (DWORD) 0xe0010000;
  Header[1] = (DWORD) 0x00800000;
  Header[2] = 0;
  RtlCopyMemory(pStream->Data, Header, 10);
}


void StreamOpen_Mpeg2(SRB* pSrb) {
  PKSDATAFORMAT pfmt;
  KS_MPEGVIDEOINFO2 *pblock;
  MIDASCMD Cmd;
  PHW_STREAM_OBJECT pStreamObj= pSrb->StreamObject;
  PSTREAM_EXTENSION pStream= (PSTREAM_EXTENSION)pStreamObj->HwStreamExtension;
  PDEVICE_EXTENSION pDevice= (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;


  // Set Blank video to black
  Cmd.hMidas=pDevice->hMidas;
  Cmd.cmd=MDS_BLANKVIDEO;
  Cmd.u.M.Arg1=2;
  Cmd.u.M.Arg2=0;     
  MidasEntry(&Cmd);

  pStreamObj->ReceiveDataPacket= VideoReceive;
  pStreamObj->ReceiveControlPacket= VideoControl;
  pStreamObj->HwEventRoutine= 0; //VideoEvent;
  pfmt= pSrb->CommandData.OpenFormat;
  pblock= (KS_MPEGVIDEOINFO2 *)((ULONG)pfmt + sizeof  (KSDATAFORMAT));
  pStream->u.video= *pblock;
  pStream->internal_streamid= STRM_VID;
  ProcessVideoFormat(pfmt,pSrb);

  if (pStream->fIsMpeg2Payload)
    InitStreamControlMdb(pSrb);

  // Make sure CSS disabled as default when opening stream
  Cmd.hMidas=pDevice->hMidas;
  Cmd.cmd= MDS_CSS_ENABLE_CHANNEL;
  Cmd.u.M.Arg1=0;
  Cmd.u.M.Arg2=0;
  MidasEntry(&Cmd);

  Cmd.cmd = MDS_LOADCPSETTING;
  Cmd.hMidas=pDevice->hMidas;
  MidasEntry(&Cmd);

  
  DbgMsg(DBG_MSGVID, "InMpeg2Video Opened\r\n");
}

void StreamOpen_Ac3(SRB* pSrb) {
  // set up the flags indicating that this stream can be a master clock
  // for syncronization purposes, also indicate the routine to call
  // for current master clock value
  PHW_STREAM_OBJECT pStreamObj= pSrb->StreamObject;
  PSTREAM_EXTENSION pStream= (PSTREAM_EXTENSION)pStreamObj->HwStreamExtension;
  PDEVICE_EXTENSION pDevice= (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  //MIDASCMD Cmd;
  //PKSDATAFORMAT pfmt;
  //KS_MPEGAUDIOINFO* pblock;

  pDevice->ddLastStreamTime=0;
  pStreamObj->HwClockObject.HwClockFunction=  AudioClock;
  pStreamObj->HwClockObject.ClockSupportFlags=
             CLOCK_SUPPORT_CAN_SET_ONBOARD_CLOCK | CLOCK_SUPPORT_CAN_READ_ONBOARD_CLOCK |
             CLOCK_SUPPORT_CAN_RETURN_STREAM_TIME;
  pStreamObj->ReceiveDataPacket= AudioReceive;
  pStreamObj->ReceiveControlPacket= AudioControl;
  pStreamObj->HwEventRoutine= AudioEvent;
  pStream->internal_streamid= STRM_AUD;
  ProcessAudioFormat((PKSDATAFORMAT)pSrb->CommandData.OpenFormat,pSrb);

  /*
  pStream->u.audio.AudioSource= MDS_AS_AC3;
  pStream->u.audio.AudioSampleRate= MDS_SR44_1;
  pStream->u.audio.AudioOutputMode= MDS_DAC;

  
  pfmt= pSrb->CommandData.OpenFormat;
  pblock= (KS_MPEGAUDIOINFO *)((ULONG)pfmt + sizeof  (KSDATAFORMAT));
  DbgMsg(DBG_MSGAUD,"Change/Load New Audio Format \r\n");
  DbgDispStreamExtension(DBG_MSGAUD,pStream);
  DbgDispAudioStruct(DBG_MSGAUD,&pStream->u.audio);

  Cmd.cmd= MDS_INITIALIZEAUDIO;
  Cmd.hMidas=pDevice->hMidas;                    
  Cmd.u.InitAudio.AudioSource= pStream->u.audio.AudioSource;                        
  Cmd.u.InitAudio.AudioSampleRate= pStream->u.audio.AudioSampleRate;                       
  Cmd.u.InitAudio.AudioOutputMode= pStream->u.audio.AudioOutputMode;                        
  MidasEntry(&Cmd);
  */
  DbgMsg(DBG_INFO|DBG_MSGAUD, "InMpeg2Ac3 Opened\r\n");
}

#if TVOUTPIN
void StreamOpen_Ntsc(SRB* pSrb) {
  PHW_STREAM_OBJECT pStreamObj= pSrb->StreamObject;
  PSTREAM_EXTENSION pStream= (PSTREAM_EXTENSION)pStreamObj->HwStreamExtension;

  pStreamObj->ReceiveDataPacket= NTSCReceive;
  pStreamObj->ReceiveControlPacket= NTSCControl;
  pStreamObj->HwEventRoutine= 0;//NTSCEvent;

  DbgMsg(DBG_MSGNTSC,"Change/Load New NTSC Out \r\n");
  DbgDispStreamExtension(DBG_MSGNTSC,pStream);
  DbgMsg(DBG_MSGNTSC, "OutNTSCVideo Opened\r\n");
}
#endif

void StreamOpen_Subpict(SRB* pSrb) {
  MIDASCMD Cmd;
  PHW_STREAM_OBJECT pStreamObj= pSrb->StreamObject;
  PSTREAM_EXTENSION pStream= (PSTREAM_EXTENSION)pStreamObj->HwStreamExtension;
  PDEVICE_EXTENSION pDevice= (PDEVICE_EXTENSION)pSrb->HwDeviceExtension;

  pStreamObj->ReceiveDataPacket= SubpictureReceive;
  pStreamObj->ReceiveControlPacket= SubpictureControl;
  pStreamObj->HwEventRoutine= 0;//SubpictureEvent;
  pStream->internal_streamid= STRM_SUB;

  Cmd.cmd= MDS_INITIALIZESUB;
  Cmd.hMidas=pDevice->hMidas;                    
  MidasEntry(&Cmd);

  DbgMsg(DBG_SUBMSG,"Change/Load New SubPicture Format \r\n");
  DbgDispStreamExtension(DBG_SUBMSG,pStream);
  DbgMsg(DBG_SUBMSG, "InMpeg2Subpicture Opened\r\n");
}

void StreamOpen_VP(SRB* pSrb) {
  PHW_STREAM_OBJECT pStreamObj= pSrb->StreamObject;
  PSTREAM_EXTENSION pStream= (PSTREAM_EXTENSION)pStreamObj->HwStreamExtension;

  pStreamObj->ReceiveDataPacket= VPReceive;
  pStreamObj->ReceiveControlPacket= VPControl;
  pStreamObj->HwEventRoutine= (PHW_EVENT_ROUTINE) VPEvent;
  pStream->internal_streamid= OutVideoPort; //RAVI

  MargiDDrawProc(MARGIDD_CHECKDDAPIVER,0);

  DbgMsg(DBG_MSGVPE,"Change/Load New VP Out \r\n");
  DbgDispStreamExtension(DBG_MSGVPE,pStream);
  DbgMsg(DBG_MSGVPE, "OutVideoPort Opened\r\n");
}

#if CCOUTPIN
void StreamOpen_CC(SRB* pSrb) {
  PHW_STREAM_OBJECT pStreamObj= pSrb->StreamObject;
  PSTREAM_EXTENSION pStream= (PSTREAM_EXTENSION)pStreamObj->HwStreamExtension;
  PDEVICE_EXTENSION pDevice= (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  MIDASCMD Cmd;

  pStreamObj->ReceiveDataPacket= CCReceive;
  pStreamObj->ReceiveControlPacket= CCControl;
  pStreamObj->HwEventRoutine= 0; //CCEvent;
  pStream->internal_streamid= STRM_CC;

  Cmd.cmd= MDS_CCINIT;
  Cmd.hMidas=pDevice->hMidas;
  MidasEntry(&Cmd);

  DbgMsg(DBG_MSGCC,"Change/Load New CC Out \r\n");
  DbgDispStreamExtension(DBG_MSGCC,pStream);
  DbgMsg(DBG_MSGCC, "OutCloseCaption Opened\r\n");
}
#endif

void StreamOpen_Unknown(SRB* pSrb) {
  pSrb->Status= STATUS_NOT_IMPLEMENTED;
  DbgMsg(DBG_WRN, "MicroSoft asking us to open an unknown stream\r\n");
}

void StreamClose_Mpeg2(SRB* pSrb) {
  PDEVICE_EXTENSION pDevice= (PDEVICE_EXTENSION)pSrb->HwDeviceExtension;
  DbgMsg(DBG_MSGVID,"InMpeg2Video Closed\r\n");
  pDevice->pStream[InMpeg2Video]= 0;
}

void StreamClose_Ac3(SRB* pSrb) {
  PDEVICE_EXTENSION pDevice= (PDEVICE_EXTENSION)pSrb->HwDeviceExtension;
  DbgMsg(DBG_MSGAUD, "InMpeg2Ac3 Closed\r\n");
  pDevice->pStream[InMpeg2Ac3]= 0;
}

#if TVOUTPIN
void StreamClose_Ntsc(SRB* pSrb) {
  PDEVICE_EXTENSION pDevice= (PDEVICE_EXTENSION)pSrb->HwDeviceExtension;
  DbgMsg(DBG_MSGNTSC, "OutNTSCVideo Closed\r\n");
  pDevice->pStream[OutNTSCVideo]= 0;
}
#endif

void StreamClose_Subpict(SRB* pSrb) {
  PDEVICE_EXTENSION pDevice= (PDEVICE_EXTENSION)pSrb->HwDeviceExtension;
  DbgMsg(DBG_SUBMSG, "InMpeg2Subpicture Closed\r\n");
  pDevice->pStream[InMpeg2Subpicture]= 0;
}

void StreamClose_VP(SRB* pSrb) {
  PDEVICE_EXTENSION pDevice= (PDEVICE_EXTENSION)pSrb->HwDeviceExtension;
  DbgMsg(DBG_MSGVPE, "OutVideoPort Closed\r\n");
  pDevice->pStream[OutVideoPort]= 0;
}

#if CCOUTPIN
void StreamClose_CC(SRB* pSrb) {
  PDEVICE_EXTENSION pDevice= (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  MIDASCMD Cmd;

  Cmd.cmd= MDS_CCFINI;
  Cmd.hMidas=pDevice->hMidas;
  MidasEntry(&Cmd);

  DbgMsg(DBG_MSGCC, "OutCloseCaption Closed\r\n");
  pDevice->pStream[OutCloseCaption]= 0;
}
#endif

void StreamClose_Unknown(SRB* pSrb) {
  pSrb->Status= STATUS_NOT_IMPLEMENTED;
  DbgMsg(DBG_WRN,"MicroSoft asking us to close an unknown stream\r\n");
}


#if(0)
void SetMdbFlags(PMHAL_DATA_BLOCK pMdb,DWORD dwOptFlags,DWORD dwTypeFlags) {
#if    MDB_FLAG_DATADISCONTINUITY==KSSTREAM_HEADER_OPTIONSF_DATADISCONTINUITY \
    && MDB_FLAG_TYPECHANGED==KSSTREAM_HEADER_OPTIONSF_TYPECHANGED \
    && MDB_FLAG_TIMEVALID==KSSTREAM_HEADER_OPTIONSF_TIMEVALID \
    && MDB_FLAG_TIMEDISCONTINUITY==KSSTREAM_HEADER_OPTIONSF_TIMEDISCONTINUITY
  pMdb->dwFlags=dwOptFlags;
#else
  pMdb->dwFlags=0;
  if(dwOptFlags&KSSTREAM_HEADER_OPTIONSF_DATADISCONTINUITY)
    pMdb->dwFlags|=MDB_FLAG_DATADISCONTINUITY;
  if(dwOptFlags&KSSTREAM_HEADER_OPTIONSF_TYPECHANGED)
    pMdb->dwFlags|=MDB_FLAG_TYPECHANGED;
  if(dwOptFlags&KSSTREAM_HEADER_OPTIONSF_TIMEVALID)
    pMdb->dwFlags|=MDB_FLAG_TIMEVALID
  if(dwOptFlags&KSSTREAM_HEADER_OPTIONSF_TIMEDISCONTINUITY)
    pMdb->dwFlags|=MDB_FLAG_TIMEDISCONTINUITY
#endif
  if(dwTypeFlags&KS_AM_UseNewCSSKey)
    pMdb->dwFlags|=MDB_FLAG_USENEWCSSKEY;
}
#endif



/**EOF: STREAM.C**/
