#include <strmini.h>
#include <mmsystem.h>
#include <ksmedia.h>
#include "DbgMsg.h"
#include "DbgFltr.h"
#include "Idents.h"

#include "Globals.h"
#include "MidasApi.h"
#include "log.h"
#include "dbgout.h"

static void SubPictSetState_Stop(SRB* pSrb);
static void SubPictSetState_Pause(SRB* pSrb);
static void SubPictSetState_Run(SRB* pSrb);
static void SubPictSetState(SRB* pSrb);
static void SubPictGetState(SRB* pSrb);
static void SubPictGetProperty_0(SRB* pSrb);
static void SubPictGetPropertyRateChange(SRB* pSrb);
static void SubPictSetPropertyRateChange(SRB* pSrb);
static void SubPictGetProperty(SRB* pSrb);
static void SubPictSetProperty_0_Palette(SRB* pSrb);
static void SubPictSetProperty_0_HLI(SRB* pSrb);
static void SubPictSetProperty_0_Composit(SRB* pSrb);
static void SubPictSetProperty_0(SRB* pSrb);
static void SubPictSetProperty(SRB* pSrb);
static void SubPictProposeFormat(SRB* pSrb);
static void SubPictSetRate(SRB* pSrb);
static void SubPictMasterClock(SRB* pSrb);
static void SubPictBeginFlush(SRB* pSrb);
static void SubPictEndFlush(SRB* pSrb);
static void SubPictUnhandledMsg(SRB* pSrb);

void __stdcall SubpictureReceive(SRB *pSrb) {
  PSTREAM_EXTENSION pStream=(PSTREAM_EXTENSION)pSrb->StreamObject->HwStreamExtension; 
  logSrb("SubpictureReceive",pSrb);
  DbgDispKsStreamHeaders(DBG_MSGSUB,pSrb);
  if(pStream->stream_state==KSSTATE_STOP) {
    pSrb->Status= STATUS_CANCELLED;
    SignalStreamRequestCompleteReadyForNextData(pSrb);
  } else if(pSrb->CommandData.DataBufferArray->OptionsFlags & KSSTREAM_HEADER_OPTIONSF_TYPECHANGED) {
//    PKSSTREAM_HEADER pStrmHdr= pSrb->CommandData.DataBufferArray;
    pSrb->Status= STATUS_SUCCESS;
    SignalStreamRequestCompleteReadyForNextData(pSrb);
  } else {
    KIRQL Irql;  
    MIDASCMD Cmd;
    PDEVICE_EXTENSION pDevice=(PDEVICE_EXTENSION)pSrb->HwDeviceExtension;
    PMDS_DATA_BLOCK  pMdb   =&((PSRB_EXTENSION)pSrb->SRBExtension)->Mdb;

    if(pSrb->CommandData.DataBufferArray->OptionsFlags & KSSTREAM_HEADER_OPTIONSF_DATADISCONTINUITY) {
      Cmd.hMidas=pDevice->hMidas;
      Cmd.cmd = MDS_CANCELDATA;
      Cmd.wCurrStream=pStream->internal_streamid;
      Cmd.u.pMdb=0;
      MidasEntry(&Cmd);
    }
#if(0)
    if(pSrb->CommandData.DataBufferArray->OptionsFlags&KSSTREAM_HEADER_OPTIONSF_TIMEVALID) {
      if(AudClockFixupStrmTime(pDevice,pSrb->CommandData.DataBufferArray->PresentationTime.Time)) {
        DbgMsg(DBG_MSGSUB,"Subpic Fixup:");
        DbgDispKsStreamHeaders(DBG_MSGSUB,pSrb);
      }
    }
#endif

    pMdb->nSize=sizeof(MDS_DATA_BLOCK);
    pMdb->dwStatus=0;
    pMdb->dwStrmType=pStream->internal_streamid;
    pMdb->dwTotalPackets=pSrb->NumberOfBuffers;
    pMdb->pPackets=(PMDS_DATA_PACKET)pSrb->CommandData.DataBufferArray;
    pMdb->pNextMdb=0;
    pMdb->dwDataFlags=0;
    pMdb->hRef=(void*)pSrb;
    pMdb->pfnCallback=StreamDataCallback;

    Cmd.hMidas=pDevice->hMidas;
    Cmd.cmd = MDS_QUEUEDATA;
    Cmd.wCurrStream=pStream->internal_streamid;
    Cmd.u.pMdb=pMdb;
    KeAcquireSpinLock(&pStream->streamExtensionLock, &Irql); 
    MidasEntry(&Cmd);
	  KeReleaseSpinLock(&pStream->streamExtensionLock, Irql);   
    SignalStreamReadyForNextData(pSrb);
  }
}

void __stdcall SubpictureControl(SRB* pSrb) {
  logSrb("SubpictureControl", pSrb);
  pSrb->Status = STATUS_NOT_IMPLEMENTED;
  // determine the type of packet.  All calls here use the pointers
  // to the individual stream functions initialised at open time
  switch (pSrb->Command) {
    case SRB_SET_STREAM_STATE:      SubPictSetState(pSrb); break;
    case SRB_GET_STREAM_STATE:      SubPictGetState(pSrb); break;
    case SRB_GET_STREAM_PROPERTY:   SubPictGetProperty(pSrb); break;
    case SRB_SET_STREAM_PROPERTY:   SubPictSetProperty(pSrb); break;
    case SRB_PROPOSE_DATA_FORMAT:   SubPictProposeFormat(pSrb); break;
    case SRB_SET_STREAM_RATE:       SubPictSetRate(pSrb); break;
    case SRB_INDICATE_MASTER_CLOCK: SubPictMasterClock(pSrb); break;
    case SRB_BEGIN_FLUSH:           SubPictBeginFlush(pSrb); break;
    case SRB_END_FLUSH:             SubPictEndFlush(pSrb); break;
    default:                        SubPictUnhandledMsg(pSrb); break;
  }
  unlogSrb("SubpictureControl", pSrb);
  if(pSrb->Status!=STATUS_PENDING) {
    SignalStreamRequestCompleteReadyForNextControl(pSrb);
  } else {
    DbgMsg(DBG_INFO,"Returning Status Pending %s\r\n",Ident(ID_STRM_REQ,pSrb->Command));
  }
}

static void SubPictUnhandledMsg(SRB* pSrb) {
  DbgMsg(DBG_MSGERR,"VidControl: Unhandled Msg= %s \r\n",Ident(ID_STRM_REQ,pSrb->Command));
}

long __stdcall SubpictureEvent(PHW_EVENT_DESCRIPTOR pEventDescriptor) {
  long status= STATUS_SUCCESS;
  PSTREAM_EXTENSION pStrm= pEventDescriptor->StreamObject->HwStreamExtension;
  if(pEventDescriptor->Enable)
    pStrm->dwEventCount++;
  else
    pStrm->dwEventCount--;
  logEvent("SubpictureEvent", pEventDescriptor);
  return status;
}

static void SubPictEndFlush(SRB* pSrb) {
  PDEVICE_EXTENSION pDevice;
  PSTREAM_EXTENSION pStream;
  MIDASCMD Cmd;
  pDevice = (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  Cmd.hMidas=pDevice->hMidas;
  Cmd.cmd = MDS_FLUSH_END;
  Cmd.wCurrStream= pStream->internal_streamid;
  MidasEntry(&Cmd);
  pSrb->Status= STATUS_SUCCESS;
}

static void SubPictBeginFlush(SRB* pSrb) {
  MIDASCMD Cmd;
  PDEVICE_EXTENSION pDevice=(PDEVICE_EXTENSION)pSrb->HwDeviceExtension;
  PSTREAM_EXTENSION pStream=(PSTREAM_EXTENSION)pSrb->StreamObject->HwStreamExtension; 
  PMDS_DATA_BLOCK   pMdb=  &((PSRB_EXTENSION)pSrb->SRBExtension)->Mdb;
  
  pMdb->nSize=sizeof(MDS_DATA_BLOCK);
  pMdb->dwStatus=0;
  pMdb->dwStrmType=pStream->internal_streamid;
  pMdb->dwTotalPackets=0;
  pMdb->pPackets=0;
  pMdb->pNextMdb=0;
  pMdb->dwDataFlags=0;
  pMdb->hRef=(void*)pSrb;
  pMdb->pfnCallback=StreamControlCallback;

  Cmd.hMidas=pDevice->hMidas;
  Cmd.cmd = MDS_FLUSH;
  Cmd.wCurrStream= pStream->internal_streamid;
  Cmd.u.pMdb=pMdb;
  MidasEntry(&Cmd);
  pSrb->Status= STATUS_PENDING;
}

static void SubPictSetState_Stop(SRB* pSrb) {
  PDEVICE_EXTENSION pDevice;
  PSTREAM_EXTENSION pStream;
  MIDASCMD Cmd;
  pDevice = (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  Cmd.hMidas=pDevice->hMidas;
  if(pStream->stream_state != KSSTATE_STOP) {
    Cmd.cmd = MDS_STOP;
    Cmd.wCurrStream= pStream->internal_streamid;
    MidasEntry(&Cmd);
  }
  StreamSetState(pStream,KSSTATE_STOP);
}

static void SubPictSetState_Pause(SRB* pSrb) {
  PDEVICE_EXTENSION pDevice;
  PSTREAM_EXTENSION pStream;
  MIDASCMD Cmd;
  pDevice = (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  Cmd.hMidas=pDevice->hMidas;
  if(pStream->stream_state != KSSTATE_PAUSE) {
    Cmd.cmd = MDS_PAUSE;
    Cmd.wCurrStream= pStream->internal_streamid;
    MidasEntry(&Cmd);
  }
  StreamSetState(pStream,KSSTATE_PAUSE);
}

static void SubPictSetState_Run(SRB* pSrb) {
  PDEVICE_EXTENSION pDevice;
  PSTREAM_EXTENSION pStream;
  MIDASCMD Cmd;
  pDevice = (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  if(DeviceCheckHardware(pSrb)) {
    Cmd.hMidas=pDevice->hMidas;
    if(pStream->stream_state != KSSTATE_RUN) {
      Cmd.cmd = MDS_PLAY;
      Cmd.u.nData = MDS_PLAY_NORMAL;
      Cmd.wCurrStream= pStream->internal_streamid;
      MidasEntry(&Cmd);
    }
    StreamSetState(pStream,KSSTATE_RUN);
  }
}

static void SubPictSetState(SRB* pSrb) {
  pSrb->Status = STATUS_SUCCESS;
  switch (pSrb->CommandData.StreamState) {
    case KSSTATE_STOP:      SubPictSetState_Stop(pSrb); break;
    case KSSTATE_PAUSE:     SubPictSetState_Pause(pSrb); break;
    case KSSTATE_RUN:       SubPictSetState_Run(pSrb); break;
  }
}

static void SubPictGetState(SRB* pSrb) {
  PSTREAM_EXTENSION pStream;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  pSrb->CommandData.StreamState = pStream->stream_state;
  pSrb->Status = STATUS_SUCCESS;
}

static void SubPictGetProperty_0(SRB* pSrb) {
  PSTREAM_EXTENSION pStream;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  switch(pSrb->CommandData.PropertyInfo->Property->Id) {
    case KSPROPERTY_DVDSUBPIC_PALETTE:
           RtlCopyMemory(pSrb->CommandData.PropertyInfo->PropertyInfo, &pStream->u.subpicture.palette, sizeof (KSPROPERTY_SPPAL));
           pSrb->Status = STATUS_SUCCESS; break;
    case KSPROPERTY_DVDSUBPIC_HLI:
           RtlCopyMemory(pSrb->CommandData.PropertyInfo->PropertyInfo, &pStream->u.subpicture.highlite, sizeof (KSPROPERTY_SPHLI));
           pSrb->Status = STATUS_SUCCESS; break;
    case KSPROPERTY_DVDSUBPIC_COMPOSIT_ON:
           *(BOOL *)pSrb->CommandData.PropertyInfo->PropertyInfo = pStream->u.subpicture.composite_on;
           pSrb->Status = STATUS_SUCCESS; break;
  }
}

static void SubPictureGetSetPropertyInvalid(SRB *pSrb) {
  DbgMsg(DBG_MSGERR,"Subpict: Unknown Get or Set Property message received \r\n");
  pSrb->Status= STATUS_NOT_IMPLEMENTED;
}

static void SubPictGetPropertyRateChange(SRB* pSrb) {
  RateChangeRequestGet(pSrb);
  DbgMsg(DBG_SUBMSG,"SubControl: GetProperty-Rate (returns)\r\n");
  DbgDispKsRateChangeInfo(DBG_SUBMSG,pSrb->CommandData.PropertyInfo->PropertyInfo,pSrb->CommandData.PropertyInfo->Property->Id);
}

static void SubPictGetProperty(SRB* pSrb) {
  logGetProperty("Subpicture", pSrb);
  switch(pSrb->CommandData.PropertyInfo->PropertySetID) {
    case 0:     SubPictGetProperty_0(pSrb); break;
    case 1:     CopyProtectionGet(pSrb); break;
    case 2:     SubPictGetPropertyRateChange(pSrb); break;
    default:    SubPictureGetSetPropertyInvalid(pSrb); break;
  }
}

static void SubPictSetProperty_0_Palette(SRB* pSrb) {
  PSTREAM_EXTENSION pStream;
  MIDASCMD Cmd;
  PDEVICE_EXTENSION pDevice;
  pDevice = (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  Cmd.hMidas=pDevice->hMidas;
  if(RtlCompareMemory(pSrb->CommandData.PropertyInfo->PropertyInfo,&pStream->u.subpicture.palette,sizeof(KSPROPERTY_SPPAL))) {
    RtlCopyMemory(&pStream->u.subpicture.palette, pSrb->CommandData.PropertyInfo->PropertyInfo, sizeof (KSPROPERTY_SPPAL));
    Cmd.cmd = MDS_SP_PALETTE;
    Cmd.u.pSPPAL = &pStream->u.subpicture.palette;
    Cmd.wCurrStream= pStream->internal_streamid;
    MidasEntry(&Cmd);
  }
  DbgMsg(DBG_SUBMSG,"SubPict: SP Palette received \r\n");
  pSrb->Status = STATUS_SUCCESS;
 }

static void SubPictSetProperty_0_HLI(SRB* pSrb) {
  PSTREAM_EXTENSION pStream;
  MIDASCMD Cmd;
  PDEVICE_EXTENSION pDevice;
  RECT pr;
  pDevice = (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  pr.left= pStream->u.subpicture.highlite.StartX;
  pr.right= pStream->u.subpicture.highlite.StopX;
  pr.top= pStream->u.subpicture.highlite.StartY;
  pr.bottom= pStream->u.subpicture.highlite.StopY;
  Cmd.hMidas=pDevice->hMidas;
  if(RtlCompareMemory(pSrb->CommandData.PropertyInfo->PropertyInfo,&pStream->u.subpicture.highlite,sizeof(KSPROPERTY_SPHLI))) {
    RtlCopyMemory(&pStream->u.subpicture.highlite, pSrb->CommandData.PropertyInfo->PropertyInfo, sizeof (KSPROPERTY_SPHLI));
    Cmd.cmd = MDS_SP_HIGHLIGHT;
    Cmd.u.pSPHLI = &pStream->u.subpicture.highlite;
    Cmd.wCurrStream= pStream->internal_streamid;
    MidasEntry(&Cmd);
  }
  DbgMsg(DBG_SUBMSG,"SubPict: HLISS     %08x \r\n",*(DWORD*)&pStream->u.subpicture.highlite.HLISS);
  DbgMsg(DBG_SUBMSG,"       : StartPTM  %08x \r\n",*(DWORD*)&pStream->u.subpicture.highlite.StartPTM);
  DbgMsg(DBG_SUBMSG,"       : EndPTM    %08x \r\n",*(DWORD*)&pStream->u.subpicture.highlite.EndPTM);
  DbgMsg(DBG_SUBMSG,"       : ColCon    %08x \r\n",*(DWORD*)&pStream->u.subpicture.highlite.ColCon);
  DbgMsg(DBG_SUBMSG,"       : Position  %d,%d,%d,%d (d) \r\n",pr.left,pr.top,pr.right,pr.bottom);
  pSrb->Status = STATUS_SUCCESS;
}

static void SubPictSetProperty_0_Composit(SRB* pSrb) {
  BOOL oldbool;
  PSTREAM_EXTENSION pStream;
  MIDASCMD Cmd;
  PDEVICE_EXTENSION pDevice;
  pDevice = (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  Cmd.hMidas=pDevice->hMidas;
  oldbool = pStream->u.subpicture.composite_on;
  pStream->u.subpicture.composite_on = *(BOOL *)pSrb->CommandData.PropertyInfo->PropertyInfo;
  if(oldbool != pStream->u.subpicture.composite_on) {
    Cmd.cmd = MDS_SP_COMPOSITEON;
    Cmd.u.nData = pStream->u.subpicture.composite_on;
    Cmd.wCurrStream= pStream->internal_streamid;
    MidasEntry(&Cmd);
  }
  DbgMsg(DBG_SUBMSG,"SubPict: Set SP Composite ON \r\n");
  pSrb->Status = STATUS_SUCCESS;
}

static void SubPictSetProperty_0(SRB* pSrb) {
  switch(pSrb->CommandData.PropertyInfo->Property->Id) {
    case KSPROPERTY_DVDSUBPIC_PALETTE:    SubPictSetProperty_0_Palette(pSrb); break;
    case KSPROPERTY_DVDSUBPIC_HLI:        SubPictSetProperty_0_HLI(pSrb); break;
    case KSPROPERTY_DVDSUBPIC_COMPOSIT_ON:SubPictSetProperty_0_Composit(pSrb); break;
  }
}

static void SubPictSetPropertyRateChange(SRB* pSrb) {
  DbgMsg(DBG_SUBMSG,"SubControl: SetProperty-Rate (returns)\r\n");
  DbgDispKsRateChangeInfo(DBG_SUBMSG,pSrb->CommandData.PropertyInfo->PropertyInfo,pSrb->CommandData.PropertyInfo->Property->Id);
  RateChangeRequestSet(pSrb);
}

static void SubPictSetProperty(SRB* pSrb) {
  logSetProperty("Subpicture", pSrb);
  switch(pSrb->CommandData.PropertyInfo->PropertySetID) {
    case 0:   SubPictSetProperty_0(pSrb); break;
    case 1:   CopyProtectionSet(pSrb); break;
    case 2:   SubPictSetPropertyRateChange(pSrb); break;
    default:  SubPictureGetSetPropertyInvalid(pSrb); break;
  }
}

static void SubPictProposeFormat(SRB* pSrb) {
  pSrb->Status= STATUS_SUCCESS;
}

static void SubPictSetRate(SRB* pSrb) {
  pSrb->Status= STATUS_SUCCESS;
  DbgMsg(DBG_TODO,"SubPictControl: Set Stream Rate \r\n");
}

static void SubPictMasterClock(SRB* pSrb) {
  PSTREAM_EXTENSION pStream;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  pStream->MasterClock = pSrb->CommandData.MasterClockHandle;
  pSrb->Status = STATUS_SUCCESS;
}

/**EOF: SUBPICT.C**/
