#include <WinDef.h>
#include <VMM.h>	//Vmm.h must be included before VxdWraps.h
#include <VmmReg.h>	//VmmReg.h must be included before VxdWraps.h
#include <VxdWraps.h>
#include <ConfigMg.h>	//ConfigMg.h must be included after VxdWraps.h

#include "DbgMsg.h"
#include "DbgFltr.h"

#include "Settings.h"

#include "cardserv.h"
#include "margipnp.h"
#include "util.h"
#include "regkeys.h"
#include "pcidefs.h"
#include "essgate.h"

static struct ErrMsg_s {
  int ErrID;
  BYTE *Msg;
} ErrMsg[]= {
  { ERR_NO_CS,  "The PC-Card driver MARGIPNP.VXD\r\n"
                "is unable to detect a Card Service Provider.\r\n"
                "Check that Card and Socket Services are enabled." },
  { ERR_CS_SIG, "The PC-Card driver MARGIPNP.VXD\r\n"
                "detected an incorrect Card Service Signature.\r\n"
                "Check that Card and Socket Services are enabled." },
  { ERR_CS_ERR, "The PC-Card driver MARGIPNP.VXD\r\n"
                "has detected an invalid Card Service Provider.\r\n" },
  { ERR_CS_VER, "The PC-Card driver MARGIPNP.VXD\r\n"
                "requires a Zoomed Video Card Service Provider\r\n"
                "version 5.00 or greater." },
  { ERR_CS_REG, "The PC-Card driver MARGIPNP.VXD\r\n"
                "is unable to register with Card Services." },
  { ERR_UNKNOWN,"The PC-Card driver MARGIPNP.VXD\r\n"
                "has detected an unrecognized error.\r\n"
                "The driver may be corrupted or incorrectly installed." }
};

char ErrCaption[] =  "MARGIPNP.VXD";
char MsgBoxCaption[]= {"MargiPNP.vxd"};
WORD gVMMVersion=0;
BYTE gfGotDbgFlag=0;
DWORD gnSysUnsafe=0;
BYTE gfEssVxdPresent=1;  // flag ESS AudDrive Vxd is present (assume it is until found otherwise)

DWORD EnterUnsafe(void) {
  return(gnSysUnsafe++);
}

DWORD ExitUnsafe(void) {
  if(gnSysUnsafe)
    gnSysUnsafe--;
  return(gnSysUnsafe);
}

long ErrBox(long Err, char *Title, char *Text, ...) {
  DbgMsg(DBG_ERR,"%s: %s\r\n",Title,Text);
  return(Err);
}

#if defined(UTIL_NO_INTRINSICS) && UTIL_NO_INTRINSICS
#pragma message("Util.c: Defining intrinsic functions.")
BYTE __inline inp(WORD iobase) {
  BYTE val;
  _asm {
		mov dx,iobase
		in  al,dx
		mov val,al
  }
  return val;
}

void __inline outp(WORD iobase,BYTE val) {
  _asm {
		mov dx,iobase
		mov al,val
		out dx,al
  }
}

WORD __inline inpw(WORD iobase) {
  WORD val;
  _asm {
		mov dx,iobase
		in  ax,dx
		mov val,ax
  }
  return val;
}

void __inline outpw(WORD iobase,WORD val) {
  _asm {
		mov dx,iobase
		mov ax,val
		out dx,ax
  }
}

void __inline memclear(void*p,int n) {
   for(n--;n>=0;n--)
      ((char*)p)[n]=0;
}

void __inline memcpy(void*d,void*s,int n) {
   for(n--;n>=0;n--)
      ((char*)d)[n]=((char*)s)[n];
}

void __inline strcpy(char*d,char*s) {
   int n;
   for(n=0;s[n]!=0;n++)
      d[n]=s[n];
   d[n]=0;
}

void __inline strcat(char*d,char*s) {
   int n,m;
   for(m=0;d[m]!=0;m++);
   for(n=0;s[n]!=0;n++,m++)
      d[m]=s[n];
   d[m]=0;
}
#endif

int strncompare(char* s1, char* s2, int nBytes) {
  int n;
  for(n=0;n<nBytes;n++)
    if(s1[n]!=s2[n])
      return 0;
  return 1;
}

char *AppendHexStr(char *Dst, DWORD Val) {
  static BYTE HexDigits[16]= {"0123456789ABCDEF"};
  int n1;
  while(*Dst) Dst++;
  for(n1=28;n1>=0;n1-=4)
    *Dst++= HexDigits[(Val>>n1) & 0x0F];
  *Dst= 0;
  return(Dst);
}

void GetVersionID(WORD *pVersionID) {
  *pVersionID= (VERMAJOR<<8)|(VERMINOR);
}

void ErrorMessage(int err) {
  struct ErrMsg_s *pErr;
  for(pErr=ErrMsg;pErr->ErrID!=err && pErr->ErrID!=ERR_UNKNOWN;pErr++);
}

WORD GetVMMVersion(void) {
  if(!gVMMVersion) {
    VMMCall(Get_VMM_Version);
    _asm mov [gVMMVersion],ax
  }
  return(gVMMVersion);
}

int IsWin31(void)
{
  return (GetVMMVersion()<=0x310);
}

void ReportVersion(char* KeyName)
{
  WORD wVersion;
  char szVer[32];
  GetVersionID(&wVersion);
  
  szVer[0]=0;
  AppendHexStr(szVer,VXD_DATE);
  strcat(szVer,"  ");
  AppendHexStr(szVer,(DWORD)wVersion);

  SettingSave(SET_STR,KEYVAL_VERSION,(DWORD)szVer);
}


int pciReadConfigDw(int nDevice, int nAddress, DWORD *pdwData)
{                                                                         
  int nErr;
  struct Client_Word_Reg_Struc Regs;
  WORD wHiWord,wLoWord;
  
  Regs.Client_AX=READ_CONFIG_WORD;
  Regs.Client_BX=(WORD)nDevice;
  Regs.Client_DI=(WORD)nAddress;

  RealModeInt(0x1a,&Regs);
  nErr=Regs.Client_AX >> 8;
  wLoWord=Regs.Client_CX;
  if(nErr==0)
  {
    Regs.Client_AX=READ_CONFIG_WORD;
    Regs.Client_BX=(WORD)nDevice;
    Regs.Client_DI=(WORD)nAddress+2;
    RealModeInt(0x1a,&Regs);
    nErr=Regs.Client_AX >> 8;
    wHiWord=Regs.Client_CX;
  }
  *pdwData = ((DWORD)wHiWord<<16) + wLoWord;
  return nErr;
}

int pciWriteConfigDw(int nDevice, int nAddress, DWORD dwData)
{                                                                         
  int nErr;
  struct Client_Word_Reg_Struc Regs;
  
  Regs.Client_AX=WRITE_CONFIG_WORD;
  Regs.Client_BX=(WORD)nDevice;
  Regs.Client_DI=(WORD)nAddress;
  Regs.Client_CX=(WORD)dwData;

  RealModeInt(0x1a,&Regs);
  nErr=Regs.Client_AX >> 8;

  if(nErr==0)
  {
    Regs.Client_AX=WRITE_CONFIG_WORD;
    Regs.Client_BX=(WORD)nDevice;
    Regs.Client_DI=(WORD)nAddress+2;
    Regs.Client_CX=(WORD)(dwData>>16);
    RealModeInt(0x1a,&Regs);
    nErr=Regs.Client_AX >> 8;
  }
  return nErr;
}

int pciFindDevice(WORD wDeviceId, WORD wVendorId, WORD wIndex, WORD* pwBus, WORD* pwDevice)
{                                                                         
  WORD   wBus=0, wDevice=0;
  int    nErr=0;
  struct Client_Word_Reg_Struc Regs;
  
  Regs.Client_AX=FIND_PCI_DEVICE;
  Regs.Client_CX=wDeviceId;
  Regs.Client_DX=wVendorId;
  Regs.Client_SI=wIndex;

  RealModeInt(0x1a,&Regs);
  nErr=Regs.Client_AX >> 8;

  if(pwBus)
    *pwBus    = Regs.Client_BX>>8;
  if(pwDevice)
    *pwDevice = Regs.Client_BX&0x00fc;
  return nErr;
}

void WriteConfigReg(BYTE* pCisMem,BYTE value)
{
  *((WORD*)(pCisMem+2))=0x0000;
  *((WORD*)(pCisMem+4))=0x03f0;
  *((WORD*)(pCisMem+8))=(WORD)value;
}

DWORD EssEnableAudio()
{
  DWORD dwStatus=0;
  if(gfEssVxdPresent)
  {
    VxDCall(ESSAUDDRV_Enable_IIS);
    _asm
    {
      jnc a_skp
      mov eax,1
      mov [dwStatus],eax
  a_skp:
    }

    if(dwStatus)
      gfEssVxdPresent=0;    // if error then don't use EssVxd in future
    else
      gfEssVxdPresent=2;    // flag EssVxd is being used
  }
  return dwStatus;
}

DWORD EssDisableAudio()
{
  DWORD dwStatus=1;
  if(gfEssVxdPresent==2)
  {
    VxDCall(ESSAUDDRV_Disable_IIS);
    _asm
    {
      jnc a_skp
      mov eax,1
      mov [dwStatus],eax
    a_skp:
    }
  }
  return dwStatus;
}

/**EOF: UTIL.C**/

