#include <Strmini.h>
#include <mmsystem.h>
#include <ksmedia.h>
#include "DbgMsg.h"
#include "DbgFltr.h"
#include "Idents.h"
#include "Globals.h"
#include "MidasApi.h"
#include "log.h"
#include "dbgout.h"
#include "settings.h"
#include "ininames.h"

//#pragma message("Video.c: Optimizer disabled.")
//#pragma optimize("",off)

static void VidSetRate(SRB* pSrb);
static void VidMasterClock(SRB* pSrb);
static void VidProposeFormat(SRB* pSrb);
static int  VidSetProperty(SRB* pSrb);
static void VidGetProperty(SRB* pSrb);
static void VidGetState(SRB* pSrb);
static void VidSetState(SRB* pSrb);
static void VidSetState_Run(SRB* pSrb);
static void VidSetState_Pause(SRB* pSrb);
static void VidSetState_Stop(SRB* pSrb);
static void VidGetProperty_Mpeg(SRB* pSrb);
static void VidSetProperty_Mpeg(SRB* pSrb);
static void VidControlUnkMsg(SRB* pSrb);
static void VidBeginFlush(SRB* pSrb);
static void VidEndFlush(SRB* pSrb);

//From Stream.c
extern KSDATAFORMAT InMpeg2Video_Fmt0;
extern KSDATAFORMAT InMpeg2Video_Fmt1;
extern KSDATAFORMAT InMpeg2Video_Fmt2;
extern KSDATAFORMAT InMpeg2Video_Fmt3;
extern KSDATAFORMAT InMpeg2Video_Fmt4;

extern int gfPreferredDisplayMode;

void ProcessVideoFormat(PKSDATAFORMAT pVidFmt,SRB *pSrb);
extern void SetDmaFunction(SRB *pSrb);
KS_AMVPDATAINFO VPFmt;

void __stdcall VideoReceive(SRB *pSrb) {
  PSTREAM_EXTENSION pStream=(PSTREAM_EXTENSION)pSrb->StreamObject->HwStreamExtension; 
  PKSSTREAM_HEADER pStrmHdr=pSrb->CommandData.DataBufferArray;
  logSrb("VideoReceive",pSrb);
  DbgDispKsStreamHeaders(DBG_MSGVID,pSrb);
  if(pStream->stream_state==KSSTATE_STOP) {
    pSrb->Status= STATUS_CANCELLED;
    SignalStreamRequestCompleteReadyForNextData(pSrb);
  } else if(pSrb->CommandData.DataBufferArray->OptionsFlags & KSSTREAM_HEADER_OPTIONSF_TYPECHANGED) {
    ProcessVideoFormat((PKSDATAFORMAT)pStrmHdr->Data,pSrb);
    pSrb->Status= STATUS_SUCCESS;
    SignalStreamRequestCompleteReadyForNextData(pSrb);
  } else {
    MIDASCMD Cmd;
    PDEVICE_EXTENSION pDevice= (PDEVICE_EXTENSION)pSrb->HwDeviceExtension;
    PSTREAM_EXTENSION pStream=(PSTREAM_EXTENSION)pSrb->StreamObject->HwStreamExtension; 
    PMDS_DATA_BLOCK  pMdb=&((PSRB_EXTENSION)pSrb->SRBExtension)->Mdb;

    if(pSrb->CommandData.DataBufferArray->OptionsFlags&KSSTREAM_HEADER_OPTIONSF_TIMEVALID) {
      if(AudClockFixupStrmTime(pDevice,pSrb->CommandData.DataBufferArray->PresentationTime.Time)) {
        DbgMsg(DBG_MSGVID,"Video Fixup:");
        DbgDispKsStreamHeaders(DBG_MSGVID,pSrb);
      }
    }

    if (pStream->fIsMpeg2Payload)
    {
      pMdb->dwRsvd[0] = (DWORD) pStream->Data;
      pMdb->dwRsvd[1] = 1;
    }
    else
    {
      pMdb->dwRsvd[0] = 0;
      pMdb->dwRsvd[1] = 0;
    }

    pMdb->nSize=sizeof(MDS_DATA_BLOCK);
    pMdb->dwStatus=0;
    pMdb->dwStrmType=pStream->internal_streamid;
    pMdb->dwTotalPackets=pSrb->NumberOfBuffers;
    pMdb->pPackets=(PMDS_DATA_PACKET)pSrb->CommandData.DataBufferArray;
    pMdb->pNextMdb=0;
    pMdb->dwDataFlags=0;
    pMdb->hRef=(void*)pSrb;
    pMdb->pfnCallback=StreamDataCallback;

    Cmd.hMidas=pDevice->hMidas;
    Cmd.cmd = MDS_QUEUEDATA;
    Cmd.wCurrStream=pStream->internal_streamid;
    Cmd.u.pMdb=pMdb;
    MidasEntry(&Cmd);
    SignalStreamReadyForNextData(pSrb);
  }
}


VOID STREAMAPI VideoControl(SRB* pSrb) {
  int fSignaled=0;
  logSrb("VidControl", pSrb);
  pSrb->Status = STATUS_NOT_IMPLEMENTED;
  switch (pSrb->Command) {
    case SRB_SET_STREAM_STATE:      VidSetState(pSrb); break;
    case SRB_GET_STREAM_STATE:      VidGetState(pSrb); break;
    case SRB_GET_STREAM_PROPERTY:   VidGetProperty(pSrb); break;
    case SRB_SET_STREAM_PROPERTY:   fSignaled=VidSetProperty(pSrb); break;
    case SRB_PROPOSE_DATA_FORMAT:   VidProposeFormat(pSrb); break;
    case SRB_SET_STREAM_RATE:       VidSetRate(pSrb); break;
    case SRB_INDICATE_MASTER_CLOCK: VidMasterClock(pSrb); break;
    case SRB_BEGIN_FLUSH:           VidBeginFlush(pSrb); break;
    case SRB_END_FLUSH:             VidEndFlush(pSrb); break;
    default:                        VidControlUnkMsg(pSrb); break;
  }
  unlogSrb("VidControl", pSrb);
  if(!fSignaled && pSrb->Status!=STATUS_PENDING) {
    SignalStreamRequestCompleteReadyForNextControl(pSrb);
  } else {
    DbgMsg(DBG_INFO,"Returning Status Pending %s\r\n",Ident(ID_STRM_REQ,pSrb->Command));
  }
}

NTSTATUS STREAMAPI VideoEvent(PHW_EVENT_DESCRIPTOR pEventDescriptor) {
  NTSTATUS status= STATUS_SUCCESS;
  PSTREAM_EXTENSION pStrm= pEventDescriptor->StreamObject->HwStreamExtension;
  if(pEventDescriptor->Enable)
    pStrm->dwEventCount++;
  else
    pStrm->dwEventCount--;
  logEvent("VidEvent", pEventDescriptor);
  DbgMsg(DBG_MSGEVENT,"VIDEvent: #d, returning 0x%08X\r\n",pStrm->dwEventCount,status);
  return status;
}

static void VidEndFlush(SRB* pSrb) {
  MIDASCMD Cmd;
  PDEVICE_EXTENSION pDevice= (PDEVICE_EXTENSION)pSrb->HwDeviceExtension;
  PSTREAM_EXTENSION pStream=(PSTREAM_EXTENSION)pSrb->StreamObject->HwStreamExtension; 

  Cmd.hMidas=pDevice->hMidas;
  Cmd.cmd = MDS_FLUSH_END;
  Cmd.wCurrStream= pStream->internal_streamid;
  MidasEntry(&Cmd);
  pSrb->Status= STATUS_SUCCESS;
}

static void VidBeginFlush(SRB* pSrb) {
  MIDASCMD Cmd;
  PDEVICE_EXTENSION pDevice=(PDEVICE_EXTENSION)pSrb->HwDeviceExtension;
  PSTREAM_EXTENSION pStream=(PSTREAM_EXTENSION)pSrb->StreamObject->HwStreamExtension; 
  PMDS_DATA_BLOCK   pMdb=  &((PSRB_EXTENSION)pSrb->SRBExtension)->Mdb;
  
  pMdb->nSize=sizeof(MDS_DATA_BLOCK);
  pMdb->dwStatus=0;
  pMdb->dwStrmType=pStream->internal_streamid;
  pMdb->dwTotalPackets=0;
  pMdb->pPackets=0;
  pMdb->pNextMdb=0;
  pMdb->dwDataFlags=0;
  pMdb->hRef=(void*)pSrb;
  pMdb->pfnCallback=StreamControlCallback;

  Cmd.hMidas=pDevice->hMidas;
  Cmd.cmd = MDS_FLUSH;
  Cmd.wCurrStream= pStream->internal_streamid;
  Cmd.u.pMdb=pMdb;
  MidasEntry(&Cmd);
  pSrb->Status= STATUS_PENDING;
}

static GUID MY_KSEVENTSETID_VPNOTIFY = {STATIC_KSEVENTSETID_VPNotify};

void ProcessVideoFormat(PKSDATAFORMAT pFmt,SRB *pSrb) {
  ULONG cXOut=0;
  MIDASCMD Cmd;
  PDEVICE_EXTENSION pDevice= (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  PSTREAM_EXTENSION pStream= (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  KS_MPEGVIDEOINFO2 VidFmt;
  KS_MPEGVIDEOINFO2 *pVidFmtMpeg2= (KS_MPEGVIDEOINFO2*)((ULONG)pFmt+sizeof(KSDATAFORMAT));
  KS_VIDEOINFOHEADER *pVidFmtMpeg1= (KS_VIDEOINFOHEADER*)((ULONG)pFmt+sizeof(KSDATAFORMAT));
  PHW_STREAM_OBJECT pVPEStreamObj= pDevice->pStream[OutVideoPort];
//  PSTREAM_EXTENSION pVPEStreamExt= (PSTREAM_EXTENSION)(pVPEStreamObj->HwStreamExtension);
  int fAspectRatio16x9=0;
  DWORD dwCropTop;

  DbgMsg(DBG_MSGVIDFORMAT,"Change/Load New Video Format \r\n");
  DbgDispStreamExtension(DBG_MSGVIDFORMAT,pStream);

  dwCropTop=pDevice->dwCropTop;
  // MPEG2.
  if( ( (IsEqualGUID2(&(pFmt->MajorFormat), &(InMpeg2Video_Fmt0.MajorFormat))) ||
        (IsEqualGUID2(&(pFmt->MajorFormat), &(InMpeg2Video_Fmt1.MajorFormat))) ||
        (IsEqualGUID2(&(pFmt->MajorFormat), &(InMpeg2Video_Fmt4.MajorFormat)))) &&
      (IsEqualGUID2(&(pFmt->SubFormat), &(InMpeg2Video_Fmt0.SubFormat))) ) {
    int fAspectRatio16x9=0;
    int fVOB=0;

    if (IsEqualGUID2(&(pFmt->MajorFormat), &(InMpeg2Video_Fmt4.MajorFormat)))
    {
      
      pStream->fIsMpeg2Payload = 1;
    }
    else
      pStream->fIsMpeg2Payload = 0;

    //DVD Encrypted Pack MPEG 2
    if(IsEqualGUID2(&(pFmt->MajorFormat),&(InMpeg2Video_Fmt0.MajorFormat)))
      fVOB=1;

    RtlCopyMemory(&VidFmt,pVidFmtMpeg2,sizeof(KS_MPEGVIDEOINFO2));

    VidFmt.dwFlags=MDS_NORMAL_RECT;
    if(VidFmt.hdr.dwPictAspectRatioX==16 && VidFmt.hdr.dwPictAspectRatioY==9) {
      fAspectRatio16x9=1;
#if(0)
      switch(gfPreferredDisplayMode) {
        case MDS_PAN_SCAN:if(pVidFmtMpeg2->dwFlags&KS_MPEG2_DoPanScan)
                           VidFmt.dwFlags=MDS_PAN_SCAN;
                         else
                           VidFmt.dwFlags=MDS_LETTER_BOX;
                         break;
        case MDS_LETTER_BOX: VidFmt.dwFlags=MDS_LETTER_BOX;
                         break;
      }
#else
      VidFmt.dwFlags=MDS_NORMAL_RECT;
      if(pVidFmtMpeg2->dwFlags&KS_MPEG2_DoPanScan)
        VidFmt.dwFlags=MDS_PAN_SCAN;
      else if(pVidFmtMpeg2->dwFlags&KS_MPEG2_LetterboxAnalogOut)
        VidFmt.dwFlags=MDS_LETTER_BOX;
#endif
    }

    DbgDispMpeg2VideoFormat(DBG_MSGVIDFORMAT,&VidFmt);
    Cmd.cmd = MDS_INITIALIZEVIDEO;
    Cmd.u.InitVideo.videoinfo = &VidFmt;
    Cmd.u.InitVideo.nMpegType=fVOB?2:0;
    Cmd.hMidas=pDevice->hMidas;
    Cmd.wCurrStream= pStream->internal_streamid;
    MidasEntry(&Cmd);

    VPFmt.dwPictAspectRatioX= VidFmt.hdr.dwPictAspectRatioX;
    VPFmt.dwPictAspectRatioY= VidFmt.hdr.dwPictAspectRatioY;
    cXOut = VidFmt.hdr.bmiHeader.biWidth;
    if(fAspectRatio16x9) {
      if(VidFmt.dwFlags==MDS_PAN_SCAN) {
        VPFmt.dwPictAspectRatioX=4;
        VPFmt.dwPictAspectRatioY=3;
      } else if(VidFmt.dwFlags==MDS_LETTER_BOX) {
        VPFmt.dwPictAspectRatioX = 4;
        VPFmt.dwPictAspectRatioY = 3;
      }
    }

    if(VidFmt.hdr.bmiHeader.biHeight>=576)
      dwCropTop+=3;

    VPFmt.amvpDimInfo.dwFieldWidth = cXOut;
    VPFmt.amvpDimInfo.dwFieldHeight= (VidFmt.hdr.bmiHeader.biHeight/2)+22;
    VPFmt.amvpDimInfo.dwVBIWidth   = 0; //cXOut;
    VPFmt.amvpDimInfo.dwVBIHeight  = 0;

    VPFmt.amvpDimInfo.rcValidRegion.left  = 0;
    VPFmt.amvpDimInfo.rcValidRegion.top   = dwCropTop;
    VPFmt.amvpDimInfo.rcValidRegion.right = cXOut;
    VPFmt.amvpDimInfo.rcValidRegion.bottom= dwCropTop+(VidFmt.hdr.bmiHeader.biHeight/2);

    if(VidFmt.hdr.bmiHeader.biHeight>=480)
      VPFmt.bDataIsInterlaced= TRUE;

  } else if( (IsEqualGUID2(&(pFmt->MajorFormat), &(InMpeg2Video_Fmt2.MajorFormat)) &&
                IsEqualGUID2(&(pFmt->SubFormat), &(InMpeg2Video_Fmt2.SubFormat))) ) {
    
    pStream->fIsMpeg2Payload = 0;
    //MPEG1 PayLoad
    RtlZeroMemory(&VidFmt,sizeof(KS_MPEGVIDEOINFO2));
    VidFmt.hdr.dwPictAspectRatioX= 4;
    VidFmt.hdr.dwPictAspectRatioY= 3;
    VidFmt.hdr.bmiHeader.biSize= sizeof(KS_BITMAPINFOHEADER);
    VidFmt.hdr.bmiHeader.biWidth= pVidFmtMpeg1->bmiHeader.biWidth;
    VidFmt.hdr.bmiHeader.biHeight= pVidFmtMpeg1->bmiHeader.biHeight;
    VidFmt.hdr.dwBitRate= pVidFmtMpeg1->dwBitRate;
    VidFmt.hdr.AvgTimePerFrame=pVidFmtMpeg1->AvgTimePerFrame;


    Cmd.cmd = MDS_INITIALIZEVIDEO;
    Cmd.u.InitVideo.videoinfo = &VidFmt;
    Cmd.u.InitVideo.nMpegType=1;
    Cmd.hMidas=pDevice->hMidas;
    Cmd.wCurrStream= pStream->internal_streamid;
    MidasEntry(&Cmd);

    pVidFmtMpeg1->bmiHeader.biWidth= 720;
    if(pVidFmtMpeg1->bmiHeader.biHeight<=252) {
      pVidFmtMpeg1->bmiHeader.biHeight= 480;  //NTSC
    } else {
      pVidFmtMpeg1->bmiHeader.biHeight= 576;  //PAL
    }

    VPFmt.dwPictAspectRatioX = 4; //pVidFmtMpeg1->bmiHeader.biWidth;
    VPFmt.dwPictAspectRatioY = 3; //pVidFmtMpeg1->bmiHeader.biHeight;

    cXOut= pVidFmtMpeg1->bmiHeader.biWidth;
    VPFmt.amvpDimInfo.dwFieldWidth= cXOut;
    VPFmt.amvpDimInfo.dwFieldHeight= (pVidFmtMpeg1->bmiHeader.biHeight/2)+22;
    VPFmt.amvpDimInfo.dwVBIWidth = 0; //cXOut;
    VPFmt.amvpDimInfo.dwVBIHeight= 0;
    VPFmt.amvpDimInfo.rcValidRegion.left  = 0;
    VPFmt.amvpDimInfo.rcValidRegion.top   = dwCropTop;
    VPFmt.amvpDimInfo.rcValidRegion.right = cXOut;
    VPFmt.amvpDimInfo.rcValidRegion.bottom= dwCropTop+pVidFmtMpeg1->bmiHeader.biHeight/2; //dwCrop+(VidFmt.hdr.bmiHeader.biHeight/2);
    //if(pVidFmtMpeg1->bmiHeader.biHeight>=480)
      VPFmt.bDataIsInterlaced= TRUE;
    //else
    //  VPFmt.bDataIsInterlaced= FALSE;

  }
  SetDmaFunction(pSrb);
  // call the IVPConfig interface here
  if(pVPEStreamObj) { // && pVPEStreamExt->dwEventCount) {
    DbgMsg(DBG_MSGEVENT,"VidFmtChange: Sending VP notify event\r\n");
    StreamClassStreamNotification(SignalMultipleStreamEvents,pVPEStreamObj,
                          &MY_KSEVENTSETID_VPNOTIFY,KSEVENT_VPNOTIFY_FORMATCHANGE);
  }
}

static void VidControlUnkMsg(SRB* pSrb) {
  DbgMsg(DBG_MSGERR,"VidControl: Unhandled Msg= %s \r\n",Ident(ID_STRM_REQ,pSrb->Command));
  pSrb->Status = STATUS_SUCCESS; //????
}

static void VidSetState_Stop(SRB* pSrb) {
  PDEVICE_EXTENSION pDevice;
  PSTREAM_EXTENSION pStream;
  MIDASCMD Cmd;
  pDevice = (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  Cmd.hMidas=pDevice->hMidas;
  if(pStream->stream_state != KSSTATE_STOP) {
    Cmd.cmd = MDS_STOP;
    Cmd.wCurrStream= pStream->internal_streamid;
    MidasEntry(&Cmd);
  }
  StreamSetState(pStream,KSSTATE_STOP);
}

static void VidSetState_Pause(SRB* pSrb) {
  PDEVICE_EXTENSION pDevice;
  PSTREAM_EXTENSION pStream;
  MIDASCMD Cmd;
  pDevice = (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  Cmd.hMidas=pDevice->hMidas;
  if(pStream->stream_state != KSSTATE_PAUSE) {
    Cmd.cmd = MDS_PAUSE;
    Cmd.wCurrStream= pStream->internal_streamid;
    MidasEntry(&Cmd);
  }
  StreamSetState(pStream,KSSTATE_PAUSE);
}

static void VidSetState_Run(SRB* pSrb) {
  PDEVICE_EXTENSION pDevice;
  PSTREAM_EXTENSION pStream;
  MIDASCMD Cmd;
  pDevice = (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  if(DeviceCheckHardware(pSrb)) {
    Cmd.hMidas=pDevice->hMidas;
    if(pStream->stream_state != KSSTATE_RUN) {
      Cmd.cmd = MDS_PLAY;
      Cmd.u.nData = MDS_PLAY_NORMAL;
      Cmd.wCurrStream= pStream->internal_streamid;
      MidasEntry(&Cmd);
    }
    StreamSetState(pStream,KSSTATE_RUN);
  }
}

static void VidSetState(SRB* pSrb) {
  pSrb->Status = STATUS_SUCCESS;
  DbgMsg(DBG_MSGVID,"VidControl: SetState to %s \r\n",Ident(ID_KS_STATE,pSrb->CommandData.StreamState));
  switch (pSrb->CommandData.StreamState) {
    case KSSTATE_STOP:    VidSetState_Stop(pSrb); break;
    case KSSTATE_PAUSE:   VidSetState_Pause(pSrb); break;
    case KSSTATE_RUN:     VidSetState_Run(pSrb); break;
  }
}

static void VidGetState(SRB* pSrb) {
  PSTREAM_EXTENSION pStream;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  pSrb->CommandData.StreamState = pStream->stream_state;
  DbgMsg(DBG_MSGVID,"VidControl: GetState returns %s \r\n",Ident(ID_KS_STATE,pSrb->CommandData.StreamState));
  pSrb->Status = STATUS_SUCCESS;
}

static void VidGetProperty_Mpeg(SRB* pSrb) {
  switch(pSrb->CommandData.PropertyInfo->Property->Id) {
    case KSPROPERTY_MPEG2VID_MODES:         // available output modes of decoder
    case KSPROPERTY_MPEG2VID_CUR_MODE:      // current mode of the decoder
    case KSPROPERTY_MPEG2VID_4_3_RECT:      // output coordinates for 4:3 source
    case KSPROPERTY_MPEG2VID_16_9_RECT:     // output coordinates for 16:9 source
    case KSPROPERTY_MPEG2VID_16_9_PANSCAN:  // pan and scan vectors
                                            pSrb->Status = STATUS_SUCCESS; break;
  }
}

static void VidGetProperty(SRB* pSrb) {
  long lID= pSrb->CommandData.PropertyInfo->PropertySetID;
  logGetProperty("Video", pSrb);
  if(lID==0) {
    VidGetProperty_Mpeg(pSrb);
    DbgMsg(DBG_MSGVIDFORMAT,"VidControl: GetProperty-Mpeg2 (returns)\r\n");
    DbgDispKsPropertyVid(DBG_MSGVIDFORMAT,pSrb->CommandData.PropertyInfo->Property);
  } else if(lID==1) {
    CopyProtectionGet(pSrb);
    DbgMsg(DBG_MSGVIDFORMAT,"VidControl: GetProperty-Css (returns)\r\n");
    DbgDispKsCssPropertyInfo(DBG_MSGVIDFORMAT,pSrb->CommandData.PropertyInfo->PropertyInfo,
                             pSrb->CommandData.PropertyInfo->Property->Id);
  } else if(lID==2) {
    RateChangeRequestGet(pSrb);
    DbgMsg(DBG_MSGVIDFORMAT,"VidControl: GetProperty-Rate (returns)\r\n");
    DbgDispKsRateChangeInfo(DBG_MSGVIDFORMAT,pSrb->CommandData.PropertyInfo->PropertyInfo,
                               pSrb->CommandData.PropertyInfo->Property->Id);
  } else {
    DbgMsg(DBG_MSGERR,"VidControl: GetProperty called with an unknown ID (%08x)\r\n",lID);
  }
}

static void VidSetProperty_Mpeg(SRB* pSrb) {
  long lId= pSrb->CommandData.PropertyInfo->Property->Id;
  MIDASCMD cmd;
  PDEVICE_EXTENSION pDevice;
  PSTREAM_EXTENSION pStream;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  pDevice = pSrb->HwDeviceExtension;
  cmd.hMidas=pDevice->hMidas;
//  cmd.cmd= MDS_VIDOUTMODE;
 cmd.u.M.Arg1= 0;
  cmd.u.M.Arg2= (DWORD)pSrb->CommandData.PropertyInfo->PropertyInfo; //Rectangle/PanScan Vectors
  switch (lId) {
    case KSPROPERTY_MPEG2VID_CUR_MODE:      // current mode of the decoder
    case KSPROPERTY_MPEG2VID_4_3_RECT:      cmd.u.M.Arg1=MDS_NORMAL_RECT; // output coordinates for 4:3 source
    case KSPROPERTY_MPEG2VID_16_9_RECT:     cmd.u.M.Arg1=MDS_NORMAL_RECT;// output coordinates for 16:9 source
    case KSPROPERTY_MPEG2VID_16_9_PANSCAN:  cmd.u.M.Arg1=MDS_PAN_SCAN;// pan and scan vectors
  }
  if(cmd.u.M.Arg1) {
    cmd.wCurrStream= pStream->internal_streamid;
//    MidasEntry(&cmd);
  }
  pSrb->Status= STATUS_SUCCESS;
}

static int VidSetProperty(SRB* pSrb) {
  int fSignaled=0;
  DWORD lID= pSrb->CommandData.PropertyInfo->PropertySetID;
  logSetProperty("Video", pSrb);
  if(lID==0) {
    DbgMsg(DBG_MSGVIDFORMAT,"VidControl: SetProperty-Mpeg2 \r\n");
    DbgDispKsPropertyVid(DBG_MSGVIDFORMAT,pSrb->CommandData.PropertyInfo->Property);
    VidSetProperty_Mpeg(pSrb);
  } else if(lID==1) {
    DbgMsg(DBG_MSGVIDFORMAT,"VidControl: SetProperty-Css \r\n");
    DbgDispKsCssPropertyInfo(DBG_MSGVIDFORMAT,pSrb->CommandData.PropertyInfo->PropertyInfo,
                             pSrb->CommandData.PropertyInfo->Property->Id);
    fSignaled=CopyProtectionSet(pSrb);
  } else if(lID==2) {
    DbgMsg(DBG_MSGVIDFORMAT,"VidControl: GetProperty-Rate (returns)\r\n");
    DbgDispKsRateChangeInfo(DBG_MSGVIDFORMAT,pSrb->CommandData.PropertyInfo->PropertyInfo,
                               pSrb->CommandData.PropertyInfo->Property->Id);
    RateChangeRequestSet(pSrb);
  } else {
    pSrb->Status= STATUS_NOT_IMPLEMENTED;
    DbgMsg(DBG_MSGERR,"VidControl: SetProperty called with an unknown ID \r\n");
  }
  return fSignaled;
}

static void VidProposeFormat(SRB* pSrb) {
  PKSDATAFORMAT pFmt= pSrb->CommandData.OpenFormat;
  KS_MPEGVIDEOINFO2* pMpegInfo= (KS_MPEGVIDEOINFO2*)(pFmt+1);
  if(pFmt->FormatSize>=sizeof(KSDATAFORMAT))
    DbgDispKsDataFormat(DBG_MSGVID,pFmt);
  if(pFmt->FormatSize>=(sizeof(KSDATAFORMAT)+sizeof(KS_MPEGVIDEOINFO2)))
    DbgDispMpeg2VideoFormat(DBG_MSGVID,pMpegInfo);
  pSrb->Status = STATUS_SUCCESS;
}

static void VidSetRate(SRB* pSrb) {
  // rates should be set on an individual stream basis
  pSrb->Status = STATUS_SUCCESS;
  DbgMsg(DBG_TODO,"VidControl: SetRate ??? Not doing anything \r\n");
}

static void VidMasterClock(SRB* pSrb) {
  PSTREAM_EXTENSION pStream;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  pStream->MasterClock = pSrb->CommandData.MasterClockHandle;
  DbgMsg(DBG_MSGVIDFORMAT,"VidControl: MasterClockHandle is set to %08x \r\n",pSrb->CommandData.MasterClockHandle);
  pSrb->Status = STATUS_SUCCESS;
}

/**EOF: VIDEO.C**/
