#include <strmini.h>
#include <mmsystem.h>
#include <ksmedia.h>
#include "DbgMsg.h"
#include "DbgFltr.h"
#include "Idents.h"

#include "Globals.h"
#include "MidasApi.h"
#include "Log.h"
#include "dbgout.h"
#include "mddraw.h"

//#pragma message("vp.c: Optimizer disabled.")
//#pragma optimize("",off)


// Following GUIDs are copied from DxSdk5.2 (dvp.h).
// These (except the two that are commented out) are not defined in KsMedia.h
// This may get affected when a New DDK/SDK is released.
GUID IID_IDDVideoPortContainer= {0x6C142760,0xA733,0x11CE,0xA5,0x21,0x00,0x20,0xAF,0x0B,0xE5,0x60};
GUID IID_IDirectDrawVideoPort=  {0xB36D93E0,0x2B43,0x11CF,0xA2,0xDE,0x00,0xAA,0x00,0xB9,0x33,0x56};
GUID DDVPTYPE_E_HREFH_VREFL= {0x92783220L,0xDA60,0x11CF,0x9B,0x06,0x00,0xA0,0xC9,0x03,0xA3,0xB8};
GUID DDVPTYPE_E_HREFL_VREFH= {0xA07A02E0L,0xDA60,0x11CF,0x9B,0x06,0x00,0xA0,0xC9,0x03,0xA3,0xB8};
//GUID DDVPTYPE_E_HREFH_VREFH= {0x54F39980L,0xDA60,0x11CF,0x9B,0x06,0x00,0xA0,0xC9,0x03,0xA3,0xB8};
//GUID DDVPTYPE_E_HREFL_VREFL= {0xE09C77E0L,0xDA60,0x11CF,0x9B,0x06,0x00,0xA0,0xC9,0x03,0xA3,0xB8};
GUID DDVPTYPE_CCIR656=       {0xFCA326A0L,0xDA60,0x11CF,0x9B,0x06,0x00,0xA0,0xC9,0x03,0xA3,0xB8};
GUID DDVPTYPE_BROOKTREE=     {0x1352A560L,0xDA61,0x11CF,0x9B,0x06,0x00,0xA0,0xC9,0x03,0xA3,0xB8};
GUID DDVPTYPE_PHILIPS=       {0x332CF160L,0xDA61,0x11CF,0x9B,0x06,0x00,0xA0,0xC9,0x03,0xA3,0xB8};

#define STATIC_DDVPTYPE_E_HREFH_VREFL 0x92783220L,0xDA60,0x11CF,0x9B,0x06,0x00,0xA0,0xC9,0x03,0xA3,0xB8
#define STATIC_DDVPTYPE_E_HREFL_VREFH 0xA07A02E0L,0xDA60,0x11CF,0x9B,0x06,0x00,0xA0,0xC9,0x03,0xA3,0xB8
#define STATIC_DDVPTYPE_CCIR656       0xFCA326A0L,0xDA60,0x11CF,0x9B,0x06,0x00,0xA0,0xC9,0x03,0xA3,0xB8
#define STATIC_DDVPTYPE_E_HREFH_VREFH 0x54F39980L,0xDA60,0x11CF,0x9B,0x06,0x00,0xA0,0xC9,0x03,0xA3,0xB8
#define STATIC_DDVPTYPE_E_HREFL_VREFL 0xE09C77E0L,0xDA60,0x11CF,0x9B,0x06,0x00,0xA0,0xC9,0x03,0xA3,0xB8

// {6C4F1A60-6996-11d2-9A33-00104B268411}  /HRefL, VRefH and AutoDblBuff always.
#define STATIC_DDVPTYPE_HL_VH_DBLBUFF 0x6c4f1a60,0x6996,0x11d2,0x9a,0x33,0x0,0x10,0x4b,0x26,0x84,0x11

#define VIDEO_WIDTH       720
#define VIDEO_HEIGHT      480

#define MAKE_FOURCC( ch0, ch1, ch2, ch3 )                       \
        ( (DWORD)(BYTE)(ch0) | ( (DWORD)(BYTE)(ch1) << 8 ) |    \
        ( (DWORD)(BYTE)(ch2) << 16 ) | ( (DWORD)(BYTE)(ch3) << 24 ) )

#define FOURCC_YUV422   MAKE_FOURCC('S','4','2','2')
#define FOURCC_VBID     MAKE_FOURCC('V','B','I','D')
#define FOURCC_YUY2     MAKE_FOURCC('Y','U','Y','2')
#define FOURCC_UYVY     MAKE_FOURCC('U','Y','V','Y')
#define FOURCC_YV12     MAKE_FOURCC('Y','V','1','2')
#define FOURCC_YUV12    FOURCC_YV12
#define FOURCC_Y12G     MAKE_FOURCC('Y','1','2','G')
#define FOURCC_YV10     MAKE_FOURCC('Y','V','1','0')
#define FOURCC_YUV10    FOURCC_YV10
#define FOURCC_YVU9     MAKE_FOURCC('Y','V','U','9')
#define FOURCC_IF09     MAKE_FOURCC('I','F','0','9')
#define FOURCC_Y10F     MAKE_FOURCC('Y','1','0','F')
#define FOURCC_Y12F     MAKE_FOURCC('Y','1','2','F')
#define FOURCC_YVUM     MAKE_FOURCC('Y','V','U','M')

/*
 * When this is set by the driver and passed to the client, this
 * indicates that the video port is capable of using an external VACT
 * signal. When this is set by the client, this indicates that the
 * video port should use the external VACT signal.
 */
#define DDVPCONNECT_VACT			      0x00000002l
#define DDVPCONNECT_INVERTPOLARITY  0x00000004l
#define DDVPCONNECT_HALFLINE        0x00000010l
#define DDVPCONNECT_INTERLACED			0x00000020l
#define NTSC_FRAME_RATE 30
#define VBISamples  (845 * 2)

#define NUM_OF_VID_FORMATS_AVAIL                0x00000002l

#define MARGI_CONN_INFO_FLAGS  DDVPCONNECT_INTERLACED
//#define MARGI_CONN_INFO_FLAGS  DDVPCONNECT_INTERLACED|DDVPCONNECT_HALFLINE|DDVPCONNECT_INVERTPOLARITY
//#define MARGI_CONN_INFO_FLAGS  DDVPCONNECT_INTERLACED|DDVPCONNECT_HALFLINE|DDVPCONNECT_VACT
//#define MARGI_CONN_INFO_FLAGS  0x0000003Fl


static DDVIDEOPORTCONNECT VPConInfo[]= {
  { sizeof(DDVIDEOPORTCONNECT), 16, STATIC_DDVPTYPE_HL_VH_DBLBUFF, MARGI_CONN_INFO_FLAGS, 0 },
  { sizeof(DDVIDEOPORTCONNECT), 16, STATIC_DDVPTYPE_E_HREFL_VREFH, MARGI_CONN_INFO_FLAGS, 0 },
  { sizeof(DDVIDEOPORTCONNECT), 16, STATIC_DDVPTYPE_E_HREFH_VREFL, MARGI_CONN_INFO_FLAGS, 0 },
//  { sizeof(DDVIDEOPORTCONNECT), 16, STATIC_DDVPTYPE_E_HREFL_VREFL, MARGI_CONN_INFO_FLAGS, 0 },
//  { sizeof(DDVIDEOPORTCONNECT), 16, STATIC_DDVPTYPE_E_HREFH_VREFH, MARGI_CONN_INFO_FLAGS, 0 },
  { sizeof(DDVIDEOPORTCONNECT),  8, STATIC_DDVPTYPE_CCIR656,       MARGI_CONN_INFO_FLAGS, 0 },
};

#define NUM_OF_CONNECTIONS_AVAIL  (long)(sizeof(VPConInfo)/sizeof(DDVIDEOPORTCONNECT))

static DWORD gdwIndex=0;    // Keep record of currently used connect info index


static void VPGetState(SRB* pSrb);
static void VPGetProperty_NumConnectInfo(SRB* pSrb);
static void VPGetProperty_NumVideoFormat(SRB* pSrb);
static void VPGetProperty_DecimateComp(SRB* pSrb);
static void VPGetProperty_DataInfo(SRB* pSrb);
static void VPGetProperty_MaxPixRate(SRB* pSrb);
static void VPGetProperty_GetConnectInfo(SRB* pSrb);
static void VPGetProperty_GetVideoFormat(SRB* pSrb);
static void VPGetProperty(SRB* pSrb);

static void VPSetState(SRB* pSrb);
static void VPSetProperty_ConnectInfo(SRB* pSrb);
static void VPSetProperty_VideoFormat(SRB* pSrb);
static void VPSetProperty_DDrawHandle(SRB* pSrb);
static void VPSetProperty_VPortID(SRB* pSrb);
static void VPSetProperty_SurfHandle(SRB* pSrb);
static void VPSetProperty_InvPolarity(SRB* pSrb);
static void VPSetProperty_ScaleFactor(SRB* pSrb);
static void VPSetProperty(SRB* pSrb);

static void VPMasterClock(SRB* pSrb);
static void VPControlUnkMsg(SRB* pSrb);

  
VOID STREAMAPI VPReceive(SRB* pSrb) {
  logSrb("VPReceive",pSrb);
  unlogSrb("VPReceive",pSrb);
  DbgMsg(DBG_CRIT," VPReceive. Unexpected call ?? Check this out \r\n");
  SignalStreamRequestCompleteReadyForNextData(pSrb);
}

VOID STREAMAPI VPControl(PHW_STREAM_REQUEST_BLOCK pSrb) {
  logSrb("VPControl", pSrb);
  pSrb->Status = STATUS_NOT_IMPLEMENTED;              // set default status
  switch (pSrb->Command) {
    case SRB_SET_STREAM_STATE:      VPSetState(pSrb); break;  
    case SRB_GET_STREAM_STATE:      VPGetState(pSrb); break;
    case SRB_GET_STREAM_PROPERTY:   VPGetProperty(pSrb); break;
    case SRB_SET_STREAM_PROPERTY:   VPSetProperty(pSrb); break;
    case SRB_INDICATE_MASTER_CLOCK: VPMasterClock(pSrb); break;
    default:                        VPControlUnkMsg(pSrb); break;
  }
  unlogSrb("VPControl", pSrb);
  if(pSrb->Status!=STATUS_PENDING) {
    SignalStreamRequestCompleteReadyForNextControl(pSrb);
  } else {
    DbgMsg(DBG_INFO,"Returning Status Pending %s\r\n",Ident(ID_STRM_REQ,pSrb->Command));
  }
}

NTSTATUS STREAMAPI VPEvent(PHW_EVENT_DESCRIPTOR pEventDescriptor) {
  NTSTATUS status= STATUS_SUCCESS;
  PSTREAM_EXTENSION pStrm= pEventDescriptor->StreamObject->HwStreamExtension;
  logEvent("VPEvent", pEventDescriptor);
  if(pEventDescriptor->Enable)
    pStrm->dwEventCount++;
  else
    pStrm->dwEventCount--;
  unlogEvent("VPEvent", status);
  DbgMsg(DBG_MSGEVENT,"VPEvent: Count=%d, returning 0x%08X\r\n",pStrm->dwEventCount,status);
  return status;
}

static void VPControlUnkMsg(SRB* pSrb) {
  DbgMsg(DBG_MSGERR,"VPControl: Unhandled Msg= %s \r\n",Ident(ID_STRM_REQ,pSrb->Command));
}

static void VPSetState(SRB* pSrb) {
  PSTREAM_EXTENSION pStream;
  MIDASCMD Cmd;
  PDEVICE_EXTENSION pDevice= (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  DbgMsg(DBG_MSGVPE,"VPControl: SetState to %s \r\n",Ident(ID_KS_STATE,pSrb->CommandData.StreamState));
  
  StreamSetState((PSTREAM_EXTENSION)pSrb->StreamObject->HwStreamExtension,pSrb->CommandData.StreamState);
  Cmd.hMidas=pDevice->hMidas;
  Cmd.cmd= MDS_VIDENABLEZV;
  switch(pStream->stream_state) {
    case KSSTATE_ACQUIRE:   
    case KSSTATE_PAUSE:     
    case KSSTATE_RUN:       Cmd.u.M.Arg1= 1; break;
    case KSSTATE_STOP:      
    default:                Cmd.u.M.Arg1= 0; break;
  }
  MidasEntry(&Cmd);
  pSrb->Status = STATUS_SUCCESS;
  //MargiDDrawProc(MARGIDD_GETSURFHANDLE,pStream);
}

static void VPGetState(SRB* pSrb) {
  PSTREAM_EXTENSION pStream;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  pSrb->CommandData.StreamState = pStream->stream_state;
  DbgMsg(DBG_MSGVPE,"VPControl: GetState returns %s \r\n",Ident(ID_KS_STATE,pSrb->CommandData.StreamState));
  pSrb->Status = STATUS_SUCCESS;
}

static void VPGetProperty_NumConnectInfo(SRB* pSrb) {
  // Number of connections that are supported
  *(PULONG) pSrb->CommandData.PropertyInfo->PropertyInfo = NUM_OF_CONNECTIONS_AVAIL;
  pSrb->ActualBytesTransferred = sizeof(DWORD);
  pSrb->Status = STATUS_SUCCESS;
}

static void VPGetProperty_NumVideoFormat(SRB* pSrb) {
  // specify the number of formats that are supported
  *(PULONG) pSrb->CommandData.PropertyInfo->PropertyInfo = NUM_OF_VID_FORMATS_AVAIL;
  pSrb->ActualBytesTransferred = sizeof(DWORD);
  pSrb->Status = STATUS_SUCCESS;
}

static void VPGetProperty_DecimateComp(SRB* pSrb) {
  // indicate that we can decimate anything, especially if it's late.
   BOOL bDec= TRUE;
  *((PBOOL)pSrb->CommandData.PropertyInfo->PropertyInfo) = bDec;
  pSrb->ActualBytesTransferred = sizeof (BOOL);
  pSrb->Status = STATUS_SUCCESS;
}

static void VPGetProperty_DataInfo(SRB* pSrb) {
  PKS_AMVPDATAINFO pAMVPDataInfo= (PKS_AMVPDATAINFO)(pSrb->CommandData.PropertyInfo->PropertyInfo);

  RtlCopyMemory(pAMVPDataInfo,&VPFmt,sizeof(KS_AMVPDATAINFO));

  pAMVPDataInfo->dwSize = sizeof(KS_AMVPDATAINFO);
  pAMVPDataInfo->dwMicrosecondsPerField = 16683;
  pAMVPDataInfo->bEnableDoubleClock = FALSE;
  pAMVPDataInfo->bEnableVACT = FALSE;
  pAMVPDataInfo->lHalfLinesOdd= 0;
  if(gdwIndex>=2)     // ATI will select index 2.  This should be changed to use HALFLINE CAP flag
    pAMVPDataInfo->lHalfLinesEven= 1;
  pAMVPDataInfo->bFieldPolarityInverted = FALSE; 
  pAMVPDataInfo->dwNumLinesInVREF= 0;

  pSrb->ActualBytesTransferred = sizeof( KS_AMVPDATAINFO);
  pSrb->Status = STATUS_SUCCESS;


}

static void VPGetProperty_MaxPixRate(SRB* pSrb) {
  PKSVPMAXPIXELRATE pMaxPixelRate;
  int decoderHeight = VPFmt.amvpDimInfo.dwFieldHeight==262 ? 525 : 598;
  int decoderWidth  = VPFmt.amvpDimInfo.dwFieldWidth;
		
  if(pSrb->CommandData.PropertyInfo->PropertyInputSize < sizeof (KSVPSIZE_PROP)) {
    pSrb->Status = STATUS_INVALID_BUFFER_SIZE;
    DbgMsg(DBG_MSGERR," Invalid Buffer Size to xfer the Pixel Rate struct \r\n");
  } else {
    pMaxPixelRate = (PKSVPMAXPIXELRATE)pSrb->CommandData.PropertyInfo->PropertyInfo;
    pMaxPixelRate->Size.dwWidth= decoderWidth;
    pMaxPixelRate->Size.dwHeight= decoderHeight;
    pMaxPixelRate->MaxPixelsPerSecond= decoderWidth * decoderHeight * NTSC_FRAME_RATE;
    pSrb->ActualBytesTransferred = sizeof(KSVPMAXPIXELRATE);
    pSrb->Status = STATUS_SUCCESS;
  }
}
							         
static void VPGetProperty_GetConnectInfo(SRB* pSrb) {
  LPDDVIDEOPORTCONNECT pConnectInfo;
  PKSMULTIPLE_ITEM  pmulitem; 
  DWORD lc;
  pmulitem= &(((PKSMULTIPLE_DATA_PROP)pSrb->CommandData.PropertyInfo->Property)->MultipleItem);
  if(pmulitem->Count>NUM_OF_CONNECTIONS_AVAIL || pmulitem->Size!=sizeof(DDVIDEOPORTCONNECT) ||
      pSrb->CommandData.PropertyInfo->PropertyOutputSize<(pmulitem->Count*sizeof(DDVIDEOPORTCONNECT))) {
    pSrb->Status = STATUS_INVALID_BUFFER_SIZE;
  } else {
    for(lc=0,pConnectInfo=(LPDDVIDEOPORTCONNECT)(pSrb->CommandData.PropertyInfo->PropertyInfo);
               lc<pmulitem->Count;lc++, pConnectInfo++) {
      RtlCopyMemory(pConnectInfo,&VPConInfo[lc],sizeof(DDVIDEOPORTCONNECT));
    }
    pSrb->ActualBytesTransferred = pmulitem->Count*sizeof(DDVIDEOPORTCONNECT);
    pSrb->Status = STATUS_SUCCESS;
  }
}

static void VPGetProperty_GetVideoFormat(SRB* pSrb) {
  LPDDPIXELFORMAT pPixelFormat;
  PKSMULTIPLE_ITEM  pmulitem; 
  DWORD lc;
  pmulitem= &(((PKSMULTIPLE_DATA_PROP)pSrb->CommandData.PropertyInfo->Property)->MultipleItem);
  if(pmulitem->Count>NUM_OF_VID_FORMATS_AVAIL || pmulitem->Size != sizeof (DDPIXELFORMAT) ||
      pSrb->CommandData.PropertyInfo->PropertyOutputSize<(pmulitem->Count*sizeof(DDPIXELFORMAT))) {
    pSrb->Status = STATUS_INVALID_BUFFER_SIZE;
  } else {
    for (lc=0,pPixelFormat=(LPDDPIXELFORMAT)(pSrb->CommandData.PropertyInfo->PropertyInfo);
			lc < pmulitem->Count;lc++,pPixelFormat++) {
      // fill in the DDPIXELFORMAT structure
      pPixelFormat->dwSize= sizeof( DDPIXELFORMAT);
      pPixelFormat->dwYUVBitCount= 16;
      pPixelFormat->dwFlags= DDPF_FOURCC;
      pPixelFormat->dwYBitMask= (DWORD)0xFF00FF00;
      pPixelFormat->dwUBitMask= (DWORD)0x000000FF;
      pPixelFormat->dwVBitMask= (DWORD)0x00FF0000;
      switch(lc) {
        case 0:     pPixelFormat->dwFourCC= FOURCC_YUY2; break;
        case 1:     pPixelFormat->dwFourCC= FOURCC_YUY2; break;
        default:    pPixelFormat->dwFourCC= FOURCC_UYVY; break;
      }
    }
    pSrb->ActualBytesTransferred = pmulitem->Count*sizeof(DDPIXELFORMAT);
    pSrb->Status = STATUS_SUCCESS;
  }
}

static void VPGetProperty(SRB* pSrb) {
  long lID= pSrb->CommandData.PropertyInfo->Property->Id;
  logGetProperty("VP", pSrb);
  if(!pSrb->CommandData.PropertyInfo->PropertySetID) {
    DbgMsg(DBG_MSGVPE,"VPControl: GetProperty %s (returns) \r\n",Ident(ID_KSPROP_VPE,lID));
    switch (lID) {
      case KSPROPERTY_VPCONFIG_NUMCONNECTINFO:      VPGetProperty_NumConnectInfo(pSrb); break;
      case KSPROPERTY_VPCONFIG_NUMVIDEOFORMAT:      VPGetProperty_NumVideoFormat(pSrb); break;
      case KSPROPERTY_VPCONFIG_DECIMATIONCAPABILITY:VPGetProperty_DecimateComp(pSrb); break;
      case KSPROPERTY_VPCONFIG_VPDATAINFO:          VPGetProperty_DataInfo(pSrb); break;
      case KSPROPERTY_VPCONFIG_MAXPIXELRATE:        VPGetProperty_MaxPixRate(pSrb); break;
      case KSPROPERTY_VPCONFIG_GETCONNECTINFO:      VPGetProperty_GetConnectInfo(pSrb); break;
      case KSPROPERTY_VPCONFIG_GETVIDEOFORMAT:      VPGetProperty_GetVideoFormat(pSrb); break;
//      case KSPROPERTY_VPCONFIG_SURFACEPARAMS:       VPGetProperty_SurfaceParams(pSrb); break;
    }
    DbgDispKsVPEPropertyInfo(DBG_MSGVPE,pSrb->CommandData.PropertyInfo->PropertyInfo,
                             pSrb->CommandData.PropertyInfo->Property->Id);
  } else {
    DbgMsg(DBG_MSGERR,"VPControl: GetProperty called with an unknown ID (%08x)\r\n",lID);
  }
}

static MDS_VPE_INFO gVPEInfo;

static void VPSetProperty_ConnectInfo(SRB* pSrb) {
  ULONG dwIndex= (*(PULONG)pSrb->CommandData.PropertyInfo->PropertyInfo);
  PDEVICE_EXTENSION pDevice;
  PSTREAM_EXTENSION pStream;
  MIDASCMD Cmd;
  LPDDVIDEOPORTCONNECT pConnectInfo= &VPConInfo[dwIndex];
  gdwIndex=dwIndex;
  pDevice= (PDEVICE_EXTENSION) pSrb->HwDeviceExtension;
  pStream= (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  Cmd.hMidas=pDevice->hMidas;
  Cmd.cmd= MDS_VPEINFO;
  Cmd.u.pVPEInfo_s= &gVPEInfo; //ExAllocatePool(NonPagedPool,sizeof(VPEInfo_s));
  if(Cmd.u.pVPEInfo_s) {
    Cmd.u.pVPEInfo_s->dwPortWidth= pConnectInfo->dwPortWidth;
    MidasEntry(&Cmd);
  }
  DbgMsg(DBG_MSGVPE,"VPControl: SetConnectInfo %d \r\n",(*(PULONG)pSrb->CommandData.PropertyInfo->PropertyInfo));
  pSrb->Status = STATUS_SUCCESS;
}

static void VPSetProperty_VideoFormat(SRB* pSrb) {
  ULONG dwIndex= (*(PULONG)pSrb->CommandData.PropertyInfo->PropertyInfo);
  if(dwIndex < NUM_OF_VID_FORMATS_AVAIL) {//If it's one of the Video Format
    pSrb->Status = STATUS_SUCCESS;
  } else {
    pSrb->Status = STATUS_NO_MATCH;
  }
}

static void VPSetProperty_DDrawHandle(SRB* pSrb) {
  PSTREAM_EXTENSION pStream;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  pStream->u.vp.ddrawHandle_Ring3 = (*(PULONG)pSrb->CommandData.PropertyInfo->PropertyInfo);
  DbgMsg(DBG_DDINFO," Ring3 Directdraw handle (from StreamClass) 0x%08x \r\n",pStream->u.vp.ddrawHandle_Ring3);
  MargiDDrawProc(MARGIDD_GETDDHANDLE,pStream);
  MargiDDrawProc(MARGIDD_GETCAPS,pStream);
  pSrb->Status = STATUS_SUCCESS;
}

static void VPSetProperty_VPortID(SRB* pSrb) {
  PSTREAM_EXTENSION pStream;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  pStream->u.vp.VidPortHandle_Ring3 = (*(PULONG)pSrb->CommandData.PropertyInfo->PropertyInfo);
  DbgMsg(DBG_DDINFO," Ring3 VideoPort handle (from StreamClass) 0x%08x \r\n",pStream->u.vp.VidPortHandle_Ring3);
  MargiDDrawProc(MARGIDD_GETVPHANDLE,pStream);
  MargiDDrawProc(MARGIDD_REGISTERVPSYNC,pStream);
  pSrb->Status = STATUS_SUCCESS;
}

static void VPSetProperty_SurfHandle(SRB* pSrb) {
  PSTREAM_EXTENSION pStream;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  pStream->u.vp.SurfaceHandle_Ring3 = (*(PULONG)pSrb->CommandData.PropertyInfo->PropertyInfo);
  DbgMsg(DBG_DDINFO," Ring3 Surface Handle (from StreamClass) 0x%08x \r\n",pStream->u.vp.SurfaceHandle_Ring3);
  MargiDDrawProc(MARGIDD_GETSURFHANDLE,pStream);                            
  pSrb->Status = STATUS_SUCCESS;
}

static void VPSetProperty_InvPolarity(SRB* pSrb) {
  // Toggles the global polarity flag, telling the output
  // of the VPE port to be inverted.  Since this hardware
  // does not support this feature, we will just return
  // success for now, although this should be returning not
  // implemented
  pSrb->Status = STATUS_SUCCESS;
}

static void VPSetProperty_ScaleFactor(SRB* pSrb) {
  PKS_AMVPSIZE pDim;
  // the sizes for the scaling factor are passed in, and the
  // image dimensions should be scaled appropriately
  // if there is a horizontal scaling available, do it here.
  pDim =(PKS_AMVPSIZE)(pSrb->CommandData.PropertyInfo->PropertyInfo);
  pSrb->Status = STATUS_SUCCESS;
}

static void VPSetProperty_SurfaceParams(SRB* pSrb) {
}

static void VPSetProperty(SRB* pSrb) {
  long lID= pSrb->CommandData.PropertyInfo->Property->Id; 
  logSetProperty("VP", pSrb);
  if(pSrb->CommandData.PropertyInfo->PropertySetID==0) {
    DbgMsg(DBG_MSGVPE,"VPControl: SetProperty %s \r\n",Ident(ID_KSPROP_VPE,lID));
    switch(lID) {
      case KSPROPERTY_VPCONFIG_SETCONNECTINFO:      VPSetProperty_ConnectInfo(pSrb); break;
      case KSPROPERTY_VPCONFIG_SETVIDEOFORMAT:      VPSetProperty_VideoFormat(pSrb); break;
      case KSPROPERTY_VPCONFIG_DDRAWHANDLE:         VPSetProperty_DDrawHandle(pSrb); break;
      case KSPROPERTY_VPCONFIG_VIDEOPORTID:         VPSetProperty_VPortID(pSrb); break;
      case KSPROPERTY_VPCONFIG_DDRAWSURFACEHANDLE:  VPSetProperty_SurfHandle(pSrb); break;
      case KSPROPERTY_VPCONFIG_INVERTPOLARITY:      VPSetProperty_InvPolarity(pSrb); break;
      case KSPROPERTY_VPCONFIG_SCALEFACTOR:         VPSetProperty_ScaleFactor(pSrb); break;
      case KSPROPERTY_VPCONFIG_SURFACEPARAMS:       VPSetProperty_SurfaceParams(pSrb); break;
    }
    DbgDispKsVPEPropertyInfo(DBG_MSGVPE,pSrb->CommandData.PropertyInfo->PropertyInfo,lID);
  } else if(pSrb->CommandData.PropertyInfo->PropertySetID==1) {
    if(lID==KSPROPERTY_COPY_MACROVISION) {
      SetMacroVisionLevel(pSrb);
    } else {
      pSrb->Status= STATUS_NOT_IMPLEMENTED;
      DbgMsg(DBG_MSGERR,"Unknown CopyProt Set Property Cmd (%u,%u) \r\n",pSrb->CommandData.PropertyInfo->PropertySetID,lID);
    }
  } else {
    pSrb->Status= STATUS_NOT_IMPLEMENTED;
    DbgMsg(DBG_MSGERR,"Unknown Set Property Cmd (%u,%u) \r\n",pSrb->CommandData.PropertyInfo->PropertySetID,lID);

  }
}

static void VPMasterClock(SRB* pSrb) {
  MIDASCMD Cmd;
  PSTREAM_EXTENSION pStream;
  PDEVICE_EXTENSION pDevice=(PDEVICE_EXTENSION)pSrb->HwDeviceExtension;
  pStream = (PSTREAM_EXTENSION) pSrb->StreamObject->HwStreamExtension;
  pStream->MasterClock = pSrb->CommandData.MasterClockHandle;
  Cmd.cmd= MDS_MASTERCLOCK;
  Cmd.hMidas=pDevice->hMidas;
  Cmd.u.M.Arg1= (DWORD)pSrb->CommandData.MasterClockHandle;
  Cmd.u.M.Arg2= 0;
  MidasEntry(&Cmd);
  DbgMsg(DBG_MSGVPE,"VPControl: MasterClockHandle is set to %08x \r\n",pSrb->CommandData.MasterClockHandle);
  pSrb->Status = STATUS_SUCCESS;
}

/**EOF: VP.C**/
